TestCafeClient.define('Automation.IFrameBehavior', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        Util = Hammerhead.Util,
        MessageSandbox = Hammerhead.MessageSandbox,
        Settings = require('Settings'),
        CrossDomainMessages = require('Base.CrossDomainMessages'),
        MovePlaybackAutomation = require('Automation.Move.Playback'),
        ScrollAutomation = require('Automation.Scroll'),
        CursorWidget = require('UI.Cursor');

    function onMessage(e) {
        var message = e.message,

            iFrameRectangle = null,
            intersectionPoint = null;

        switch (message.cmd) {
            case CrossDomainMessages.CURSOR_START_REQUEST_CMD:
                if (!Settings.RECORDING || Settings.PLAYBACK) {
                    CursorWidget.start(message.position, function () {
                        MessageSandbox.sendServiceMsg({ cmd: CrossDomainMessages.CURSOR_START_RESPONSE_CMD }, e.source);
                    }, e.source);
                }
                break;

            case CrossDomainMessages.MOVE_TO_IFRAME_REQUEST_CMD:
                var curCursorPosition = CursorWidget.getPosition();

                if (!curCursorPosition) {
                    MessageSandbox.sendServiceMsg({
                        cmd: CrossDomainMessages.MOVE_TO_IFRAME_RESPONSE_CMD,
                        point: null
                    }, e.source);
                    return;
                }

                var fixedPoint = Util.getFixedPosition(message.point, e.source),
                    pageCursorPosition = Util.clientToOffsetCoord(curCursorPosition);

                iFrameRectangle = Util.getIFrameCoordinates(e.source);

                if (Util.checkPresenceInRectangle(pageCursorPosition, iFrameRectangle) && Util.checkPresenceInRectangle(fixedPoint, iFrameRectangle)) {
                    MessageSandbox.sendServiceMsg({
                        cmd: CrossDomainMessages.MOVE_TO_IFRAME_RESPONSE_CMD,
                        point: message.point
                    }, e.source);
                }
                else {
                    intersectionPoint = Util.findLineAndRectangelIntersection(pageCursorPosition, fixedPoint, iFrameRectangle);

                    MovePlaybackAutomation.run(intersectionPoint, false, {}, function () {
                        MessageSandbox.sendServiceMsg({
                            cmd: CrossDomainMessages.MOVE_TO_IFRAME_RESPONSE_CMD,
                            point: Util.getFixedPositionForIFrame(intersectionPoint, e.source)
                        }, e.source);
                    });
                }

                break;

            case CrossDomainMessages.MOVE_CURSOR_IN_IFRAME_PING:
                if (message.isPingRequest) {
                    MessageSandbox.sendServiceMsg({
                        cmd: CrossDomainMessages.MOVE_CURSOR_IN_IFRAME_PING,
                        isPingResponse: true
                    }, e.source);
                }
                break;

            case CrossDomainMessages.MOVE_FROM_IFRAME_REQUEST_CMD:
                var frameDoc = document.documentElement,
                    iFrameVerticalScroll = frameDoc.scrollHeight > frameDoc.clientHeight ? Util.getScrollbarSize() : 0,
                    iFrameHorizontalScroll = frameDoc.scrollWidth > frameDoc.clientWidth ? Util.getScrollbarSize() : 0;

                iFrameRectangle = {
                    left: message.rectangle.left,
                    right: message.rectangle.right - iFrameVerticalScroll,
                    top: message.rectangle.top,
                    bottom: message.rectangle.bottom - iFrameHorizontalScroll
                };

                if (!Util.checkPresenceInRectangle(message.endPoint, iFrameRectangle)) {
                    MessageSandbox.sendServiceMsg({
                        cmd: CrossDomainMessages.MOVE_FROM_IFRAME_RESPONSE_CMD,
                        point: null
                    }, e.source);
                }
                else {
                    //NOTE: after scroll in top window cursor position in iframe could be changed (if cursor was above iframe)
                    CursorWidget.setPosition(message.cursorPosition);

                    intersectionPoint = Util.findLineAndRectangelIntersection(message.startPoint, message.endPoint, iFrameRectangle);

                    //NOTE: convert for IFrame
                    intersectionPoint = {
                        x: intersectionPoint.x - iFrameRectangle.left,
                        y: intersectionPoint.y - iFrameRectangle.top
                    };

                    MovePlaybackAutomation.run(Util.clientToOffsetCoord(intersectionPoint), false, {}, function () {
                        MessageSandbox.sendServiceMsg({
                            cmd: CrossDomainMessages.MOVE_FROM_IFRAME_RESPONSE_CMD,
                            point: intersectionPoint
                        }, e.source);
                    });
                }

                break;

            case CrossDomainMessages.SCROLL_TOP_WINDOW_REQUEST_CMD:
                ScrollAutomation.setScroll(e.source, message.point, message.options, function () {
                    MessageSandbox.sendServiceMsg({ cmd: CrossDomainMessages.SCROLL_TOP_WINDOW_RESPONSE_CMD }, e.source);
                });
                break;

            case CrossDomainMessages.GET_IFRAME_POSITION_DATA_REQUEST_CMD:
                var data = ScrollAutomation.getScrollData(e.source);

                data.cmd = CrossDomainMessages.GET_IFRAME_POSITION_DATA_RESPONSE_CMD;

                MessageSandbox.sendServiceMsg(data, e.source);
                break;
        }
    }

    exports.init = function () {
        MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
    };

    exports.destroy = function () {
        MessageSandbox.off(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
    };
});