TestCafeClient.define('Automation.Click.Playback', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        async = Hammerhead.async,
        Settings = require('Settings'),
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,
        EventSandbox = Hammerhead.EventSandbox,
        EventSimulator = Hammerhead.EventSimulator,
        SharedErrors = require('Shared.Errors'),
        AutomationUtil = require('Automation.Util'),
        AutomationSettings = require('Automation.Settings'),
        CursorWidget = require('UI.Cursor'),

        SelectElementUi = require('UI.SelectElement'),
        MovePlaybackAutomation = require('Automation.Move.Playback');

    function clickOnSelectChildElement(childElement, clickOptions, actionCallback, errorCallback) {
        var isClickOnOption = childElement.tagName.toLowerCase() === 'option',
            select = Util.getSelectParent($(childElement));

        if (!select) {
            EventSimulator.click(childElement, clickOptions);
            actionCallback();
            return;
        }

        var $select = $(select),
            selectedIndex = $select[0].selectedIndex,
            isOptionListExpanded = SelectElementUi.isOptionListExpanded($select),
            childIndex = Util.getChildIndex($select, childElement),
            targetElement = null;

        if (!isOptionListExpanded) {
            var selectSizeValue = Util.getSelectSizeValue($select);

            if ((!Settings.RECORDING || Settings.PLAYBACK) && selectSizeValue <= 1) {
                errorCallback({
                    code: SharedErrors.API_INVISIBLE_ACTION_ELEMENT,
                    element: Util.getElementDescription(childElement)
                });
                return;
            }

            targetElement = childElement;
        }
        else
            targetElement = SelectElementUi.getEmulatedChildElement(childIndex, !isClickOnOption);

        async.series({
                moveCursorToElement: function (callback) {
                    //NOTE: 'target' is option from emulated optionList or the point (x,y-coordinates) of real option element
                    var target = null;

                    if (isOptionListExpanded)
                        target = targetElement;
                    else {
                        SelectElementUi.scrollOptionListByChild(childElement);
                        target = SelectElementUi.getSelectChildCenter(childElement);
                    }

                    MovePlaybackAutomation.run(target, false, {}, function () {
                        NativeMethods.setTimeout.call(window, callback, AutomationSettings.ACTION_STEP_DELAY);
                    });
                },

                click: function () {
                    if (!isOptionListExpanded && ((Util.isWebKit && Util.getSelectSizeValue($select) === 1) || (Util.isIE && Util.getSelectSizeValue($select) > 1)))
                        EventSimulator.click($select[0], clickOptions);
                    else
                        EventSimulator.click(targetElement, clickOptions);

                    if (!isOptionListExpanded) {
                        if (isClickOnOption && !targetElement.disabled)
                            $select[0].selectedIndex = childIndex;

                        EventSandbox.focus($select[0], function () {
                            NativeMethods.setTimeout.call(window, function () {
                                if (isClickOnOption && $(childElement).index() !== selectedIndex)
                                    EventSimulator.change($select[0]);
                                actionCallback();
                            }, Util.isTouchDevice ? 0 : AutomationSettings.CLICK_STEP_DELAY);
                        }, false, true);
                    }
                    else
                        actionCallback();
                }
            }
        );
    }

    exports.run = function (el, options, runCallback, errorCallback) {
        options = {
            ctrl: options.ctrl,
            alt: options.alt,
            shift: options.shift,
            meta: options.meta,
            offsetX: options.offsetX,
            offsetY: options.offsetY,
            caretPos: options.caretPos
        };

        if (el.tagName.toLowerCase() === 'option' || el.tagName.toLowerCase() === 'optgroup') {
            clickOnSelectChildElement(el, options, runCallback, errorCallback);

            return;
        }

        var isSvgElement = Util.isSvgElement(el),
            screenPoint = null,
            eventPoint = null,
            eventOptions = null,
            target = Util.isContainOffset(el, options.offsetX, options.offsetY) ?
                el : AutomationUtil.getMouseActionPoint(el, options, false),
            notPrevented = true,
            topElement = null,
            isInvisibleElement = false,
            currentTopElement = null,
            skipClickSimulation = false;

        if (options.offsetX)
            options.offsetX = Math.round(options.offsetX);
        if (options.offsetY)
            options.offsetY = Math.round(options.offsetY);

        async.series({
            moveCursorToElement: function (callback) {
                if (Settings.RECORDING && !Settings.PLAYBACK && !Util.isElementVisible(el)) {
                    topElement = el;

                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.ACTION_STEP_DELAY);
                    return;
                }

                MovePlaybackAutomation.run(target, false, options, function () {
                    if ((isSvgElement && Util.isOpera) || ($(el).is('tref')))
                        topElement = el; //NOTE: document.elementFromPoint can't find this element
                    else {
                        screenPoint = AutomationUtil.getMouseActionPoint(el, options, true);
                        eventPoint = AutomationUtil.getEventOptionCoordinates(el, screenPoint);

                        eventOptions = $.extend({
                            clientX: eventPoint.x,
                            clientY: eventPoint.y
                        }, options);

                        topElement = AutomationUtil.getElementUnderCursor(screenPoint.x, screenPoint.y, null, target);

                        if (!topElement) {
                            isInvisibleElement = true;
                            topElement = el;
                        }
                    }
                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.ACTION_STEP_DELAY);
                });
            },

            //NOTE: touch devices only
            touchstart: function (callback) {
                if (Util.isTouchDevice)
                    EventSimulator.touchstart(topElement, eventOptions);
                callback();
            },

            //NOTE: touch devices only
            cursorTouchMouseDown: function (callback) {
                if (Util.isTouchDevice)
                    CursorWidget.lMouseDown(callback);
                else
                    callback();
            },

            //NOTE: touch devices only
            touchend: function (callback) {
                if (Util.isTouchDevice) {
                    EventSimulator.touchend(topElement, eventOptions);

                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.ACTION_STEP_DELAY);
                }
                else
                    callback();
            },

            cursorMouseDown: function (callback) {
                if (!Util.isTouchDevice)
                    CursorWidget.lMouseDown(callback);
                else
                    callback();
            },

            mousedown: function (callback) {
                //NOTE: in webkit and ie raising mousedown event opens select element's dropdown,
                // therefore we should handle it and hide the dropdown (B236416)
                var needHandleMousedown = (Util.isWebKit || Util.isIE) && Util.isSelectElement(el),
                    wasPrevented = null,
                    blurRaised = false,
                    activeElement = Util.getActiveElement(),
                //in IE focus is not raised if element was focused before click, even if focus is lost during mousedown
                    needFocus =  !(Util.isIE && activeElement === topElement) ||
                                 (topElement.tagName.toLowerCase() === 'body' && Util.isContentEditableElement(topElement)),
                    mouseDownCallback = function(){
                        NativeMethods.setTimeout.call(window, callback, AutomationSettings.ACTION_STEP_DELAY);
                    };

                if (needHandleMousedown) {
                    var onmousedown = function (e) {
                        wasPrevented = e.defaultPrevented;
                        Util.preventDefault(e);
                        NativeMethods.removeEventListener.call(el, 'mousedown', onmousedown, false);
                    };

                    NativeMethods.addEventListener.call(el, 'mousedown', onmousedown, false);
                }

                var onblur = function () {
                    blurRaised = true;
                    NativeMethods.removeEventListener.call(activeElement, 'blur', onblur, true);
                };
                NativeMethods.addEventListener.call(activeElement, 'blur', onblur, true);

                notPrevented = EventSimulator.mousedown(topElement, eventOptions);

                if (!isInvisibleElement && screenPoint) {
                    currentTopElement = AutomationUtil.getElementUnderCursor(screenPoint.x, screenPoint.y, null, target);

                    if (currentTopElement && currentTopElement !== topElement) {
                        skipClickSimulation = true;
                        topElement = currentTopElement;
                    }
                }

                //In some cases (B239273, B253520) mousedown may lead to active element changing. Browsers raise blur in this cases.
                //We simulate blur event if active element was changed after mousedown and blur event was not raised automatically
                if (Util.getActiveElement() !== activeElement && !blurRaised)
                    if (Util.isIE && Util.browserVersion < 12)
                    //All ways to blur element from client script raise blur event asynchronously in IE (in MSEdge focus/blur is sync)
                        NativeMethods.setTimeout.call(window, function () {
                            if (!blurRaised)
                                EventSimulator.blur(activeElement);
                        }, 0);
                    else
                        EventSimulator.blur(activeElement);

                if (notPrevented === false) {
                    if (needHandleMousedown && !wasPrevented)
                        notPrevented = true;
                    else {
                        mouseDownCallback();
                        return;
                    }
                }

                //NOTE: For contentEditable elements we should call focus directly for action's element because
                //option 'caretPos' is indicated for this element and topElement may be a child of this element
                AutomationUtil.focusAndSetSelection(Util.isContentEditableElement(el) ? el : topElement, options, needFocus, mouseDownCallback);
            },

            cursorMouseUp: function (callback) {
                CursorWidget.mouseUp(callback);
            },

            mouseup: function (callback) {
                EventSimulator.mouseup(topElement, eventOptions);

                if (!isInvisibleElement && screenPoint)
                    currentTopElement = AutomationUtil.getElementUnderCursor(screenPoint.x, screenPoint.y, null, target);

                callback();
            },

            click: function () {
                var $el = $(topElement);

                if ((!Settings.RECORDING || Settings.PLAYBACK) && topElement.tagName.toLowerCase() === 'option') {
                    runCallback();
                    return;
                }

                if (topElement.tagName.toLowerCase() === 'option') {
                    var select = Util.getSelectParent($el),
                        $select = $(select);

                    if (select && ((Util.isWebKit && Util.getSelectSizeValue($select) <= 1) || (Util.isIE && Util.getSelectSizeValue($select) > 1)))
                        EventSimulator.click($select[0], eventOptions);
                    else
                        EventSimulator.click($el[0], eventOptions);

                    if (select)
                        EventSimulator.change($select[0]);

                    runCallback();
                    return;
                }

                //NOTE: If the element under the cursor has changed after 'mousedown' event then we should not raise 'click' event
                if (!skipClickSimulation)
                    EventSimulator.click(topElement, eventOptions);

                AutomationUtil.focusLabelChildElement(topElement);

                //NOTE: emulating click event on 'select' element doesn't expand dropdown with options (except chrome),
                // therefore we should emulate it.
                if ((!Settings.RECORDING || Settings.PLAYBACK) && Util.isSelectElement(topElement) && notPrevented !== false) {
                    //if this select already have options list
                    if (SelectElementUi.isOptionListExpanded($el))
                        SelectElementUi.collapseOptionList();
                    else
                        SelectElementUi.expandOptionList(topElement);
                }

                runCallback();
            }
        });
    };
});
