TestCafeClient.define('Automation.DblClick.Playback', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,

        SelectElementUi = require('UI.SelectElement'),
        async = Hammerhead.async,
        EventSimulator = Hammerhead.EventSimulator,
        Settings = require('Settings'),
        CursorWidget = require('UI.Cursor'),
        AutomationUtil = require('Automation.Util'),
        AutomationSettings = require('Automation.Settings'),
        ClickPlaybackAutomation = require('Automation.Click.Playback');

    exports.run = function (el, options, actionCallback) {
        options = {
            ctrl: options.ctrl,
            alt: options.alt,
            shift: options.shift,
            meta: options.meta,
            offsetX: options.offsetX,
            offsetY: options.offsetY,
            caretPos: options.caretPos
        };

        var curElement = null,
            point = null,
            currentTopElement = null,
            skipClickSimulation = false;

        if (!Util.isContainOffset(el, options.offsetX, options.offsetY)) {
            point = AutomationUtil.getMouseActionPoint(el, options, true);
            curElement = CursorWidget.getElementUnderCursor(point.x, point.y);
        }

        if (!curElement)
            curElement = el;

        var isInvisibleElement = (Settings.RECORDING && !Settings.PLAYBACK) && !Util.isElementVisible(el),
            screenPoint = AutomationUtil.getMouseActionPoint(el, options, true),
            eventPoint = AutomationUtil.getEventOptionCoordinates(el, screenPoint),
            eventOptions = $.extend({
                clientX: eventPoint.x,
                clientY: eventPoint.y
            }, options);

        async.series({
            firstClick: function (callback) {
                ClickPlaybackAutomation.run(el, options, function () {
                    if (!isInvisibleElement) {
                        currentTopElement = AutomationUtil.getElementUnderCursor(screenPoint.x, screenPoint.y, null, el);
                        if (currentTopElement && currentTopElement !== curElement)
                            curElement = currentTopElement;
                    }
                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.CLICK_STEP_DELAY);
                });
            },

            secondClick: function (callback) {
                var notPrevented = true;
                async.series({
                    //NOTE: touch devices only
                    touchstart: function (callback) {
                        if (Util.isTouchDevice)
                            EventSimulator.touchstart(curElement, eventOptions);
                        callback();
                    },

                    //NOTE: touch devices only
                    cursorTouchMouseDown: function (callback) {
                        if (Util.isTouchDevice)
                            CursorWidget.lMouseDown(callback);
                        else
                            callback();
                    },

                    //NOTE: touch devices only
                    touchend: function (callback) {
                        if (Util.isTouchDevice) {
                            EventSimulator.touchend(curElement, eventOptions);

                            NativeMethods.setTimeout.call(window, callback, AutomationSettings.ACTION_STEP_DELAY);
                        }
                        else
                            callback();
                    },

                    cursorMouseDown: function (callback) {
                        if (!Util.isTouchDevice)
                            CursorWidget.lMouseDown(callback);
                        else
                            callback();
                    },

                    mousedown: function (callback) {
                        //NOTE: in webkit and ie raising mousedown event opens select element's dropdown,
                        // therefore we should handle it and hide the dropdown (B236416)
                        var needHandleMousedown = (Util.isWebKit || Util.isIE) && Util.isSelectElement(curElement),
                            wasPrevented = null,
                            activeElement = Util.getActiveElement(),
                        //in IE focus is not raised if element was focused before click, even if focus is lost during mousedown
                            needFocus = !(Util.isIE && activeElement === curElement);

                        if (needHandleMousedown) {
                            var onmousedown = function (e) {
                                wasPrevented = e.defaultPrevented;
                                Util.preventDefault(e);
                                NativeMethods.removeEventListener.call(curElement, 'mousedown', onmousedown, false);
                            };

                            NativeMethods.addEventListener.call(curElement, 'mousedown', onmousedown, false);
                        }

                        notPrevented = EventSimulator.mousedown(curElement, eventOptions);

                        if (!isInvisibleElement) {
                            currentTopElement = AutomationUtil.getElementUnderCursor(screenPoint.x, screenPoint.y, null, el);

                            if (currentTopElement && currentTopElement !== curElement) {
                                skipClickSimulation = true;
                                curElement = currentTopElement;
                            }
                        }

                        if (notPrevented === false) {
                            if (needHandleMousedown && !wasPrevented)
                                notPrevented = true;
                            else {
                                callback();
                                return;
                            }
                        }

                        //NOTE: we should not call it after the second click because of the native browser behavior
                        if (!Util.isIE) {
                            //NOTE: For contentEditable elements we should call focus directly for action's element because
                            //option 'caretPos' is indicated for this element and topElement may be a child of this element
                            AutomationUtil.focusAndSetSelection(Util.isContentEditableElement(el) ? el : curElement, options, needFocus, callback);
                        }
                        else
                            callback();
                    },

                    cursorMouseUp: function (callback) {
                        CursorWidget.mouseUp(callback);
                    },

                    mouseup: function (callback) {
                        EventSimulator.mouseup(curElement, eventOptions);
                        callback();
                    },

                    click: function () {
                        if (curElement.tagName.toLowerCase() === 'option')
                            callback();
                        else {
                            //NOTE: If the element under the cursor has changed after 'mousedown' event then we should not raise 'click' event
                            if (!skipClickSimulation)
                                EventSimulator.click(curElement, eventOptions);

                            //NOTE: emulating click event on 'select' element doesn't expand dropdown with options (except chrome),
                            // therefore we should emulate it.
                            if ((!Settings.RECORDING || Settings.PLAYBACK) && Util.isSelectElement(curElement) && notPrevented !== false) {
                                //if this select already have options list
                                if (SelectElementUi.isOptionListExpanded($(curElement)))
                                    SelectElementUi.collapseOptionList();
                                else
                                    SelectElementUi.expandOptionList(curElement);
                            }
                            callback();
                        }
                    }
                });
            },

            dblclick: function () {
                //NOTE: If the element under the cursor has changed after 'mousedown' event then we should not raise 'dblclick' event
                if (!skipClickSimulation)
                    EventSimulator.dblclick(curElement, eventOptions);
                actionCallback();
            }
        });


        /*exports.runToFinalize(el, options, function (currentElement, runToFinalizeOptions) {
         //NOTE: For contentEditable elements we should call finalize directly for action's element because
         //option 'caretPos' is indicated for this element and topElement may be a child of this element
         finalize(Util.isContentEditableElement(el) ? el : currentElement, options, actionCallback, runToFinalizeOptions);
         });*/
    };
});
