TestCafeClient.define('Automation.Drag.Playback', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        async = Hammerhead.async,
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,
        EventSandbox = Hammerhead.EventSandbox,
        EventSimulator = Hammerhead.EventSimulator,
        Settings = require('Settings'),
        AutomationUtil = require('Automation.Util'),
        AutomationSettings = require('Automation.Settings'),
        CursorWidget = require('UI.Cursor'),
        ContentEditableHelper = Hammerhead.ContentEditableHelper,

        MovePlaybackAutomation = require('Automation.Move.Playback');

    exports.run = function (el, to, options, runCallback) {
        var target = Util.isContainOffset(el, options.offsetX, options.offsetY) ?
                el : AutomationUtil.getMouseActionPoint(el, options, false),

            screenPointFrom = null,
            eventPointFrom = null,
            eventOptionsStart = null,
            topElement = null,
            skipDragEmulation = Settings.RECORDING && !Settings.PLAYBACK && !Util.isElementVisible(el),
            oldOffset = null,
            currentDocument = Util.findDocument(el),
            pointTo = null,
            startPosition = null,
            screenPointTo = null,
            eventPointTo = null,
            eventOptionsEnd = null;

        if (options.offsetX)
            options.offsetX = Math.round(options.offsetX);
        if (options.offsetY)
            options.offsetY = Math.round(options.offsetY);

        if (skipDragEmulation) {
            runCallback(el);
            return;
        }

        async.series({
            moveCursorToElement: function (callback) {
                MovePlaybackAutomation.run(target, false, options, function () {
                    startPosition = AutomationUtil.getMouseActionPoint(el, options, false);
                    screenPointFrom = Util.offsetToClientCoords(startPosition);
                    eventPointFrom = AutomationUtil.getEventOptionCoordinates(el, screenPointFrom);

                    eventOptionsStart = $.extend({
                        clientX: eventPointFrom.x,
                        clientY: eventPointFrom.y
                    }, options);

                    if (Util.isDomElement(to))
                        pointTo = Util.findCenter(to);
                    else
                        pointTo = AutomationUtil.getDragEndPoint(startPosition, to, currentDocument);

                    topElement = CursorWidget.getElementUnderCursor(screenPointFrom.x, screenPointFrom.y);

                    if (!topElement) {
                        runCallback(el);
                        return;
                    }

                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.DRAG_ACTION_STEP_DELAY);
                });
            },

            cursorMouseDown: function (callback) {
                CursorWidget.lMouseDown(callback);
            },

            take: function (callback) {
                if (Util.isTouchDevice)
                    EventSimulator.touchstart(topElement, eventOptionsStart);
                else
                    EventSimulator.mousedown(topElement, eventOptionsStart);

                //NOTE: For contentEditable elements we should call focus directly for action's element
                EventSandbox.focus(Util.isContentEditableElement(el) ? ContentEditableHelper.findContentEditableParent(el) : topElement, function () {
                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.DRAG_ACTION_STEP_DELAY);
                }, false, true);
            },

            drag: function (callback) {
                oldOffset = {
                    x: options.offsetX,
                    y: options.offsetY
                };

                delete options.offsetX;
                delete options.offsetY;

                MovePlaybackAutomation.run(Util.isDomElement(to) ? to : pointTo, true, options, function () {
                    options.offsetX = oldOffset.x;
                    options.offsetY = oldOffset.y;

                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.DRAG_ACTION_STEP_DELAY);
                }, currentDocument);
            },


            cursorMouseUp: function (callback) {
                if (pointTo)
                    screenPointTo = Util.offsetToClientCoords(pointTo);
                else {
                    var offsetPos = Util.getOffsetPosition(el);

                    screenPointTo = Util.offsetToClientCoords({
                        x: offsetPos.left,
                        y: offsetPos.top
                    });
                }

                eventPointTo = AutomationUtil.getEventOptionCoordinates(el, screenPointTo);

                if (Util.isElementInIframe(el)) {
                    var currentIFrame = Util.getIFrameByElement(el);
                    if (currentIFrame) {
                        var screenPointToInIFrame = {
                            x: screenPointTo.x - $(currentIFrame.contentWindow).scrollLeft(),
                            y: screenPointTo.y - $(currentIFrame.contentWindow).scrollTop()
                        };

                        topElement = CursorWidget.getElementUnderCursor(screenPointToInIFrame.x, screenPointToInIFrame.y);
                    }
                }
                else
                    topElement = CursorWidget.getElementUnderCursor(screenPointTo.x, screenPointTo.y);

                if (!topElement) {
                    runCallback();
                    return;
                }

                eventOptionsEnd = $.extend({
                    clientX: eventPointTo.x,
                    clientY: eventPointTo.y
                }, options);

                CursorWidget.mouseUp(callback);
            },

            mouseUp: function (callback) {
                if (Util.isTouchDevice)
                    EventSimulator.touchend(topElement, eventOptionsEnd);
                else
                    EventSimulator.mouseup(topElement, eventOptionsEnd);

                callback();
            },

            click: function () {
                //B231323
                if (CursorWidget.getElementUnderCursor(screenPointTo.x, screenPointTo.y) === topElement)
                    EventSimulator.click(topElement, eventOptionsEnd);

                runCallback();
            }
        });
    };
});
