TestCafeClient.define('Automation.Move.Playback', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        async = Hammerhead.async,
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,
        EventSimulator = Hammerhead.EventSimulator,
        MessageSandbox = Hammerhead.MessageSandbox,
        CrossDomainMessages = require('Base.CrossDomainMessages'),
        AutomationUtil = require('Automation.Util'),
        AutomationSettings = require('Automation.Settings'),
        ScrollPlaybackAutomation = require('Automation.Scroll.Playback'),
        CursorWidget = require('UI.Cursor');

    //NOTE: We should save last over element between runs because of T286582
    var lastOverElement = null;

    exports.run = function (to, inDragging, options, actionCallback, currentDocument, skipEvents, inSelect) {
        currentDocument = currentDocument || document;

        var targetPoint = Util.isDomElement(to) ?
                AutomationUtil.getMouseActionPoint(to, options, false) :
            {
                x: Math.floor(to.x),
                y: Math.floor(to.y)
            },

            isMovingInIFrame = currentDocument !== document,
            targetScreenPoint = null,
            startX = null,
            startY = null,
            dragElement = null,

        // moving settings
            distanceX = null,
            distanceY = null,

            startTime = null,
            endTime = null,
            movingTime = null,

            currentCursorPosition = CursorWidget.getPosition(),

            $window = $(window);

        // moving step
        function nextStep(movingStepCallback) {
            async.series({
                setPosition: function (callback) {
                    if (Util.isTouchDevice && !inDragging) {
                        currentCursorPosition = targetScreenPoint;
                        CursorWidget.move(currentCursorPosition, movingStepCallback);
                        return;
                    }

                    if (!startTime) {
                        startTime = Util.dateNow();
                        endTime = startTime + movingTime;
                    }

                    var currentTime = Math.min(Util.dateNow(), endTime),
                        progress = (currentTime - startTime) / (endTime - startTime);

                    currentCursorPosition = {
                        x: Math.round(startX + (distanceX * progress)),
                        y: Math.round(startY + (distanceY * progress))
                    };

                    //NOTE: mousemove event can't be simulated on the point when cursor was at the start. Therefore we increases
                    // a minimal distance 1 px.
                    if (currentCursorPosition.x === startX && currentCursorPosition.y === startY) {
                        if (inSelect) {
                            movingStepCallback();
                            return;
                        }

                        if (distanceX !== 0)
                            currentCursorPosition.x = currentCursorPosition.x + (distanceX > 0 ? 1 : -1);
                        else if (distanceY !== 0)
                            currentCursorPosition.y = currentCursorPosition.y + (distanceY > 0 ? 1 : -1);

                    }

                    CursorWidget.move(currentCursorPosition, callback);
                },
                emulateEvents: function () {
                    if (!skipEvents) {
                        // moving events
                        var currentElement = CursorWidget.getElementUnderCursor(currentCursorPosition.x, currentCursorPosition.y),
                            eventPoint = currentCursorPosition;

                        if (inDragging && !dragElement)
                            dragElement = currentElement;

                        if (currentElement)
                            eventPoint = AutomationUtil.getEventOptionCoordinates(currentElement, currentCursorPosition);

                        var eventOptions = $.extend({
                            clientX: eventPoint.x,
                            clientY: eventPoint.y,
                            button: 0,
                            which: Util.isWebKit ? (inDragging ? Util.WHICH_PARAMETER.LEFT_BUTTON : Util.WHICH_PARAMETER.NO_BUTTON) : 1,
                            buttons: inDragging ? Util.BUTTONS_PARAMETER.LEFT : Util.BUTTONS_PARAMETER.NO_BUTTON
                        }, options);

                        var currentElementChanged = true;

                        //NOTE: when lastOverElement was in an iframe that removed, ie raises exception when we try to
                        // compare it with current element
                        var isLastOverElementInRemovedIframe = lastOverElement &&
                                                               Util.isElementInIframe(lastOverElement) &&
                                                               !Util.getIFrameByElement(lastOverElement);

                        if (Util.isIE && isLastOverElementInRemovedIframe) {
                            lastOverElement       = null;
                            currentElementChanged = true;
                        }
                        else
                            currentElementChanged = currentElement !== lastOverElement;

                        if (currentElementChanged && lastOverElement)
                            EventSimulator.mouseout(lastOverElement, $.extend({relatedTarget: currentElement}, eventOptions));

                        var eventName = Util.isTouchDevice ? 'touchmove' : 'mousemove',
                            el = Util.isTouchDevice ? dragElement : currentElement;

                        //NOTE: only in IE a 'mousemove' event is raised before a 'mouseover' one (B236966)
                        if (Util.isIE && currentElement)
                            EventSimulator[eventName](el, eventOptions);

                        if (currentElementChanged) {
                            if (currentElement)
                                EventSimulator.mouseover(currentElement, $.extend({relatedTarget: lastOverElement}, eventOptions));
                            lastOverElement = currentElement;
                        }

                        if (!Util.isIE && currentElement)
                            EventSimulator[eventName](el, eventOptions);

                        //NOTE: we need add extra 'mousemove' if element was changed
                        // because sometimes client script require several 'mousemove' events for element (T246904)
                        if (currentElementChanged && currentElement)
                            EventSimulator[eventName](el, eventOptions);
                    }
                    movingStepCallback();
                }
            });
        }

        async.series({
            scrollToTarget: function (callback) {
                var elementOffset = Util.isDomElement(to) ? Util.getOffsetPosition(to) : null;
                ScrollPlaybackAutomation.run(Util.isDomElement(to) ? to : targetPoint, options, currentDocument, function () {
                    if (Util.isDomElement(to)) {
                        var newElementOffset = Util.getOffsetPosition(to),
                            elementScroll = Util.getElementScroll($(to));

                        if (to !== document.documentElement) {
                            targetPoint.x += newElementOffset.left - elementOffset.left;
                            targetPoint.y += newElementOffset.top - elementOffset.top;
                        }

                        if (!/html/i.test(to.tagName) && Util.hasScroll(to, currentDocument)) {
                            targetPoint.x -= elementScroll.left;
                            targetPoint.y -= elementScroll.top;
                        }
                    }
                    targetScreenPoint = Util.offsetToClientCoords(targetPoint);
                    callback();
                });
            },

            setCursor: function (callback) {
                if (targetPoint.x < 0 || targetScreenPoint.x > $window.width() ||
                    targetScreenPoint.y < 0 || targetScreenPoint.y > $window.height) {
                    actionCallback();
                    return;
                }

                var windowTopResponse = null;

                if (window.top === window.self) {
                    var curCursorPosition = CursorWidget.getPosition(),
                        currentElement = curCursorPosition ?
                            CursorWidget.getElementUnderCursor(curCursorPosition.x, curCursorPosition.y) : null;

                    if (!currentElement || !(currentElement.tagName && currentElement.tagName.toLowerCase() === 'iframe')) {
                        CursorWidget.ensureCursorPosition(targetScreenPoint, false, callback);
                        return;
                    }

                    var pageCursorPosition = Util.clientToOffsetCoord(curCursorPosition),
                    //NOTE: after scroll in top window cursor position in iframe could be changed (if cursor was above iframe)
                        fixedPositionForIFrame = Util.getFixedPositionForIFrame(pageCursorPosition, currentElement.contentWindow);

                    if (fixedPositionForIFrame.x <= 0 || fixedPositionForIFrame.y <= 0)
                        CursorWidget.ensureCursorPosition(targetScreenPoint, false, callback);

                    MessageSandbox.pingIFrame(currentElement, CrossDomainMessages.MOVE_CURSOR_IN_IFRAME_PING, function (err) {
                        if (!err) {
                            //NOTE: move over iframe then move above top document
                            windowTopResponse = function (e) {
                                if (e.message.cmd === CrossDomainMessages.MOVE_FROM_IFRAME_RESPONSE_CMD) {
                                    MessageSandbox.off(MessageSandbox.SERVICE_MSG_RECEIVED, windowTopResponse);

                                    if (!e.message.point)
                                        CursorWidget.ensureCursorPosition(targetScreenPoint, false, callback);
                                    else if (CursorWidget.getPosition()) {
                                        CursorWidget.setPosition(Util.getFixedPosition(e.message.point, currentElement.contentWindow, true));
                                        NativeMethods.setTimeout.call(window, callback, 0);
                                    }
                                    else
                                        CursorWidget.ensureCursorPosition(Util.getFixedPosition(e.message.point, currentElement.contentWindow, true), true, callback);
                                }
                            };

                            MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, windowTopResponse);


                            MessageSandbox.sendServiceMsg({
                                cmd: CrossDomainMessages.MOVE_FROM_IFRAME_REQUEST_CMD,
                                rectangle: Util.getIFrameCoordinates(currentElement.contentWindow),
                                startPoint: Util.clientToOffsetCoord(targetScreenPoint),
                                endPoint: pageCursorPosition,
                                //NOTE: after scroll in top window cursor position in iframe could be changed (if cursor was above iframe)
                                cursorPosition: Util.getFixedPositionForIFrame(pageCursorPosition, currentElement.contentWindow)
                            }, currentElement.contentWindow);
                        }
                        else {
                            CursorWidget.ensureCursorPosition(targetScreenPoint, false, callback);
                            return;
                        }
                    }, true);
                }
                else {
                    //NOTE: move over top document than move above iframe
                    windowTopResponse = function (e) {
                        if (e.message.cmd === CrossDomainMessages.MOVE_TO_IFRAME_RESPONSE_CMD) {
                            MessageSandbox.off(MessageSandbox.SERVICE_MSG_RECEIVED, windowTopResponse);

                            if (!e.message.point || (e.message.point.x === targetScreenPoint.x && e.message.point.y === targetScreenPoint.y))
                                CursorWidget.ensureCursorPosition(targetScreenPoint, false, callback);
                            else if (CursorWidget.getPosition()) {
                                CursorWidget.setPosition(e.message.point);
                                NativeMethods.setTimeout.call(window, callback, 0);
                            }
                            else
                                CursorWidget.ensureCursorPosition(e.message.point, true, callback);
                        }
                    };

                    MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, windowTopResponse);

                    MessageSandbox.sendServiceMsg({
                        cmd: CrossDomainMessages.MOVE_TO_IFRAME_REQUEST_CMD,
                        point: targetScreenPoint
                    }, window.top);
                }
            },

            moveToTarget: function (callback) {
                currentCursorPosition = CursorWidget.getPosition();

                if (!currentCursorPosition) {
                    actionCallback();
                    return;
                }

                startX = currentCursorPosition.x;
                startY = currentCursorPosition.y;
                distanceX = targetScreenPoint.x - startX;
                distanceY = targetScreenPoint.y - startY;

                if (isMovingInIFrame) {
                    startX -= $(currentDocument).scrollLeft();
                    startY -= $(currentDocument).scrollTop();
                }

                movingTime = Math.max(Math.abs(distanceX), Math.abs(distanceY)) / (inDragging ? AutomationSettings.MOVING_SPEED_IN_DRAGGING : AutomationSettings.MOVING_SPEED);

                if (inDragging)
                    movingTime = Math.max(movingTime, AutomationSettings.MINIMUM_MOVING_TIME);

                async.whilst(
                    //is cursor in the target
                    function () {
                        if (isMovingInIFrame)
                            return (currentCursorPosition.x + $(currentDocument).scrollLeft()) !== targetScreenPoint.x ||
                                (currentCursorPosition.y + $(currentDocument).scrollTop()) !== targetScreenPoint.y;

                        return currentCursorPosition.x !== targetScreenPoint.x || currentCursorPosition.y !== targetScreenPoint.y;
                    },

                    //moving step
                    function (movingCallback) {
                        NativeMethods.setTimeout.call(window, function () {
                            nextStep(movingCallback);
                        }, 0);
                    },

                    //save cursor position
                    function (err) {
                        if (err)
                            return;
                        callback();
                    }
                );
            },


            callback: function () {
                actionCallback();
            }
        });
    };
});