TestCafeClient.define('Automation.RClick.Playback', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        async = Hammerhead.async,
        Util = Hammerhead.Util,
        EventSimulator = Hammerhead.EventSimulator,
        NativeMethods = Hammerhead.NativeMethods,
        Settings = require('Settings'),
        AutomationUtil = require('Automation.Util'),
        AutomationSettings = require('Automation.Settings'),
        CursorWidget = require('UI.Cursor'),

        MovePlaybackAutomation = require('Automation.Move.Playback');

    exports.run = function (el, options, actionCallback) {
        options = {
            ctrl: options.ctrl,
            alt: options.alt,
            shift: options.shift,
            meta: options.meta,
            offsetX: options.offsetX,
            offsetY: options.offsetY,
            caretPos: options.caretPos
        };

        var isSvgElement = Util.isSvgElement(el),
            target = Util.isContainOffset(el, options.offsetX, options.offsetY) ?
                el : AutomationUtil.getMouseActionPoint(el, options, false),

            notPrevented = true,
            screenPoint = null,
            eventPoint = null,
            eventOptions = null,
            topElement = null,
            isInvisibleElement = false,
            currentTopElement = null;

        if (options.offsetX)
            options.offsetX = Math.round(options.offsetX);
        if (options.offsetY)
            options.offsetY = Math.round(options.offsetY);

        async.series({
            moveCursorToElement: function (callback) {
                if (Settings.RECORDING && !Settings.PLAYBACK && !Util.isElementVisible(el)) {
                    topElement = el;

                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.ACTION_STEP_DELAY);
                    return;
                }

                MovePlaybackAutomation.run(target, false, options, function () {
                    if ((isSvgElement && Util.isOpera) || ($(el).is('tref')))
                        topElement = el; //NOTE: document.elementFromPoint can't find this element
                    else {
                        screenPoint = AutomationUtil.getMouseActionPoint(el, options, true);
                        eventPoint = AutomationUtil.getEventOptionCoordinates(el, screenPoint);

                        eventOptions = $.extend({
                            clientX: eventPoint.x,
                            clientY: eventPoint.y,
                            button: Util.BUTTON.RIGHT,
                            which: Util.WHICH_PARAMETER.RIGHT_BUTTON,
                            buttons: Util.BUTTONS_PARAMETER.RIGHT_BUTTON
                        }, options);

                        topElement = AutomationUtil.getElementUnderCursor(screenPoint.x, screenPoint.y, null, target);

                        if (!topElement) {
                            isInvisibleElement = true;
                            topElement = el;
                        }
                    }
                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.ACTION_STEP_DELAY);
                });
            },

            cursorMouseDown: function (callback) {
                CursorWidget.rMouseDown(callback);
            },

            mousedown: function (callback) {
                var activeElement = Util.getActiveElement(),
                //in IE focus is not raised if element was focused before click, even if focus is lost during mousedown
                    needFocus = !(Util.isIE && activeElement === topElement);

                notPrevented = EventSimulator.mousedown(topElement, eventOptions);

                if (!isInvisibleElement && screenPoint) {
                    currentTopElement = AutomationUtil.getElementUnderCursor(screenPoint.x, screenPoint.y, null, target);

                    if (currentTopElement)
                        topElement = currentTopElement;
                }

                if (notPrevented === false) {
                    callback();
                    return;
                }

                //NOTE: For contentEditable elements we should call focus directly for action's element because
                //option 'caretPos' is indicated this element and topElement may be a child of this element
                AutomationUtil.focusAndSetSelection(Util.isContentEditableElement(el) ? el : topElement, options, needFocus, callback);
            },

            cursorMouseUp: function (callback) {
                CursorWidget.mouseUp(callback);
            },

            mouseup: function (callback) {
                EventSimulator.mouseup(topElement, eventOptions);

                if (!isInvisibleElement && screenPoint) {
                    currentTopElement = CursorWidget.getElementUnderCursor(screenPoint.x, screenPoint.y, null, target);

                    if (currentTopElement)
                        topElement = currentTopElement;
                }
                NativeMethods.setTimeout.call(window, callback, AutomationSettings.ACTION_STEP_DELAY);
            },

            contextmenu: function () {
                EventSimulator.contextmenu(topElement, eventOptions);
                AutomationUtil.focusLabelChildElement(topElement);

                actionCallback();
            }
        });
    };
});
