TestCafeClient.define('Automation.Select.Playback', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        async = Hammerhead.async,
        Util = Hammerhead.Util,
        EventSandbox = Hammerhead.EventSandbox,
        EventSimulator = Hammerhead.EventSimulator,
        NativeMethods = Hammerhead.NativeMethods,
        Settings = require('Settings'),
        ContentEditableHelper = Hammerhead.ContentEditableHelper,
        CursorWidget = require('UI.Cursor'),
        TextSelection = Hammerhead.TextSelection,

        AutomationSettings = require('Automation.Settings'),
        AutomationUtil = require('Automation.Util'),
        AutomationSelectUtil = require('Automation.Select.Util'),
        ScrollPlaybackAutomation = require('Automation.Scroll.Playback'),
        MovePlaybackAutomation = require('Automation.Move.Playback');

    exports.run = function (el, options, runCallback) {
        var isTextarea = el.tagName.toLowerCase() === 'textarea',
            isTextEditable = Util.isTextEditableElement(el),
            isContentEditable = Util.isContentEditableElement(el),
            topElement = null,
            currentTopElement = null,

            startPosition = null,
            endPosition = null,

            pointFrom = null,
            pointTo = null,
            screenPointTo = null,

            screenPointFrom = null,
            eventPointTo = null,
            eventPointFrom = null,
            eventOptionsStart = null,
            eventOptionsEnd = null,
            notPrevented = true,
            point = null,

            //NOTE: options to get right selection position to mouse movement
            correctOptions = null;

        //determination of selection positions by arguments
        var processedOptions = AutomationSelectUtil.getProcessedOptions(el, options);

        startPosition = processedOptions.startPosition;
        endPosition = processedOptions.endPosition;

        if (Settings.RECORDING && !Settings.PLAYBACK && !Util.isElementVisible(el)) {
            topElement = el;

            NativeMethods.setTimeout.call(window, runCallback, AutomationSettings.ACTION_STEP_DELAY);
            return;
        }

        async.series({
            scrollToElement: function (callback) {
                ScrollPlaybackAutomation.run(el, {}, null, function () {
                        AutomationSelectUtil.getCorrectOptions(el, function (opt) {
                            correctOptions = opt;
                            callback();
                        });
                    }
                );
            },

            moveToStart: function (callback) {
                if ((isTextEditable && el.value.length > 0) ||
                    (isContentEditable && ContentEditableHelper.getContentEditableValue(el).length)) {
                    pointFrom = AutomationSelectUtil.getSelectPositionCoordinates(el, startPosition, endPosition, true, correctOptions);
                    pointTo = AutomationSelectUtil.getSelectPositionCoordinates(el, startPosition, endPosition, false, correctOptions);
                }
                else {
                    pointFrom = Util.findCenter(el);
                    pointTo = pointFrom;
                }

                AutomationSelectUtil.scrollElementByPoint(el, pointFrom);

                point = AutomationSelectUtil.updatePointByScrollElement(el, pointFrom);

                MovePlaybackAutomation.run(point, false, options, function () {
                    screenPointFrom = Util.offsetToClientCoords(point);
                    eventPointFrom = AutomationUtil.getEventOptionCoordinates(el, screenPointFrom);

                    eventOptionsStart = $.extend({
                        clientX: eventPointFrom.x,
                        clientY: eventPointFrom.y
                    }, options);

                    topElement = CursorWidget.getElementUnderCursor(screenPointFrom.x, screenPointFrom.y);

                    if (!topElement || !Util.isEditableElement(topElement)) {
                        runCallback();
                        return;
                    }

                    isTextEditable = Util.isTextEditableElement(topElement);
                    isTextarea = topElement.tagName.toLowerCase() === 'textarea';

                    NativeMethods.setTimeout.call(window, callback, AutomationSettings.DRAG_ACTION_STEP_DELAY);
                });
            },

            cursorMousseDown: function (callback) {
                CursorWidget.lMouseDown(callback);
            },

            mousedown: function (callback) {
                //NOTE: in webkit and ie raising mousedown event opens select element's dropdown,
                // therefore we should handle it and hide the dropdown (B236416)
                var needHandleMousedown = (Util.isWebKit || Util.isIE) && Util.isSelectElement(el),
                    wasPrevented = null;

                if (needHandleMousedown) {
                    var onmousedown = function (e) {
                        wasPrevented = e.defaultPrevented;
                        Util.preventDefault(e);
                        NativeMethods.removeEventListener.call(el, 'mousedown', onmousedown, false);
                    };

                    NativeMethods.addEventListener.call(el, 'mousedown', onmousedown, false);
                }

                if (Util.isTouchDevice)
                    notPrevented = EventSimulator.touchstart(topElement, eventOptionsStart);
                else
                    notPrevented = EventSimulator.mousedown(topElement, eventOptionsStart);

                currentTopElement = CursorWidget.getElementUnderCursor(screenPointFrom.x, screenPointFrom.y);

                if (currentTopElement)
                    topElement = currentTopElement;

                if (notPrevented === false) {
                    if (needHandleMousedown && !wasPrevented)
                        notPrevented = true;
                    else {
                        callback();
                        return;
                    }
                }

                //NOTE: For contentEditable elements we should call focus directly for action's element because
                //option 'caretPos' is indicated for this element and topElement may be a child of this element
                if (isContentEditable)
                    topElement = el;

                EventSandbox.focus(isContentEditable ? ContentEditableHelper.findContentEditableParent(topElement) : topElement, function () {
                    pointTo = AutomationSelectUtil.getSelectPositionCoordinates(topElement, startPosition, endPosition, false, correctOptions);

                    if (isContentEditable && !pointTo) {
                        pointTo = AutomationSelectUtil.getSelectionLastVisiblePosition(el, startPosition, endPosition, correctOptions) ||
                                  Util.findCenter(el);
                    }

                    AutomationSelectUtil.scrollElementByPoint(topElement, pointTo);
                    pointTo = AutomationSelectUtil.updatePointByScrollElement(topElement, pointTo);

                    MovePlaybackAutomation.run(pointTo, false, options, function () {
                        callback();
                    });
                }, false, true);
            },

            getCorrectOptions: function (callback) {
                if ((isTextEditable && topElement.value.length > 0) ||
                    (isContentEditable && ContentEditableHelper.getContentEditableValue(topElement).length)) {
                    if (pointTo) {
                        point = pointTo;
                        callback();
                        return;
                    }

                    AutomationSelectUtil.getCorrectOptions(topElement, function (opt) {
                        point = AutomationSelectUtil.getSelectPositionCoordinates(topElement, startPosition, endPosition, false, opt);
                        callback();
                    });
                }
                else
                    callback();
            },

            setFinalSelection: function (callback) {
                if (!point)
                    point = Util.findCenter(topElement);

                AutomationSelectUtil.scrollElementByPoint(topElement, point);
                point = AutomationSelectUtil.updatePointByScrollElement(topElement, point);

                screenPointTo = Util.offsetToClientCoords(point);
                eventPointTo = AutomationUtil.getEventOptionCoordinates(topElement, screenPointTo);

                eventOptionsEnd = $.extend({
                    clientX: eventPointTo.x,
                    clientY: eventPointTo.y
                }, options);

                if ((isTextEditable || isContentEditable) && notPrevented !== false) {
                    //NOTE: The same cursor position may correspond to different nodes
                    //only if we know which nodes should be selected in result we should select it directly
                    if (isContentEditable && options.startNode && options.endNode)
                        AutomationSelectUtil.selectContentEditableByOptions(el, startPosition, endPosition, options);
                    else
                        TextSelection.select(topElement, startPosition, endPosition);
                }
                callback();
            },

            cursorMouseUp: function (callback) {
                CursorWidget.mouseUp(callback);
            },

            mouseup: function () {
                if (Util.isTouchDevice)
                    EventSimulator.touchend(topElement, eventOptionsEnd);
                else
                    EventSimulator.mouseup(topElement, eventOptionsEnd);
                runCallback();
            }
        });
    };
});
