TestCafeClient.define('Automation.Type.Playback', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        async = Hammerhead.async,
        Util = Hammerhead.Util,
        EventSandbox = Hammerhead.EventSandbox,
        EventSimulator = Hammerhead.EventSimulator,
        NativeMethods = Hammerhead.NativeMethods,
        Settings = require('Settings'),
        TextSelection = Hammerhead.TextSelection,
        ContentEditableHelper = Hammerhead.ContentEditableHelper,

        AutomationSettings = require('Automation.Settings'),
        TypeCharPlaybackAutomation = require('Automation.TypeChar.Playback'),
        ClickPlaybackAutomation = require('Automation.Click.Playback');

    function findTextEditableChild(el) {
        var isContentEditable = Util.isContentEditableElement(el);

        if (!Util.isTextEditableElement(el) && !isContentEditable) {
            var $innerInputElement = $(el).find('*').filter(function () {
                return Util.isTextEditableElementAndEditingAllowed(this);
            });

            if ($innerInputElement.length)
                return $innerInputElement[0];
        }

        return el;
    }

    exports.run = function (el, text, options, actionCallback) {
        text = text.toString();

        var length = text.length,
            curElement = null,
            elementForTyping = null,
            currPos = 0,
            isTextEditable = null,
            isContentEditable = Util.isContentEditableElement(el),
            notPrevented = true;

        if (options.offsetX)
            options.offsetX = Math.round(options.offsetX);
        if (options.offsetY)
            options.offsetY = Math.round(options.offsetY);

        curElement = findTextEditableChild(el);
        isTextEditable = Util.isTextEditableElementAndEditingAllowed(curElement);

        if (Settings.RECORDING && !Settings.PLAYBACK && !Util.isElementVisible(curElement)) {
            actionCallback();
            return;
        }

        async.series({
            click: function (seriaCallback) {
                if (Util.getActiveElement() !== curElement)
                    ClickPlaybackAutomation.run(curElement, options, function () {
                        NativeMethods.setTimeout.call(window, seriaCallback, AutomationSettings.ACTION_STEP_DELAY);
                    });
                else {
                    if (isTextEditable)
                        EventSandbox.watchElementEditing(curElement);

                    if (isTextEditable || isContentEditable) {
                        if (!isNaN(parseInt(options.caretPos)) && options.caretPos !== TextSelection.getSelectionStart(curElement))
                            TextSelection.select(curElement, options.caretPos, options.caretPos);
                    }
                    seriaCallback();
                }
            },

            calculateTargetElement: function (seriaCallback) {
                if ((!isContentEditable && Util.getActiveElement() !== curElement) ||
                    (isContentEditable && Util.getActiveElement() !== ContentEditableHelper.findContentEditableParent(curElement))) {
                    actionCallback();
                    return;
                }

                curElement = isContentEditable ? el : Util.getActiveElement();

                seriaCallback();
            },

            type: function () {
                if (options.replace) {
                    if (isTextEditable)
                        TextSelection.select(curElement);
                    else if (isContentEditable)
                        TextSelection.deleteSelectionContents(curElement, true);
                }

                var hasTextDigits = /^\d/.test(text);

                async.whilst(
                    //are not all symbols typed
                    function () {
                        return currPos < length;
                    },

                    //typing symbol
                    function (typingCallback) {
                        var char = text.charAt(currPos),
                            keyCode = Util.getKeyCodeByChar(char),
                            charCode = text.charCodeAt(currPos),
                            key = keyCode === 13 ? 'Enter' : char;

                        async.series({
                            startTypeSymbol: function (seriaCallback) {
                                notPrevented = EventSimulator.keydown(curElement, $.extend(options, {
                                    keyCode: keyCode,
                                    key: key
                                }));
                                delete options['keyCode'];

                                if (notPrevented !== false) {
                                    //T162478: Wrong typing and keys pressing in editor
                                    if (!isContentEditable && curElement !== Util.getActiveElement()) {
                                        curElement = Util.getActiveElement();
                                        curElement = findTextEditableChild(curElement);
                                        isTextEditable = Util.isTextEditableElementAndEditingAllowed(curElement);
                                    }

                                    //Element for typing can change last time only after keydown event
                                    elementForTyping = curElement;

                                    notPrevented = EventSimulator.keypress(curElement, $.extend(options, {
                                        keyCode:  keyCode,
                                        charCode: charCode,
                                        key: key
                                    }));
                                    delete options['charCode'];

                                    //T162478: Wrong typing and keys pressing in editor
                                    if (!isContentEditable && curElement !== Util.getActiveElement()) {
                                        curElement = Util.getActiveElement();
                                        curElement = findTextEditableChild(curElement);
                                    }

                                    if (notPrevented === false){
                                        //change event must not be raised after prevented keypress even if element value was changed (B253816)
                                        EventSandbox.restartWatchingElementEditing(elementForTyping);
                                        NativeMethods.setTimeout.call(window, seriaCallback, AutomationSettings.ACTION_STEP_DELAY);
                                        return;
                                    }
                                    var currentChar = text.charAt(currPos),
                                        isDigit = /^\d$/.test(currentChar),
                                        prevChar = currPos === 0 ? null : text.charAt(currPos - 1),
                                        isInputTypeNumber = Util.isInputTypeNumber(elementForTyping);

                                    if(isInputTypeNumber){
                                        var selectionStart = TextSelection.getSelectionStart(elementForTyping),
                                            valueLength = elementForTyping.value.length;

                                        if (!isDigit) {
                                            // NOTE: we type '.' or '-' only if it first symbol and input already has value
                                            if(!hasTextDigits &&
                                               (currentChar === '.' || currentChar === '-' && valueLength) &&
                                               selectionStart === 0)
                                                TypeCharPlaybackAutomation.run(elementForTyping, currentChar);

                                            NativeMethods.setTimeout.call(window, seriaCallback, AutomationSettings.ACTION_STEP_DELAY);
                                            return;
                                        }

                                        //NOTE: we add '.' or '-' to digit to be able set value to input
                                        if(prevChar === '.' || prevChar === '-' && !valueLength)
                                            TypeCharPlaybackAutomation.run(elementForTyping, prevChar + currentChar);
                                        else
                                            TypeCharPlaybackAutomation.run(elementForTyping, currentChar);

                                        NativeMethods.setTimeout.call(window, seriaCallback, AutomationSettings.ACTION_STEP_DELAY);
                                        return;
                                    }
                                    else
                                        TypeCharPlaybackAutomation.run(elementForTyping, currentChar);
                                }

                                NativeMethods.setTimeout.call(window, seriaCallback, AutomationSettings.ACTION_STEP_DELAY);
                            },

                            endTypeSymbol: function () {
                                EventSimulator.keyup(curElement, $.extend(options, {
                                    keyCode: keyCode,
                                    key: key
                                }));
                                delete options['keyCode'];
                                currPos++;

                                typingCallback();
                            }
                        });
                    },

                    //callback
                    function () {
                        actionCallback();
                    });
            }
        });
    };
});
