TestCafeClient.define('Base.IFrameDispatcher', function (require, exports) {
    var testRunInitializedCallback = null,
        runStep = null;

    (function () {
        var Hammerhead = HammerheadClient.get('Hammerhead'),
            MessageSandbox = Hammerhead.MessageSandbox,
            NativeMethods = Hammerhead.NativeMethods,
            Util = Hammerhead.Util,
            XhrBarrier = require('ActionBarrier.Xhr'),
            CrossDomainMessages = require('Base.CrossDomainMessages'),
            Automation = require('Automation'),
            AutomationIFrameBehavior = require('Automation.IFrameBehavior'),
            CursorWidget = require('UI.Cursor'),
            IFrameTestRunner = require('TestRunner.IFrameTestRunner');

        var pageInitialzied = false,
            actionsQueue = [],
            testRunnerInitialized = false,

            testRunner = new IFrameTestRunner();

        exports.MESSAGE_RECEIVED = 'messageReceived';

        function runOrPushInQueue(fn) {
            if (!pageInitialzied)
                actionsQueue.push(fn);
            else
                fn();
        }

        function onRunStepsMsg(msg) {
            if (!testRunnerInitialized) {
                testRunnerInitialized = true;
                testRunInitializedCallback(testRunner, function (runStepInContext) {
                    runStep = runStepInContext;
                });

                runOrPushInQueue(function () {
                    runStep(msg.stepName, msg.step, 0, function () {
                        testRunner.act._start.apply(testRunner, arguments);
                    });
                });
            }
            else {
                runOrPushInQueue(function () {
                    runStep(msg.stepName, msg.step, 0, function () {
                        testRunner.run.apply(testRunner, arguments);
                    });
                });
            }
        }

        exports.init = function (onTestRunInitialized) {
            Automation.init();

            testRunInitializedCallback = onTestRunInitialized;

            var eventEmitter = new Util.EventEmitter();

            exports.on = function () {
                eventEmitter.on.apply(eventEmitter, arguments);
            };

            exports.off = function () {
                eventEmitter.off.apply(eventEmitter, arguments);
            };

            MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, function (e) {
                var msg = e.message;
                switch (msg.cmd) {
                    case CrossDomainMessages.IFRAME_TEST_RUNNER_RUN_CMD:
                        onRunStepsMsg(msg);
                        break;

                    case CrossDomainMessages.IFRAME_TEST_RUNNER_PING_DISPATCHER_CMD:
                        if (msg.isPingRequest) {
                            MessageSandbox.sendServiceMsg({
                                cmd: CrossDomainMessages.IFRAME_TEST_RUNNER_PING_DISPATCHER_CMD,
                                isPingResponse: true
                            }, window.top);
                        }
                        break;

                    default:
                        eventEmitter.emit(exports.MESSAGE_RECEIVED, msg);
                }
            });
        };

        //Const
        var PAGE_LOAD_TIMEOUT = 3000,
            ANIMATIONS_WAIT_DELAY = 200;

        //Util
        function waitPageLoad(callback) {
            var internal$ = Hammerhead.$,
                loaded = false,
                callbackWrapper = function () {
                    if (!loaded) {
                        loaded = true;
                        callback();
                    }
                };

            internal$(window).load(callbackWrapper);
            internal$(document).ready(function () {
                //NOTE: an iFrame may be removed in this moment
                //In IE, a removed iframe may contain window.top while the setTimeout method is missing.
                // This happens because after the iFrame is removed, all overridden methods are set to 'undefined'.
                if (Util.isIFrameWindowInDOM(window))
                    NativeMethods.setTimeout.call(window, callbackWrapper, PAGE_LOAD_TIMEOUT);
            });
        }

        XhrBarrier.init();

        var initialized = false;

        AutomationIFrameBehavior.init();
        CursorWidget.init();

        waitPageLoad(function () {
            if (!initialized) {
                initialized = true;

                NativeMethods.setTimeout.call(window, function () {
                    XhrBarrier.waitPageInitialRequests(function () {
                        MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, function (e) {
                            var msg = e.message;

                            if (msg.cmd === CrossDomainMessages.IFRAME_TEST_RUNNER_WAITING_STEP_COMPLETION_RESPONSE_CMD) {
                                testRunner.onStepCompleted();

                                if (!testRunner.testIterator.state.stopped) {
                                    MessageSandbox.sendServiceMsg({
                                        cmd: CrossDomainMessages.IFRAME_TEST_RUNNER_STEP_COMPLETED_CMD
                                    }, window.top);
                                }
                            }
                        });

                        var stepWaitingRequestMsg = {
                            cmd: CrossDomainMessages.IFRAME_TEST_RUNNER_WAITING_STEP_COMPLETION_REQUEST_CMD
                        };

                        MessageSandbox.sendServiceMsg(stepWaitingRequestMsg, window.top);

                        pageInitialzied = true;

                        for (var i = 0; i < actionsQueue.length; i++) {
                            actionsQueue[i]();
                        }
                    });
                }, ANIMATIONS_WAIT_DELAY);
            }
        });
    })();
});