TestCafeClient.define('Base.JavascriptExecutor', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,
        ShadowUI = Hammerhead.ShadowUI,
        MessageSandbox = Hammerhead.MessageSandbox,
        JSProcessor = Hammerhead.JSProcessor,
        CustomSelectors = require('Base.jQueryExtensions.Selectors'),

        SAFE_EXECUTOR_DIV_CLASS = 'safeExecutor',
        SAFE_EXECUTOR_IFRAME_CLASS = 'safeExecutorIFrame',

        $safeExecutorDiv = null,
        safeExecutorIFrame = null,
        safeExecutorDocumentBodyOverrided = null;

    //Errors
    exports.EMPTY_SELECTOR_ERROR = 'Empty element selector';
    exports.INVALID_ELEMENTS_IN_JQUERY_OBJECT_ERROR = 'jQuery object contains invalid elements';
    exports.INVALID_ELEMENTS_IN_ARRAY_ERROR = 'Array contains invalid elements';
    exports.INVALID_OBJECT_ERROR = 'Invalid object type';
    exports.RECURSIVE_JQUERY_CALLING_ERROR = 'Maximum call stack size exceeded';
    exports.SELECTOR_DOES_NOT_CONTAIN_ELEMENT_ERROR = 'Selector doesn\'t contain any element';
    exports.JAVASCRIPT_ERROR_PREFIX = 'Javascript error: ';

    var isInIFrameWindow = window.top !== window.self;

    var getSafeExecutorDiv = function () {
        if (!$safeExecutorDiv || !isElementInDom($safeExecutorDiv)) {
            $safeExecutorDiv = ShadowUI.getRoot().find(' > div[class]').filter(function () {
                return $(this).attr('class').indexOf(SAFE_EXECUTOR_DIV_CLASS) === 0;
            });
            if (!$safeExecutorDiv.length) {
                $safeExecutorDiv = $(NativeMethods.createElement.call(document, 'div'));
                NativeMethods.appendChild.call(ShadowUI.getRoot()[0], $safeExecutorDiv[0]);
                ShadowUI.addClass($safeExecutorDiv, SAFE_EXECUTOR_DIV_CLASS);
            }
        }
        return $safeExecutorDiv;
    };

    var getSafeExecutorIFrame = function () {
        if (!safeExecutorIFrame || !isElementInDom($(safeExecutorIFrame)))
            initSafeExecutorIFrame();
        return safeExecutorIFrame;
    };

    var initSafeExecutorIFrame = function () {
        createSafeExecutorIFrame();
        overrideSafeExecutorIFrameProperties();
    };

    var createSafeExecutorIFrame = function () {
        var $safeExecutorIFrame = getSafeExecutorDiv().find('iframe');
        if (!$safeExecutorIFrame.length) {
            $safeExecutorIFrame = $(NativeMethods.createElement.call(document, 'iframe'));
            $safeExecutorIFrame.css('display', 'none');
            NativeMethods.appendChild.call(getSafeExecutorDiv()[0], $safeExecutorIFrame[0]);
            ShadowUI.addClass($safeExecutorIFrame, SAFE_EXECUTOR_IFRAME_CLASS);
        }
        safeExecutorIFrame = $safeExecutorIFrame.get(0);
        $safeExecutorIFrame.load(overrideSafeExecutorIFrameProperties);
    };

    var overrideSafeExecutorIFrameProperties = function () {
        if (!safeExecutorIFrame || !isElementInDom($(safeExecutorIFrame)))
            createSafeExecutorIFrame();

        //NOTE: we override safeExecutorIFrame.contentWindow.document.body property to allow parsing strings like 'document.body.childNodes[0]'
        // or '$('div:first', document.body)' . Safari throws exception on this, so we use another approach in it (change input string during parsing)
        try {
            if (Object.getOwnPropertyDescriptor && Object.getOwnPropertyDescriptor(safeExecutorIFrame.contentWindow.document, 'body') && Object.getOwnPropertyDescriptor(safeExecutorIFrame.contentWindow.document, 'body').configurable) {
                Object.defineProperty(safeExecutorIFrame.contentWindow.document, 'body', {
                    get: function () {
                        return window.document.body;
                    },
                    configurable: true
                });
                safeExecutorDocumentBodyOverrided = true;
            }
            else safeExecutorDocumentBodyOverrided = false;
        }
        catch (e) {
            safeExecutorDocumentBodyOverrided = false;
        }

        var sandboxedJQuery = CustomSelectors.create(Hammerhead.SandboxedJQuery);

        CustomSelectors.init();

        safeExecutorIFrame.contentWindow.$ = safeExecutorIFrame.contentWindow.jQuery = function (selector, context) {
            if (selector === arguments.callee || context === arguments.callee)
                throw new Error(exports.RECURSIVE_JQUERY_CALLING_ERROR);
            else
                return sandboxedJQuery(selector, context);
        };
        safeExecutorIFrame.contentWindow.alert = safeExecutorIFrame.contentWindow.confirm = safeExecutorIFrame.contentWindow.prompt = new Function();

        safeExecutorIFrame.contentWindow[JSProcessor.CALL_METHOD_METH_NAME] = window[JSProcessor.CALL_METHOD_METH_NAME];
        safeExecutorIFrame.contentWindow[JSProcessor.GET_LOCATION_METH_NAME] = window[JSProcessor.GET_LOCATION_METH_NAME];
        safeExecutorIFrame.contentWindow[JSProcessor.GET_PROPERTY_METH_NAME] = window[JSProcessor.GET_PROPERTY_METH_NAME];
        safeExecutorIFrame.contentWindow[JSProcessor.PROCESS_SCRIPT_METH_NAME] = window[JSProcessor.PROCESS_SCRIPT_METH_NAME];
        safeExecutorIFrame.contentWindow[JSProcessor.SET_LOCATION_METH_NAME] = new Function();
        safeExecutorIFrame.contentWindow[JSProcessor.SET_PROPERTY_METH_NAME] = new Function();
    };

    var isElementInDom = function($el){
        return !!$el.parents('body').length;
    };

    var convertToJQueryObject = function (obj) {
        var $result = null,
            error = null;

        var createResultObject = function () {
            return {
                result: $result,
                error: error
            };
        };

        if (Util.isJQueryObj(obj)) {
            if (isDomElementsCollection(obj))
                $result = obj;
            else
                error = exports.INVALID_ELEMENTS_IN_JQUERY_OBJECT_ERROR;
            return createResultObject();
        }

        if (typeof obj === 'string' || Util.isDomElement(obj) || obj instanceof NodeList || obj instanceof HTMLCollection) {
            try {
                var res = $(obj);
                if (Util.isJQueryObj(res))
                    $result = res;
                return createResultObject();
            }
            catch (err) {
                error = exports.JAVASCRIPT_ERROR_PREFIX + err;
                return createResultObject();
            }
        }

        if ($.isArray(obj)) {
            $result = $();
            for (var i = 0; i < obj.length; i++) {
                var converted = convertToJQueryObject(obj[i]);
                if (converted.error) {
                    error = converted.error;
                    return createResultObject();
                }
                else if (!converted.result) {
                    error = exports.INVALID_ELEMENTS_IN_ARRAY_ERROR;
                    return createResultObject();
                }
                $result = $result.add(converted.result);
            }
            return createResultObject();
        }

        if (typeof obj === 'function') {
            $result = exports.eval(obj + '()', function (err) {
                error = exports.JAVASCRIPT_ERROR_PREFIX + err;
            });
            if (error)
                return createResultObject();
            return convertToJQueryObject($result);
        }

        return createResultObject();
    };

    function isDomElementsCollection(obj) {
        if (obj instanceof NodeList || obj instanceof HTMLCollection || $.isArray(obj) || Util.isJQueryObj(obj)) {
            for (var i = 0; i < obj.length; i++)
                if (!Util.isDomElement(obj[i]))
                    return false;
            return true;
        }
        else
            return false;
    }

    function evalExpression(expression) {
        var scriptToEval = [];

        scriptToEval.push('(function (window, document, jQuery, $) {');
        scriptToEval.push('var savedAlert = window.alert;window.alert = new Function();');
        scriptToEval.push('var savedConfirm = window.confirm;window.confirm = new Function();');
        scriptToEval.push('var savedPrompt = window.prompt;window.prompt = new Function();');
        scriptToEval.push('var savedConsoleLog = null; if(window.console) {savedConsoleLog = window.console.log;window.console.log = new Function();}');
        scriptToEval.push('var saved$ = window.$, savedJQuery = window.jQuery; window.$ = $; window.jQuery = $;');
        scriptToEval.push('var err = null;');
        scriptToEval.push('try { var res = (function () { return ' + JSProcessor.process(expression) + '})(); } catch (e) {err = e;}');
        scriptToEval.push('window.alert = savedAlert;');
        scriptToEval.push('window.confirm = savedConfirm;');
        scriptToEval.push('window.prompt = savedPrompt;');
        scriptToEval.push('if(savedConsoleLog){window.console.log = savedConsoleLog;}');
        scriptToEval.push('window.$ = saved$; window.jQuery = savedJQuery;');
        scriptToEval.push('if(err)throw err;');
        scriptToEval.push('return res;');
        scriptToEval.push('})(window.parent, window.parent.document, window.$, window.$)');

        return getSafeExecutorIFrame().contentWindow.eval(scriptToEval.join(''));
    }

    exports.eval = function (expression, errorCallback) {
        try {
            if (!safeExecutorDocumentBodyOverrided)
                expression = expression.replace(/(^|[^a-z0-9_\$])document.body($|[^a-z0-9_\$])/g, function (substr, charBefore, charAfter) {
                    return charBefore + (isInIFrameWindow ? 'window.document.body' : 'window.top.document.body') + charAfter;
                });

            return evalExpression(expression);
        }
        catch (err) {
            errorCallback(err);
        }
    };

    exports.parseSelector = function (selector, parseDomElementsOrJqueryObjectsOnly, callback, context) {
        if (!context || !(window.top === window.self && context.top !== context.self)) {
            callback(exports.parseSelectorSync(selector, parseDomElementsOrJqueryObjectsOnly));
            return;
        }

        currentParseCallback = callback;

        var msg = {
            cmd: IFRAME_PARSE_SELECTOR_REQUEST_MSG_CMD,
            selector: selector,
            parseDomElementsOrJqueryObjectsOnly: parseDomElementsOrJqueryObjectsOnly
        };

        MessageSandbox.sendServiceMsg(msg, context);
    };

    exports.parseSelectorSync = function (selector, parseDomElementsOrJqueryObjectsOnly) {
        var evalResults = null,
            $elements = null,
            $visibleElements = null,
            errorMessage = '';

        function isCombinationOfSelectAndChild() {
            return $elements.length === 2 && Util.isSelectElement($elements[0]) &&
                /option|optgroup/.test($elements[1].tagName.toLowerCase()) && $elements.eq(0).has($elements.last()).length;
        }

        if (!selector.length)
            return {
                $elements: null,
                length: 0,
                $visibleElements: null,
                visibleLength: 0,
                iframeContext: false,
                error: exports.EMPTY_SELECTOR_ERROR,
                selector: selector
            };

        evalResults = exports.eval(selector, function (err) {
            errorMessage = exports.JAVASCRIPT_ERROR_PREFIX + err.message;
        });

        if (!errorMessage) {
            if (Util.isJQueryObj(evalResults)) {
                if (isDomElementsCollection(evalResults))
                    $elements = evalResults;
                else
                    errorMessage = exports.INVALID_ELEMENTS_IN_JQUERY_OBJECT_ERROR;
            }
            else if (parseDomElementsOrJqueryObjectsOnly) {
                if (Util.isDomElement(evalResults) || isDomElementsCollection(evalResults))
                    $elements = $(evalResults);
                else {
                    errorMessage = exports.SELECTOR_DOES_NOT_CONTAIN_ELEMENT_ERROR;
                }
            }
            else {
                var converted = convertToJQueryObject(evalResults);
                if (converted.error) {
                    errorMessage = converted.error;
                }
                else if (!converted.result)
                    errorMessage = exports.INVALID_OBJECT_ERROR;
                else
                    $elements = converted.result;
            }
        }

        //NOTE: we combine clicks on select and option tags to the one action
        var combineSelectAndOption = false;

        if ($elements && !errorMessage) {
            combineSelectAndOption = isCombinationOfSelectAndChild();

            $visibleElements = combineSelectAndOption ? $elements :
                $elements.filter(function (index) {
                    return Util.isElementVisible($elements[index]);
                });
        }

        return {
            $elements: $elements,
            length: combineSelectAndOption ? 1 : ($elements ? $elements.length : 0),
            $visibleElements: $visibleElements,
            visibleLength: combineSelectAndOption ? 1 : ($visibleElements ? $visibleElements.length : 0),
            iframeContext: false,
            error: errorMessage,
            selector: selector,
            evalResults: evalResults
        };
    };

    exports.init = function () {
        //B238559 - iframe creating breaks text selection so we create safe executor iframe after recording starts
        initSafeExecutorIFrame();
    };

    //Cross-domain
    var IFRAME_PARSE_SELECTOR_REQUEST_MSG_CMD = 'parseSelectorRequest',
        IFRAME_PARSE_SELECTOR_RESPONSE_MSG_CMD = 'parseSelectorResponse';

    var currentParseCallback = null;

    MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, function (e) {
        var msg = e.message;

        switch (msg.cmd) {
            case IFRAME_PARSE_SELECTOR_REQUEST_MSG_CMD:
                var res = exports.parseSelectorSync(msg.selector, msg.parseDomElementsOrJqueryObjectsOnly);

                res.length = res.$elements ? res.$elements.length : 0;
                res.iframeContext = true;

                //NOTE: prepare to JSON serialization
                delete res.$elements;
                delete res.$visibleElements;
                delete res.evalResults;

                var responseMsg = {
                    cmd: IFRAME_PARSE_SELECTOR_RESPONSE_MSG_CMD,
                    parsedSelector: res,
                    selector: msg.selector
                };

                MessageSandbox.sendServiceMsg(responseMsg, window.top);

                break;

            case IFRAME_PARSE_SELECTOR_RESPONSE_MSG_CMD:
                if (typeof currentParseCallback === 'function') {
                    currentParseCallback(msg.parsedSelector);
                    currentParseCallback = null;
                }
                break;
        }
    });
});