TestCafeClient.define('Base.Transport', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        Transport = Hammerhead.Transport,
        NativeMethods = Hammerhead.NativeMethods,
        Settings = require('Settings'),
        ServiceCommands = require('Shared.ServiceCommands');

    //Const
    var MAX_INACTIVITY_DURATION = 110000;

    //Globals
    var inactivityHandler = null,
        inactivityTimeout = null;

    function resetInactivityTimeout(expectedInactivityDuration) {
        if (inactivityHandler) {
            window.clearTimeout(inactivityTimeout);
            inactivityTimeout = NativeMethods.setTimeout.call(window, inactivityHandler, expectedInactivityDuration || MAX_INACTIVITY_DURATION);
        }
    }

    //Exports
    //-------------------------------------------------------------------------------------
    exports.syncServiceMsg = function (msg, callback) {
        resetInactivityTimeout();

        return Transport.syncServiceMsg(msg, callback);
    };

    exports.asyncServiceMsg = function (msg, callback) {
        resetInactivityTimeout();

        return Transport.asyncServiceMsg(msg, callback);
    };

    exports.waitForServiceMessagesCompleted = Transport.waitForServiceMessagesCompleted;
    exports.batchUpdate = Transport.batchUpdate;
    exports.queuedAsyncServiceMsg = Transport.queuedAsyncServiceMsg;

    exports.switchToWorkerIdle = function () {
        window.location.href = Settings.WORKER_IDLE_URL;
    };

    exports.switchToStartRecordingUrl = function (recordingUrl) {
        var removeHashRegExp = /#.*$/,
            replacedLocationHref = window.location.href.replace(removeHashRegExp, '').toLowerCase(),
            replacedRecordingUrl = recordingUrl.replace(removeHashRegExp, '').toLowerCase();

        window.location.href = recordingUrl;

        //T210251 - Playback doesn't start on page with location with hash
        if (window.location.hash && replacedLocationHref === replacedRecordingUrl && removeHashRegExp.test(recordingUrl))
            window.location.reload(true);
    };

    exports.expectInactivity = function (duration, callback) {
        var maxDuration = duration + MAX_INACTIVITY_DURATION;

        //NOTE: order is important here. serviceMsg should go first because it also resets inactivity timeout
        var inactivityExpectedMsg = {
            cmd: ServiceCommands.INACTIVITY_EXPECTED,
            duration: maxDuration
        };

        exports.asyncServiceMsg(inactivityExpectedMsg, function () {
            resetInactivityTimeout(maxDuration);
            callback();
        });
    };

    exports.fail = function (err) {
        var testFailMsg = {
            cmd: ServiceCommands.TEST_FAIL,
            err: err
        };

        exports.asyncServiceMsg(testFailMsg, function () {
            exports.switchToWorkerIdle();
        });

        //HACK: this helps stop current JS context execution
        window.onerror = function () {
        };
        throw 'STOP';
    };

    exports.assertionFailed = function (err) {
        var assertionFailedMsg = {
            cmd: ServiceCommands.ASSERTION_FAILED,
            err: err
        };

        exports.asyncServiceMsg(assertionFailedMsg);
    };


    //NOTE: we are using transport messages as a test activity monitor. If we are not receiving any service
    //message for a long time period then something is definitely went wrong.
    exports.startInactivityMonitor = function (onInactivity) {
        inactivityHandler = onInactivity;
        resetInactivityTimeout();
    };

    exports.stopInactivityMonitor = function () {
        if (inactivityHandler) {
            inactivityHandler = null;
            window.clearTimeout(inactivityTimeout);
        }
    };
});