TestCafeClient.define('ActionBarrier.Xhr', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,
        SharedErrors = require('Shared.Errors');

    //Const
    var INITIAL_REQUESTS_COLLECTION_DELAY = 300,
        ADDITIONAL_REQUESTS_COLLECTION_DELAY = 100,
        WAIT_PAGE_INITIAL_REQUESTS_DELAY = 50,
        PAGE_INITIALIZATION_CALLBACK_DELAY = 100;

    exports.BARRIER_TIMEOUT = 3000;
    exports.XHR_BARRIER_ERROR = 'error';
    exports.GLOBAL_START_XHR_BARRIER = 'tc-gsxb-d1417385';
    exports.GLOBAL_WAIT_XHR_BARRIER = 'tc-gwxb-d1417385';

    //Globals
    var barrierCtx = null,
        pageInitialReqsComplete = null,
        eventEmitter = new Util.EventEmitter();

    exports.events = eventEmitter;

    function initBarrierCtx(callback) {
        barrierCtx = {
            passed: false,
            collectingReqs: true,
            requests: [],
            reqCount: 0,

            callback: function () {
                if (!barrierCtx.passed) {
                    barrierCtx.passed = true;

                    if (barrierCtx.watchdog)
                        window.clearTimeout(barrierCtx.watchdog);

                    callback.apply(window, arguments);
                }
            },
            watchdog: null
        };
    }

    function onRequestsCollected() {
        barrierCtx.collectingReqs = false;

        if (barrierCtx.reqCount) {
            barrierCtx.watchdog = NativeMethods.setTimeout.call(window, function () {
                eventEmitter.emit(exports.XHR_BARRIER_ERROR, {code: SharedErrors.XHR_REQUEST_TIMEOUT});
                barrierCtx.callback();
            }, exports.BARRIER_TIMEOUT);
        } else
            barrierCtx.callback();
    }

    function removeXhrFromQueue(xhr) {
        for (var i = 0; i < barrierCtx.requests.length; i++) {
            if (xhr === barrierCtx.requests[i]) {
                barrierCtx.requests.splice(i, 1);
                return true;
            }
        }

        return false;
    }

    //API
    exports.init = function () {
        pageInitialReqsComplete = false;

        //NOTE: init barrier ctx and collect all page initial requests
        initBarrierCtx(function () {
            pageInitialReqsComplete = true;
        });

        function onXhrFinish (xhr) {
            if (removeXhrFromQueue(xhr))
                barrierCtx.reqCount--;

            if (!barrierCtx.collectingReqs && !barrierCtx.reqCount)
                barrierCtx.callback();
        }

        Hammerhead.on(Hammerhead.XHR_SEND, function (e) {
            if (barrierCtx) {
                barrierCtx.reqCount++;
                barrierCtx.requests.push(e.xhr);
            }
        });

        Hammerhead.on(Hammerhead.XHR_ERROR, function (e) {
            //NOTE: previously this error used in possible fail causes, which are R.I.P. now.
            //So we just through errot to the console for the debugging purposes. In IE9 console.log
            //is defined only if dev tools are open.
            if (window.console && window.console.log)
                window.console.log('TestCafe encountered XHR error on page: ' + e.err);
            exports.events.emit(exports.XHR_BARRIER_ERROR, e.err);

            onXhrFinish(e.xhr);
        });

        Hammerhead.on(Hammerhead.XHR_COMPLETED, function (e) {
            //NOTE: let last real xhr-handler finish it's job and try to obtain any additional requests
            //if they were initiated by this handler
            NativeMethods.setTimeout.call(window, function () {
                onXhrFinish(e.xhr);
            }, ADDITIONAL_REQUESTS_COLLECTION_DELAY);
        });
    };

    exports.waitPageInitialRequests = function (callback) {
        onRequestsCollected();

        var wait = function () {
            if (pageInitialReqsComplete)
                NativeMethods.setTimeout.call(window, callback, PAGE_INITIALIZATION_CALLBACK_DELAY);
            else
                NativeMethods.setTimeout.call(window, wait, WAIT_PAGE_INITIAL_REQUESTS_DELAY);
        };

        wait();
    };

    exports.startBarrier = function (callback) {
        initBarrierCtx(callback);
    };

    exports.waitBarrier = function () {
        //NOTE: collect initial XHR requests set, then wait for the requests to finish
        NativeMethods.setTimeout.call(window, onRequestsCollected, INITIAL_REQUESTS_COLLECTION_DELAY);
    };

    window[exports.GLOBAL_START_XHR_BARRIER] = exports.startBarrier;
    window[exports.GLOBAL_WAIT_XHR_BARRIER] = exports.waitBarrier;
});
