TestCafeClient.define('TestRunner.IFrameTestRunner', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        Util = Hammerhead.Util,
        MessageSandbox = Hammerhead.MessageSandbox,
        jQuerySelectorExtensions = require('Base.jQueryExtensions.Selectors'),
        DialogsAPI = require('TestRunner.API.Dialogs'),

        TestRunnerBase = require('TestRunner.TestRunnerBase'),
        Settings = require('Settings');

    var IFrameTestRunner = this.exports = function (startedCallback) {
        TestRunnerBase.apply(this, [startedCallback]);
    };

    Util.inherit(IFrameTestRunner, TestRunnerBase);

    IFrameTestRunner.prototype._initPageLoadBarrier = function () {
    };

    IFrameTestRunner.prototype._prepareStepsExecuting = function (callback) {
        jQuerySelectorExtensions.init();
        callback();
    };

    IFrameTestRunner.prototype._onTestComplete = function (e) {
        var completeMsg = {
            cmd: TestRunnerBase.IFRAME_STEP_COMPLETED_CMD
        };

        this._onSetStepsSharedData({
            stepsSharedData: this.testIterator.getSharedData(),
            callback: function () {
                MessageSandbox.sendServiceMsg(completeMsg, window.top);
                e.callback();
            }
        });
    };

    IFrameTestRunner.prototype._onNextStepStarted = function (e) {
        var msg = {
            cmd: TestRunnerBase.IFRAME_NEXT_STEP_STARTED_CMD
        };

        MessageSandbox.sendServiceMsg(msg, window.top);
        e.callback();
    };

    IFrameTestRunner.prototype._onActionTargetWaitingStarted = function (e) {
        var msg = {
            cmd: TestRunnerBase.IFRAME_ACTION_TARGET_WAITING_STARTED_CMD,
            params: e
        };

        MessageSandbox.sendServiceMsg(msg, window.top);
    };

    IFrameTestRunner.prototype._onActionRun = function () {
        MessageSandbox.sendServiceMsg({ cmd: TestRunnerBase.IFRAME_ACTION_RUN_CMD }, window.top);
    };

    IFrameTestRunner.prototype._onError = function (err) {

        if (!Settings.PLAYBACK || err.dialog)
            this.testIterator.stop();

        var msg = {
            cmd: TestRunnerBase.IFRAME_ERROR_CMD,
            err: err
        };

        MessageSandbox.sendServiceMsg(msg, window.top);
    };

    IFrameTestRunner.prototype._onAssertionFailed = function (e) {
        var msg = {
            cmd: TestRunnerBase.IFRAME_FAILED_ASSERTION_CMD,
            err: e
        };

        this.testIterator.state.needScreeshot = true;

        MessageSandbox.sendServiceMsg(msg, window.top);

        if (Settings.PLAYBACK)
            this.testIterator.stop();
    };

    IFrameTestRunner.prototype._onSetStepsSharedData = function (e) {
        var msg = {
            cmd: TestRunnerBase.IFRAME_SET_SHARED_DATA_CMD,
            sharedData: e.stepsSharedData
        };

        MessageSandbox.sendServiceMsg(msg, window.top);
        e.callback();
    };

    IFrameTestRunner.prototype._onGetStepsSharedData = function (e) {
        var msg = {
            cmd: TestRunnerBase.IFRAME_GET_SHARED_DATA_REQUEST_CMD
        };

        MessageSandbox.sendServiceMsg(msg, window.top);

        function onMessage(response) {
            if (response.message.cmd === TestRunnerBase.IFRAME_GET_SHARED_DATA_RESPONSE_CMD) {
                MessageSandbox.off(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
                e.callback(response.message.sharedData);
            }
        }

        MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
    };

    IFrameTestRunner.prototype._onExpectInactivity = function (e) {
        var msg = {
            cmd: TestRunnerBase.IFRAME_EXPECT_INACTIVITY_CMD,
            duration: e.duration
        };

        MessageSandbox.sendServiceMsg(msg, window.top);
        e.callback();
    };

    IFrameTestRunner.prototype._onTakeScreenshot = function (e) {
        var msg = {
            cmd: TestRunnerBase.IFRAME_TAKE_SCREENSHOT_REQUEST_CMD,
            isFailedStep: e.isFailedStep
        };

        MessageSandbox.sendServiceMsg(msg, window.top);

        function onMessage(response) {
            if (response.message.cmd === TestRunnerBase.IFRAME_TAKE_SCREENSHOT_RESPONSE_CMD) {
                MessageSandbox.off(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
                if (e && e.callback)
                    e.callback();
            }
        }

        MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
    };

    IFrameTestRunner.prototype.onStepCompleted = function () {
        DialogsAPI.checkExpectedDialogs();
    };

    IFrameTestRunner.prototype._onDialogsInfoChanged = function (info) {
        var msg = {
            cmd: TestRunnerBase.IFRAME_NATIVE_DIALOGS_INFO_CHANGED_CMD,
            info: info
        };

        MessageSandbox.sendServiceMsg(msg, window.top);
    };

    IFrameTestRunner.prototype._onBeforeUnload = function () {
        var iFrameTestRunner = this,
            msg = {
                cmd: TestRunnerBase.IFRAME_BEFORE_UNLOAD_REQUEST_CMD
            };

        function onMessage(response) {
            if (response.message.cmd === TestRunnerBase.IFRAME_BEFORE_UNLOAD_RESPONSE_CMD) {

                MessageSandbox.off(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);

                if (response.message.res) {
                    if (iFrameTestRunner.testIterator.state.stepDelayTimeout) {
                        window.clearTimeout(iFrameTestRunner.testIterator.state.stepDelayTimeout);
                        iFrameTestRunner.testIterator.state.stepDelayTimeout = null;
                    }

                    iFrameTestRunner.testIterator.state.pageUnloading = false;
                    iFrameTestRunner.testIterator._runStep();
                }
            }
        }

        MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);

        MessageSandbox.sendServiceMsg(msg, window.top);
    };
});
