TestCafeClient.define('TestRunner.TestIterator', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        async = Hammerhead.async,
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,
        JSON = Hammerhead.JSON,
        ActionBarrier = require('ActionBarrier'),
        XhrBarrier = require('ActionBarrier.Xhr'),
        Settings = require('Settings'),
        SharedErrors = require('Shared.Errors');

    //Const
    var STEP_DELAY = 500,
        PROLONGED_STEP_DELAY = 3000,
        SHORT_PROLONGED_STEP_DELAY = 30;

    //Iterator
    var TestIterator = this.exports = function (pingIFrame) {
        this.state = {
            step: 0,
            stepNames: null,
            testSteps: null,
            pageUnloading: false,
            stepDelayTimeout: null,
            inAsyncAction: false,
            prolongStepDelay: false,
            shortProlongStepDelay: false,
            stepsSharedData: {},
            lastSyncedSharedDataJSON: null,
            stopped: false,
            waitedIFrame: null,
            needScreeshot: false
        };

        this.pingIFrame = pingIFrame;
        this.globalWaitForEvent = null;
        this.globalWaitForTimeout = null;

        this.eventEmitter = new Util.EventEmitter();
    };

    //Events
    TestIterator.TEST_COMPLETE_EVENT = 'testComplete';
    TestIterator.NEXT_STEP_STARTED_EVENT = 'nextStepStarted';
    TestIterator.ACTION_TARGET_WAITING_STARTED_EVENT = 'actionTargetWaitingStarted';
    TestIterator.ACTION_RUN_EVENT = 'actionRun';
    TestIterator.ERROR_EVENT = 'error';
    TestIterator.ASSERTION_FAILED_EVENT = 'assertionFailed';
    TestIterator.SET_STEPS_SHARED_DATA_EVENT = 'setStepsSharedData';
    TestIterator.GET_STEPS_SHARED_DATA_EVENT = 'getStepsSharedData';
    TestIterator.EXPECT_INACTIVITY_EVENT = 'expectInactivity';
    TestIterator.TAKE_SCREENSHOT_EVENT = 'takeScreenshot';
    TestIterator.BEFORE_UNLOAD_EVENT_RAISED = 'beforeUnload';
    TestIterator.UNLOAD_EVENT_RAISED = 'unload';

    TestIterator.prototype.on = function () {
        return this.eventEmitter.on.apply(this.eventEmitter, arguments);
    };

    TestIterator.prototype._checkSharedDataSerializable = function () {
        var error = null;

        if (!JSON.isSerializable(this.state.stepsSharedData)) {
            error = {
                code: SharedErrors.STORE_DOM_NODE_OR_JQUERY_OBJECT,
                stepName: Settings.CURRENT_TEST_STEP_NAME,
                stepNum: this.state.step - 1
            };

            this.eventEmitter.emit(TestIterator.ERROR_EVENT, error);

            return false;
        }

        return true;
    };

    TestIterator.prototype._runStep = function () {
        this.state.stopped = false;

        var testIterator = this;

        if (testIterator.state.step > 0 && !testIterator.state.stepDoneCalled && typeof testIterator.state.stepDone === 'function') {
            testIterator.state.stepDone();
            testIterator.state.stepDoneCalled = true;
        }

        if (this.state.stopped)
            return;

        if (testIterator.state.step >= testIterator.state.testSteps.length) {
            this.eventEmitter.emit(TestIterator.TEST_COMPLETE_EVENT, {
                callback: function () {
                    testIterator.state.runningCopmlete = true;
                }
            });

            return;
        }

        Settings.CURRENT_TEST_STEP_NAME = testIterator.state.stepNames[testIterator.state.step];

        var stepToRun = testIterator.state.testSteps[testIterator.state.step];

        testIterator.state.step++;

        testIterator.eventEmitter.emit(TestIterator.NEXT_STEP_STARTED_EVENT, {
            nextStep: testIterator.state.step,
            callback: function () {
                testIterator.__waitFor(function () {
                    var error = null;

                    if (typeof testIterator.state.stepSetup === 'function')
                        testIterator.state.stepSetup();

                    testIterator.state.stepDoneCalled = false;

                    testIterator.state.inAsyncAction = false;
                    testIterator.state.prolongStepDelay = false;
                    testIterator.state.shortProlongStepDelay = false;

                    try {
                        testIterator.callWithSharedDataContext(stepToRun);
                    } catch (err) {
                        error = {
                            code: SharedErrors.UNCAUGHT_JS_ERROR_IN_TEST_CODE_STEP,
                            scriptErr: (err && err.message) || err,
                            stepName: Settings.CURRENT_TEST_STEP_NAME,
                            stepNum: testIterator.state.step - 1
                        };

                        testIterator.eventEmitter.emit(TestIterator.ERROR_EVENT, error);
                    }

                    if (testIterator.state.stopped)
                        return;

                    var runCallback = function () {
                        if (typeof testIterator.state.stepDone === 'function' && !testIterator.state.stepDoneCalled) {
                            testIterator.state.stepDone();
                            testIterator.state.stepDoneCalled = true;
                        }

                        testIterator._runStep();
                    };

                    //NOTE: don't run next step if previous step initiated async action
                    if (!testIterator.state.inAsyncAction) {
                        //NOTE: validate shared data changes that were made on this step (see: B236594)
                        //If there was an action in the step, validation must be performed in _syncSharedData
                        //  before serialization.
                        if (!testIterator._checkSharedDataSerializable())
                            return;

                        if (Settings.TAKE_SCREENSHOT_ON_FAILS && testIterator.state.needScreeshot) {
                            testIterator.takeScreenshot(function () {
                                testIterator.state.needScreeshot = false;
                                runCallback();
                            }, true);
                        }
                        else
                            runCallback();
                    }
                });
            }
        });
    };

    TestIterator.prototype._setupUnloadPrediction = function () {
        var testIterator = this,
            $form = $('form'),
            prolong = function () {
                testIterator.state.prolongStepDelay = true;
            },
            shortProlong = function () {
                testIterator.state.shortProlongStepDelay = true;
            },
            beforeUnload = function () {
                testIterator.state.pageUnloading = true;

                testIterator.eventEmitter.emit(TestIterator.BEFORE_UNLOAD_EVENT_RAISED);
            },
            unload = function () {
                testIterator.state.pageUnloading = true;

                testIterator.eventEmitter.emit(TestIterator.UNLOAD_EVENT_RAISED);
            };

        $(document).on('submit', 'form', prolong);

        $form.each(function () {
            var submit = this.submit;

            this.submit = function () {
                prolong();
                submit.apply(this, arguments);
            };
        });

        var skipBeforeUnloadEvent = false;

        NativeMethods.addEventListener.call(document, 'click', function (e) {
            var target = (e.srcElement || e.target);

            if (!e.defaultPrevented && target.tagName && target.tagName.toLowerCase() === 'a') {
                var href = $(target).attr('href');

                if (target.hasAttribute('href') && !/(^javascript:)|(^mailto:)|(^tel:)|(^#)/.test(href))
                    prolong();
                else if (Util.isIE)
                    skipBeforeUnloadEvent = true;
            }
        });

        //NOTE: IE fires onbeforeunload even if link was just clicked without actual unloading
        function onBeforeUnloadIE() {
            shortProlong();

            NativeMethods.setTimeout.call(window, function () {
                //NOTE: except file downloading
                if (document.readyState === 'loading' && !(document.activeElement && document.activeElement.tagName.toLowerCase() === 'a' && document.activeElement.getAttribute('download') !== null))
                    beforeUnload();
            }, 0);
        }

        Hammerhead.on(Hammerhead.BEFORE_UNLOAD_EVENT, Util.isIE ? onBeforeUnloadIE : beforeUnload);

        Hammerhead.on(Hammerhead.BEFORE_UNLOAD_EVENT, function () {
            skipBeforeUnloadEvent = false;
        });

        NativeMethods.windowAddEventListener.call(window, 'unload', unload);
    };

    TestIterator.prototype._syncSharedDataWithServer = function (callback) {
        var testIterator = this,
            sharedDataJSON = '';

        if (!testIterator._checkSharedDataSerializable())
            return;

        sharedDataJSON = JSON.stringify(testIterator.state.stepsSharedData);

        //NOTE: avoid unnecessary shared data sync if it's not changed
        if (testIterator.state.lastSyncedSharedDataJSON === sharedDataJSON) {
            if (typeof callback === 'function')
                callback();
        }
        else {
            testIterator.eventEmitter.emit(TestIterator.SET_STEPS_SHARED_DATA_EVENT, {
                stepsSharedData: testIterator.state.stepsSharedData,
                callback: function () {
                    testIterator.state.lastSyncedSharedDataJSON = sharedDataJSON;

                    if (typeof callback === 'function')
                        callback();
                }
            });
        }
    };

    TestIterator.prototype._completeAsyncAction = function () {
        var testIterator = this;

        if (testIterator.state.stopped)
            return;

        var run = function () {
            if (!testIterator.state.pageUnloading) {
                testIterator._runStep();
                window.clearTimeout(testIterator.state.stepDelayTimeout);
                testIterator.state.stepDelayTimeout = null;
            }
        };

        //NOTE: browsers continues to execute script even if the request for the new page occurs. To workaround this
        //we are using heuristic-based delays for the next step execution (see setupUnloadPrediction() method).
        testIterator.state.stepDelayTimeout = NativeMethods.setTimeout.call(window, function () {
            if (testIterator.state.prolongStepDelay || testIterator.state.shortProlongStepDelay) {
                testIterator.state.stepDelayTimeout = NativeMethods.setTimeout.call(window, function () {
                    run();
                }, testIterator.state.prolongStepDelay ? PROLONGED_STEP_DELAY : SHORT_PROLONGED_STEP_DELAY);
            } else
                run();
        }, STEP_DELAY);
    };

    TestIterator.prototype.callWithSharedDataContext = function (func) {
        return func.apply(this.state.stepsSharedData);
    };

    TestIterator.prototype._checkIFrame = function (element, callback) {
        if (window.top !== window.self || !Util.isElementInIframe(element)) {
            callback(null);
            return;
        }

        var iFrame = Util.getIFrameByElement(element);

        this.pingIFrame(iFrame, function () {
            callback(iFrame);
        });

    };

    TestIterator.prototype.asyncAction = function (action) {
        var testIterator = this;

        this.state.inAsyncAction = true;

        var actionRun = function () {
            testIterator._syncSharedDataWithServer(function () {
                ActionBarrier.waitActionSideEffectsCompletion(action, function () {
                    testIterator._completeAsyncAction.apply(testIterator, arguments);
                });
            });
        };

        if (Settings.TAKE_SCREENSHOT_ON_FAILS && this.state.needScreeshot) {
            this.takeScreenshot(function () {
                testIterator.state.needScreeshot = false;
                actionRun();
            }, true);
        }
        else
            actionRun();
    };

    TestIterator.prototype.asyncActionSeries = function (items, runArgumentsIterator, action) {
        var testIterator = this;

        var actionsRun = function () {
            var seriesActionsRun = function (elements, callback) {
                async.forEachSeries(
                    elements,
                    function (element, asyncCallback) {
                        //NOTE: since v.14.1.5 it's recommended to run actions with the inIFrame function. But we should support old-style iframes
                        //using, so, it'll be resolved here.
                        testIterator._checkIFrame(element, function (iframe) {
                            if (!iframe) {
                                ActionBarrier.waitActionSideEffectsCompletion(function (barrierCallback) {
                                    action(element, barrierCallback);
                                }, asyncCallback);
                            }
                            else {
                                var iFrameStartXhrBarrier = iframe.contentWindow[XhrBarrier.GLOBAL_START_XHR_BARRIER],
                                    iFrameWaitXhrBarrier = iframe.contentWindow[XhrBarrier.GLOBAL_WAIT_XHR_BARRIER];

                                ActionBarrier.waitActionSideEffectsCompletion(function (barrierCallback) {
                                    var iFrameBeforeUnloadRaised = false;

                                    testIterator.iFrameActionCallback = function () {
                                        testIterator.iFrameActionCallback = null;
                                        testIterator.waitedIFrame = null;
                                        barrierCallback();
                                    };

                                    testIterator.waitedIFrame = iframe;

                                    iFrameStartXhrBarrier(function () {
                                        if (!iFrameBeforeUnloadRaised)
                                            testIterator.iFrameActionCallback();
                                    });

                                    function onBeforeUnload() {
                                        NativeMethods.windowRemoveEventListener.call(iframe.contentWindow, 'beforeunload', onBeforeUnload);
                                        iFrameBeforeUnloadRaised = true;
                                    }

                                    NativeMethods.windowAddEventListener.call(iframe.contentWindow, 'beforeunload', onBeforeUnload, true);

                                    action(element, function () {
                                        iFrameWaitXhrBarrier();
                                    }, iframe);
                                }, asyncCallback);
                            }
                        });
                    },
                    function () {
                        if (testIterator.state.stopped)
                            return;

                        callback();
                    });
            };

            testIterator._syncSharedDataWithServer(function () {
                runArgumentsIterator(items, seriesActionsRun, function () {
                    testIterator._completeAsyncAction.apply(testIterator, arguments);
                });
            });
        };

        testIterator.state.inAsyncAction = true;

        if (Settings.TAKE_SCREENSHOT_ON_FAILS && this.state.needScreeshot) {
            this.takeScreenshot(function () {
                testIterator.state.needScreeshot = false;
                actionsRun();
            }, true);
        }
        else
            actionsRun();
    };

    TestIterator.prototype._init = function () {
        this.initialized = true;

        this._setupUnloadPrediction();
    };

    TestIterator.prototype.start = function (stepNames, testSteps, stepSetup, stepDone, nextStep) {
        this._init();

        this.runSteps(stepNames, testSteps, stepSetup, stepDone, nextStep);
    };

    TestIterator.prototype.stop = function () {
        //NOTE: this flag created for playback in recording mode
        // to prevent test playback after error raised
        //and in test running to prevent playback during screenshot making
        this.state.stopped = true;
    };

    TestIterator.prototype.runSteps = function (stepNames, testSteps, stepSetup, stepDone, nextStep) {
        if (!this.initialized)
            this._init();

        var testIterator = this;

        testIterator.state.testSteps = testSteps;
        testIterator.state.stepNames = stepNames;
        testIterator.state.inAsyncAction = false;
        testIterator.state.step = nextStep;
        testIterator.state.stepSetup = stepSetup;
        testIterator.state.stepDone = stepDone;
        testIterator.state.runningCopmlete = false;

        testIterator.eventEmitter.emit(TestIterator.GET_STEPS_SHARED_DATA_EVENT, {
            callback: function (sharedData) {
                testIterator.state.stepsSharedData = sharedData || {};
                testIterator.state.lastSyncedSharedDataJSON = JSON.stringify(testIterator.state.stepsSharedData);
                testIterator._runStep();
            }
        });
    };

    TestIterator.prototype.getSharedData = function () {
        return this.state.stepsSharedData;
    };

    TestIterator.prototype.setSharedData = function (data) {
        this.state.stepsSharedData = data;
        this._syncSharedDataWithServer();
    };

    TestIterator.prototype.onError = function (err) {
        if (this.state.stopped)
            return;

        this.eventEmitter.emit(TestIterator.ERROR_EVENT, $.extend({
            stepNum: this.state.step - 1
        }, err));
    };

    TestIterator.prototype.onAssertionFailed = function (err) {
        if (this.state.stopped)
            return;

        this.eventEmitter.emit(TestIterator.ASSERTION_FAILED_EVENT, {
            err: err,
            stepNum: this.state.step - 1,
            isAssertion: true
        });
    };

    TestIterator.prototype.expectInactivity = function (duration, callback) {
        this.eventEmitter.emit(TestIterator.EXPECT_INACTIVITY_EVENT, {
            duration: duration,
            callback: callback
        });
    };

    TestIterator.prototype.runNext = function () {
        this._runStep();
    };

    TestIterator.prototype.runLast = function () {
        this.state.step--;
        this.runNext();
    };

    TestIterator.prototype.getCurrentStep = function () {
        return this.state.stepNames ? this.state.stepNames[this.state.step - 1] : Settings.CURRENT_TEST_STEP_NAME;
    };

    TestIterator.prototype.getCurrentStepNum = function () {
        return this.state.step - 1;
    };

    TestIterator.prototype.onActionTargetWaitingStarted = function (e) {
        this.eventEmitter.emit(TestIterator.ACTION_TARGET_WAITING_STARTED_EVENT, e);
    };

    TestIterator.prototype.onActionRun = function () {
        this.eventEmitter.emit(TestIterator.ACTION_RUN_EVENT, {});
    };

    //Global __waitFor()
    TestIterator.prototype.setGlobalWaitFor = function (event, timeout) {
        this.globalWaitForEvent = event;
        this.globalWaitForTimeout = timeout;
    };

    TestIterator.prototype.__waitFor = function (callback) {
        var testIterator = this;

        if (typeof this.globalWaitForEvent !== 'function') {
            callback();
            return;
        }

        if (typeof this.globalWaitForTimeout !== 'number')
            this.globalWaitForTimeout = 0;

        var timeoutID = NativeMethods.setTimeout.call(window, function () {
            testIterator.onError({
                code: SharedErrors.API_WAIT_FOR_ACTION_TIMEOUT_EXCEEDED,
                stepName: Settings.CURRENT_TEST_STEP_NAME
            });
        }, this.globalWaitForTimeout);


        this.expectInactivity(this.globalWaitForTimeout, function () {
            testIterator.callWithSharedDataContext(function () {
                testIterator.globalWaitForEvent.call(this, function () {
                    window.clearTimeout(timeoutID);
                    callback();
                });
            });
        });
    };

    TestIterator.prototype.takeScreenshot = function (callback, isFailedStep) {
        this.eventEmitter.emit(TestIterator.TAKE_SCREENSHOT_EVENT, {
            isFailedStep: isFailedStep,
            callback: callback
        });
    };
});