//NOTE: this jshint directive disables warning: "Value of '{a}' may be overwritten in IE 8 and earlier."
/*jshint -W002 */

TestCafeClient.define('Recorder.ElementPicker', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        EventSandbox = Hammerhead.EventSandbox,
        EventSimulator = Hammerhead.EventSimulator,
        ShadowUI = Hammerhead.ShadowUI,
        MessageSandbox = Hammerhead.MessageSandbox,

        eventEmitter = new Util.EventEmitter(),
        RecorderUtil = require('Recorder.Util'),
        SelectorGenerator = require('Recorder.SelectorGenerator'),
        SharedConst = require('Shared.Const');

    //Const
    var ELEMENT_FRAME_CLASS = RecorderUtil.ELEMENT_PICKING_CLASSES.elementFrame,
        ELEMENT_FRAME_PART_CLASS = RecorderUtil.ELEMENT_PICKING_CLASSES.elementFramePart;

    //Globals
    var $recorder = null,

        $elementFrame = null,
        $divTop = null,
        $divBottom = null,
        $divLeft = null,
        $divRight = null,

        $cursorStyle = null,

        lastOverElement = null,

        inProcess = false,
        elementPickerCallback = null;

    //Const
    var ELEMENT_FRAME_BORDER = 2;

    //internal events
    var IFRAME_ELEMENT_HOVERED = 'iFrameElementHovered',

        GET_ELEMENT_RECTANGLE_REQUEST = 'getElementRectangleRequest',
        GET_ELEMENT_RECTANGLE_RESPONSE = 'getElementRectangleResponse',

        SET_PREVENT_MOUSEOVER_HANDLER_REQUEST = 'setPreventMouseOverHandlerRequest',
        SET_PREVENT_MOUSEOVER_HANDLER_RESPONSE = 'setPreventMouseOverHandlerResponse',

        IFRAME_ELEMENT_PICKED_REQUEST = 'iFrameElementPickedRequest',
        IFRAME_ELEMENT_PICKED_RESPONSE = 'iFrameElementPickedResponse';

    var isInIFrame = window.top !== window.self;

    //For Testing Purposes
    exports.ELEMENT_HOVERED_EVENT = 'elementHovered';

    //Utils
    function getElementUnderFrame(e) {
        var curElement = e.target;

        if ($elementFrame && $elementFrame[0] && e.target === $elementFrame[0]) {
            $elementFrame.css('visibility', 'hidden');

            curElement = Util.getElementFromPoint(e.clientX, e.clientY, null, true);

            $elementFrame.css('visibility', 'visible');
        }
        return curElement;
    }

    function emulateRealEvents(e, curElement) {
        var eventOptions = {
            clientX: e.clientX,
            clientY: e.clientY,
            which: Util.isWebKit ? Util.WHICH_PARAMETER.NO_BUTTON : 1,
            buttons: Util.BUTTONS_PARAMETER.NO_BUTTON
        };

        var currentElementChanged = true;

        try {
            //NOTE: when lastOverElement was in an iframe that removed, ie raises exception when we try to
            // compare it with current element
            currentElementChanged = curElement !== lastOverElement;
        }
        catch (e) {
            lastOverElement = null;
            currentElementChanged = true;
        }

        if (currentElementChanged && lastOverElement)
            EventSimulator.mouseout(lastOverElement, $.extend({relatedTarget: curElement}, eventOptions));

        var eventName = Util.isTouchDevice ? 'touchmove' : 'mousemove';

        //NOTE: only in IE a 'mousemove' event is raised before a 'mouseover' one (B236966)
        if (Util.isIE && curElement)
            EventSimulator[eventName](curElement, eventOptions);

        if (currentElementChanged) {
            if (curElement)
                EventSimulator.mouseover(curElement, $.extend({relatedTarget: lastOverElement}, eventOptions));

            lastOverElement = curElement;
        }

        if (!Util.isIE && curElement)
            EventSimulator[eventName](curElement, eventOptions);
    }

    function getOffset(el, point) {
        var elementOffset = Util.getOffsetPosition(el);

        return {
            x: point.x > elementOffset.left ? point.x - elementOffset.left : 0,
            y: point.y > elementOffset.top ? point.y - elementOffset.top : 0
        };
    }

    //Handlers
    function onMouseMove(e, dispatched, prevent) {
        var curElement = getElementUnderFrame(e),
            needEmulate = $elementFrame && $elementFrame[0] && e.target === $elementFrame[0];

        if (!curElement || Util.isShadowUIElement(curElement))
            return;

        var rect = Util.getElementRectangleForMarking(curElement, ELEMENT_FRAME_BORDER, ELEMENT_FRAME_BORDER);

        if (isInIFrame) {
            var documentScroll = Util.getElementScroll($(document));
            rect.top -= documentScroll.top;
            rect.left -= documentScroll.left;

            MessageSandbox.sendServiceMsg({
                cmd: IFRAME_ELEMENT_HOVERED,
                rect: rect
            }, window.top);
        }
        else {
            if (curElement.tagName && curElement.tagName.toLowerCase() === 'iframe' && (!currentIFrameContext || currentIFrameContext === curElement.contentWindow)) {
                if (needEmulate)
                    emulateRealEvents(e, curElement);

                var point = Util.getFixedPositionForIFrame({x: e.pageX, y: e.pageY}, curElement.contentWindow);

                if (Util.isIE && $elementFrame && $elementFrame[0])
                    $elementFrame.css('visibility', 'hidden');

                MessageSandbox.sendServiceMsg({
                    cmd: GET_ELEMENT_RECTANGLE_REQUEST,
                    point: point,
                    needEmulate: needEmulate,
                    clientX: e.clientX,
                    clientY: e.clientY
                }, curElement.contentWindow);
            }
            else if (!currentIFrameContext) {
                if (needEmulate)
                    emulateRealEvents(e, curElement);

                createElementFrame(rect);

                eventEmitter.emit(exports.ELEMENT_HOVERED_EVENT, {
                    element: curElement
                });
            }
        }

        //NOTE: T199100 - we should prevent action on 'element frame' because they can be caught on document
        // and it may lead unexpected behavior
        if (needEmulate)
            prevent();
    }

    function onClick(e, dispatched, prevent) {
        var curElement = getElementUnderFrame(e);

        if (!curElement || Util.isShadowUIElement(curElement))
            return;

        var options = {
            ctrl: e.ctrlKey,
            alt: e.altKey,
            shift: e.shiftKey,
            meta: e.metaKey
        };

        if (!isInIFrame && curElement.tagName && curElement.tagName.toLowerCase() === 'iframe' && (!currentIFrameContext || currentIFrameContext === curElement.contentWindow)) {
            MessageSandbox.sendServiceMsg({
                cmd: SET_PREVENT_MOUSEOVER_HANDLER_REQUEST,
                point: Util.getFixedPositionForIFrame({x: e.pageX, y: e.pageY}, curElement.contentWindow),
                iFrameSelectors: SelectorGenerator.generate($(curElement)),
                options: options
            }, curElement.contentWindow);
        }
        else {
            var actionOffset = getOffset(curElement, Util.getEventAbsoluteCoordinates(e));

            options.offsetX = actionOffset.x;
            options.offsetY = actionOffset.y;

            if (isInIFrame) {
                if (!Util.isIE)
                    EventSandbox.addInternalEventListener(window, ['mouseover'], preventMouseOverAfterElementFrameRemoved);

                MessageSandbox.sendServiceMsg({
                    cmd: IFRAME_ELEMENT_PICKED_RESPONSE,
                    elementSelectors: SelectorGenerator.generate($(curElement)),
                    options: options
                }, window.top);
            }
            else if (!currentIFrameContext) {
                cleanUI();

                EventSandbox.addInternalEventListener(window, ['mouseover'], preventMouseOverAfterElementFrameRemoved);
                elementPickerCallback(SelectorGenerator.generate($(curElement)), null, options);
            }
        }

        prevent();
    }

    function onKeydown(e, dispatched, prevent) {
        if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.esc) {
            cleanUI();

            if (lastOverElement && lastOverElement.tagName.toLowerCase() === 'iframe')
                ShadowUI.bind($(lastOverElement.contentWindow), 'mouseover', preventMouseOverAfterElementFrameRemoved);
            else
                EventSandbox.addInternalEventListener(window, ['mouseover'], preventMouseOverAfterElementFrameRemoved);

            elementPickerCallback(null);
            prevent();
        }
    }

    function preventEvent(e, dispatched, prevent) {
        if (!Util.isShadowUIElement(e.target))
            prevent();
    }

    function onMouseOut(e, dispatched, prevent) {
        if ($elementFrame && e.target === $elementFrame[0]) {
            EventSimulator.mouseout(lastOverElement, $.extend({relatedTarget: e.relatedTarget}, {
                clientX: e.clientX,
                clientY: e.clientY,
                buttons: Util.BUTTONS_PARAMETER.NO_BUTTON,
                which: Util.isWebKit ? Util.WHICH_PARAMETER.NO_BUTTON : 1
            }));

            //NOTE: T199100 - we should prevent action on 'element frame' because they can be caught on document
            // and it may lead unexpected behavior
            prevent();
        }
        else if (!Util.isShadowUIElement(e.target) && e.target === lastOverElement &&
            $elementFrame && e.relatedTarget === $elementFrame[0])
            prevent();
    }

    function onMouseOver(e, dispanched, prevent) {
        if (!dispanched && (!$elementFrame || e.target !== $elementFrame[0]) && !Util.isShadowUIElement(e.target))
            lastOverElement = e.target;

        var curElement = getElementUnderFrame(e);

        //T198725 - Can't select element for the assertion
        if (!Util.isIE || lastOverElement !== curElement)
            Util.storeElementAttributes(SharedConst.OLD_ATTR_VALUES, curElement);

        //NOTE: T199100 - we should prevent action on 'element frame' because they can be caught on document
        // and it may lead unexpected behavior
        if ($elementFrame && e.target === $elementFrame[0])
            prevent();
    }

    function preventMouseOverAfterElementFrameRemoved(e, dispanched, prevent) {
        if (!dispanched && lastOverElement)
            prevent();

        EventSandbox.removeInternalEventListener(window, ['mouseover'], preventMouseOverAfterElementFrameRemoved);
    }

    //Markup
    function createElementFrame(rect, inParts) {
        if (!inProcess)
            return;

        //NOTE: we're cutting rectangle if it's more than document size to prevent extra scrolling after other element highlighted
        var rectHeight = Util.getDocumentElementHeight() < rect.top + rect.height ? Util.getDocumentElementHeight() - rect.top : rect.height,
            rectWidth = Util.getDocumentElementWidth() < rect.left + rect.width ? Util.getDocumentElementWidth() - rect.left : rect.width,

            $divs = ShadowUI.select('.' + ELEMENT_FRAME_PART_CLASS);

        if (!Util.isIE || !inParts) {
            if ($divs && $divs.length)
                $divs.css('visibility', 'hidden');

            if (!$elementFrame || !$elementFrame[0]) {
                $elementFrame = $('<div></div>').appendTo($recorder);
                ShadowUI.addClass($elementFrame, ELEMENT_FRAME_CLASS);

                $elementFrame.bind('mousedown', function (e) {
                    Util.preventDefault(e);
                });
            }

            $elementFrame.css({
                top: rect.top,
                left: rect.left,
                width: rectWidth,
                height: rectHeight,
                visibility: 'visible'
            });
        }
        else {
            if ($elementFrame && $elementFrame[0])
                $elementFrame.css('visibility', 'hidden');

            //NOTE: we need emulate element frame for iframes in IE because otherwise we can't get element under cursor
            // (out element frame is blocking iframe's element and getElementUnderFrame method returns null)
            if (!$divTop || !$divTop.length) {
                $divTop = $('<div></div>').appendTo($recorder);
                ShadowUI.addClass($divTop, ELEMENT_FRAME_PART_CLASS);
            }

            $divTop.css({
                top: rect.top,
                left: rect.left,
                width: rectWidth,
                height: ELEMENT_FRAME_BORDER
            });

            if (!$divBottom || !$divBottom.length) {
                $divBottom = $('<div></div>').appendTo($recorder);
                ShadowUI.addClass($divBottom, ELEMENT_FRAME_PART_CLASS);
            }

            $divBottom.css({
                top: rect.top + rectHeight - ELEMENT_FRAME_BORDER,
                left: rect.left,
                width: rectWidth,
                height: ELEMENT_FRAME_BORDER
            });

            if (!$divLeft || !$divLeft.length) {
                $divLeft = $('<div></div>').appendTo($recorder);
                ShadowUI.addClass($divLeft, ELEMENT_FRAME_PART_CLASS);
            }

            $divLeft.css({
                top: rect.top + ELEMENT_FRAME_BORDER,
                left: rect.left,
                width: ELEMENT_FRAME_BORDER,
                height: rectHeight - ELEMENT_FRAME_BORDER
            });

            if (!$divRight || !$divRight.length) {
                $divRight = $('<div></div>').appendTo($recorder);
                ShadowUI.addClass($divRight, ELEMENT_FRAME_PART_CLASS);
            }

            $divRight.css({
                top: rect.top + ELEMENT_FRAME_BORDER,
                left: rect.left + rectWidth - ELEMENT_FRAME_BORDER,
                width: ELEMENT_FRAME_BORDER,
                height: rectHeight - ELEMENT_FRAME_BORDER
            });

            $divs.css('visibility', 'visible');
            $divs.bind('mousedown', function (e) {
                Util.preventDefault(e);
            });
        }
    }

    //Behavior
    function cleanUI() {
        if ($elementFrame && $elementFrame[0])
            $elementFrame.remove();
        $elementFrame = null;

        ShadowUI.select('.' + ELEMENT_FRAME_PART_CLASS).remove();

        $divTop = null;
        $divBottom = null;
        $divLeft = null;
        $divRight = null;

        if ($cursorStyle && $cursorStyle[0])
            $cursorStyle.remove();
        $cursorStyle = null;
    }

    function createCursorStyle() {
        var $head = $(document).find('head');

        $cursorStyle = $('<style>*{cursor: crosshair !important;}</style>').appendTo($head[0]);
    }

    function bindHandlers(target) {
        EventSandbox.addInternalEventListener(target, ['mousemove'], onMouseMove);

        EventSandbox.addInternalEventListener(target, ['mouseout'], onMouseOut);

        EventSandbox.addInternalEventListener(target, ['mouseover'], onMouseOver);

        EventSandbox.addInternalEventListener(target, ['click'], onClick);

        EventSandbox.addInternalEventListener(target, ['keydown'], onKeydown);

        EventSandbox.addInternalEventListener(target, ['mousedown', 'mouseup'], preventEvent);

        //NOTE: we create special cursor style for all elements because
        // we want to assign 'crosshair' cursor during element picking
        // (it doesn't work for input and textarea because it's impossible to change their default cursor)
        if (Util.isIE)
            createCursorStyle();
    }

    function unbindHandlers(target) {
        EventSandbox.removeInternalEventListener(target, ['mousemove'], onMouseMove);

        EventSandbox.removeInternalEventListener(target, ['mouseout'], onMouseOut);

        EventSandbox.removeInternalEventListener(target, ['mouseover'], onMouseOver);

        EventSandbox.removeInternalEventListener(target, ['click'], onClick);

        EventSandbox.removeInternalEventListener(target, ['keydown'], onKeydown);

        EventSandbox.removeInternalEventListener(target, ['mousedown', 'mouseup'], preventEvent);
    }

    //API
    exports.on = function () {
        return eventEmitter.on.apply(eventEmitter, arguments);
    };

    exports.off = function () {
        return eventEmitter.off.apply(eventEmitter, arguments);
    };

    var currentIFrameContext = false;

    exports.start = function (callback, iFrameContext) {
        inProcess = true;
        elementPickerCallback = callback;

        currentIFrameContext = iFrameContext;

        bindHandlers(window);
    };

    exports.stop = function () {
        inProcess = false;
        elementPickerCallback = null;

        cleanUI();

        unbindHandlers(window);
    };

    exports.setRecorderUI = function ($recorderUI) {
        $recorder = $recorderUI;
    };

    MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, function (e) {
        var msg = e.message,
            curElement = null,
            rect = null,
            fixedPoint = null;

        switch (msg.cmd) {
            //Messages to iframes
            case GET_ELEMENT_RECTANGLE_REQUEST:
                curElement = Util.getElementFromPoint(msg.point.x, msg.point.y);

                if (curElement) {
                    rect = Util.getElementRectangleForMarking(curElement, ELEMENT_FRAME_BORDER, ELEMENT_FRAME_BORDER);

                    var documentScroll = Util.getElementScroll($(document));
                    rect.top -= documentScroll.top;
                    rect.left -= documentScroll.left;

                    if (msg.needEmulate)
                        emulateRealEvents({clientX: msg.clientX, clientY: msg.clientY}, curElement);
                }

                MessageSandbox.sendServiceMsg({
                    cmd: GET_ELEMENT_RECTANGLE_RESPONSE,
                    rect: rect
                }, window.top);
                break;

            case IFRAME_ELEMENT_PICKED_REQUEST:
                curElement = Util.getElementFromPoint(msg.point.x, msg.point.y);

                if (curElement) {
                    var actionOffset = getOffset(curElement, msg.point);

                    msg.options.offsetX = actionOffset.x;
                    msg.options.offsetY = actionOffset.y;

                    elementPickerCallback(SelectorGenerator.generate($(curElement)), msg.iFrameSelectors, msg.options);
                }
                else
                //NOTE: click on cross-domain iframe
                    elementPickerCallback(msg.iFrameSelectors, msg.options);
                break;

            case SET_PREVENT_MOUSEOVER_HANDLER_REQUEST:
                EventSandbox.addInternalEventListener(window, ['mouseover'], preventMouseOverAfterElementFrameRemoved);

                MessageSandbox.sendServiceMsg({
                    cmd: SET_PREVENT_MOUSEOVER_HANDLER_RESPONSE,
                    point: msg.point,
                    iFrameSelectors: msg.iFrameSelectors,
                    options: msg.options
                }, window.top);
                break;

            //Messages to top window
            case GET_ELEMENT_RECTANGLE_RESPONSE:
                rect = e.message.rect;

                if (!rect) {
                    curElement = Util.getIFrameByWindow(e.source);

                    rect = Util.getElementRectangleForMarking(curElement, ELEMENT_FRAME_BORDER, ELEMENT_FRAME_BORDER);
                }
                else {
                    fixedPoint = Util.getFixedPosition({x: rect.left, y: rect.top}, e.source);

                    rect.left = fixedPoint.x;
                    rect.top = fixedPoint.y;
                }

                createElementFrame(rect, true);
                break;

            case SET_PREVENT_MOUSEOVER_HANDLER_RESPONSE:
                cleanUI();

                MessageSandbox.sendServiceMsg({
                    cmd: IFRAME_ELEMENT_PICKED_REQUEST,
                    point: msg.point,
                    iFrameSelectors: msg.iFrameSelectors,
                    options: msg.options
                }, e.source);

                break;

            case IFRAME_ELEMENT_HOVERED:
                rect = e.message.rect;

                fixedPoint = Util.getFixedPosition({x: rect.left, y: rect.top}, e.source);

                rect.left = fixedPoint.x;
                rect.top = fixedPoint.y;

                createElementFrame(rect, true);
                break;

            case IFRAME_ELEMENT_PICKED_RESPONSE:
                var iFrame = Util.getIFrameByWindow(e.source);

                elementPickerCallback(msg.elementSelectors, SelectorGenerator.generate($(iFrame)), msg.options);
                break;
        }
    });
});
