TestCafeClient.define('Recorder.EventListener', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        Util = Hammerhead.Util,
        EventSandbox = Hammerhead.EventSandbox,
        EventParser = require('Recorder.EventParser');

    //handlers
    var listeningStarted = false,
        lastListenedEvent = null,
        lastEventPrevented = false,
        preventEvents = false;

    function parseEvent(ev) {
        lastListenedEvent = ev;
        lastEventPrevented = EventParser.parseEvent(ev);
        return lastEventPrevented || preventEvents;
    }

    function genericEventHandler(ev, dispatched) {
        if (dispatched)
            return;

        if (!listeningStarted)
            return;

        //NOTE: if lastListenedEvent contains an target-element which was in an removed iframe,
        // IE raises exception when we try to compare it with current event target-element
        try {
            if (lastListenedEvent === ev) {
                if (lastEventPrevented || preventEvents) {
                    Util.preventDefault(ev);
                    return false;
                }
                else
                    return;
            }
        }
        catch (e) {
            lastListenedEvent = null;
        }

        if (parseEvent(ev)) {
            Util.preventDefault(ev);
            return false;
        }
        else {
            lastEventPrevented = false;
        }

        if (preventEvents) {
            Util.preventDefault(ev);
            return false;
        }
    }

    function cancelHandlersForShadowUITarget(ev, dispatched, preventDefault, cancelHandlers) {
        if (Util.isShadowUIElement(ev.target || ev.srcElement)) {
            cancelHandlers();
        }
    }

    function callGenericEventHandler(ev, dispatched, preventDefault) {
        if (!Util.isShadowUIElement(ev.target || ev.srcElement) &&
            listeningStarted && genericEventHandler.call(this, ev, dispatched) === false)
            preventDefault();
    }

    function bindWindowHandlers() {
        EventSandbox.addInternalEventListener(window, Util.DOM_EVENTS, cancelHandlersForShadowUITarget);
        EventSandbox.addInternalEventListener(window, Util.RECORDING_LISTENED_EVENTS, callGenericEventHandler);
    }

    function bindDocumentHandlers(doc) {
        EventSandbox.addInternalEventListener(doc, Util.DOM_EVENTS, cancelHandlersForShadowUITarget);
    }

    function eventBindingHandler(e) {
        if (e.document) {
            EventSandbox.initElementListening(e.document, Util.RECORDING_LISTENED_EVENTS);
            bindDocumentHandlers(e.document);
        }
    }

    var eventParserCallback = null;

    //API
    Hammerhead.on(Hammerhead.DOM_DOCUMENT_CLEARED, eventBindingHandler);

    exports.start = function (callback, options) {
        bindWindowHandlers();
        bindDocumentHandlers(document);

        listeningStarted = true;

        eventParserCallback = function (action) {
            callback(action);
        };

        EventParser.init(eventParserCallback, options);
    };

    exports.stop = function () {
        listeningStarted = false;
        EventParser.destroy();
    };

    exports.pause = function () {
        listeningStarted = false;
        EventParser.resetParsing();
    };

    exports.resume = function () {
        listeningStarted = true;
    };

    exports.reset = function () {
        EventParser.destroy(true);
        EventParser.init(eventParserCallback);
    };

    exports.createAllDeferred = function () {
        EventParser.createAllDeferred();
    };

    exports.setPreventingMode = function (prevent) {
        preventEvents = prevent;
    };

    exports.checkActionsBeforeAdditionNativeDialogHandler = function(){
      EventParser.checkActionsBeforeAdditionNativeDialogHandler();
    };
});
