TestCafeClient.define('Recorder.PlaybackRunner', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        Util = Hammerhead.Util,
        Transport = require('Base.Transport'),
        ServiceCommands = require('Shared.ServiceCommands'),

        TestRunnerBase = require('TestRunner.TestRunnerBase');

    var PlaybackRunner = this.exports = function () {
        TestRunnerBase.apply(this);
    };

    Util.inherit(PlaybackRunner, TestRunnerBase);

    PlaybackRunner.prototype.TEST_FAILED_EVENT = 'testFailed';

    PlaybackRunner.prototype._onTestComplete = function (e) {
        PlaybackRunner.base._destroy.call(this);

        PlaybackRunner.base._onTestComplete.call(this, e);
    };

    PlaybackRunner.prototype._onError = function (err) {
        if (!err.pageError)
            this.testIterator.stop();

        TestRunnerBase.prototype._onError.call(this, err);
    };

    PlaybackRunner.prototype.skipStep = function (stepNames, steps, nextStep) {
        if (typeof nextStep !== 'undefined')
            this.testIterator.state.step = nextStep;

        this.testIterator.state.stepNames = stepNames;
        this.testIterator.state.testSteps = steps;

        this.testIterator.runNext();
    };

    PlaybackRunner.prototype.resumePlayback = function (stepNames, steps) {
        this.testIterator.state.stepNames = stepNames;
        this.testIterator.state.testSteps = steps;

        this.testIterator.runLast();
    };

    PlaybackRunner.prototype._onAssertionFailed = function (e) {
        this.testIterator.stop();
        this.eventEmitter.emit(this.TEST_FAILED_EVENT, e);
    };

    PlaybackRunner.prototype._onNextStepStarted = function (e) {
        var nextStepMsg = {
            cmd: ServiceCommands.SET_NEXT_STEP_PLAYBACK,
            nextStep: e.nextStep
        };

        this.eventEmitter.emit(this.NEXT_STEP_STARTED_EVENT, e);

        Transport.asyncServiceMsg(nextStepMsg, e.stop ? null : e.callback);
    };

    PlaybackRunner.prototype._onActionTargetWaitingStarted = function (e) {
        TestRunnerBase.prototype._onActionTargetWaitingStarted.apply(this, [e]);
    };

    PlaybackRunner.prototype._onActionRun = function () {
        TestRunnerBase.prototype._onActionRun.apply(this, []);
    };

    PlaybackRunner.prototype._onDialogsInfoChanged = function (info) {
        Transport.asyncServiceMsg({
            cmd: ServiceCommands.NATIVE_DIALOGS_INFO_SET,
            info: info,
            timeStamp: Date.now()
        });
    };
});
