TestCafeClient.define('Recorder.SelectorGenerator.Rules', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        SharedConst = require('Shared.Const'),
        jQuerySelectorExtensions = require('Base.jQueryExtensions.Selectors');

    var rules = null,

    //Const
        MAX_TEXT_LENGTH_IN_SELECTOR = 150,
        ASP_AUTOGENERATED_ID_PART_PATTERN = /_ctl\d+|ctl\d+_|^ctl\d+$/g,
        ASP_AUTOGENERATED_ID_PART_PATTERN_WITH_$ = /\$ctl\d+|ctl\d+\$/g,
        ATTRIBUTE_NAMES_FOR_SELECTORS_REGEXPS = [/^alt$/, /^name$/, /^class$/, /^title$/, /^data-\S+/],
        ANY_NUMBER_CONST = '!!!!!anyNumber!!!!!',
        SEPARATOR_CONST = '!!!!!separator!!!!!',

        ELEMENT_WHOSE = 'element whose ',
        ATTRS_SEPARATOR = ', ',
        CLASS_CONTAINS = '{0} contains "{1}"',
        ID_IS = '{0} is "{1}"',
        ATTR_VALUE_IS = '{0} attribute value is "{1}"',
        ATTR_MATCHES_REGEXP = '{0} attribute value matches the regular expression "{1}"',
        ELEMENT_THAT_CONTAIN_TEXT = 'element that contains text "{0}", ignoring child node text',
        TAG_NAME_DESCRIPTION = '{0} element',
        SEARCH_ELEMENT = 'Searches for an {0}.',
        SEARCH_ELEMENT_IN_ANCESTOR = 'Searches through descendants of the {1} for an {0}.',
        SEARCH_INPUT_IN_FORM = 'Searches through descendants of the form {1} for an input {0}.',
        SEARCH_ELEMENT_BY_DOM_PATH = 'Searches through descendants of the <span>body</span> element for an element at a specific location in the DOM hierarchy: {0}.',
        SEARCH_ELEMENT_IN_ANCESTOR_BY_DOM_PATH = 'Searches through descendants of the {1} for an element at a specific location in the DOM hierarchy: {0}.',
        NTH_ELEMENT_IS_RETURNED = 'The {0} matching element is returned.';


    exports.ruleIDs = {
        FOR_HTML: 'html',
        FOR_BODY: 'body',
        BY_ID: 'id',
        BY_TEXT: 'text',
        BY_ANCESTOR_AND_TEXT: 'id-text',
        BY_ATTR: 'attr',
        BY_ANCESTOR_AND_ATTR: 'id-attr',
        BY_TAGS_TREE: 'dom',
        BY_ANCESTOR_WITH_TEXT_AND_TAGS_TREE: 'text-dom',
        BY_ANCESTOR_WITH_ATTR_AND_TAGS_TREE: 'attr-dom',
        BY_INPUT_NAME_AND_FORM: 'form-input',
        USER_INPUT: 'custom'
    };

    //Utils
    var formatString = function (sourceString, args, dontWrapArguments) {
        return sourceString.replace(/{(\d+)}/g, function (match, number) {
            return typeof args[number] === 'undefined' ? match :
                (dontWrapArguments ? args[number] : '<span>' + args[number] + '</span>');
        });
    };

    //Descriptions generating
    var getSelectorParts = function (selector, hasParentSelector) {
        var SELECTOR_START = '$("',
            SELECTOR_END = '")',
            FIND_SUBSTRING = ').find("',
            contextRegEx = /", \$\("[\s|\S]+"\)\.contents\(\)/g;
        if (!selector || selector.indexOf(SELECTOR_START) !== 0 || !(selector.lastIndexOf(SELECTOR_END) === selector.length - SELECTOR_END.length || (!hasParentSelector && new RegExp(contextRegEx)).test(selector)))
            return {};
        if (hasParentSelector) {
            if (selector.indexOf(FIND_SUBSTRING) > 0) {
                var ancestorPart = selector.substring(SELECTOR_START.length, selector.lastIndexOf(FIND_SUBSTRING));
                var elementPart = selector.substring(selector.lastIndexOf(FIND_SUBSTRING) + FIND_SUBSTRING.length);
                if (elementPart.lastIndexOf(SELECTOR_END) !== elementPart.length - SELECTOR_END.length)
                    return {};
                elementPart = elementPart.substring(0, elementPart.lastIndexOf(SELECTOR_END));
                if (new RegExp(contextRegEx).test(ancestorPart)) {
                    return {
                        elementSelector: elementPart,
                        ancestorSelector: ancestorPart.substring(0, ancestorPart.search(new RegExp(contextRegEx))),
                        contextSelector: ancestorPart.substring(ancestorPart.search(new RegExp(contextRegEx)))
                    };
                }
                else {
                    return {
                        elementSelector: elementPart,
                        ancestorSelector: ancestorPart.substring(0, ancestorPart.lastIndexOf('"'))

                    };
                }
            }
            else
                return {};
        }
        else {
            if (new RegExp(contextRegEx).test(selector)) {
                return {
                    elementSelector: selector.substring(SELECTOR_START.length, selector.search(new RegExp(contextRegEx))),
                    contextSelector: selector.substring(selector.search(new RegExp(contextRegEx)))
                };
            }
            else {
                return {
                    elementSelector: selector.substring(SELECTOR_START.length, selector.lastIndexOf(SELECTOR_END))
                };
            }
        }
    };

    var getAttrSelectorDescription = function (selector) {
        if (!selector)
            return '';

        var attrRegExpStart = ':' + jQuerySelectorExtensions.ATTR_REGEXP_METHOD_NAME + '(',
            attrRegExpSeparator = jQuerySelectorExtensions.REGEXP_START_SUBSTR,
            attrRegExpEnd = jQuerySelectorExtensions.REGEXP_END_SUBSTR + ')',
            attrStart = '[',
            attrSeparator = '=\'',
            attrEnd = '\']',
            classStart = '.',
            idStart = '#',
            expression = null,
            attrName = null,

            description = '',
            rest = selector,

            replaceDoubleSlashWithSlash = function (text) {
                return text.replace(/\\\\/g, '\\');
            };

        while (rest) {
            description += description ? ATTRS_SEPARATOR : ELEMENT_WHOSE;
            if (rest.indexOf(idStart) === 0) {
                var id = rest.substring(idStart.length);
                description += formatString(ID_IS, ['id', id]);
                rest = '';
            }
            else if (rest.indexOf(classStart) === 0) {
                var classes = '';
                var end = rest.indexOf(attrRegExpStart) > 0 ? rest.indexOf(attrStart) > 0 ? Math.min(rest.indexOf(attrRegExpStart), rest.indexOf(attrStart)) : rest.indexOf(attrRegExpStart) : rest.indexOf(attrStart);
                if (end > 0) {
                    classes = rest.substring(0, end);
                    rest = rest.substring(end);
                }
                else {
                    classes = rest;
                    rest = '';
                }
                classes = classes.substring(classStart.length).replace(/\./g, ' ');
                if (!classes)
                    return '';
                description += formatString(CLASS_CONTAINS, ['class', classes]);
            }
            else if (rest.indexOf(attrRegExpStart) === 0) {
                if (rest.indexOf(attrRegExpEnd) < 0)
                    return '';
                expression = rest.substring(attrRegExpStart.length, rest.indexOf(attrRegExpEnd));
                if (expression.indexOf(attrRegExpSeparator) < 0)
                    return '';
                rest = rest.substring(rest.indexOf(attrRegExpEnd) + attrRegExpEnd.length);
                attrName = expression.substring(0, expression.indexOf(attrRegExpSeparator));
                var regExp = expression.substring(expression.indexOf(attrRegExpSeparator) + attrRegExpSeparator.length);
                regExp = replaceDoubleSlashWithSlash(regExp);
                description += formatString(ATTR_MATCHES_REGEXP, [attrName, regExp]);
            }
            else if (rest.indexOf(attrStart) === 0) {
                //TODO: handle case when attr value contains "']" substring
                if (rest.indexOf(attrEnd) < 0)
                    return '';
                expression = rest.substring(attrStart.length, rest.indexOf(attrEnd));
                if (expression.indexOf(attrSeparator) < 0)
                    return '';
                rest = rest.substring(rest.indexOf(attrEnd) + attrEnd.length);
                attrName = expression.substring(0, expression.indexOf(attrSeparator));
                var attrValue = expression.substring(expression.indexOf(attrSeparator) + attrSeparator.length);
                attrValue = replaceDoubleSlashWithSlash(attrValue);
                if (attrName === 'id')
                    description += formatString(ID_IS, [attrName, attrValue]);
                else
                    description += formatString(ATTR_VALUE_IS, [attrName, attrValue]);
            }
            else return '';
        }
        return description;
    };

    var getTextSelectorDescription = function (selector) {
        var containsExcludeChildrenStart = ':' + jQuerySelectorExtensions.CONTAINS_OWN_TEXT_METHOD_NAME + '(',
            containsExcludeChildrenEnd = ')';
        if (selector.indexOf(containsExcludeChildrenStart) === 0 && selector.indexOf(containsExcludeChildrenEnd) > 0)
            return formatString(ELEMENT_THAT_CONTAIN_TEXT, [selector.substring(containsExcludeChildrenStart.length, selector.lastIndexOf(containsExcludeChildrenEnd))]);
        else return '';
    };

    var getNumberAsWordFromIndex = function (index) {
        var number = index + 1,
            ending = 'th',
            lastDigit = number % 10,
            prevDigit = (number % 100 - lastDigit) / 10;
        if (prevDigit !== 1) {
            if (lastDigit === 1)
                ending = 'st';
            else if (lastDigit === 2)
                ending = 'nd';
            else if (lastDigit === 3)
                ending = 'rd';
        }
        return number.toString() + ending;
    };

    var getDomPathSelectorDescription = function (selector) {
        var description = '',
            rest = selector,
            separator = ' > ',
            beforeNumber = ':nth(',
            afterNumber = ')';
        while (rest) {
            if (rest.indexOf(separator) < 0)
                return '';
            rest = rest.substring(rest.indexOf(separator) + separator.length);
            var indexBeforeNumber = rest.indexOf(beforeNumber);
            var indexAfterNumber = rest.indexOf(afterNumber);
            if (indexBeforeNumber < 0 || indexAfterNumber < 0 || indexAfterNumber < indexBeforeNumber)
                return '';
            var tagName = rest.substring(0, indexBeforeNumber);
            var elementIndex = rest.substring(indexBeforeNumber + beforeNumber.length, indexAfterNumber);
            if (!tagName || !elementIndex)
                return '';
            rest = rest.substring(indexAfterNumber + afterNumber.length);
            if (description)
                description += ', then ';
            description += formatString('{0}', [getNumberAsWordFromIndex(parseInt(elementIndex)) + ' ' + tagName]);
        }
        return description;
    };

    var getTagNameSelectorDescription = function (selector) {
        if (selector)
            return formatString(TAG_NAME_DESCRIPTION, [selector]);
    };

    var buildSelectorDescription = function (selector, baseString, elementDescriptionGetter, ancestorDescriptionGetter) {
        var selectorParts = null,
            elementDescription = null;
        if (ancestorDescriptionGetter) {
            selectorParts = getSelectorParts(selector, true);
            if (selectorParts.elementSelector && selectorParts.ancestorSelector) {
                var ancestorDescription = ancestorDescriptionGetter(selectorParts.ancestorSelector);

                elementDescription = elementDescriptionGetter(selectorParts.elementSelector);

                if (ancestorDescription && elementDescription)
                    return formatString(baseString, [elementDescription, ancestorDescription], true);
            }
        }
        else {
            selectorParts = getSelectorParts(selector, false);
            if (selectorParts.elementSelector) {
                elementDescription = elementDescriptionGetter(selectorParts.elementSelector);

                if (elementDescription)
                    return formatString(baseString, [elementDescription], true);
            }
        }
        return selector;
    };

    //Selectors generating
    var getAttrRegExpSelector = function (attrName, regExp, needForEndOfStringSymbol) {
        return  [':', jQuerySelectorExtensions.ATTR_REGEXP_METHOD_NAME, '(', attrName, jQuerySelectorExtensions.REGEXP_START_SUBSTR, '^', regExp, needForEndOfStringSymbol ? '$' : '', jQuerySelectorExtensions.REGEXP_END_SUBSTR, ')'].join('');
    };

    var getAncestorWithId = function ($element) {
        return $element.parent().closest('[id]');
    };

    var getTextSelector = function ($element) {
        return [
            ':', jQuerySelectorExtensions.CONTAINS_OWN_TEXT_METHOD_NAME, '(', jQuerySelectorExtensions.getOwnTextForSelector($element[0]).substr(0, MAX_TEXT_LENGTH_IN_SELECTOR), ')'
        ].join('');
    };

    var getAttributeNamesForSelector = function ($element) {
        var attributeNames = [];

        if ($element[0][SharedConst.OLD_ATTR_VALUES]) {
            for (var attribute in $element[0][SharedConst.OLD_ATTR_VALUES])
                if ($element[0][SharedConst.OLD_ATTR_VALUES].hasOwnProperty(attribute))
                    if (isAttributeAcceptableForSelector($element, attribute, $element[0][SharedConst.OLD_ATTR_VALUES][attribute]))
                        attributeNames.push(attribute);
        }
        else
            $.each($element[0].attributes, function (index, attribute) {
                if (isAttributeAcceptableForSelector($element, attribute.nodeName, attribute.nodeValue))
                    attributeNames.push(attribute.nodeName);
            });

        return attributeNames;
    };

    var isAttributeAcceptableForSelector = function ($element, attributeName, attributeValue) {
        //NOTE: we don't take into account attributes added by TestCafe
        var storedAttrRegExp = new RegExp(Hammerhead.DOM_SANDBOX_STORED_ATTR_POSTFIX + '$', 'g');

        if (attributeValue)
            for (var i = 0; i < ATTRIBUTE_NAMES_FOR_SELECTORS_REGEXPS.length; i++)
                if (ATTRIBUTE_NAMES_FOR_SELECTORS_REGEXPS[i].test(attributeName) && !storedAttrRegExp.test(attributeValue) && !storedAttrRegExp.test(attributeName))
                    return true;
    };

    var getAttributesSelector = function ($element, attributeNames) {
        var getAttributeValue = function ($element, attributeName) {
                if ($element[0][SharedConst.OLD_ATTR_VALUES])
                    return $element[0][SharedConst.OLD_ATTR_VALUES][attributeName];
                else
                    return $element.attr(attributeName);
            },
            selector = '';
        $.each(attributeNames, function (index, attributeName) {
            var attributeValue = getAttributeValue($element, attributeName);
            var valueWasCut = false;
            var valueWasChanged = false;
            if (attributeValue.replace(/\s+/g, ' ').length > MAX_TEXT_LENGTH_IN_SELECTOR) {
                attributeValue = attributeValue.substr(0, MAX_TEXT_LENGTH_IN_SELECTOR);
                valueWasCut = true;
                valueWasChanged = true;
            }

            if (new RegExp(ASP_AUTOGENERATED_ID_PART_PATTERN).test(attributeValue) || new RegExp(ASP_AUTOGENERATED_ID_PART_PATTERN_WITH_$).test(attributeValue)) {
                var matchingRegExp = new RegExp(ASP_AUTOGENERATED_ID_PART_PATTERN).test(attributeValue) ? ASP_AUTOGENERATED_ID_PART_PATTERN : ASP_AUTOGENERATED_ID_PART_PATTERN_WITH_$;
                attributeValue = attributeValue.replace(new RegExp(matchingRegExp), function (substr) {
                    valueWasChanged = true;
                    return substr.replace(/\d+/, ANY_NUMBER_CONST);
                });
            }
            attributeValue = attributeValue.replace(/\s+/g, function (substr) {
                if (substr === ' ')
                    return substr;
                else {
                    valueWasChanged = true;
                    return SEPARATOR_CONST;
                }
            });

            if (!valueWasChanged) {
                attributeValue = replaceAttrSymbolsWithEscapeSequences(attributeValue);
                if (attributeName === 'class') {
                    selector += '.' + attributeValue.replace(/ +/g, '.');
                }
                else selector += [
                    '[', attributeName, '=\'', attributeValue, '\']'
                ].join('');
            }
            else {
                attributeValue = replaceRegExpSymbolsWithEscapeSequences(attributeValue);
                attributeValue = attributeValue.replace(new RegExp(ANY_NUMBER_CONST, 'g'), '\\\\d+');
                attributeValue = attributeValue.replace(new RegExp(SEPARATOR_CONST, 'g'), '\\\\s+');
                selector += getAttrRegExpSelector(attributeName, attributeValue, !valueWasCut);
            }
        });

        return selector;
    };

    var replaceRegExpSymbolsWithEscapeSequences = function (text) {
        return text.replace(/'|"|\(|\)|\||\-|\*|\?|\+|\\|\^|\$|\[|\]/g, function (substr) {
            if (substr === '\\')
                return '\\\\\\\\';
            else if (substr === '(' || substr === ')')
                return '\\\\S';
            else if (substr === '"' || substr === '\'')
                return '\\' + substr;
            else return "\\\\" + substr;
        });
    };

    var replaceAttrSymbolsWithEscapeSequences = function (text) {
        return text.replace(/'|"|\\/g, function (substr) {
            if (substr === '\\')
                return '\\\\\\\\';
            else if (substr === '\'')
                return '\\\\' + substr;
            else if (substr === '"')
                return '\\' + substr;
            else return substr;
        });
    };

    var replaceIDSymbolsWithEscapeSequences = function (text) {
        return text.replace(/\!|"|#|\$|%|&|\'|\(|\||\)|\*|\+|,|\.|\/|:|;|<|=|>|\?|@|\[|\\|\]|\^|`|{|\||}|~/g, function (substr) {
            if (substr === '\\')
                return '\\\\\\\\';
            else if (substr === '"')
                return '\\\\\\' + substr;
            else
                return "\\\\" + substr;
        });
    };

    var getTagsTreeSelector = function ($element, $lastAncestor) {
        var getNthSelectorString = function ($el) {
            if ($el.parent().length)
                return ' > ' + $el[0].tagName.toLowerCase() + ':nth(' + $el.parent().find('>' + $el[0].tagName).index($el) + ')';
        };
        var tagsTreeSelector = getNthSelectorString($element);
        $element.parentsUntil($lastAncestor).each(function () {
            tagsTreeSelector = getNthSelectorString($(this)) + tagsTreeSelector;
        });
        return tagsTreeSelector;
    };

    var wasElementRemoved = function ($element) {
        return !$element.closest('html').length;
    };

    var wereElementAttributesChanged = function ($element) {
        if ($element[0][SharedConst.OLD_ATTR_VALUES]) {
            for (var attribute in $element[0][SharedConst.OLD_ATTR_VALUES])
                if ($element[0][SharedConst.OLD_ATTR_VALUES].hasOwnProperty(attribute))
                    if (isAttributeAcceptableForSelector($element, attribute, $element[0][SharedConst.OLD_ATTR_VALUES][attribute]))
                        if ($element[0][SharedConst.OLD_ATTR_VALUES][attribute] !== $element.attr(attribute))
                            return true;
        }
        return false;
    };

    //Generating rules

    var SelectorGeneratingRule = function () {
    };

    SelectorGeneratingRule.prototype.wasElementChanged = function ($el) {
        return wasElementRemoved($el);
    };

    SelectorGeneratingRule.prototype.getDescription = function (selector) {
        return selector;
    };

    var selectorByIdRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                return !!$el.attr('id');
            },
            generate: function ($el) {
                var id = $el.attr('id');
                //new RexExp is created because firefox doesn't reset lastIndex of RegExp object
                if (new RegExp(ASP_AUTOGENERATED_ID_PART_PATTERN).test(id)) {
                    var idRegExp = replaceRegExpSymbolsWithEscapeSequences(id);
                    idRegExp = idRegExp.replace(new RegExp(ASP_AUTOGENERATED_ID_PART_PATTERN), function (substr) {
                        return substr.replace(/\d+/, "\\\\d+");
                    });

                    return {
                        selector: getAttrRegExpSelector('id', idRegExp, true),
                        ancestorExpression: null
                    };
                }
                
                return {
                    selector: '#' + replaceIDSymbolsWithEscapeSequences(id),
                    ancestorExpression: null
                };
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT, getAttrSelectorDescription);
            },
            id: exports.ruleIDs.BY_ID
        }
    );

    var selectorForHtmlRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                return $el.is('html');
            },
            generate: function ($el) {
                return {
                    selector: $el[0].tagName.toLowerCase(),
                    ancestorExpression: null
                };
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT, getTagNameSelectorDescription);
            },
            id: exports.ruleIDs.FOR_HTML
        }
    );

    var selectorForBodyRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                return $el.is('body');
            },
            generate: function ($el) {
                return {
                    selector: $el[0].tagName.toLowerCase(),
                    ancestorExpression: null
                };
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT, getTagNameSelectorDescription);
            },
            id: exports.ruleIDs.FOR_BODY
        }
    );

    var selectorByTextRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                return !!jQuerySelectorExtensions.getOwnTextForSelector($el[0]);
            },
            generate: function ($el) {
                return {
                    selector: getTextSelector($el),
                    ancestorExpression: null
                };
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT, getTextSelectorDescription);
            },
            id: exports.ruleIDs.BY_TEXT
        }
    );

    var selectorByAncestorAndTextRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                return jQuerySelectorExtensions.getOwnTextForSelector($el[0]) && getAncestorWithId($el).length;
            },
            generate: function ($el) {
                return {
                    selector: getTextSelector($el),
                    ancestor: {
                        rule: selectorByIdRule,
                        $el: getAncestorWithId($el)
                    }
                };
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT_IN_ANCESTOR, getTextSelectorDescription, getAttrSelectorDescription);
            },
            id: exports.ruleIDs.BY_ANCESTOR_AND_TEXT
        }
    );

    var selectorByAttributesRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                return !!getAttributeNamesForSelector($el).length;
            },
            generate: function ($el) {
                var attributeNames = getAttributeNamesForSelector($el);

                return {
                    selector: getAttributesSelector($el, attributeNames),
                    ancestorExpression: null
                };
            },
            wasElementChanged: function ($el) {
                return wasElementRemoved($el) || wereElementAttributesChanged($el);
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT, getAttrSelectorDescription);
            },
            id: exports.ruleIDs.BY_ATTR
        }
    );

    var selectorByAncestorAndAttributesRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                return getAttributeNamesForSelector($el).length && getAncestorWithId($el).length;
            },
            generate: function ($el) {
                var attributeNames = getAttributeNamesForSelector($el);

                return {
                    selector: getAttributesSelector($el, attributeNames),
                    ancestor: {
                        rule: selectorByIdRule,
                        $el: getAncestorWithId($el)
                    }
                };
            },
            wasElementChanged: function ($el) {
                return wasElementRemoved($el) && wereElementAttributesChanged($el);
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT_IN_ANCESTOR, getAttrSelectorDescription, getAttrSelectorDescription);
            },
            id: exports.ruleIDs.BY_ANCESTOR_AND_ATTR
        }
    );

    var selectorByTagsTreeRule = $.extend(new SelectorGeneratingRule(), {
            check: function () {
                return true;
            },
            generate: function ($el) {
                return {
                    selector: 'body' + getTagsTreeSelector($el, $('body', Util.findDocument($el[0]))),
                    ancestorExpression: null
                };
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT_BY_DOM_PATH, getDomPathSelectorDescription);
            },
            id: exports.ruleIDs.BY_TAGS_TREE
        }
    );

    var selectorByAncestorWithTextAndTagsTreeRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                if (/html|body/.test($el[0].tagName.toLowerCase()))
                    return false;
                var $parents = $el.parentsUntil('body');
                for (var parentIndex = 0; parentIndex < $parents.length; parentIndex++)
                    if (selectorByTextRule.check($parents.eq(parentIndex)))
                        return true;
                return false;
            },
            generate: function ($el) {
                if (/html|body/.test($el[0].tagName.toLowerCase()))
                    return false;
                var $parents = $el.parentsUntil('body');
                for (var i = 0; i < $parents.length; i++) {
                    if (selectorByTextRule.check($parents.eq(i))) {
                        return {
                            selector: getTagsTreeSelector($el, $parents.eq(i)),
                            ancestor: {
                                rule: selectorByTextRule,
                                $el: $parents.eq(i)
                            }
                        };
                    }
                }
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT_IN_ANCESTOR_BY_DOM_PATH, getDomPathSelectorDescription, getTextSelectorDescription);
            },
            id: exports.ruleIDs.BY_ANCESTOR_WITH_TEXT_AND_TAGS_TREE
        }
    );

    var selectorByAncestorWithAttrAndTagsTreeRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                if (/html|body/.test($el[0].tagName.toLowerCase()))
                    return false;
                var $parents = $el.parentsUntil('body');
                var acceptableRules = [selectorByIdRule, selectorByAttributesRule];
                for (var parentIndex = 0; parentIndex < $parents.length; parentIndex++)
                    for (var ruleIndex = 0; ruleIndex < acceptableRules.length; ruleIndex++)
                        if (acceptableRules[ruleIndex].check($parents.eq(parentIndex)))
                            return true;
                return false;
            },
            generate: function ($el) {
                if (/html|body/.test($el[0].tagName.toLowerCase()))
                    return false;
                var $parents = $el.parentsUntil('body');
                var acceptableRules = [selectorByIdRule, selectorByAttributesRule];

                for (var i = 0; i < $parents.length; i++) {
                    var rule = null,
                        $parent = $parents.eq(i);

                    for (var j = 0; j < acceptableRules.length; j++) {
                        if (acceptableRules[j].check($parent))
                            rule = acceptableRules[j];
                    }

                    if (rule) {
                        return {
                            selector: getTagsTreeSelector($el, $parents.eq(i)),
                            ancestor: {
                                rule: rule,
                                $el: $parent
                            }
                        };
                    }
                }
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_ELEMENT_IN_ANCESTOR_BY_DOM_PATH, getDomPathSelectorDescription, getAttrSelectorDescription);
            },
            id: exports.ruleIDs.BY_ANCESTOR_WITH_ATTR_AND_TAGS_TREE
        }
    );

    var selectorForInputInFormRule = $.extend(new SelectorGeneratingRule(), {
            check: function ($el) {
                if ($el.is('input') && $el.attr('name')) {
                    var $form = $el.closest('form');
                    return $form.length && (selectorByIdRule.check($form) || selectorByAttributesRule.check($form));
                }
                return false;
            },
            generate: function ($el) {
                var $form = $el.closest('form'),
                    rule = null,
                    acceptableRules = [selectorByIdRule, selectorByAttributesRule];

                if ($form.length) {
                    for (var j = 0; j < acceptableRules.length; j++) {
                        if (acceptableRules[j].check($form))
                            rule = acceptableRules[j];
                    }

                    if (rule) {
                        return {
                            selector: getAttributesSelector($el, ['name']),
                            ancestor: {
                                rule: rule,
                                $el: $form
                            }
                        };
                    }
                }
            },
            getDescription: function (selector) {
                return buildSelectorDescription(selector, SEARCH_INPUT_IN_FORM, getAttrSelectorDescription, getAttrSelectorDescription);
            },
            id: exports.ruleIDs.BY_INPUT_NAME_AND_FORM
        }
    );

    exports.makeSelectorUnique = function (selectorObject, $element, $selectorResult) {
        var elementIndex = $selectorResult.index($element);
        if ($selectorResult.length === 1 && elementIndex >= 0)
            return;
        else if ($selectorResult.length === 0 && elementIndex === -1)
            return;
        if (elementIndex === -1) {
            elementIndex = 0;
        }
        //TODO: do this for not-jQuery selectors if we'll add custom rules feature
        selectorObject.selector = selectorObject.selector + '.eq(' + elementIndex + ')';
        selectorObject.description = selectorObject.description + ' ' + formatString(NTH_ELEMENT_IS_RETURNED, [getNumberAsWordFromIndex(elementIndex)]);
    };

    exports.init = function () {
        rules = [
            selectorForHtmlRule,
            selectorForBodyRule,
            selectorForInputInFormRule,
            selectorByIdRule,
            selectorByAncestorAndTextRule,
            selectorByTextRule,
            selectorByAncestorAndAttributesRule,
            selectorByAttributesRule,
            selectorByAncestorWithTextAndTagsTreeRule,
            selectorByAncestorWithAttrAndTagsTreeRule,
            selectorByTagsTreeRule
        ];
    };

    exports.getRules = function () {
        if (!rules)
            exports.init();
        return rules;
    };
});