TestCafeClient.define('Recorder.StepNameGenerator', function (require, exports) {
        var Hammerhead = HammerheadClient.get('Hammerhead'),
            $ = Hammerhead.$;

        var ELEMENTS_NAME_DICTIONARY = {
            'a': 'link',
            'img': 'image',
            'li': 'list item',
            'ol': 'ordered list',
            'p': 'paragraph',
            'q': 'quotation',
            'td': 'table cell',
            'th': 'header cell',
            'tr': 'table row',
            'ul': 'unordered list',
            'textarea': 'text area'
        };

        var ELEMENTS_WITH_ANGLE_BRACKETS = ['b', 'bdi', 'bdo', 'big', 'blockquote', 'br', 'center', 'cite', 'code', 'col',
            'colgroup', 'del', 'dfn', 'em', 'font', 'header', 'hgroup', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'i',
            'ins', 'kbd', 'mark', 'optgroup', 'pre', 'ruby', 'rp', 'rt', 's', 'samp', 'small', 'strike', 'strong', 'sub',
            'sup', 'thead', 'tfoot', 'tbody', 'tt', 'u', 'var', 'wbr', 'svg', 'text', 'path', 'textpath', 'tspan', 'tref',
            'dd', 'dir', 'dl', 'dt', 'nav'];


        function getElementName(element) {
            var tagName = (element && element.tagName) ? element.tagName.toLowerCase() : '',
                elementType = (element && element.type) ? element.type.toLowerCase() : '';

            if (ELEMENTS_NAME_DICTIONARY[tagName])
                return ELEMENTS_NAME_DICTIONARY[tagName];

            if ($.inArray(tagName, ELEMENTS_WITH_ANGLE_BRACKETS) !== -1)
                return '<' + tagName + '>';

            if (tagName === 'button' && elementType && elementType !== 'button')
                return elementType + ' button';

            if (tagName === 'command' && elementType && elementType !== 'command')
                return elementType + ' command';

            if (tagName === 'input') {
                if (!elementType || (elementType && elementType === 'text'))
                    return tagName;

                if (elementType && /hidden|password/.test(elementType))
                    return elementType + ' input';

                if (elementType && /file|image|reset|submit|radio/.test(elementType))
                    return elementType + ' button';

                if (elementType && /checkbox/.test(elementType))
                    return'check box';

                if (elementType && /button/.test(elementType))
                    return 'button';
            }

            return tagName;
        }

        //API
        exports.generateStepName = function (actionDescriptor) {
            var stepName = actionDescriptor.type[0].toUpperCase() + actionDescriptor.type.substring(1) + ' ',
                element = actionDescriptor.element,
                elements = actionDescriptor.elements,
                tagName = (element && element.tagName) ? element.tagName.toLowerCase() : '',
                elementType = (element && element.type) ? element.type.toLowerCase() : '',

                currentName = '',
                currentDescriptor = null;

            var cropText = function (text) {
                var wordBounds = [],
                    wordBoundRegExp = /(\s|$)/g,
                    maxLength = 20,
                    result = '';

                text = $.trim(text).replace(/(\n)|(\r\n)/g, '').replace(/\s+/g, ' ');

                if (text.length <= maxLength)
                    return text;

                while (wordBoundRegExp.exec(text) && wordBounds.length < 3)
                    wordBounds.push(wordBoundRegExp.lastIndex);

                $.each(wordBounds.reverse(), function (index, value) {
                    if (value < maxLength) {
                        result = text.substring(0, value - 1) + '...';
                        return false;
                    }
                });
                return result || text.substring(0, maxLength - 1) + '...';
            };

            var generateElementDescriptionByAttributes = function () {
                var elementAttr = null,
                    description = '',
                    text = '';

                $.each(arguments[0], function (index, value) {
                    if (value === 'text') {
                        //NOTE: we should take own element's text only (not children's text)
                        for (var i = 0; i < element.childNodes.length; ++i)
                            if (element.childNodes[i].nodeType === 3)
                                text += element.childNodes[i].textContent ? element.childNodes[i].textContent : element.innerText;
                        elementAttr = $.trim(text);
                    }
                    else
                        elementAttr = element[value];

                    if (elementAttr) {
                        if (value === 'text')
                            description = ' "' + cropText(elementAttr) + '"';
                        else
                            description = ' "' + cropText(elementAttr) + '"';
                        return false;
                    }
                });

                return description;
            };

            if (actionDescriptor.type === 'press') {
                if (/\+/.test(actionDescriptor.apiArguments.keysCommand))
                    stepName += 'key combination ' + actionDescriptor.apiArguments.keysCommand.toUpperCase();
                else
                    stepName += 'key ' + actionDescriptor.apiArguments.keysCommand.toUpperCase();
                return stepName;
            }
            else if (actionDescriptor.type === 'wait') {
                stepName += actionDescriptor.apiArguments.ms + ' milliseconds';
                return stepName;
            }
            else if (actionDescriptor.type === 'screenshot'){
                return 'Take a ' + actionDescriptor.type;
            }
            else if (actionDescriptor.type === 'type')
                stepName += 'in ';
            else if (actionDescriptor.type === 'hover')
                stepName += 'over ';
            else if (actionDescriptor.type === 'select')
                stepName += 'in ';
            else if(actionDescriptor.type === 'upload') {
                var fileNames = actionDescriptor.serviceInfo.fileNames;

                if(fileNames.join(''))
                    stepName += '"' + fileNames.join('", "') + '" file' + (fileNames.length > 1 ? 's' : '');
                else
                    stepName = 'Clean up file input';

                return stepName;
            }

            stepName += getElementName(element);

            var $label = $('label[for="' + element.id + '"]'),
                croppedLabelForElement = $label.length ? cropText($label.text()) : '';

            switch (tagName) {
                case 'body':
                case 'html':
                    break;

                case 'a':
                case 'map':
                case 'object':
                case 'output':
                    stepName += generateElementDescriptionByAttributes(['text', 'title', 'name']);
                    break;

                case 'area':
                case 'img':
                    stepName += generateElementDescriptionByAttributes(['title', 'alt']);
                    break;

                case 'form':
                case 'iframe':
                case 'keygen':
                case 'select':
                    stepName += generateElementDescriptionByAttributes(['title', 'name']);
                    break;

                case 'optgroup':
                case 'track':
                    stepName += generateElementDescriptionByAttributes(['label', 'title']);
                    break;

                case 'button':
                    stepName += generateElementDescriptionByAttributes(['text', 'value', 'title', 'name']);
                    break;

                case 'command':
                    stepName += generateElementDescriptionByAttributes(['text', 'label', 'title']);
                    break;

                case 'details':
                    var nameByText = generateElementDescriptionByAttributes(['text']);
                    if (nameByText)
                        stepName += nameByText;
                    else {
                        var $summary = $(element).children('summary');
                        if ($summary.length && $.trim($summary.text()))
                            stepName += ' "' + cropText($summary.text()) + '"';
                        else
                            stepName += generateElementDescriptionByAttributes(['title']);
                    }
                    break;

                case 'fieldset':
                    var $legend = $(element).children('legend');
                    if ($legend.length && $.trim($legend.text()))
                        stepName += ' "' + cropText($legend.text()) + '"';
                    else {
                        var nameWithoutLegend = generateElementDescriptionByAttributes(['text', 'title', 'name']);
                        if (nameWithoutLegend)
                            stepName += nameWithoutLegend;
                        else if ($legend.length && $legend[0].title)
                            stepName += ' "' + cropText($legend[0].title) + '"';
                    }
                    break;

                case 'figure':
                    var $figcaption = $(element).children('figcaption');
                    if ($figcaption.length && $.trim($figcaption.text()))
                        stepName += ' "' + cropText($figcaption.text()) + '"';
                    else
                        stepName += generateElementDescriptionByAttributes(['text', 'title']);
                    break;

                case 'input':
                    if (!elementType || (elementType && (elementType === 'text' || elementType === 'password' ||
                        elementType === 'hidden' || elementType === 'file'))) {
                        if (croppedLabelForElement)
                            stepName += ' "' + croppedLabelForElement + '"';
                        else stepName += generateElementDescriptionByAttributes(['title', 'name']);
                    }
                    else if (elementType && elementType === 'image') {
                        if (croppedLabelForElement)
                            stepName += ' "' + croppedLabelForElement + '"';
                        else stepName += generateElementDescriptionByAttributes(['title', 'alt', 'name']);
                    }
                    else if (elementType && (elementType === 'button' || elementType === 'submit' ||
                        elementType === 'reset')) {
                        if (generateElementDescriptionByAttributes(['value']))
                            stepName += generateElementDescriptionByAttributes(['value']);
                        else if (croppedLabelForElement)
                            stepName += ' "' + croppedLabelForElement + '"';
                        else
                            stepName += generateElementDescriptionByAttributes(['title', 'name']);
                    }
                    else if (elementType && (elementType === 'checkbox' || elementType === 'radio')) {
                        if (croppedLabelForElement)
                            stepName += ' "' + croppedLabelForElement + '"';
                        else
                            stepName += generateElementDescriptionByAttributes(['title', 'value', 'name']);
                    }
                    break;

                case 'menu':
                    stepName += generateElementDescriptionByAttributes(['text', 'label', 'title']);
                    break;

                case 'option':
                    stepName += generateElementDescriptionByAttributes(['label', 'text', 'title', 'value']);
                    break;

                case 'table':
                    var $caption = $(element).children('caption');
                    if ($caption.length && $.trim($caption.text()))
                        stepName += ' "' + cropText($caption.text()) + '"';
                    else
                        stepName += generateElementDescriptionByAttributes(['text', 'title']);
                    break;

                case 'textarea':
                    if (croppedLabelForElement)
                        stepName += ' "' + croppedLabelForElement + '"';
                    else
                        stepName += generateElementDescriptionByAttributes(['title', 'name']);
                    break;

                default :
                    stepName += generateElementDescriptionByAttributes(['text', 'title']);
            }

            if (elements) {
                if (elements.length === 2 && elements[0].tagName.toLowerCase() === 'select' &&
                    elements[1].tagName.toLowerCase() === 'select' && elements[0] === elements[1]) {
                    stepName += ' twice';
                }
                else {
                    $.each(elements, function (index, el) {
                        if (index !== 0) {
                            currentDescriptor = $.extend(currentDescriptor, actionDescriptor);
                            currentDescriptor.element = el;
                            delete currentDescriptor.elements;
                            currentName = exports.generateStepName(currentDescriptor);
                            stepName += ' and ' + currentName.charAt(0).toLowerCase() + currentName.substring(1);
                        }
                    });
                }
            }

            return stepName;
        };
    }
);