TestCafeClient.define('UI', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        NativeMethods = Hammerhead.NativeMethods,
        ProgressPanel = require('UI.ProgressPanel');

    //Progress Panel
    var PROGRESS_PANEL_UPDATE_INTERVAL = 100,
        PROGRESS_PANEL_OPENING_DELAY = 300,
        PROGRESS_PANEL_MIN_SHOWING_TIME = 1000;

    var progressPanel = null,
        progressTimeout = null,
        progressInterval = null,
        progressStartTime = null;

    exports.showProgressPanel = function (text, maxTimeout, callback) {
        progressStartTime = Date.now();

        function getProgress() {
            return Math.round((Date.now() - progressStartTime) / maxTimeout * 100);
        }

        progressTimeout = NativeMethods.setTimeout.call(window, function () {
            progressPanel = new ProgressPanel(text, getProgress());
            progressTimeout = null;
            progressInterval = NativeMethods.setInterval.call(window, function () {
                progressPanel.setValue(getProgress());
            }, PROGRESS_PANEL_UPDATE_INTERVAL);

            if (typeof callback === 'function')
                callback();
        }, PROGRESS_PANEL_OPENING_DELAY);
    };

    exports.closeProgressPanel = function (callback, success) {
        if (progressPanel && success) {
            progressPanel.setSuccess(true);
        }

        if (progressTimeout) {
            window.clearTimeout(progressTimeout);
            progressTimeout = null;
        }

        if (progressInterval) {
            window.clearInterval(progressInterval);
            progressInterval = null;
        }

        if (success && (progressPanel && progressStartTime && Date.now() - progressStartTime < PROGRESS_PANEL_MIN_SHOWING_TIME)) {
            progressStartTime = null;

            NativeMethods.setTimeout.call(window, function () {
                exports.closeProgressPanel(callback, success);
            }, PROGRESS_PANEL_UPDATE_INTERVAL);

            return;
        }

        function close(force) {
            progressPanel.close(force);
            progressPanel = null;

            if (typeof callback === 'function')
                callback();
        }

        if (progressPanel) {
            if (success)
                NativeMethods.setTimeout.call(window, close, 200);
            else
                close(true);
        }

        progressStartTime = null;
    };
});