TestCafeClient.define('UI.Cursor', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        NativeMethods = Hammerhead.NativeMethods,
        Settings = require('Settings'),
        CursorBehavior = require('UI.Cursor.Behavior'),
        CursorIFrameBehavior = require('UI.Cursor.IFrameBehavior');

    //Global
    var cursorBehavior = null;

    exports.init = function () {
        if (!cursorBehavior)
            cursorBehavior = window.top !== window.self ? new CursorIFrameBehavior() : new CursorBehavior();
    };

    exports.ensureCursorPosition = function (position, withoutOffset, callback) {
        if (cursorBehavior.isStarted() && cursorBehavior.getPosition()) {
            callback();
            return;
        }

        var cursorPosition = {
            x: Math.max(0, position.x - (withoutOffset ? 0 : 50)),
            y: Math.max(0, position.y - (withoutOffset ? 0 : 50))
        };

        exports.start(cursorPosition, callback);
    };

    exports.setPosition = function (position) {
        if (!cursorBehavior.isStarted()) {
            exports.start(position, function () {
            });
        }
        else
            cursorBehavior.cursorPosition = position;
    };

    exports.start = function (position, callback, iFrameInitiator) {
        if (!cursorBehavior.isStarted()) {
            if (!Settings.RECORDING || Settings.PLAYBACK) {
                cursorBehavior.start(position, iFrameInitiator);

                cursorBehavior.on(cursorBehavior.STARTED_EVENT, callback);
            }
            else
                callback();
        }
        else {
            cursorBehavior.move(position, function () {
                NativeMethods.setTimeout.call(window, callback, 0);
            }, iFrameInitiator);
        }
    };

    exports.move = function (to, callback) {
        cursorBehavior.move(to, callback);
    };

    exports.lMouseDown = function (callback) {
        cursorBehavior.lMouseDown(callback);
    };

    exports.rMouseDown = function (callback) {
        cursorBehavior.rMouseDown(callback);
    };

    exports.mouseUp = function (callback) {
        cursorBehavior.mouseUp(callback);
    };

    exports.hide = function (callback) {
        cursorBehavior.hide(callback);
    };

    exports.show = function (callback) {
        cursorBehavior.show(callback);
    };

    exports.getElementUnderCursor = function (x, y, currentDocument) {
        if (cursorBehavior)
            return cursorBehavior.getElementUnderCursor(x, y, currentDocument);
    };

    exports.getPosition = function () {
        if (cursorBehavior.getPosition)
            return cursorBehavior.getPosition();

        return null;
    };

    //NOTE: for testing purposes
    exports.getAbsolutePosition = function () {
        if (cursorBehavior.getAbsolutePosition)
            return cursorBehavior.getAbsolutePosition();

        return null;
    };
});