TestCafeClient.define('UI.Cursor.Behavior', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        MessageSandbox = Hammerhead.MessageSandbox,
        NativeMethods = Hammerhead.NativeMethods,

        Settings = require('Settings'),
        CursorBaseBehavior = require('UI.Cursor.BaseBehavior');

    //Const
    var CURSOR_CLASS = 'cursor',
        TOUCH_CLASS = 'touch';

    var CursorBehavior = this.exports = function () {
        var cursorBehavior = this;

        $(window).scroll(function () {
            var cursorPosition = cursorBehavior ? cursorBehavior.cursorPosition : null;

            if (cursorPosition)
                cursorBehavior.move({
                    x: cursorPosition.x,
                    y: cursorPosition.y
                });
        });

        this._initCursorIFrameBehavior();

        this.$cursor = $('<div></div>');
        ShadowUI.addClass(this.$cursor, CURSOR_CLASS);

        this.$cursor.appendTo(ShadowUI.getRoot());
        this.hide();

        CursorBaseBehavior.call(this);
    };

    Util.inherit(CursorBehavior, CursorBaseBehavior);

    function withCursorEmulation() {
        return !Settings.RECORDING || Settings.PLAYBACK;
    }

    CursorBehavior.prototype.start = function (position, iFrameInitiator) {
        var cursorBehavior = this;

        //NOTE: For IE in Cross domain iframe we can't use touch cursor because we won't be able to get element under cursor
        //for more information look at HACK in cursor_iframe_behavior
        if (Util.isTouchDevice && !(Util.isIE && iFrameInitiator)) {
            ShadowUI.addClass(this.$cursor, TOUCH_CLASS);

            //NOTE: in touch mode pointer should be in the center of the cursor
            this.pointerOffsetX = Math.ceil(this.$cursor.width() / 2);
            this.pointerOffsetY = Math.ceil(this.$cursor.height() / 2);
        }

        this.move(Util.getFixedPosition(position, iFrameInitiator, true));

        if (withCursorEmulation())
            this.show();

        CursorBaseBehavior.prototype.start.call(this);

        NativeMethods.setTimeout.call(window, function () {
            cursorBehavior.eventEmitter.emit(cursorBehavior.STARTED_EVENT, null);
        }, 0);
    };

    CursorBehavior.prototype._initCursorIFrameBehavior = function () {
        var cursor = this;

        function onMessage(e) {
            var message = e.message;

            switch (message.cmd) {
                case CursorBaseBehavior.CURSOR_MOVE_REQUEST_CMD:
                    cursor.move(message.position, function () {
                        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_MOVE_RESPONSE_CMD}, e.source);
                    }, e.source);
                    break;

                case CursorBaseBehavior.CURSOR_LMOUSEDOWN_REQUEST_CMD:
                    cursor.lMouseDown(function () {
                        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_LMOUSEDOWN_RESPONSE_CMD}, e.source);
                    });
                    break;

                case CursorBaseBehavior.CURSOR_RMOUSEDOWN_REQUEST_CMD:
                    cursor.rMouseDown(function () {
                        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_RMOUSEDOWN_RESPONSE_CMD}, e.source);
                    });
                    break;

                case CursorBaseBehavior.CURSOR_MOUSEUP_REQUEST_CMD:
                    cursor.mouseUp(function () {
                        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_MOUSEUP_RESPONSE_CMD}, e.source);
                    });
                    break;

                case CursorBaseBehavior.CURSOR_HIDE_REQUEST_CMD:
                    cursor.hide(function () {
                        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_HIDE_RESPONSE_CMD}, e.source);
                    });
                    break;

                case CursorBaseBehavior.CURSOR_SHOW_REQUEST_CMD:
                    cursor.show(function () {
                        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_SHOW_RESPONSE_CMD}, e.source);
                    });
                    break;
            }
        }

        MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
    };

    CursorBehavior.prototype.getElementUnderCursor = function (x, y, currentDocument) {
        var isCursorVisible = this.$cursor.css('visibility') !== 'hidden';

        if (isCursorVisible)
            this.hide();

        var element = Util.getElementFromPoint(x, y, currentDocument);

        if (isCursorVisible)
            this.show();

        return element;
    };

    //NOTE: for testing purposes
    CursorBehavior.prototype.getAbsolutePosition = function () {
        if (this.$cursor) {
            var offset = Util.getOffsetPosition(this.$cursor[0]),
                x = Math.round(offset.left) + this.pointerOffsetX,
                y = Math.round(offset.top) + this.pointerOffsetY;

            return { x: x, y: y };
        }

        return null;
    };
});