TestCafeClient.define('UI.Cursor.IFrameBehavior', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        Util = Hammerhead.Util,
        MessageSandbox = Hammerhead.MessageSandbox,

        CursorBaseBehavior = require('UI.Cursor.BaseBehavior'),
        CrossDomainMessages = require('Base.CrossDomainMessages');

    //HACK: we can't get element under cursor in cross-domain iframe in IE without hide cursor.
    //Instead make 'getElement' method asynchronous, we set cursor to position by one pixel farther.
    var RECOGNITION_INCREMENT = Util.isIE ? 1 : 0;

    //NOTE: iFrameInitiator - only for organization of common interface
    var CursorIFrameBehavior = this.exports = function () {
        CursorBaseBehavior.call(this);
    };

    Util.inherit(CursorIFrameBehavior, CursorBaseBehavior);

    CursorIFrameBehavior.prototype._bindMessageHandler = function (msg, callback) {
        function _onMessageHandler(e) {
            if (e.message && e.message.cmd === msg) {
                MessageSandbox.off(MessageSandbox.SERVICE_MSG_RECEIVED, _onMessageHandler);

                if (callback)
                    callback();
            }
        }

        MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, _onMessageHandler);
    };

    CursorIFrameBehavior.prototype.start = function (position) {
        var cursorBehavior = this,
            msg = {
                cmd: CrossDomainMessages.CURSOR_START_REQUEST_CMD,
                position: {
                    x: position.x + RECOGNITION_INCREMENT,
                    y: position.y + RECOGNITION_INCREMENT
                }
            };

        this.cursorPosition = position;

        this._bindMessageHandler(CrossDomainMessages.CURSOR_START_RESPONSE_CMD, function () {
            CursorBaseBehavior.prototype.start.call(cursorBehavior);
            cursorBehavior.eventEmitter.emit(cursorBehavior.STARTED_EVENT, null);
        });

        MessageSandbox.sendServiceMsg(msg, window.top);
    };

    CursorIFrameBehavior.prototype.move = function (to, callback) {
        this.cursorPosition = to;

        //NOTE: we need to wait for response message to call callback
        this._bindMessageHandler(CursorBaseBehavior.CURSOR_MOVE_RESPONSE_CMD, callback);

        var msg = {
            cmd: CursorBaseBehavior.CURSOR_MOVE_REQUEST_CMD,
            position: {
                x: to.x + RECOGNITION_INCREMENT,
                y: to.y + RECOGNITION_INCREMENT
            }
        };

        MessageSandbox.sendServiceMsg(msg, window.top);
    };

    CursorIFrameBehavior.prototype.lMouseDown = function (callback) {
        //NOTE: we need to wait for response message to call callback
        this._bindMessageHandler(CursorBaseBehavior.CURSOR_LMOUSEDOWN_RESPONSE_CMD, callback);

        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_LMOUSEDOWN_REQUEST_CMD}, window.top);
    };

    CursorIFrameBehavior.prototype.rMouseDown = function (callback) {
        //NOTE: we need to wait for response message to call callback
        this._bindMessageHandler(CursorBaseBehavior.CURSOR_RMOUSEDOWN_RESPONSE_CMD, callback);

        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_RMOUSEDOWN_REQUEST_CMD}, window.top);
    };

    CursorIFrameBehavior.prototype.mouseUp = function (callback) {
        //NOTE: we need to wait for response message to call callback
        this._bindMessageHandler(CursorBaseBehavior.CURSOR_MOUSEUP_RESPONSE_CMD, callback);

        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_MOUSEUP_REQUEST_CMD}, window.top);
    };

    CursorIFrameBehavior.prototype.hide = function (callback) {
        //NOTE: we need to wait for response message to call callback
        this._bindMessageHandler(CursorBaseBehavior.CURSOR_HIDE_RESPONSE_CMD, callback);

        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_HIDE_REQUEST_CMD}, window.top);
    };

    CursorIFrameBehavior.prototype.show = function (callback) {
        //NOTE: we need to wait for response message to call callback
        this._bindMessageHandler(CursorBaseBehavior.CURSOR_SHOW_RESPONSE_CMD, callback);

        MessageSandbox.sendServiceMsg({cmd: CursorBaseBehavior.CURSOR_SHOW_REQUEST_CMD}, window.top);
    };

    CursorIFrameBehavior.prototype.getElementUnderCursor = function (x, y, currentDocument) {
        return Util.getElementFromPoint(x, y, currentDocument);
    };
});