TestCafeClient.define('UI.ModalBackground', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        ShadowUI = Hammerhead.ShadowUI,
        NativeMethods = Hammerhead.NativeMethods;

    //Const
    var LOADING_TEXT = 'Loading page...',
        BACKGROUND_CLASS = 'modal-background',
        LOADING_TEXT_CLASS = 'loading-text',
        BACKGROUND_OPACITY = 0.7,
        BACKGROUND_OPACITY_WITH_LOADING_TEXT = 0.8,

        LOADING_ICON_CLASS = 'loading-icon';

    //Globals
    var $window = $(window),
        $backgroundDiv = null,
        $loadingText = null,
        $loadingIcon = null,
        initialized = false;

    //Markup
    function createBackground() {
        var $root = ShadowUI.getRoot();

        $backgroundDiv = $('<div></div>').appendTo($root);
        ShadowUI.addClass($backgroundDiv, BACKGROUND_CLASS);

        $loadingText = $('<div></div>')
            .appendTo($root)
            .text(LOADING_TEXT);

        ShadowUI.addClass($loadingText, LOADING_TEXT_CLASS);

        $loadingIcon = $('<div></div>').css('visibility', 'hidden').appendTo($root);
        ShadowUI.addClass($loadingIcon, LOADING_ICON_CLASS);
    }

    //Behavior
    function adjustLoadingTextPos() {
        var wHeight = $window.height(),
            wWidth = $window.width();

        var loadingTextVisible = $loadingText.is(':visible');

        if (!loadingTextVisible) {
            $loadingText.attr('visibility', 'hidden');
            $loadingText.show();
        }

        $loadingText.css({
            left: Math.max((wWidth - $loadingText.width()) / 2, 0),
            top: Math.max((wHeight - $loadingText.height()) / 2, 0)
        });

        if (!loadingTextVisible) {
            $loadingText.hide();
            $loadingText.attr('visibility', '');
        }
    }

    function initSizeAdjustments() {
        var adjust = function () {
            var wHeight = $window.height(),
                wWidth = $window.width();

            $backgroundDiv.width(wWidth);
            $backgroundDiv.height(wHeight);

            $loadingIcon.css('top', Math.round((wHeight - $loadingIcon.height()) / 2));
            $loadingIcon.css('left', Math.round((wWidth - $loadingIcon.width()) / 2));
        };

        adjust();

        $window.resize(adjust);
    }

    function init() {
        createBackground();
        initSizeAdjustments();
        adjustLoadingTextPos();

        initialized = true;
    }

    exports.initAndShowLoadingText = function () {
        var shown = false;

        //NOTE: init and show modal background as soon as possible
        var initAndShow = function () {
            init();

            $backgroundDiv.css({opacity: BACKGROUND_OPACITY_WITH_LOADING_TEXT });
            $backgroundDiv.show();
            $loadingText.show();

            shown = true;
        };

        var tryShowBeforeReady = function () {
            if (!shown) {
                if (document.body)
                    initAndShow();
                else
                    NativeMethods.setTimeout.call(window, tryShowBeforeReady, 0);
            }
        };

        tryShowBeforeReady();

        //NOTE: ensure that background was shown on ready
        $(document).ready(function () {
            if (!shown)
                initAndShow();
        });
    };

    exports.show = function (transparent) {
        if (!initialized)
            init();

        $backgroundDiv.css({opacity: transparent ? 0 : BACKGROUND_OPACITY});
        $backgroundDiv.show();
    };

    exports.hide = function () {
        if (!initialized)
            return;

        $loadingText.hide();
        $backgroundDiv.hide();
    };

    exports.showLoadingIcon = function () {
        $loadingIcon.css('visibility', 'visible');
    };

    exports.hideLoadingIcon = function () {
        $loadingIcon.css('visibility', 'hidden');
    };
});