TestCafeClient.define('UI.SelectElement', function (require, exports) {
    //NOTE: we can't manipulate (open/close option list) with a native select element during test running, so we
    // draw our custom option list to emulate this.
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        EventSimulator = Hammerhead.EventSimulator,
        NativeMethods = Hammerhead.NativeMethods;

    var OPTION_LIST_CLASS = 'tcOptionList',
        OPTION_GROUP_CLASS = 'tcOptionGroup',
        OPTION_CLASS = 'tcOption',
        DISABLED_CLASS = 'disabled';

    var $curSelectEl = null,
        $optionList = null,
        $groups = null,
        $options = null;

    function onDocumentMouseDown(e) {
        if ((e.target || e.srcElement) !== $curSelectEl[0])
            exports.collapseOptionList();
    }

    function createOption(option, $parent) {
        var $option = $('<div></div>')
            .text(option.text)
            .appendTo($parent);

        ShadowUI.addClass($option, OPTION_CLASS);

        if (option.disabled || (option.parentElement.tagName.toLowerCase() === 'optgroup' && option.parentElement.disabled)) {
            ShadowUI.addClass($option, DISABLED_CLASS);

            $option.css('color', $(option).css('color'));
            $option.click(function () {
                return false;
            });
        }
        else {
            $option.click(function () {
                var curSelectEl = $curSelectEl[0],
                    optionIndex = $.inArray(this, $options),
                    curSelectIndex = curSelectEl.selectedIndex;

                curSelectEl.selectedIndex = optionIndex;

                NativeMethods.setTimeout.call(window, function () {
                    if (optionIndex !== curSelectIndex)
                        EventSimulator.change(curSelectEl);

                    if (Util.isWebKit)
                        EventSimulator.click(curSelectEl);
                    else
                        EventSimulator.click($(curSelectEl).find('option')[curSelectEl.selectedIndex]);
                }, 0);

                exports.collapseOptionList();
            });
        }

        $options = !$options || !$options.length ? $option : $options.add($option);
    }

    function createGroup(group, $parent) {
        var $group = $('<div></div>')
            .text(group.label || ' ')
            .appendTo($parent);

        ShadowUI.addClass($group, OPTION_GROUP_CLASS);

        if (group.disabled) {
            ShadowUI.addClass($group, DISABLED_CLASS);

            $group.css('color', $(group).css('color'));
        }

        createChildren($(group).children(), $group);

        $groups = !$groups || !$groups.length ? $group : $groups.add($group);
    }

    function createChildren($children, $parent) {
        $.each($children, function (index, item) {
            if (item.tagName.toLowerCase() === 'option')
                createOption(item, $parent);
            else if (item.tagName.toLowerCase() === 'optgroup')
                createGroup(item, $parent);
        });
    }

    exports.expandOptionList = function (select) {
        var $select = $(select),
            $selectChildren = $(select).children();

        if (!$selectChildren.length)
            return;

        //NOTE: check is option list expanded
        if ($curSelectEl) {
            var isSelectExpanded = $select[0] === $curSelectEl[0];

            exports.collapseOptionList();

            if (isSelectExpanded)
                return;
        }

        $curSelectEl = $select;

        $optionList = $('<div></div>').appendTo(ShadowUI.getRoot());
        ShadowUI.addClass($optionList, OPTION_LIST_CLASS);

        createChildren($selectChildren, $optionList);

        NativeMethods.setTimeout.call(window, function () {
            $(document).bind('mousedown', onDocumentMouseDown);
        }, 0);

        $optionList.css({
            position: 'absolute',
            fontSize: $curSelectEl.css('fontSize'),
            fontFamily: $curSelectEl.css('fontFamily'),
            minWidth: $curSelectEl.width(),
            left: Util.getOffsetPosition($curSelectEl[0]).left,
            height: Util.getSelectVisibleChildren($select).length > Util.MAX_OPTION_LIST_LENGTH ? Util.getOptionHeight($(select)) * Util.MAX_OPTION_LIST_LENGTH : ''
        });

        var $window = $(window),
            selectTopPosition = Util.getOffsetPosition($curSelectEl[0]).top,
            optionListHeight = $optionList.height(),
            optionListTopPosition = selectTopPosition + $curSelectEl.height() + 2;

        if (optionListTopPosition + optionListHeight > $window.scrollTop() + $window.height()) {
            var topPositionAboveSelect = selectTopPosition - 3 - optionListHeight;

            if (topPositionAboveSelect >= $window.scrollTop())
                optionListTopPosition = topPositionAboveSelect;
        }

        $optionList.css('top', optionListTopPosition);
    };

    exports.collapseOptionList = function () {
        $optionList.remove();
        $(document).unbind('mousedown', onDocumentMouseDown);

        $optionList = null;
        $curSelectEl = null;
        $options = null;
        $groups = null;
    };

    exports.isOptionListExpanded = function ($select) {
        return $select ? $select.is($curSelectEl) : !!$curSelectEl;
    };

    exports.getEmulatedChildElement = function (elementIndex, isGroup) {
        if (!isGroup)
            return $options[elementIndex];

        return $groups[elementIndex];
    };

    exports.scrollOptionListByChild = function (child) {
        var select = Util.getSelectParent($(child));

        if (!select)
            return;

        var $select = $(select),
            realSizeValue = Util.getSelectSizeValue($select, true),
            optionHeight = Util.getOptionHeight($select),
            scrollIndent = 0,

            topVisibleIndex = Math.max($select.scrollTop() / optionHeight, 0),
            bottomVisibleIndex = topVisibleIndex + realSizeValue - 1,

            childIndex = Util.getChildVisibleIndex($select, child);

        if (childIndex < topVisibleIndex) {
            scrollIndent = optionHeight * (topVisibleIndex - childIndex);
            $select.scrollTop(Math.max($select.scrollTop() - scrollIndent, 0));
        }
        else if (childIndex > bottomVisibleIndex) {
            scrollIndent = optionHeight * (childIndex - bottomVisibleIndex);
            $select.scrollTop($select.scrollTop() + scrollIndent);
        }
    };

    exports.getSelectChildCenter = function (child) {
        var select = Util.getSelectParent($(child));

        if (!select) {
            return {
                x: 0,
                y: 0
            };
        }

        var optionHeight = Util.getOptionHeight($(select)),
            childRectangle = Util.getElementRectangle(child);

        return {
            x: Math.round(childRectangle.left + childRectangle.width / 2),
            y: Math.round(childRectangle.top + optionHeight / 2)
        };
    };

    exports.switchOptionsByKeys = function (command) {
        var $select = $(Util.getActiveElement());

       if ($select[0].tagName.toLowerCase() !== 'select')
          return;

        if (/enter|tab|esc/.test(command) && exports.isOptionListExpanded($select))
            exports.collapseOptionList();

        if (/down|up/.test(command) ||
            (!Util.isIE && (Util.getSelectSizeValue($select) <= 1 || Util.isMozilla) && (!ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible') || Util.isMozilla) && /left|right/.test(command))) {

            var $options = $select.find('option'),
                $enabledOptions = $options.filter(function () {
                    var parent = $(this).parent()[0];
                    return !this.disabled && !(parent.tagName.toLowerCase() === 'optgroup' && parent.disabled);
                }),
                nextIndex = $.inArray($select.find('option:selected')[0], $enabledOptions);

            nextIndex += /down|right/.test(command) ? 1 : -1;

            if (nextIndex >= 0 && nextIndex < $enabledOptions.length) {
                $select[0].selectedIndex = $.inArray($enabledOptions[nextIndex], $options);
                EventSimulator.change($select[0]);
            }
        }
    };
});