TestCafeClient.define('UI.DraggingBehavior', function () {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        EventSandbox = Hammerhead.EventSandbox;

    var DraggingBehavior = this.exports = function ($area, $draggableElement, dragHandlers) {
        this.$dragArea = $area;
        this.$draggable = $draggableElement;
        this.handlers = dragHandlers;

        this.dragging = false;
        this.draggingStopped = false;
        this.dragEventRaised = false;
        this.maxTop = null;
        this.dragOffsetX = 0;
        this.dragOffsetY = 0;

        this._init();
    };

    DraggingBehavior.prototype._init = function () {
        var draggingBehavior = this;

        this.$dragArea.css('cursor', 'move');

        ShadowUI.bind(this.$dragArea, 'mousedown', function (evt) {
            if (draggingBehavior.draggingStopped)
                return;

            draggingBehavior.dragging = true;

            var offset = Util.getOffsetPosition(draggingBehavior.$draggable[0]);
            draggingBehavior.dragOffsetX = (evt.pageX || evt.x) - offset.left;
            draggingBehavior.dragOffsetY = (evt.pageY || evt.y) - offset.top;

            //NOTE: prevent text selection
            Util.preventDefault(evt);
            return false;
        });

        function onMove(evt) {
            if (draggingBehavior.dragging) {
                if (!draggingBehavior.dragEventRaised) {
                    if (draggingBehavior.handlers && draggingBehavior.handlers.onDragStart)
                        draggingBehavior.handlers.onDragStart();
                    draggingBehavior.dragEventRaised = true;
                }
                var left = Math.max(evt.clientX - draggingBehavior.dragOffsetX, 0),
                    top = Math.max(evt.clientY - draggingBehavior.dragOffsetY, 0),
                    draggableWidth = draggingBehavior.$draggable.width(),
                    draggableHeight = draggingBehavior.$draggable.height(),
                    windowWidth = $(window).width(),
                    windowHeight = $(window).height();

                if (left + draggableWidth > windowWidth)
                    left = windowWidth - draggableWidth;

                if (top + draggableHeight > windowHeight)
                    top = windowHeight - draggableHeight;

                draggingBehavior.$draggable.css({
                    left: Math.max(left, 0),
                    top: draggingBehavior.maxTop !== null ? Math.min(draggingBehavior.maxTop, Math.max(top, 0)) : Math.max(top, 0)});

                if (draggingBehavior.handlers && draggingBehavior.handlers.onMove)
                    draggingBehavior.handlers.onMove();

                //NOTE: prevent text selection
                Util.preventDefault(evt);
                return false;
            }
        }

        EventSandbox.addInternalEventListener(window, ['mousemove'], function (e, dispatched) {
            if (!dispatched)
                onMove(e);
        });

        function onMouseUp(evt) {
            if (draggingBehavior.dragging) {
                draggingBehavior.dragging = false;

                if (draggingBehavior.dragEventRaised) {
                    if (draggingBehavior.handlers && draggingBehavior.handlers.onDragEnd)
                        draggingBehavior.handlers.onDragEnd();

                    draggingBehavior.dragEventRaised = false;
                }
                Util.preventDefault(evt);
            }
        }

        EventSandbox.addInternalEventListener(window, ['mouseup'], function (e, dispatched) {
            if (!dispatched)
                onMouseUp(e);
        });
    };

    DraggingBehavior.prototype.resetDragging = function () {
        if (this.$dragArea) {
            this.$dragArea.css('cursor', 'auto');

            this.draggingStopped = true;
        }
    };

    DraggingBehavior.prototype.restoreDragging = function () {
        this.$dragArea.css('cursor', 'move');
        this.draggingStopped = false;
    };

    DraggingBehavior.prototype.setMaxTop = function (top) {
        this.maxTop = top;
    };
});