TestCafeClient.define('UI.Recorder', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        EventSandbox = Hammerhead.EventSandbox,
        NativeMethods = Hammerhead.NativeMethods,
        ShadowUI = Hammerhead.ShadowUI,
        Transport = require('Base.Transport'),
        ServiceCommands = require('Shared.ServiceCommands'),
        Settings = require('Settings'),
        Automation = require('Automation'),
        RecorderUtil = require('Recorder.Util'),
        ModalBackground = require('UI.ModalBackground'),
        ToolbarWidget = require('UI.RecorderWidgets.Toolbar'),
        StepsPanel = require('UI.RecorderWidgets.StepsPanel'),
        SetTestNameDialog = require('UI.RecorderWidgets.SetTestNameDialog'),
        UnsavedChangesDialog = require('UI.RecorderWidgets.UnsavedChangesDialog'),
        BackToInitialPageDialog = require('UI.RecorderWidgets.BackToInitialPage'),
        AddActionDialog = require('UI.RecorderWidgets.AddActionDialog'),
        ElementPicker = require('Recorder.ElementPicker'),
        Tooltip = require('UI.RecorderWidgets.Tooltip');

    //Const
    var ADD_HOVER_ACTION_SHORTCUT = Automation.ADD_ACTION_SHORTCUTS.hover,
        ADD_WAIT_ACTION_SHORTCUT = Automation.ADD_ACTION_SHORTCUTS.wait,
        ADD_SCREENSHOT_SHORTCUT = Automation.ADD_ACTION_SHORTCUTS.screenshot,
        ADD_ASSERTION_STEP_SHORTCUT = 'Ctrl+R',
        RUN_PLAYBACK_SHORTCUT = 'Ctrl+D',
        SAVE_TEST_SHORTCUT = 'Ctrl+S',
        EXIT_RECORDING_SHORTCUT = 'Ctrl+E',
        COMPLETE_TYPING_SHORTCUT = 'Ctrl+G',

        RECORDER_CLASS = 'recorder',
        COMPLETE_BUTTON_CLASS = 'complete-icon',
        COMPLETE_TYPING_BUTTON_CLASS = 'typing',
        COMPLETE_TYPING_BUTTON_TITLE = 'Complete Typing' + ' (' + COMPLETE_TYPING_SHORTCUT + ')',
        COMPLETE_CLICK_EDITOR_BUTTON_CLASS = 'click',
        COMPLETE_CLICK_EDITOR_BUTTON_TITLE = 'Complete Click' + ' (' + COMPLETE_TYPING_SHORTCUT + ')';

    exports.ADD_HOVER_ACTION_EVENT = 'addHoverAction';
    exports.ADD_WAIT_ACTION_EVENT = 'addWaitAction';
    exports.ADD_SCREENSHOT_ACTION_EVENT = 'addScreenshotAction';
    exports.ADD_ASSERTIONS_STEP_EVENT = 'addAssertionsStep';
    exports.RUN_PLAYBACK_EVENT = 'runPlayback';
    exports.SAVE_TEST_EVENT = 'saveTest';
    exports.EXIT_RECORDING_EVENT = 'exitRecording';
    exports.SEND_STEPS_INFO_EVENT = 'sendStepsInfo';
    exports.RESTART_RECORDING_EVENT = 'restartRecording';
    exports.RESUME_PLAYBACK_EVENT = 'resumePlayback';
    exports.POPUP_DIALOG_OPENING_EVENT = 'popupDialogOpening';
    exports.POPUP_DIALOG_CLOSED_EVENT = 'popupDialogClosed';
    exports.START_PICKING_ELEMENT_EVENT = 'startPickingElement';
    exports.STOP_PICKING_ELEMENT_EVENT = 'stopPickingElement';
    exports.STEP_EDITING_STARTED = 'stepEditingStarted';
    exports.STEP_EDITING_FINISHED = 'stepEditingFinished';

    exports.Shortcuts = {};
    exports.Shortcuts[ADD_HOVER_ACTION_SHORTCUT] = null;
    exports.Shortcuts[ADD_WAIT_ACTION_SHORTCUT] = null;
    exports.Shortcuts[ADD_SCREENSHOT_SHORTCUT] = null;
    exports.Shortcuts[ADD_ASSERTION_STEP_SHORTCUT] = null;
    exports.Shortcuts[RUN_PLAYBACK_SHORTCUT] = null;
    exports.Shortcuts[SAVE_TEST_SHORTCUT] = null;
    exports.Shortcuts[EXIT_RECORDING_SHORTCUT] = null;
    exports.Shortcuts[COMPLETE_TYPING_SHORTCUT] = null;

    var UI_MODES = {
        initialized: 'initializedMode',
        playback: 'playbackMode',
        recording: 'recordingMode'
    };

//Globals
    var eventEmitter = new Util.EventEmitter(),
        $recorder = null,
        activeDialog = null,
        openingDialog = false,
        $completeTypingButton = null,
        $completeClickEditorButton = null,
        currentCompleteTypingHandler = null,
        currentCompleteClickEditorHandler = null,
        stepsInfoGetter = null,
        hasChangesGetter = null,
        testSavedGetter = null,
        stepsPanelPlaybackHandlersAssigned = false,

        stepPanelExpanded = true,
        currentMode = null;

//Markup
    function getRecorderContainer() {
        if ($recorder)
            return $recorder;

        $recorder = $('<div>').appendTo(ShadowUI.getRoot());
        ShadowUI.addClass($recorder, RECORDER_CLASS);

        return $recorder;
    }

//Behavior
    function saveTest(complete, args) {
        if (!testSavedGetter()) {
            onPopupDialogOpening();
            setTestName(args, complete);
        }
        else {
            ModalBackground.show();
            ModalBackground.showLoadingIcon();

            eventEmitter.emit(exports.SAVE_TEST_EVENT, {
                callback: function () {
                    ModalBackground.hideLoadingIcon();
                    ModalBackground.hide();
                },
                complete: complete || false
            });
        }
    }

    function setTestName(args, complete) {
        openingDialog = true;
        SetTestNameDialog.init(getRecorderContainer(), args || {});
        openingDialog = false;
        activeDialog = SetTestNameDialog;

        activeDialog.events.on(SetTestNameDialog.CANCEL_SAVE_BUTTON_CLICK_EVENT, function () {
            activeDialog = null;
            onPopupDialogClosed();
        });

        activeDialog.events.on(SetTestNameDialog.SAVE_TEST_BUTTON_CLICK_EVENT, function (e) {
            activeDialog.blind(true);
            ModalBackground.showLoadingIcon();

            eventEmitter.emit(exports.SAVE_TEST_EVENT, {
                callback: function (err) {
                    if (!err) {
                        activeDialog.closeDialog(function () {
                            ModalBackground.hideLoadingIcon();
                            activeDialog = null;
                            if (!complete)
                                onPopupDialogClosed(true);
                        });
                    }
                    else {
                        ModalBackground.hideLoadingIcon();
                        activeDialog.blind(false);
                        SetTestNameDialog.onError(err);
                    }
                },
                testName: e.testName,
                complete: complete || false
            });
        });
    }

    function changesWarning(hasErrors) {
        openingDialog = true;
        UnsavedChangesDialog.init(getRecorderContainer(), hasErrors);
        openingDialog = false;
        activeDialog = UnsavedChangesDialog;

        activeDialog.events.on(UnsavedChangesDialog.CANCEL_BUTTON_CLICK_EVENT, function () {
            activeDialog = null;
            onPopupDialogClosed(true);
        });

        activeDialog.events.on(UnsavedChangesDialog.EXIT_BUTTON_CLICK_EVENT, function () {
            activeDialog = null;
            eventEmitter.emit(exports.EXIT_RECORDING_EVENT, {
                skipCheckChanges: true
            });
        });

        activeDialog.events.on(UnsavedChangesDialog.SAVE_BUTTON_CLICK_EVENT, function () {
            activeDialog = null;
            saveTest(true);
        });
    }

    function openStepWithError(stepNum, err, handlers, recordingMode) {
        ToolbarWidget.updateExpandStepsButtonAppearance(true);

        //T176225 - Cannot hide test steps once a test is failed.
        //Steps Panel should be non collapsible only when open failed assertion step
        if (!recordingMode)
            ToolbarWidget.setExpandStepsButtonState(false);

        StepsPanel.openStepWithError(stepNum, err, recordingMode);

        if (!stepsPanelPlaybackHandlersAssigned && handlers) {
            stepsPanelPlaybackHandlersAssigned = true;

            StepsPanel.events.on(StepsPanel.SKIP_STEP_BUTTON_CLICK_EVENT, function (e) {
                onPopupDialogClosed({
                    sendStepsInfo: true,
                    showToolbar: false
                });

                if (!stepPanelExpanded) {
                    StepsPanel.hide();
                    ToolbarWidget.updateExpandStepsButtonAppearance(stepPanelExpanded);
                }

                ToolbarWidget.setExpandStepsButtonState(true);

                if (handlers && handlers.skipStep)
                    handlers.skipStep(e.hasChanges);
            });

            StepsPanel.events.on(StepsPanel.REPLAY_STEP_BUTTON_CLICK_EVENT, function (e) {
                onPopupDialogClosed({
                    sendStepsInfo: true,
                    showToolbar: false
                });

                if (!stepPanelExpanded) {
                    StepsPanel.hide();
                    ToolbarWidget.updateExpandStepsButtonAppearance(stepPanelExpanded);
                }

                ToolbarWidget.setExpandStepsButtonState(true);

                if (handlers && handlers.resumePlayback)
                    handlers.resumePlayback(e.hasChanges);
            });
        }
    }

//Handlers
    function onPopupDialogOpening(transparentBackground) {
        ModalBackground.show(transparentBackground);

        exports.hideToolbar();

        if ($completeClickEditorButton)
            $completeClickEditorButton.css('visibility', 'hidden');

        if ($completeTypingButton)
            $completeTypingButton.css('visibility', 'hidden');

        eventEmitter.emit(exports.POPUP_DIALOG_OPENING_EVENT, {});
    }

    function onPopupDialogClosed(options) {
        var opts = options || {},
            needSendStepsInfo = typeof opts.sendStepsInfo !== 'undefined' ? opts.sendStepsInfo : false,
            cancelAction = typeof opts.cancelAction !== 'undefined' ? opts.cancelAction : false,
            needShowToolbar = typeof opts.showToolbar !== 'undefined' ? opts.showToolbar : true;

        ModalBackground.hide();

        if (needShowToolbar)
            exports.showToolbar();

        if ($completeClickEditorButton) {
            if (cancelAction) {
                $completeClickEditorButton.remove();
                $completeClickEditorButton = null;
            }
            else
                $completeClickEditorButton.css('visibility', 'visible');
        }
        if ($completeTypingButton)
            $completeTypingButton.css('visibility', 'visible');

        eventEmitter.emit(exports.POPUP_DIALOG_CLOSED_EVENT, {
            needSendStepsInfo: needSendStepsInfo
        });
    }

//Toolbar event handlers
    function onAddHoverActionButtonPressed() {
        eventEmitter.emit(exports.ADD_HOVER_ACTION_EVENT);
    }

    function onAddWaitActionButtonPressed() {
        eventEmitter.emit(exports.ADD_WAIT_ACTION_EVENT);
    }

    function onScreenshotActionButtonPressed(){
        eventEmitter.emit(exports.ADD_SCREENSHOT_ACTION_EVENT);
    }

    function onAddAssertionStepButtonPressed(e) {
        eventEmitter.emit(exports.ADD_ASSERTIONS_STEP_EVENT, e);
    }

    function onRunPlaybackButtonPressed() {
        if (playbackActive)
            eventEmitter.emit(exports.RUN_PLAYBACK_EVENT);
    }

    function onSaveTestButtonPressed() {
        if (!hasChangesGetter())
            return;

        saveTest(false);
    }

    function onExitRecordingButtonPressed(e) {
        eventEmitter.emit(exports.EXIT_RECORDING_EVENT, e);
    }

    function initToolbarEventsHandlers() {
        ToolbarWidget.events.on(ToolbarWidget.ADD_HOVER_ACTION_BUTTON_PRESSED_EVENT, onAddHoverActionButtonPressed);

        ToolbarWidget.events.on(ToolbarWidget.ADD_WAIT_ACTION_BUTTON_PRESSED_EVENT, onAddWaitActionButtonPressed);

        ToolbarWidget.events.on(ToolbarWidget.ADD_ASSERTIONS_STEP_EVENT, onAddAssertionStepButtonPressed);

        ToolbarWidget.events.on(ToolbarWidget.ADD_SCREENSHOT_ACTION_BUTTON_PRESSED_EVENT, onScreenshotActionButtonPressed);

        ToolbarWidget.events.on(ToolbarWidget.RUN_PLAYBACK_BUTTON_PRESSED_EVENT, onRunPlaybackButtonPressed);

        ToolbarWidget.events.on(ToolbarWidget.SAVE_TEST_BUTTON_PRESSED_EVENT, onSaveTestButtonPressed);

        ToolbarWidget.events.on(ToolbarWidget.EXIT_RECORDING_BUTTON_PRESSED_EVENT, onExitRecordingButtonPressed);
    }

    function initStepsPanelHandlers() {
        StepsPanel.events.on(StepsPanel.STEPS_INFO_CHANGED_EVENT, function (e) {
            if (!e.draft)
                eventEmitter.emit(exports.SEND_STEPS_INFO_EVENT);

            exports.updateToolbarButtons();
        });

        StepsPanel.events.on(StepsPanel.ALL_STEPS_DELETED_EVENT, function () {
            Transport.asyncServiceMsg({cmd: ServiceCommands.WERE_ACTIONS_RECORDED}, function (wereActionsRecorded) {
                if (wereActionsRecorded) {
                    onPopupDialogOpening();
                    Tooltip.hideAll();

                    openingDialog = true;
                    BackToInitialPageDialog.init(getRecorderContainer());
                    openingDialog = false;
                    activeDialog = BackToInitialPageDialog;

                    activeDialog.events.on(BackToInitialPageDialog.BACK_TO_INITIAL_PAGE_EVENT, function () {
                        ModalBackground.showLoadingIcon();
                        activeDialog = null;
                        eventEmitter.emit(exports.RESTART_RECORDING_EVENT);
                    });
                }
            });
        });

        StepsPanel.events.on(StepsPanel.EDIT_ASSERTION_START_EVENT, function (e) {
            exports.hideToolbar();

            if (e.showBackground)
                ModalBackground.show();
        });

        StepsPanel.events.on(StepsPanel.EDIT_ASSERTION_COMPLETE_EVENT, function (e) {
            exports.showToolbar(true);

            if (e.hideBackground)
                ModalBackground.hide();

        });

        StepsPanel.events.on(StepsPanel.START_PICKING_ELEMENT_EVENT, function (e) {
            eventEmitter.emit(exports.START_PICKING_ELEMENT_EVENT, e);
        });

        StepsPanel.events.on(StepsPanel.STEP_INFO_SHOWN_EVENT, function (e) {
            ToolbarWidget.updateMaxTopForDragging(e.$el);

            eventEmitter.emit(exports.STEP_EDITING_STARTED, {});
        });

        StepsPanel.events.on(StepsPanel.STEP_INFO_HIDDEN_EVENT, function (e) {
            ToolbarWidget.updateMaxTopForDragging(null);

            if (e.editingFinished)
                eventEmitter.emit(exports.STEP_EDITING_FINISHED, {});
        });

        StepsPanel.events.on(StepsPanel.STEPS_PANEL_BLINDED_EVENT, function (e) {
            ToolbarWidget.setBlind(e.blind);
        });
    }

    function createCompleteTypingButton(rect) {
        var $button = $('<div></div>').attr('title', COMPLETE_TYPING_BUTTON_TITLE).appendTo(getRecorderContainer());

        ShadowUI.addClass($button, COMPLETE_BUTTON_CLASS);
        ShadowUI.addClass($button, COMPLETE_TYPING_BUTTON_CLASS);

        setCompleteTypingButtonPosition($button, rect);

        return $button;
    }

    function createCompleteClickEditorButton(elementRect) {
        var $button = $('<div></div>').attr('title', COMPLETE_CLICK_EDITOR_BUTTON_TITLE).appendTo(getRecorderContainer());

        ShadowUI.addClass($button, COMPLETE_BUTTON_CLASS);
        ShadowUI.addClass($button, COMPLETE_CLICK_EDITOR_BUTTON_CLASS);

        setCompleteTypingButtonPosition($button, elementRect);

        return $button;
    }

    function setCompleteTypingButtonPosition($button, rect) {
        $button.css({
            top: rect.top - $button.height(),
            left: rect.left + rect.width - $button.width()
        });
    }

    function initShortcutHandlers() {
        exports.Shortcuts[ADD_HOVER_ACTION_SHORTCUT] = onAddHoverActionButtonPressed;
        exports.Shortcuts[ADD_WAIT_ACTION_SHORTCUT] = onAddWaitActionButtonPressed;
        exports.Shortcuts[ADD_SCREENSHOT_SHORTCUT] = onScreenshotActionButtonPressed;
        exports.Shortcuts[ADD_ASSERTION_STEP_SHORTCUT] = onAddAssertionStepButtonPressed;
        exports.Shortcuts[RUN_PLAYBACK_SHORTCUT] = onRunPlaybackButtonPressed;
        exports.Shortcuts[SAVE_TEST_SHORTCUT] = onSaveTestButtonPressed;
        exports.Shortcuts[EXIT_RECORDING_SHORTCUT] = onExitRecordingButtonPressed;
        exports.Shortcuts[COMPLETE_TYPING_SHORTCUT] = function () {
            if (typeof currentCompleteTypingHandler === 'function')
                currentCompleteTypingHandler();
            else if (typeof currentCompleteClickEditorHandler === 'function')
                currentCompleteClickEditorHandler();
        };
    }

    function initPreventScrollingUnderRootElements() {
        NativeMethods.addEventListener.call(document, typeof document.onwheel !== 'undefined' ? 'wheel' : 'mousewheel', function (e) {
            var $target = $(e.target);

            if (Util.isShadowUIElement(e.target) && !ShadowUI.hasClass($target, RecorderUtil.ELEMENT_PICKING_CLASSES.elementFrame) && !ShadowUI.hasClass($target, RecorderUtil.ELEMENT_PICKING_CLASSES.elementFramePart) && !ShadowUI.select('.' + RecorderUtil.ELEMENTS_MARKER_CLASS).has($target).length)
                Util.preventDefault(e);
        }, false);
    }

    function createToolbar(mode) {
        var recorderContainer = getRecorderContainer(),
            toolbarOptions = {
                stepsInfo: stepsInfoGetter(),
                hasChanges: hasChangesGetter(),
                toolbarPosition: {
                    left: Settings.RECORDER_TOOLBAR_POS_LEFT,
                    top: Settings.RECORDER_TOOLBAR_POS_TOP
                },
                collapsedMode: mode === UI_MODES.playback,
                shortcuts: {
                    addHover: ADD_HOVER_ACTION_SHORTCUT,
                    addWait: ADD_WAIT_ACTION_SHORTCUT,
                    addScreenshot: ADD_SCREENSHOT_SHORTCUT,
                    addAssertion: ADD_ASSERTION_STEP_SHORTCUT,
                    playback: RUN_PLAYBACK_SHORTCUT,
                    saveTest: SAVE_TEST_SHORTCUT,
                    exitRecording: EXIT_RECORDING_SHORTCUT
                },
                showSteps: stepPanelExpanded,
                enableStepListInteractive: mode === UI_MODES.recording
            };

        ToolbarWidget.init(recorderContainer, toolbarOptions);

        ToolbarWidget.events.on(ToolbarWidget.TOOLBAR_MOVED_EVENT, function (e) {
            var toolbarPosSetMsg = {
                cmd: ServiceCommands.TOOLBAR_POSITION_SET,
                toolbarPosition: e.toolbarPosition
            };

            Transport.queuedAsyncServiceMsg(toolbarPosSetMsg);
        });

        ToolbarWidget.events.on(ToolbarWidget.STEPS_PANEL_VISIBILITY_CHANGED, function (e) {
            stepPanelExpanded = e.shown;

            var toolbarPosSetMsg = {
                cmd: ServiceCommands.CHANGE_SHOW_STEPS,
                show: e.shown
            };

            Transport.queuedAsyncServiceMsg(toolbarPosSetMsg);
        });

        initStepsPanelHandlers();
    }

    //API
    var initialized = false,
        playbackActive = false;

    exports.init = function (getStepsInfoFunction, getHasChangesFlagFunction, isTestSavedFunction, showSteps) {
        stepsInfoGetter = getStepsInfoFunction;
        hasChangesGetter = getHasChangesFlagFunction;
        testSavedGetter = isTestSavedFunction;
        stepPanelExpanded = showSteps;

        currentMode = UI_MODES.initialized;
        initialized = true;
    };

    exports.isInitialized = function () {
        return initialized;
    };

    exports.events = {
        on: function (ev, listener) {
            eventEmitter.on(ev, listener);
        }
    };

    exports.activateRecordingMode = function () {
        var preventUiEvents = function (e, dispatched, preventEvent, cancelHandlers, stopPropagation) {
            if ((activeDialog && (!activeDialog.popup || !activeDialog.popup.isHidden())) || openingDialog) {
                cancelHandlers();

                if (!Util.isShadowUIElement(e.target))
                    stopPropagation();
            }
        };

        EventSandbox.addInternalEventListener(window, Util.DOM_EVENTS, preventUiEvents);
        EventSandbox.addInternalEventListener(document, Util.DOM_EVENTS, preventUiEvents);

        if (currentMode === UI_MODES.initialized)
            createToolbar(UI_MODES.recording);
        else if (currentMode === UI_MODES.playback) {
            StepsPanel.turnOffPlaybackMode();
            StepsPanel.activateInteractive();
        }

        ToolbarWidget.expand();

        initToolbarEventsHandlers();
        initShortcutHandlers();
        initPreventScrollingUnderRootElements();

        exports.updateToolbarButtons();

        currentMode = UI_MODES.recording;
    };

    exports.activatePlaybackMode = function () {
        if (currentMode === UI_MODES.initialized)
            createToolbar(UI_MODES.playback);

        initPreventScrollingUnderRootElements();

        currentMode = UI_MODES.playback;
    };

    exports.highlightStep = function (stepNum) {
        StepsPanel.highlightItem(stepNum);
    };


    exports.confirmAction = function (stepNum, stepInfo, callback) {
        onPopupDialogOpening(!stepInfo.isAssertion);

        openingDialog = true;

        var addActionDialog = new AddActionDialog(getRecorderContainer(), stepNum, stepInfo);
        activeDialog = addActionDialog;

        openingDialog = false;

        addActionDialog.on(AddActionDialog.ADD_ACTION_BUTTON_CLICK_EVENT, function (e) {
            activeDialog = null;
            onPopupDialogClosed();

            callback(true, e.stepInfo);
        });

        addActionDialog.on(AddActionDialog.CANCEL_BUTTON_CLICK_EVENT, function () {
            activeDialog = null;
            onPopupDialogClosed({
                cancelAction: true
            });

            callback(false);
        });

        addActionDialog.on(AddActionDialog.START_PICKING_ELEMENT_EVENT, function (e) {
            eventEmitter.emit(exports.START_PICKING_ELEMENT_EVENT, e);
        });
    };

    exports.stepAdded = function (stepInfo) {
        if (!stepInfo.isAssertion && !stepPanelExpanded)
            ToolbarWidget.addActionIndicator(stepInfo.actionDescriptor.type);

        StepsPanel.addItem();
    };

    exports.handlerAdded = function (dialogType) {
        if (!stepPanelExpanded)
            ToolbarWidget.addActionIndicator(dialogType);
    };

    exports.onTypingStateChanged = function (isTypingStarted, completeCallback, elementRect) {
        if (isTypingStarted) {
            elementRect = elementRect || Util.getElementRectangle(Util.getActiveElement());

            if (!$completeTypingButton) {
                if ($completeClickEditorButton) {
                    $completeClickEditorButton.remove();
                    $completeClickEditorButton = null;
                    currentCompleteClickEditorHandler = null;
                }
                $completeTypingButton = createCompleteTypingButton(elementRect);
            }
            else
                setCompleteTypingButtonPosition($completeTypingButton, elementRect);

            currentCompleteTypingHandler = function () {
                $completeTypingButton.remove();
                currentCompleteTypingHandler = null;
                completeCallback();
            };

            ShadowUI.bind($completeTypingButton, 'mousedown', function (e) {
                Util.preventDefault(e);
                currentCompleteTypingHandler();
            });
        }
        else if ($completeTypingButton) {
            $completeTypingButton.remove();
            $completeTypingButton = null;
        }
    };

    exports.onClickEditorStateChanged = function (isClickOnEditorStarted, completeCallback, elementRect) {
        if (isClickOnEditorStarted) {
            elementRect = elementRect || Util.getElementRectangle(Util.getActiveElement());

            if (!$completeClickEditorButton)
                $completeClickEditorButton = createCompleteClickEditorButton(elementRect);
            else
                setCompleteTypingButtonPosition($completeClickEditorButton, elementRect);

            currentCompleteClickEditorHandler = function () {
                $completeClickEditorButton.remove();
                $completeClickEditorButton = null;
                currentCompleteClickEditorHandler = null;
                completeCallback();
            };

            ShadowUI.bind($completeClickEditorButton, 'mousedown', function (e) {
                Util.preventDefault(e);
                currentCompleteClickEditorHandler();
            });
        }
        else if ($completeClickEditorButton) {
            $completeClickEditorButton.remove();
            $completeClickEditorButton = null;
        }
    };

    exports.onStartPickElement = function () {
        exports.hideToolbar();

        if (activeDialog) {
            activeDialog.popup.hide();
            ModalBackground.hide();
        }
        ElementPicker.setRecorderUI(getRecorderContainer());
    };

    exports.onStopPickElement = function () {
        if (activeDialog) {
            if (activeDialog.popup && activeDialog.popup.isHidden()) {
                activeDialog.popup.show();
                ModalBackground.show(activeDialog.step && !activeDialog.step.getStepInfo().isAssertion);
            }
        }
        else
            exports.showToolbar();
    };

    exports.hideToolbar = function () {
        ToolbarWidget.hide();
        StepsPanel.hide();
    };

    exports.showToolbar = function (skipPageStateSaving) {
        ToolbarWidget.show();

        if (stepPanelExpanded)
            StepsPanel.show(skipPageStateSaving);

        exports.updateToolbarButtons();
    };

    exports.setToolbarBlind = function (blind) {
        ToolbarWidget.setBlind(blind);
    };

    exports.updateToolbarButtons = function () {
        var hasErrors = false,
            steps = stepsInfoGetter();

        playbackActive = !!steps.length;

        for (var i = 0; i < steps.length; i++) {
            if (steps[i].error) {
                hasErrors = true;
                playbackActive = false;
                break;
            }
        }

        ToolbarWidget.updateState(hasChangesGetter(), !!steps.length, hasErrors);
    };

    exports.updateSteps = function () {
        StepsPanel.updateSteps();
    };

    exports.openStepWithError = openStepWithError;

    exports.openChangesWarningDialog = function (hasErrors) {
        onPopupDialogOpening();
        changesWarning(hasErrors);
    };
});
