TestCafeClient.define('UI.RecorderWidgets.ActionDialogsEditor', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,

        SharedErrors = require('Shared.Errors'),
        RecorderUtil = require('Recorder.Util'),
        DialogPropertyWidget = require('UI.RecorderWidgets.DialogProperty'),
        CheckboxWidget = require('UI.RecorderWidgets.Checkbox'),
        RadioGroupWidget = require('UI.RecorderWidgets.RadioGroup');

    //Const
    var EDITOR_BUTTON_TEXT = 'Browser dialogs',
        BROWSER_DIALOGS_ORDER = ['alert', 'confirm', 'prompt', 'beforeUnload'],
        BROWSER_DIALOG_INDEXES = RecorderUtil.BROWSER_DIALOG_INDEXES,
        PROMPT_DIALOG_NAME = 'prompt',
        CONFIRM_DIALOG_NAME = 'confirm',

        ACTION_OPTIONS_EDITOR_CLASS = 'action-dialogs-editor',
        EDITOR_BUTTON_CLASS = 'editor-button',
        EDITOR_BUTTON_TEXT_CLASS = 'editor-button-text',
        EDITOR_POPUP_CLASS = 'editor-popup',
        SEPARATOR_CLASS = 'separator',
        OPENED_CLASS = 'opened',
        DIALOG_OPTION_AREA = 'dialog-area',
        DIALOG_OPTION_EDITOR_AREA = 'editor-area',
        DIALOG_INDEX_DATA = 'dialog-index',
        CLEAR_CLASS = 'clear',
        FAILED_CLASS = 'failed',

        CONFIRM_RADIO_GROUP_TITLES = ['ok', 'cancel'],
        CONFIRM_VALUE = {
            ok: true,
            cancel: false
        };

    //ActionOptions
    var ActionDialogsEditor = this.exports = function ($container, stepInfo) {
        var dialogsEditor = this;

        this.stepInfo = stepInfo;
        this.dialogNameWithError = stepInfo.dialogError ? stepInfo.dialogError.dialog : null;

        this.$editor = null;
        this.$button = null;
        this.$popup = null;
        this.$dialogAreaWithError = null;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            dialogsEditor.eventEmitter.on(ev, listener);
        };

        this._createMarkup($container);
        this._init();
    };

    //Events
    ActionDialogsEditor.OPTION_CHANGED_EVENT = 'optionChangedEvent';

    //Utils
    function addClearElement($to) {
        ShadowUI.addClass($('<div></div>').appendTo($to), CLEAR_CLASS);
    }

    //Markup
    ActionDialogsEditor.prototype._createMarkup = function ($container) {
        this.$editor = $('<div></div>').appendTo($container);
        ShadowUI.addClass(this.$editor, ACTION_OPTIONS_EDITOR_CLASS);

        this.$button = $('<div></div>').appendTo(this.$editor);
        ShadowUI.addClass(this.$button, EDITOR_BUTTON_CLASS);

        var $buttonText = $('<div></div>').text(EDITOR_BUTTON_TEXT).appendTo(this.$button);
        ShadowUI.addClass($buttonText, EDITOR_BUTTON_TEXT_CLASS);

        if (this.dialogNameWithError && $.inArray(this.dialogNameWithError, BROWSER_DIALOGS_ORDER) !== -1)
            ShadowUI.addClass(this.$button, FAILED_CLASS);
    };

    ActionDialogsEditor.prototype._createSeparator = function () {
        if (this.$popup.children().length) {
            var $separator = $('<div></div>').appendTo(this.$popup);
            ShadowUI.addClass($separator, SEPARATOR_CLASS);
        }
    };

    ActionDialogsEditor.prototype._createUncheckedDialog = function (dialogName) {
        this._createSeparator();
        this._createDialogProperty(null, dialogName);
    };

    ActionDialogsEditor.prototype._createPopup = function () {
        this.$popup = $('<div></div>').appendTo(this.$editor);
        ShadowUI.addClass(this.$popup, EDITOR_POPUP_CLASS);

        for (var i = 0; i < BROWSER_DIALOGS_ORDER.length; i++)
            this._createDialogs(BROWSER_DIALOGS_ORDER[i]);

        this.$popup.css('left', this.$button.position().left - this.$popup.width() + this.$button[0].offsetWidth);
    };

    ActionDialogsEditor.prototype._createDialogs = function (dialogName) {
        var dialogsEditor = this,
            stepInfoHandlers = this.stepInfo.nativeDialogHandlers,
            dialogIndex = BROWSER_DIALOG_INDEXES[dialogName],
            handlers = stepInfoHandlers && stepInfoHandlers[dialogIndex] ? stepInfoHandlers[dialogIndex] : null;

        //NOTE: Create unchecked handler
        if (!handlers || !handlers.length) {
            this._createUncheckedDialog(dialogName);
            return;
        }

        var wasCreated = false; //NOTE: problem with disappeared dialog handler after unchecking and reopen dialog popup

        //NOTE: Create all checked handlers
        handlers.forEach(function (handler, index) {
            if (!handler)
                return;

            wasCreated = true;
            dialogsEditor._createSeparator();
            dialogsEditor._createDialogProperty(handler, dialogName, index);
        });

        if (!wasCreated) {
            this._createUncheckedDialog(dialogName);
            return;
        }

        //NOTE: if was some expected dialog and one unexpected dialog we should create unchecked handler to give an opportunity to correct dialog error
        if (this.dialogNameWithError && this.dialogNameWithError.toLowerCase() === dialogName.toLowerCase() && this.stepInfo.dialogError.code === SharedErrors.API_UNEXPECTED_DIALOG) {
            dialogsEditor._createSeparator();
            dialogsEditor._createDialogProperty(null, dialogName, handlers.length);
        }
    };

    ActionDialogsEditor.prototype._createDialogProperty = function (handler, dialogName, index) {
        var dialogsEditor = this,
            $dialogArea = null;

        if (dialogName === PROMPT_DIALOG_NAME)
            $dialogArea = this._createPromptDialogProperty(handler);
        else if (dialogName === CONFIRM_DIALOG_NAME)
            $dialogArea = this._createConfirmDialogProperty(handler);
        else {
            var checkbox = new CheckboxWidget(this.$popup, !!handler, dialogName);
            $dialogArea = checkbox.$checkboxContainer;

            checkbox.on(CheckboxWidget.STATE_CHANGED_EVENT, function (e) {
                if (e.state)
                    dialogsEditor._createDialogHandler($dialogArea.data(DIALOG_INDEX_DATA), dialogName);
                else
                    dialogsEditor._deleteDialogHandler($dialogArea.data(DIALOG_INDEX_DATA), dialogName);

                dialogsEditor.eventEmitter.emit(ActionDialogsEditor.OPTION_CHANGED_EVENT, {dialog: dialogName, state: e.state});
            });
        }

        if (typeof index !== 'undefined')
            $dialogArea.data(DIALOG_INDEX_DATA, index);

        if (this.dialogNameWithError && this.dialogNameWithError.toLowerCase() === dialogName.toLowerCase() && !this.$dialogAreaWithError && !(handler && this.stepInfo.dialogError.code === SharedErrors.API_UNEXPECTED_DIALOG)) {
            ShadowUI.addClass($dialogArea, FAILED_CLASS);
            this.$dialogAreaWithError = $dialogArea;
        }
    };

    ActionDialogsEditor.prototype._createPromptDialogProperty = function (handler) {
        var dialogsEditor = this,
            $dialogArea = $('<div></div>').appendTo(this.$popup),
            checkbox = null,
            $dialogEditorArea = null,
            editor = null;

        ShadowUI.addClass($dialogArea, DIALOG_OPTION_AREA);
        checkbox = new CheckboxWidget($dialogArea, !!handler, PROMPT_DIALOG_NAME);

        checkbox.on(CheckboxWidget.STATE_CHANGED_EVENT, function (e) {
            if (e.state) {
                editor.enable();

                if (editor.isValid())
                    dialogsEditor._createDialogHandler($dialogArea.data(DIALOG_INDEX_DATA), PROMPT_DIALOG_NAME, editor.getEditorText() === null ? null : editor.getEditorText());
            }
            else {
                editor.disable();
                dialogsEditor._deleteDialogHandler($dialogArea.data(DIALOG_INDEX_DATA), PROMPT_DIALOG_NAME);
            }

            dialogsEditor.eventEmitter.emit(ActionDialogsEditor.OPTION_CHANGED_EVENT, {dialog: PROMPT_DIALOG_NAME, state: e.state});
        });

        $dialogEditorArea = $('<div></div>').appendTo($dialogArea);
        ShadowUI.addClass($dialogEditorArea, DIALOG_OPTION_EDITOR_AREA);

        if (!handler) {
            editor = new DialogPropertyWidget($dialogEditorArea, '', '');
            editor.disable();
        }
        else
            editor = new DialogPropertyWidget($dialogEditorArea, '', handler.retValue === null ? 'null' : handler.retValue);

        editor.on(DialogPropertyWidget.VALUE_CHANGED_EVENT, function (e) {
            var dialogIndex = BROWSER_DIALOG_INDEXES[PROMPT_DIALOG_NAME],
                index = $dialogArea.data(DIALOG_INDEX_DATA);

            if (editor.isValid()) {
                dialogsEditor.stepInfo.nativeDialogHandlers[dialogIndex][typeof index === 'undefined' ? 0 : index] = {
                    dialog: PROMPT_DIALOG_NAME,
                    retValue: e.value === 'null' ? null : e.value
                };

                dialogsEditor.eventEmitter.emit(ActionDialogsEditor.OPTION_CHANGED_EVENT, {});
            }
        });

        addClearElement($dialogArea);

        return $dialogArea;
    };

    ActionDialogsEditor.prototype._createConfirmDialogProperty = function (handler) {
        var dialogsEditor = this,
            $dialogArea = $('<div></div>').appendTo(this.$popup),
            checkbox = null,
            $dialogEditorArea = null,
            editor = null;

        ShadowUI.addClass($dialogArea, DIALOG_OPTION_AREA);
        checkbox = new CheckboxWidget($dialogArea, !!handler, CONFIRM_DIALOG_NAME);

        checkbox.on(CheckboxWidget.STATE_CHANGED_EVENT, function (e) {
            if (e.state) {
                editor.enable();
                dialogsEditor._createDialogHandler($dialogArea.data(DIALOG_INDEX_DATA), CONFIRM_DIALOG_NAME, CONFIRM_VALUE[CONFIRM_RADIO_GROUP_TITLES[editor.checkedIndex]]);
            }
            else {
                editor.disable();
                dialogsEditor._deleteDialogHandler($dialogArea.data(DIALOG_INDEX_DATA), CONFIRM_DIALOG_NAME);
            }

            dialogsEditor.eventEmitter.emit(ActionDialogsEditor.OPTION_CHANGED_EVENT, {dialog: CONFIRM_DIALOG_NAME, state: e.state});
        });

        $dialogEditorArea = $('<div></div>').appendTo($dialogArea);
        ShadowUI.addClass($dialogEditorArea, DIALOG_OPTION_EDITOR_AREA);

        editor = new RadioGroupWidget($dialogEditorArea, CONFIRM_RADIO_GROUP_TITLES, !handler || handler.retValue === true ? 0 : 1);

        if (!handler)
            editor.disable();

        editor.on(RadioGroupWidget.CHECKED_INDEX_CHANGED, function (e) {
            var dialogIndex = BROWSER_DIALOG_INDEXES[CONFIRM_DIALOG_NAME],
                index = $dialogArea.data(DIALOG_INDEX_DATA);

            dialogsEditor.stepInfo.nativeDialogHandlers[dialogIndex][typeof index === 'undefined' ? 0 : index] = {
                dialog: CONFIRM_DIALOG_NAME,
                retValue: CONFIRM_VALUE[CONFIRM_RADIO_GROUP_TITLES[e.checkedIndex]]
            };

            dialogsEditor.eventEmitter.emit(ActionDialogsEditor.OPTION_CHANGED_EVENT, {});
        });

        addClearElement($dialogArea);

        return $dialogArea;
    };

    //Behavior
    ActionDialogsEditor.prototype._init = function () {
        var dialogsEditor = this;

        this.$button.mousedown(function () {
            if (!dialogsEditor.$popup || dialogsEditor.$popup.css('visibility') === 'hidden')
                dialogsEditor.showPopup();
            else
                dialogsEditor.hidePopup();
        });
    };

    ActionDialogsEditor.prototype._createDialogHandler = function (index, dialogName, retValue) {
        var dialogIndex = BROWSER_DIALOG_INDEXES[dialogName],
            dialogHandler = {
                dialog: dialogName,
                retValue: retValue
            };

        if (!this.stepInfo.nativeDialogHandlers)
            this.stepInfo.nativeDialogHandlers = [];

        if (!this.stepInfo.nativeDialogHandlers[dialogIndex])
            this.stepInfo.nativeDialogHandlers[dialogIndex] = [];

        this.stepInfo.nativeDialogHandlers[dialogIndex][typeof index === 'undefined' ? 0 : index] = dialogHandler;
    };

    ActionDialogsEditor.prototype._deleteDialogHandler = function (index, dialogName) {
        var dialogIndex = BROWSER_DIALOG_INDEXES[dialogName];

        if (this.stepInfo.nativeDialogHandlers[dialogIndex][index]) {
            delete this.stepInfo.nativeDialogHandlers[dialogIndex][index];

            if (!this.stepInfo.nativeDialogHandlers[dialogIndex].length)
                delete this.stepInfo.nativeDialogHandlers[dialogIndex];
        }
        else if (this.stepInfo.nativeDialogHandlers[dialogIndex])
            delete this.stepInfo.nativeDialogHandlers[dialogIndex];

        if (!this.stepInfo.nativeDialogHandlers.length)
            delete this.stepInfo.nativeDialogHandlers;
        else {
            var isEmpty = true;

            $.each(this.stepInfo.nativeDialogHandlers, function (index, handlers) {
                if (handlers && handlers.length) {
                    isEmpty = false;
                    return false;
                }
            });

            if (isEmpty)
                delete this.stepInfo.nativeDialogHandlers;
        }
    };

    ActionDialogsEditor.prototype.removeError = function () {
        this.dialogNameWithError = null;

        if (this.$dialogAreaWithError) {
            ShadowUI.removeClass(this.$dialogAreaWithError, FAILED_CLASS);
            ShadowUI.removeClass(this.$button, FAILED_CLASS);
        }

        this.$dialogAreaWithError = null;
    };

    ActionDialogsEditor.prototype.showPopup = function () {
        var dialogsEditor = this;

        if (!this.$popup)
            this._createPopup();
        else
            this.$popup.css('visibility', '');

        ShadowUI.addClass(this.$button, OPENED_CLASS);

        ShadowUI.bind($(document), 'mousedown', function (e) {
            if (!dialogsEditor.$editor.find(e.target).length) {
                ShadowUI.unbind($(document), 'mousedown', arguments.callee);
                dialogsEditor.hidePopup();
            }
        });
    };

    ActionDialogsEditor.prototype.hidePopup = function () {
        if (this.$popup) {
            this.$popup.css('visibility', 'hidden');
            ShadowUI.removeClass(this.$button, OPENED_CLASS);
        }
    };
});