TestCafeClient.define('UI.RecorderWidgets.ActionOptionsEditor', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,

        DialogPropertyWidget = require('UI.RecorderWidgets.DialogProperty'),
        CheckboxWidget = require('UI.RecorderWidgets.Checkbox');

    //Const
    var API_ARGUMENTS_OPTIONS_ORDER = ['ctrl', 'alt', 'shift', 'meta', 'caretPos', 'offsetX', 'offsetY'],
        OFFSET_X_PROPERTY_NAME = 'offsetX',
        OFFSET_Y_PROPERTY_NAME = 'offsetY',
        OFFSET_X_LABEL_TEXT = 'X:',
        OFFSET_Y_LABEL_TEXT = 'Y:',

        OFFSET_OPTIONS_CHECKBOX_TITLE = 'Use offsets',

        ACTION_OPTIONS_EDITOR_CLASS = 'action-options-editor',
        EDITOR_BUTTON_CLASS = 'editor-button',
        EDITOR_POPUP_CLASS = 'editor-popup',
        SEPARATOR_CLASS = 'separator',
        OPENED_CLASS = 'opened',
        OFFSETS_AREA_CLASS = 'offsets-area',
        OFFSET_EDITORS_AREA_CLASS = 'offset-editors-area';

    //ActionOptions
    var ActionOptionsEditor = this.exports = function ($container, stepInfo) {
        var editor = this;

        this.stepInfo = stepInfo;
        this.$editor = null;
        this.$button = null;
        this.$popup = null;

        this.propertyEditors = [];
        this.offsetPropertyEditors = [];

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            editor.eventEmitter.on(ev, listener);
        };

        this._createMarkup($container);

        this._init();
    };

    //Events
    ActionOptionsEditor.OPTION_CHANGED_EVENT = 'optionChangedEvent';

    //Markup
    ActionOptionsEditor.prototype._createMarkup = function ($container) {
        this.$editor = $('<div></div>').appendTo($container);
        ShadowUI.addClass(this.$editor, ACTION_OPTIONS_EDITOR_CLASS);

        this.$button = $('<div></div>').appendTo(this.$editor);
        ShadowUI.addClass(this.$button, EDITOR_BUTTON_CLASS);

        var $buttonText = $('<div></div>').text('Options').appendTo(this.$button);
        ShadowUI.addClass($buttonText, 'editor-button-text');
    };

    ActionOptionsEditor.prototype._createPopup = function () {
        var editor = this,
            offsetAreaCreated = false;

        this.$popup = $('<div></div>').appendTo(this.$editor);
        ShadowUI.addClass(this.$popup, EDITOR_POPUP_CLASS);

        for (var i = 0; i < API_ARGUMENTS_OPTIONS_ORDER.length; i++) {
            var optionName = API_ARGUMENTS_OPTIONS_ORDER[i];
            var isOffsetOption = optionName === OFFSET_X_PROPERTY_NAME || optionName === OFFSET_Y_PROPERTY_NAME;
            if (this.stepInfo.actionDescriptor.apiArguments.options.hasOwnProperty(optionName)) {
                if (isOffsetOption && offsetAreaCreated)
                    continue;

                if (this.$popup.children().length) {
                    var $separator = $('<div></div>').appendTo(this.$popup);
                    ShadowUI.addClass($separator, SEPARATOR_CLASS);
                }

                if (isOffsetOption) {
                    this._createOffsetsArea();
                    offsetAreaCreated = true;
                }
                else
                    this._createOption(optionName, optionName, this.$popup);
            }
        }

        editor._updateOffsetOptionsEditability();

        this.$popup.css('left', this.$button.position().left - this.$popup.width() + this.$button[0].offsetWidth);
    };

    ActionOptionsEditor.prototype._createOffsetsArea = function () {
        var editor = this;

        var $offsetsArea = $('<div></div>').appendTo(this.$popup);
        ShadowUI.addClass($offsetsArea, OFFSETS_AREA_CLASS);

        var checkBox = new CheckboxWidget($offsetsArea, this.stepInfo.useOffsets, OFFSET_OPTIONS_CHECKBOX_TITLE);

        checkBox.on(CheckboxWidget.STATE_CHANGED_EVENT, function (e) {
            editor.stepInfo.useOffsets = e.state;
            editor._updateOffsetOptionsEditability();
            editor.eventEmitter.emit(ActionOptionsEditor.OPTION_CHANGED_EVENT, {});
        });

        var $offsetEditorsArea = $('<div></div>').appendTo($offsetsArea);
        ShadowUI.addClass($offsetEditorsArea, OFFSET_EDITORS_AREA_CLASS);

        this.offsetPropertyEditors.push(this._createOption(OFFSET_X_PROPERTY_NAME, OFFSET_X_LABEL_TEXT, $offsetEditorsArea));
        this.offsetPropertyEditors.push(this._createOption(OFFSET_Y_PROPERTY_NAME, OFFSET_Y_LABEL_TEXT, $offsetEditorsArea));
    };

    ActionOptionsEditor.prototype._createOption = function (name, labelText, $container) {
        var editor = this;

        var options = this.stepInfo.actionDescriptor.apiArguments.options;

        if (typeof options[name] === 'boolean') {
            var checkBox = new CheckboxWidget($container, options[name], labelText);

            checkBox.on(CheckboxWidget.STATE_CHANGED_EVENT, function (e) {
                options[name] = e.state;
                editor.eventEmitter.emit(ActionOptionsEditor.OPTION_CHANGED_EVENT, {});
            });

            return checkBox;
        }
        else {
            var property = new DialogPropertyWidget($container, labelText, options[name], {
                isNumericProperty: true,
                isNotNegativeNumericProperty: true
            });

            this.propertyEditors.push(property);

            property.on(DialogPropertyWidget.VALUE_CHANGED_EVENT, function (e) {
                if (property.isValid())
                    options[name] = e.value;
                editor.eventEmitter.emit(ActionOptionsEditor.OPTION_CHANGED_EVENT, {});
            });

            return property;
        }
    };

    ActionOptionsEditor.prototype._updateOffsetOptionsEditability = function () {
        for (var i = 0; i < this.offsetPropertyEditors.length; i++) {
            if (this.stepInfo.useOffsets && this.offsetPropertyEditors[i].isDisabled)
                this.offsetPropertyEditors[i].enable();
            else if (!this.stepInfo.useOffsets && !this.offsetPropertyEditors[i].isDisabled)
                this.offsetPropertyEditors[i].disable();
        }
    };

    //Behavior
    ActionOptionsEditor.prototype._init = function () {
        var editor = this;

        this.$button.mousedown(function () {
            if (!editor._isPopupVisible())
                editor._showPopup();
            else
                editor._hidePopup();
        });
    };

    ActionOptionsEditor.prototype._showPopup = function () {
        var editor = this;

        if (!this.$popup)
            this._createPopup();
        else
            this.$popup.css('visibility', '');

        ShadowUI.addClass(this.$button, OPENED_CLASS);

        ShadowUI.bind($(document), 'mousedown', function (e) {
            if (!editor.$editor.find(e.target).length) {
                ShadowUI.unbind($(document), 'mousedown', arguments.callee);
                editor._hidePopup();
            }
        });
    };

    ActionOptionsEditor.prototype._hidePopup = function () {
        if (this.$popup) {
            this.$popup.css('visibility', 'hidden');
            ShadowUI.removeClass(this.$button, OPENED_CLASS);
        }
    };

    ActionOptionsEditor.prototype._isPopupVisible = function () {
        return this.$popup && this.$popup.css('visibility') !== 'hidden';
    };

    ActionOptionsEditor.prototype.isValid = function () {
        for (var i = 0; i < this.propertyEditors.length; i++)
            if (!this.propertyEditors[i].isValid())
                return false;
        return true;
    };
});