TestCafeClient.define('UI.RecorderWidgets.AddActionDialog', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,

        PopupWidget = require('UI.RecorderWidgets.Popup'),
        ButtonWidget = require('UI.RecorderWidgets.Button'),
        BaseStepWidget = require('UI.RecorderWidgets.BaseStep'),
        ActionStepWidget = require('UI.RecorderWidgets.ActionStep'),
        AssertionsStepWidget = require('UI.RecorderWidgets.AssertionsStep');

    //Const
    var BUTTONS_CLASS = 'buttons',
        POPUP_HEADER_NUMBER_CLASS = 'number',
        ADD_ACTION_DIALOG_WIDTH = 550,
        ADD_ASSERTION_DIALOG_WIDTH = 606;

    var AddActionDialog = this.exports = function ($container, stepNum, stepInfo) {
        var dialog = this;

        this.popup = null;
        this.step = null;

        this.$addActionButton = null;
        this.$cancelButton = null;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            dialog.eventEmitter.on(ev, listener);
        };

        this._createDialog($container, stepNum, stepInfo);

        this._init();
    };

    //Events
    AddActionDialog.ADD_ACTION_BUTTON_CLICK_EVENT = 'addActionButtonClick';
    AddActionDialog.CANCEL_BUTTON_CLICK_EVENT = 'cancelButtonClick';
    AddActionDialog.START_PICKING_ELEMENT_EVENT = 'startPickingElement';

    //Markup
    AddActionDialog.prototype._createDialog = function ($container, stepNum, stepInfo) {
        var $stepArea = $('<div></div>');

        var popupOptions = {
            width: stepInfo.isAssertion ? ADD_ASSERTION_DIALOG_WIDTH : ADD_ACTION_DIALOG_WIDTH,
            headerIconText: stepNum,
            headerIconClass: POPUP_HEADER_NUMBER_CLASS,
            headerText: stepInfo.name,
            content: $stepArea,
            footerContent: this._createButtons(stepInfo),
            backgroundOpacity: true,
            showAtWindowCenter: true,
            footerWithoutTopMargins: stepInfo.isAssertion
        };

        this.popup = new PopupWidget($container, popupOptions);

        if (stepInfo.isAssertion) {
            this.step = new AssertionsStepWidget($stepArea, stepNum, stepInfo, {
                enableAssertionsValidation: true,
                parentPopup: this.popup
            });
            this.popup.showAtWindowCenter();
        }
        else {
            this.step = new ActionStepWidget($stepArea, stepNum, stepInfo, {
                enableFloatMode: true,
                $floatingParent: this.popup.getContainer(),
                enableSelectorValidation: true,
                enableElementsMarking: true
            });

            var parsedSelector = this.step.getParsedSelector();

            if (parsedSelector && parsedSelector.length && parsedSelector.$elements)
                this.popup.disposeRelativeToElement(parsedSelector.$elements);
        }
    };

    AddActionDialog.prototype._createButtons = function () {
        var $buttons = $('<div></div>');
        ShadowUI.addClass($buttons, BUTTONS_CLASS);

        this.$addActionButton = ButtonWidget.create($buttons, 'OK');
        this.$cancelButton = ButtonWidget.create($buttons, 'Cancel');

        return $buttons;
    };

    AddActionDialog.prototype._onError = function () {
        this.$addActionButton.attr('disabled', 'disabled');
    };

    AddActionDialog.prototype._onSuccess = function () {
        this.$addActionButton.removeAttr('disabled');
    };

    //Behavior
    AddActionDialog.prototype._init = function () {
        var dialog = this;

        ShadowUI.bind(this.$addActionButton, 'click', function () {
            dialog._close(function () {
                dialog.eventEmitter.emit(AddActionDialog.ADD_ACTION_BUTTON_CLICK_EVENT, {
                    stepInfo: dialog.step.getStepInfo()
                });
            });
        });

        ShadowUI.bind(this.$cancelButton, 'click', function () {
            dialog._close(function () {
                dialog.eventEmitter.emit(AddActionDialog.CANCEL_BUTTON_CLICK_EVENT, {});
            });
        });

        this.popup.onkeydown(function (e) {
            if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.enter) {
                if (dialog.$addActionButton && dialog.$addActionButton.css('visibility') !== 'hidden')
                    dialog.$addActionButton.trigger('click');
                Util.preventDefault(e);
            }

            if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.esc) {
                dialog.$cancelButton.trigger('click');
                Util.preventDefault(e);
            }

            //NOTE: we should prevent page scrolling
            if (e.target === dialog.popup.$popup[0] && Util.isArrowKey(e.keyCode))
                Util.preventDefault(e);
        }, true);

        this.step.on(BaseStepWidget.STEP_INFO_CHANGED_EVENT, function () {
            if (!dialog.step.isValid())
                dialog._onError();
            else
                dialog._onSuccess();
        });

        this.step.on(BaseStepWidget.STEP_NAME_CHANGED_EVENT, function (e) {
            dialog.popup.setHeaderText(e.name);
        });

        if (this.step.getStepInfo().isAssertion) {
            this.step.on(AssertionsStepWidget.START_PICKING_ELEMENT_EVENT, function (e) {
                dialog.eventEmitter.emit(AddActionDialog.START_PICKING_ELEMENT_EVENT, e);
            });
        }
        else {
            this.step.on(ActionStepWidget.SELECTOR_EDITOR_FOCUSED_EVENT, function () {
                dialog.popup.$header.css('cursor', 'default');
                dialog.popup.blind(true);
            });

            this.step.on(ActionStepWidget.SELECTOR_EDITOR_BLURED_EVENT, function () {
                dialog.popup.blind(false);
                dialog.popup.$header.css('cursor', 'move');
                dialog.popup.getContainer().focus();
            });
        }

        if (!dialog.step.isValid())
            dialog._onError();
    };

    AddActionDialog.prototype._close = function (callback) {
        this.step.destroy();
        this.popup.close(callback);
    };
});