TestCafeClient.define('UI.RecorderWidgets.AddAssertionButton', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        ButtonWidget = require('UI.RecorderWidgets.Button'),

        OPERATORS = ['eq', 'notEq', 'ok', 'notOk'],
        OPERATOR_ICON_CLASSES = {
            eq: 'eq-icon',
            notEq: 'not-eq-icon',
            ok: 'ok-icon',
            notOk: 'not-ok-icon'
        },

        OPERATOR_ICON_CLASS = 'operator-icon',
        ADD_ASSERTION_BUTTON_CONTAINER_CLASS = 'add-assertion-button-container',
        ADD_ASSERTION_BUTTON_CLASS = 'add-assertion-button',
        ADD_ASSERTION_BUTTON_POPUP_CLASS = 'add-assertion-button-popup',
        ADD_ASSERTION_BUTTON_POPUP_ITEM_CLASS = 'add-assertion-button-popup-item',
        ADD_ASSERTION_BUTTON_POPUP_SEPARATOR_CLASS = 'add-assertion-button-popup-separator';

    function addEventEmitter(target) {
        target.eventEmitter = new Util.EventEmitter();
        target.on = function (ev, listener) {
            target.eventEmitter.on(ev, listener);
        };
    }

    var AddAssertionButton = this.exports = function ($container) {
        var button = this;
        this.$container = null;
        this.$button = null;
        this.$popup = null;

        addEventEmitter(this);

        this.$container = $('<div></div>').appendTo($container);
        ShadowUI.addClass(this.$container, ADD_ASSERTION_BUTTON_CONTAINER_CLASS);

        this.$button = ButtonWidget.create(this.$container, '', true);
        ShadowUI.addClass(this.$button, ADD_ASSERTION_BUTTON_CLASS);

        this.$button.click(function () {
            if (!button.$popup)
                button._showPopup();
            else
                button._closePopup();
        });
    };

    AddAssertionButton.prototype.getContainer = function () {
        return this.$container;
    };

    AddAssertionButton.ITEM_CLICKED_EVENT = 'itemClicked';
    AddAssertionButton.POPUP_CLOSED_EVENT = 'popupClosed';

    AddAssertionButton.prototype._showPopup = function () {
        var button = this;

        this.$popup = $('<div></div>');
        ShadowUI.addClass(this.$popup, ADD_ASSERTION_BUTTON_POPUP_CLASS);

        for (var i = 0; i < OPERATORS.length; i++) {
            this._addPopupItem(OPERATORS[i]);

            if (i < OPERATORS.length - 1) {
                var $separator = $('<div></div>').appendTo(this.$popup);
                ShadowUI.addClass($separator, ADD_ASSERTION_BUTTON_POPUP_SEPARATOR_CLASS);
            }
        }

        this.$popup.appendTo(this.$container);

        var popupHeight = this.$popup.height();

        if(this.$popup.offset().top - $(document).scrollTop() + popupHeight > $(window).height())
            this.$popup.css('top', parseInt(this.$popup.css('top').replace('px', '')) - popupHeight - this.$button.height());

        var documentMouseDownHandler = function (e) {
            if (!button.$container.find(e.target).length && button.$button[0] !== e.target)
                button._closePopup();
        };

        ShadowUI.bind($(document), 'mousedown', documentMouseDownHandler);
        this.on(AddAssertionButton.POPUP_CLOSED_EVENT, function () {
            ShadowUI.unbind($(document), 'mousedown', documentMouseDownHandler);
        });
    };

    AddAssertionButton.prototype._closePopup = function () {
        if (this.$popup) {
            var popupHeight = this.$popup.outerHeight(true);
            this.$popup.remove();
            this.$popup = null;
            this.eventEmitter.emit(AddAssertionButton.POPUP_CLOSED_EVENT, { popupHeight: popupHeight });
        }
    };

    AddAssertionButton.prototype._addPopupItem = function (text) {
        var button = this,
            $item = $('<div></div>').appendTo(this.$popup),
            $operatorIcon = $('<div></div>').appendTo($item);

        ShadowUI.addClass($item, ADD_ASSERTION_BUTTON_POPUP_ITEM_CLASS);
        ShadowUI.addClass($operatorIcon, OPERATOR_ICON_CLASS);
        ShadowUI.addClass($operatorIcon, OPERATOR_ICON_CLASSES[text]);

        $item.click(function (e) {
            button._closePopup();
            button.eventEmitter.emit(AddAssertionButton.ITEM_CLICKED_EVENT, { operator: text });
            Util.preventDefault(e);
            return false;
        });
    };
});