TestCafeClient.define('UI.RecorderWidgets.AssertionArgument', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        CodeEditor = require('UI.RecorderWidgets.CodeEditor'),
        ChooseSelectorDialog = require('UI.RecorderWidgets.ChooseSelectorDialog'),
        ChoosePropertyDialog = require('UI.RecorderWidgets.ChoosePropertyDialog'),
        ValidationMessageFactory = require('UI.ValidationMessageFactory'),
        JavascriptExecutor = require('Base.JavascriptExecutor'),
        ModalBackground = require('UI.ModalBackground'),
        ObjectViewer = require('UI.RecorderWidgets.ObjectViewer'),
        ButtonWidget = require('UI.RecorderWidgets.Button'),

        UXLog = require('UI.UXLog'),

        CODE_EDITOR_WIDTH = 364,
        ARGUMENT_HEIGHT = 119,
        CODE_EDITOR_HEIGHT = 116,

        VALUE_LABEL_POSTFIX = ' Value:',
        OPEN_WIZARD_BUTTON_TITLE = 'Open assertion argument wizard',

        ARGUMENT_CLASS = 'argument',
        ARGUMENT_LABEL_CLASS = 'argument-label',
        VALUE_LABEL_CLASS = 'value-label',
        ARGUMENT_VALUE_CLASS = 'argument-value';

    //Argument
    var ArgumentEditor = this.exports = function ($container, options) {
        options = options || {};
        var argumentEditor = this;

        this.value = options.value || '';
        this.error = null;
        this.editor = null;
        this.objectViewer = null;
        this.iFrameContext = options.iFrameContext || null;
        this.iFrameSelector = options.iFrameSelector || null;
        this.enableValidation = options.enableValidation || false;
        this.parentPopup = options.parentPopup;
        this.savedIFrameContext = null;
        this.lastParsedValue = null;
        this.parsingInProcess = false;
        this.changeDelayed = false;
        this.isReproducibleValue = false;

        this.$argument = null;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            argumentEditor.eventEmitter.on(ev, listener);
        };

        var $argumentLabel = this._createLabel(ARGUMENT_LABEL_CLASS, options.argumentName + ':');
        $argumentLabel.appendTo($container);

        this.$argument = $('<div></div>').css('height', ARGUMENT_HEIGHT).appendTo($container);
        ShadowUI.addClass(this.$argument, ARGUMENT_CLASS);

        this._createInput();
        this._createOpenWizardButton();

        var $valueLabel = this._createLabel(VALUE_LABEL_CLASS, options.argumentName + VALUE_LABEL_POSTFIX),
            $argumentValue = $('<div></div>');

        ShadowUI.addClass($argumentValue, ARGUMENT_VALUE_CLASS);

        if (this.enableValidation) {
            $valueLabel.appendTo($container);
            $argumentValue.appendTo($container);
        }

        this.objectViewer = new ObjectViewer($argumentValue);

        this._update();
    };

    //Argument events
    ArgumentEditor.ARGUMENT_CHANGED_EVENT = 'argumentChanged';
    ArgumentEditor.START_PICKING_ELEMENT_EVENT = 'startPickingElement';
    ArgumentEditor.ARGUMENT_REPRODUCIBLE_STATE_UPDATED = 'argumentReproducibleStateUpdated';

    //Argument private methods
    //Markup
    ArgumentEditor.prototype._createInput = function () {
        var argumentEditor = this;

        var $codeEditorContainer = $('<div></div>').appendTo(this.$argument);

        this.editor = new CodeEditor($codeEditorContainer, {
            width: CODE_EDITOR_WIDTH,
            height: CODE_EDITOR_HEIGHT,
            fixedHeight: true,
            text: this.value,
            allowEdit: true,
            floatingWidth: false
        });

        this.editor.events.on(CodeEditor.CHANGE_EVENT, function (e) {
            argumentEditor._setArgumentValue(e.text);
        });
    };

    ArgumentEditor.prototype._createLabel = function (className, labelText) {
        var $label = $('<label></label>').text(labelText);
        ShadowUI.addClass($label, className);

        return $label;
    };

    ArgumentEditor.prototype._createOpenWizardButton = function () {
        var argumentEditor = this,
            lastEditorValue = null;

        function startPicking() {
            UXLog.write('Assertions: start assertion wizard');
            lastEditorValue = argumentEditor.editor.getText();

            if (argumentEditor.parentPopup)
                argumentEditor.parentPopup.hide();

            ModalBackground.hide();

            argumentEditor.eventEmitter.emit(ArgumentEditor.START_PICKING_ELEMENT_EVENT, {
                callback: function (elementSelectors, iFrameSelectors) {
                    if (!elementSelectors) {
                        ModalBackground.show();

                        if (argumentEditor.parentPopup)
                            argumentEditor.parentPopup.show();
                    }
                    else {
                        var $iFrame = (iFrameSelectors && iFrameSelectors.length) ? JavascriptExecutor.parseSelectorSync(iFrameSelectors[0].selector).$elements : null;

                        argumentEditor.savedIFrameContext = argumentEditor.iFrameContext;
                        argumentEditor.iFrameContext = ($iFrame && $iFrame.length) ? $iFrame[0].contentWindow : null;
                        argumentEditor.iFrameSelector = ($iFrame && $iFrame.length) ? iFrameSelectors[0].selector : null;

                        showChooseSelectorDialog(elementSelectors, 0, $iFrame);

                    }
                },

                iFrameContext: argumentEditor.iFrameContext
            });
        }

        function showChooseSelectorDialog(elementSelectors, elementSelectorsIndex, $iFrame, popupPosition) {
            ModalBackground.show(true);

            var chooseSelectorDialog = new ChooseSelectorDialog(elementSelectors, elementSelectorsIndex, $iFrame, popupPosition);

            chooseSelectorDialog.on(ChooseSelectorDialog.CONFIRM_BUTTON_CLICK_EVENT, function (e) {
                showChoosePropertyDialog({
                    parsedElementSelector: e.parsedElementSelector,
                    elementSelectors: e.elementSelectors,
                    elementSelectorsIndex: e.elementSelectorsIndex,
                    popupPosition: e.popupPosition,
                    iFrameContext: argumentEditor.iFrameContext,
                    $iFrame: $iFrame
                });
            });

            chooseSelectorDialog.on(ChooseSelectorDialog.CLOSE_BUTTON_CLICK_EVENT, function () {
                argumentEditor.iFrameContext = this.savedIFrameContext;

                if (argumentEditor.parentPopup)
                    argumentEditor.parentPopup.show();
                ModalBackground.show();
            });
        }

        function showChoosePropertyDialog(options) {
            ModalBackground.show();

            var choosePropertyDialog = new ChoosePropertyDialog(options.parsedElementSelector, options.popupPosition, options.iFrameContext);

            choosePropertyDialog.on(ChoosePropertyDialog.CONFIRM_BUTTON_CLICK_EVENT, function (e) {
                UXLog.write('Assertions: assertion argument created by wizard');
                if (argumentEditor.parentPopup)
                    argumentEditor.parentPopup.show();

                argumentEditor.editor.focus();
                argumentEditor.editor.setText(e.text);

                if (lastEditorValue === e.text)
                    argumentEditor._onChange();  //raise change event
            });
            choosePropertyDialog.on(ChoosePropertyDialog.CLOSE_BUTTON_CLICK_EVENT, function () {
                argumentEditor.iFrameContext = this.savedIFrameContext;

                if (argumentEditor.parentPopup)
                    argumentEditor.parentPopup.show();
            });

            choosePropertyDialog.on(ChoosePropertyDialog.PREVIOUS_BUTTON_CLICK_EVENT, function (e) {
                showChooseSelectorDialog(options.elementSelectors, options.elementSelectorsIndex, options.$iFrame, e.popupPosition);
            });
        }

        var $openWizardButton = ButtonWidget.create(this.$argument, '', true);
        $openWizardButton.attr('title', OPEN_WIZARD_BUTTON_TITLE);

        $openWizardButton.click(startPicking);
    };

    //Behaior
    ArgumentEditor.prototype._setArgumentValue = function (value) {
        this.value = value;
        this._onChange();
    };

    ArgumentEditor.prototype._onChange = function () {
        this._update();

        if (this.parsingInProcess)
            this.changeDelayed = true;
        else
            this._emitChangeEvent();
    };

    ArgumentEditor.prototype._emitChangeEvent = function () {
        var argumentEditor = this;

        this.eventEmitter.emit(ArgumentEditor.ARGUMENT_CHANGED_EVENT, {
            value: argumentEditor.value,
            iFrameContext: argumentEditor.iFrameContext,
            iFrameSelector: argumentEditor.iFrameSelector
        });
    };

    ArgumentEditor.prototype._update = function () {
        var argumentEditor = this;

        this._resetError();

        this.objectViewer.clear();

        if (this.value) {
            this.parsingInProcess = true;
            this.objectViewer.show(this.value, this.iFrameContext, function (e) {
                argumentEditor.isReproducibleValue = e.isReproducibleValue;
                argumentEditor.eventEmitter.emit(ArgumentEditor.ARGUMENT_REPRODUCIBLE_STATE_UPDATED);

                if (e.error)
                    argumentEditor._onError(e.error);
                else
                    argumentEditor.lastParsedValue = e.parsedValue;

                if (argumentEditor.changeDelayed) {
                    argumentEditor._emitChangeEvent();
                    argumentEditor.changeDelayed = false;
                }

                argumentEditor.parsingInProcess = false;
            });
        }
        else
            this.eventEmitter.emit(ArgumentEditor.ARGUMENT_REPRODUCIBLE_STATE_UPDATED);
    };

    ArgumentEditor.prototype._onError = function (message) {
        this.error = message;
        ValidationMessageFactory.error(this.editor.getContainer(), message);
    };

    ArgumentEditor.prototype._resetError = function () {
        this.error = null;
        ValidationMessageFactory.success(this.editor.getContainer());
    };

    ArgumentEditor.prototype.getLastParsedValue = function () {
        return this.lastParsedValue;
    };

    ArgumentEditor.prototype.isValid = function () {
        return !this.error;
    };

    ArgumentEditor.prototype.setValue = function (value) {
        this.editor.setText(value);
    };

    ArgumentEditor.prototype.removeIFrameContext = function () {
        this.iFrameContext = null;
        this._onChange();
    };

    ArgumentEditor.prototype.setIFrameContext = function (context) {
        this.iFrameContext = context;
        this._onChange();
    };
});