TestCafeClient.define('UI.RecorderWidgets.AssertionsStep', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        BaseStep = require('UI.RecorderWidgets.BaseStep'),
        ScrollBarWidget = require('UI.RecorderWidgets.ScrollBar'),
        EditAssertionDialog = require('UI.RecorderWidgets.EditAssertionDialog'),
        AssertionWidget = require('UI.RecorderWidgets.Assertion'),
        AssertionsBlockWidget = require('UI.RecorderWidgets.AssertionsBlock'),
        AddAssertionButtonWidget = require('UI.RecorderWidgets.AddAssertionButton'),

        UXLog = require('UI.UXLog'),

        ASSERTIONS_STEP_CLASS = 'assertions-step',
        ASSERTIONS_AREA_CLASS = 'assertions-area',
        ASSERTION_LIST_CLASS = 'assertion-list',
        ASSERTIONS_CLASS = 'assertions',
        EMPTY_CLASS = 'empty';

    var AssertionsStep = this.exports = function ($container, stepNum, stepInfo, options) {
        options = options || {};
        this.options = {
            enableAssertionsValidation: !!options.enableAssertionsValidation,
            updateAssertionsState: options.updateAssertionsState,
            parentPopup: options.parentPopup
        };

        this.$step = null;
        this.$assertionsArea = null;
        this.$assertionList = null;
        this.$assertions = null;
        this.blocks = [];
        this.stepInfo = stepInfo;
        this.addAssertionButton = null;
        this.scrollBar = null;
        this.$iFrameContextLabelContainer = null;

        BaseStep.apply(this, [$container, stepNum, stepInfo, options]);

        ShadowUI.addClass(this.$step, ASSERTIONS_STEP_CLASS);

        this._createAssertionsArea();
    };

    Util.inherit(AssertionsStep, BaseStep);

    //Step events
    AssertionsStep.STEP_NAME_CHANGED_EVENT = BaseStep.STEP_NAME_CHANGED_EVENT;
    AssertionsStep.STEP_INFO_CHANGED_EVENT = BaseStep.STEP_INFO_CHANGED_EVENT;
    AssertionsStep.EDIT_ASSERTION_START_EVENT = 'editAssertionStart';
    AssertionsStep.EDIT_ASSERTION_COMPLETE_EVENT = 'editAssertionComplete';
    AssertionsStep.START_PICKING_ELEMENT_EVENT = 'startPickingElement';

    //Step private methods
    AssertionsStep.prototype._createAssertionsList = function () {
        this.$assertionList = $('<div></div>').appendTo(this.$assertionsArea);
        ShadowUI.addClass(this.$assertionList, ASSERTION_LIST_CLASS);

        this.scrollBar = new ScrollBarWidget(this.$assertionsArea);

        this.$assertions = $('<div></div>').appendTo(this.$assertionList);
        ShadowUI.addClass(this.$assertions, ASSERTIONS_CLASS);

        this.scrollBar.init(this.$assertionList, this.$assertions);
    };

    AssertionsStep.prototype._createAssertionsArea = function () {
        this.$assertionsArea = $('<div></div>').appendTo(this.$step);
        ShadowUI.addClass(this.$assertionsArea, ASSERTIONS_AREA_CLASS);

        if (this.stepInfo.blocks.length)
            this._createAssertionsList();
        else
            ShadowUI.addClass(this.$assertionsArea, EMPTY_CLASS);

        this._createAddAssertionButton();

        for (var i = 0; i < this.stepInfo.blocks.length; i++) {
            for (var j = 0; j < this.stepInfo.blocks[i].assertions.length; j++)
                this._addAssertion(this.stepInfo.blocks[i].assertions[j], this.stepInfo.blocks[i].iFrameSelector);
        }

        this._updateBlocks();

        if (this.scrollBar) {
            this.scrollBar.recalculateSize();
            this._scrollAssertionListToBottom();
        }

    };

    AssertionsStep.prototype._createAddAssertionButton = function () {
        var step = this;

        this.addAssertionButton = new AddAssertionButtonWidget(this.$step);

        this.addAssertionButton.on(AddAssertionButtonWidget.ITEM_CLICKED_EVENT, function (e) {
            UXLog.write('Assertions: add assertion button is pressed');
            step._showEditAssertionDialog(step._getEmptyAssertionInfo(e.operator), e.iFrameSelector, function (e) {
                UXLog.write('Assertions: assertion is added');
                step._addAssertion(e.assertionInfo, e.iFrameSelector, e.isValid);
                step._updateBlocks();
                step.scrollBar.recalculateSize();
                //NOTE: after new assertion added we should scroll assertion list to bottom
                step._scrollAssertionListToBottom(step._findBlock(e.iFrameSelector));
            });
        });
    };

    AssertionsStep.prototype._onChange = function () {
        this.eventEmitter.emit(AssertionsStep.STEP_INFO_CHANGED_EVENT, { });
    };

    AssertionsStep.prototype._findBlock = function (iFrameSelector) {
        for (var i = 0; i < this.blocks.length; i++) {
            if (this.blocks[i].getContext() === iFrameSelector)
                return this.blocks[i];
        }

        return null;
    };

    AssertionsStep.prototype._removeBlock = function (block) {
        var index = $.inArray(block, this.blocks);

        if (index > -1)
            this.blocks.splice(index, 1);

        if (!this.blocks.length) {
            this.$assertionsArea.html('');
            this.scrollBar = null;
            this.$assertionList = null;
        }
    };

    AssertionsStep.prototype._updateBlocks = function () {
        var i = 0;

        if (this.blocks.length > 1) {
            for (; i < this.blocks.length; i++)
                this.blocks[i].showTitle();
        }
        else if (this.blocks.length && !this.blocks[0].getContext())
            this.blocks[0].hideTitle();

        this.stepInfo.blocks = [];

        for (i = 0; i < this.blocks.length; i++) {
            this.stepInfo.blocks.push({
                assertions: this.blocks[i].assertionsInfo,
                iFrameSelector: this.blocks[i].getContext()
            });
        }

        if (this.stepInfo.blocks.length)
            ShadowUI.removeClass(this.$assertionsArea, EMPTY_CLASS);
        else
            ShadowUI.addClass(this.$assertionsArea, EMPTY_CLASS);
    };

    AssertionsStep.prototype._addAssertion = function (assertionInfo, iFrameSelector, isValid) {
        var step = this,
            block = this._findBlock(iFrameSelector);

        if (!this.$assertionList)
            this._createAssertionsList();

        if (!block) {
            block = new AssertionsBlockWidget(this.$assertions, iFrameSelector, !!iFrameSelector);
            this.blocks.push(block);
        }

        var $assertion = $('<div></div>'),
            assertion = new AssertionWidget($assertion, assertionInfo, {
                enableValidation: this.options.enableAssertionsValidation,
                isValid: isValid,
                iFrameSelector: iFrameSelector
            });

        block.addAssertion($assertion, assertionInfo);

        if (this.options.updateAssertionsState)
            assertionInfo.failed = assertion.isValid === false;

        assertion.on(AssertionWidget.VALIDITY_CHANGED_EVENT, function (e) {
            assertionInfo.failed = e.isValid === false;
        });

        assertion.on(AssertionWidget.ASSERTION_CLICKED_EVENT, function () {
            UXLog.write('Assertions: edit assertion is started');
            step._showEditAssertionDialog(assertion.assertionInfo, iFrameSelector, function (e) {
                UXLog.write('Assertions: assertion is changed');
                assertionInfo.failed = step.options.updateAssertionsState ? !e.isValid : false;
                assertion.update(e.assertionInfo, e.isValid, e.iFrameSelector);
            });
        });

        assertion.on(AssertionWidget.ASSERTION_REMOVED_EVENT, function () {
            UXLog.write('Assertions: assertion is removed');

            var index = $.inArray(assertion.$assertion, block.assertions),
                scrollBarTop = parseInt(step.scrollBar.$scrollBar.css('top').replace('px', ''));

            block.removeAssertion($assertion);

            if (!block.getAssertionsCount()) {
                block.remove();
                step._removeBlock(block);
                step._updateBlocks();
            }

            assertion.getContainer().remove();

            if (step.scrollBar) {
                step.scrollBar.recalculateSize();

                if (block.assertions && block.assertions.length) {
                    //NOTE: we should  try to restore previous scroll value
                    step.scrollBar.restoreScroll(scrollBarTop);
                    step.scrollBar.updateScroll(block.assertions[index]);
                }
            }

            step._onChange();
        });
    };

    //TODO: move this to scrollBarWidget code and use in other places
    AssertionsStep.prototype._onScrollableContentHeightChange = function (difference) {
        if (!this.scrollBar)
            return;

        var scrollTop = parseInt(this.$assertions.css('top').replace('px', ''));
        if (scrollTop < 0) {
            var newScrollTop = Math.min(scrollTop - difference, 0);
            this.$assertions.css('top', newScrollTop + 'px');
            this.scrollBar.recalculateSize();
        }
    };

    AssertionsStep.prototype._showEditAssertionDialog = function (assertionInfo, iFrameSelector, onConfirm) {
        var step = this;

        if (step.options.parentPopup)
            step.options.parentPopup.hide();

        this.eventEmitter.emit(AssertionsStep.EDIT_ASSERTION_START_EVENT, {});

        var editAssertionDialog = new EditAssertionDialog(assertionInfo, {
            enableAssertionValidation: step.options.enableAssertionsValidation,
            iFrameSelector: iFrameSelector
        });

        editAssertionDialog.on(EditAssertionDialog.CONFIRM_BUTTON_CLICK_EVENT, function (e) {
            if (step.options.parentPopup)
                step.options.parentPopup.show();

            onConfirm(e);
            step._onChange();
            step.eventEmitter.emit(AssertionsStep.EDIT_ASSERTION_COMPLETE_EVENT);
        });

        editAssertionDialog.on(EditAssertionDialog.CANCEL_BUTTON_CLICK_EVENT, function () {
            if (step.options.parentPopup)
                step.options.parentPopup.show();

            step.eventEmitter.emit(AssertionsStep.EDIT_ASSERTION_COMPLETE_EVENT);
        });

        editAssertionDialog.on(EditAssertionDialog.START_PICKING_ELEMENT_EVENT, function (e) {
            step.eventEmitter.emit(AssertionsStep.START_PICKING_ELEMENT_EVENT, e);
        });
    };

    AssertionsStep.prototype._getEmptyAssertionInfo = function (methodName) {
        return {
            operator: methodName,
            message: '',
            arguments: new Array(/eq/i.test(methodName) ? 2 : 1)
        };
    };

    AssertionsStep.prototype._scrollAssertionListToBottom = function (block) {
        var lastBlock = block || this.blocks[this.blocks.length - 1],
            $lastAssertion = lastBlock.assertions[lastBlock.assertions.length - 1];

        this.scrollBar.updateScroll($lastAssertion);
    };

    //Step API
    AssertionsStep.prototype.isValid = function () {
        return !!this.stepInfo.blocks.length;
    };

    AssertionsStep.prototype.getStepInfo = function () {
        return this.stepInfo;
    };
});