TestCafeClient.define('UI.RecorderWidgets.ChoosePropertyDialog', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,

        PropertyListGenerator = require('Recorder.PropertyListGenerator'),
        PopupWidget = require('UI.RecorderWidgets.Popup'),
        ButtonWidget = require('UI.RecorderWidgets.Button'),
        ScrollBarWidget = require('UI.RecorderWidgets.ScrollBar'),
        TabsWidget = require('UI.RecorderWidgets.Tabs');

    //Const
    var BUTTONS_CLASS = 'buttons',
        WIDGET_SWITCHER_LEFT_CLASS = 'widget-switcher left',
        WIDGET_SWITCHER_RIGHT_CLASS = 'widget-switcher right',
        CLEAR_CLASS = 'clear',
        CHOOSE_PROPERTY_DIALOG_CLASS = 'choose-property-dialog',
        PROPERTY_LIST_CONTAINER_CLASS = 'property-list-container',
        PROPERTY_LIST_CLASS = 'property-list',
        PROPERTY_LIST_ITEMS_CLASS = 'property-list-items',
        PROPERTY_LIST_ITEM_CLASS = 'property-list-item',
        HOVERED_CLASS = 'hovered',
        SELECTED_CLASS = 'selected',
        EVEN_CLASS = 'even',
        PROPERTY_NAME_CLASS = 'property-name',
        PROPERTY_VALUE_CLASS = 'property-value',
        RECORDER_CLASS = 'recorder',
        RESULT_CLASS = 'result',
        COLUMN_HEADERS_CLASS = 'column-headers',
        HEADER_CLASS = 'header',
        DIALOG_WIDTH = 629;

    var ChoosePropertyDialog = this.exports = function (parsedSelector, popupPosition, iFrameContext) {
        var dialog = this;

        this.tabs = null;

        this.parsedSelector = parsedSelector;
        this.popup = null;
        this.popupPosition = popupPosition || null;
        this.selectedProperty = null;
        this.iFrameContext = iFrameContext;
        this.$selectedPropertyItem = null;

        this.$backButton = null;
        this.$nextButton = null;
        this.$dialog = null;
        this.$resultBox = null;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            dialog.eventEmitter.on(ev, listener);
        };

        this._createDialog();
        this._init();
    };

    //Events
    ChoosePropertyDialog.CONFIRM_BUTTON_CLICK_EVENT = 'confirmButtonClick';
    ChoosePropertyDialog.CLOSE_BUTTON_CLICK_EVENT = 'closeButtonClick';
    ChoosePropertyDialog.PREVIOUS_BUTTON_CLICK_EVENT = 'previousButtonClick';

    //Markup
    ChoosePropertyDialog.prototype._createDialog = function () {
        var dialog = this,
            $recorder = ShadowUI.select('.' + RECORDER_CLASS, ShadowUI.getRoot());

        this.$dialog = $('<div></div>');

        ShadowUI.addClass(this.$dialog, CHOOSE_PROPERTY_DIALOG_CLASS);

        var popupOptions = {
            width: DIALOG_WIDTH,
            headerText: 'Choose property',
            content: this.$dialog,
            footerContent: this._createButtons(),
            backgroundOpacity: true,
            headerCloseButton: true
        };

        this.popup = new PopupWidget($recorder, popupOptions);

        this._createResultBox();

        this.tabs = new TabsWidget(this.$dialog, {
            titles: ['common', 'css properties', 'attributes'],
            getContentCallback: function (index, callback) {
                if (index === 0) {
                    PropertyListGenerator.getGeneralProperties(dialog.parsedSelector.evalResults, dialog.parsedSelector.selector, function (properties) {
                        dialog._createProperties(properties, function ($container, createPropertiesCallback) {
                            callback($container, function () {
                                createPropertiesCallback();

                                if (dialog.popupPosition)
                                    dialog.popup.moveTo(dialog.popupPosition.left, dialog.popupPosition.top);
                            });
                        });
                    }, dialog.iFrameContext);
                }
                else if (index === 1) {
                    PropertyListGenerator.getCssProperties(dialog.parsedSelector.evalResults, dialog.parsedSelector.selector, function (properties) {
                        dialog._createProperties(properties, callback);
                    }, dialog.iFrameContext);
                }
                else {
                    PropertyListGenerator.getAttributes(dialog.parsedSelector.evalResults, dialog.parsedSelector.selector, function (properties) {
                        dialog._createProperties(properties, callback);
                    }, dialog.iFrameContext);
                }
            }
        });

        if (this.popupPosition)
            this.popup.moveTo(this.popupPosition.left, this.popupPosition.top);
        else
            this.popup.showAtWindowCenter();
    };

    ChoosePropertyDialog.prototype._createButtons = function () {
        var $buttons = $('<div></div>');
        ShadowUI.addClass($buttons, BUTTONS_CLASS);

        this.$backButton = ButtonWidget.create($buttons, 'Back', true);
        ShadowUI.addClass(this.$backButton, WIDGET_SWITCHER_LEFT_CLASS);

        this.$nextButton = ButtonWidget.create($buttons, 'Next', true);
        ShadowUI.addClass(this.$nextButton, WIDGET_SWITCHER_RIGHT_CLASS);

        //TODO: try to find common solution for all browsers
        this.$backButton.css({
            paddingLeft: Util.isMozilla ? '6px' : '9px',
            paddingRight: Util.isMozilla ? '6px' : '9px',
            width: Util.isMozilla ? '144px' : '138px'
        });

        this.$nextButton.css({
            paddingLeft: Util.isMozilla ? '6px' : '9px',
            paddingRight: Util.isMozilla ? '6px' : '9px',
            width: Util.isMozilla ? '144px' : '138px'
        });

        ShadowUI.addClass($('<div></div>').appendTo($buttons), CLEAR_CLASS);

        return $buttons;
    };

    ChoosePropertyDialog.prototype._createResultBox = function () {
        this.$resultBox = $('<div></div>').text(this.parsedSelector.selector).appendTo(this.$dialog);
        ShadowUI.addClass(this.$resultBox, RESULT_CLASS);
    };

    function createColumnHeaders($container) {
        var $columnHeaders = $('<div></div>').appendTo($container),
            $variableHeader = $('<div></div>').text('variable').appendTo($columnHeaders),
            $valueHeader = $('<div></div>').text('value').appendTo($columnHeaders);

        ShadowUI.addClass($columnHeaders, COLUMN_HEADERS_CLASS);
        ShadowUI.addClass($variableHeader, HEADER_CLASS);
        ShadowUI.addClass($valueHeader, HEADER_CLASS);
    }

    ChoosePropertyDialog.prototype._createProperties = function (properties, callback) {
        var $container = $('<div></div>');

        createColumnHeaders($container);

        var $propertyListContainer = $('<div></div>').appendTo($container),
            $propertyList = $('<div></div>').appendTo($propertyListContainer),
            $items = $('<div></div>').appendTo($propertyList);

        ShadowUI.addClass($propertyListContainer, PROPERTY_LIST_CONTAINER_CLASS);
        ShadowUI.addClass($propertyList, PROPERTY_LIST_CLASS);
        ShadowUI.addClass($items, PROPERTY_LIST_ITEMS_CLASS);

        for (var i = 0; i < properties.length; i++)
            this._createPropertyListItem(properties[i], i % 2 !== 0).appendTo($items);

        var scrollBar = new ScrollBarWidget($propertyListContainer);
        scrollBar.init($propertyList, $items);

        callback($container, function () {
            scrollBar.recalculateSize();
        });
    };

    ChoosePropertyDialog.prototype._createPropertyListItem = function (property, isEven) {
        var dialog = this;

        var $item = $('<div></div>');
        ShadowUI.addClass($item, PROPERTY_LIST_ITEM_CLASS);

        if (isEven)
            ShadowUI.addClass($item, EVEN_CLASS);

        $item.data('property', property);

        var $propertyName = $('<div></div>').text(property.name).appendTo($item);
        ShadowUI.addClass($propertyName, PROPERTY_NAME_CLASS);

        var $propertyValue = $('<div></div>').text(property.value).appendTo($item);
        ShadowUI.addClass($propertyValue, PROPERTY_VALUE_CLASS);

        if ($propertyValue[0].offsetWidth < $propertyValue[0].scrollWidth)
            $propertyValue.attr('title', property.value);

        $item.click(function () {
            if (dialog.selectedProperty === property) {
                dialog._clearSelectedPropertyItem();
                dialog.selectedProperty = null;
                dialog.$resultBox.text(dialog._getGeneratedArgument());
            }
            else
                dialog._setSelectedProperty($item, property);
        });

        $item.hover(function () {
            ShadowUI.addClass($item, HOVERED_CLASS);
        }, function () {
            ShadowUI.removeClass($item, HOVERED_CLASS);
        });

        if (property.isDefault)
            this._setSelectedProperty($item, property);

        return $item;
    };

    ChoosePropertyDialog.prototype._clearSelectedPropertyItem = function () {
        if (this.$selectedPropertyItem)
            ShadowUI.removeClass(this.$selectedPropertyItem, SELECTED_CLASS);
    };

    ChoosePropertyDialog.prototype._getGeneratedArgument = function () {
        return this.selectedProperty ? this.selectedProperty.getter : this.parsedSelector.selector;
    };

    ChoosePropertyDialog.prototype._setSelectedProperty = function ($item, property) {
        this.selectedProperty = property;
        this._clearSelectedPropertyItem();
        this.$selectedPropertyItem = $item;

        ShadowUI.addClass($item, SELECTED_CLASS);
        ShadowUI.removeClass($item, HOVERED_CLASS);

        this.$resultBox.text(this._getGeneratedArgument());
    };

//Behavior
    ChoosePropertyDialog.prototype._init = function () {
        var dialog = this;

        ShadowUI.bind(this.$backButton, 'click', function () {
            dialog._close(function () {
                dialog.eventEmitter.emit(ChoosePropertyDialog.PREVIOUS_BUTTON_CLICK_EVENT, {
                    popupPosition: dialog.popup.getPosition()
                });
            });
        });

        ShadowUI.bind(this.$nextButton, 'click', function () {
            dialog._close(function () {
                dialog.eventEmitter.emit(ChoosePropertyDialog.CONFIRM_BUTTON_CLICK_EVENT, {
                    text: dialog._getGeneratedArgument()
                });
            });
        });

        this.popup.on(PopupWidget.CLOSE_BUTTON_CLICK_EVENT, function () {
            dialog._close(function () {
                dialog.eventEmitter.emit(ChoosePropertyDialog.CLOSE_BUTTON_CLICK_EVENT, {});
            });
        });

        this.popup.onkeydown(function (e) {
            if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.enter) {
                if (dialog.$nextButton && dialog.$nextButton.css('visibility') !== 'hidden')
                    dialog.$nextButton.trigger('click');
                Util.preventDefault(e);
            }

            //NOTE: we should prevent page scrolling
            if (e.target === dialog.popup.$popup[0] && Util.isArrowKey(e.keyCode))
                Util.preventDefault(e);
        }, true);
    };

    ChoosePropertyDialog.prototype._close = function (callback) {
        this.popup.close(callback);
    };
});