TestCafeClient.define('UI.RecorderWidgets.ChooseSelectorDialog', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,

        PopupWidget = require('UI.RecorderWidgets.Popup'),
        ButtonWidget = require('UI.RecorderWidgets.Button'),
        SelectorEditorWidget = require('UI.RecorderWidgets.SelectorEditor'),
        JavascriptExecutor = require('Base.JavascriptExecutor');

    //Const
    var BUTTONS_CLASS = 'buttons',
        WIDGET_SWITCHER_LEFT_CLASS = 'widget-switcher left',
        WIDGET_SWITCHER_RIGHT_CLASS = 'widget-switcher right',
        CLEAR_CLASS = 'clear',
        CHOOSE_SELECTOR_DIALOG_CLASS = 'choose-selector-dialog',
        SELECTOR_EDITOR_CONTAINER_CLASS = 'selector-editor-container',
        RECORDER_CLASS = 'recorder',

        HEADER_TEXT = 'Element Selector',

        DIALOG_WIDTH = 629;

    var ChooseSelectorDialog = this.exports = function (elementSelectors, elementSelectorsIndex, $iFrame, popupPosition) {
        var dialog = this;

        this.popup = null;
        this.popupPosition = popupPosition || null;
        this.selectorEditor = null;
        this.elementSelectors = elementSelectors;
        this.elementSelectorsIndex = elementSelectorsIndex || 0;

        this.$nextButton = null;
        this.$backButton = null;
        this.$dialog = null;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            dialog.eventEmitter.on(ev, listener);
        };

        this.$iFrame = $iFrame;

        this._createDialog();

        this._init();
    };

    //Events
    ChooseSelectorDialog.CONFIRM_BUTTON_CLICK_EVENT = 'confirmButtonClick';
    ChooseSelectorDialog.CLOSE_BUTTON_CLICK_EVENT = 'closeButtonClick';

    //Markup
    ChooseSelectorDialog.prototype._createDialog = function () {
        var $recorder = ShadowUI.select('.' + RECORDER_CLASS, ShadowUI.getRoot());

        this.$dialog = $('<div></div>').css('visibility', 'hidden');
        ShadowUI.addClass(this.$dialog, CHOOSE_SELECTOR_DIALOG_CLASS);

        var popupOptions = {
            width: DIALOG_WIDTH,
            headerText: HEADER_TEXT,
            content: this.$dialog,
            footerContent: this._createButtons(),
            backgroundOpacity: true,
            headerCloseButton: true
        };

        this.popup = new PopupWidget($recorder, popupOptions);

        //selector editor must be created before popup positioning to correctly calculate its height,
        //but elements marking should be enabled after popup positioning to prevent wrong elements marking
        this._createSelectorEditor();

        if (this.popupPosition)
            this.popup.moveTo(this.popupPosition.left, this.popupPosition.top);
        else {
            var $actualElement = this.$iFrame || JavascriptExecutor.parseSelectorSync(this.elementSelectors[0].selector).$elements;

            this.popup.disposeRelativeToElement($actualElement);
        }

        this.selectorEditor.enableElementsMarking();

        this.$dialog.css('visibility', '');
    };

    ChooseSelectorDialog.prototype._createButtons = function () {
        var $buttons = $('<div></div>');
        ShadowUI.addClass($buttons, BUTTONS_CLASS);

        this.$backButton = ButtonWidget.create($buttons, 'Back', true);
        ShadowUI.addClass(this.$backButton, WIDGET_SWITCHER_LEFT_CLASS);

        this.$nextButton = ButtonWidget.create($buttons, 'Next', true);
        ShadowUI.addClass(this.$nextButton, WIDGET_SWITCHER_RIGHT_CLASS);

        //TODO: try to find common solution for all browsers
        this.$backButton.css({
            paddingLeft: Util.isMozilla ? '6px' : '9px',
            paddingRight: Util.isMozilla ? '6px' : '9px',
            width: Util.isMozilla ? '144px' : '138px'
        });

        this.$nextButton.css({
            paddingLeft: Util.isMozilla ? '6px' : '9px',
            paddingRight: Util.isMozilla ? '6px' : '9px',
            width: Util.isMozilla ? '144px' : '138px'
        });

        ShadowUI.addClass($('<div></div>').appendTo($buttons), CLEAR_CLASS);

        return $buttons;
    };

    ChooseSelectorDialog.prototype._createSelectorEditor = function () {
        var dialog = this;

        var $selectorEditorContainer = $('<div></div>').appendTo(this.$dialog);
        ShadowUI.addClass($selectorEditorContainer, SELECTOR_EDITOR_CONTAINER_CLASS);

        this.selectorEditor = new SelectorEditorWidget($selectorEditorContainer, {
            selectors: dialog.elementSelectors,
            currentSelectorIndex: dialog.elementSelectorsIndex,
            enableElementsMarking: false,
            parseDomElementsOrJqueryObjectsOnly: true,
            allowMultipleElements: true,
            allowEdit: true,
            $floatingParent: this.popup.getContainer(),
            enableValidation: true,
            context: this.$iFrame ? this.$iFrame[0].contentWindow : null
        });
    };

    ChooseSelectorDialog.prototype._onError = function () {
        this.$nextButton.css('visibility', 'hidden');
    };

    ChooseSelectorDialog.prototype._onSuccess = function () {
        this.$nextButton.css('visibility', '');
    };

//Behavior
    ChooseSelectorDialog.prototype._init = function () {
        var dialog = this;

        function onCloseButtonClick() {
            dialog._close(function () {
                dialog.eventEmitter.emit(ChooseSelectorDialog.CLOSE_BUTTON_CLICK_EVENT, {});
            });
        }

        ShadowUI.bind(this.$nextButton, 'click', function () {
            dialog._close(function () {
                dialog.eventEmitter.emit(ChooseSelectorDialog.CONFIRM_BUTTON_CLICK_EVENT, {
                    elementSelectors: dialog.elementSelectors,
                    elementSelectorsIndex: dialog.elementSelectorsIndex,
                    parsedElementSelector: dialog.selectorEditor.getParsedSelector(),
                    popupPosition: dialog.popup.getPosition()
                });
            });
        });

        ShadowUI.bind(this.$backButton, 'click', onCloseButtonClick);
        this.popup.on(PopupWidget.CLOSE_BUTTON_CLICK_EVENT, onCloseButtonClick);

        this.popup.onkeydown(function (e) {
            if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.enter) {
                if (dialog.$nextButton && dialog.$nextButton.css('visibility') !== 'hidden')
                    dialog.$nextButton.trigger('click');
                Util.preventDefault(e);
            }

            if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.esc) {
                dialog.$backButton.trigger('click');
                Util.preventDefault(e);
            }

            //NOTE: we should prevent page scrolling
            if (e.target === dialog.popup.$popup[0] && Util.isArrowKey(e.keyCode))
                Util.preventDefault(e);
        }, true);

        this.selectorEditor.on(SelectorEditorWidget.SELECTOR_CHANGED_EVENT, function (e) {
            if (!dialog.selectorEditor.isValid(true) || (dialog.selectorEditor.getParsedSelector() && !dialog.selectorEditor.getParsedSelector().length))
                dialog._onError();
            else {
                dialog._onSuccess();
                dialog.elementSelectors = e.selectors;
                dialog.elementSelectorsIndex = e.index;
            }
        });

        if (!dialog.selectorEditor.isValid(true) || (dialog.selectorEditor.getParsedSelector() && !dialog.selectorEditor.getParsedSelector().$elements))
            dialog._onError();
    };

    ChooseSelectorDialog.prototype._close = function (callback) {
        this.selectorEditor.destroy();
        this.popup.close(callback);
    };
});