TestCafeClient.define('UI.RecorderWidgets.AuthenticationDialog', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        ValidationMessageFactory = require('UI.ValidationMessageFactory'),
        PopupWidget = require('UI.RecorderWidgets.Popup'),
        ButtonWidget = require('UI.RecorderWidgets.Button');

    //Const
    var HEADER_TEXT = 'Authentication Required',
        LOGIN_BUTTON_TEXT = 'Log In',
        CANCEL_BUTTON_TEXT = 'Cancel',
        USERNAME_LABEL_TEXT = 'User Name:',
        PASSWORD_LABEL_TEXT = 'Password:',
        SERVER_MESSAGE_TEXT_PATTERN = '<p>The %s website requires authentication. Please fill in the username and password to log in.</p>' +
            '<br /><p>The login information will be saved to the fixture file.</p>',

    //Events
        LOG_IN_BUTTON_CLICK_EVENT = 'logInButtonClick',
        CANCEL_BUTTON_CLICK_EVENT = 'cancelButtonClick',

    //Classes
        AUTHENTICATION_DIALOG_CONTENT_CLASS = 'authentication-dialog-area',
        CREDENTIALS_AREA_CLASS = 'credentials-area',
        LOGIN_AREA_CLASS = 'login-area',
        MESSAGE_AREA_CLASS = 'message',
        FIELD_CLASS = 'field',
        VALUE_CLASS = 'value',
        INPUT_CLASS = 'input',
        INPUT_SEPARATOR_CLASS = 'input-separator',
        BUTTONS_CLASS = 'buttons';

    //Globals
    var eventEmitter = null,
        popup = null,
        $usernameInput = null,
        $passwordInput = null,
        $inputSeparator = null;

    //Util
    var createMessage = function (url, message) {
        return SERVER_MESSAGE_TEXT_PATTERN.replace('%s', url).replace('%s', message);
    };

    //Markup
    var createDialog = function ($container, options) {
        var pageUrl = options.originUrl,
            serverMessage = options.serverMessage || '',
            username = options.username || '',
            password = options.password || '',
            errorMessage = options.errorMessage;

        var popupOptions = {
            width: 750,
            content: createContent(createMessage(pageUrl, serverMessage), username, password),
            headerText: HEADER_TEXT,
            footerContent: createButtons(options.cancelButtonText),
            showAtWindowCenter: true,
            notDialog: true
        };

        if (errorMessage)
            exports.showErrorMessage(errorMessage);

        popup = new PopupWidget($container, popupOptions);

        popup.onkeydown(function (e) {
            if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.enter) {
                Util.getActiveElement().blur();
                onLoginButtonClick();
                Util.preventDefault(e);
            }
        });

        $usernameInput.focus();
    };

    var createContent = function (message, username, password) {
        var usernameInputId = 'tcc-uni-1b73311b',
            passwordInputId = 'tcc-pi-a7c77c94';

        var $contentContainer = $('<div>'),
            $credentialsArea = $('<div>').appendTo($contentContainer),
            $messageArea = $('<div>').appendTo($credentialsArea),
            $usernameArea = $('<div>').appendTo($credentialsArea),
            $passwordArea = $('<div>').appendTo($credentialsArea),
            $usernameLabel = $('<label>').attr('for', usernameInputId).html(USERNAME_LABEL_TEXT).appendTo($usernameArea),
            $usernameInputDiv = $('<div>').appendTo($usernameArea),
            $passwordLabel = $('<label>').attr('for', passwordInputId).html(PASSWORD_LABEL_TEXT).appendTo($passwordArea),
            $passwordInputDiv = $('<div>').appendTo($passwordArea);

        $usernameInput = $('<input>').attr('id', usernameInputId).attr('value', username).appendTo($usernameInputDiv);
        $passwordInput = $('<input type="password">').attr('id', passwordInputId).attr('value', password).appendTo($passwordInputDiv);
        $inputSeparator = $('<div>').insertAfter($usernameArea);

        ShadowUI.addClass($contentContainer, AUTHENTICATION_DIALOG_CONTENT_CLASS);
        ShadowUI.addClass($credentialsArea, CREDENTIALS_AREA_CLASS);
        ShadowUI.addClass($messageArea, MESSAGE_AREA_CLASS);
        ShadowUI.addClass($usernameArea, LOGIN_AREA_CLASS);
        ShadowUI.addClass($inputSeparator, INPUT_SEPARATOR_CLASS);
        ShadowUI.addClass($usernameLabel, FIELD_CLASS);
        ShadowUI.addClass($passwordLabel, FIELD_CLASS);
        ShadowUI.addClass($usernameInputDiv, VALUE_CLASS);
        ShadowUI.addClass($passwordInputDiv, VALUE_CLASS);
        ShadowUI.addClass($usernameInput, INPUT_CLASS);
        ShadowUI.addClass($passwordInput, INPUT_CLASS);

        if (Util.isWebKit || $.browser.opera) {
            var correctLabelStyle = {
                position: 'relative',
                top: '1px'
            };

            $usernameLabel.css(correctLabelStyle);
            $passwordLabel.css(correctLabelStyle);
        }

        return $contentContainer;
    };

    var createButtons = function (cancelButtonText) {
        var $buttonArea = $('<div>');

        ButtonWidget.create($buttonArea, LOGIN_BUTTON_TEXT).click(onLoginButtonClick);

        ButtonWidget.create($buttonArea, cancelButtonText || CANCEL_BUTTON_TEXT)
            .click(function () {
                eventEmitter.emit(CANCEL_BUTTON_CLICK_EVENT, {});
            });

        ShadowUI.addClass($buttonArea, BUTTONS_CLASS);

        return $buttonArea;
    };


    //Behavior
    var onLoginButtonClick = function () {
        var username = $usernameInput.attr('value');

        if (!username) {
            var onUsernameChange = function () {
                ValidationMessageFactory.success($usernameInput);
                $usernameInput.unbind('change', onUsernameChange);
            };
            ValidationMessageFactory.error($usernameInput);
            $usernameInput.bind('change', onUsernameChange);
        }
        else {
            ValidationMessageFactory.success($usernameInput);
            eventEmitter.emit(LOG_IN_BUTTON_CLICK_EVENT, {
                username: username,
                password: $passwordInput.attr('value')
            });
        }

        ValidationMessageFactory.success($passwordInput);
        $inputSeparator.css('visibility', 'hidden');
    };

    //API
    exports.init = function ($container, options) {
        eventEmitter = new Util.EventEmitter();
        createDialog($container, options);
    };

    exports.close = function (callback) {
        popup.close(callback);
    };

    exports.blind = function (blind) {
        popup.blind(blind);
    };

    exports.showErrorMessage = function (message) {
        $inputSeparator.css('visibility', 'visible');
        ValidationMessageFactory.error($usernameInput);
        ValidationMessageFactory.error($passwordInput, message, true);
    };

    exports.events = {
        on: function (event, handler) {
            eventEmitter.on(event, handler);
        }
    };
});
