TestCafeClient.define('UI.RecorderWidgets.BaseStep', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        DialogPropertyWidget = require('UI.RecorderWidgets.DialogProperty'),

        STEP_CLASS = 'step',
        FIELD_CLASS = 'field',
        SEPARATOR_CLASS = 'separator';

    //Step
    var BaseStep = this.exports = function ($container, stepNum, stepInfo, options) {
        var step = this;

        this.options = options || {};

        this.$step = null;

        this.stepNamePropertyEditor = null;

        this.stepInfo = stepInfo;
        this.stepNum = stepNum;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            step.eventEmitter.on(ev, listener);
        };

        this._createBaseStep($container);

        this._initBaseStep();
    };

    //Events
    BaseStep.STEP_NAME_CHANGED_EVENT = 'stepNameChanged';
    BaseStep.STEP_INFO_CHANGED_EVENT = 'stepInfoChanged';

    //Markup
    BaseStep.prototype._createBaseStep = function ($container) {
        this.$step = $('<div></div>').appendTo($container);
        ShadowUI.addClass(this.$step, STEP_CLASS);

        this._createStepName(this.$step);
    };

    BaseStep.prototype._createLabel = function ($container, labelText, inputId) {
        var $label = $('<label></label>')
            .appendTo($container)
            .text(labelText);

        ShadowUI.addClass($label, FIELD_CLASS);
        if (inputId)
            $label.attr('for', inputId);

        return $label;
    };

    BaseStep.prototype._createStepName = function ($container) {
        this.stepNamePropertyEditor = new DialogPropertyWidget($container, 'Name:', this.stepInfo.name, {maximizeInput: true});

        var $separator = $('<div></div>').appendTo($container);
        ShadowUI.addClass($separator, SEPARATOR_CLASS);
    };

    //Behavior
    BaseStep.prototype._initBaseStep = function () {
        var step = this;

        this.stepNamePropertyEditor.on(DialogPropertyWidget.VALUE_CHANGED_EVENT, function (e) {
            if (step.stepNamePropertyEditor.isValid()) {
                step.stepInfo.name = e.value;
                step.eventEmitter.emit(BaseStep.STEP_NAME_CHANGED_EVENT, { name: step.stepInfo.name });
            }
        });
    };

    //API
    BaseStep.prototype.isValid = function () {
        return true;
    };

    BaseStep.prototype.getStepInfo = function () {
        return this.stepInfo;
    };

    BaseStep.prototype.destroy = function () {
    };
});