TestCafeClient.define('UI.RecorderWidgets.Checkbox', function () {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI;

    //Const
    var CHECKBOX_CLASS = 'checkbox',
        CHECKBOX_ICON_CLASS = 'checkbox-icon',
        CHECKBOX_TITLE_CLASS = 'checkbox-title',
        PRESSED_CLASS = 'pressed';

    var Checkbox = this.exports = function ($container, state, title) {
        var checkbox = this;

        this.$checkbox = null;
        this.$checkboxTitle = null;
        this.$checkboxContainer = null;

        this.state = state || false;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            checkbox.eventEmitter.on(ev, listener);
        };

        this._create($container, title);
        this._init();
    };

    //Events
    Checkbox.STATE_CHANGED_EVENT = 'stateChanged';

    //Markup
    Checkbox.prototype._create = function ($container, title) {
        this.$checkboxContainer = $('<div></div>').appendTo($container);
        ShadowUI.addClass(this.$checkboxContainer, CHECKBOX_CLASS);

        this.$checkbox = $('<div></div>').appendTo(this.$checkboxContainer);
        ShadowUI.addClass(this.$checkbox, CHECKBOX_ICON_CLASS);

        if (this.state)
            ShadowUI.addClass(this.$checkbox, PRESSED_CLASS);

        if (title) {
            this.$checkboxTitle = $('<span></span>').html(title).appendTo(this.$checkboxContainer);
            ShadowUI.addClass(this.$checkboxTitle, CHECKBOX_TITLE_CLASS);

            if (Util.isIE && Util.browserVersion === 9)
                this.$checkboxTitle[0].setAttribute('unselectable', 'on');
        }

        this._init();
    };

//Behavior
    Checkbox.prototype._init = function () {
        var checkbox = this;

        var checkboxClickHandler = function (e) {
            checkbox.changeState();
            Util.preventDefault(e);
        };

        ShadowUI.bind(this.$checkboxContainer, 'click', checkboxClickHandler);
    };

    //API
    Checkbox.prototype.changeState = function () {
        if (this.state)
            ShadowUI.removeClass(this.$checkbox, PRESSED_CLASS);
        else
            ShadowUI.addClass(this.$checkbox, PRESSED_CLASS);

        this.state = !this.state;

        this.eventEmitter.emit(Checkbox.STATE_CHANGED_EVENT, { state: this.state });
    };

    Checkbox.prototype.getContainer = function () {
        return this.$checkbox;
    };
});