TestCafeClient.define('UI.RecorderWidgets.CodeFormatter', function () {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        NativeMethods = Hammerhead.NativeMethods;

    //consts
    var CHUNK_SIZE = 20;

    var COMMENT_CLASS = 'comment',
        KEYWORD_CLASS = 'keyword',
        STRING_CLASS = 'string',
        OPERATOR_CLASS = 'operator',
        LITERAL_CLASS = 'literal',
        ARGUMENT_CLASS = 'argument',
        FUNCTION_NAME_CLASS = 'functionName',
        FUNCTION_CLASS = 'function',
        PROPERTY_CLASS = 'property',
        LANGUAGE_CLASS = 'language',
        NUMBER_CLASS = 'number',
        OBJECT_CLASS = 'object',

        KEYWORDS = ['break', 'case', 'catch', 'continue', 'debugger', 'default', 'delete', 'do', 'else', 'finally',
            'for', 'function', 'if', 'in', 'instanceof', 'new', 'return', 'switch', 'this', 'throw', 'try', 'typeof',
            'var', 'void', 'while', 'with'],

        OPERATORS = ['&lt;', '&gt;', '=', '\\!', '\\+', '\\-', '\\*', '\\%', '&amp;', '\\|', '\\b[0-9]+\\b', '\\$(?=\\.|\\()'],

        LITERALS = ['null', 'true', 'false', 'undefined', 'NaN', 'Infinity'],

        FUNCTIONS = ['alert', 'eval', 'toString', 'apply', 'call', 'test', 'toLocaleString', 'concat', 'join', 'pop',
            'push', 'reverse', 'shift', 'slice', 'sort', 'slice', 'unshift', 'indexOf', 'lastIndexOf',
            //String
            'fromCharCode', 'valueOf', 'charAt', 'charCodeAt', 'match', 'replace', 'search', 'slice', 'split', 'substring',
            'substr', 'toLowerCase', 'toLocaleLowerCase', 'toUpperCase', 'toLocaleUpperCase', 'trim',
            //Math
            'abs', 'acos', 'asin', 'atan', 'atan2', 'ceil', 'cos', 'exp', 'floor', 'log', 'max', 'min', 'pow', 'random',
            'round', 'sin', 'sqrt', 'tan',
            //Date
            'parse', 'UTC', 'now', 'toDateString', 'toTimeString', 'toLocaleString', 'toLocaleDateString', 'toLocaleTimeString',
            'getTime', 'getFullYear', 'getUTCFullYear', 'getMonth', 'getUTCMonth', 'getDate', 'getUTCDate', 'getDay',
            'getUTCDay', 'getHours', 'getUTCHours', 'getMinutes', 'getUTCMinutes', 'getSeconds', 'getUTCSeconds',
            'getMilliseconds', 'getUTCMilliseconds', 'getTimezoneOffset', 'setTime', 'setMilliseconds', 'setUTCMilliseconds',
            'setSeconds', 'setUTCSeconds', 'setMinutes', 'setUTCMinutes', 'setHours', 'setUTCHours', 'setDate', 'setUTCDate',
            'setMonth', 'setUTCMonth', 'setFullYear', 'setUTCFullYear', 'toUTCString', 'toJSON',
            //Other
            'exec', 'test', 'parse', 'stringify', 'parseInt', 'parseFloat', 'isNaN', 'isFinite'
        ],

        PROPERTIES = ['length', 'constructor', 'prototype', 'MAX_VALUE', 'MIN_VALUE', 'NEGATIVE_INFINITY',
            'POSITIVE_INFINITY', 'E', 'LN10', 'LN2', 'LOG2E', 'LOG10E', 'PI', 'SQRT1_2', 'SQRT2', 'name'],

        OBJECTS = ['Array', 'Boolean', 'Date', 'Math', 'Number', 'Object', 'RegExp', 'String', 'window'],

        SAFE = { '<': '&lt;', '>': '&gt;', '&': '&amp;' };


    function createRegExpFromArray(array, notWords) {
        return notWords ?
            new RegExp('' + array.join('|'), 'g') :
            new RegExp('\\b(' + array.join('|') + ')\\b', 'g');
    }

    function getLanguageFunctionsRegExp() {
        var functions = $.map(FUNCTIONS, function (item) {
            return item + '(?=\\(([^\\)]*\\))?)';
        });
        return new RegExp('\\b' + functions.join('|'), 'g');
    }

    function getLanguagePropertiesRegExp() {
        var functions = $.map(PROPERTIES, function (item) {
            return '\\.' + item;
        });
        return new RegExp(functions.join('|') + '\\b', 'g');
    }

    function getCommentReplacer(index) {
        return '~~~C' + index + '~~~';
    }

    function getFormattedHtml(className, text) {
        var $span = $('<span>').html(text);

        ShadowUI.addClass($span, className);

        return $span.wrap('<p>')
            .parent()
            .html();
    }

    //CodeFormatter

    var CodeFormatter = this.exports = function () {
        this.commentsState = [];
        this.prevCommentsState = [];
        this.lineStartPositions = [];
        this.timeouts = [];
    };

    CodeFormatter.prototype._clearTimeouts = function () {
        while (this.timeouts.length) {
            window.clearTimeout(this.timeouts.pop());
        }
    };

    CodeFormatter.prototype._getLineFromTextPosition = function (position) {
        var i = 0;
        while (this.lineStartPositions[i + 1]) {
            if (position <= this.lineStartPositions[i + 1])
                return i;
            i++;
        }
        return i;
    };

    CodeFormatter.prototype._isCommentsStateChanged = function (newState) {
        var getComparableString = function (state) {
            return $.map(state,function (item) {
                return '' + item.start.line + ':' + item.start.position + '-' + item.end.line + ':' + item.end.position;
            }).join(' ');
        };

        return getComparableString(newState) !== getComparableString(this.commentsState);
    };

    CodeFormatter.prototype._isLineCommentsStateChanged = function (index) {
        var getComparableString = function (state) {
            var res = '';
            $.map(state, function (item) {
                res = res.concat(
                    typeof item.start === 'undefined' ? 'start' : item.start,
                    '-',
                    typeof item.end === 'undefined' ? 'end' : item.end
                );
            });
            return res;
        };

        return getComparableString(this._getLineCommentsState(index, true)) !== getComparableString(this._getLineCommentsState(index, false));
    };

    CodeFormatter.prototype._updateCommentsState = function (text) {
        var startCommentOrStringRegExp = /(?:\/\*)|(?:\/\/)|(?:'|")|(\\n)/gm,
            startBlockPosition = -1,
            endBlockPosition = -1,
            inComment = false,
            inString = false,

            startBlock = startCommentOrStringRegExp.exec(text),

            endCommentPosition = null,
            startLine = null,
            startPosition = null,
            endLine = null,
            endPosition = null,

            newState = [];

        var startStringBlock = function (quote) {
            var closedPosition = text.indexOf(quote, startCommentOrStringRegExp.lastIndex),
                lastLineSymbolPosition = text.indexOf('\n', startCommentOrStringRegExp.lastIndex) - 1;

            if (lastLineSymbolPosition < 0)
                lastLineSymbolPosition = text.length - 1;

            var endStringPosition = closedPosition < 0 ?
                lastLineSymbolPosition :
                Math.min(lastLineSymbolPosition, closedPosition);

            startBlockPosition = startCommentOrStringRegExp.lastIndex - 1;
            endBlockPosition = endStringPosition;
            inString = true;
        };

        while (startBlock) {
            var value = startBlock[0];
            if (inComment || inString) {
                if (startCommentOrStringRegExp.lastIndex - value.length <= endBlockPosition) {
                    if (startBlock[0] === '//')
                        startCommentOrStringRegExp.lastIndex--;
                    startBlock = startCommentOrStringRegExp.exec(text);
                    continue;
                }
                else {
                    inComment = false;
                    inString = false;
                }
            }

            switch (value) {
                case '/*':
                    var closePosition = text.indexOf('*/', startCommentOrStringRegExp.lastIndex);

                    endCommentPosition = closePosition < 0 ? text.length - 1 : closePosition;

                    startBlockPosition = startCommentOrStringRegExp.lastIndex - '/*'.length;
                    endBlockPosition = endCommentPosition + (closePosition < 0 ? 0 : 1);

                    startLine = this._getLineFromTextPosition(startBlockPosition);
                    startPosition = startBlockPosition - this.lineStartPositions[startLine];
                    endLine = this._getLineFromTextPosition(endBlockPosition);
                    endPosition = endBlockPosition - this.lineStartPositions[endLine];

                    newState.push({start: { line: startLine, position: startPosition, multiline: true },
                        end: {line: endLine, position: endPosition}});
                    inComment = true;
                    break;

                case '//':
                    endCommentPosition = text.indexOf('\n', startCommentOrStringRegExp.lastIndex - 1);

                    if (endCommentPosition < 0)
                        endCommentPosition = text.length - 1;

                    startBlockPosition = startCommentOrStringRegExp.lastIndex - '//'.length;
                    endBlockPosition = endCommentPosition;

                    startLine = this._getLineFromTextPosition(startBlockPosition);
                    startPosition = startBlockPosition - this.lineStartPositions[startLine];
                    endLine = this._getLineFromTextPosition(endBlockPosition);
                    endPosition = endBlockPosition - this.lineStartPositions[endLine];

                    if (text[endCommentPosition] === '\n')
                        endPosition--;

                    newState.push({start: {line: startLine, position: startPosition},
                        end: {line: endLine, position: endPosition}});
                    inComment = true;
                    break;

                case '\'':
                    startStringBlock('\'');
                    break;
                case '\"':
                    startStringBlock('\"');
                    break;
            }
            startBlock = startCommentOrStringRegExp.exec(text);
        }

        if (this._isCommentsStateChanged(newState)) {
            this.prevCommentsState = this.commentsState;
            this.commentsState = newState;
            return true;
        }
        return false;
    };

    CodeFormatter.prototype._getLineCommentsState = function (lineIndex, prev) {
        var state = [];
        $.each((prev ? this.prevCommentsState : this.commentsState), function (index, item) {
            if (item.start.line === lineIndex) {
                if (item.end.line === lineIndex)
                    state.push({start: item.start.position, end: item.end.position});
                else
                    state.push({start: item.start.position});
            }
            else if (item.end.line === lineIndex) {
                state.push({end: item.end.position});
            }
            else if (item.start.line < lineIndex && item.end.line > lineIndex)
                state.push({ start: 0 });
        });
        return state;
    };

    CodeFormatter.prototype._highlightLine = function (text, index) {
        var newText = text.replace(/[<>&]/g, function (m) {
                return SAFE[m];
            }),

            commentsState = this._getLineCommentsState(index),
            length = newText.length;

        commentsState = $.map(commentsState, function (item) {
            return { start: item.start || 0, end: item.end || length - 1 };
        });

        var strings = [],
            shift = 0;

        var replacedComments = $.map(commentsState, function (comment, index) {
            var start = comment.start + shift,
                end = comment.end + shift + 1,
                length = end - start,
                replacer = getCommentReplacer(index),
                value = newText.substring(start, end);
            shift += replacer.length - length;
            newText = newText.substring(0, start) + replacer + newText.substring(end, newText.length);
            return { replacer: replacer, value: value };
        });
        newText = newText
            //strings
            .replace(/(^|[^\\])((?:'(?:\\'|[^'])*(?:'|$))|(?:"(?:\\"|[^"])*(?:"|$)))/g, function (m, f, s) {
                var l = strings.length;
                strings.push(s);
                return f + '~~~S' + l + '~~~';
            })
            //operators and numbers
            .replace(createRegExpFromArray(OPERATORS, true), function (str) {
                if (str[0] >= '0' && str[0] <= '9')
                    return getFormattedHtml(NUMBER_CLASS, str);
                return getFormattedHtml(OPERATOR_CLASS, str);
            })
            //object in 'someObject.someFuction = function() ...' construction
            .replace(/(\S+)(\.\S+\s?(?:<[^>]+>)?=(?:<[^>]+>)?\s?function\s?\([^\)]*\))/g, function (str, p1, p2) {
                return getFormattedHtml(OBJECT_CLASS, p1) + p2;
            })
            //object in 'someObject.prototype.someFunction = ...' construction
            .replace(/(\S+)(\.prototype\.\S+\s?(?:<[^>]+>)?=(?:<[^>]+>)?)/g, function (str, p1, p2) {
                return getFormattedHtml(OBJECT_CLASS, p1) + p2;
            })
            //arguments in function
            .replace(/(\bfunction(?:\s?|(?:\s[^\(]+))\()([^\)]+)(\))/g, function (str, p1, p2, p3) {
                return p1 + getFormattedHtml(ARGUMENT_CLASS, p2) + p3;
            })
            //function name in 'function someName(args)' construction
            .replace(/(\bfunction\s)(\w+)(\([^\)]*\))/g, function (str, p1, p2, p3) {
                return p1 + getFormattedHtml(FUNCTION_NAME_CLASS, p2) + p3;
            })
            //function name in 'someName = function (args)...' construction
            .replace(/(\s|^|\.)(\w+)(\s?(?:(?:\s?(?:<[^>]+>)?=(?:<[^>]+>)?\s?)|=)\s?function\s?\([^\)]*\))/g, function (str, p1, p2, p3) {
                return p1 + getFormattedHtml(FUNCTION_NAME_CLASS, p2) + p3;
            })
            .replace(createRegExpFromArray(KEYWORDS), function (str) {
                return getFormattedHtml(KEYWORD_CLASS, str);
            })
            .replace(createRegExpFromArray(LITERALS), function (str) {
                return getFormattedHtml(LITERAL_CLASS, str);
            })
            .replace(getLanguageFunctionsRegExp(), function (str) {
                return getFormattedHtml(FUNCTION_CLASS, str);
            })
            .replace(getLanguagePropertiesRegExp(), function (str) {
                return '.' + getFormattedHtml(PROPERTY_CLASS, str.substring(1));
            })
            .replace(createRegExpFromArray(OBJECTS), function (str) {
                return getFormattedHtml(LANGUAGE_CLASS, str);
            })
            .replace(/~~~(S)(\d+)~~~/g, function (m, t, i) {
                return getFormattedHtml(STRING_CLASS, strings[i]);
            });

        $.each(replacedComments, function (index, comment) {
            var value = getFormattedHtml(COMMENT_CLASS, comment.value);
            newText = newText.replace(comment.replacer, value);
        });

        return newText;
    };

    CodeFormatter.prototype._highlightLines = function ($lines, changedLines, currentLine, checkAll) {
        var formatter = this,
            checkingLines = null;

        if (changedLines === undefined || changedLines.length === 0 || checkAll) {
            formatter._clearTimeouts();
            checkingLines = {start: 0, end: $lines.length - 1 };
        }
        else if (changedLines.length === 1)
            checkingLines = {start: changedLines[0], end: changedLines[0]};
        else {
            var sortedIndexes = changedLines.sort();
            checkingLines = { start: sortedIndexes[0], end: sortedIndexes[changedLines.length - 1] };
        }

        var highLightChunksRecursive = function (array, startIndex, backward) {
            var chunkSize = Math.min(CHUNK_SIZE, array.length),

                chunk = backward ? array.splice(array.length - chunkSize, chunkSize) : array.splice(0, chunkSize),

                text = $.map(chunk, function (item) {
                    return item.text();
                });

            $.each(chunk, function (index, item) {
                var curIndex = backward ? startIndex - chunkSize + index + 1 : startIndex + index;

                if (!changedLines || !changedLines.length || $.inArray(curIndex, changedLines) > -1 || formatter._isLineCommentsStateChanged(curIndex)) {
                    var html = formatter._highlightLine(text[index], curIndex),
                        $span = item.children('span:first');
                    $span.html(html);
                    if (Util.isIE)
                        Util.setUnselectableAttributeRecursive($span[0]);
                }
            });

            if (array.length) {
                formatter.timeouts.push(NativeMethods.setTimeout.call(window, function () {
                    highLightChunksRecursive(array, startIndex + chunkSize * (backward ? -1 : 1), backward);
                }, 0));
            }
        };

        var $checkingLines = $lines.slice(checkingLines.start, checkingLines.end + 1);

        if ($checkingLines.length > CHUNK_SIZE * 2 && currentLine > checkingLines.start && currentLine < checkingLines.end) {
            highLightChunksRecursive($checkingLines.slice(checkingLines.start, currentLine), currentLine - 1, true);
            highLightChunksRecursive($checkingLines.slice(currentLine), currentLine);
        }
        else
            highLightChunksRecursive($checkingLines, checkingLines.start);
    };

    //API
    CodeFormatter.prototype.highlight = function ($lines, changedLines, currentLine) {
        var formatter = this;
        formatter.lineStartPositions = [0];
        var curPosition = 0,
            text = $.map($lines,function ($item) {
                var itemText = $item.text();
                curPosition += itemText.length + 1;
                formatter.lineStartPositions.push(curPosition);
                return itemText;
            }).join('\n');
        var commentsStateChanged = formatter._updateCommentsState(text);
        formatter._highlightLines($lines, changedLines, currentLine, commentsStateChanged);
    };
});