TestCafeClient.define('UI.RecorderWidgets.Combobox', function () {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,
        ShadowUI = Hammerhead.ShadowUI,
        TextSelection = Hammerhead.TextSelection;

    //Const
    var COMBOBOX_CLASS = 'combobox',
        DROPDOWN_BUTTON_CLASS = 'dropdown-button',
        DROPDOWN_IMAGE_CLASS = 'dropdown-img',
        DROPDOWN_BUTTON_PRESSED_CLASS = 'pressed',
        ITEM_HOVERED_CLASS = 'hovered',
        ITEM_LIST_CLASS = 'items',
        ITEM_CLASS = 'item',

        ITEM_LIST_MARGIN = 3,

    //Events
        VALUE_CHANGED_EVENT = 'valueChanged';

    var Combobox = this.exports = function ($container, items, value, inputId) {
        var combobox = this;

        this.$combobox = null;
        this.$dropdownButton = null;
        this.$input = null;
        this.$itemList = null;
        this.items = [];
        this.itemListCreated = null;

        this.eventEmitter = new Util.EventEmitter();
        this.events = {
            on: function (ev, listener) {
                combobox.eventEmitter.on(ev, listener);
            }
        };

        this._createCombobox($container, items, value, inputId);
    };

    //Markup
    Combobox.prototype._createCombobox = function ($container, items, value, inputId) {
        this.$combobox = $('<div></div>')
            .appendTo($container);
        ShadowUI.addClass(this.$combobox, COMBOBOX_CLASS);

        this.items = items;

        this.$input = $('<input type="text">')
            .appendTo(this.$combobox)
            .attr('id', inputId)
            .attr('value', value);

        this.$dropdownButton = $('<div></div>').appendTo(this.$combobox);
        ShadowUI.addClass(this.$dropdownButton, DROPDOWN_BUTTON_CLASS);

        var $dropdownImg = $('<div></div>').appendTo(this.$dropdownButton);
        ShadowUI.addClass($dropdownImg, DROPDOWN_IMAGE_CLASS);
        this._init();
    };

    Combobox.prototype._createItemList = function () {
        var combobox = this;

        combobox.itemListCreated = true;

        this.$itemList = $('<div></div>').appendTo(this.$combobox);
        ShadowUI.addClass(this.$itemList, ITEM_LIST_CLASS);

        $.each(this.items, function (index, item) {
            var $item = $('<div></div>')
                .text(item)
                .appendTo(combobox.$itemList);
            ShadowUI.addClass($item, ITEM_CLASS);

            if (index === 0)
                $item.css('margin-top', '6px');

            $item.mouseenter(function () {
                ShadowUI.removeClass(combobox._getHoveredItem(), ITEM_HOVERED_CLASS);
                ShadowUI.addClass($item, ITEM_HOVERED_CLASS);
            });

            $item.click(function () {
                combobox._setValue($item.text());
                combobox._hideItemList();
            });
        });

        this.$itemList.css({
            width: this.$combobox.width(),
            display: 'none'
        });

        ShadowUI.addClass(combobox.$dropdownButton, DROPDOWN_BUTTON_PRESSED_CLASS);

        this._updateItemListPosition();
    };

//Behavior
    Combobox.prototype._onArrowPressed = function (offset) {
        var combobox = this,
            $hoveredItem = combobox._getHoveredItem(),
            hoveredItemValue = $hoveredItem.text(),
            hoveredItemIndex = $.inArray(hoveredItemValue, combobox.items),
            activeValue = combobox._getValue(),
            activeItemIndex = $.inArray(activeValue, combobox.items),
            isStepListVisible = (combobox.itemListCreated && combobox.$itemList.is(':visible')),
            currentItemIndex = isStepListVisible ? hoveredItemIndex : activeItemIndex;

        if ((offset === 1 && (currentItemIndex !== -1 && currentItemIndex !== combobox.items.length - 1)) ||
            (offset === -1 && (currentItemIndex !== -1 && currentItemIndex !== 0))) {
            if (isStepListVisible) {
                ShadowUI.removeClass(combobox._getHoveredItem(), ITEM_HOVERED_CLASS);
                ShadowUI.addClass(ShadowUI.select('.' + COMBOBOX_CLASS + '>.' + ITEM_LIST_CLASS + '>.' + ITEM_CLASS + ':' + 'visible').eq(currentItemIndex + offset), ITEM_HOVERED_CLASS);
            }
            combobox._setValue(combobox.items[currentItemIndex + offset]);
        }
    };

    Combobox.prototype._init = function () {
        var combobox = this;

        var onMouseDown = function (e) {
            var inputValueLength = combobox.$input[0].value.length;

            if ((e.button === Util.BUTTON.LEFT || Util.hasTouchEvents) && !combobox.itemListCreated)
                combobox._createItemList();

            if (e.button === Util.BUTTON.LEFT || Util.hasTouchEvents) {
                if (!combobox.$itemList.is(':visible'))
                    combobox._showItemList();
                else
                    combobox._hideItemList();
            }

            NativeMethods.focus.call(combobox.$input[0]);
            TextSelection.select(combobox.$input[0], inputValueLength, inputValueLength);

            Util.preventDefault(e);
        };

        ShadowUI.bind(this.$input, 'mousedown', onMouseDown);
        ShadowUI.bind(this.$dropdownButton, 'mousedown', onMouseDown);

        ShadowUI.bind(this.$input, 'focus', function () {
            var inputValueLength = combobox.$input[0].value.length;

            NativeMethods.focus.call(combobox.$input[0]);
            TextSelection.select(combobox.$input[0], inputValueLength, inputValueLength);
        });

        ShadowUI.bind(this.$input, 'keydown', function (e) {
            var keyCode = e.keyCode;
            if (Util.isCharByKeyCode(keyCode, Util.isShadowUIElement) || /^(backspace|delete)$/.test(Util.KEYS_MAPS.REVERSED_SPECIAL_KEYS[keyCode])) {
                Util.preventDefault(e);
                return;
            }

            if (/^(down|right)$/.test(Util.KEYS_MAPS.REVERSED_SPECIAL_KEYS[keyCode])) {
                combobox._onArrowPressed(1);
                Util.preventDefault(e);
            }
            else if (/^(up|left)$/.test(Util.KEYS_MAPS.REVERSED_SPECIAL_KEYS[keyCode])) {
                combobox._onArrowPressed(-1);
                Util.preventDefault(e);
            }
            else if (/^(enter|esc)$/.test(Util.KEYS_MAPS.REVERSED_SPECIAL_KEYS[keyCode]) &&
                     (combobox.$itemList && combobox.$itemList.css('display') !== 'none')) {
                combobox._hideItemList();
                Util.preventDefault(e);
            }
            else if (combobox.itemListCreated)
                combobox._hideItemList();
        });
    };

    Combobox.prototype._setValue = function (value) {
        if (this.$input.attr('value') !== value) {
            this.$input.attr('value', value);
            this.eventEmitter.emit(VALUE_CHANGED_EVENT, { value: value });
        }
    };

    Combobox.prototype._getValue = function () {
        return this.$input.attr('value');
    };

    Combobox.prototype._updateItemListPosition = function () {
        var $window = $(window),
            comboboxPosition = Util.getOffsetPosition(this.$combobox[0]),
            itemListHeight = this.$itemList.height(),
            itemListTopPosition = comboboxPosition.top - $window.scrollTop() + this.$combobox.height();

        if (itemListTopPosition + itemListHeight > $window.scrollTop() + $window.height()) {
            var topPositionAboveCombobox = comboboxPosition.top - ITEM_LIST_MARGIN - itemListHeight;
            if (topPositionAboveCombobox >= $window.scrollTop())
                itemListTopPosition = topPositionAboveCombobox;
        }
        this.$itemList.css({
            left: comboboxPosition.left - $window.scrollLeft(),
            top: itemListTopPosition
        });
    };

    Combobox.prototype._showItemList = function () {
        var combobox = this;

        var onDocumentClick = function (e) {
            var target = e.target || e.srcElement;
            if (!combobox.$combobox.has(target).length) {
                combobox._hideItemList();
                ShadowUI.unbind($(document), 'mousedown', onDocumentClick);
                ShadowUI.unbind($(document), 'scroll', onDocumentScroll);
                ShadowUI.unbind($(window), 'resize', onResize);
            }
        };

        var onResize = function () {
            if (combobox.itemListCreated && combobox.$itemList.is(':visible')) {
                combobox._hideItemList();
                ShadowUI.unbind($(document), 'mousedown', onDocumentClick);
                ShadowUI.unbind($(document), 'scroll', onDocumentScroll);
            }
            ShadowUI.unbind($(window), 'resize', onResize);
        };

        var onDocumentScroll = function () {
            combobox._hideItemList();
            ShadowUI.unbind($(document), 'mousedown', onDocumentClick);
            ShadowUI.unbind($(document), 'scroll', onDocumentScroll);
            ShadowUI.unbind($(window), 'resize', onDocumentScroll);
        };

        this._updateItemListPosition();
        this.$itemList.css('display', '');

        var $activeItem = this.$itemList.children().filter(function () {
            return $(this).text() === combobox._getValue();
        });
        ShadowUI.addClass($activeItem, ITEM_HOVERED_CLASS);

        ShadowUI.addClass(combobox.$dropdownButton, DROPDOWN_BUTTON_PRESSED_CLASS);
        ShadowUI.bind($(document), 'mousedown', onDocumentClick);
        ShadowUI.bind($(document), 'scroll', onDocumentScroll);
        ShadowUI.bind($(window), 'resize', onResize);
        return false;
    };

    Combobox.prototype._hideItemList = function () {
        ShadowUI.removeClass(this._getHoveredItem(), ITEM_HOVERED_CLASS);
        this.$itemList.css('display', 'none');
        ShadowUI.removeClass(this.$dropdownButton, DROPDOWN_BUTTON_PRESSED_CLASS);
        return false;
    };

    Combobox.prototype._getHoveredItem = function () {
        return ShadowUI.select('.' + COMBOBOX_CLASS + '>.' + ITEM_LIST_CLASS + '>.' + ITEM_CLASS + '.' + ITEM_HOVERED_CLASS);
    };
});