TestCafeClient.define('UI.RecorderWidgets.ElementsMarker', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        MessageSandbox = Hammerhead.MessageSandbox,
        NativeMethods = Hammerhead.NativeMethods,
        RecorderUtil = require('Recorder.Util'),
        JavascriptExecutor = require('Base.JavascriptExecutor');

    //Consts
    var POINTERS_BASE_FRAME_WIDTH = 0,
        ELEMENT_FRAME_WIDTH = 4,
        ELEMENT_FRAME_PADDING = 3,
        SHADOW_OFFSET = 4,
        POINTER_START_WIDTH = 20,

        DRAWING_TIME_INTERVAL = 17,

        ELEMENTS_MARKER_CLASS = RecorderUtil.ELEMENTS_MARKER_CLASS,
    //following class names are used in tests for svg elements identifying
        BACKGROUND_CLASS = 'background',
        POINTERS_BASE_FRAME_CLASS = 'pointersBaseFrame',
        POINTERS_BASE_FRAME_SHADOW_CLASS = 'pointersBaseFrameShadow',
        ELEMENT_FRAME_CLASS = 'elementFrame',
        ELEMENT_FRAME_SHADOW_CLASS = 'elementFrameShadow',
        POINTER_CLASS = 'pointer',

        SVG_OPEN_TAG = '<svg xmlns="http://www.w3.org/2000/svg" version="1.1">',
        SVG_CLOSE_TAG = '</svg>';

    //Events
    exports.ELEMENT_RECTANGLES_COUNT_CHANGED = 'elementRectanglesCountChanged';

    //Global
    var $elementsMarker = null,
        $svgElement = null,
        elementsMarkerIntervalId = null,
        forceUpdateMarking = false,
        savedBaseRectangle = null,
        savedElementsRectangles = null,
        eventEmitter = null;

    //Rectangle
    var Rectangle = function (left, top, width, height) {
        this.top = top;
        this.left = left;
        this.width = width;
        this.height = height;
        this.right = left + width;
        this.bottom = top + height;
    };

    Rectangle.prototype.intersectsWith = function (rectangle) {
        return (rectangle.left < this.right && this.left < rectangle.right && rectangle.top < this.bottom && this.top < rectangle.bottom);
    };

    Rectangle.prototype.getIntersectionPoints = function (figure) {
        //figure may be a rectangle or a line
        var points = [];

        if (figure instanceof Rectangle) {
            if (figure.top < this.top && figure.bottom > this.top) {
                if (figure.left > this.left && figure.left < this.right)
                    points.push({left: figure.left, top: this.top, isTop: true, isIntersection: true});

                if (figure.right > this.left && figure.right < this.right)
                    points.push({left: figure.right, top: this.top, isTop: true, isIntersection: true});
            }

            if (figure.top < this.bottom && figure.bottom > this.bottom) {
                if (figure.left > this.left && figure.left < this.right)
                    points.push({left: figure.left, top: this.bottom, isBottom: true, isIntersection: true});

                if (figure.right > this.left && figure.right < this.right)
                    points.push({left: figure.right, top: this.bottom, isBottom: true, isIntersection: true});
            }

            if (figure.left < this.left && figure.right > this.left) {
                if (figure.top > this.top && figure.top < this.bottom)
                    points.push({left: this.left, top: figure.top, isLeft: true, isIntersection: true});

                if (figure.bottom > this.top && figure.bottom < this.bottom)
                    points.push({left: this.left, top: figure.bottom, isLeft: true, isIntersection: true});
            }

            if (figure.left < this.right && figure.right > this.right) {
                if (figure.top > this.top && figure.top < this.bottom)
                    points.push({left: this.right, top: figure.top, isRight: true, isIntersection: true});

                if (figure.bottom > this.top && figure.bottom < this.bottom)
                    points.push({left: this.right, top: figure.bottom, isRight: true, isIntersection: true});
            }
        }
        else if (figure.start && figure.end) {
            var start = figure.start,
                end = figure.end,
            //here we find the points of infinite lines intersection
                leftSideIntersectionTop = start.top + ((end.top - start.top) * (this.left - start.left)) / (end.left - start.left),
                rightSideIntersectionTop = start.top + ((end.top - start.top) * (this.right - start.left)) / (end.left - start.left),
                topSideIntersectionLeft = start.left + ((end.left - start.left) * (this.top - start.top)) / (end.top - start.top),
                bottomSideIntersectionLeft = start.left + ((end.left - start.left) * (this.bottom - start.top)) / (end.top - start.top);

            //and here we determine whether the points lay within the bounds of the figures
            if (leftSideIntersectionTop <= this.bottom && leftSideIntersectionTop >= this.top &&
                leftSideIntersectionTop <= Math.max(start.top, end.top) && leftSideIntersectionTop >= Math.min(start.top, end.top) &&
                this.left > Math.min(start.left, end.left) && this.left < Math.max(start.left, end.left)) {
                points.push({left: this.left, top: leftSideIntersectionTop, isLeft: true, isIntersection: true});
            }

            if (rightSideIntersectionTop <= this.bottom && rightSideIntersectionTop >= this.top &&
                rightSideIntersectionTop <= Math.max(start.top, end.top) && rightSideIntersectionTop >= Math.min(start.top, end.top) &&
                this.right > Math.min(start.left, end.left) && this.right < Math.max(start.left, end.left)) {
                points.push({left: this.right, top: rightSideIntersectionTop, isRight: true, isIntersection: true});
            }

            if (topSideIntersectionLeft <= this.right && topSideIntersectionLeft >= this.left &&
                topSideIntersectionLeft <= Math.max(start.left, end.left) && topSideIntersectionLeft >= Math.min(start.left, end.left) &&
                this.top > Math.min(start.top, end.top) && this.top < Math.max(start.top, end.top)) {
                points.push({left: topSideIntersectionLeft, top: this.top, isTop: true, isIntersection: true});
            }

            if (bottomSideIntersectionLeft <= this.right && bottomSideIntersectionLeft >= this.left &&
                bottomSideIntersectionLeft <= Math.max(start.left, end.left) && bottomSideIntersectionLeft >= Math.min(start.left, end.left) &&
                this.bottom > Math.min(start.top, end.top) && this.bottom < Math.max(start.top, end.top)) {
                points.push({left: bottomSideIntersectionLeft, top: this.bottom, isBottom: true, isIntersection: true});
            }
        }
        return points;
    };

    Rectangle.prototype.contains = function (figure, excludeBorders) {
        //figure may be a rectangle or a point
        if (figure instanceof Rectangle) {
            if (excludeBorders)
                return this.left < figure.left && figure.right < this.right && this.top < figure.top && figure.bottom < this.bottom;
            else
                return this.left <= figure.left && figure.right <= this.right && this.top <= figure.top && figure.bottom <= this.bottom;
        }
        else {
            if (excludeBorders)
                return this.left < figure.left && figure.left < this.right && this.top < figure.top && figure.top < this.bottom;
            else
                return this.left <= figure.left && figure.left <= this.right && this.top <= figure.top && figure.top <= this.bottom;
        }
    };

    Rectangle.prototype.equals = function (rectangle) {
        return rectangle && this.left === rectangle.left && this.top === rectangle.top &&
            this.width === rectangle.width && this.height === rectangle.height;
    };

    Rectangle.prototype.getCenter = function () {
        return {
            left: this.left + this.width / 2,
            top: this.top + this.height / 2
        };
    };

    function getRectangle($element, padding) {
        var rect = Util.getElementRectangleForMarking($element[0], padding, ELEMENT_FRAME_WIDTH);

        return new Rectangle(rect.left, rect.top, rect.width, rect.height);
    }

    function mergeArraysUnique() {
        var merged = [];

        $.each(arguments, function (index, array) {
            $.each(array, function (index, element) {
                if ($.inArray(element, merged) < 0)
                    merged.push(element);
            });
        });

        return merged;
    }

    function getFirstIndexOfArrayContainingJointElements(arrays, currentArrayIndex) {
        for (var i = currentArrayIndex + 1; i < arrays.length; i++) {
            for (var j = 0; j < arrays[currentArrayIndex].length; j++)
                if ($.inArray(arrays[currentArrayIndex][j], arrays[i]) >= 0) {
                    return i;
                }
        }

        return -1;
    }

    function mergeArraysContainingJointElements(arraysForMerging) {
        var merged = arraysForMerging.concat();

        for (var i = 0; i < merged.length; i++) {
            var indexOfArrayContainingJointElements = getFirstIndexOfArrayContainingJointElements(merged, i);

            if (indexOfArrayContainingJointElements > 0) {
                merged[i] = mergeArraysUnique(merged[i], merged[indexOfArrayContainingJointElements]);
                merged.splice(indexOfArrayContainingJointElements, 1);
                i--;
            }
        }

        return merged;
    }

    function getIntersectingRectanglesGroups(rectangles) {
        var groups = [];

        for (var i = 0; i < rectangles.length; i++) {
            var group = [rectangles[i]];

            for (var j = i + 1; j < rectangles.length; j++) {
                if (rectangles[i].intersectsWith(rectangles[j]))
                    group.push(rectangles[j]);
            }

            if (group.length > 1)
                groups.push(group);
        }

        return mergeArraysContainingJointElements(groups);
    }

    function getRectanglesIntersectionPoints(rectangles) {
        var intersectionPoints = [];

        for (var i = 0; i < rectangles.length; i++)
            for (var j = i + 1; j < rectangles.length; j++)
                intersectionPoints = intersectionPoints.concat(rectangles[i].getIntersectionPoints(rectangles[j]));

        for (var n = 0; n < intersectionPoints.length - 1; n++)
            for (var k = n + 1; k < intersectionPoints.length; k++) {
                if (intersectionPoints[n].left === intersectionPoints[k].left && intersectionPoints[n].right === intersectionPoints[k].left)
                    intersectionPoints.splice(k--, 1);
            }

        return intersectionPoints;
    }

    function isCornerNotLyingInsideRectangles(corner, cornerRectangle, rectangles) {
        for (var i = 0; i < rectangles.length; i++)
            if (rectangles[i] !== cornerRectangle) {

                if (rectangles[i].contains(corner, true))
                    return false;

                else if (rectangles[i].contains(corner)) {
                    if (corner.isTop && corner.isLeft) {
                        if (rectangles[i].top === corner.top ? rectangles[i].left !== corner.left : rectangles[i].left === corner.left)
                            return false;
                    }
                    else if (corner.isTop) {
                        if (rectangles[i].top === corner.top ? rectangles[i].right !== corner.left : rectangles[i].right === corner.left)
                            return false;
                    }
                    else if (corner.isLeft) {
                        if (rectangles[i].bottom === corner.top ? rectangles[i].left !== corner.left : rectangles[i].left === corner.left)
                            return false;
                    }
                    else {
                        if (rectangles[i].bottom === corner.top ? rectangles[i].right !== corner.left : rectangles[i].right === corner.left)
                            return false;
                    }
                }
            }

        return true;
    }

    function isCornerInArray(corner, corners) {
        for (var i = 0; i < corners.length; i++) {
            if (corners[i].left === corner.left && corners[i].top === corner.top && corners[i].isTop === corner.isTop && corners[i].isLeft === corner.isLeft)
                return true;
        }

        return false;
    }

    function getRectanglesCorners(rectangles) {
        var corners = [];

        for (var i = 0; i < rectangles.length; i++) {
            var topLeftCorner = {left: rectangles[i].left, top: rectangles[i].top, isLeft: true, isTop: true};
            var topRightCorner = {left: rectangles[i].right, top: rectangles[i].top, isTop: true};
            var bottomLeftCorner = {left: rectangles[i].left, top: rectangles[i].bottom, isLeft: true};
            var bottomRightCorner = {left: rectangles[i].right, top: rectangles[i].bottom};

            if (isCornerNotLyingInsideRectangles(topLeftCorner, rectangles[i], rectangles) && !isCornerInArray(topLeftCorner, corners))
                corners.push(topLeftCorner);

            if (isCornerNotLyingInsideRectangles(topRightCorner, rectangles[i], rectangles) && !isCornerInArray(topRightCorner, corners))
                corners.push(topRightCorner);

            if (isCornerNotLyingInsideRectangles(bottomLeftCorner, rectangles[i], rectangles) && !isCornerInArray(bottomLeftCorner, corners))
                corners.push(bottomLeftCorner);

            if (isCornerNotLyingInsideRectangles(bottomRightCorner, rectangles[i], rectangles) && !isCornerInArray(bottomRightCorner, corners))
                corners.push(bottomRightCorner);
        }

        return corners;
    }

    function getTopLeftPoint(points) {
        var topLeftPoint = points[0];
        for (var i = 1; i < points.length; i++) {
            if (points[i].top < topLeftPoint.top)
                topLeftPoint = points[i];

            else if (points[i].top === topLeftPoint.top && points[i].left < topLeftPoint.left)
                topLeftPoint = points[i];
        }

        return topLeftPoint;
    }

    function getClosestPointOnVector(vector, points) {
        var closestPoint = null;

        if (vector.directionRight)
            $.each(points, function (index, point) {
                if (point.top === vector.top)
                    if (point.left > vector.left && (!closestPoint || point.left < closestPoint.left))
                        closestPoint = point;
            });
        else if (vector.directionDown)
            $.each(points, function (index, point) {
                if (point.left === vector.left)
                    if (point.top > vector.top && (!closestPoint || point.top < closestPoint.top))
                        closestPoint = point;
            });
        else if (vector.directionLeft)
            $.each(points, function (index, point) {
                if (point.top === vector.top)
                    if (point.left < vector.left && (!closestPoint || point.left > closestPoint.left))
                        closestPoint = point;
            });
        else if (vector.directionUp)
            $.each(points, function (index, point) {
                if (point.left === vector.left)
                    if (point.top < vector.top && (!closestPoint || point.top > closestPoint.top))
                        closestPoint = point;
            });

        return closestPoint;
    }

    function getIntersectedRectanglesBackgroundHole(intersectedRectangles) {
        var points = getRectanglesIntersectionPoints(intersectedRectangles).concat(getRectanglesCorners(intersectedRectangles));
        var currentVector = getTopLeftPoint(points);

        points.splice(points.indexOf(currentVector), 1);
        currentVector.directionRight = true;

        var vectors = [currentVector];

        while (points.length) {
            var closestPoint = getClosestPointOnVector(currentVector, points);

            if (closestPoint) {
                closestPoint.directionRight = closestPoint.isIntersection ? currentVector.directionDown : currentVector.directionUp;
                closestPoint.directionLeft = closestPoint.isIntersection ? currentVector.directionUp : currentVector.directionDown;
                closestPoint.directionUp = closestPoint.isIntersection ? currentVector.directionRight : currentVector.directionLeft;
                closestPoint.directionDown = closestPoint.isIntersection ? currentVector.directionLeft : currentVector.directionRight;

                currentVector = closestPoint;
                points.splice(points.indexOf(closestPoint), 1);
                vectors.push(currentVector);
            }
            else
                break;
        }

        var figure = ' M ' + vectors[0].left + ',' + vectors[0].top;

        for (var i = 1; i < vectors.length; i++) {
            if (vectors[i - 1].directionLeft || vectors[i - 1].directionRight)
                figure += ' H ' + vectors[i].left;
            else
                figure += ' V ' + vectors[i].top;
        }

        figure += ' z ';

        return figure;
    }

    //Pointers coordinates calculating
    function getPointerStart(rectangle, baseRectangle) {
        var left = null,
            top = null,
            isHorizontal = null,

            intersectionPoints = baseRectangle.getIntersectionPoints({
                start: baseRectangle.getCenter(),
                end: rectangle.getCenter()
            });

        if (intersectionPoints.length) {
            var startPoint = intersectionPoints[0];
            if (startPoint.isLeft || startPoint.isRight) {
                left = startPoint.left;
                top = Math.min(
                    Math.max(startPoint.top, baseRectangle.top + POINTER_START_WIDTH / 2),
                    baseRectangle.bottom - POINTER_START_WIDTH / 2);
            }
            else if (startPoint.isTop || startPoint.isBottom) {
                top = startPoint.top;
                left = Math.min(
                    Math.max(startPoint.left, baseRectangle.left + POINTER_START_WIDTH / 2),
                    baseRectangle.right - POINTER_START_WIDTH / 2);
                isHorizontal = true;
            }
        }

        return {
            left: left,
            top: top,
            isHorizontal: isHorizontal
        };
    }

    function getPointerEnd(rectangle, baseRectangle) {
        var left = null,
            top = null,

            intersectionPoints = rectangle.getIntersectionPoints({
                start: baseRectangle.getCenter(),
                end: rectangle.getCenter()
            });

        if (intersectionPoints.length) {
            var endPoint = intersectionPoints[0];

            left = endPoint.left;
            top = endPoint.top;

            if (endPoint.isLeft)
                left -= ELEMENT_FRAME_WIDTH / 2;
            else if (endPoint.isRight)
                left += ELEMENT_FRAME_WIDTH / 2;
            else if (endPoint.isTop)
                top -= ELEMENT_FRAME_WIDTH / 2;
            else if (endPoint.isBottom)
                top += ELEMENT_FRAME_WIDTH / 2;
        }

        return {
            left: left,
            top: top
        };
    }

    function getPointerCoordinates(rectangle, baseRectangle) {
        var pointerStart = getPointerStart(rectangle, baseRectangle),
            pointerEnd = getPointerEnd(rectangle, baseRectangle);

        return {
            pointerStart: pointerStart,
            pointerEnd: pointerEnd
        };
    }

    //Markup
    function createElementsMarker() {
        $elementsMarker = $('<div></div>').appendTo(ShadowUI.getRoot());
        ShadowUI.addClass($elementsMarker, ELEMENTS_MARKER_CLASS);
        return $elementsMarker;
    }

    //SVG generation
    function createSvgString(baseRectangle, elementsRectangles) {
        var pointersCoordinates = [];

        $.each(elementsRectangles, function (index, rectangle) {
            if (!baseRectangle.intersectsWith(rectangle)) {
                pointersCoordinates.push(getPointerCoordinates(rectangle, baseRectangle));
            }
        });

        var svgStrings = [SVG_OPEN_TAG];

        svgStrings.push(getBackgroundSvgString(elementsRectangles));
        svgStrings.push(getFrameShadowSvgString(baseRectangle, POINTERS_BASE_FRAME_SHADOW_CLASS));

        $.each(elementsRectangles, function (index) {
            svgStrings.push(getFrameShadowSvgString(elementsRectangles[index], ELEMENT_FRAME_SHADOW_CLASS));
        });

        svgStrings.push(getFrameSvgString(baseRectangle, POINTERS_BASE_FRAME_CLASS));

        $.each(elementsRectangles, function (index) {
            svgStrings.push(getFrameSvgString(elementsRectangles[index], ELEMENT_FRAME_CLASS));
        });

        $.each(pointersCoordinates, function (index) {
            svgStrings.push(getPointerSvgString(pointersCoordinates[index]));
        });

        svgStrings.push(SVG_CLOSE_TAG);

        return svgStrings.join('');
    }

    function getBackgroundSvgString(rectangles) {
        var standaloneRectangles = [],
            intersectionGroups = getIntersectingRectanglesGroups(rectangles);

        for (var i = 0; i < rectangles.length; i++) {
            var intersected = false;

            for (var j = 0; j < intersectionGroups.length; j++) {
                if ($.inArray(rectangles[i], intersectionGroups[j]) >= 0) {
                    intersected = true;
                    break;
                }
            }
            if (!intersected)
                standaloneRectangles.push(rectangles[i]);
        }

        var background = ['<path class="' + ShadowUI.patchClassNames(BACKGROUND_CLASS) + '" d="M 0,0 H ', Util.getDocumentElementWidth(), ' V ', Util.getDocumentElementHeight(), ' H 0 z '];

        $.each(standaloneRectangles, function (index, rectangle) {
            background.push(getRectangleBackgroundHoleSvgString(rectangle));
        });
        $.each(intersectionGroups, function (index, intersectedRectangles) {
            background.push(getIntersectedRectanglesBackgroundHole(intersectedRectangles));
        });

        background.push('"/>');

        return background.join('');
    }

    function getRectangleBackgroundHoleSvgString(rectangle) {
        return ' M ' + rectangle.left + ',' + rectangle.top + ' H ' + rectangle.right + ' V ' + rectangle.bottom + ' H ' + rectangle.left + ' z ';
    }

    function getFrameSvgString(rectangle, className) {
        var width = className === ELEMENT_FRAME_CLASS ? ELEMENT_FRAME_WIDTH : POINTERS_BASE_FRAME_WIDTH;

        return [
            '<rect class="' + ShadowUI.patchClassNames(className) + '" x="', rectangle.left, '" y="', rectangle.top, '" width="', rectangle.width, '" height="', rectangle.height, '"',
            'style="stroke-width:', width, ';"/>'
        ].join('');
    }

    function getFrameShadowSvgString(rectangle, className) {
        var frameLineWidth = className === ELEMENT_FRAME_SHADOW_CLASS ? ELEMENT_FRAME_WIDTH : POINTERS_BASE_FRAME_WIDTH;

        return [
            '<path class="' + ShadowUI.patchClassNames(className) + '" d="M', rectangle.left + rectangle.width, rectangle.top + SHADOW_OFFSET,
            'H', rectangle.left + rectangle.width + SHADOW_OFFSET,
            'V', rectangle.bottom + SHADOW_OFFSET,
            'H', rectangle.left + SHADOW_OFFSET,
            'V', rectangle.bottom, '"',
            'style="stroke-width:', frameLineWidth, ';"/>'
        ].join(' ');
    }

    function getPointerSvgString(pointerCoordinates) {
        return [
            '<path class="' + ShadowUI.patchClassNames(POINTER_CLASS) + '" d="M', pointerCoordinates.pointerStart.isHorizontal ? pointerCoordinates.pointerStart.left - POINTER_START_WIDTH / 2 : pointerCoordinates.pointerStart.left,
            pointerCoordinates.pointerStart.isHorizontal ? pointerCoordinates.pointerStart.top : pointerCoordinates.pointerStart.top - POINTER_START_WIDTH / 2,

            'L', pointerCoordinates.pointerStart.isHorizontal ? pointerCoordinates.pointerStart.left + POINTER_START_WIDTH / 2 : pointerCoordinates.pointerStart.left,
            pointerCoordinates.pointerStart.isHorizontal ? pointerCoordinates.pointerStart.top : pointerCoordinates.pointerStart.top + POINTER_START_WIDTH / 2,

            'L', pointerCoordinates.pointerEnd.left, pointerCoordinates.pointerEnd.top, 'Z', '" />'
        ].join(' ');
    }

    //Behavior
    function needUpdateMarking(baseRectangle, elementsRectangles) {
        if (forceUpdateMarking) {
            forceUpdateMarking = false;
            return true;
        }

        if (!savedBaseRectangle || !savedElementsRectangles || !baseRectangle.equals(savedBaseRectangle))
            return true;

        if (elementsRectangles.length !== savedElementsRectangles.length)
            return true;
        else {
            for (var i = 0; i < elementsRectangles.length; i++) {
                if (!elementsRectangles[i].equals(savedElementsRectangles[i]))
                    return true;
            }
        }
        return false;
    }

    function rectanglesCountChanged(elementsRectangles, correctedRectangles) {
        return savedElementsRectangles &&
            (elementsRectangles.length !== correctedRectangles.length || elementsRectangles.length !== savedElementsRectangles.length);
    }

    function deleteEmptyRectangles($elements, elementsRectangles) {
        var correctedRectangles = [];
        for (var i = 0; i < elementsRectangles.length; i++) {
            if (elementsRectangles[i].width > 1 && elementsRectangles[i].height > 1 && Util.isElementVisible($elements[i]))
                correctedRectangles.push(elementsRectangles[i]);
        }
        return correctedRectangles;
    }

    function updateRectanglesMarking($base, elementsRectangles, correctedRectangles) {
        var baseRectangle = getRectangle($base);

        if (!needUpdateMarking(baseRectangle, correctedRectangles))
            return;

        if (rectanglesCountChanged(elementsRectangles, correctedRectangles))
            eventEmitter.emit(exports.ELEMENT_RECTANGLES_COUNT_CHANGED, {});

        savedBaseRectangle = baseRectangle;
        savedElementsRectangles = correctedRectangles;

        var svgString = createSvgString(baseRectangle, correctedRectangles);

        var $newSvgElement = $(svgString).width(Util.getDocumentElementWidth())
            .height(Util.getDocumentElementHeight())
            .css('position', 'absolute');

        $elementsMarker.append($newSvgElement);
        removeSvgElement();
        $svgElement = $newSvgElement;
    }

    function updateElementsMarking($base, $elements) {
        var elementsRectangles = $.map($elements, function (element) {
                return getRectangle($(element), ELEMENT_FRAME_PADDING);
            }),
            correctedRectangles = deleteEmptyRectangles($elements, elementsRectangles);

        updateRectanglesMarking($base, elementsRectangles, correctedRectangles);
    }

    function svgBoundsChanged() {
        forceUpdateMarking = true;
    }

    function clearMarkupUpdateInterval() {
        if (elementsMarkerIntervalId) {
            window.clearInterval(elementsMarkerIntervalId);
            elementsMarkerIntervalId = null;
        }
    }

    function removeSvgElement() {
        if ($svgElement)
            $svgElement.remove();
    }

    //API
    function initMarking() {
        if (!$elementsMarker)
            createElementsMarker();

        eventEmitter = new Util.EventEmitter();
        clearMarkupUpdateInterval();
    }

    exports.mark = function ($pointersBase, $elements) {
        initMarking();

        updateElementsMarking($pointersBase, $elements);
        elementsMarkerIntervalId = NativeMethods.setInterval.call(window, function () {
            updateElementsMarking($pointersBase, $elements);
        }, DRAWING_TIME_INTERVAL);

        $(window).on('resize', svgBoundsChanged);
        $(window).on('scroll', svgBoundsChanged);
    };

    exports.markInContext = function ($pointersBase, selector, context, parseDomElementsOrJqueryObjectsOnly) {
        initMarking();

        currentContext = context;
        $currentBase = $pointersBase;

        var msg = {
            cmd: GET_ELEMENT_RECTANGLES_REQUEST_CMD,
            selector: selector,
            parseDomElementsOrJqueryObjectsOnly: parseDomElementsOrJqueryObjectsOnly
        };

        MessageSandbox.sendServiceMsg(msg, context);

        elementsMarkerIntervalId = NativeMethods.setInterval.call(window, function () {
            MessageSandbox.sendServiceMsg(msg, context);
        }, DRAWING_TIME_INTERVAL);
    };

    exports.clear = function () {
        currentContext = null;
        $currentBase = null;

        clearMarkupUpdateInterval();
        removeSvgElement();
        savedBaseRectangle = null;
        savedElementsRectangles = null;

        $(window).unbind('resize', svgBoundsChanged);
        $(window).unbind('scroll', svgBoundsChanged);
    };

    exports.events = {
        on: function (ev, listener) {
            eventEmitter.on(ev, listener);
        }
    };

    //Cross-domain behavior
    var GET_ELEMENT_RECTANGLES_REQUEST_CMD = 'getElementRectanglesRequest',
        GET_ELEMENT_RECTANGLES_RESPONSE_CMD = 'getElementRectanglesResponse';

    var currentContext = null,
        $currentBase = null;

    MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, function (e) {
        var msg = e.message;

        switch (msg.cmd) {
            case GET_ELEMENT_RECTANGLES_REQUEST_CMD:
                var $elements = JavascriptExecutor.parseSelectorSync(msg.selector, msg.parseDomElementsOrJqueryObjectsOnly).$visibleElements,
                    elementRectangles = $.map($elements, function (element) {
                        var rect = getRectangle($(element), ELEMENT_FRAME_PADDING),
                            clientCoords = Util.offsetToClientCoords({
                                x: rect.left,
                                y: rect.top
                            });

                        rect.left = clientCoords.x;
                        rect.top = clientCoords.y;

                        return rect;
                    }),
                    correctedRectangles = deleteEmptyRectangles($elements, elementRectangles),

                    responseMsg = {
                        cmd: GET_ELEMENT_RECTANGLES_RESPONSE_CMD,
                        elementRectangles: elementRectangles,
                        correctedRectangles: correctedRectangles
                    };

                MessageSandbox.sendServiceMsg(responseMsg, window.top);
                break;

            case GET_ELEMENT_RECTANGLES_RESPONSE_CMD:
                if (currentContext === e.source) {
                    updateRectanglesMarking($currentBase,
                        $.map(msg.elementRectangles, function (rect) {
                            var fixedPoint = Util.getFixedPosition({
                                x: rect.left,
                                y: rect.top
                            }, e.source);

                            return new Rectangle(fixedPoint.x, fixedPoint.y, rect.width, rect.height);
                        }),
                        $.map(msg.correctedRectangles, function (rect) {
                            var fixedPoint = Util.getFixedPosition({
                                x: rect.left,
                                y: rect.top
                            }, e.source);

                            return new Rectangle(fixedPoint.x, fixedPoint.y, rect.width, rect.height);
                        }));
                }


                break;
        }
    });
});