TestCafeClient.define('UI.RecorderWidgets.ObjectViewer', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        MessageSandbox = Hammerhead.MessageSandbox,
        JavascriptExecutor = require('Base.JavascriptExecutor'),
        IFrameMessages = require('Base.CrossDomainMessages'),


        ARGUMENT_VALUE_CONTAINER_CLASS = 'container',
        ARGUMENT_VALUE_SPAN_CLASS = 'argument-value-span',
        ARGUMENT_VALUE_DIV_CLASS = 'argument-value-div',
        EMBEDDED_IN_OBJECT_CLASS = 'embedded-in-object',
        OBJECT_TITLE_CLASS = 'title';

    //Util
    function isInstanceOf(className, target) {
        var targetClass = Object.prototype.toString.call(target).slice(8, -1);

        return typeof(target) !== 'undefined' && target !== null && targetClass === className;
    }

    function createRow(str, title) {
        return {
            str: str,
            title: title || ''
        };
    }

    function getArrayRows(array, stringValue, context) {
        var rows = [createRow('[')],
            itemView = null;

        $.each(array, function (i) {
            itemView = getObjectViewSync(stringValue + '[' + i + ']', context);

            if (itemView.type === 'Object')
                itemView.rows[0].str = itemView.rows[0].str.substring(0, itemView.rows[0].str.length - 1);

            rows.push(itemView.type === 'String' ? itemView.rows[0] : itemView);

            if (i !== array.length - 1)
                rows.push(createRow(', '));
        });

        rows.push(createRow(']'));

        return rows;
    }

    function getObjectWithSortedProperties(object) {
        var objectArray = [];

        for (var prop in object) {
            //NOTED: properties 'fileCreatedDate', 'fileModifiedDate' and 'external' raise errors in IE
            if (prop !== 'fileCreatedDate' && prop !== 'fileModifiedDate') {
                if (Util.isIE && prop === 'external')
                    objectArray.push([prop, 'Object {}']);
                else
                    objectArray.push([prop, object[prop]]);
            }
        }

        objectArray.sort();

        var sortedObject = {};
        $.each(objectArray, function (i, prop) {
            sortedObject[prop[0]] = prop[1];
        });

        return sortedObject;
    }

    function getObjectRows(object, className) {
        var objectString = className || 'Object';
        objectString += (Object.getOwnPropertyNames(object).length ? ' {...} ' : ' {} ');

        return [createRow(objectString)];
    }

    function getDomElementString(element) {
        var tagName = element.tagName.toLowerCase();

        return '<' + tagName + '> ... </' + tagName + '>';
    }

    function getDomElementTitle(element) {
        return element.outerHTML.replace(element.innerHTML, '');
    }

    function createSpan($container, text, title) {
        var $span = $('<span></span>').text(text).appendTo($container);
        ShadowUI.addClass($span, ARGUMENT_VALUE_SPAN_CLASS);

        if (title)
            $span.attr('title', title);

        return $span;
    }

    function createArrayMarkup($container, objectView, embedded, context) {
        var $array = embedded ? createDiv($container, '') : createSpan($container, '');

        $.each(objectView.rows, function (i, row) {
            if (row.type) {
                if (row.type === 'Array')
                    createArrayMarkup($array, row, false, context);
                else
                    createObjectMarkup($array, row, false, context);
            }
            else
                createSpan($array, row.str, row.title);
        });

        return $array;
    }

    function createObjectMarkup($container, objectView, embedded, context) {
        var $object = embedded ? createDiv($container, '') : createSpan($container, ''),
            $objectTitle = createDiv($object, objectView.rows[0].str);

        ShadowUI.addClass($objectTitle, OBJECT_TITLE_CLASS);

        initObject($object, objectView.stringValue, objectView.getInheritedProperties, context);

        return $object;
    }

    function initObject($object, stringValue, getInheritedProperties, context) {
        var $objectTitle = ShadowUI.select('.' + OBJECT_TITLE_CLASS, $object),
            titleText = $objectTitle.text();

        ShadowUI.bind($objectTitle, 'click', function () {
            if ($.data($object, 'state') !== 'opened') {
                ObjectViewer.getObjectPropertiesRows(stringValue, getInheritedProperties, context, function (rows) {
                    if (rows.length) {
                        $objectTitle.text(titleText.substring(0, titleText.indexOf('{') + 1) + '    ');

                        if (titleText.indexOf(',') !== -1)
                            rows[rows.length - 1].str += ',';

                        createObjectPropertiesMarkup($object, rows, context);

                        $.data($object, 'state', 'opened');
                    }
                });
            }
            else {
                var $children = $object.children('span, div'),
                    $lastChild = $children.last(),
                    $lastChildText = $lastChild.text();

                $objectTitle.text(titleText.substring(0, titleText.indexOf('{') + 1) + '...}' +
                    ($lastChildText.indexOf(',') !== -1 ? ', ' : ''));

                $.each($children, function (i, child) {
                    if (child !== $objectTitle[0])
                        $(child).remove();
                });

                $.data($object, 'state', 'closed');
            }
        });
    }

    function createObjectPropertiesMarkup($object, rows, context) {
        var $property = null;

        $.each(rows, function (i, stringObj) {
            if (stringObj.type) {
                if (stringObj.type === 'Array')
                    $property = createArrayMarkup($object, stringObj, true, context);
                else {
                    $property = createObjectMarkup($object, stringObj, true, context);
                    ShadowUI.addClass($property, EMBEDDED_IN_OBJECT_CLASS);
                }

                addIndentToElement($property);
            }
            else {
                $property = createDiv($object, stringObj.str, stringObj.title);

                if (i !== rows.length - 1)
                    addIndentToElement($property);
            }
        });
    }

    function createDiv($container, string, title) {
        var $div = $('<div></div>').text(string).appendTo($container);
        ShadowUI.addClass($div, ARGUMENT_VALUE_DIV_CLASS);

        if (title)
            $div.attr('title', title);

        return $div;
    }

    function addIndentToElement($el) {
        $el.css({
            position: 'relative',
            left: '25px'
        });
    }

    //cross-domain functions
    function getObjectViewSync(stringValue, context) {
        var view = {
                rows: [],
                type: '',
                stringValue: stringValue,
                parsedValue: null,
                getInheritedProperties: false,
                error: null,
                isReproducibleValue: false
            },
            row = createRow('');

        view.parsedValue = JavascriptExecutor.eval(stringValue, function (err) {
            view.error = err.message;
        });

        //cross-domain access error protection (for example 'window.top' stringValue in iFrame context in webkit)
        if (!view.error && view.parsedValue)
            try {
                void view.parsedValue.constructor;
            }
            catch (e) {
                view.error = e.message;
            }

        if (view.error)
            return view;

        view.isReproducibleValue = isReproducibleValue(view.parsedValue);

        if (isInstanceOf('Array', view.parsedValue) || Util.isJQueryObj(view.parsedValue)) {
            view.type = 'Array';
            view.rows = getArrayRows(view.parsedValue, stringValue, context);
        }

        else if (Util.isDocumentInstance(view.parsedValue) || Util.isWindowInstance(view.parsedValue)) {
            var sortedObject = getObjectWithSortedProperties(view.parsedValue);
            view.type = 'Object';

            view.getInheritedProperties = true;
            view.rows = getObjectRows(sortedObject, view.parsedValue.constructor ? view.parsedValue.constructor.name : 'Object');
        }

        else if (isInstanceOf('Object', view.parsedValue)) {
            view.type = 'Object';
            view.rows = getObjectRows(view.parsedValue, view.parsedValue.constructor ? view.parsedValue.constructor.name : 'Object');
        }
        else {
            view.type = 'String';

            if (view.parsedValue === null)
                row.str = 'null';
            else if (typeof view.parsedValue === 'undefined')
                row.str = 'undefined';
            else if (Util.isDomElement(view.parsedValue)) {
                row.str = getDomElementString(view.parsedValue);
                row.title = getDomElementTitle(view.parsedValue);
            }

            else if (isInstanceOf('String', view.parsedValue)) {
                var result = view.parsedValue.replace(/\\/g, '\\\\').replace(/\n/g, '\\n').replace(/\r/g, '\\r').replace(/\xA0/g, '\\xA0');

                if (result.indexOf('"') < 0)
                    result = '"' + result + '"';
                else if (result.indexOf("'") < 0)
                    result = "'" + result + "'";
                else
                    result = '"' + result.replace(/"/g, '\\"') + '"';

                row.str = result;
            }
            else
                row.str = view.parsedValue.toString();

            view.rows.push(row);
        }

        return view;
    }

    function getObjectPropertiesRowsSync(stringValue, getInheritedProperties) {
        var rows = [],
            propView = null,
            propertyPrefix = null,
            object = getObjectWithSortedProperties(JavascriptExecutor.eval(stringValue));

        if (Object.getOwnPropertyNames(object).length === 0)
            return rows;

        for (var item in object) {
            if (object.hasOwnProperty(item) || getInheritedProperties) {
                propertyPrefix = item + ': ';
                propView = getObjectViewSync(stringValue + '["' + item + '"]');

                if (propView.type === 'String') {
                    if (isInstanceOf('Function', object[item])) {
                        var functionStr = propView.rows[0].str;

                        functionStr = propertyPrefix + functionStr.substring(0, functionStr.indexOf('{') + 1) + ' ... ,';
                        rows.push(createRow(functionStr));
                    }
                    else
                        rows.push(createRow(propertyPrefix + propView.rows[0].str + ',', propView.rows[0].title));
                }
                else {
                    if (propView.type === 'Object')
                        propView.rows[0].str = propertyPrefix + propView.rows[0].str.substring(0, propView.rows[0].str.length - 1) + ', ';
                    else {
                        propView.rows[0].str = propertyPrefix + propView.rows[0].str;
                        propView.rows[propView.rows.length - 1].str += ', ';
                    }

                    rows.push(propView);
                }
            }
        }

        var lastIndex = rows.length - 1;

        if (rows[lastIndex].type) {
            if (rows[lastIndex].type === 'Object')
                rows[lastIndex].rows[0].str = rows[lastIndex].rows[0].str.substring(0, rows[lastIndex].rows[0].str.length - 2);
            else {
                var lastStringNum = rows[lastIndex].rows.length - 1,
                    lastString = rows[lastIndex].rows[lastStringNum].str;

                rows[lastIndex].rows[lastStringNum].str = lastString.substring(0, lastString.length - 2);
            }
        }
        else
            rows[lastIndex].str = rows[lastIndex].str.substring(0, rows[lastIndex].str.length - 1);


        rows.push(createRow('}'));

        return rows;
    }

    //reproducible value - the value for which you can easily create an equal value
    function isReproducibleValue(value) {
        return isPrimitiveValue(value) || isArrayOfReproducibleValues(value);
    }

    function isPrimitiveValue(value) {
        return /^(string|number|boolean|undefined)$/.test(typeof value);
    }

    function isArrayOfReproducibleValues(value) {
        if ($.isArray(value)) {
            for (var i = 0; i < value.length; i++)
                if (!isReproducibleValue(value[i]))
                    return false;
            return true;
        }
        else
            return false;
    }

    //ObjectViewer
    var ObjectViewer = this.exports = function ($container) {
        this.$viewer = $('<div></div>').appendTo($container);
        ShadowUI.addClass(this.$viewer, ARGUMENT_VALUE_CONTAINER_CLASS);

        this._init();
    };

    ObjectViewer.prototype._init = function () {
        ShadowUI.bind(this.$viewer, typeof document.onwheel !== 'undefined' ? 'wheel' : 'mousewheel', function (e) {
            var delta = e.wheelDelta || (e.originalEvent ? (-e.originalEvent.deltaY * 8) : -e.deltaY * 100),
                top = parseInt($(this).css('top').replace('px', '')),
                sizeDifference = this.scrollHeight - this.clientHeight,
                newTop = Math.max(Math.min(top + delta, 0), -sizeDifference);

            $(this).css('top', newTop + 'px');
            Util.preventDefault(e);

            return false;
        });
    };

    ObjectViewer.getObjectView = function (stringValue, context, callback) {
        function onMessage(e) {
            if (e.message.cmd === IFrameMessages.OBJECT_VIEWER_GET_OBJECT_VIEW_RESPONSE_CMD && e.message.stringValue === stringValue) {
                callback(e.message.objectView);
                MessageSandbox.off(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
            }
        }

        if (context) {
            var msg = {
                cmd: IFrameMessages.OBJECT_VIEWER_GET_OBJECT_VIEW_REQUEST_CMD,
                stringValue: stringValue
            };

            MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
            MessageSandbox.sendServiceMsg(msg, context);
        }
        else
            callback(getObjectViewSync(stringValue, context));
    };

    ObjectViewer.getObjectPropertiesRows = function (stringValue, getInheritedProperties, context, callback) {
        function onMessage(e) {
            if (e.message.cmd === IFrameMessages.OBJECT_VIEWER_GET_OBJECT_PROPERTIES_ROWS_RESPONSE_CMD && e.message.stringValue === stringValue) {
                callback(e.message.rows);
                MessageSandbox.off(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
            }
        }

        if (context) {
            var msg = {
                cmd: IFrameMessages.OBJECT_VIEWER_GET_OBJECT_PROPERTIES_ROWS_REQUEST_CMD,
                getInheritedProperties: getInheritedProperties,
                stringValue: stringValue
            };

            MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, onMessage);
            MessageSandbox.sendServiceMsg(msg, context);
        }
        else
            callback(getObjectPropertiesRowsSync(stringValue, getInheritedProperties));
    };

    ObjectViewer.prototype.show = function (stringValue, context, callback) {
        var viewer = this;

        ObjectViewer.getObjectView(stringValue, context, function (objectView) {
            viewer.$viewer[0].innerHTML = '';
            viewer.$viewer.css('top', 0);

            if (!objectView.error) {
                if (objectView.type === 'String')
                    createSpan(viewer.$viewer, objectView.rows[0].str, objectView.rows[0].title);

                else if (objectView.type === 'Array')
                    createArrayMarkup(viewer.$viewer, objectView, false, context);

                else if (objectView.type === 'Object')
                    createObjectMarkup(viewer.$viewer, objectView, false, context);
            }

            callback(objectView);
        });
    };

    ObjectViewer.prototype.clear = function () {
        this.$viewer[0].innerHTML = '';
        this.$viewer.css('top', 0);
    };

    ObjectViewer.prototype.getText = function () {
        return this.$viewer.text();
    };
});