TestCafeClient.define('UI.RecorderWidgets.PlaybackErrorMessages', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        ShadowUI = Hammerhead.ShadowUI,
        PageState = Hammerhead.PageState,
        PopupWidget = require('UI.RecorderWidgets.Popup'),
        ButtonWidget = require('UI.RecorderWidgets.Button');

    var PLAYBACK_RESULT_CLASS = 'playback-result',
        ERROR_CLASS = 'error',
        ERROR_MESSAGE_CLASS = 'error-message',
        BUTTON_CONTAINER_CLASS = 'button-container',
        BUTTON_TEXT = 'Resume playback';

    var MAX_VISIBLE_ERRORS = 5;

    var popup = null,
        pageState = null;

    exports.show = function (errors, callback) {
        if (popup)
            close();

        pageState = PageState.saveState();

        var $container = $('<div></div>');

        ShadowUI.addClass($container, PLAYBACK_RESULT_CLASS);
        ShadowUI.addClass($container, ERROR_CLASS);

        createText($container);
        createErrors($container, errors);
        createButton($container, callback);

        popup = new PopupWidget(ShadowUI.getRoot(), {
            width: 530,
            content: $container,
            showAtWindowCenter: true,
            defaultStyle: false
        });
    };

    function createText($container) {
        var $text = $('<div></div>').text('The following unhandled JavaScript errors occurred within the page: ').appendTo($container);

        ShadowUI.addClass($text, ERROR_MESSAGE_CLASS);
    }

    function createErrors($container, errors) {
        var $list = $('<ul></ul>').appendTo($container);

        for (var i = 0; i < errors.length; i++) {
            if (i < MAX_VISIBLE_ERRORS)
                $('<li></li>').text(errors[i]).appendTo($list);
            else {
                $('<li></li>').text('...').appendTo($list);
                break;
            }
        }
    }

    function close(onClosed) {
        popup.close();
        popup = null;

        PageState.restoreState(pageState, true, function () {
            if (typeof onClosed === 'function')
                onClosed();
        });
    }

    function createButton($container, onClosed) {
        var $buttonContainer = $('<div></div>').appendTo($container);
        ShadowUI.addClass($buttonContainer, BUTTON_CONTAINER_CLASS);

        ButtonWidget.create($buttonContainer, BUTTON_TEXT).click(function () {
            close(onClosed);
        });
    }
});