TestCafeClient.define('UI.RecorderWidgets.PlaybackResult', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        ShadowUI = Hammerhead.ShadowUI,
        PageState = Hammerhead.PageState,
        PopupWidget = require('UI.RecorderWidgets.Popup'),
        ButtonWidget = require('UI.RecorderWidgets.Button');

    var PLAYBACK_RESULT_CLASS = 'playback-result',
        SUCCESS_CLASS = 'success',
        FAILED_CLASS = 'failed',
        HEADER_CLASS = 'header',
        SEPARATOR_CLASS = 'separator',
        COUNT_CLASS = 'count',
        TEXT_CLASS = 'text',
        TOP_TEXT_CLASS = 'top',
        ERROR_TEXT_CLASS = 'error-text',

        HEADER_TEXT_COMPLETED = 'Test completed',
        HEADER_TEXT_FAILED = 'Test failed',
        PLAYBACK_SUCCESS_TEXT = 'steps played successfully',
        PLAYBACK_FAILED_TEXT = 'steps failed',
        STEP_FAILED_TEXT = 'failed with an error:',
        BUTTON_TEXT = 'Return to recording';

    var popup = null,
        pageState = null;

    exports.STATES = {
        FINISHED: 'finished',
        FAILED: 'failed'
    };

    exports.show = function (state, options) {
        var total = options.total,
            failed = options.failed,
            onClosed = options.onClosed,
            failedStepNum = options.failedStepNum,
            error = options.error;

        if (popup)
            close();

        pageState = PageState.saveState();

        var $container = $('<div></div>');

        ShadowUI.addClass($container, PLAYBACK_RESULT_CLASS);
        ShadowUI.addClass($container, (failed || failedStepNum) ? FAILED_CLASS : SUCCESS_CLASS);

        createHeader($container, state);
        createSeparator($container);

        if (state === exports.STATES.FINISHED)
            createCount($container, total, failedStepNum || failed, state);

        createTextBlock($container, state, failedStepNum || failed, state === exports.STATES.FAILED ? TOP_TEXT_CLASS : null);

        if (error)
            createError($container, error);

        createSeparator($container);
        createButton($container, onClosed);

        popup = new PopupWidget(ShadowUI.getRoot(), {
            width: 530,
            content: $container,
            showAtWindowCenter: true,
            defaultStyle: false
        });
    };

    function createHeader($container, state) {
        var $header = $('<div></div>').text(state === exports.STATES.FINISHED ? HEADER_TEXT_COMPLETED : HEADER_TEXT_FAILED).appendTo($container);

        ShadowUI.addClass($header, HEADER_CLASS);
    }

    function createSeparator($container) {
        var $separator = $('<div></div>').appendTo($container);

        ShadowUI.addClass($separator, SEPARATOR_CLASS);
    }

    function createCount($container, total, failed, state) {
        var $count = $('<div></div>').appendTo($container);

        ShadowUI.addClass($count, COUNT_CLASS);

        if (state === exports.STATES.FINISHED)
            $count.text(failed ? failed + '/' + total : total);
        else
            $count.text(failed);
    }

    function createTextBlock($container, state, failed, className) {
        if (state === exports.STATES.FINISHED)
            createText($container, failed ? PLAYBACK_FAILED_TEXT : PLAYBACK_SUCCESS_TEXT, className);
        else
            createText($container, 'Step ' + failed + ' ' + STEP_FAILED_TEXT, className);
    }

    function createText($container, text, className) {
        var $message = $('<div></div>').appendTo($container);
        ShadowUI.addClass($message, TEXT_CLASS);

        if (className)
            ShadowUI.addClass($message, className);

        $message.text(text);
    }

    function createError($container, error) {
        var $errorMessage = $('<div></div>').appendTo($container);
        ShadowUI.addClass($errorMessage, ERROR_TEXT_CLASS);

        $errorMessage.text(error);
    }

    function close(onClosed) {
        popup.close();
        popup = null;

        PageState.restoreState(pageState, true, function () {
            if (typeof onClosed === 'function')
                onClosed();
        });
    }

    function createButton($container, onClosed) {
        ButtonWidget.create($container, BUTTON_TEXT).click(function () {
            close(onClosed);
        });
    }
});