TestCafeClient.define('UI.RecorderWidgets.Popup', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,
        ShadowUI = Hammerhead.ShadowUI,
        EventSandbox = Hammerhead.EventSandbox,
        PageState = Hammerhead.PageState,
        TextSelection = Hammerhead.TextSelection,
        DraggingBehavior = require('UI.DraggingBehavior');

    //Const
    var POPUP_CLASS = 'popup',
        HEADER_CLASS = 'header',
        CONTENT_CLASS = 'content',
        FOOTER_CLASS = 'footer',
        BLIND_CLASS = 'blind',
        DEFAULT_STYLE_CLASS = 'default',
        NO_BORDERS_CLASS = 'no-borders',
        NO_TOP_MARGIN_CLASS = 'no-top-margin',

        SEPARATOR_CLASS = 'separator',
        TITLE_CLASS = 'title',
        CLOSE_BUTTON_CLASS = 'close-button',
        HIDE_ICON_CLASS = 'hide-icon',
        OPACITY_CLASS = 'opacity';

    //Globals
    var windowResizeHandler = null;

    var Popup = this.exports = function ($container, options) {
        var popup = this;

        options = options || {};

        this.options = {
            width: options.width || '',
            headerText: options.headerText || '',
            headerIconText: options.headerIconText || '',
            headerIconClass: options.headerIconClass || '',
            headerCloseButton: options.headerCloseButton || false,
            headerHideIcon: options.headerHideIcon || false,
            headerTextSize: options.headerTextSize || '',
            content: options.content,
            footerContent: options.footerContent || null,
            backgroundOpacity: options.backgroundOpacity || false,
            showAtWindowCenter: options.showAtWindowCenter || false,
            notDialog: options.notDialog || false,
            prevActiveElement: options.prevActiveElement || Util.getActiveElement(),
            restoreActiveElement: typeof options.restoreActiveElement === 'undefined' ? true : options.restoreActiveElement,
            defaultStyle: typeof options.defaultStyle === 'undefined' ? true : options.defaultStyle,
            noBordersStyle: options.noBordersStyle || false,
            prependToContainer: options.prependToContainer || false,
            footerWithoutTopMargins: options.footerWithoutTopMargins || false,
            hasParentPopup: options.hasParentPopup || false
        };

        this.$popup = null;
        this.$header = null;
        this.$headerTitle = null;
        this.$content = null;
        this.$footer = null;

        this.$closeButton = null;
        this.$hideIcon = null;

        this.selectionState = null;

        this.$focusableElement = null;
        this.popupBlurHandler = null;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            popup.eventEmitter.on(ev, listener);
        };

        this._createPopup($container);
        this._init();
    };

    //Events
    Popup.CLOSE_BUTTON_CLICK_EVENT = 'closeButtonClick';
    Popup.HIDE_BUTTON_CLICK_EVENT = 'hideButtonClick';
    Popup.DRAG_STARTED_EVENT = 'dragStarted';
    Popup.MOVED_EVENT = 'moved';

    //Utils
    Popup.prototype._getPopupRelativePosition = function ($baseElement) {
        var MIN_WINDOW_PADDING = 10,
            MARGIN = 50,
            VERTICAL_POSITION_PREFERENCE = 500;

        var $window = $(window),
            windowHeight = $window.height(),
            windowWidth = $window.width(),
            scrollTop = $window.scrollTop(),
            scrollLeft = $window.scrollLeft(),

            baseRectangle = Util.getElementRectangle($baseElement[0]),

            baseClientLeft = Math.max(baseRectangle.left - scrollLeft, 0),
            baseClientRight = Math.min(baseRectangle.left + baseRectangle.width - scrollLeft, windowWidth),
            baseClientWidth = baseClientRight - baseClientLeft,
            baseClientCenterX = Math.floor(baseClientWidth / 2 + baseClientLeft),

            baseClientTop = Math.max(baseRectangle.top - scrollTop, 0),
            baseClientBottom = Math.min(baseRectangle.top + baseRectangle.height - scrollTop, windowHeight),
            baseClientHeight = baseClientBottom - baseClientTop,
            baseClientCenterY = Math.floor(baseClientHeight / 2 + baseClientTop),

            popupWidth = this.$popup.width(),
            popupHeight = this.$popup.height(),
            top = 0,
            left = 0,

            verticalPosition = '',
            verticalSpace = 0,
            horizontalPosition = '',
            horizontalSpace = 0;


        var topSpace = baseClientTop,
            bottomSpace = windowHeight - baseClientTop - baseClientHeight;
        if (topSpace - bottomSpace >= 0) {
            if (topSpace >= popupHeight + MIN_WINDOW_PADDING)
                verticalPosition = 'top';
            verticalSpace = topSpace;
        }
        else {
            if (bottomSpace >= popupHeight + MIN_WINDOW_PADDING)
                verticalPosition = 'bottom';
            verticalSpace = bottomSpace;
        }

        var leftSpace = baseClientLeft,
            rightSpace = windowWidth - baseClientLeft - baseClientWidth;
        if (leftSpace - rightSpace >= 0) {
            if (leftSpace >= popupWidth + MIN_WINDOW_PADDING)
                horizontalPosition = 'left';
            horizontalSpace = leftSpace;
        }
        else {
            if (rightSpace >= popupWidth + MIN_WINDOW_PADDING)
                horizontalPosition = 'right';
            horizontalSpace = rightSpace;
        }

        if (verticalPosition && !(horizontalPosition && (horizontalSpace - verticalSpace > VERTICAL_POSITION_PREFERENCE))) {
            top = verticalPosition === 'top' ?
                Math.max(baseClientTop - MARGIN - popupHeight, MIN_WINDOW_PADDING) :
                Math.min(baseClientTop + baseClientHeight + MARGIN, windowHeight - popupHeight - MIN_WINDOW_PADDING);
            left = Math.floor((windowWidth - popupWidth) * (baseClientCenterX / windowWidth));
        }
        else if (horizontalPosition) {
            left = horizontalPosition === 'left' ?
                Math.max(baseClientLeft - MARGIN - popupWidth, MIN_WINDOW_PADDING) :
                Math.min(baseClientLeft + baseClientWidth + MARGIN, windowWidth - popupWidth - MIN_WINDOW_PADDING);
            top = Math.floor((windowHeight - popupHeight) * (baseClientCenterY / windowHeight));
        }
        else {
            left = Math.floor((windowWidth - popupWidth) * (baseClientCenterX / windowWidth));
            top = Math.floor((windowHeight - popupHeight) * (baseClientCenterY / windowHeight));
        }

        return {
            left: left,
            top: top
        };
    };

    //Markup
    Popup.prototype._createPopup = function ($container) {
        var popup = this,
            popupOpened = this.options.hasParentPopup || !!(ShadowUI.select('.' + POPUP_CLASS).length);

        this.$popup = $('<div></div>')
            .width(this.options.width)
            .attr('tabindex', 1);

        if (this.options.prependToContainer)
            this.$popup.prependTo($container);
        else
            this.$popup.appendTo($container);

        ShadowUI.addClass(this.$popup, POPUP_CLASS);

        if (this.options.noBordersStyle)
            ShadowUI.addClass(this.$popup, NO_BORDERS_CLASS);

        if (this.options.defaultStyle)
            ShadowUI.addClass(this.$popup, DEFAULT_STYLE_CLASS);

        if (!this.options.notDialog) {
            if (!popupOpened)
                this.selectionState = PageState.saveState();

            //B238660 - Wrong behavior of selection in editable elements (during the show recorder's dialogs) in IE
            if (Util.isIE && Util.isTextEditableElement(popup.options.prevActiveElement))
                TextSelection.select(popup.options.prevActiveElement, 0, 0);

            EventSandbox.disableOuterFocusHandlers();
            this._bindPopupBlurHandler(this.$popup);
        }

        if (this.options.headerIconText || this.options.headerText || this.options.headerIconClass)
            this._createHeader();
        this._createContent();

        if (this.options.footerContent)
            this._createFooter();

        if (this.options.showAtWindowCenter)
            this._disposeRelativeToWindowCenter();
    };

    Popup.prototype._createHeader = function () {
        var popup = this,
            $number = null,
            headerElementsWidth = 0;

        this.$header = $('<div></div>').appendTo(this.$popup);
        ShadowUI.addClass(this.$header, HEADER_CLASS);

        if (this.options.headerIconClass) {
            $number = $('<div></div>').appendTo(this.$header);

            if (this.options.headerIconText)
                $number.text(this.options.headerIconText);

            ShadowUI.addClass($number, this.options.headerIconClass);
            headerElementsWidth += $number.width();

            if (this.options.headerTextSize)
                this.$headerTitle.css('fontSize', this.options.headerTextSize);

            var $separator = $('<div></div>').appendTo(this.$header);
            ShadowUI.addClass($separator, SEPARATOR_CLASS);
            headerElementsWidth += $separator.width();
        }

        this.$headerTitle = $('<div></div>').appendTo(this.$header);
        ShadowUI.addClass(this.$headerTitle, TITLE_CLASS);

        if (Util.isWebKit || $.browser.opera) {
            if ($number)
                $number.css('marginTop', '-1px');
            this.$headerTitle.css('marginTop', '-1px');
        }

        if (this.options.headerTextSize)
            this.$headerTitle.css('fontSize', this.options.headerTextSize);

        if (this.options.headerCloseButton) {
            this.$closeButton = $('<div></div>').appendTo(this.$header);
            ShadowUI.addClass(this.$closeButton, CLOSE_BUTTON_CLASS);
            headerElementsWidth += this.$closeButton.width();
        }

        if (this.options.headerHideIcon) {
            this.$hideIcon = $('<div></div>').appendTo(this.$header);
            ShadowUI.addClass(this.$hideIcon, HIDE_ICON_CLASS);
            headerElementsWidth += this.$hideIcon.width();
        }

        //NOTE: set max width for header title
        NativeMethods.setTimeout.call(window, function () {
            var popupPadding = Util.getElementPadding(popup.$popup),
                titlePadding = Util.getElementPadding(popup.$headerTitle),
                titleWidth = popup.$popup.width() - (popupPadding.left + popupPadding.right) -
                    headerElementsWidth - (titlePadding.left + titlePadding.right);

            popup.$headerTitle.css('max-width', titleWidth);
            popup.setHeaderText(popup.options.headerText);
        }, 0);
    };

    Popup.prototype._createContent = function () {
        this.$content = $('<div></div>').appendTo(this.$popup);
        ShadowUI.addClass(this.$content, CONTENT_CLASS);

        if (this.options.backgroundOpacity)
            ShadowUI.addClass(this.$content, OPACITY_CLASS);

        if (this.options.content)
            $(this.options.content).appendTo(this.$content);
    };

    Popup.prototype._createFooter = function () {
        this.$footer = $('<div></div>').appendTo(this.$content);
        ShadowUI.addClass(this.$footer, FOOTER_CLASS);

        if (this.options.footerWithoutTopMargins)
            ShadowUI.addClass(this.$footer, NO_TOP_MARGIN_CLASS);

        $(this.options.footerContent).appendTo(this.$footer);
    };

    //Behavior
    Popup.prototype._onResizeHandler = function () {
        if (this.options.showAtWindowCenter)
            this._disposeRelativeToWindowCenter();

        if (this.positionElement && Util.isElementVisible(this.positionElement)) {
            var popupPosition = Util.getOffsetPosition(this.$popup[0]);
            if (popupPosition.left >= 0 && popupPosition.top >= 0)
                this.disposeRelativeToElement($(this.positionElement));
        }
    };

    Popup.prototype._init = function () {
        var popup = this;

        if (this.$closeButton)
            this.$closeButton.click(function () {
                popup.close();
                popup.eventEmitter.emit(Popup.CLOSE_BUTTON_CLICK_EVENT, {});
            });
        if (this.$hideIcon)
            this.$hideIcon.click(function () {
                popup.eventEmitter.emit(Popup.HIDE_BUTTON_CLICK_EVENT, {});
            });
        if (this.$header) {
            new DraggingBehavior(this.$header, this.$popup, {
                onDragStart: function () {
                    popup.eventEmitter.emit(Popup.DRAG_STARTED_EVENT, {});
                },
                onMove: function () {
                    popup.eventEmitter.emit(Popup.MOVED_EVENT, {});
                }
            });
        }

        windowResizeHandler = function () {
            popup._onResizeHandler();
        };

        ShadowUI.bind($(window), 'resize', windowResizeHandler);

        //NOTE: we can use the popup widget without EventSandbox initializing
        if (typeof EventSandbox.fixHoveredElement === 'function')
            EventSandbox.fixHoveredElement();
    };

    Popup.prototype._disposeRelativeToWindowCenter = function () {
        var $window = $(window),
            top = Math.round($window.height() / 2 - (this.$popup.height() / 2)),
            left = Math.round($window.width() / 2 - (this.$popup.width() / 2));
        this.$popup.css({
            top: top,
            left: left
        });
    };

    Popup.prototype._bindPopupBlurHandler = function ($focusableElement) {
        var popup = this,
            activeElement = null;

        popup.$focusableElement = $focusableElement;

        //NOTE: to prevent blur handlers on active element on the page we make it unfocusable without handlers
        EventSandbox.focus(popup.$focusableElement[0], function () {
            popup.popupBlurHandler = function () {
                //NOTE: timeout to define focused element (to ensure 'tab')
                NativeMethods.setTimeout.call(window, function () {
                    activeElement = document.activeElement;
                    if (activeElement === $('body')[0] ||
                        (popup.$focusableElement !== popup.$popup && activeElement === popup.$popup[0]) ||
                        (Util.isIE && Util.isShadowUIElement(activeElement) &&
                            (ShadowUI.hasClass($(activeElement), 'modal-background') || activeElement.tagName.toLowerCase() === 'svg' || activeElement === ShadowUI.getRoot()[0]))) {
                        NativeMethods.focus.call(popup.$focusableElement[0]);
                    }
                }, 50);
            };
            ShadowUI.bind(popup.$focusableElement, 'blur', popup.popupBlurHandler);
        }, true);
    };

    //API
    Popup.prototype.close = function (callback) {
        var popupCallback = callback ? callback : function () {
        };

        if (!this.options.notDialog)
            ShadowUI.unbind(this.$focusableElement, 'blur', this.popupBlurHandler);

        EventSandbox.enableOuterFocusHandlers();

        if (this.keydownHandler)
            NativeMethods.removeEventListener.call(this.$popup[0], 'keydown', this.keydownHandler, false);

        this.$popup.remove();

        ShadowUI.unbind($(window), 'resize', windowResizeHandler);

        if (this.options.restoreActiveElement) {
            if (this.selectionState)
                PageState.restoreState(this.selectionState, true, popupCallback);
            else if (this.options.prevActiveElement) {
                if (Util.isShadowUIElement(this.options.prevActiveElement)) {
                    NativeMethods.focus.call(this.options.prevActiveElement);
                    popupCallback();
                }
                else
                    EventSandbox.focus(this.options.prevActiveElement, popupCallback, true);
            }
        }
        else
            popupCallback();

        //NOTE: we can use the popup widget without EventSandbox initializing
        if (typeof EventSandbox.freeHoveredElement === 'function')
            EventSandbox.freeHoveredElement();
    };

    Popup.prototype.getContainer = function () {
        return this.$popup;
    };

    Popup.prototype.setHeaderText = function (text) {
        if (this.$headerTitle) {
            this.$headerTitle.text(text);
        }
    };

    Popup.prototype.getHeaderHeight = function () {
        return this.$header.outerHeight();
    };

    Popup.prototype.blind = function (blind) {
        if (blind) {
            ShadowUI.addClass(this.$popup, BLIND_CLASS);
            this.$popup.css('zIndex', this.$popup.css('zIndex') - 3);
        }
        else {
            ShadowUI.removeClass(this.$popup, BLIND_CLASS);
            this.$popup.css('zIndex', '');
        }
    };

    Popup.prototype.hide = function () {
        ShadowUI.unbind(this.$focusableElement, 'blur', this.popupBlurHandler);
        this.$popup.css('visibility', 'hidden');
        EventSandbox.enableOuterFocusHandlers();
    };

    Popup.prototype.show = function () {
        this.$popup.css('visibility', '');
        EventSandbox.disableOuterFocusHandlers();
        this._bindPopupBlurHandler(this.$focusableElement);
    };

    Popup.prototype.isHidden = function () {
        return this.$popup.css('visibility') === 'hidden';
    };

    Popup.prototype.moveTo = function (left, top, minLeft) {
        var $window = $(window),
            windowWidth = $window.width(),
            windowHeight = $window.height(),
            popupWidth = this.$popup.width(),
            popupHeight = this.$popup.height();

        if (typeof minLeft !== 'undefined' && minLeft !== null)
            left = Math.max(left, minLeft);
        else if (left + popupWidth > windowWidth)
            left = windowWidth - popupWidth;

        if (top + popupHeight > windowHeight)
            top = windowHeight - popupHeight;

        this.$popup.css({
            'left': left,
            'top': top
        });
    };

    Popup.prototype.getPosition = function () {
        return {
            left: parseInt(this.$popup.css('left').replace('px', '')),
            top: parseInt(this.$popup.css('top').replace('px', ''))
        };
    };

    Popup.prototype.disposeRelativeToElement = function ($elements) {
        this.positionElement = $elements[0];
        var popupPos = this._getPopupRelativePosition($elements);
        this.moveTo(popupPos.left, popupPos.top);
    };

    Popup.prototype.showAtWindowCenter = function () {
        this._disposeRelativeToWindowCenter();
    };

    Popup.prototype.changeFocusableElement = function ($focusableElement) {
        if (!this.options.notDialog && $focusableElement.length) {
            ShadowUI.unbind(this.$focusableElement, 'blur', this.popupBlurHandler);
            this._bindPopupBlurHandler($focusableElement);
        }
    };

    Popup.prototype.onkeydown = function (handler, preventBackspace) {
        var popup = this;

        this.keydownHandler = !preventBackspace ? handler : function (e) {
            if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.backspace && e.target === popup.$popup[0])
                Util.preventDefault(e);
            else
                handler(e);

            Util.stopPropagation(e);
        };

        NativeMethods.addEventListener.call(this.$popup[0], 'keydown', this.keydownHandler, false);

        //T202650
        NativeMethods.addEventListener.call(this.$popup[0], 'keypress', function (e) {
            Util.stopPropagation(e);
        });
        NativeMethods.addEventListener.call(this.$popup[0], 'keyup', function (e) {
            Util.stopPropagation(e);
        });
    };
});