TestCafeClient.define('UI.RecorderWidgets.RadioGroup', function () {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI;

    //Const
    var RADIO_GROUP_CLASS = 'radio-group',
        RADIO_BUTTON_CLASS = 'radio-button',
        RADIO_BUTTON_ICON_CLASS = 'icon',
        RADIO_BUTTON_TITLE_CLASS = 'title',
        PRESSED_CLASS = 'pressed',
        DISABLED_CLASS = 'disabled';

    var RadioGroup = this.exports = function ($container, titles, checkedIndex) {
        var radioGroup = this;

        this.$radioGroup = null;
        this.$radioButtons = [];

        this.titles = titles;
        this.checkedIndex = titles && titles.length ? Math.min(Math.max(0, checkedIndex), titles.length - 1) : -1;
        this.isDisabled = false;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            radioGroup.eventEmitter.on(ev, listener);
        };

        this._create($container);
        this._init();
    };

    //Events
    RadioGroup.CHECKED_INDEX_CHANGED = 'checkedIndexChanged';

    //Markup
    RadioGroup.prototype._create = function ($container) {
        var radioGroup = this,

            $button = null,
            $icon = null,
            $title = null;

        this.$radioGroup = $('<div></div>').appendTo($container);
        ShadowUI.addClass(this.$radioGroup, RADIO_GROUP_CLASS);

        $.each(this.titles, function (index, title) {
            $button = $('<div></div>').appendTo(radioGroup.$radioGroup);
            radioGroup.$radioButtons.push($button);
            ShadowUI.addClass($button, RADIO_BUTTON_CLASS);

            $icon = $('<div></div>').appendTo($button);
            ShadowUI.addClass($icon, RADIO_BUTTON_ICON_CLASS);

            $title = $('<span></span>').html(title).appendTo($button);
            ShadowUI.addClass($title, RADIO_BUTTON_TITLE_CLASS);

            if (Util.isIE && Util.browserVersion === 9)
                $title[0].setAttribute('unselectable', 'on');
        });

        if (this.checkedIndex !== -1)
            ShadowUI.addClass(radioGroup.$radioButtons[this.checkedIndex], PRESSED_CLASS);

        this._init();
    };

//Behavior
    RadioGroup.prototype._init = function () {
        var radioGroup = this;

        $.each(this.$radioButtons, function (index, button) {
            ShadowUI.bind($(button), 'click', function (e) {
                radioGroup.changeState(index);
                Util.preventDefault(e);
            });
        });
    };

    //API
    RadioGroup.prototype.changeState = function (newIndex) {
        if (!this.isDisabled && newIndex !== this.checkedIndex) {
            ShadowUI.removeClass(this.$radioButtons[this.checkedIndex], PRESSED_CLASS);
            ShadowUI.addClass(this.$radioButtons[newIndex], PRESSED_CLASS);
            this.checkedIndex = newIndex;

            this.eventEmitter.emit(RadioGroup.CHECKED_INDEX_CHANGED, {
                checkedIndex: this.checkedIndex
            });
        }
    };

    RadioGroup.prototype.disable = function () {
        this.isDisabled = true;

        ShadowUI.addClass(this.$radioGroup, DISABLED_CLASS);
    };

    RadioGroup.prototype.enable = function () {
        this.isDisabled = false;

        ShadowUI.removeClass(this.$radioGroup, DISABLED_CLASS);
    };
});