TestCafeClient.define('UI.RecorderWidgets.ScrollBar', function () {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI;

    //Const
    var SCROLL_CONTAINER_CLASS = 'scrollContainer',
        SCROLL_CLASS = 'scroll';

    var ScrollBar = this.exports = function ($container, onScroll) {
        this.$container = $container;
        this.state = {
            disabled: false,
            containerInnerHeight: 0,
            scrollBarHeight: 0,
            sizeDifference: 0,
            factor: 0,

            //scroll moving
            inScroll: false,
            startY: 0,
            startScrollTop: 0
        };
        this.onScroll = onScroll;
        this.minScroll = null;
        this.maxScroll = null;

        this.$scrollBarContainer = $('<div></div>').appendTo(this.$container);
        ShadowUI.addClass(this.$scrollBarContainer, SCROLL_CONTAINER_CLASS);

        this.$scrollBar = $('<div></div>').appendTo(this.$scrollBarContainer);
        ShadowUI.addClass(this.$scrollBar, SCROLL_CLASS);

        this.$scrollBarContainer.css('display', 'none');
    };

    ScrollBar.prototype._setScroll = function (newTop) {
        var scrollContentTop = Math.max(newTop, -this.state.sizeDifference);

        if (this.minScroll !== null && Math.abs(scrollContentTop) < this.minScroll)
            scrollContentTop = (scrollContentTop / Math.abs(scrollContentTop)) * this.minScroll;

        if (this.maxScroll !== null && Math.abs(scrollContentTop) > this.maxScroll)
            scrollContentTop = (scrollContentTop / Math.abs(scrollContentTop)) * this.maxScroll;

        var scrollBarTop = -scrollContentTop * this.state.factor;

        this.$scrollBar.css('top', Math.floor(scrollBarTop));
        this.$scrollableContent.css('top', Math.floor(scrollContentTop));

        if (typeof this.onScroll === 'function')
            this.onScroll();
    };

    ScrollBar.prototype._getRelativeItemTop = function ($item) {
        var itemOffset = Util.getOffsetPosition($item[0]),
            $itemParents = $item.parents(),
            newItemTop = itemOffset.top;

        $itemParents.each(function (i, parent) {
            if ($(parent).css('position') !== 'static' && (!Util.isMozilla || $(parent).css('display') !== 'table')) {
                var parentOffset = Util.getOffsetPosition(parent);
                newItemTop -= parentOffset.top;
                return false;
            }
        });

        return newItemTop;
    };

    ScrollBar.prototype.init = function ($scrollableContainer, $scrollableContent) {
        this.$scrollableContainer = $scrollableContainer;
        this.$scrollableContent = $scrollableContent;

        var scrollBar = this;

        this.recalculateSize();

        if (this.state.sizeDifference <= 0)
            this.state.disabled = true;

        ShadowUI.bind(scrollBar.$container, typeof document.onwheel !== 'undefined' ? 'wheel' : 'mousewheel', function (e) {
            scrollBar.updateOnMouseWheel(e);
        });

        //mouse
        ShadowUI.bind(scrollBar.$scrollBarContainer, 'mousedown', function (e) {
            if (scrollBar.state.disabled)
                return;

            scrollBar.state.startY = e.clientY;
            scrollBar.state.startScrollTop = parseInt(scrollBar.$scrollBar.css('top').replace('px', ''));
            scrollBar.state.inScroll = true;
            Util.preventDefault(e);

            return false;
        });

        ShadowUI.bind($(document), 'mouseup', function () {
            if (scrollBar.state.disabled)
                return;

            scrollBar.state.inScroll = false;
            return false;
        });

        ShadowUI.bind($(document), 'mousemove', function (e) {
            if (!scrollBar.state.inScroll || scrollBar.state.disabled)
                return;

            var newTop = Math.min(Math.max(e.clientY - scrollBar.state.startY + scrollBar.state.startScrollTop, 0),
                scrollBar.state.containerInnerHeight - scrollBar.state.scrollBarHeight);

            scrollBar._setScroll(-newTop / scrollBar.state.factor);
            Util.preventDefault(e);

            return false;
        });
    };

    ScrollBar.prototype.isVisible = function () {
        return this.$scrollBarContainer.css('display') !== 'none';
    };

    ScrollBar.prototype.updateScroll = function (item) {
        if (this.state.disabled)
            return;

        var $newItem = $(item),
            newItemOffset = Util.getOffsetPosition($newItem[0]),
            newItemHeight = $newItem.height() + Util.getElementMargin($newItem).bottom,

            scrollBarTop = parseInt(this.$scrollBar.css('top').replace('px', '')),
            containerPadding = Util.getElementPadding(this.$scrollableContainer),
            containerInnerHeight = this.$container.height() - (containerPadding.top + containerPadding.bottom),

            containerOffset = Util.getOffsetPosition(this.$container),
            containerHeight = this.$container.height(),

            needTopScroll = newItemOffset.top < containerOffset.top,
            needBottomScroll = newItemOffset.top + newItemHeight > containerOffset.top + containerHeight,

            newTop = null;

        this.state.factor = containerInnerHeight / this.$scrollableContent.height();

        if (this._getRelativeItemTop($newItem) === 0) {
            needTopScroll = true;
            newTop = 0;
        }

        if (needTopScroll || needBottomScroll) {
            var scrollIndent = needTopScroll ? newItemOffset.top - containerOffset.top :
                newItemOffset.top + newItemHeight - containerOffset.top - containerHeight;

            newTop = newTop === 0 ? 0 :
                Math.min(Math.max(this.state.factor * scrollIndent + scrollBarTop, 0), containerInnerHeight - Math.floor(containerInnerHeight * this.state.factor));

            this._setScroll(-newTop / this.state.factor);
        }
    };

    ScrollBar.prototype.restoreScroll = function (scrollBarTop) {
        var containerPadding = Util.getElementPadding(this.$scrollableContainer),
            containerInnerHeight = this.$container.height() - (containerPadding.top + containerPadding.bottom),
            factor = containerInnerHeight / this.$scrollableContent.height(),

            newTop = Math.min(scrollBarTop, containerInnerHeight - Math.floor(containerInnerHeight * factor));

        this.$scrollBar.css('top', newTop);
        this.$scrollableContent.css('top', Math.floor(-newTop / factor));
    };

    ScrollBar.prototype.recalculateSize = function () {
        var containerPadding = Util.getElementPadding(this.$scrollableContainer),
            contentHeight = this.$scrollableContent.height();

        this.state.containerInnerHeight = this.$container.height() - (containerPadding.top + containerPadding.bottom);
        this.state.sizeDifference = contentHeight - this.state.containerInnerHeight;

        this.state.factor = this.state.containerInnerHeight / contentHeight;

        if (this.state.sizeDifference <= 0) {
            if (this.$scrollBarContainer.css('display') !== 'none')
                this.$scrollBarContainer.css('display', 'none');

            if (this.$scrollBar.height() !== 0)
                this.$scrollBar.height(0);

            this.state.disabled = true;

            return;
        }

        this.state.disabled = false;
        this.$scrollBarContainer.css('display', '');

        this.state.scrollBarHeight = Math.floor(this.state.containerInnerHeight * this.state.factor);

        this.$scrollBarContainer.height(this.state.containerInnerHeight);
        this.$scrollBar.height(this.state.scrollBarHeight);

        this.restoreScroll(parseInt(this.$scrollBar.css('top').replace('px', '')));
    };

    ScrollBar.prototype.updateOnMouseWheel = function (e) {
        if (this.state.disabled)
            return;

        var delta = e.wheelDelta || (e.originalEvent ? (-e.originalEvent.deltaY * 8) : -e.deltaY * 100),
            top = parseInt(this.$scrollableContent.css('top').replace('px', '')),
            newTop = Math.min(top + delta, 0);


        this._setScroll(newTop);
        Util.preventDefault(e);

        return false;
    };

    ScrollBar.prototype.setMinScroll = function (value) {
        this.minScroll = value;
    };

    ScrollBar.prototype.setMaxScroll = function (value) {
        this.maxScroll = value;
    };
});