TestCafeClient.define('UI.RecorderWidgets.SelectorEditor', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        ShadowUI = Hammerhead.ShadowUI,
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods,

        GeneratorRules = require('Recorder.SelectorGenerator.Rules'),

        CodeEditorWidget = require('UI.RecorderWidgets.CodeEditor'),
        SelectorSwitcherWidget = require('UI.RecorderWidgets.SelectorSwitcher'),
        PopupWidget = require('UI.RecorderWidgets.Popup'),
        ElementsMarkerWidget = require('UI.RecorderWidgets.ElementsMarker'),
        JavascriptExecutor = require('Base.JavascriptExecutor'),
        ValidationMessageFactory = require('UI.ValidationMessageFactory');

    //Const
    var SELECTOR_EDITOR_CLASS = 'selector-editor',
        SELECTOR_SWITCHER_CONTAINER_CLASS = 'selector-switcher-container',
        CODE_EDITOR_CONTAINER_CLASS = 'code-editor-container',
        ACTIVE_CLASS = 'active',

        SELECTOR_EDITOR_POPUP_TITLE = 'Target selector',
        SELECTOR_EDITOR_HEADER_FONT_SIZE = 18,

    //Selector editor validation errors/warnings
        EMPTY_SELECTOR = 'Selector doesn\'t contain any element',
        SEVERAL_ELEMENTS = 'Selector contains more than one element',
        SEVERAL_ELEMENTS_WITH_INVISIBLE = 'Selector contains more than one element (one or more are invisible)',
        INVISIBLE_ELEMENT = 'Selector contains an invisible element',

        CUSTOM_SELECTOR_DESCRIPTION = 'Custom expression';

    //SelectorEditor
    var SelectorEditor = this.exports = function ($container, options) {
        var selectorEditor = this;

        this.options = {
            width: options.width || '100%',
            height: options.height || '100%',
            selectors: options.selectors || [],
            currentSelectorIndex: options.currentSelectorIndex || 0,
            allowEdit: options.allowEdit || false,
            expandDirection: options.expandDirection || 'right',
            enableFloatMode: options.enableFloatMode || false,
            enableValidation: options.enableValidation || false,
            $floatingParent: options.$floatingParent || null,
            enableElementsMarking: options.enableElementsMarking || false,
            allowVisibleElementsOnly: options.allowVisibleElementsOnly || false,
            context: options.context || null,
            parseDomElementsOrJqueryObjectsOnly: options.parseDomElementsOrJqueryObjectsOnly || false,
            allowMultipleElements: options.allowMultipleElements || false
        };

        if (typeof this.options.width === 'string' && this.options.width.indexOf('%') > -1) {
            var width = parseInt(this.options.width.replace('%', ''));

            if (!isNaN(width))
                this.options.width = Math.round($container.width() * width / 100);
        }

        if (typeof this.options.height === 'string' && this.options.height.indexOf('%') > -1) {
            var height = parseInt(this.options.height.replace('%', ''));

            if (!isNaN(height))
                this.options.height = Math.round($container.height() * height / 100);
        }

        this.$selectorEditor = null;
        this.$selectorSwitcherContainer = null;
        this.$codeEditorContainer = null;

        this.selectorSwitcher = null;
        this.codeEditor = null;
        this.selectorEditorPopup = null;

        this.parsedSelector = null;
        this.hasJSErrors = false;
        this.hasVisibilityErrors = false;

        this.eventEmitter = new Util.EventEmitter();

        this.$container = $container;

        this._createMarkup();
        this._initBehavior();

        JavascriptExecutor.parseSelector(this.codeEditor.getText(), selectorEditor.options.parseDomElementsOrJqueryObjectsOnly, function (parsedSelector) {
            selectorEditor.parsedSelector = parsedSelector;
            selectorEditor._validate();
            selectorEditor._markElements();
        }, this.options.context);
    };

    //Events
    SelectorEditor.INIT_EVENT = 'init';
    SelectorEditor.SELECTOR_CHANGED_EVENT = 'selectorChanged';
    SelectorEditor.FOCUS_EVENT = 'focus';
    SelectorEditor.BLUR_EVENT = 'blur';

    //Markup
    SelectorEditor.prototype._createMarkup = function () {
        this.$selectorEditor = $('<div></div>').appendTo(this.$container);
        ShadowUI.addClass(this.$selectorEditor, SELECTOR_EDITOR_CLASS);

        this.$selectorSwitcherContainer = $('<div></div>').appendTo(this.$selectorEditor);
        ShadowUI.addClass(this.$selectorSwitcherContainer, SELECTOR_SWITCHER_CONTAINER_CLASS);

        this.$codeEditorContainer = $('<div></div>').appendTo(this.$selectorEditor);
        ShadowUI.addClass(this.$codeEditorContainer, CODE_EDITOR_CONTAINER_CLASS);

        this.selectorSwitcher = new SelectorSwitcherWidget(this.$selectorEditor, this.$selectorSwitcherContainer,
            this.options.selectors, this.options.currentSelectorIndex, this.options.height);

        var $codeEditor = $('<div></div>').appendTo(this.$codeEditorContainer);

        this.codeEditor = new CodeEditorWidget($codeEditor, {
            width: this.options.width - this.$selectorSwitcherContainer.width(),
            height: this.options.height,
            text: this.options.selectors[this.options.currentSelectorIndex].selector,
            allowEdit: this.options.allowEdit,
            expandDirection: this.options.expandDirection
        });
    };

    //Behavior
    SelectorEditor.prototype._markElements = function () {
        var selectorEditor = this;

        if (this.options.enableElementsMarking) {
            var $base = (this.selectorEditorPopup && this.selectorEditorPopup.getContainer()) || this.options.$floatingParent || this.$selectorEditor;

            if (this.parsedSelector && this.parsedSelector.visibleLength) {
                if (this.options.context)
                    ElementsMarkerWidget.markInContext($base, this.parsedSelector.selector, this.options.context, this.options.parseDomElementsOrJqueryObjectsOnly);
                else
                    ElementsMarkerWidget.mark($base, this.parsedSelector.$visibleElements);
            }
            else
                ElementsMarkerWidget.mark($base, $());

            ElementsMarkerWidget.events.on(ElementsMarkerWidget.ELEMENT_RECTANGLES_COUNT_CHANGED, function () {
                //NOTE: we need reparse selector because the ELEMENT_RECTANGLES_COUNT_CHANGED event means than
                //count of visible elements changed
                JavascriptExecutor.parseSelector(selectorEditor.codeEditor.getText(), selectorEditor.options.parseDomElementsOrJqueryObjectsOnly, function (parsedSelector) {
                    selectorEditor.parsedSelector = parsedSelector;
                    selectorEditor._validate();
                }, selectorEditor.options.context);
            });
        }
    };

    SelectorEditor.prototype._onCodeEditorChange = function () {
        var selectorEditor = this,
            text = this.codeEditor.getText();

        selectorEditor.inChanging = true;

        var selectors = selectorEditor.options.selectors,
            curIndex = selectorEditor.options.currentSelectorIndex;

        if (selectors[curIndex].selector !== text) {
            var i = 0,
                exist = false;

            for (; i < selectors.length; i++) {
                if (selectors[i].selector === text) {
                    selectorEditor.selectorSwitcher._setValue(selectors[i]);
                    selectorEditor.options.currentSelectorIndex = i;
                    exist = true;
                }
            }

            if (!exist) {
                for (i = 0; i < selectors.length; i++) {
                    if (selectors[i].id === GeneratorRules.ruleIDs.USER_INPUT) {
                        selectors[i].selector = text;
                        selectorEditor.selectorSwitcher._setValue(selectors[i]);
                        selectorEditor.options.currentSelectorIndex = i;
                        exist = true;
                    }
                }
            }

            if (!exist) {
                var customSelector = {id: GeneratorRules.ruleIDs.USER_INPUT, selector: text, description: CUSTOM_SELECTOR_DESCRIPTION};

                selectors.splice(0, 0, customSelector);
                selectorEditor.selectorSwitcher._setValue(selectors[0]);
                selectorEditor.options.currentSelectorIndex = 0;
            }
        }

        JavascriptExecutor.parseSelector(text, selectorEditor.options.parseDomElementsOrJqueryObjectsOnly, function (parsedSelector) {
            selectorEditor.parsedSelector = parsedSelector;
            selectorEditor._validate();

            selectorEditor.eventEmitter.emit(SelectorEditor.SELECTOR_CHANGED_EVENT, {
                text: text,
                index: selectorEditor.options.currentSelectorIndex,
                selectors: selectorEditor.options.selectors,
                parsedSelector: selectorEditor.parsedSelector
            });

            selectorEditor._markElements();

            selectorEditor.inChanging = false;
        }, selectorEditor.options.context);
    };

    SelectorEditor.prototype._initBehavior = function () {
        var selectorEditor = this;
        this.inChanging = false;

        this.selectorSwitcher.on(SelectorSwitcherWidget.VALUE_CHANGED_EVENT, function (e) {
            if (e.value) {
                selectorEditor.options.currentSelectorIndex = selectorEditor.options.selectors.indexOf(e.value);

                if (!selectorEditor.inChanging) {
                    selectorEditor.codeEditor.setText(e.value.selector);
                    selectorEditor.codeEditor._textChanged();
                }
            }
        });

        this.codeEditor.events.on(CodeEditorWidget.CHANGE_EVENT, function (e) {
            selectorEditor._onCodeEditorChange(e.text);
        });


        if (this.options.enableFloatMode) {
            this.codeEditor.events.on(CodeEditorWidget.FOCUS_EVENT, function () {
                selectorEditor._enableFloat();

                if (!selectorEditor.floatModifications)
                    selectorEditor.eventEmitter.emit(SelectorEditor.FOCUS_EVENT, {});
            });

            this.codeEditor.events.on(CodeEditorWidget.BLUR_EVENT, function () {
                selectorEditor._disableFloat();

                if (!selectorEditor.floatModifications)
                    selectorEditor.eventEmitter.emit(SelectorEditor.BLUR_EVENT, {});
            });
        }
    };

    SelectorEditor.prototype._enableFloat = function () {
        if (this.float || this.floatModifications)
            return;

        this.floatModifications = true;
        this.float = true;

        var selectorEditor = this,
            $window = $(window),
            codeEditorPosition = this.$codeEditorContainer.offset();

        this.selectorEditorPopup = new PopupWidget(ShadowUI.getRoot(), {
            headerText: SELECTOR_EDITOR_POPUP_TITLE,
            headerHideIcon: true,
            content: this.$codeEditorContainer,
            headerTextSize: SELECTOR_EDITOR_HEADER_FONT_SIZE,
            notDialog: true,
            restoreActiveElement: false
        });

        //NOTE: we should call it async because of async 'focus' function in IE
        NativeMethods.setTimeout.call(window, function () {
            selectorEditor.floatModifications = false;
            selectorEditor.codeEditor.focus();
        }, 0);

        var $popupContainer = this.selectorEditorPopup.getContainer(),
            topCorrection = this.selectorEditorPopup.getHeaderHeight() +
                ($popupContainer.outerHeight() - $popupContainer.innerHeight()) / 2,
            leftCorrection = ($popupContainer.outerWidth() - $popupContainer.innerWidth()) / 2;

        this.selectorEditorPopup.moveTo(codeEditorPosition.left - leftCorrection - $window.scrollLeft(),
            codeEditorPosition.top - topCorrection - $window.scrollTop());

        ShadowUI.addClass(this.selectorEditorPopup.getContainer(), ACTIVE_CLASS);

        selectorEditor.selectorEditorPopup.on(PopupWidget.HIDE_BUTTON_CLICK_EVENT, function () {
            selectorEditor._disableFloat(true);
        });

        if (this.options.$floatingParent) {
            var parentOffset = {
                left: codeEditorPosition.left - this.options.$floatingParent.offset().left,
                top: codeEditorPosition.top - this.options.$floatingParent.offset().top
            };

            selectorEditor.selectorEditorPopup.on(PopupWidget.MOVED_EVENT, function () {
                var curEditorPos = selectorEditor.$codeEditorContainer.offset();

                selectorEditor.options.$floatingParent.css({
                    left: curEditorPos.left - parentOffset.left - $window.scrollLeft(),
                    top: curEditorPos.top - parentOffset.top - $window.scrollTop()
                });
            });
        }

        this._markElements();
    };

    SelectorEditor.prototype._disableFloat = function (forceBlur) {
        if (!this.float || this.floatModifications)
            return;

        this.floatModifications = true;
        this.float = false;

        //NOTE: we call it to correct focus/blur events raising
        if (Util.isIE || Util.isMozilla)
            this.codeEditor.blur();

        this.$codeEditorContainer.css({
            left: 0,
            top: 0,
            position: ''
        }).appendTo(this.$selectorEditor);

        this.selectorEditorPopup.close();
        this.selectorEditorPopup = null;

        this._markElements();

        this.floatModifications = false;

        //NOTE: we call it to correct focus/blur events raising
        if (forceBlur && !Util.isIE)
            this.codeEditor.blur();
    };

    SelectorEditor.prototype._validate = function () {
        this._jsValidation();

        if (!this.hasJSErrors && this.options.enableValidation)
            this._elementsValidation();
    };

    SelectorEditor.prototype._jsValidation = function () {
        if (this.parsedSelector.error) {
            ValidationMessageFactory.error(this.codeEditor.getContainer(), this.parsedSelector.error);
            this.hasJSErrors = true;
        }
        else {
            ValidationMessageFactory.success(this.codeEditor.getContainer(), this.parsedSelector.error);
            this.hasJSErrors = false;
        }
    };

    SelectorEditor.prototype._elementsValidation = function () {
        this.hasVisibilityErrors = false;

        if (!this.parsedSelector)
            return;

        if (!this.parsedSelector.length) {
            if (this.options.allowVisibleElementsOnly) {
                this.hasVisibilityErrors = true;
                ValidationMessageFactory.error(this.codeEditor.getContainer(), EMPTY_SELECTOR);
            }
            else
                ValidationMessageFactory.warning(this.codeEditor.getContainer(), EMPTY_SELECTOR);
            return;
        }

        if (this.parsedSelector.length > 1) {
            if (!this.options.allowMultipleElements)
                this.hasVisibilityErrors = true;

            var errMsg = this.parsedSelector.length !== this.parsedSelector.visibleLength ? SEVERAL_ELEMENTS_WITH_INVISIBLE : SEVERAL_ELEMENTS;

            if (this.options.allowMultipleElements)
                ValidationMessageFactory.warning(this.codeEditor.getContainer(), errMsg);
            else
                ValidationMessageFactory.error(this.codeEditor.getContainer(), errMsg);
            return;
        }

        if (this.parsedSelector.length !== this.parsedSelector.visibleLength) {
            if (this.options.allowVisibleElementsOnly) {
                this.hasVisibilityErrors = true;
                ValidationMessageFactory.error(this.codeEditor.getContainer(), INVISIBLE_ELEMENT);
            }
            else
                ValidationMessageFactory.warning(this.codeEditor.getContainer(), INVISIBLE_ELEMENT);
            return;
        }

        ValidationMessageFactory.success(this.codeEditor.getContainer());
    };

    //API
    SelectorEditor.prototype.on = function (event, handler) {
        this.eventEmitter.on(event, handler);
    };

    SelectorEditor.prototype.getParsedSelector = function () {
        return this.parsedSelector;
    };

    SelectorEditor.prototype.isValid = function (skipVisibilityErrors) {
        if (skipVisibilityErrors)
            return !this.hasJSErrors;
        else
            return !this.hasJSErrors && !this.hasVisibilityErrors;
    };

    SelectorEditor.prototype.destroy = function () {
        ElementsMarkerWidget.clear();
    };

    SelectorEditor.prototype.setContext = function (context) {
        this.options.context = context;

        this._onCodeEditorChange();
    };

    SelectorEditor.prototype.enableElementsMarking = function () {
        this.options.enableElementsMarking = true;
        this._markElements();
    };
});