TestCafeClient.define('UI.RecorderWidgets.SelectorSwitcher', function (require) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        ScrollBarWidget = require('UI.RecorderWidgets.ScrollBar');

    //Const
    var SELECTOR_SWITCHER_CLASS = 'selector-switcher',
        BAR_CLASS = 'bar',
        ARROW_CLASS = 'arrow',
        DROP_DOWN_CLASS = 'drop-down',
        ITEM_HOVERED_CLASS = 'hovered',
        ITEM_LIST_CLASS = 'item-list',
        ITEMS_CLASS = 'items',
        ITEM_CLASS = 'item',
        ICON_CLASS = 'icon',
        ITEM_DESCRIPTION_CLASS = 'description',
        TEXT_CONTAINER_CLASS = 'text-container',
        HIGHLIGHTED_TEXT_CLASS = 'highlighted',
        ICON_CLASS_PREFIX = 'icon-',
        ICON_GREY_CLASS_PREFIX = 'icon-grey-',
        SEPARATOR_CLASS = 'separator',
        GRADIENT_CLASS = 'gradient',

        MIN_GRADIENT_HEIGHT = 80,
        MIN_DROPDOWN_HEIGHT = 150;

    var SelectorSwitcher = this.exports = function ($selectorArea, $selectorSwitcherContainer, items, currentIndex, height) {
        var switcher = this;

        this.$selectorArea = $selectorArea;
        this.$selectorSwitcherContainer = $selectorSwitcherContainer;

        this.$switcher = $('<div></div>')
            .appendTo($selectorSwitcherContainer);
        ShadowUI.addClass(this.$switcher, SELECTOR_SWITCHER_CLASS);

        if (height)
            this.$switcher.height(height);

        this.$bar = null;
        this.$dropDown = null;
        this.$items = null;
        this.items = items.slice(0);
        this.value = null;
        this.dropDownOpened = false;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            switcher.eventEmitter.on(ev, listener);
        };

        this._createBar();
        this._setValue(items[currentIndex]);
    };

    //Events
    SelectorSwitcher.VALUE_CHANGED_EVENT = 'valueChanged';

    //Markup
    SelectorSwitcher.prototype._createBar = function () {
        var switcher = this,
            $gradient = null;

        this.$bar = $('<div></div>')
            .appendTo(this.$switcher);
        ShadowUI.addClass(this.$bar, BAR_CLASS);

        var $icon = $('<div></div>')
            .appendTo(this.$bar);
        ShadowUI.addClass($icon, ICON_CLASS);

        switcher.$barIconImg = $('<div></div>')
            .appendTo($icon);

        var $arrow = $('<div></div>')
            .appendTo(this.$bar);
        ShadowUI.addClass($arrow, ARROW_CLASS);

        if (this.$bar.height() >= MIN_GRADIENT_HEIGHT) {
            $gradient = $('<div></div>').appendTo(this.$bar);

            ShadowUI.addClass($gradient, GRADIENT_CLASS);

            $gradient.css('left', this.$selectorSwitcherContainer.position().left + 'px');
            $gradient.css('top', (this.$selectorSwitcherContainer.position().top + this.$selectorSwitcherContainer.height() - $gradient.height()) + 'px');
        }

        var onClick = function (e) {
            if (e.button === Util.BUTTON.LEFT || Util.hasTouchEvents) {
                if (switcher.dropDownOpened) {
                    switcher._hideItemList();
                }
                else {
                    switcher._showItemList();
                }
            }
            Util.preventDefault(e);
        };

        ShadowUI.bind(this.$bar, 'click', onClick);

        ShadowUI.bind(this.$switcher, 'keydown', function (e) {
            var keyCode = e.keyCode;
            if (Util.isCharByKeyCode(keyCode, Util.isShadowUIElement) || /^(backspace|delete)$/.test(Util.KEYS_MAPS.REVERSED_SPECIAL_KEYS[keyCode])) {
                Util.preventDefault(e);
                return;
            }

            if (/^(down|right)$/.test(Util.KEYS_MAPS.REVERSED_SPECIAL_KEYS[keyCode])) {
//TODO
                Util.preventDefault(e);
            }
            else if (/^(up|left)$/.test(Util.KEYS_MAPS.REVERSED_SPECIAL_KEYS[keyCode])) {
//TODO
                Util.preventDefault(e);
            }
            else if (/^(enter|esc)$/.test(Util.KEYS_MAPS.REVERSED_SPECIAL_KEYS[keyCode]) && switcher.dropDownOpened) {
                switcher._hideItemList();
                Util.preventDefault(e);
            }
        });
    };

    SelectorSwitcher.prototype._createItemList = function () {
        var switcher = this;

        this.$dropDown = $('<div></div>');
        this.$dropDown.css('visibility', 'hidden');
        this.$dropDown.appendTo(this.$switcher);

        ShadowUI.addClass(this.$dropDown, DROP_DOWN_CLASS);
        var dropDownPadding = Util.getElementPadding(this.$dropDown);
        var dropDownBorders = Util.getBordersWidth(this.$dropDown);
        this.$dropDown.css('width', (this.$selectorArea.width() - dropDownPadding.left - dropDownPadding.right - dropDownBorders.left - dropDownBorders.right) + 'px');
        this.$dropDown.css('max-height', Math.max(MIN_DROPDOWN_HEIGHT,
            (this.$selectorArea.height() - dropDownPadding.top - dropDownPadding.bottom - dropDownBorders.top - dropDownBorders.bottom)) + 'px');
        var barArrowPosition = ShadowUI.select('.' + ARROW_CLASS, switcher.$bar).position();
        this.$dropDown.css({
            left: barArrowPosition.left,
            top: barArrowPosition.top
        });

        this.scrollBar = new ScrollBarWidget(this.$dropDown);

        var $itemList = $('<div></div>').appendTo(this.$dropDown);
        ShadowUI.addClass($itemList, ITEM_LIST_CLASS);

        this.$items = $('<div></div>').appendTo($itemList);
        ShadowUI.addClass(this.$items, ITEMS_CLASS);

        $.each(switcher.items, function (index, item) {
            var $item = $('<div></div>')
                .data('item', item)
                .appendTo(switcher.$items);
            ShadowUI.addClass($item, ITEM_CLASS);

            $item.mouseenter(function () {
                switcher._setHoveredItem($item);
            });

            $item.click(function () {
                switcher._setValue($item.data('item'));
                switcher._hideItemList();
            });

            var $icon = $('<div></div>').appendTo($item);
            ShadowUI.addClass($icon, ICON_CLASS);

            var $iconImg = $('<div></div>').appendTo($icon);
            ShadowUI.addClass($iconImg, ICON_GREY_CLASS_PREFIX + item.id);

            var $description = $('<div></div>').appendTo($item);
            ShadowUI.addClass($description, ITEM_DESCRIPTION_CLASS);
            var $textContainer = $('<div>' + item.description + '</div>');
            ShadowUI.addClass($textContainer, TEXT_CONTAINER_CLASS);
            //we have to set width here manually to make 'word-wrap' css property work
            $textContainer.width($description.width());
            $textContainer.appendTo($description);
            ShadowUI.addClass($textContainer.find('span'), HIGHLIGHTED_TEXT_CLASS);

            if (index !== switcher.items.length - 1) {
                var $separator = $('<div></div>').appendTo(switcher.$items);
                ShadowUI.addClass($separator, SEPARATOR_CLASS);
            }
        });

        this.scrollBar.init($itemList, this.$items);
        if (this.scrollBar.isVisible())
            $.each(this.$items.children(), function () {
                $(this).css('padding-right', '0px');
            });

        this._setDropDownSelectedItem(this.value);
        this._setHoveredItem(this._getDropDownItem(this.value));
    };

    SelectorSwitcher.prototype._setValue = function (value) {
        if (this.value === value)
            return;
        if (this.items.indexOf(value) < 0) {
            this.items.splice(0, 0, value);
            if (this.$dropDown) {
                this.$dropDown.remove();
                this.$dropDown = null;
            }
        }
        this._setBarSelectedItem(value);
        if (this.$dropDown) {
            this._setDropDownSelectedItem(value);
            this._setHoveredItem(this._getDropDownItem(value));
        }
        this.value = value;
        this.eventEmitter.emit(SelectorSwitcher.VALUE_CHANGED_EVENT, { value: value });
    };

    SelectorSwitcher.prototype._setBarSelectedItem = function (value) {
        if (this.value)
            ShadowUI.removeClass(this.$barIconImg, ICON_CLASS_PREFIX + this.value.id);
        ShadowUI.addClass(this.$barIconImg, ICON_CLASS_PREFIX + value.id);
    };

    SelectorSwitcher.prototype._setDropDownSelectedItem = function (value) {
        if (this.value) {
            var $selectedItemIcon = ShadowUI.select('.' + ICON_CLASS_PREFIX + this.value.id, this.$items);
            ShadowUI.removeClass($selectedItemIcon, ICON_CLASS_PREFIX + this.value.id);
            ShadowUI.addClass($selectedItemIcon, ICON_GREY_CLASS_PREFIX + this.value.id);
        }
        var $iconForSelect = ShadowUI.select('.' + ICON_GREY_CLASS_PREFIX + value.id, this.$items);
        ShadowUI.removeClass($iconForSelect, ICON_GREY_CLASS_PREFIX + value.id);
        ShadowUI.addClass($iconForSelect, ICON_CLASS_PREFIX + value.id);
    };

    SelectorSwitcher.prototype._getDropDownItem = function (value) {
        var $itemsCollection = this.$items.children();
        for (var i = 0; i < $itemsCollection.length; i++)
            if ($itemsCollection.eq(i).data('item') === value)
                return $itemsCollection.eq(i);
    };

    SelectorSwitcher.prototype._setHoveredItem = function ($item) {
        var $hoveredItem = ShadowUI.select('.' + ITEM_HOVERED_CLASS, this.$items);
        ShadowUI.removeClass($hoveredItem, ITEM_HOVERED_CLASS);
        ShadowUI.addClass($item, ITEM_HOVERED_CLASS);
    };

    SelectorSwitcher.prototype._showItemList = function () {
        var switcher = this;

        if (!switcher.$dropDown)
            switcher._createItemList();

        var unbindAndHideItemList = function () {
            switcher._hideItemList();
            ShadowUI.unbind($(document), 'mousedown', onDocumentMouseDown);
        };

        var onDocumentMouseDown = function (e) {
            var target = e.target || e.srcElement;
            if (!switcher.$switcher.has(target).length)
                unbindAndHideItemList();
        };

        this.$dropDown.css('visibility', '');

        ShadowUI.bind($(document), 'mousedown', onDocumentMouseDown);

        this.dropDownOpened = true;
    };

    SelectorSwitcher.prototype._hideItemList = function () {
        this.$dropDown.css('visibility', 'hidden');
        this.dropDownOpened = false;
    };
});