TestCafeClient.define('UI.RecorderWidgets.SetTestNameDialog', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        NativeMethods = Hammerhead.NativeMethods,
        TextSelection = Hammerhead.TextSelection,
        ValidationMessageFactory = require('UI.ValidationMessageFactory'),
        PopupWidget = require('UI.RecorderWidgets.Popup'),
        ButtonWidget = require('UI.RecorderWidgets.Button');

    //Const
    var TEST_NAME_INPUT_ID = '989b5ac6-476f-4c86-a53b-37c9decad66b-test-name',
        BUTTONS_CLASS = 'buttons',
        BUTTON_SMALL_CLASS = 'small',

        TEST_NAME_AREA_CLASS = 'test-name-area',
        TEST_NAME_CLASS = 'test-name',

        FIELD_CLASS = 'field',
        VALUE_CLASS = 'value',
        INPUT_CLASS = 'input',

    //Popup header text
        HEADER_TEXT = 'Save test',
        POPUP_MESSAGE_TEXT = 'Test name:';

    //Events
    exports.SAVE_TEST_BUTTON_CLICK_EVENT = 'saveTestButtonClick';
    exports.CANCEL_SAVE_BUTTON_CLICK_EVENT = 'cancelSaveButtonClick';

    //Globals
    var eventEmitter = null,
        popup = null,
        $saveTestButton = null,
        $cancelSaveButton = null,
        $testNameInput = null,
        selectedStepNum = null,
        actualStepNum = null;


    //Markup
    var createDialog = function ($container, args) {
        selectedStepNum = args.selectedStepNum;
        actualStepNum = args.actualStepNum;

        var popupOptions = {
            width: 608,
            content: createTestNameArea($('<div></div>')),
            headerText: HEADER_TEXT,
            footerContent: createButtons(),
            showAtWindowCenter: true
        };

        popup = new PopupWidget($container, popupOptions);

        init();
    };

    var createTestNameArea = function ($testNameArea) {
        ShadowUI.addClass($testNameArea, TEST_NAME_AREA_CLASS);

        var $testName = $('<div></div>').appendTo($testNameArea);
        ShadowUI.addClass($testName, TEST_NAME_CLASS);

        var $label = $('<label></label>').appendTo($testName)
            .text(POPUP_MESSAGE_TEXT)
            .attr('for', TEST_NAME_INPUT_ID);
        ShadowUI.addClass($label, FIELD_CLASS);

        if (Util.isWebKit || $.browser.opera) {
            $label.css('position', 'relative');
            $label.css('top', '1px');
        }

        var $value = $('<div></div>').appendTo($testName);
        ShadowUI.addClass($value, VALUE_CLASS);

        $testNameInput = $('<input type="text">')
            .appendTo($value)
            .attr('id', TEST_NAME_INPUT_ID);
        ShadowUI.addClass($testNameInput, INPUT_CLASS);

        NativeMethods.setTimeout.call(window, function () {
            NativeMethods.focus.call($testNameInput[0]);
            NativeMethods.setTimeout.call(window, function () {
                popup.changeFocusableElement($testNameInput);
            }, 0);
        }, 0);

        return $testNameArea;
    };

    var createButtons = function () {
        var $buttons = $('<div></div>');
        ShadowUI.addClass($buttons, BUTTONS_CLASS);

        $saveTestButton = ButtonWidget.create($buttons, 'Save');
        $cancelSaveButton = ButtonWidget.create($buttons, 'Cancel');

        ShadowUI.addClass($saveTestButton, BUTTON_SMALL_CLASS);
        ShadowUI.addClass($cancelSaveButton, BUTTON_SMALL_CLASS);

        return $buttons;
    };

//Behavior
    var init = function () {
        ShadowUI.bind($cancelSaveButton, 'click', function () {
            closeDialog(function () {
                eventEmitter.emit(exports.CANCEL_SAVE_BUTTON_CLICK_EVENT, {
                    selectedStepNum: selectedStepNum,
                    actualStepNum: actualStepNum
                });
            });
        });

        ShadowUI.bind($saveTestButton, 'click', function () {
            eventEmitter.emit(exports.SAVE_TEST_BUTTON_CLICK_EVENT, {testName: $testNameInput.val()});
        });

        popup.onkeydown(function (e) {
            var testNameStartSelection = TextSelection.getSelectionStart($testNameInput[0]),
                testNameEndSelection = TextSelection.getSelectionEnd($testNameInput[0]);

            if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.enter) {
                $saveTestButton.trigger('click');
                Util.preventDefault(e);
            }

            if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.esc) {
                $cancelSaveButton.trigger('click');
                Util.preventDefault(e);
            }

            //NOTE: we should prevent page scrolling in Chrome
            if (Util.isWebKit && testNameStartSelection === testNameEndSelection) {
                if ((e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.left && testNameStartSelection === 0) ||
                    (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.right && testNameStartSelection === $testNameInput[0].value.length))
                    Util.preventDefault(e);
            }
        });
    };

    var closeDialog = function (callback) {
        popup.close(callback);
    };

    //API
    exports.onError = function (err) {
        ValidationMessageFactory.error($testNameInput, err);
        NativeMethods.focus.call($testNameInput[0]);
    };


    exports.blind = function (blind) {
        popup.blind(blind);
    };

    exports.init = function ($container, args) {
        eventEmitter = new Util.EventEmitter();
        createDialog($container, args);
    };

    exports.closeDialog = function (callback) {
        popup.close(callback);
    };

    exports.events = {
        on: function (ev, listener) {
            eventEmitter.on(ev, listener);
        }
    };
});