TestCafeClient.define('UI.RecorderWidgets.StepsPanel', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        EventSandbox = Hammerhead.EventSandbox,
        ScrollBarWidget = require('UI.RecorderWidgets.ScrollBar'),
        BaseStepWidget = require('UI.RecorderWidgets.BaseStep'),
        AssertionsStepWidget = require('UI.RecorderWidgets.AssertionsStep'),
        ActionStepWidget = require('UI.RecorderWidgets.ActionStep'),
        SortingBehavior = require('UI.SortingBehavior'),
        ButtonWidget = require('UI.RecorderWidgets.Button'),
        ModalBackground = require('UI.ModalBackground'),
        TooltipWidget = require('UI.RecorderWidgets.Tooltip'),
        PopupWidget = require('UI.RecorderWidgets.Popup'),

        UXLog = require('UI.UXLog');

    var STEPS_PANEL_CLASS = 'steps-panel',
        STEP_LIST_CLASS = 'step-list-panel',
        SCROLLABLE_CLASS = 'scrollable',
        STEPS_PANEL_HIDDEN_CLASS = 'hidden',

        DISABLED_CLASS = 'disabled',
        LIST_CONTAINER_CLASS = 'container',
        LIST_CONTENT_CLASS = 'content',
        LIST_ITEM_CLASS = 'item',
        LIST_ITEM_SORTABLE_AREA = 'sortable-area',
        LIST_ITEM_SORTABLE_ICON_CONTAINER_AREA = 'sortable-icon-container',
        LIST_ITEM_SORTABLE_AREA_ICON = 'sortable-icon',
        LIST_ITEM_NUMBER_CLASS = 'number',
        LIST_ITEM_SEPARATOR_CLASS = 'list-separator',
        LIST_ITEM_ICON_CONTAINER_CLASS = 'icon-container',
        ACTION_ICON_CLASS = 'action-icon',
        ASSERTION_ICON_CLASS = 'assertion',
        LIST_ITEM_NAME_CLASS = 'name',
        ITEM_ARROW_RIGHT_CLASS = 'item-arrow-right',
        ITEM_ARROW_LEFT_CLASS = 'item-arrow-left',
        DRAGGING_ITEM_SHADOW_CLASS = 'dragging-item-shadow',

        DELETE_STEP_BUTTON_AREA_CLASS = 'delete-step-button-area',
        DELETE_STEP_BUTTON_CLASS = 'delete-step-button',
        DELETE_STEP_ICON_CLASS = 'delete-step-icon',
        DELETE_STEP_LABEL_CLASS = 'label',

        LIST_ITEM_OPENED_CLASS = 'opened',
        LIST_ITEM_SELECTED_CLASS = 'selected',
        LIST_ITEM_HIGHLIGHTED_CLASS = 'highlighted',
        FAILED_ITEM_CLASS = 'failed',

        STEP_INFO_PANEL_CLASS = 'stepInfo-panel',
        NO_STEPS_TITLE_CLASS = 'no-steps-title',
        DELETE_STEP_LABEL_TEXT = 'Delete step(s)',

        LAST_STEP_DELETION_WARNING = 'Deletion of the last step might affect the tested page\'s state making it not actual.' +
            '<br>Before adding new test steps, it is recommended to replay the test to check its integrity.',

        BLIND_CLASS = 'blind',
        BUTTONS_CLASS = 'buttons',

        DIALOG_POPUP_WIDTH = 475,
        MAXIMUM_MOUSE_MOVE_INDENT = 5;

    //Events
    exports.STEPS_INFO_CHANGED_EVENT = 'stepsInfoChanged';
    exports.ALL_STEPS_DELETED_EVENT = 'allStepsDeleted';
    exports.EDIT_ASSERTION_COMPLETE_EVENT = 'editAssertionComplete';
    exports.EDIT_ASSERTION_START_EVENT = 'editAssertionStart';
    exports.START_PICKING_ELEMENT_EVENT = 'startPickingElement';
    exports.SKIP_STEP_BUTTON_CLICK_EVENT = 'skipStepButtonClick';
    exports.REPLAY_STEP_BUTTON_CLICK_EVENT = 'replayStepButtonClick';
    exports.STEP_INFO_SHOWN_EVENT = 'stepInfoShown';
    exports.STEP_INFO_HIDDEN_EVENT = 'stepInfoHidden';
    exports.STEPS_PANEL_BLINDED_EVENT = 'stepsPanelBlinded';

    //Globals
    var eventEmitter = null,

        $stepsPanel = null,
        $stepList = null,
        $stepsContainer = null,
        $stepsContent = null,
        $deleteStepButtonArea = null,
        $deleteStepButton = null,
        $noStepsTitleDiv = null,
        scrollBar = null,
        $itemArrow = null,
        $dragItemShadow = null,
        $replayStepButton = null,
        $skipStepButton = null,

        popup = null,
        $stepInfo = null,
        step = null,

        stepsInfo = null,
        listItems = [],
        items = [],

        validateStepNum = null,
        validatedStepItem = null,

        openedStepNum = null,
        highlightedStepNum = null,
        selectedStepNums = [],
        startItemSelectionNum = null,

        playbackMode = false,
        stepHasChanges = false,
        stepPanelHasChanges = false,
        isVisible = false,
        codeEditorSelectionStarted = false,

        $window = $(window),
        updateStepInfoPanelPosition = null,
        holdStepInfo = false,
        mousedownCoordY = null,
        mousedownEvent = null;

    //Markup
    function initHandlers() {
        setListItemsHandlers();

        SortingBehavior.init($stepsContent, {
            itemClass: LIST_ITEM_CLASS,
            draggableClass: LIST_ITEM_CLASS,
            waitSortingStart: true,
            handlers: {
                onItemMoved: function ($item) {
                    scrollBar.updateScroll($item[0]);
                    $dragItemShadow.css('top', $item.offset().top - $window.scrollTop());
                },

                onMouseWheel: function (e) {
                    scrollBar.updateOnMouseWheel(e);
                },

                onDragStart: function ($item, index) {
                    selectItem(index);

                    var borderWidth = 2,
                        borderRule = borderWidth + 'px solid ' + $item.css('backgroundColor'),
                        itemOffset = $item.offset();

                    $dragItemShadow = $('<div></div>')
                        .css({
                            width: $item.width(),
                            height: $item.height(),
                            top: itemOffset.top - $window.scrollTop(),
                            left: itemOffset.left - $window.scrollLeft() - borderWidth,
                            borderLeft: borderRule,
                            borderRight: borderRule
                        })
                        .appendTo(ShadowUI.getRoot());

                    ShadowUI.addClass($dragItemShadow, DRAGGING_ITEM_SHADOW_CLASS);
                },

                onDragEnd: function (oldIndex, newIndex) {
                    $dragItemShadow.remove();
                    $dragItemShadow = null;
                    validateStepNum = null;

                    var item = stepsInfo.splice(oldIndex, 1)[0];
                    stepsInfo.splice(newIndex, 0, item);

                    item = listItems.splice(oldIndex, 1)[0];
                    listItems.splice(newIndex, 0, item);

                    item = items.splice(oldIndex, 1)[0];
                    items.splice(newIndex, 0, item);

                    if (openedStepNum !== null)
                        openedStepNum = $.inArray(getOpenedItem()[0], listItems);
                    else if (selectedStepNums.length) {
                        var $selectedItems = getSelectedItems();

                        $.each($selectedItems, function (index, item) {
                            selectedStepNums[index] = $.inArray(item, listItems);
                        });
                    }

                    if (playbackMode)
                        validateStepNum = $.inArray(validatedStepItem, listItems);

                    updateStepsNumbering();
                    UXLog.write('Steps panel: an item is moved');
                    eventEmitter.emit(exports.STEPS_INFO_CHANGED_EVENT, {});
                }
            }
        });
    }

    function createStepsPanel($container, enableHandlers) {
        $stepList = $('<div></div>').appendTo($container);
        ShadowUI.addClass($stepList, STEP_LIST_CLASS);
        ShadowUI.addClass($stepList, SCROLLABLE_CLASS);

        createStepList(enableHandlers);

        if (enableHandlers)
            initHandlers();
    }

    function createStepList(enableHandlers) {
        $stepsContainer = $('<div></div>').appendTo($stepList);
        ShadowUI.addClass($stepsContainer, LIST_CONTAINER_CLASS);

        $stepsContent = $('<div></div>').appendTo($stepsContainer);
        ShadowUI.addClass($stepsContent, LIST_CONTENT_CLASS);

        $.each(stepsInfo, function (index) {
            createItem(index);
        });

        scrollBar = new ScrollBarWidget($stepList, function () {
            if (updateStepInfoPanelPosition)
                updateStepInfoPanelPosition();
        });
        scrollBar.init($stepsContainer, $stepsContent);

        if (selectedStepNums.length && listItems.length)
            selectItem(selectedStepNums[0]);
        else
            showNoStepsTitle();

        createDeleteStepButton();

        if (listItems.length && enableHandlers)
            showDeleteStepButton();
        else
            hideDeleteStepButton();
    }

    function createItem(stepNum) {
        var stepInfo = stepsInfo[stepNum];

        var $item = $('<div></div>').appendTo($stepsContent);
        ShadowUI.addClass($item, LIST_ITEM_CLASS);

        if (stepInfo.failed)
            ShadowUI.addClass($item, FAILED_ITEM_CLASS);

        var $sortableArea = $('<div></div>').appendTo($item);
        ShadowUI.addClass($sortableArea, LIST_ITEM_SORTABLE_AREA);

        var $sortableIconContainer = $('<div></div>').appendTo($sortableArea);
        ShadowUI.addClass($sortableIconContainer, LIST_ITEM_SORTABLE_ICON_CONTAINER_AREA);

        var $sortableAreaIcon = $('<div></div>').appendTo($sortableIconContainer);
        ShadowUI.addClass($sortableAreaIcon, LIST_ITEM_SORTABLE_AREA_ICON);

        var $number = $('<div>').appendTo($sortableArea);
        ShadowUI.addClass($number, LIST_ITEM_NUMBER_CLASS);
        $number.text(stepNum + 1);

        var $separator = $('<div></div>').appendTo($sortableArea);
        ShadowUI.addClass($separator, LIST_ITEM_SEPARATOR_CLASS);

        var $icon = $('<div></div>').appendTo($item);
        ShadowUI.addClass($icon, LIST_ITEM_ICON_CONTAINER_CLASS);
        var $iconImg = $('<div></div>').appendTo($icon);

        ShadowUI.addClass($iconImg, ACTION_ICON_CLASS);

        if (stepInfo.isAssertion)
            ShadowUI.addClass($iconImg, ASSERTION_ICON_CLASS);
        else
            ShadowUI.addClass($iconImg, stepInfo.actionDescriptor.type.toLocaleLowerCase());

        var $name = $('<div></div>').appendTo($item);
        ShadowUI.addClass($name, LIST_ITEM_NAME_CLASS);
        $name.text(stepInfo.name);

        listItems.push($item[0]);
        items.push({
            $item: $item,
            $sortableArea: $sortableArea,
            $number: $number,
            $separator: $separator,
            $icon: $icon,
            $name: $name
        });
    }

    function createDeleteStepButton() {
        $deleteStepButtonArea = $('<div></div>').appendTo($stepsPanel);
        $deleteStepButton = $('<div></div>').appendTo($deleteStepButtonArea);

        var $icon = $('<div></div>').appendTo($deleteStepButton),
            $title = $('<div></div>').text(DELETE_STEP_LABEL_TEXT).appendTo($deleteStepButton);

        ShadowUI.addClass($deleteStepButtonArea, DELETE_STEP_BUTTON_AREA_CLASS);
        ShadowUI.addClass($deleteStepButton, DELETE_STEP_BUTTON_CLASS);
        ShadowUI.addClass($icon, DELETE_STEP_ICON_CLASS);
        ShadowUI.addClass($title, DELETE_STEP_LABEL_CLASS);

        ShadowUI.bind($deleteStepButton, 'click', deleteItems);
    }

    function showDeleteStepButton() {
        if ($deleteStepButtonArea)
            $deleteStepButtonArea.css('visibility', 'visible');
    }

    function hideDeleteStepButton() {
        if ($deleteStepButtonArea)
            $deleteStepButtonArea.css('visibility', 'hidden');
    }

    function showNoStepsTitle() {
        $noStepsTitleDiv = $('<div></div>').appendTo($stepsContent);
        ShadowUI.addClass($noStepsTitleDiv, NO_STEPS_TITLE_CLASS);

        $('<div></div>').appendTo($noStepsTitleDiv).text('There are');
        $('<div></div>').appendTo($noStepsTitleDiv).text('no recorded steps');
    }

    function removeNoStepsTitleDiv() {
        if ($noStepsTitleDiv && $noStepsTitleDiv.length) {
            $noStepsTitleDiv.remove();
            $noStepsTitleDiv = null;
        }
    }

    function getParentItem(el) {
        var $parents = $(el).parents(),
            $item = null;

        if (ShadowUI.hasClass($(el), LIST_ITEM_CLASS))
            return $(el);

        $.each($parents, function (index, parent) {
            if (ShadowUI.hasClass($(parent), LIST_ITEM_CLASS)) {
                $item = $(parent);
                return false;
            }
        });

        return $item;
    }

    function getOpenedItem() {
        return ShadowUI.select('.' + LIST_ITEM_CLASS + '.' + LIST_ITEM_OPENED_CLASS);
    }

    function getSelectedItems() {
        return ShadowUI.select('.' + LIST_ITEM_CLASS + '.' + LIST_ITEM_SELECTED_CLASS);
    }

    function getHighlightedItem() {
        return ShadowUI.select('.' + LIST_ITEM_CLASS + '.' + LIST_ITEM_HIGHLIGHTED_CLASS);
    }

    function updateStepsNumbering() {
        $.each(items, function (index, item) {
            if ($.trim(item.$number.text()) !== (index + 1).toString())
                item.$number.text(index + 1);
        });
    }

    function updateItemArrow() {
        var $openedItem = getOpenedItem();

        if ($itemArrow) {
            $itemArrow.remove();
            $itemArrow = null;
        }

        if (isVisible && $openedItem.length) {
            var itemOffset = $openedItem.offset(),
                itemPos = $openedItem.position(),
                isRight = popup.getPosition().left >= itemOffset.left;

            $itemArrow = $('<div></div>').appendTo(ShadowUI.getRoot());
            ShadowUI.addClass($itemArrow, isRight ? ITEM_ARROW_RIGHT_CLASS : ITEM_ARROW_LEFT_CLASS);

            if (ShadowUI.hasClass($openedItem, FAILED_ITEM_CLASS))
                ShadowUI.addClass($itemArrow, FAILED_ITEM_CLASS);

            $itemArrow.css({
                left: (isRight ? itemOffset.left + $openedItem.width() : itemOffset.left - $itemArrow.width()) - $window.scrollLeft(),
                top: itemOffset.top - $window.scrollTop()
            });

            scrollBar.setMinScroll($openedItem.height() + itemPos.top - $stepsPanel.height() + $deleteStepButtonArea[0].offsetHeight);
            scrollBar.setMaxScroll(itemPos.top);
        }
        else {
            scrollBar.setMinScroll(null);
            scrollBar.setMaxScroll(null);
        }
    }

    //Behavior
    function mousemoveEventHandler(e) {
        var coordY = e.pageY || e.y;

        if (Math.abs(coordY - mousedownCoordY) > MAXIMUM_MOUSE_MOVE_INDENT) {
            SortingBehavior.startSorting(mousedownEvent);
            mouseupEventHandler();
        }
    }

    function mouseupEventHandler() {
        mousedownEvent = null;
        mousedownCoordY = null;
        ShadowUI.unbind($stepsContent, 'mousemove', mousemoveEventHandler);
        ShadowUI.unbind($(document), 'mouseup', mouseupEventHandler);
    }

    function setListItemsHandlers() {
        ShadowUI.bind($stepsContent, 'mousedown', function (e) {
            var $curTarget = getParentItem(e.target),
                index = $curTarget && $curTarget.length ? $.inArray($curTarget[0], listItems) : -1;

            if (e.button !== Util.BUTTON.LEFT || index === -1)
                return;

            if (ShadowUI.hasClass($(e.target), LIST_ITEM_SORTABLE_AREA) || items[index].$sortableArea.has(e.target).length) {
                SortingBehavior.startSorting(e);
                return;
            }

            if (!e.ctrlKey && !e.shiftKey) {
                mousedownEvent = e;
                mousedownCoordY = e.pageY || e.y;

                ShadowUI.bind($stepsContent, 'mousemove', mousemoveEventHandler);
                ShadowUI.bind($(document), 'mouseup', mouseupEventHandler);
            }
        });

        ShadowUI.bind($stepsContent, 'click', function (e) {
            var $curTarget = getParentItem(e.target),
                index = $curTarget && $curTarget.length ? $.inArray($curTarget[0], listItems) : -1;

            if (e.button !== Util.BUTTON.LEFT || index === -1 ||
                ShadowUI.hasClass($(e.target), LIST_ITEM_SORTABLE_AREA) ||
                items[index].$sortableArea.has(e.target).length)
                return;

            if (e.ctrlKey || e.shiftKey) {
                if (!playbackMode && index !== openedStepNum) {
                    if (e.ctrlKey) {
                        if ($.inArray(index, selectedStepNums) === -1) {
                            UXLog.write('Steps panel: an item is selected');
                            addItemToSelected(index, true);
                        }
                        else if (selectedStepNums.length > 1) {
                            UXLog.write('Steps panel: an item is unselected');
                            removeItemFromSelected(index, true);
                        }

                        startItemSelectionNum = index;
                    }
                    else if ($.inArray(index, selectedStepNums) === -1 || selectedStepNums.length > 1) {
                        UXLog.write('Steps panel: several items are selected');
                        addItemsToSelected(startItemSelectionNum, index);
                    }
                }
            }
            else {
                if (openedStepNum === null || index !== openedStepNum || !popup) {
                    UXLog.write('Steps panel: an item is opened');
                    openItem(index);
                }
                else {
                    UXLog.write('Steps panel: an item is closed');
                    exports.closeItem();
                    if (playbackMode)
                        highlightItem(index);
                    else
                        selectItem(index);
                }
            }

            Util.preventDefault(e, true);
        });
    }

    function deleteItem($item) {
        var index = $.inArray($item[0], listItems),
            scrollBarTop = parseInt(scrollBar.$scrollBar.css('top').replace('px', '')),
            scrollContentTop = parseInt($stepsContent.css('top').replace('px', '')),

            itemHeight = $item.height(),
            itemMargin = Util.getElementMargin($item),

            openedStepItem = getOpenedItem(),

            isOpenedItem = index === openedStepNum;

        if (index <= validateStepNum)
            validateStepNum = null;

        if (index === stepsInfo.length - 1) {
            if (stepsInfo.length === 1)
                eventEmitter.emit(exports.ALL_STEPS_DELETED_EVENT);
            else if (!stepsInfo[index].isAssertion)
                TooltipWidget.show(LAST_STEP_DELETION_WARNING);
        }

        stepsInfo.splice(index, 1);

        if (isOpenedItem) {
            //NOTE: to save changed stepsInfo
            stepHasChanges = true;
            exports.closeItem();
        }

        $(listItems[index]).remove();
        listItems.splice(index, 1);
        items.splice(index, 1);

        updateStepsNumbering();

        if (!isOpenedItem && openedStepNum !== null)
            openedStepNum = $.inArray(openedStepItem, listItems);

        if (scrollContentTop < 0)
            $stepsContent.css('top', Math.min(scrollContentTop + itemHeight + itemMargin.top + itemMargin.bottom, 0));

        scrollBar.recalculateSize();

        if (stepsInfo.length && listItems && listItems.length) {
            index = Math.min(index, stepsInfo.length - 1);
            selectItem(index);

            //NOTE: we should  try to restore previous scroll value
            scrollBar.restoreScroll(scrollBarTop);
            scrollBar.updateScroll(listItems[index]);
        }
        else {
            selectedStepNums = [];
            startItemSelectionNum = null;
            showNoStepsTitle();
            hideDeleteStepButton();
        }
    }

    function deleteItems() {
        var $deletedItems = getSelectedItems();

        if (playbackMode || !$deletedItems.length)
            return;

        UXLog.write('Steps panel: delete step button is pressed');
        $.each($deletedItems, function (index, item) {
            deleteItem($(item));
        });


        eventEmitter.emit(exports.STEPS_INFO_CHANGED_EVENT, {});
    }

    function openItem(newIndex, stepError) {
        var $newOpenedItem = $(listItems[newIndex]),
            $oldSelectedItems = getSelectedItems(),
            $oldHighlightedItem = getHighlightedItem();

        exports.closeItem();

        openedStepNum = newIndex;
        selectedStepNums = [];
        startItemSelectionNum = newIndex;

        $.each($oldSelectedItems, function (index, item) {
            ShadowUI.removeClass($(item), LIST_ITEM_SELECTED_CLASS);
        });

        if ($oldHighlightedItem)
            ShadowUI.removeClass($oldHighlightedItem, LIST_ITEM_HIGHLIGHTED_CLASS);

        ShadowUI.addClass($newOpenedItem, LIST_ITEM_OPENED_CLASS);
        scrollBar.updateScroll($newOpenedItem[0]);

        if (stepError)
            ShadowUI.addClass($newOpenedItem, FAILED_ITEM_CLASS);

        showStepInfo(stepError);
    }

    function addItemToSelected(index, needUpdateScroll) {
        if ($.inArray(index, selectedStepNums) === -1) {
            selectedStepNums.push(index);
            ShadowUI.addClass($(listItems[index]), LIST_ITEM_SELECTED_CLASS);
        }

        if (needUpdateScroll)
            scrollBar.updateScroll($(listItems[index]));
    }

    function addItemsToSelected(startIndex, endIndex) {
        var isInverted = startIndex > endIndex,
            start = Math.min(startIndex, endIndex),
            end = Math.max(startIndex, endIndex),
            $oldSelectedItems = getSelectedItems(),
            itemIndex = null;

        $.each($oldSelectedItems, function (index, item) {
            itemIndex = $.inArray(item, listItems);

            if ((isInverted && (itemIndex > startIndex || itemIndex < endIndex)) ||
                (!isInverted && (itemIndex < startIndex || itemIndex > endIndex)))
                removeItemFromSelected(itemIndex);
        });

        for (var i = start; i <= end; i++)
            addItemToSelected(i);

        scrollBar.updateScroll($(listItems[endIndex]));
    }

    function removeItemFromSelected(index, needUpdateScroll) {
        var oldIndex = $.inArray(index, selectedStepNums);

        if (oldIndex !== -1)
            selectedStepNums.splice(oldIndex, 1);

        ShadowUI.removeClass($(listItems[index]), LIST_ITEM_SELECTED_CLASS);

        if (needUpdateScroll)
            scrollBar.updateScroll($(listItems[index]));
    }

    function selectItem(newIndex) {
        var $oldSelectedItems = getSelectedItems(),
            $oldHighlightedItem = getHighlightedItem();

        $.each($oldSelectedItems, function (index, item) {
            ShadowUI.removeClass($(item), LIST_ITEM_SELECTED_CLASS);
        });

        selectedStepNums = [];
        selectedStepNums.push(newIndex);
        startItemSelectionNum = newIndex;
        ShadowUI.addClass($(listItems[newIndex]), LIST_ITEM_SELECTED_CLASS);

        if ($oldHighlightedItem)
            ShadowUI.removeClass($oldHighlightedItem, LIST_ITEM_HIGHLIGHTED_CLASS);

        exports.closeItem();
        scrollBar.updateScroll($(listItems[newIndex]));
    }

    function highlightItem(newIndex) {
        var $newItem = $(listItems[newIndex]),
            $oldItem = getHighlightedItem(),
            $oldSelectedItems = getSelectedItems();

        highlightedStepNum = newIndex;
        selectedStepNums = [];
        startItemSelectionNum = null;

        if ($oldItem)
            ShadowUI.removeClass($oldItem, LIST_ITEM_HIGHLIGHTED_CLASS);

        $.each($oldSelectedItems, function (index, item) {
            ShadowUI.removeClass($(item), LIST_ITEM_SELECTED_CLASS);
        });

        ShadowUI.addClass($newItem, LIST_ITEM_HIGHLIGHTED_CLASS);
        scrollBar.updateScroll($newItem[0]);
    }

    function getPopupPosition($item, popupWidth) {
        var POPUP_OFFSET_TOP = -15,
            MIN_SPACE_FOR_POPUP_ON_THE_RIGHT = 90;

        var itemPos = $item.offset(),
            itemClientPos = Util.offsetToClientCoords({x: itemPos.left, y: itemPos.top}),
            itemWidth = $item.width(),
            windowWidth = $window.width(),

            left = itemClientPos.x + itemWidth,
            top = itemClientPos.y + POPUP_OFFSET_TOP,

            isLeft = (left + popupWidth > windowWidth && itemClientPos.x - popupWidth >= 0) ||
                $window.width() - left <= MIN_SPACE_FOR_POPUP_ON_THE_RIGHT;

        return {
            top: top,
            left: isLeft ? itemClientPos.x - popupWidth : left,
            minLeft: isLeft ? null : left
        };
    }

    function preventEvent(e, dispatched, prevent, cancelHandlers, stopPropagation) {
        //NOTE: we need to check is this event for ending of step list scrolling
        if (!Util.isShadowUIElement(e.target || e.srcElement) && !(e.type === 'mouseup' && scrollBar && scrollBar.state.inScroll)) {

            if (Util.isMozilla && e.type === 'mouseup' && codeEditorSelectionStarted) {
                codeEditorSelectionStarted = false;
                step.selectorEditor.codeEditor.stopSelectionProcess();
            }

            stopPropagation();
            prevent();
        }
    }

    function onPageUnload() {
        selectItem(openedStepNum);
        exports.closeItem();
    }

    function onStepPanelMousedownHandler(e, dispatched, prevent, cancelHandlers, stopPropagation) {
        var target = e.target || e.srcElement,
            $openedItem = getOpenedItem();

        if (step.selectorEditor && step.selectorEditor.codeEditor &&
            (step.selectorEditor.codeEditor.$editor[0] === e.target || step.selectorEditor.codeEditor.$editor.has(e.target).length))
            codeEditorSelectionStarted = true;

        //NOTE: we need prevent this event to prevent setting selection on testing page
        if (Util.isShadowUIElement(target) && $stepsPanel.has(target).length && !($openedItem.has(target).length || target === $openedItem[0])) {
            prevent();
            stopPropagation();
        }
    }

    function onClickOutsideStepInfoPanel(e, dispatched, prevent, cancelHandlers, stopPropagation) {
        var target = e.target || e.srcElement,
            $openedItem = getOpenedItem();

        if (!Util.isMozilla && codeEditorSelectionStarted) {
            codeEditorSelectionStarted = false;
            step.selectorEditor.codeEditor.stopSelectionProcess();
            prevent();
            stopPropagation();
        }
        else if (Util.isShadowUIElement(target)) {
            if (holdStepInfo || $stepInfo.has(target).length || ($openedItem[0] === target || $openedItem.has(target).length))
                return;
            else {
                selectItem(openedStepNum);
                UXLog.write('Steps panel: an item is closed');
                exports.closeItem();
            }
        }
        else {
            selectItem(openedStepNum);
            UXLog.write('Steps panel: an item is closed');
            exports.closeItem();
            prevent();
            stopPropagation();
        }
    }

    function showStepInfo(stepError, skipPageStateSaving) {
        $stepInfo = $('<div></div>');
        ShadowUI.addClass($stepInfo, STEP_INFO_PANEL_CLASS);

        var popupOptions = {
                width: DIALOG_POPUP_WIDTH,
                content: $stepInfo,
                footerContent: playbackMode ? createButtons() : null,
                noBordersStyle: true,
                prependToContainer: true,
                hasParentPopup: skipPageStateSaving
            },
            popupPosition = getPopupPosition(items[openedStepNum].$item, popupOptions.width);

        popup = new PopupWidget(ShadowUI.getRoot(), popupOptions);

        var validate = ((playbackMode || validateStepNum !== null) && openedStepNum === validateStepNum) ||
            (stepsInfo[openedStepNum].isAssertion && openedStepNum === stepsInfo.length - 1);

        if (stepError) {
            stepsInfo[openedStepNum].failed = true;

            if (stepError.dialog)
                stepsInfo[openedStepNum].dialogError = stepError;
        }

        if (stepsInfo[openedStepNum].isAssertion) {
            step = new AssertionsStepWidget($stepInfo, openedStepNum, stepsInfo[openedStepNum], {
                enableAssertionsValidation: validate,
                updateAssertionsState: validate && !!stepError,
                parentPopup: popup
            });

            step.on(AssertionsStepWidget.START_PICKING_ELEMENT_EVENT, function (e) {
                eventEmitter.emit(exports.START_PICKING_ELEMENT_EVENT, e);
            });

            step.on(AssertionsStepWidget.EDIT_ASSERTION_START_EVENT, function () {
                eventEmitter.emit(exports.EDIT_ASSERTION_START_EVENT, {
                    showBackground: true
                });
            });

            step.on(AssertionsStepWidget.EDIT_ASSERTION_COMPLETE_EVENT, function () {
                eventEmitter.emit(exports.EDIT_ASSERTION_COMPLETE_EVENT, {
                    hideBackground: !playbackMode
                });

                if ($replayStepButton && step.isValid(true))
                    $replayStepButton.removeAttr('disabled');
            });

            if (playbackMode)
                ModalBackground.show();
        }
        else {
            step = new ActionStepWidget($stepInfo, openedStepNum, stepsInfo[openedStepNum], {
                enableFloatMode: validate && playbackMode,
                enableSelectorValidation: validate && !(stepError && stepError.dialog), //NOTE: we should not check selector if we have dialog error, because step may lead redirect
                enableElementsMarking: validate && playbackMode,
                stretchSelectorEditor: true,
                allowVisibleElementsOnly: true,
                dialogError: validate ? stepError : null
            });

            step.on(ActionStepWidget.SELECTOR_EDITOR_FOCUSED_EVENT, function () {
                setBlind(true);
                holdStepInfo = true;
            });

            step.on(ActionStepWidget.SELECTOR_EDITOR_BLURED_EVENT, function () {
                setBlind(false);
                holdStepInfo = false;
            });

            if (playbackMode) {
                if (openedStepNum !== validateStepNum)
                    ModalBackground.show();
                else
                    ModalBackground.hide();
            }
            else
                step.$step.css('paddingBottom', '18px');

            popup.$content.css('minHeight', '234px');
        }

        step.on(BaseStepWidget.STEP_NAME_CHANGED_EVENT, function (e) {
            stepHasChanges = true;
            stepPanelHasChanges = true;
            items[openedStepNum].$name.text(e.name);
        });

        step.on(BaseStepWidget.STEP_INFO_CHANGED_EVENT, function () {
            stepHasChanges = true;
            stepPanelHasChanges = true;

            if (!step.isValid(openedStepNum !== validateStepNum)) {
                ShadowUI.addClass(items[openedStepNum].$item, FAILED_ITEM_CLASS);

                if ($itemArrow)
                    ShadowUI.addClass($itemArrow, FAILED_ITEM_CLASS);

                stepsInfo[openedStepNum].failed = true;
                stepsInfo[openedStepNum].error = !step.isValid(true);

                if ($replayStepButton)
                    $replayStepButton.attr('disabled', 'disabled');

                if ($skipStepButton && (!stepsInfo[openedStepNum].isAssertion || !stepsInfo[openedStepNum].blocks.length))
                    $skipStepButton.attr('disabled', 'disabled');
            }
            else {
                var isAssertionsValid = true;

                if (stepsInfo[openedStepNum].isAssertion) {
                    for (var i = 0; i < stepsInfo[openedStepNum].blocks.length && isAssertionsValid; i++) {
                        for (var j = 0; j < stepsInfo[openedStepNum].blocks[i].assertions.length; j++) {
                            if (stepsInfo[openedStepNum].blocks[i].assertions[j].failed)
                                isAssertionsValid = false;
                        }
                    }
                }

                if (!playbackMode && isAssertionsValid) {
                    ShadowUI.removeClass(items[openedStepNum].$item, FAILED_ITEM_CLASS);

                    if ($itemArrow)
                        ShadowUI.removeClass($itemArrow, FAILED_ITEM_CLASS);

                    if (step && step.dialogsEditor)
                        step.dialogsEditor.removeError();
                }

                stepsInfo[openedStepNum].failed = !isAssertionsValid;
                stepsInfo[openedStepNum].error = false;

                if ($replayStepButton)
                    $replayStepButton.removeAttr('disabled');

                if ($skipStepButton)
                    $skipStepButton.removeAttr('disabled');
            }

            eventEmitter.emit(exports.STEPS_INFO_CHANGED_EVENT, {
                draft: true
            });
        });

        popup.moveTo(popupPosition.left, popupPosition.top, popupPosition.minLeft);
        popupPosition = popup.getPosition();

        if (!updateStepInfoPanelPosition) {
            updateStepInfoPanelPosition = function () {
                var requiredPopupPosition = getPopupPosition(items[openedStepNum].$item, popupOptions.width),
                    currentPosition = popup.getPosition();

                if (requiredPopupPosition.left !== currentPosition.left || requiredPopupPosition.top !== currentPosition.top) {
                    popupPosition = requiredPopupPosition;
                    popup.moveTo(popupPosition.left, popupPosition.top, popupPosition.minLeft);
                    updateItemArrow();
                }
            };

            ShadowUI.bind($window, 'resize', updateStepInfoPanelPosition);
        }

        updateItemArrow();
        eventEmitter.emit(exports.STEP_INFO_SHOWN_EVENT, {
            $el: popup ? popup.$popup : null
        });

        if (!playbackMode) {
            EventSandbox.addInternalEventListener(window, ['mousedown'], onStepPanelMousedownHandler);
            EventSandbox.addInternalEventListener(window, ['click'], onClickOutsideStepInfoPanel);
            EventSandbox.addInternalEventListener(window, ['unload'], onPageUnload);
            EventSandbox.addInternalEventListener(window, Util.RECORDING_LISTENED_EVENTS, preventEvent);
        }
    }

    function createButtons() {
        var $buttons = $('<div></div>');
        ShadowUI.addClass($buttons, BUTTONS_CLASS);

        if (playbackMode) {
            var curStep = openedStepNum;

            $skipStepButton = ButtonWidget.create($buttons, 'Skip step');
            $replayStepButton = ButtonWidget.create($buttons, 'Replay step').attr('disabled', 'disabled');

            ShadowUI.bind($skipStepButton, 'click', function () {
                exports.closeItem();
                highlightItem(curStep);
                eventEmitter.emit(exports.SKIP_STEP_BUTTON_CLICK_EVENT, {
                    hasChanges: stepPanelHasChanges
                });
                stepPanelHasChanges = false;
            });

            ShadowUI.bind($replayStepButton, 'click', function () {
                exports.closeItem();
                highlightItem(curStep);

                ShadowUI.removeClass(items[validateStepNum].$item, FAILED_ITEM_CLASS);

                eventEmitter.emit(exports.REPLAY_STEP_BUTTON_CLICK_EVENT, {
                    hasChanges: stepPanelHasChanges
                });
                stepPanelHasChanges = false;
            });
        }

        return $buttons;
    }

    function hideStepInfo() {
        var callback = function () {
            popup = null;

            if (step)
                step.destroy();

            if (stepHasChanges) {
                UXLog.write('Steps panel: step info is changed');
                eventEmitter.emit(exports.STEPS_INFO_CHANGED_EVENT, {});
            }

            stepHasChanges = false;

            if (playbackMode)
                ModalBackground.hide();

            if (updateStepInfoPanelPosition)
                ShadowUI.unbind($window, 'resize', updateStepInfoPanelPosition);
            updateStepInfoPanelPosition = null;
        };

        if (popup) {
            popup.close(callback);
            eventEmitter.emit(exports.STEP_INFO_HIDDEN_EVENT, {
                editingFinished: openedStepNum === null
            });
            EventSandbox.removeInternalEventListener(window, ['click'], onClickOutsideStepInfoPanel);
            EventSandbox.removeInternalEventListener(window, ['mousedown'], onStepPanelMousedownHandler);
            EventSandbox.removeInternalEventListener(window, ['unload'], onPageUnload);
            EventSandbox.removeInternalEventListener(window, Util.RECORDING_LISTENED_EVENTS, preventEvent);
        }
        else
            callback();
    }

    function onBeforeItemClosed() {
        var $oldOpenedItem = getOpenedItem();

        if ($oldOpenedItem.length)
            ShadowUI.removeClass($oldOpenedItem, LIST_ITEM_OPENED_CLASS);
    }

    function setBlind(blind) {
        popup.blind(blind);

        if ($itemArrow) {
            if (blind)
                ShadowUI.addClass($itemArrow, BLIND_CLASS);
            else
                ShadowUI.removeClass($itemArrow, BLIND_CLASS);
        }

        eventEmitter.emit(exports.STEPS_PANEL_BLINDED_EVENT, {
            blind: blind
        });
    }

    //API
    exports.init = function ($container, options) {
        eventEmitter = new Util.EventEmitter();
        isVisible = options.show;

        stepsInfo = options.stepsInfo;

        if (stepsInfo && stepsInfo.length)
            selectedStepNums.push(stepsInfo.length - 1);

        $stepsPanel = $('<div></div>').appendTo($container || ShadowUI.getRoot());
        ShadowUI.addClass($stepsPanel, STEPS_PANEL_CLASS);

        if (!options.show)
            ShadowUI.addClass($stepsPanel, STEPS_PANEL_HIDDEN_CLASS);

        createStepsPanel($stepsPanel, options.enableHandlers);

        if (!options.enableHandlers)
            ShadowUI.addClass($stepsPanel, DISABLED_CLASS);
    };

    exports.events = {
        on: function (ev, listener) {
            eventEmitter.on(ev, listener);
        }
    };

    exports.activateInteractive = function () {
        initHandlers();
        showDeleteStepButton();
        ShadowUI.removeClass($stepsPanel, DISABLED_CLASS);
    };

    exports.turnOffPlaybackMode = function () {
        playbackMode = false;
        validateStepNum = null;
        validatedStepItem = null;
        SortingBehavior.resumeSorting();

        if (stepsInfo && stepsInfo.length)
            selectItem(stepsInfo.length - 1);
    };

    exports.openStepWithError = function (stepNum, error, recordingMode) {
        isVisible = true;

        playbackMode = !recordingMode;
        validateStepNum = stepNum;
        validatedStepItem = listItems[stepNum];

        if ($stepsPanel && $stepsPanel.length)
            ShadowUI.removeClass($stepsPanel, STEPS_PANEL_HIDDEN_CLASS);

        if (!recordingMode)
            SortingBehavior.stopSorting();

        openItem(stepNum, error);
    };

    exports.hide = function () {
        isVisible = false;

        if ($stepsPanel && $stepsPanel.length) {
            ShadowUI.addClass($stepsPanel, STEPS_PANEL_HIDDEN_CLASS);

            if (openedStepNum !== null) {
                hideStepInfo();
                updateItemArrow();
            }
        }
    };

    exports.show = function (skipPageStateSaving) {
        isVisible = true;

        if ($stepsPanel && $stepsPanel.length) {
            ShadowUI.removeClass($stepsPanel, STEPS_PANEL_HIDDEN_CLASS);

            scrollBar.recalculateSize();

            var $selectedItems = getSelectedItems();
            if ($selectedItems.length)
                scrollBar.updateScroll($selectedItems.last()[0]);

            if (openedStepNum !== null) {
                showStepInfo(null, skipPageStateSaving);
                updateItemArrow();
            }
        }
    };

    exports.addItem = function () {
        var newItemIndex = stepsInfo.length - 1;
        validateStepNum = null;

        removeNoStepsTitleDiv();
        createItem(newItemIndex);
        showDeleteStepButton();

        scrollBar.recalculateSize();
        selectItem(newItemIndex);
    };

    exports.highlightItem = function (itemNum) {
        if (itemNum >= listItems.length)
            return;

        if (itemNum !== highlightedStepNum)
            highlightItem(itemNum);
    };

    exports.closeItem = function () {
        if (openedStepNum !== null) {
            onBeforeItemClosed();
            openedStepNum = null;
            updateItemArrow();
            hideStepInfo();

            if ($replayStepButton)
                $replayStepButton.remove();

            if ($skipStepButton)
                $skipStepButton.remove();
        }
    };

    exports.isVisible = function () {
        return !ShadowUI.hasClass($stepsPanel, STEPS_PANEL_HIDDEN_CLASS);
    };

    exports.updateStepInfoPanelPosition = function () {
        if (updateStepInfoPanelPosition)
            updateStepInfoPanelPosition();
    };

    exports.updateSteps = function () {
        for (var i = 0; i < items.length; i++) {
            if (stepsInfo[i].failed)
                ShadowUI.addClass(items[i].$item, FAILED_ITEM_CLASS);
            else
                ShadowUI.removeClass(items[i].$item, FAILED_ITEM_CLASS);
        }
    };
});