TestCafeClient.define('UI.RecorderWidgets.Tabs', function () {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI;

    var TABS_PANEL_CLASS = 'tabs-panel',
        TAB_CLASS = 'tab',
        SELECTED_CLASS = 'selected';

    var Tabs = this.exports = function ($container, options) {
        var tabs = this;

        this.titles = options.titles || [];
        this.getContentCallback = options.getContentCallback || null;

        this.eventEmitter = new Util.EventEmitter();
        this.on = function (ev, listener) {
            tabs.eventEmitter.on(ev, listener);
        };

        this.$tabs = [];
        this.$contentContainer = null;
        this.$contents = [];
        this.activeTabIndex = -1;

        this._createTabs($container);
        this._createContentContainer($container);

        this._init();
    };

    function _createTab($container, text) {
        var $tab = $('<div></div>').text(text).appendTo($container);
        ShadowUI.addClass($tab, TAB_CLASS);

        var $underline = $('<div></div>').appendTo($tab);
        ShadowUI.addClass($underline, 'tab-underline');

        return $tab;
    }

    Tabs.prototype._createTabs = function ($container) {
        var $panel = $('<div></div>').appendTo($container);
        ShadowUI.addClass($panel, TABS_PANEL_CLASS);

        for (var i = 0; i < this.titles.length; i++)
            this.$tabs.push(_createTab($panel, this.titles[i]));
    };

    Tabs.prototype._createContentContainer = function ($container) {
        this.$contentContainer = $('<div></div>').appendTo($container);

        for (var i = 0; i < this.titles.length; i++)
            this.$contents.push($('<div></div>').css('display', 'none').appendTo(this.$contentContainer));
    };

    Tabs.prototype._init = function () {
        var tabs = this;

        $.each(this.$tabs, function (index, $tab) {
            $tab.click(function () {
                tabs._selectTab(index);
            });
        });

        this._selectTab(0);
    };

    Tabs.prototype._selectTab = function (index) {
        var tabs = this,
            prevActiveTabIndex = this.activeTabIndex,
            $activeContent = this.$contents[index];

        function hidePrevTab() {
            if (prevActiveTabIndex > -1) {
                tabs.$contents[prevActiveTabIndex].css('display', 'none');
                ShadowUI.removeClass(tabs.$tabs[prevActiveTabIndex], SELECTED_CLASS);
            }
        }

        this.activeTabIndex = index;
        if (!$activeContent.data('filled') && typeof this.getContentCallback === 'function') {
            this.getContentCallback(this.activeTabIndex, function ($content, callback) {
                $content.appendTo($activeContent);

                hidePrevTab();
                tabs.$contents[tabs.activeTabIndex].css('display', '');

                callback();
            });

            $activeContent.data('filled', true);
        }
        else {
            hidePrevTab();
            this.$contents[this.activeTabIndex].css('display', '');
        }

        ShadowUI.addClass(this.$tabs[this.activeTabIndex], SELECTED_CLASS);
    };
});