TestCafeClient.define('UI.RecorderWidgets.Toolbar', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        NativeMethods = Hammerhead.NativeMethods,
        Settings = require('Settings'),
        TextSelection = Hammerhead.TextSelection,
        DraggingBehavior = require('UI.DraggingBehavior'),
        StepsPanel = require('UI.RecorderWidgets.StepsPanel'),

        UXLog = require('UI.UXLog');

    var ADD_HOVER_ACTION_TITLE = 'Add a step containing the hover action',
        ADD_WAIT_ACTION_TITLE = 'Add a step containing the wait action',
        ADD_ASSERTIONS_TITLE = 'Add a step containing one or more assertions',
        ADD_SCREENSHOT_TITLE = 'Add a step containing the screenshot action',
        ADD_SCREENSHOT_TITLE_LINUX = 'This action is not supported in the Linux version',
        RUN_PLAYBACK_TITLE = 'Playback the test step by step from the beginning',
        DISABLE_SAVE_TEST_TITLE = 'No changes to save',
        DISABLE_PLAYBACK_TEST_TITLE = 'No steps to playback',
        HAS_INCORRECT_STEP_PLAYBACK_TITLE = 'Cannot start playback due to one or more incorrect step(s)',
        HAS_INCORRECT_STEP_SAVE_TEST_TITLE = 'Cannot save the test due to one or more incorrect step(s)',
        SAVE_TEST_TITLE = 'Save the test',
        EXIT_TITLE = 'Exit test recording',
        EXPAND_BUTTON_SHOW_STEPS_TITLE = 'Display a list of the recorded steps',
        EXPAND_BUTTON_HIDE_STEPS_TITLE = 'Hide a list of the recorded steps',

        ADD_HOVER_ACTION_TEXT = 'Add hover',
        ADD_WAIT_ACTION_TEXT = 'Add wait',
        ADD_ASSERTIONS_TEXT = 'Add assertions',
        ADD_SCREENSHOT_TEXT = 'Add screenshot',
        RUN_PLAYBACK_TEXT = 'Playback',
        SAVE_TEST_TEXT = 'Save',
        EXIT_TEXT = 'Exit',
        SHOW_STEPS_TEXT = 'Show steps',
        HIDE_STEPS_TEXT = 'Hide steps',

        TOOLBAR_AREA_CLASS = 'toolbar-area',
        TOOLBAR_CLASS = 'toolbar',
        TOOLBAR_TOP_CONTAINER_CLASS = 'top-container',
        DRAG_AREA_CLASS = 'drag-area',
        TEST_CAFE_LOGO_CLASS = 'logo-icon',
        STATE_ICON_CLASS = 'state-icon',
        RECORD_ICON_CLASS = 'icon record',
        PLAYBACK_ICON_CLASS = 'icon playback',
        DISABLE_BUTTON_AREA_CLASS = 'disable',
        BUTTON_BAR_CLASS = 'button-bar',
        BUTTON_BAR_CONTENT_CLASS = 'content',
        BUTTONS_CONTAINER = 'buttons-container',
        ADD_ACTIONS_AREA_CLASS = 'add-actions-area',
        BOTTOM_BUTTONS_AREA_CLASS = 'bottom-buttons-area',
        SHOW_STEPS_BUTTON_AREA_CLASS = 'show-steps-buttons-area',
        HIDE_STEPS_BUTTON_AREA_CLASS = 'hide-steps-buttons-area',
        BUTTON_AREA_CLASS = 'button-area',
        AREA_TITLE_CLASS = 'title',
        SEPARATOR_CLASS = 'separator',

        CLEAR_CLASS = 'clear',
        TOOLBAR_HIDDEN_CLASS = 'hidden',
        TOOLBAR_BLINDED_CLASS = 'blind',
        BUTTONS_BAR_HIDDEN_CLASS = 'buttons-hidden',

        ACTION_INDICATOR_CLASS = 'action-indicator',
        ACTION_ICON_CLASS = 'action-icon',
        INDICATOR_WIDE_SIZE = 'wide-size',

        ADD_HOVER_ACTION_ICON_CLASS = 'toolbar-icon hover',
        ADD_WAIT_ACTION_ICON_CLASS = 'toolbar-icon wait',
        ADD_ASSERTIONS_ICON_CLASS = 'toolbar-icon add-assertions',
        ADD_SCREENSHOT_ICON_CLASS = 'toolbar-icon screenshot',
        RUN_PLAYBACK_ICON_CLASS = 'toolbar-icon playback',
        SAVE_TEST_ICON_CLASS = 'toolbar-icon save',
        EXIT_ICON_CLASS = 'toolbar-icon exit',
        SHOW_STEPS_ICON_CLASS = 'toolbar-icon show-steps',
        HIDE_STEPS_ICON_CLASS = 'toolbar-icon hide-steps',

        TOOLBAR_ELEMENTS_FADING_DELAY = 2000,
        ACTION_INDICATOR_FADING_TIME = 3000;

    //Events
    exports.ADD_HOVER_ACTION_BUTTON_PRESSED_EVENT = 'addHoverActionButtonPressed';
    exports.ADD_WAIT_ACTION_BUTTON_PRESSED_EVENT = 'addWaitActionButtonPressed';
    exports.ADD_ASSERTIONS_STEP_EVENT = 'addAssertionsStepButtonPressed';
    exports.ADD_SCREENSHOT_ACTION_BUTTON_PRESSED_EVENT = 'addScreenshotActionButtonPressed';
    exports.RUN_PLAYBACK_BUTTON_PRESSED_EVENT = 'runPlaybackButtonPressed';
    exports.SAVE_TEST_BUTTON_PRESSED_EVENT = 'saveTestButtonPressed';
    exports.EXIT_RECORDING_BUTTON_PRESSED_EVENT = 'exitRecordingButtonPressed';
    exports.TOOLBAR_MOVED_EVENT = 'toolbarMoved';
    exports.STEPS_PANEL_VISIBILITY_CHANGED = 'stepsPanelVisibilityChanged';

    //Globals
    var eventEmitter = null,
        draggingBehavior = null,
        dragging = false,
        prevActiveElement = null,
        prevSelectionStart = null,
        prevSelectionEnd = null,
        prevSelectionInverse = null,
        $toolbarArea = null,
        $toolbar = null,
        $topContainer = null,
        $dragArea = null,
        $stateIcon = null,
        $buttonBar = null,

        $addHoverActionArea = null,
        $addWaitActionArea = null,
        $addAssertionArea = null,
        $addScreenshotActionArea = null,
        $runPlaybackArea = null,
        $saveTestArea = null,
        $exitRecordingArea = null,
        $expandStepsArea = null,

        buttonShortcuts = null;

    //Util
    function getButtonTitlePostfix(shortcut) {
        return shortcut ? ' (' + shortcut + ')' : '';
    }

    //Markup
    function addClearElement($to) {
        ShadowUI.addClass($('<div></div>').appendTo($to), CLEAR_CLASS);
    }

    function createDragArea() {
        $dragArea = $('<div></div>').appendTo($topContainer);
        ShadowUI.addClass($dragArea, DRAG_AREA_CLASS);

        var $logotype = $('<div></div>').appendTo($dragArea);
        ShadowUI.addClass($logotype, TEST_CAFE_LOGO_CLASS);

        $stateIcon = $('<div></div>').appendTo($dragArea);
        ShadowUI.addClass($stateIcon, STATE_ICON_CLASS);
    }

    function createSeparator($parent, isVertical) {
        var $separator = $('<div></div>').appendTo($parent);
        ShadowUI.addClass($separator, SEPARATOR_CLASS);

        if (isVertical)
            ShadowUI.addClass($separator, 'vertical');
    }

    function createButtonArea($parent, areaTitle, iconClass, title) {
        var $buttonArea = $('<div></div>').appendTo($parent);
        ShadowUI.addClass($buttonArea, BUTTON_AREA_CLASS);
        $buttonArea.attr('title', areaTitle);

        var $icon = $('<div></div>').appendTo($buttonArea);
        ShadowUI.addClass($icon, iconClass);

        var $span = $('<span></span>').html(title).appendTo($buttonArea);
        ShadowUI.addClass($span, AREA_TITLE_CLASS);

        return $buttonArea;
    }

    function createButtonBar() {
        $buttonBar = $('<div></div>').appendTo($topContainer);
        ShadowUI.addClass($buttonBar, BUTTON_BAR_CLASS);

        addClearElement($toolbar);

        var $buttonBarContent = $('<div></div>').appendTo($buttonBar);
        ShadowUI.addClass($buttonBarContent, BUTTON_BAR_CONTENT_CLASS);

        var $topButtonsArea = $('<div></div>').appendTo($buttonBarContent);
        ShadowUI.addClass($topButtonsArea, BUTTONS_CONTAINER);
        ShadowUI.addClass($topButtonsArea, ADD_ACTIONS_AREA_CLASS);

        $addAssertionArea = createButtonArea($topButtonsArea, ADD_ASSERTIONS_TITLE + getButtonTitlePostfix(buttonShortcuts.addAssertion), ADD_ASSERTIONS_ICON_CLASS, ADD_ASSERTIONS_TEXT);
        $addHoverActionArea = createButtonArea($topButtonsArea, ADD_HOVER_ACTION_TITLE + getButtonTitlePostfix(buttonShortcuts.addHover), ADD_HOVER_ACTION_ICON_CLASS, ADD_HOVER_ACTION_TEXT);
        $addWaitActionArea = createButtonArea($topButtonsArea, ADD_WAIT_ACTION_TITLE + getButtonTitlePostfix(buttonShortcuts.addWait), ADD_WAIT_ACTION_ICON_CLASS, ADD_WAIT_ACTION_TEXT);

        var addScreenshotTitle = Settings.LINUX_PLATFORM ?
            ADD_SCREENSHOT_TITLE_LINUX :
            ADD_SCREENSHOT_TITLE + getButtonTitlePostfix(buttonShortcuts.addScreenshot);

        $addScreenshotActionArea = createButtonArea($topButtonsArea, addScreenshotTitle, ADD_SCREENSHOT_ICON_CLASS, ADD_SCREENSHOT_TEXT);

        if (Settings.LINUX_PLATFORM)
            ShadowUI.addClass($addScreenshotActionArea, DISABLE_BUTTON_AREA_CLASS);

        createSeparator($buttonBarContent);

        var $bottomButtonsArea = $('<div></div>').appendTo($buttonBarContent);
        ShadowUI.addClass($bottomButtonsArea, BUTTONS_CONTAINER);
        ShadowUI.addClass($bottomButtonsArea, BOTTOM_BUTTONS_AREA_CLASS);

        $runPlaybackArea = createButtonArea($bottomButtonsArea, RUN_PLAYBACK_TITLE + getButtonTitlePostfix(buttonShortcuts.playback), RUN_PLAYBACK_ICON_CLASS, RUN_PLAYBACK_TEXT);
        createSeparator($bottomButtonsArea, true);
        $saveTestArea = createButtonArea($bottomButtonsArea, '', SAVE_TEST_ICON_CLASS, SAVE_TEST_TEXT);
        createSeparator($bottomButtonsArea, true);
        $exitRecordingArea = createButtonArea($bottomButtonsArea, EXIT_TITLE + getButtonTitlePostfix(buttonShortcuts.exitRecording), EXIT_ICON_CLASS, EXIT_TEXT);

        createSeparator($buttonBarContent);
    }

    function createToolbarArea($recorder, toolbarPosition) {
        $toolbarArea = $('<div></div>').appendTo($recorder);
        ShadowUI.addClass($toolbarArea, TOOLBAR_AREA_CLASS);

        if (toolbarPosition && toolbarPosition.left && toolbarPosition.top) {
            $toolbarArea.css({
                left: toolbarPosition.left,
                top: toolbarPosition.top
            });
        }
    }

    function createToolbar(stepsInfo, showSteps, enableStepListInteractive) {
        $toolbar = $('<div></div>').appendTo($toolbarArea);
        ShadowUI.addClass($toolbar, TOOLBAR_CLASS);

        $topContainer = $('<div></div>').appendTo($toolbar);
        ShadowUI.addClass($topContainer, TOOLBAR_TOP_CONTAINER_CLASS);

        createDragArea();
        addClearElement($topContainer);
        createButtonBar(stepsInfo, showSteps, enableStepListInteractive);

        var $expandStepsButtonArea = $('<div></div>').appendTo($topContainer);
        ShadowUI.addClass($expandStepsButtonArea, BUTTONS_CONTAINER);
        ShadowUI.addClass($expandStepsButtonArea, showSteps ? HIDE_STEPS_BUTTON_AREA_CLASS : SHOW_STEPS_BUTTON_AREA_CLASS);

        $expandStepsArea = showSteps ?
            createButtonArea($expandStepsButtonArea, EXPAND_BUTTON_HIDE_STEPS_TITLE, HIDE_STEPS_ICON_CLASS, HIDE_STEPS_TEXT) :
            createButtonArea($expandStepsButtonArea, EXPAND_BUTTON_SHOW_STEPS_TITLE, SHOW_STEPS_ICON_CLASS, SHOW_STEPS_TEXT);

        addClearElement($buttonBar);

        StepsPanel.init($toolbar, {
            stepsInfo: stepsInfo,
            show: showSteps,
            enableHandlers: enableStepListInteractive
        });
    }

    function createActionIndicator(action) {
        var $indicator = $('<div></div>').appendTo($toolbarArea);
        ShadowUI.addClass($indicator, ACTION_INDICATOR_CLASS);

        $('<span></span>').html(action.charAt(0).toUpperCase() + action.slice(1)).appendTo($indicator);

        if (action === 'beforeUnload')
            ShadowUI.addClass($indicator, INDICATOR_WIDE_SIZE);

        var $icon = $('<div></div>').appendTo($indicator);
        ShadowUI.addClass($icon, ACTION_ICON_CLASS);
        ShadowUI.addClass($icon, action);

        return $indicator;
    }

    //Behavior
    function checkWindowSize() {
        var moved = false;
        NativeMethods.setTimeout.call(window, function () {
            var $window = $(window),
                windowWidth = $window.width(),
                windowHeight = $window.height(),
                offset = Util.getOffsetPosition($toolbar[0]),
                clientOffset = Util.offsetToClientCoords({x: offset.left, y: offset.top}),
                toolbarHeight = $toolbar.height(),
                toolbarWidth = $toolbar.width();

            if (clientOffset.x + toolbarWidth > windowWidth) {
                $toolbarArea.css('left', Math.max(windowWidth - toolbarWidth, 0));
                moved = true;
            }

            if (clientOffset.y + toolbarHeight > windowHeight) {
                $toolbarArea.css('top', Math.max(windowHeight - toolbarHeight, 0));
                moved = true;
            }

            if (moved) {
                StepsPanel.updateStepInfoPanelPosition();

                eventEmitter.emit(exports.TOOLBAR_MOVED_EVENT, {toolbarPosition: {
                    left: $toolbarArea.css('left'),
                    top: $toolbarArea.css('top')
                }});
            }
        }, 0);
    }

    function initDragging() {
        draggingBehavior = new DraggingBehavior($dragArea, $toolbarArea, {
            onDragStart: function () {
                if (!Settings.PLAYBACK)
                    StepsPanel.closeItem();

                dragging = true;
            },
            onMove: function () {
                if (Settings.PLAYBACK)
                    StepsPanel.updateStepInfoPanelPosition();
            },
            onDragEnd: function () {
                dragging = false;
                eventEmitter.emit(exports.TOOLBAR_MOVED_EVENT, {toolbarPosition: {
                    left: $toolbarArea.css('left'),
                    top: $toolbarArea.css('top')
                }});
            }
        });
    }

    function hideSteps() {
        StepsPanel.hide();
        exports.updateExpandStepsButtonAppearance();

        eventEmitter.emit(exports.STEPS_PANEL_VISIBILITY_CHANGED, {
            shown: false
        });
    }

    function showSteps() {
        StepsPanel.show();
        exports.updateExpandStepsButtonAppearance(true);

        eventEmitter.emit(exports.STEPS_PANEL_VISIBILITY_CHANGED, {
            shown: true
        });
    }

    function runPlayback() {
        UXLog.write('Toolbar: run playback button is pressed');
        eventEmitter.emit(exports.RUN_PLAYBACK_BUTTON_PRESSED_EVENT, {});
    }

    function exitRecording() {
        UXLog.write('Toolbar: exit button is pressed');
        eventEmitter.emit(exports.EXIT_RECORDING_BUTTON_PRESSED_EVENT);
    }

    function exitRecordingWithErrors() {
        UXLog.write('Toolbar: exit recording button is pressed but there is an error in the steps');
        eventEmitter.emit(exports.EXIT_RECORDING_BUTTON_PRESSED_EVENT, {
            hasErrors: true
        });
    }

    function initButtons() {
        ShadowUI.bind($toolbarArea, 'mousedown', function (e) {
            prevActiveElement = Util.getActiveElement();

            if (Util.isTextEditableElement(prevActiveElement)) {
                prevSelectionStart = TextSelection.getSelectionStart(prevActiveElement);
                prevSelectionEnd = TextSelection.getSelectionEnd(prevActiveElement);
                prevSelectionInverse = TextSelection.hasInverseSelection(prevActiveElement);
            }

            if (!Util.isShadowUIElement(prevActiveElement))
                Util.preventDefault(e, true);
        });

        ShadowUI.bind($addHoverActionArea, 'click', function () {
            UXLog.write('Toolbar: add hover action button is pressed');
            eventEmitter.emit(exports.ADD_HOVER_ACTION_BUTTON_PRESSED_EVENT, {
                prevActiveElement: prevActiveElement,
                prevSelectionStart: prevSelectionStart,
                prevSelectionEnd: prevSelectionEnd,
                prevSelectionInverse: prevSelectionInverse
            });
        });

        ShadowUI.bind($addWaitActionArea, 'click', function () {
            UXLog.write('Toolbar: add wait action button is pressed');
            eventEmitter.emit(exports.ADD_WAIT_ACTION_BUTTON_PRESSED_EVENT, {
                prevActiveElement: prevActiveElement,
                prevSelectionStart: prevSelectionStart,
                prevSelectionEnd: prevSelectionEnd,
                prevSelectionInverse: prevSelectionInverse
            });
        });

        ShadowUI.bind($addScreenshotActionArea, 'click', function () {
            if (ShadowUI.hasClass($addScreenshotActionArea, DISABLE_BUTTON_AREA_CLASS))
                return;

            UXLog.write('Toolbar: add screenshot action button is pressed');
            eventEmitter.emit(exports.ADD_SCREENSHOT_ACTION_BUTTON_PRESSED_EVENT, {
                prevActiveElement: prevActiveElement,
                prevSelectionStart: prevSelectionStart,
                prevSelectionEnd: prevSelectionEnd,
                prevSelectionInverse: prevSelectionInverse
            });
        });

        ShadowUI.bind($addAssertionArea, 'click', function () {
            UXLog.write('Toolbar: add assertion button is pressed');
            eventEmitter.emit(exports.ADD_ASSERTIONS_STEP_EVENT, {
                prevActiveElement: prevActiveElement,
                prevSelectionStart: prevSelectionStart,
                prevSelectionEnd: prevSelectionEnd,
                prevSelectionInverse: prevSelectionInverse
            });
        });

        ShadowUI.bind($expandStepsArea, 'click', function () {
            if (ShadowUI.hasClass($expandStepsArea, DISABLE_BUTTON_AREA_CLASS))
                return;

            if (StepsPanel.isVisible()) {
                UXLog.write('Toolbar: hide steps button is pressed');
                hideSteps();
            }
            else {
                UXLog.write('Toolbar: show steps button is pressed');
                showSteps();
            }
        });
    }

    function initActionIndicator($indicator) {
        NativeMethods.setTimeout.call(window, function () {
            $indicator.fadeOut(ACTION_INDICATOR_FADING_TIME, function () {
                $indicator.remove();
            });
        }, TOOLBAR_ELEMENTS_FADING_DELAY);
    }

    function saveTestButtonClickHandler() {
        UXLog.write('Toolbar: save button is pressed');
        eventEmitter.emit(exports.SAVE_TEST_BUTTON_PRESSED_EVENT, {});
    }

    //API
    exports.init = function ($recorder, options) {
        var optionsDefault = {
                stepsInfo: [],
                hasChanges: false,
                toolbarPosition: {left: 0, top: 0},
                collapsedMode: false,
                shortcuts: {},
                showSteps: true,
                enableStepListInteractive: true
            },
            opt = options || optionsDefault;

        eventEmitter = new Util.EventEmitter();
        buttonShortcuts = opt.shortcuts || {};

        if (!$toolbarArea) {
            createToolbarArea($recorder, opt.toolbarPosition);
            createToolbar(opt.stepsInfo, opt.showSteps, opt.enableStepListInteractive);
            initDragging();
            initButtons();
            //NOTE: init window resize
            ShadowUI.bind($(window), 'resize', checkWindowSize);
        }

        exports.updateState(opt.hasChanges, !!opt.stepsInfo.length);

        if (opt.collapsedMode)
            exports.collapse();
        else
            exports.expand();
    };

    exports.events = {
        on: function (ev, listener) {
            eventEmitter.on(ev, listener);
        }
    };


    exports.expand = function () {
        ShadowUI.removeClass($stateIcon, PLAYBACK_ICON_CLASS);
        ShadowUI.addClass($stateIcon, RECORD_ICON_CLASS);
        ShadowUI.removeClass($topContainer, BUTTONS_BAR_HIDDEN_CLASS);
    };

    exports.collapse = function () {
        ShadowUI.removeClass($stateIcon, RECORD_ICON_CLASS);
        ShadowUI.addClass($stateIcon, PLAYBACK_ICON_CLASS);
        ShadowUI.addClass($topContainer, BUTTONS_BAR_HIDDEN_CLASS);
    };

    exports.hide = function () {
        if ($toolbarArea) {
            ShadowUI.addClass($toolbarArea, TOOLBAR_HIDDEN_CLASS);
            draggingBehavior.resetDragging();
            ShadowUI.unbind($(window), 'resize', checkWindowSize);
        }
    };

    exports.show = function () {
        ShadowUI.removeClass($toolbarArea, TOOLBAR_HIDDEN_CLASS);
        checkWindowSize();
        draggingBehavior.restoreDragging();
        ShadowUI.bind($(window), 'resize', checkWindowSize);
    };

    exports.setBlind = function (blind) {
        if ($toolbarArea) {
            if (blind) {
                draggingBehavior.resetDragging();
                ShadowUI.addClass($toolbarArea, TOOLBAR_BLINDED_CLASS);
            }
            else {
                ShadowUI.removeClass($toolbarArea, TOOLBAR_BLINDED_CLASS);
                draggingBehavior.restoreDragging();
            }
        }
    };

    exports.updateState = function (hasChanges, hasSteps, hasErrors) {
        if (!$toolbarArea || !$toolbarArea.length)
            return;

        if (hasErrors) {
            ShadowUI.unbind($exitRecordingArea, 'click', exitRecording);
            ShadowUI.bind($exitRecordingArea, 'click', exitRecordingWithErrors);
        }
        else {
            ShadowUI.unbind($exitRecordingArea, 'click', exitRecordingWithErrors);
            ShadowUI.bind($exitRecordingArea, 'click', exitRecording);

        }

        if (hasErrors) {
            ShadowUI.addClass($saveTestArea, DISABLE_BUTTON_AREA_CLASS);
            $saveTestArea.attr('title', HAS_INCORRECT_STEP_SAVE_TEST_TITLE);
            ShadowUI.unbind($saveTestArea, 'click', saveTestButtonClickHandler);

            ShadowUI.addClass($runPlaybackArea, DISABLE_BUTTON_AREA_CLASS);
            $runPlaybackArea.attr('title', HAS_INCORRECT_STEP_PLAYBACK_TITLE);
            ShadowUI.unbind($runPlaybackArea, 'click', runPlayback);

            ShadowUI.unbind($runPlaybackArea, 'click', runPlayback);

            return;
        }

        if (hasSteps) {
            ShadowUI.removeClass($runPlaybackArea, DISABLE_BUTTON_AREA_CLASS);
            $runPlaybackArea.attr('title', RUN_PLAYBACK_TITLE + getButtonTitlePostfix(buttonShortcuts.playback));
            ShadowUI.bind($runPlaybackArea, 'click', runPlayback);
        }
        else {
            ShadowUI.addClass($runPlaybackArea, DISABLE_BUTTON_AREA_CLASS);
            $runPlaybackArea.attr('title', DISABLE_PLAYBACK_TEST_TITLE);
            ShadowUI.unbind($runPlaybackArea, 'click', runPlayback);
        }

        if (hasChanges) {
            ShadowUI.removeClass($saveTestArea, DISABLE_BUTTON_AREA_CLASS);
            $saveTestArea.attr('title', SAVE_TEST_TITLE + getButtonTitlePostfix(buttonShortcuts.saveTest));
            ShadowUI.bind($saveTestArea, 'click', saveTestButtonClickHandler);
        }
        else {
            ShadowUI.addClass($saveTestArea, DISABLE_BUTTON_AREA_CLASS);
            $saveTestArea.attr('title', DISABLE_SAVE_TEST_TITLE);
            ShadowUI.unbind($saveTestArea, 'click', saveTestButtonClickHandler);
        }
    };

    exports.updateExpandStepsButtonAppearance = function (stepsShown) {
        var $icon = $expandStepsArea.children('div');

        if (stepsShown) {
            ShadowUI.removeClass($expandStepsArea.parent(), SHOW_STEPS_BUTTON_AREA_CLASS);
            ShadowUI.addClass($expandStepsArea.parent(), HIDE_STEPS_BUTTON_AREA_CLASS);
            $expandStepsArea.attr('title', EXPAND_BUTTON_HIDE_STEPS_TITLE);
            $expandStepsArea.children('span').html(HIDE_STEPS_TEXT);
            ShadowUI.removeClass($icon, SHOW_STEPS_ICON_CLASS);
            ShadowUI.addClass($icon, HIDE_STEPS_ICON_CLASS);
        }
        else {
            ShadowUI.removeClass($expandStepsArea.parent(), HIDE_STEPS_BUTTON_AREA_CLASS);
            ShadowUI.addClass($expandStepsArea.parent(), SHOW_STEPS_BUTTON_AREA_CLASS);
            $expandStepsArea.attr('title', EXPAND_BUTTON_SHOW_STEPS_TITLE);
            $expandStepsArea.children('span').html(SHOW_STEPS_TEXT);
            ShadowUI.removeClass($icon, HIDE_STEPS_ICON_CLASS);
            ShadowUI.addClass($icon, SHOW_STEPS_ICON_CLASS);
        }
    };

    exports.setExpandStepsButtonState = function (enabled) {
        if (enabled)
            ShadowUI.removeClass($expandStepsArea, DISABLE_BUTTON_AREA_CLASS);
        else
            ShadowUI.addClass($expandStepsArea, DISABLE_BUTTON_AREA_CLASS);
    };

    exports.addActionIndicator = function (action) {
        var $indicator = createActionIndicator(action);
        initActionIndicator($indicator);
    };

    exports.updateMaxTopForDragging = function ($el) {
        var windowHeight = $(window).height();

        draggingBehavior.setMaxTop($el && $el.length ? windowHeight - $el.height() : null);
    };
});