TestCafeClient.define('UI.RecorderWidgets.Tooltip', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),

        $ = Hammerhead.$,
        ShadowUI = Hammerhead.ShadowUI,
        Util = Hammerhead.Util,
        NativeMethods = Hammerhead.NativeMethods;

    // Const
    var CLOSE_BUTTON_CLASS = 'close-button',
        CONTENT_CLASS = 'content',
        TOOLTIP_CLASS = 'tooltip',
        TOOLBAR_CLASS = 'toolbar',
        RECORDER_CLASS = 'recorder',

        TOOLTIP_DEFAULT_CSS_RIGHT = 15,
        FADING_DELAY = 2000,
        FADING_TIME = 5000;

    var tooltips = [];

    function Tooltip(message) {
        this.message = message;
        this.animationTimeoutId = null;
        this.visible = null;
        this.$tooltip = null;
        this.$closeButton = null;

        this._createMarkup();
        this._init();
    }

    Tooltip.prototype._createMarkup = function () {
        var $recorder = ShadowUI.select('.' + RECORDER_CLASS);

        this.$tooltip = $('<div>');
        ShadowUI.addClass(this.$tooltip, TOOLTIP_CLASS);

        this.$closeButton = $('<div>').appendTo(this.$tooltip);
        ShadowUI.addClass(this.$closeButton, CLOSE_BUTTON_CLASS);

        var $content = $('<div>').appendTo(this.$tooltip);
        ShadowUI.addClass($content, CONTENT_CLASS);

        var $message = $('<span>').appendTo($content);
        $message.html(this.message);

        this.hide();

        this.$tooltip.appendTo($recorder);
    };

    Tooltip.prototype._init = function () {
        var tooltip = this;

        this.$tooltip.mouseenter(function () {
            tooltip._stopAnimation();
        });

        this.$tooltip.mouseleave(function () {
            if (tooltip.visible)
                tooltip._startAnimation();
        });

        this.$tooltip.mousedown(function (e) {
            Util.preventDefault(e);
        });

        ShadowUI.bind(this.$closeButton, 'click', function () {
            tooltip.hide();
        });
    };

    Tooltip.prototype._startAnimation = function () {
        var tooltip = this;

        this._makeVisible();

        this.animationTimeoutId = NativeMethods.setTimeout.call(window, function () {
            tooltip.$tooltip.fadeOut(FADING_TIME, function () {
                tooltip.hide();
            });
        }, FADING_DELAY);
    };

    Tooltip.prototype._stopAnimation = function () {
        if (this.animationTimeoutId) {
            window.clearTimeout(this.animationTimeoutId);
            this.animationTimeoutId = null;
        }

        this.$tooltip.stop(true);
        this._makeVisible();
    };

    Tooltip.prototype._makeVisible = function () {
        this.visible = true;
        this.$tooltip.css('display', 'inline');
        this.$tooltip.css('opacity', '1');
    };

    Tooltip.prototype._updatePosition = function () {
        this.$tooltip.css('right', TOOLTIP_DEFAULT_CSS_RIGHT + 'px');

        var visibleTooltips = [];
        for (var i = 0; i < tooltips.length; i++)
            if (tooltips[i] !== this && tooltips[i].visible)
                visibleTooltips.push(tooltips[i]);

        //place tooltip below currently visible tooltips
        if (visibleTooltips.length) {
            var bottomTooltip = visibleTooltips[0];
            for (var j = 1; j < visibleTooltips.length; j++)
                if (parseInt(visibleTooltips[j].$tooltip.css('top')) > parseInt(bottomTooltip.$tooltip.css('top')))
                    bottomTooltip = visibleTooltips[j];
            this.$tooltip.css('top', parseInt(bottomTooltip.$tooltip.css('top')) + bottomTooltip.$tooltip.outerHeight(true) + 'px');
        }

        //place tooltip on the left of toolbar if they intersect
        var $toolbar = ShadowUI.select('.' + TOOLBAR_CLASS);
        if ($toolbar.length && $toolbar.is(':visible')) {
            var toolbarRectangle = Util.getElementRectangle($toolbar[0]),
                tooltipRectangle = Util.getElementRectangle(this.$tooltip[0]);

            if (toolbarRectangle.left < tooltipRectangle.left + tooltipRectangle.width && tooltipRectangle.left < toolbarRectangle.left + toolbarRectangle.width &&
                toolbarRectangle.top < tooltipRectangle.top + tooltipRectangle.height && tooltipRectangle.top < toolbarRectangle.top + toolbarRectangle.height) {
                if (toolbarRectangle.left - tooltipRectangle.width - TOOLTIP_DEFAULT_CSS_RIGHT >= 0)
                    this.$tooltip.css('right', (tooltipRectangle.left - toolbarRectangle.left + tooltipRectangle.width + TOOLTIP_DEFAULT_CSS_RIGHT * 2) + 'px');
            }
        }
    }
    ;

    Tooltip.prototype.show = function () {
        this._stopAnimation();
        this._startAnimation();
        this._updatePosition();
    };

    Tooltip.prototype.hide = function () {
        this.visible = false;
        this.$tooltip.css('display', 'none');
        this.$tooltip.css('opacity', '0');
    };

    exports.show = function (message) {
        for (var i = 0; i < tooltips.length; i++)
            if (tooltips[i].message === message) {
                tooltips[i].show();
                return;
            }

        var tooltip = new Tooltip(message);
        tooltip.show();
        tooltips.push(tooltip);
    };

    exports.hideAll = function () {
        for (var i = 0; i < tooltips.length; i++)
            if (tooltips[i].visible)
                tooltips[i].hide();
    };
});