TestCafeClient.define('UI.RecorderWidgets.UnsavedChangesDialog', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI,
        ConfirmDialog = require('UI.RecorderWidgets.ConfirmDialog'),
        ButtonWidget = require('UI.RecorderWidgets.Button');

    //Const
    var BUTTONS_CLASS = 'buttons',
        BUTTON_SMALL_CLASS = 'small',

    //Popup text
        HEADER_TEXT = 'Unsaved changes',
        SAVE_CHANGES_MESSAGE_TEXT = ['Do you want to save the changes before you quit the test recorder?'],
        EXIT_MESSAGE_TEXT = ['There are changes that can be saved after all incorrect steps are corrected. Do you want to quit losing all unsaved changes?'];

    //Events
    exports.SAVE_BUTTON_CLICK_EVENT = 'saveButtonClick';
    exports.CANCEL_BUTTON_CLICK_EVENT = 'cancelButtonClick';
    exports.EXIT_BUTTON_CLICK_EVENT = 'exitButtonClick';

    //Globals
    var dialog = null,
        eventEmitter = null,
        onlyExit = false,
        $saveButton = null,
        $cancelButton = null,
        $exitButton = null;

    //Markup
    var createButtons = function () {
        var $buttons = $('<div></div>');
        ShadowUI.addClass($buttons, BUTTONS_CLASS);

        if (onlyExit) {
            $exitButton = ButtonWidget.create($buttons, 'Yes');
            $cancelButton = ButtonWidget.create($buttons, 'No');
        }
        else {
            $saveButton = ButtonWidget.create($buttons, 'Yes');
            $exitButton = ButtonWidget.create($buttons, 'No');
            $cancelButton = ButtonWidget.create($buttons, 'Cancel');

            ShadowUI.addClass($saveButton, BUTTON_SMALL_CLASS);
        }

        ShadowUI.addClass($exitButton, BUTTON_SMALL_CLASS);
        ShadowUI.addClass($cancelButton, BUTTON_SMALL_CLASS);

        return $buttons;
    };

    var createDialog = function ($container) {
        dialog = new ConfirmDialog($container, {
            headerText: HEADER_TEXT,
            message: onlyExit ? EXIT_MESSAGE_TEXT : SAVE_CHANGES_MESSAGE_TEXT,
            footerContent: createButtons()
        });

        init();
    };

//Behavior
    var init = function () {
        ShadowUI.bind($cancelButton, 'click', function () {
            closeDialog(function () {
                eventEmitter.emit(exports.CANCEL_BUTTON_CLICK_EVENT);
            });
        });

        ShadowUI.bind($exitButton, 'click', function () {
            closeDialog(function () {
                eventEmitter.emit(exports.EXIT_BUTTON_CLICK_EVENT);
            });
        });

        if (onlyExit) {
            dialog.popup.onkeydown(function (e) {
                if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.enter) {
                    $exitButton.trigger('click');
                    Util.preventDefault(e);
                }

                if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.esc) {
                    $cancelButton.trigger('click');
                    Util.preventDefault(e);
                }

                //NOTE: we should prevent page scrolling
                if (e.target === dialog.popup.$popup[0] && Util.isArrowKey(e.keyCode))
                    Util.preventDefault(e);
            }, true);
        }
        else {
            ShadowUI.bind($saveButton, 'click', function () {
                closeDialog(function () {
                    eventEmitter.emit(exports.SAVE_BUTTON_CLICK_EVENT);
                });
            });

            dialog.popup.onkeydown(function (e) {
                if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.enter) {
                    $saveButton.trigger('click');
                    Util.preventDefault(e);
                }

                if (e.keyCode === Util.KEYS_MAPS.SPECIAL_KEYS.esc) {
                    $cancelButton.trigger('click');
                    Util.preventDefault(e);
                }

                //NOTE: we should prevent page scrolling
                if (e.target === dialog.popup.$popup[0] && Util.isArrowKey(e.keyCode))
                    Util.preventDefault(e);
            }, true);
        }
    };

    var closeDialog = function (callback) {
        dialog.popup.close(callback);
    };

    //API
    exports.init = function ($container, hasErrors) {
        eventEmitter = new Util.EventEmitter();
        onlyExit = hasErrors;
        createDialog($container);
    };

    exports.events = {
        on: function (ev, listener) {
            eventEmitter.on(ev, listener);
        }
    };
});