TestCafeClient.define('UI.SortingBehavior', function (require, exports) {
    var Hammerhead = HammerheadClient.get('Hammerhead'),
        $ = Hammerhead.$,
        Util = Hammerhead.Util,
        ShadowUI = Hammerhead.ShadowUI;

    //Const
    var SORTABLE_CLASS = 'sortable',
        PLACE_HOLDER_CLASS = 'place-holder',

        DEFAULT_OPTIONS = {
            handlers: {},
            waitSortingStart: false, //we should not bind mousedown handler
            itemTag: '', //it's tag of draggable element require for placeholder element
            itemClass: '', //class of draggable elements
            draggableClass: '', //dragging runs beyond the element with this class
            exceptClass: '' //dragging beyond the element with this class is impossible
        };

    //vars
    var opts = null,
        container = null,

        $draggedItem = null,
        $placeHolderItem = null,

        mousedownOffset = {},
        itemOffsetLimit = null,
        currentPositions = null,
        lastPosTop = null,

        originalItemIndex = null,
        originalItemStyle = '',

        stopped = false;

    var getItemByElement = function (el) {
        var $parents = $(el).parents(),
            $item = null;

        if (opts.exceptClass && ShadowUI.hasClass($(el), opts.exceptClass))
            return null;

        if (opts.draggableClass && !ShadowUI.hasClass($(el), opts.draggableClass)) {
            var $draggableParent = $parents.filter(function () {
                return ShadowUI.hasClass($(this), opts.draggableClass);
            });

            if (!$draggableParent.length)
                return null;
        }
        else if ((!opts.draggableClass ||  opts.draggableClass === opts.itemClass) && ShadowUI.hasClass($(el), opts.itemClass))
            return $(el);

        $.each($parents, function (index, parent) {
            if (ShadowUI.hasClass($(parent), opts.itemClass)) {
                $item = $(parent);
                return false;
            }
        });

        return $item;
    };

    var getItems = function () {
        return $(container).children(opts.itemTag).map(function (i, item) {
            return ShadowUI.hasClass($(item), opts.itemClass) ? item : null;
        });
    };

    var saveItemPositions = function () {
        var positions = [],
            $itemsForSave = getItems(),
            offset = null;

        $itemsForSave = $itemsForSave.map(function (i, item) {
            return (ShadowUI.hasClass($(item), SORTABLE_CLASS) || ShadowUI.hasClass($(item), PLACE_HOLDER_CLASS)) ? null : item;
        });

        $itemsForSave.each(function (i, item) {
            offset = Util.getOffsetPosition(item);
            offset.right = offset.left + $(item).outerWidth();
            offset.bottom = offset.top + $(item).outerHeight();
            offset.el = item;
            positions[i] = offset;
        });

        currentPositions = positions;
    };

    var setDraggedItemPosition = function (x, y) {
        var itemNewTop = y - mousedownOffset.top,
            itemNewLeft = x - mousedownOffset.left;

        itemNewTop = Math.min(itemOffsetLimit.bottom, Math.max(itemNewTop, itemOffsetLimit.top));
        itemNewLeft = Math.min(itemOffsetLimit.right, Math.max(itemNewLeft, itemOffsetLimit.left));

        var $itemParents = $draggedItem.parents();

        $itemParents.each(function (i, parent) {
            if ($(parent).css('position') !== 'static' && (!Util.isMozilla || $(parent).css('display') !== 'table')) {
                var parentOffset = Util.getOffsetPosition(parent);
                itemNewTop -= parentOffset.top;
                itemNewLeft -= parentOffset.left;
                return false;
            }
        });

        $draggedItem.css({
            top: itemNewTop,
            left: itemNewLeft
        });
    };

    var findElementForReplace = function () {
        var offset = Util.getOffsetPosition($draggedItem[0]),
            currentY = offset.top + mousedownOffset.top;

        for (var i = 0; i < currentPositions.length; i++) {
            if (currentY >= currentPositions[i].top && currentY <= currentPositions[i].bottom)
                return currentPositions[i].el;
        }

        return null;
    };

    var updateOffsetLimit = function () {
        var containerHeight = Math.floor($(container).outerHeight());

        itemOffsetLimit = Util.getOffsetPosition(container);
        itemOffsetLimit.right = itemOffsetLimit.left + $(container).outerWidth() - $draggedItem.outerWidth();
        itemOffsetLimit.bottom = itemOffsetLimit.top + containerHeight - $draggedItem.outerHeight();
    };

    //handlers
    var onMousedown = function (e) {
        if (e.button !== Util.BUTTON.LEFT || stopped)
            return;

        var $curTarget = getItemByElement(e.target);

        if (!$curTarget || !$curTarget.length)
            return;

        //NOTE: we should prevent this event for prevent text selection
        Util.preventDefault(e);

        if ($draggedItem !== null)
            onDropItem();

        $draggedItem = $curTarget;

        var itemMargin = Util.getElementMargin($draggedItem),
            itemOffset = Util.getOffsetPosition($draggedItem[0]),
            itemHeight = $draggedItem.height(),
            itemWidth = $draggedItem.width();

        originalItemIndex = getItems().index($draggedItem);

        //calculate mouse offset relative to draggedItem
        mousedownOffset.top = (e.pageY || e.y) - itemOffset.top + (isNaN(itemMargin.top) ? 0 : itemMargin.top);
        mousedownOffset.left = (e.pageX || e.x) - itemOffset.left + (isNaN(itemMargin.left) ? 0 : itemMargin.left);

        updateOffsetLimit();

        //create placeholder item
        $draggedItem.after('<' + opts.itemTag + '>&nbsp</' + opts.itemTag + '>');
        $placeHolderItem = $draggedItem.next();
        ShadowUI.addClass($placeHolderItem, opts.itemClass);
        ShadowUI.addClass($placeHolderItem, PLACE_HOLDER_CLASS);
        $placeHolderItem.css({
            height: itemHeight,
            width: itemWidth
        });

        originalItemStyle = $draggedItem.attr('style') || '';
        ShadowUI.addClass($draggedItem, SORTABLE_CLASS);

        saveItemPositions();

        ShadowUI.bind($(document), 'mousemove', onDragging);
        ShadowUI.bind($(document), 'mouseup', onDropItem);
        ShadowUI.bind($(document), typeof document.onwheel !== 'undefined' ? 'wheel' : 'mousewheel', onMouseWheel);

        if (opts.handlers && opts.handlers.onDragStart)
            opts.handlers.onDragStart($draggedItem, originalItemIndex);
    };

    var onMouseWheel = function (e) {
        //update scrolling
        if (opts.handlers && opts.handlers.onMouseWheel)
            opts.handlers.onMouseWheel(e);

        onDragging(e);

        Util.preventDefault(e);
        return false;
    };

    var onDragging = function (e) {
        if ($draggedItem === null)
            return false;

        var x = e.pageX || e.x,
            y = e.pageY || e.y;

        setDraggedItemPosition(x, y);

        //update scroll
        if (opts.handlers && opts.handlers.onItemMoved)
            opts.handlers.onItemMoved($draggedItem);

        //update elements offsets
        saveItemPositions();
        updateOffsetLimit();

        var replacedElement = findElementForReplace();

        if (replacedElement === null)
            return false;

        if (lastPosTop === null || lastPosTop > y)
            $(replacedElement).before($placeHolderItem);
        else
            $(replacedElement).after($placeHolderItem);

        saveItemPositions();
        lastPosTop = y;

        return false;
    };

    var onDropItem = function () {
        if ($draggedItem === null)
            return;

        ShadowUI.removeClass($draggedItem, SORTABLE_CLASS);

        if (originalItemStyle !== '')
            $draggedItem.attr('style', originalItemStyle);
        else
            $draggedItem.removeAttr('style');


        $placeHolderItem.before($draggedItem);
        $placeHolderItem.remove();
        $placeHolderItem = null;

        var newIndex = getItems().index($draggedItem);

        if (opts.handlers && opts.handlers.onDragEnd)
            opts.handlers.onDragEnd(originalItemIndex, newIndex);

        $draggedItem = null;
        ShadowUI.unbind($(document), 'mousemove', onDragging);
        ShadowUI.unbind($(document), 'mouseup', onDropItem);
        ShadowUI.unbind($(document), typeof document.onwheel !== 'undefined' ? 'wheel' : 'mousewheel', onMouseWheel);

        return false;
    };

    exports.init = function (itemsContainer, options) {
        container = itemsContainer;
        opts = $.extend({}, DEFAULT_OPTIONS, options);

        var firstChild = $(container).children(':first')[0];

        if (!firstChild)
            return;

        if (!opts.itemTag)
            opts.itemTag = $(container).children(':first')[0].tagName.toLowerCase();

        if (!opts.waitSortingStart)
            ShadowUI.bind($(container), 'mousedown', onMousedown);
    };

    exports.startSorting = function(e){
        onMousedown(e);
    };

    exports.stopSorting = function () {
        stopped = true;
    };

    exports.resumeSorting = function () {
        stopped = false;
    };
});