ControlPanel.define('OptionsView', function (require, exports) {
    require('Widgets.Dialog');
    require('Widgets.Draggable');
    require('Widgets.Resizable');
    require('Widgets.ScrollBar');
    require('Widgets.TreeView');

    var $ = require('jQuery'),
        Util = require('Util'),
        UXLog = require('UXLog');

    exports.init = function () {
        //Constants
        var
        //Selectors
            HEADER_BACK = '.header .back',
            FOOTER = '.footer',
            ADD_BROWSER_BTN = '#add-browser-btn',
            DETECT_BROWSERS_BTN = '#detect-browsers-btn',
            BROWSER_ROW = '.browser-row',
            OPTIONS_TAB_CONT = '.options-tabs-cont',
            SERVER_OPTIONS = '#server-options',
            BROWSERS_OPTIONS = '#browsers-options',
            RESTART_BTN = '#restart-btn',
            SERVICE_PORT1 = '#service_post1',
            SERVICE_PORT2 = '#service_post2',
            CONTROL_PANEL_PORT = '#control_panel_port',
            HOSTNAME = '#hostname',
            REPORTS_PATH_INPUT = '#reports-path-input',
            OPTION_TABS = '.options-tabs li',
            TAB_CONTENT = '.tab-content',
            MSG_CONT = '.msg-cont',
            CHANGE_BROWSER_DIALOG = '#change-browser-dialog',
            FS_TREEVIEW_CONT = '#fs-tree',
            CHANGE_BROWSER_INPUT = '#input-path',
            TITLE_CONT = '.err-text',
            CHANGE_BROWSER_CANCEL_BTN = '#change_browser_cancel',
            CHANGE_BROWSER_OK_BTN = '#change_browser_btn',
            UI_DIALOG_TITLE_CONT = '.dialog-header',
            APPLY_NETWORK_OPTIONS_BTN = '#apply-network-options',
            APPLY_REPORTS_OPTIONS_BTN = '#apply-reports-options',
            OPTIONS_CONT = '.options-cont',
            LOADING_MSG_CLASS = 'loading-img',
            AUTO_DETECT_NETWORK = '#server-options input[type="checkbox"]',
            INPUTS_NETWORK = '#server-options input[type="text"]',
            EDIT_BROWSER_BTN = '.edit-browser-btn',
            BROWSER_OPTIONS_DIALOG_CONT = '#browser-options-dialog',
            EDIT_BROWSER_NAME = '#edit-browser-name',
            EDIT_BROWSER_PATH = '#edit-browser-path',
            EDIT_BROWSER_CMD = '#edit-browser-cmd',
            DELETE_BROWSER_DIALOG_CONT = '#delete-browser-dialog',
            DELETE_BROWSER_YES_BTN = '#delete-browser-yes-btn',
            DELETE_BROWSER_NO_BTN = '#delete-browser-no-btn',
            DELETE_BROWSER_BUTTON = '.delete-browser-btn',
            PORT_BUSY = '.port-busy',
            DIALOG_TEXT = '.dialog-text h2',
            ICON_TYPES = '.icon-types',
            ICON_UNKNOWN = '.unknown',
            BROWSER_FILE = '#browser-file',
            BROWSER_OPTIONS_OK_BTN = '#configure-browser-ok-btn',
            BROWSER_OPTIONS_CANCEL_BTN = '#configure-browser-cancel-btn',
            ICON_CHECKED = '.icon-checked',
            REPORT_OPTIONS = '#reports-options',
            REPORTS_PATH_DIALOG = '#reports-path-dialog',
            REPORTS_PATH_DIALOG_INPUT = '#reports-path-dialog-input',
            REPORTS_PATH_OK_BTN = '#reports-path-ok',
            REPORTS_PATH_CANCEL_BTN = '#reports-path-cancel',
            CHANGE_REPORT_PATH_BTN = '#change-report-btn',
            REPORTS_PATH_TREEVIEW_CONT = '#reports-path-fs-tree',
            HOME_DIR_BTN = '.home-dir',

        //Classes
            ERR_VALUE_CLASS = 'err-value',
            CHANGE_BROWSER_CLASS = 'change-browser',
            TREE_ITEM_SELECTED_CLASS = 'selected',
            READONLY_CLASS = 'readonly',
            BROWSER_OPTIONS_DIALOG_CLASS = 'browser-options-dialog',
            DELETE_BROWSER_DIALOG_CLASS = 'delete-browser',
            ACTIVE_TAB_CLASS = 'active',
            ICON_CHECKED_CLASS = 'icon-checked',
            PORT_BUSY_CLASS = 'port-busy',
            RESIZE_MARKER_CHANGE_BROWSER_FILE_CLASS = 'resize-marker-change-browser',
            ERR_MSG_CLASS = 'err-msg',
            INFO_MSG_CLASS = 'info-msg',
            REPORTS_PATH_CLASS = 'reports-path',
            REPORTS_PATH_SCROLLBAR_CLASS = 'reports-path-scrollbar',
            RESIZE_MARKER_REPORTS_PATH_CLASS = 'reports-path-resize-marker',

        //Urls
            CHANGE_BROWSER_SCROLLBAR_CLASS = 'change-browser-scrollbar',
            RESTART_URL = '/network/save/',
            APPLY_REPORTS_OPTIONS_URL = '/reports/configure',
            DETECT_BROWSER_URL = '/browsers/detect/',
            CONFIGURE_BROWSER_URL = '/browser/configure/',
            GET_FS_BY_PATH_URL = '/files_by_path/',
            GET_DIR_BY_PATH_URL = '/dirs_by_path/',

        //Text
            PORT_ALREADY_IN_USE_TEXT = 'The port is already in use.',
            EMPTY_FIELD_TEXT = 'Required field.',
            IDENTICAL_VALUES_TEXT = 'Proxy and control panel ports must have different values.',
            PORT_IS_AUTO_DETECTING_TEXT = 'The port was detected automatically, because the specified port is already in use.',
            INVALID_PORT_VALUE_TEXT = 'Invalid port value. Please enter a number in the range from 0 to 65535',
            NEED_RESTART_TEXT = 'Network options have been changed. The changes will be applied after the server restarts.',
            REPORTS_OPTIONS_UPDATED_TEXT = 'Report options have been changed.',
            ERR_MSG_TYPE = 'error',
            WARN_MSG_TYPE = 'warning',
            DELETE_DIALOG_TEXT_PATTERN = 'Are you sure you want to remove "{0}"?',
            EDIT_BROWSER_DIALOG_TITLE_TEXT = '"{0}" options',
            ADD_BROWSER_DIALOG_TITLE_TEXT = 'Add new browser',
            BROWSER_PATH_NOT_EXISTS_TEXT = 'Browser path not exists',
            BROWSER_IS_ALREADY_EXISTS = 'Browser with same name is already exists',

        //Attr
            DATA_BROWSER_ICON_ATTR = 'data-browser-icon',
            DATA_TAB_ATTR = 'data-tab',
            PATH_ATTR = 'data-path',
            DATA_BROWSER_NAME_ATTR = 'data-browser-name',
            DATA_BROWSER_PATH_ATTR = 'data-browser-path',
            DATA_BROWSER_CMD_ATTR = 'data-browser-cmd',

        //Metrics
            ENTER_BTN_CODE = 13,
            FS_EXPAND_DURATION = 200,
            BROWSERS_ANIMATION_DURATION = 500,
            CHANGE_BROWSER_DIALOG_WIDTH = 560,
            REPORTS_PATH_DIALOG_WIDTH = 560,
            BROWSER_DIALOG_WIDTH = 600,
            LOADING_MSG_DURATION = 750,
            PORT_MIN_VALUE = 0,
            PORT_MAX_VALUE = 65535;

        //Globals
        var $document = $(document),
            $changeBrowserDialog = null,
            $browserOptionsDialog = null,
            optionsPageContent = null,
            $reportsPathDialog = null,
            loadAnimationStartTime = 0;


        //Utils
        var portValueIsValid = function (port) {
            port = parseInt(port);
            return (port > PORT_MIN_VALUE && port < PORT_MAX_VALUE);
        };

        var showMsg = function (text, type) {
            var $msgCont = $(MSG_CONT);

            if (type === ERR_MSG_TYPE)
                $msgCont.addClass(ERR_MSG_CLASS);
            else
                $msgCont.addClass(INFO_MSG_CLASS);

            $msgCont.show();
            $msgCont.find('p').html(text);
        };

        var removeMsg = function () {
            var $msgCont = $(MSG_CONT);

            $msgCont.hide();
            $msgCont.find('p').html('');
        };

        var setInCenter = function ($elm) {
            var $cont = $(OPTIONS_CONT),
                topMargin = (Util.getContentHeight() - $elm.height() ) / 2;

            if (topMargin > 0)
                $elm.css('margin-top', topMargin);

            var leftMargin = ($cont.outerWidth(false) - $elm.width()) / 2;

            if (leftMargin > 0)
                $elm.css('margin-left', leftMargin);
        };

        var restoreContent = function (actionBeforeInit) {
            var animationDuration = (new Date()).getTime() - loadAnimationStartTime;

            var restore = function () {
                $(OPTIONS_CONT).html(optionsPageContent);

                if (typeof actionBeforeInit === 'function')
                    actionBeforeInit();

                initContent();

                loadAnimationStartTime = 0;
            };

            if (animationDuration < LOADING_MSG_DURATION)
                setTimeout(restore, LOADING_MSG_DURATION - animationDuration);
            else
                restore();
        };

        var createLoadingMsg = function () {
            var $pageCont = $(OPTIONS_CONT),
                $loadingMsgContent = $('<div></div>')
                    .addClass(LOADING_MSG_CLASS);

            optionsPageContent = $pageCont.html();

            $pageCont.html($loadingMsgContent);
            setInCenter($loadingMsgContent);

            loadAnimationStartTime = (new Date()).getTime();
        };

        var initEnterBtn = function () {
            $(SERVER_OPTIONS).find('input').keypress(function (e) {
                if (e.which === ENTER_BTN_CODE)
                    $(RESTART_BTN).click();
            });
        };

        var initTabs = function () {
            var $tab = $(OPTION_TABS);

            $tab.click(function () {
                var $curTab = $(this);
                var tabContent = $curTab.attr(DATA_TAB_ATTR);

                $(TAB_CONTENT).hide();
                $('#' + tabContent).show();
                $tab.removeClass(ACTIVE_TAB_CLASS);
                $curTab.addClass(ACTIVE_TAB_CLASS);
                UXLog.write('Settings: "' + $curTab.text() + '" tab opened');
            });

            if ($(PORT_BUSY).length) {
                $tab.eq(1).click();
            }
        };

        //Dialogs
        var configureBrowser = function (name, oldName, path, cmd, icon, callback) {
            var browserInfo = {
                path: path,
                cmd: cmd || '',
                icon: icon || ''
            };

            createLoadingMsg();

            $.post(CONFIGURE_BROWSER_URL, {
                browserName: name,
                browserInfo: browserInfo,
                oldName: oldName
            },function (content) {
                restoreContent(function () {
                    $(BROWSERS_OPTIONS).html(content).fadeIn(BROWSERS_ANIMATION_DURATION);
                    callback();
                });
            }).error(function (err) {
                    restoreContent(function () {
                        callback(JSON.parse(err.responseText));
                    });
                });
        };

        var deleteBrowser = function (browserName) {
            createLoadingMsg();

            $.post(CONFIGURE_BROWSER_URL, {
                browserName: browserName
            },function (content) {
                restoreContent(function () {
                    $(BROWSERS_OPTIONS).html(content).fadeIn(BROWSERS_ANIMATION_DURATION);
                });
            }).error(function () {
                    restoreContent();
                });
        };

        var detectBrowser = function () {
            createLoadingMsg();

            $.post(DETECT_BROWSER_URL,function (content) {
                restoreContent(function () {
                    $(BROWSERS_OPTIONS).html(content).fadeIn(BROWSERS_ANIMATION_DURATION);
                });
            }).error(function () {
                    restoreContent();
                });
        };

        var initDeleteBrowserDialog = function () {
            var $deleteDialog = null,
                $btn = null,
                browserName = null,
                $cont = $(DELETE_BROWSER_DIALOG_CONT);

            $deleteDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: DELETE_BROWSER_DIALOG_CLASS,
                width: BROWSER_DIALOG_WIDTH
            });

            $document.on('click', DELETE_BROWSER_BUTTON, function () {
                UXLog.write('Settings: "Remove browser" clicked');

                $btn = $(this);
                browserName = $btn.closest(BROWSER_ROW).attr(DATA_BROWSER_NAME_ATTR);

                $cont.find(DIALOG_TEXT).text(Util.formatText(DELETE_DIALOG_TEXT_PATTERN, browserName));
                $deleteDialog.dialog('open');
            });

            $(DELETE_BROWSER_YES_BTN).click(function () {
                deleteBrowser(browserName);
                $deleteDialog.dialog('close');
            });

            $(DELETE_BROWSER_NO_BTN).click(function () {
                $deleteDialog.dialog('close');
            });
        };

        var initReportsPathDialog = function() {
            var $cont = $(REPORTS_PATH_DIALOG),
                $fsTreeView = $(REPORTS_PATH_TREEVIEW_CONT),
                $fsTreeViewCont = $fsTreeView.parent(),
                $inputPath = $(REPORTS_PATH_DIALOG_INPUT),
                $inputPathNextElm = $inputPath.next(),
                $inputErr = $inputPathNextElm.find(TITLE_CONT),
                reportsPath = '',
                inputFocus = false,
                loadItems = function () {
                    $fsTreeViewCont.scrollBar({
                        scrollTo: REPORTS_PATH_TREEVIEW_CONT + ' li.' + TREE_ITEM_SELECTED_CLASS,
                        scrollClass: REPORTS_PATH_SCROLLBAR_CLASS
                    });
                },
                treeviewOptions = {
                    url: GET_DIR_BY_PATH_URL,
                    open: '',
                    effect: 'animate',
                    time: FS_EXPAND_DURATION,
                    asyncAction: loadItems
                };

            $reportsPathDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: REPORTS_PATH_CLASS,
                width: 'auto'
            });

            var clearInputDirErr = function () {
                $inputPath.removeClass(ERR_VALUE_CLASS);
                $inputPathNextElm.hide();
                $inputErr.html();
            };

            $document.on('click', CHANGE_REPORT_PATH_BTN, function (e) {
                var initValue = $(REPORTS_PATH_INPUT).val();

                $inputPath.val(initValue);
                reportsPath = treeviewOptions.open = initValue;

                $reportsPathDialog.dialog('open');

                $cont.parent().find(UI_DIALOG_TITLE_CONT).draggable();
                $fsTreeViewCont.scrollBar({
                    scrollTo: REPORTS_PATH_TREEVIEW_CONT + ' li.' + TREE_ITEM_SELECTED_CLASS,
                    scrollClass: REPORTS_PATH_SCROLLBAR_CLASS
                });

                $fsTreeViewCont.resizable({
                    minWidth: REPORTS_PATH_DIALOG_WIDTH,
                    minHeight: $fsTreeViewCont.height(),
                    class: RESIZE_MARKER_REPORTS_PATH_CLASS,
                    action: function () {
                        $fsTreeViewCont.scrollBar({
                            scrollClass: REPORTS_PATH_SCROLLBAR_CLASS
                        });
                    }
                });

                $fsTreeView.treeView(treeviewOptions);

                e.stopImmediatePropagation();
            });

            $inputPath.focusout(function (e) {
                inputFocus = false;
                e.stopImmediatePropagation();
            });

            $inputPath.click(function (e) {
                if (!inputFocus) {
                    inputFocus = true;
                    $(this).select();
                }

                e.stopImmediatePropagation();
            });

            $inputPath.keyup(function (e) {
                treeviewOptions.open = $inputPath.val();
                treeviewOptions.effect = false;
                $fsTreeView.treeView(treeviewOptions);
                treeviewOptions.effect = 'animate';

                if (e.keyCode !== ENTER_BTN_CODE) {
                    $inputPath.removeClass(ERR_VALUE_CLASS);
                    $inputPathNextElm.hide();
                    $inputPathNextElm.find(TITLE_CONT).html();
                }

                e.stopImmediatePropagation();
            });

            $document.on('click', REPORTS_PATH_OK_BTN, function (e) {
                var inputValue = $inputPath.val(),
                    $reportsPathInput = $(REPORTS_PATH_INPUT);

                Util.removeInputMsgs($(REPORT_OPTIONS));

                $inputPathNextElm.hide();

                if ($.trim(inputValue))
                    $inputPath.val(decodeURIComponent(reportsPath));

                clearInputDirErr();

                if(!$.trim(inputValue))
                    Util.showInputMsg($inputPath, EMPTY_FIELD_TEXT, ERR_MSG_TYPE);
                else {
                    $reportsPathInput.val(inputValue);
                    $reportsPathDialog.dialog('close');
                }


                e.stopImmediatePropagation();
            });

            $document.on('click', REPORTS_PATH_CANCEL_BTN, function (e) {
                $reportsPathDialog.dialog('close');
                e.stopImmediatePropagation();
            });

            $document.on('click', REPORTS_PATH_TREEVIEW_CONT + ' li', function () {
                reportsPath = $(this).attr(PATH_ATTR);
                $inputPath.val(decodeURIComponent(reportsPath));
            });

            $document.on('click', [REPORTS_PATH_DIALOG, HOME_DIR_BTN].join(' '), function(e) {
                $inputPath.val(window.HOME_DIR);
                $inputPath.keyup();

                e.stopImmediatePropagation();
            });
        };

        var initReportsOptions = function() {
            $document.on('click', APPLY_REPORTS_OPTIONS_BTN, function(e) {
                var $cont = $(REPORT_OPTIONS),
                    $reportsPathInput = $(REPORTS_PATH_INPUT),
                    reportsPathVal = $reportsPathInput.val();

                UXLog.write('Settings: "Apply reports settings" clicked');

                Util.removeInputMsgs($cont);

                if(!reportsPathVal) {
                    Util.showInputMsg($reportsPathInput, EMPTY_FIELD_TEXT, ERR_MSG_TYPE);
                    return;
                }

                //NOTE: save attribute value for inputs for restore content
                $reportsPathInput.attr('value', reportsPathVal);

                removeMsg();
                createLoadingMsg();
                Util.removeInputMsgs($cont);

                $.post(APPLY_REPORTS_OPTIONS_URL, {
                    reportsPath: reportsPathVal
                }).success(function (warn) {
                    restoreContent(function () {
                        if (warn.hasChanged)
                            showMsg(REPORTS_OPTIONS_UPDATED_TEXT);
                    });
                }).error(function (data) {
                    restoreContent(function () {
                        //NOTE: reinitialisation after inserting content
                        $reportsPathInput = $(REPORTS_PATH_INPUT);
                        Util.showInputMsg($reportsPathInput, data.responseText, ERR_MSG_TYPE);
                    });
                });

                e.stopImmediatePropagation();
            });

            initReportsPathDialog();
        };

        var initNetworkOptions = function () {
            var $hostnameInput = $(HOSTNAME),
                $controlPanelPortInput = $(CONTROL_PANEL_PORT),
                $servicePort1Input = $(SERVICE_PORT1),
                $servicePort2Input = $(SERVICE_PORT2);

            $(PORT_BUSY).each(function (index, input) {
                Util.showInputMsg($(input), PORT_IS_AUTO_DETECTING_TEXT, WARN_MSG_TYPE);
            });

            $(AUTO_DETECT_NETWORK).click(function () {
                var $input = $(this).closest('tr').find('td input[type="text"]');

                if ($(this).prop('checked')) {
                    $input.attr('readonly', 'readonly').addClass(READONLY_CLASS);
                    $(this).attr('checked', 'checked');
                }
                else {
                    $input.removeAttr('readonly').removeClass(READONLY_CLASS);
                    $(this).removeAttr('checked');
                }
            });

            $(APPLY_NETWORK_OPTIONS_BTN).click(function () {
                var $cont = $(SERVER_OPTIONS),
                    hostname = $hostnameInput.val(),
                    controlPanelPort = parseInt($controlPanelPortInput.val()),
                    servicePort1 = parseInt($servicePort1Input.val()),
                    servicePort2 = parseInt($servicePort2Input.val()),
                    options = {};

                var uxLogCbState = [];

                $(AUTO_DETECT_NETWORK).each(function () {
                    uxLogCbState.push(this.id + ':' + $(this).prop('checked'));
                });

                UXLog.write('Settings: "Apply network settings" clicked - ' + uxLogCbState.join(' | '));

                options.hostname = !$hostnameInput.hasClass(READONLY_CLASS) ? hostname : "";
                if (!$servicePort1Input.hasClass(PORT_BUSY_CLASS) || !$servicePort1Input.hasClass(READONLY_CLASS))
                    options.service_port1 = !$servicePort1Input.hasClass(READONLY_CLASS) ? servicePort1 : "";

                if (!$servicePort2Input.hasClass(PORT_BUSY_CLASS) || !$servicePort2Input.hasClass(READONLY_CLASS))
                    options.service_port2 = !$servicePort2Input.hasClass(READONLY_CLASS) ? servicePort2 : "";

                if (!$controlPanelPortInput.hasClass(PORT_BUSY_CLASS) || !$controlPanelPortInput.hasClass(READONLY_CLASS))
                    options.control_panel_port = !$controlPanelPortInput.hasClass(READONLY_CLASS) ? controlPanelPort : "";

                removeMsg();
                Util.removeInputMsgs($cont);
                $(PORT_BUSY).removeClass(PORT_BUSY_CLASS);

                var portsInputArr = [
                    {
                        value: servicePort1,
                        $el: $servicePort1Input
                    },
                    {
                        value: servicePort2,
                        $el: $servicePort2Input
                    },
                    {
                        value: controlPanelPort,
                        $el: $controlPanelPortInput
                    }
                ], isValuesValid = true;

                while (portsInputArr.length > 0) {
                    var port = portsInputArr.pop();

                    if(port.$el.hasClass(READONLY_CLASS))
                        continue;

                    if (!port.value) {
                        Util.showInputMsg(port.$el, EMPTY_FIELD_TEXT, ERR_MSG_TYPE);
                        isValuesValid = false;
                    }

                    if (!portValueIsValid(port.value)) {
                        Util.showInputMsg(port.$el, INVALID_PORT_VALUE_TEXT, ERR_MSG_TYPE);
                        isValuesValid = false;
                    }

                    for (var i = 0, length = portsInputArr.length; i < length; i++) {
                        if (port.value && !portsInputArr[i].$el.hasClass(READONLY_CLASS) && port.value === portsInputArr[i].value) {
                            Util.showInputMsg(port.$el, IDENTICAL_VALUES_TEXT, ERR_MSG_TYPE);
                            Util.showInputMsg(portsInputArr[i].$el, IDENTICAL_VALUES_TEXT, ERR_MSG_TYPE);
                            isValuesValid = false;
                        }
                    }
                }

                if (!hostname) {
                    Util.showInputMsg($hostnameInput, EMPTY_FIELD_TEXT, ERR_MSG_TYPE);
                    isValuesValid = false;
                }

                if (!isValuesValid)
                    return;

                //NOTE: save attribute value for inputs for restore content
                if(hostname)
                    $hostnameInput.attr('value', hostname);
                if(servicePort1)
                    $servicePort1Input.attr('value', servicePort1);
                if(servicePort2)
                    $servicePort2Input.attr('value', servicePort2);
                if(controlPanelPort)
                    $controlPanelPortInput.attr('value', controlPanelPort);

                $(AUTO_DETECT_NETWORK).each(function (index, el) {
                    var $el = $(el);

                    if (!$el.prop('checked'))
                        $el.removeAttr('checked');
                    else
                        $el.prop('checked', 'checked');
                });

                $(INPUTS_NETWORK).each(function(index, el) {
                    var $el = $(el);

                    if(!$el.prop('readonly'))
                        $el.removeAttr('readonly');
                    else
                        $el.attr('readonly', 'readonly');
                });

                createLoadingMsg();

                Util.removeInputMsgs($cont);

                $.post(RESTART_URL, {options: options}).success(function (warn) {
                    restoreContent(function () {
                        if (warn.hasChanged)
                            showMsg(NEED_RESTART_TEXT);

                        if (warn.controlPanelPort && !$controlPanelPortInput.hasClass(READONLY_CLASS))
                            Util.showInputMsg($(CONTROL_PANEL_PORT), PORT_ALREADY_IN_USE_TEXT, WARN_MSG_TYPE);
                        else if (warn.servicePort && !$servicePort1Input.hasClass(READONLY_CLASS))
                            Util.showInputMsg($(SERVICE_PORT1), PORT_ALREADY_IN_USE_TEXT, WARN_MSG_TYPE);
                    });
                }).error(function (data) {
                        restoreContent();
                        showMsg(data.responseText, ERR_MSG_TYPE);
                    });
            });
        };

        var initBrowserOptionsDialog = function () {
            var $cont = $(BROWSER_OPTIONS_DIALOG_CONT),
                $nameInput = $(EDIT_BROWSER_NAME),
                $pathInput = $(EDIT_BROWSER_PATH),
                $cmdInput = $(EDIT_BROWSER_CMD),
                $iconInput = $(ICON_TYPES),
                $iconInputIcons = $iconInput.find('li'),
                $fsTreeView = $(FS_TREEVIEW_CONT),
                $dialogTitleText = null,
                initBrowserName = "";

            $browserOptionsDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: BROWSER_OPTIONS_DIALOG_CLASS,
                width: BROWSER_DIALOG_WIDTH
            });

            $dialogTitleText = $cont.parent().find(UI_DIALOG_TITLE_CONT).find('span');

            var initBrowserDialogOptions = function ($browserOptions) {
                Util.removeInputMsgs($cont);

                if ($browserOptions) {
                    var $tr = $browserOptions.closest('tr'),
                        $icon = $iconInput.find('span.' + $tr.attr(DATA_BROWSER_ICON_ATTR));

                    $dialogTitleText.text(Util.formatText(EDIT_BROWSER_DIALOG_TITLE_TEXT, $tr.attr(DATA_BROWSER_NAME_ATTR)));

                    $nameInput.val($tr.attr(DATA_BROWSER_NAME_ATTR));
                    $pathInput.val($tr.attr(DATA_BROWSER_PATH_ATTR));
                    $cmdInput.val($tr.attr(DATA_BROWSER_CMD_ATTR));

                    initBrowserName = $tr.attr(DATA_BROWSER_NAME_ATTR);

                    $iconInputIcons.removeClass(ICON_CHECKED_CLASS);

                    if ($icon.length)
                        $icon.closest('li').addClass(ICON_CHECKED_CLASS);
                    else
                        $iconInput.find(ICON_UNKNOWN).closest('li').addClass(ICON_CHECKED_CLASS);
                } else {
                    $dialogTitleText.text(ADD_BROWSER_DIALOG_TITLE_TEXT);

                    $nameInput.val('');
                    $pathInput.val('');
                    $cmdInput.val('');
                    $iconInputIcons.removeClass(ICON_CHECKED_CLASS);
                    $iconInput.find(ICON_UNKNOWN).closest('li').addClass(ICON_CHECKED_CLASS);
                    $fsTreeView.html('');
                }
            };

            $document.on('click', EDIT_BROWSER_BTN, function () {
                UXLog.write('Settings: "Modify browser settings" clicked');
                initBrowserDialogOptions($(this));
                $browserOptionsDialog.dialog('open');
            });

            $document.on('click', ADD_BROWSER_BTN, function () {
                UXLog.write('Settings: "Add new browser" clicked');
                initBrowserDialogOptions(null);
                $browserOptionsDialog.dialog('open');
            });

            $iconInputIcons.click(function () {
                $iconInputIcons.removeClass(ICON_CHECKED_CLASS);
                $(this).addClass(ICON_CHECKED_CLASS);
            });

            $(BROWSER_FILE).click(function () {
                $browserOptionsDialog.dialog('close');
                $changeBrowserDialog.dialog('open');
            });

            $(BROWSER_OPTIONS_OK_BTN).click(function () {
                var name = $nameInput.val(),
                    path = $pathInput.val(),
                    cmd = $cmdInput.val(),
                    browserIcon = $iconInputIcons.filter(ICON_CHECKED).attr(DATA_BROWSER_ICON_ATTR);

                Util.removeInputMsgs($cont);

                var browserWithSameName = $([BROWSER_ROW, '[', DATA_BROWSER_NAME_ATTR, '="', name, '"]'].join('')),
                    browserIsAlreadyExists = browserWithSameName.length && name !== initBrowserName;

                if (!name || !path || browserIsAlreadyExists) {
                    if (!name)
                        Util.showInputMsg($nameInput, EMPTY_FIELD_TEXT, ERR_MSG_TYPE);
                    if (!path)
                        Util.showInputMsg($pathInput, EMPTY_FIELD_TEXT, ERR_MSG_TYPE);
                    if (browserIsAlreadyExists)
                        Util.showInputMsg($nameInput, BROWSER_IS_ALREADY_EXISTS, ERR_MSG_TYPE);

                    return;
                }

                configureBrowser(name, initBrowserName, path, cmd, browserIcon, function (err) {
                    if (err) {
                        var $input = err.pathError ? $pathInput : $nameInput,
                            errMsg = err.saveError || BROWSER_PATH_NOT_EXISTS_TEXT;

                        Util.showInputMsg($input, errMsg, ERR_MSG_TYPE);
                        $browserOptionsDialog.dialog('open');
                    } else
                        $browserOptionsDialog.dialog('close');
                });
            });

            $(BROWSER_OPTIONS_CANCEL_BTN).click(function () {
                $browserOptionsDialog.dialog('close');
            });
        };

        var initChangeBrowserDialog = function () {
            var $cont = $(CHANGE_BROWSER_DIALOG),
                $fsTreeView = $(FS_TREEVIEW_CONT),
                $fsTreeViewCont = $fsTreeView.parent(),
                $inputPath = $(CHANGE_BROWSER_INPUT),
                $inputPathNextElm = $inputPath.next(),
                $inputErr = $inputPathNextElm.find(TITLE_CONT),
                browserFile = '',
                inputFocus = false,
                loadItems = function () {
                    $fsTreeViewCont.scrollBar({
                        scrollTo: FS_TREEVIEW_CONT + ' li.' + TREE_ITEM_SELECTED_CLASS,
                        scrollClass: CHANGE_BROWSER_SCROLLBAR_CLASS
                    });
                },
                treeviewOptions = {
                    url: GET_FS_BY_PATH_URL,
                    open: '',
                    effect: 'animate',
                    time: FS_EXPAND_DURATION,
                    asyncAction: loadItems
                };

            $changeBrowserDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: CHANGE_BROWSER_CLASS,
                width: 'auto'
            });

            var clearInputDirErr = function () {
                $inputPath.removeClass(ERR_VALUE_CLASS);
                $inputPathNextElm.hide();
                $inputErr.html();
            };

            $document.on('click', BROWSER_FILE, function () {
                var initValue = $(EDIT_BROWSER_PATH).val();

                $inputPath.val(initValue);
                browserFile = treeviewOptions.open = initValue;

                $changeBrowserDialog.dialog('open');

                $cont.parent().find(UI_DIALOG_TITLE_CONT).draggable();
                $fsTreeViewCont.scrollBar({
                    scrollTo: FS_TREEVIEW_CONT + ' li.' + TREE_ITEM_SELECTED_CLASS,
                    scrollClass: CHANGE_BROWSER_SCROLLBAR_CLASS
                });

                $fsTreeViewCont.resizable({
                    minWidth: CHANGE_BROWSER_DIALOG_WIDTH,
                    minHeight: $fsTreeViewCont.height(),
                    class: RESIZE_MARKER_CHANGE_BROWSER_FILE_CLASS,
                    action: function () {
                        $fsTreeViewCont.scrollBar({
                            scrollClass: CHANGE_BROWSER_SCROLLBAR_CLASS
                        });
                    }
                });

                $fsTreeView.treeView(treeviewOptions);
            });

            $inputPath.focusout(function () {
                inputFocus = false;
            });

            $inputPath.click(function () {
                if (!inputFocus) {
                    inputFocus = true;
                    $(this).select();
                }
            });

            $inputPath.keyup(function (e) {
                treeviewOptions.open = $inputPath.val();
                treeviewOptions.effect = false;
                $fsTreeView.treeView(treeviewOptions);
                treeviewOptions.effect = 'animate';

                if (e.keyCode !== ENTER_BTN_CODE) {
                    $inputPath.removeClass(ERR_VALUE_CLASS);
                    $inputPathNextElm.hide();
                    $inputPathNextElm.find(TITLE_CONT).html();
                }
            });

            $document.on('click', [CHANGE_BROWSER_DIALOG, HOME_DIR_BTN].join(' '), function(e) {
                $inputPath.val(window.HOME_DIR);
                $inputPath.keyup();

                e.stopImmediatePropagation();
            });

            $(CHANGE_BROWSER_OK_BTN).click(function () {
                var inputValue = $inputPath.val();

                $inputPathNextElm.hide();

                if ($.trim(inputValue)) {
                    browserFile = inputValue;
                    $inputPath.val(decodeURIComponent(browserFile));
                }

                clearInputDirErr();
                $changeBrowserDialog.dialog('close');
                $browserOptionsDialog.find(EDIT_BROWSER_PATH).val(inputValue);
                $browserOptionsDialog.dialog('open');
            });

            $(CHANGE_BROWSER_CANCEL_BTN).click(function () {
                $changeBrowserDialog.dialog('close');
                $browserOptionsDialog.dialog('open');
            });

            $document.on('click', FS_TREEVIEW_CONT + ' li', function () {
                browserFile = $(this).attr(PATH_ATTR);
                $inputPath.val(decodeURIComponent(browserFile));
            });
        };

        var initDetectBrowsersBtn = function () {
            $(DETECT_BROWSERS_BTN).click(function () {
                UXLog.write('Settings: "Find installed browsers" clicked');
                detectBrowser();
            });
        };

        var initContent = function () {
            initNetworkOptions();
            initReportsOptions();
            initDetectBrowsersBtn();
            initEnterBtn();
        };

        var initHorizontalScrollHandler = function () {
            $(window).scroll(function () {
                var scrollLeft = $(this).scrollLeft();

                $(HEADER_BACK).css('left', -scrollLeft);
                $(OPTIONS_TAB_CONT).css('left', -scrollLeft);
                $(FOOTER).css('left', -scrollLeft);
            });
        };


        $document.ready(function () {
            UXLog.write('Settings opened');

            initHorizontalScrollHandler();
            initTabs();
            initDeleteBrowserDialog();
            initBrowserOptionsDialog();
            initChangeBrowserDialog();

            initContent();
        });
    };
});