/* global History: true */
/* global io: true */

ControlPanel.define('ProjectView', function (require, exports) {
    require('Widgets.Dialog');
    require('Widgets.Draggable');
    require('Widgets.Resizable');
    require('Widgets.ScrollBar');
    require('Widgets.TreeView');

    var $ = require('jQuery'),
        FixtureEditor = require('Widgets.FixtureEditor'),
        NotificationsList = require('Widgets.NotificationsList'),
        Util = require('Util'),
        UXLog = require('UXLog');

    exports.init = function () {
        //Constants
        var
            FIXTURE_FILE_EXTENSION = '.test.js',
            FIXTURE_FILE_EXTENSION_PATTERN = /\.test\.js$/,
            FILENAME_ILLEGAL_CHARACTERS_PATTERN = /[\\\/:*?"<>|]/g,
            INVALID_URL_CHARACTER_PATTERN = /[!@#&'*,%:;<=>?\[\]^`\{\}]/g,
            ENTER_BTN_CODE = 13,

        //Cookie
            PROJECT_NAVIGATION_STATE_COOKIE = 'project_navigation_state',
            PROJECT_PAGE_STATE_COOKIE = 'project_page_state',
            RUN_DIALOG_STATE_COOKIE = 'run_dialog_state',
            COOKIE_EXPIRED_DAYS = 1,

        //Selectors
            HEADER = '.header',
            FOOTER = '.footer',
            HEADER_BACK = '.header .back',
            NO_TESTS = '.no-tests',
            TOP_BUTTONS = '.fixtures-top-buttons',
            BROWSER = '#project-browser',
            PROJECT_VIEW = '.project-view',
            BUILD_ERRS = '#build-errs',
            BUILD_ERRS_CONTAINER = '.build-errs-container',
            RENAME_DIR_DIALOG_CONT = '#rename-dir-dialog',
            RENAME_DIR_BTN = '#rename-dir-btn',
            RENAME_DIR_INPUT = '#rename-dir-dir-name',
            DELETE_DIR_DIALOG_CONT = '#delete-dir-dialog',
            DELETE_DIR_YES_BTN = '#delete-dir-yes-btn',
            DELETE_DIR_NO_BTN = '#delete-dir-no-btn',
            RENAME_TEST_DIALOG_CONT = '#rename-test-dialog',
            RENAME_TEST_INPUT = '#test-new-name',
            RENAME_TEST_BTN = '#rename-test-btn',
            EDIT_FIXTURE_DIALOG_CONT = '#edit-fixture-dialog',
            EDIT_FIXTURE_OK_BTN = '#edit-fixture-btn',
            EDIT_FIXTURE_NAME_INPUT = '#edit-fixture-name',
            EDIT_FILENAME_INPUT = '#edit-fixture-filename',
            EDIT_FIXTURE_PAGE_INPUT = '#edit-fixture-page',
            EDIT_FIXTURE_AUTH_CREDENTIALS_GROUP = '#edit-fixture-auth-credentials-group',
            EDIT_FIXTURE_SHOW_CREDENTIALS_LINK = '#edit-fixture-show-credentials',
            EDIT_FIXTURE_AUTH_USERNAME_INPUT = '#edit-fixture-auth-username',
            EDIT_FIXTURE_AUTH_PASSWORD_INPUT = '#edit-fixture-auth-password',
            DELETE_FIXTURE_DIALOG_CONT = '#delete-fixture-dialog',
            DELETE_FIXTURE_YES_BTN = '#delete-fixture-yes-btn',
            DELETE_FIXTURE_NO_BTN = '#delete-fixture-no-btn',
            ADD_FIXTURE_DIALOG_CONT = '#add-fixture-dialog',
            ADD_FIXTURE_INPUT_NAME = '#add-fixture-name',
            ADD_FIXTURE_INPUT_FILENAME = '#add-fixture-filename',
            ADD_FIXTURE_INPUT_PAGE = '#add-fixture-page',
            ADD_FIXTURE_OK_BTN = '#add-fixture-btn',
            ADD_FIXTURE_AUTH_CREDENTIALS_GROUP = '#add-fixture-auth-credentials-group',
            ADD_FIXTURE_SHOW_CREDENTIALS_LINK = '#add-fixture-show-credentials',
            ADD_FIXTURE_AUTH_USERNAME_INPUT = '#add-fixture-auth-username',
            ADD_FIXTURE_AUTH_PASSWORD_INPUT = '#add-fixture-auth-password',
            ADD_DIR_CONT = '#add-dir-dialog',
            ADD_DIR_INPUT_DIR_NAME = '#add-dir-name',
            ADD_DIR_OK_BTN = '#add-dir-btn',
            DELETE_TEST_DIALOG_CONT = '#delete-test-dialog',
            DELETE_TEST_YES_BTN = '#delete-test-yes-btn',
            DELETE_TEST_NO_BTN = '#delete-test-no-btn',
            OPEN_PROJECT_CONT = '#open-project-dialog',
            OPEN_PROJECT_OK_BTN = '#open-project-ok',
            OPEN_PROJECT_CANCEL_BTN = '#open-project-cancel',
            MSG_CONTAINER = '#msg-container',
            DIRS_TREEVIEW_CONT = '#fs-tree',
            OPEN_PROJECT_PATH_INPUT = '#input-path',
            BUILD_ERRS_TITLE = '.title',
            BUILD_ERRS_LIST = '.list',
            BUILD_ERRS_EXP_COL_BUTTON = '.expand-collapse-button',
            FIXTURE_ICON = '.fixture-icon',
            FIXTURE_INFO = '.fixture-info',
            FIXTURE_BUTTON = '.fixture-button',
            NAV_LINK = 'a.nav-link',
            NAV_BUTTON = '.nav-button',
            ANIM_CONTAINER = '.anim-container',
            RUN_BUTTON = '.run-button',
            RECORD_BUTTON = '.record-button',
            BROWSER_ROW = '.browser-row',
            REVISION_CONTAINER = '#revision-container',
            EXPANDED = '.expanded',
            BROWSER_ROW_EXPANDED = BROWSER_ROW + EXPANDED,
            RENAME_DIR_BUTTON = '.rename-dir-btn',
            DELETE_DIR_BUTTON = '.delete-dir-btn',
            EDIT_FIXTURE_BUTTON = '.edit-fixture-btn',
            DELETE_FIXTURE_BUTTON = '.delete-fixture-btn',
            RENAME_TEST_BUTTON = '.rename-test-btn',
            DELETE_TEST_BUTTON = '.delete-test-btn',
            ADD_FIXTURE_BTN = '.add-fixture-btn',
            ADD_DIR_BTN = '.add-dir-btn',
            OPEN_PROJECT_BTN = '.open-project-btn',
            UI_DIALOG_TITLE_CONT = '.dialog-header',
            UI_DIALOG_TITLE = '.dialog-header span',
            UI_DIALOG = 'div.dialog',
            UI_DIALOG_BTN = '.button',
            ADD_FIXTURE_DIALOG = '.add-fixture',
            ADD_DIR_DIALOG = '.add-dir',
            OPEN_PROJECT_DIALOG = '.open-project',
            FIXTURE_FILENAME = '.fixture-filename',
            DIALOG_TEXT = '.dialog-text h2',
            BREADCRUMBS = '.breadcrumbs',
            EDIT_FIXTURE_CODE_BTN = '.edit-fixture-code-btn',
            EXPAND_TESTS_GROUP_BTN = '.expand-test-group',
            COLLAPSE_TESTS_GROUP_BTN = '.collapse-test-group',
            TESTS_GROUP = '.group-tests',
            TESTS_GROUP_CONTENT = '.group-tests-content',
            CREATE_PROJECT_DIALOG = '.create-project',
            CREATE_PROJECT_DIALOG_CONT = '#create-project-dialog',
            CREATE_PROJECT_FS_TREEVIEW_CONT = '#create-project-fs-tree',
            PROJECT_PATH_INPUT = '#project-path',
            PROJECT_NAME_INPUT = '#project-name',
            CREATE_PROJECT_BTN = '.create-project-btn',
            CREATE_PROJECT_OK_BTN = '#create-project-ok-btn',
            CREATE_PROJECT_CANCEL_BTN = '#create-project-cancel',
            FIXTURE_TOP_BTNS = '.fixtures-top-buttons',
            EXAMPLE_PROJECT_BTN = '.example-project-btn',
            CLOSE_PROJECT_BTN = '#close-project-btn',
            START_PAGE_CONT = '.start-page-cont',
            HOME_DIR_BTN = '.home-dir',
            WORKER_LIST_DIALOG_WRAPPER = '#worker-list-dialog-wrapper',
            WORKER_LIST_DIALOG = '#worker-list-dialog',
            DIALOG_RUN_BUTTON = '#dialog-run-btn',
            TAKE_SCREENSHOT_OPTION = '#take-screenshot-option',
            FAIL_ON_JS_ERROR_OPTION = '#fail-on-js-errors-option',
            SELECT_ALL_BROWSERS_BTN = '#select-all-browsers',
            DESELECT_ALL_BROWSERS_BTN = '#deselect-all-browsers',
            SELECT_ALL_WORKERS_BTN = '#select-all-workers',
            DESELECT_ALL_WORKERS_BTN = '#deselect-all-workers',
            BROWSER_SELECT = '.select-browser',
            WORKER_SELECT = '.select-worker',
            SELECTED_BROWSER = BROWSER_SELECT + ':checked',
            SELECTED_WORKER = WORKER_SELECT + ':checked',
            RUN_DIALOG_WORKERS_LIST = '.connected-workers-wrap',
            RUN_DIALOG_BROWSERS_LIST = '.browser-workers-wrap',

        //Attrs
            DIR_NAME_ATTR = 'data-dir-name',
            FIXTURE_FILE_ATTR = 'data-fixture-file',
            FIXTURE_NAME_ATTR = 'data-fixture-name',
            FIXTURE_PAGE_ATTR = 'data-fixture-page',
            FIXTURE_PASSWORD_ATTR = 'data-fixture-password',
            FIXTURE_UID_ATTR = 'data-fixtureuid',
            FIXTURE_USERNAME_ATTR = 'data-fixture-username',
            GROUP_NAME_ATTR = 'data-group',
            INPUT_PATH_ATTR = 'data-path',
            SOURCE_ATTR = 'data-source',
            SOURCE_TYPE_ATTR = 'data-source-type',
            TEST_NAME_ATTR = 'data-test-name',
            BROWSER_NAME_ATTR = 'data-browsername',
            WORKER_NAME_ATTR = 'data-workername',

        //Classes
            BROWSER_ROW_EXPANDED_CLASS = 'expanded',
            BTN_COLLAPSE_CLASS = 'collapse',
            BTN_EXPAND_CLASS = 'expand',
            SUCCESS_STATUS_CLASS = 'project-browser-status success',
            WITH_ERRORS_STATUS_CLASS = 'project-browser-status with-errors',
            FAIL_STATUS_CLASS = 'project-browser-status fail',
            UPDATE_STATUS_CLASS = 'project-browser-status update',
            ADD_FIXTURE_CLASS = 'add-fixture',
            ADD_DIR_CLASS = 'add-dir',
            EDIT_FIXTURE_CLASS = 'edit-fixture',
            DELETE_FIXTURE_CLASS = 'delete-fixture',
            RENAME_DIR_CLASS = 'rename-dir',
            DELETE_DIR_CLASS = 'delete-dir',
            RENAME_TEST_CLASS = 'rename-test',
            DELETE_TEST_CLASS = 'delete-test',
            LOADING_MSG_CLASS = 'loading-img',
            AUTO_INPUT_CLASS = 'auto-input',
            HOVER_EMULATION_CLASS = 'hover-emulation',
            GROUP_HEADER_CLASS = 'group-header',
            OPEN_PROJECT_CLASS = 'open-project',
            TREEVIEW_SELECTED_ITEM_CLASS = 'selected',
            OPEN_PROJECT_SCROLLBAR_CLASS = 'open-project-scrollbar',
            RESIZE_MARKER_OPEN_PROJECT_DIALOG_CLASS = 'resize-marker-open-project',
            CREATE_PROJECT_CLASS = 'create-project',
            CREATE_PROJECT_SCROLLBAR_CLASS = 'create-project-scrollbar',
            RESIZE_MARKER_CREATE_PROJECT_CLASS = 'resize-create-project-dialog',
            RUN_DIALOG_WORKERS_SCROLLBAR_CLASS = 'connected-workers-scrollbar',
            RUN_DIALOG_BROWSERS_SCROLLBAR_CLASS = 'browser-workers-scrollbar',

        //Urls
            SLIDE_QUERY_PARAM = '?slide=1',
            REBUILD_REVISION_PARAM = '?rebuild=1',
            RUN_TASK_URL = '/tests_run/',
            START_RECORDING_URL = '/recording/start/',
            TRACK_TASK_URL = '/results/',
            PROJECT_URL = '/project/',
            RENAME_DIR_URL = '/rename_dir/',
            DELETE_DIR_URL = '/delete_dir/',
            CREATE_FIXTURE_URL = '/create_fixture/',
            CREATE_DIR_URL = '/create_dir/',
            EDIT_FIXTURE_URL = '/edit_fixture/',
            DELETE_FIXTURE_URL = '/delete_fixture/',
            RENAME_TEST_URL = '/rename_test/',
            DELETE_TEST_URL = '/delete_test/',
            IDLE_WORKER_URL = '/worker/idle/',
            OPEN_PROJECT_URL = '/open_project/',
            OPEN_EXAMPLE_PROJECT_URL = '/open_project?example=1',
            CLOSE_PROJECT_URL = '/close_project/',
            GET_DIR_BY_PATH_URL = '/dirs_by_path/',
            RETURN_URL_QUERY_PARAM = '?returnUrl=',
            EXPAND_ROW_PARAM = '?expand=',
            CREATE_PROJECT_URL = '/create_project/',
            LIST_WORKERS_URL = '/worker/list?render=1',

        //Status messages
            TASK_START_SUCCESS_MSG = 'Task was successfully started',
            TASK_STARTED_WITH_ERRORS_MSG = 'Task was started with errors',
            TASK_START_FAILED_MSG = 'Failed to start task',
            START_RECORDING_FAILED_MSG = 'Failed to start test recording for the selected test fixture',
            VFS_REBUILDED_MSG = 'File system has been updated',
            CREATE_FIXTURE_SUCCESS_MSG_PATTERN = 'The "{0}" test fixture has been successfully created',
            CREATE_DIR_SUCCESS_MSG_PATTERN = 'The "{0}" directory has been successfully created',
            RENAME_DIR_SUCCESS_MSG_PATTERN = 'The directory has been renamed. <span>Previous name: "{0}".<br/>New name: "{1}".</span>',
            DELETE_DIR_SUCCESS_MSG_PATTERN = 'The "{0}" directory has been successfully deleted',
            EDIT_FIXTURE_SUCCESS_MSG_PATTERN = 'The test fixture has been changed',
            EDIT_FIXTURE_FILENAME_MSG_PATTERN = '<span>Previous file name: "{0}".<br/>New file name: "{1}".</span>',
            EDIT_FIXTURE_NAME_MSG_PATTERN = '<span>Previous name: "{0}".<br/>New name: "{1}".</span>',
            EDIT_FIXTURE_PAGE_MSG_PATTERN = '<span>Previous web page: "{0}".<br/>New web page: "{1}".</span>',
            RENAME_TEST_SUCCESS_MSG_PATTERN = 'Test has been renamed. <span>Previous name: "{0}".<br/>New name: "{1}".</span>',
            DELETE_FIXTURE_SUCCESS_MSG_PATTERN = 'The "{0}" test fixture has been successfully deleted',
            DELETE_TEST_SUCCESS_MSG_PATTERN = 'The "{0}" test has been successfully deleted',
            SAVE_FIXTURE_CODE_SUCCESS_MSG_PATTERN = 'The "{0}" test fixture has been changed',

        //Error msgs
            EMPTY_NAME_ERR_MSG = 'Required field.',
            EMPTY_PAGE_ERR_MSG = 'Required field.',
            EMPTY_FILENAME_ERR_MSG = 'Required field.',
            EMPTY_DIR_NAME_ERR_MSG = 'Required field.',
            EMPTY_AUTH_USERNAME_ERR_MSG = 'Required field.',

        //Ui text
            RENAME_DIR_DIALOG_TITLE_PATTERN = 'Rename directory: "{0}"',
            RENAME_TEST_DIALOG_TITLE_PATTERN = 'Rename test: "{0}"',
            EDIT_FIXTURE_DIALOG_TITLE_PATTERN = 'Modify fixture: "{0}"',
            DELETE_DIALOG_TEXT_PATTERN = 'Are you sure you want to delete "{0}"?',
            FIXTURE_ALREADY_EXISTS_ERR_MSG_PATTERN = 'The "{0}" fixture already exists in the current directory.',

        //Metrics
            BASE_PATH_ARRAY_LENGTH = 4,
            CREATE_PROJECT_DIALOG_HEIGHT = 300,
            CREATE_PROJECT_TREEVIEW_WIDTH = 570,
            DIFFERENCE_BETWEEN_SELECT_AND_CLICK = 5,
            FS_EXPAND_DURATION = 200,
            FS_OPERATION_DIALOG_WIDTH = 610,
            LOADING_MSG_DURATION = 750,
            OPEN_PROJECT_DIALOG_HEIGHT = 300,
            OPEN_PROJECT_TREEVIEW_WIDTH = 560,
            SLIDE_DURATION = 200,
            WORKER_LIST_DIALOG_WIDTH = 570;

        //Globals
        var socket = null,
            $document = $(document),
            $window = $(window),
            $notificationContainer = null,
            $runDialog = null,
            $body = $('html, body'),
            pageTitle = document.title,
            scrollTop = 0,
            fsOperationFlag = false,
            stopObservationForRevision = false,
            fsOperation = {text: '', status: ''},
            expFixtureFilenames = [],
            expGroupFilenames = [],
            activeFixture = null,
            $addFixtureDialog = null,
            $addDialog = null,
            $projectBrowserContent = null,
            slideCallback = null,
            expandedErr = false,
            currentProject = window.CURRENT_PROJECT,
            loadAnimationStartTime = 0;

        //Utils
        var getValidFilename = function (filename) {
            return  filename.replace(FILENAME_ILLEGAL_CHARACTERS_PATTERN, '')
                .replace(/\s/g, '_')
                .toLowerCase();
        };

        var isStartPage = function () {
            return $(START_PAGE_CONT).length;
        };

        var getFixtureRow = function(fixtureName) {
            return $('[' + FIXTURE_NAME_ATTR + '="' + fixtureName + '"]');
        };

        var openTestTarget = function(fixtureName, testName, callback) {
            var $fixture = getFixtureRow(fixtureName);

            if(!testName)
                expandedBrowserRow($fixture.attr(FIXTURE_FILE_ATTR), true, false);
            else {
                expandedBrowserRow($fixture.attr(FIXTURE_FILE_ATTR), false, false);
                var $test =  $fixture.next().find('[' + TEST_NAME_ATTR + '="' + testName + '"]').eq(0);

                $test.effect('pulsate', {times: 1}, 'slow', function() {});
            }

            callback($fixture);
        };

        var expandedBrowserRow = function (fixtureFileName, enableAnimation, goToRow, callback) {
            //NOTE: Fix opera problem :contains filter
            var $titleSearch = $(BROWSER_ROW).filter(function () {
                var fileName = $.trim($(this).attr(FIXTURE_FILE_ATTR));
                return fixtureFileName === fileName;
            });

            if ($titleSearch) {
                $titleSearch.addClass(BROWSER_ROW_EXPANDED_CLASS);
                $titleSearch.find(FIXTURE_ICON).addClass(BTN_COLLAPSE_CLASS);
                if (enableAnimation)
                    $titleSearch.next().find(ANIM_CONTAINER).show('slow', function () {
                        if(goToRow)
                            Util.scrollTo($titleSearch);

                        $titleSearch.effect('pulsate', {times: 1}, 'slow', function() {});
                        $titleSearch.next().effect('pulsate', {times: 1}, 'slow', function() {
                            if (typeof callback === 'function')
                                callback($titleSearch);
                        });
                    });
                else {
                    $titleSearch.next().find(ANIM_CONTAINER).show();
                    if (typeof callback === 'function')
                        callback($titleSearch);
                }
            }
        };

        var expandTestsGroup = function (groupItem) {
            var $fixture = $(BROWSER_ROW).filter(function () {
                var fileName = $.trim($(this).attr(FIXTURE_FILE_ATTR));
                return groupItem.filename === fileName;
            });

            var $tr = $fixture.next().find('tr[' + GROUP_NAME_ATTR + '="' + groupItem.group + '"]');

            $tr.eq(0).hide();
            $tr.eq(1).css('display', 'block');
        };

        var execActionAfterAnimationTimeout = function (action) {
            var animationDuration = (new Date()).getTime() - loadAnimationStartTime;

            var actionByTimeout = function () {
                action();
                loadAnimationStartTime = 0;
            };

            if (animationDuration < LOADING_MSG_DURATION)
                setTimeout(actionByTimeout, LOADING_MSG_DURATION - animationDuration);
            else
                actionByTimeout();
        };

        //Navigation
        var slideToUpperLevel = function () {
            clearDialogData();

            var to = location.href.split('/');

            if (to.length > BASE_PATH_ARRAY_LENGTH)
                to.pop();

            to = to.join('/');

            if (History.enabled)
                History.pushState(null, pageTitle, to);
            else
                location.href = to;
        };

        var slide = function (from, to, disableAnimation) {
            var $container = $(BROWSER).parent(),
                forward = Util.getPathDepth(to) > Util.getPathDepth(from);

            $.cookie(PROJECT_NAVIGATION_STATE_COOKIE, window.location.pathname, { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });
            $.cookie(PROJECT_PAGE_STATE_COOKIE, '', { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });

            $.get([to, SLIDE_QUERY_PARAM].join(''),function (res) {
                if (!disableAnimation)
                    $container.hide('drop', { direction: forward ? 'left' : 'right' }, SLIDE_DURATION, function () {
                        clearDialogData();
                        $container.html(res);

                        $container.show('drop', { direction: forward ? 'right' : 'left' }, SLIDE_DURATION, function () {
                            initContent();

                            if (typeof slideCallback === 'function')
                                slideCallback();
                        });
                    });
                else {
                    clearDialogData();
                    $container.html(res);
                    initContent();

                    if (typeof slideCallback === 'function')
                        slideCallback();
                }
            }).error(function () {
                    slideToUpperLevel();
                });
        };

        var setStartPageOffset = function () {
            var $startPageMsgCont = $(START_PAGE_CONT),
                topMargin = Math.ceil((Util.getContentHeight() - $(BUILD_ERRS_CONTAINER).outerHeight(true) - $startPageMsgCont.height()) / 4);

            if (topMargin > 0 && $startPageMsgCont.length)
                $startPageMsgCont.css('margin-top', topMargin);
        };

        //State management
        var saveState = function () {
            scrollTop = $window.scrollTop();
            expFixtureFilenames = [];
            expGroupFilenames = [];
            expandedErr = $(BUILD_ERRS).find(BUILD_ERRS_LIST).css('display') === 'block';

            $(BROWSER).find(BROWSER_ROW_EXPANDED).each(function () {
                var $expItem = $(this),
                    fixtureFilename = $.trim($expItem.attr(FIXTURE_FILE_ATTR));

                expFixtureFilenames.push(fixtureFilename);
            });

            $(TESTS_GROUP).each(function () {
                var $el = $(this);

                if ($el.css('display') !== 'none')
                    expGroupFilenames.push({
                        group: $el.attr(GROUP_NAME_ATTR),
                        filename: $el.closest('td').attr(FIXTURE_FILE_ATTR)
                    });
            });
        };

        var restoreState = function (enableAnimation) {
            for (var i = 0; i < expFixtureFilenames.length; i++)
                expandedBrowserRow(expFixtureFilenames[i], enableAnimation, false);

            //NOTE: animation for the restoring state of group tests is always disabled
            for (i = 0; i < expGroupFilenames.length; i++)
                expandTestsGroup(expGroupFilenames[i]);

            if (expandedErr) {
                var $buildErrs = $(BUILD_ERRS);

                $buildErrs.find(BUILD_ERRS_LIST).show();
                $buildErrs.find(BUILD_ERRS_EXP_COL_BUTTON).toggleClass(BTN_COLLAPSE_CLASS);
            }

            if (!activeFixture)
                $body.scrollTop(scrollTop);
            else
                expandedBrowserRow(activeFixture, true, true, function () {
                    activeFixture = null;
                });
        };

        var saveStateToCookie = function () {
            saveState();

            var pageState = {
                expFixtureFilenames: expFixtureFilenames,
                expGroupFileNames: expGroupFilenames,
                expandedErr: expandedErr,
                location: window.location.href
            };

            $.cookie(PROJECT_PAGE_STATE_COOKIE, JSON.stringify(pageState), { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });
        };

        var initPageStateFromCookie = function () {
            var pageState = $.cookie(PROJECT_PAGE_STATE_COOKIE);

            if (pageState) {
                pageState = JSON.parse($.cookie(PROJECT_PAGE_STATE_COOKIE));

                if (Util.cutQueryString(pageState.location) === Util.cutQueryString(window.location.href)) {
                    expFixtureFilenames = pageState.expFixtureFilenames;
                    expGroupFilenames = pageState.expGroupFileNames;
                    expandedErr = pageState.expandedErr;

                    restoreState(false);
                }
            }
        };

        var setInCenter = function ($elm) {
            var $projectView = $(PROJECT_VIEW),
                topMargin = (Util.getContentHeight() - $(TOP_BUTTONS).height() - $(BUILD_ERRS_CONTAINER).outerHeight(false) - $projectView.outerHeight(true) + $projectView.innerHeight() - $elm.height()) / 2;

            if (topMargin > 0)
                $elm.css('margin-top', topMargin - $(BREADCRUMBS).height());

            var leftMargin = ($(REVISION_CONTAINER).outerWidth(false) - $elm.width()) / 2;

            if (leftMargin > 0)
                $elm.css('margin-left', leftMargin);
        };

        //Loading message for file operations.
        var createLoadingMsg = function () {
            var $revisionContainer = $(REVISION_CONTAINER);

            $projectBrowserContent = $revisionContainer.html();

            var $loadingMsgContent = $('<div></div>')
                .addClass(LOADING_MSG_CLASS);

            $notificationContainer.removeFixedState();
            $(BUILD_ERRS).html('');
            $(BROWSER).html($loadingMsgContent);

            hideTopPanel();
            $notificationContainer.clearList();
            setInCenter($loadingMsgContent);

            loadAnimationStartTime = (new Date()).getTime();
        };

        var restoreContent = function () {
            var restore = function () {
                $(REVISION_CONTAINER).html($projectBrowserContent);
                showTopPanel();
                initBuildErrs();
                initContent();
            };

            execActionAfterAnimationTimeout(restore);
        };

        //Dialogs
        var createAddFixtureDialog = function () {
            $addFixtureDialog = $(ADD_FIXTURE_DIALOG_CONT).dialog({
                autoOpen: false,
                dialogClass: ADD_FIXTURE_CLASS,
                width: FS_OPERATION_DIALOG_WIDTH
            });
        };

        var createDirDialog = function () {
            $addDialog = $(ADD_DIR_CONT).dialog({
                autoOpen: false,
                dialogClass: ADD_DIR_CLASS,
                width: FS_OPERATION_DIALOG_WIDTH
            });
        };

        var clearDialogData = function (clearAllDialogs) {
            if (clearAllDialogs)
                $(UI_DIALOG).find('.dialog-content').dialog('destroy');
            else
                $(UI_DIALOG).not(ADD_FIXTURE_DIALOG).not(ADD_DIR_DIALOG)
                    .not(OPEN_PROJECT_DIALOG).not(CREATE_PROJECT_DIALOG).find('.dialog-content').dialog('destroy');
        };

        //VFS operations
        var addFixture = function (fixtureName, fixtureFilename, fixturePage, fixtureUsername, fixturePassword, callback) {
            fsOperationFlag = true;
            activeFixture = fixtureFilename;
            saveState();
            createLoadingMsg();

            var curPath = Util.getCurPath();

            $.post(CREATE_FIXTURE_URL,
                {   curPath: curPath,
                    fixtureName: fixtureName,
                    filename: fixtureFilename,
                    fixturePage: fixturePage,
                    fixtureUsername: fixtureUsername,
                    fixturePassword: fixturePassword
                },
                function () {
                    fsOperation = {text: Util.formatText(CREATE_FIXTURE_SUCCESS_MSG_PATTERN, fixtureName), status: 'success'};

                    callback();

                }).error(function (data) {
                    fsOperationFlag = false;
                    restoreContent();

                    callback(data.responseText);
                });
        };

        var addDir = function (dirName, callback) {
            fsOperationFlag = true;
            saveState();
            createLoadingMsg();

            var curPath = Util.getCurPath();

            $.post(CREATE_DIR_URL,
                {   curPath: curPath,
                    dirName: dirName
                },
                function () {
                    fsOperation = {text: Util.formatText(CREATE_DIR_SUCCESS_MSG_PATTERN, dirName), status: 'success'};

                    callback();
                }).error(function (data) {
                    fsOperationFlag = false;
                    restoreContent();
                    callback(data.responseText);
                });
        };

        var renameDir = function (dirName, newName, callback) {
            fsOperationFlag = true;
            saveState();
            createLoadingMsg();

            var curPath = Util.getCurPath() + '/' + dirName;

            $.post(RENAME_DIR_URL,
                {   curPath: curPath,
                    newName: newName
                },
                function () {
                    fsOperation = {text: Util.formatText(RENAME_DIR_SUCCESS_MSG_PATTERN, dirName, newName), status: 'success'};

                    callback();
                }).error(function (data) {
                    fsOperationFlag = false;
                    restoreContent();
                    callback(data.responseText);
                });
        };

        var deleteDir = function (dirName) {
            fsOperationFlag = true;
            saveState();
            createLoadingMsg();

            var curPath = Util.getCurPath() + '/' + dirName;

            $.post(DELETE_DIR_URL, {curPath: curPath},function () {
                fsOperation = {text: Util.formatText(DELETE_DIR_SUCCESS_MSG_PATTERN, dirName), status: 'success'};
            }).error(function (data) {
                    fsOperationFlag = false;
                    restoreContent();
                    $notificationContainer.showPermanentErrorMsg(data.responseText);
                });
        };

        var editFixture = function (oldFilename, newFilename, oldName, newName, oldPage, newPage, newUsername, newPassword, callback) {
            fsOperationFlag = true;
            saveState();
            createLoadingMsg();

            var curPath = Util.getCurPath();

            $.post(EDIT_FIXTURE_URL,
                {   curPath: curPath,
                    newName: newName,
                    newPage: newPage,
                    oldFilename: oldFilename,
                    newFilename: newFilename,
                    newUsername: newUsername,
                    newPassword: newPassword
                },
                function () {
                    var text = EDIT_FIXTURE_SUCCESS_MSG_PATTERN +
                        (oldName === newName ? '' : Util.formatText(EDIT_FIXTURE_NAME_MSG_PATTERN, oldName, newName)) +
                        (oldFilename === newFilename ? '' : Util.formatText(EDIT_FIXTURE_FILENAME_MSG_PATTERN, oldFilename, newFilename)) +
                        (oldPage === newPage ? '' : Util.formatText(EDIT_FIXTURE_PAGE_MSG_PATTERN, oldPage, newPage));
                    fsOperation = {text: text, status: 'success'};

                    callback();
                }).error(function (data) {
                    fsOperationFlag = false;
                    restoreContent();
                    callback(data.responseText);
                });
        };

        var deleteFixture = function (fixtureFilename, fixtureName) {
            fsOperationFlag = true;
            saveState();
            createLoadingMsg();

            var curPath = Util.getCurPath();

            $.post(DELETE_FIXTURE_URL,
                {   curPath: curPath,
                    fixtureFilename: fixtureFilename
                },
                function () {
                    fsOperation = {text: Util.formatText(DELETE_FIXTURE_SUCCESS_MSG_PATTERN, fixtureName), status: 'success'};
                }).error(function (data) {
                    fsOperationFlag = false;
                    restoreContent();
                    $notificationContainer.showPermanentErrorMsg(data.responseText);
                });
        };

        var renameTest = function (filename, oldName, newName, callback) {
            fsOperationFlag = true;
            saveState();
            createLoadingMsg();

            var curPath = Util.getCurPath();

            $.post(RENAME_TEST_URL,
                {curPath: curPath,
                    filename: filename,
                    oldName: oldName,
                    newName: newName
                },
                function () {
                    fsOperation = {text: Util.formatText(RENAME_TEST_SUCCESS_MSG_PATTERN, oldName, newName), status: 'success'};

                    callback();
                }).error(function (data) {
                    fsOperationFlag = false;
                    restoreContent();
                    callback(data.responseText);
                });
        };

        var deleteTest = function (filename, testName) {
            fsOperationFlag = true;
            saveState();
            createLoadingMsg();

            var curPath = Util.getCurPath();

            $.post(DELETE_TEST_URL,
                {
                    curPath: curPath,
                    filename: filename,
                    testName: testName
                },
                function () {
                    fsOperation = {text: Util.formatText(DELETE_TEST_SUCCESS_MSG_PATTERN, testName), status: 'success'};
                }).error(function (data) {
                    fsOperationFlag = false;
                    restoreContent();
                    $notificationContainer.showPermanentErrorMsg(data.responseText);
                }
            );
        };

        var createProject = function (projectPath, name, $createProjectDialog, callback) {
            saveState();
            createLoadingMsg();
            $createProjectDialog.dialog('close');

            $.post(CREATE_PROJECT_URL,
                {
                    projectPath: projectPath,
                    name: name
                }, callback).error(function (data) {
                    $createProjectDialog.dialog('open');
                    restoreContent();
                    callback(data.responseText);
                }
            );
        };

        var clearNavigationState = function () {
            $.cookie(PROJECT_PAGE_STATE_COOKIE, '', { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });
            $.cookie(PROJECT_NAVIGATION_STATE_COOKIE, '', { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });
        };

        var openProject = function (projectName, $openProejctDialog, callback) {
            clearNavigationState();
            createLoadingMsg();
            $openProejctDialog.dialog('close');

            $.post(OPEN_PROJECT_URL, {projectName: projectName}, callback, 'text').error(function (data) {
                $openProejctDialog.dialog('open');
                fsOperationFlag = false;
                restoreContent();
                callback(data.responseText);
            });
        };

        //Run task
        var runTask = function (opt) {
            var runXhr = $.ajax({
                type: 'POST',
                url: RUN_TASK_URL,
                data: {opt: opt}
            });

            runXhr.always(function () {
                var res = JSON.parse(runXhr.responseText);

                if (res.workerName && !res.errs.length) {
                    location.href = IDLE_WORKER_URL + res.workerName + RETURN_URL_QUERY_PARAM + location.href;
                    return;
                }

                $.each(res.errs, function () {
                    $notificationContainer.showPermanentErrorMsg(this);
                });

                var statusClassName = {
                        'success': SUCCESS_STATUS_CLASS,
                        'with-errors': WITH_ERRORS_STATUS_CLASS,
                        'fail': FAIL_STATUS_CLASS
                    }[res.status],
                    statusText = {
                        'success': TASK_START_SUCCESS_MSG,
                        'with-errors': TASK_STARTED_WITH_ERRORS_MSG,
                        'fail': TASK_START_FAILED_MSG
                    }[res.status],
                    $statusContent = $('<span></span>').text(statusText),
                    taskResultsLink = TRACK_TASK_URL + res.taskUid;

                if (res.status !== 'fail')
                    $notificationContainer.showStatus(statusClassName, $statusContent, taskResultsLink);
                else
                    $notificationContainer.showStatus(statusClassName, $statusContent);
            });
        };

        //Handle filename input
        var filenameInputHandler = function ($nameInput, $filenameInput) {
            var filenameChanged = false,
                tmpFilename = '';

            $filenameInput.addClass(AUTO_INPUT_CLASS);

            var nameInputHandler = function () {
                if (!filenameChanged)
                    $filenameInput.val(getValidFilename($nameInput.val() + FIXTURE_FILE_EXTENSION));
            };

            var filenameInputHandler = function () {
                if ($filenameInput.val() && $filenameInput.val() !== getValidFilename($nameInput.val() + FIXTURE_FILE_EXTENSION)) {
                    filenameChanged = true;
                    $filenameInput.removeClass(AUTO_INPUT_CLASS);
                }
            };

            $nameInput.keypress(function () {
                nameInputHandler();
            });

            $nameInput.keyup(function () {
                nameInputHandler();
            });

            $filenameInput.keypress(function () {
                filenameInputHandler();
            });

            $filenameInput.keyup(function () {
                filenameInputHandler();
            });

            $filenameInput.focusin(function () {
                if (!filenameChanged)
                    tmpFilename = $filenameInput.val();
            });

            $filenameInput.focusout(function () {
                var filenameValue = $filenameInput.val();

                if (!filenameChanged || !filenameValue) {
                    $filenameInput.val(tmpFilename);
                    $filenameInput.addClass(AUTO_INPUT_CLASS);
                    filenameChanged = false;
                }
                else if (!(new RegExp(FIXTURE_FILE_EXTENSION_PATTERN)).test(filenameValue))
                    $filenameInput.val((filenameValue + FIXTURE_FILE_EXTENSION));
            });
        };

        var cutNotUrlChars = function ($input) {
            if (INVALID_URL_CHARACTER_PATTERN.test($input.val()))
                $input.val($input.val().replace(INVALID_URL_CHARACTER_PATTERN, ''));
        };

        //Handle directory name input
        var dirnameInputHandler = function ($nameInput) {
            $nameInput.keypress(function (e) {
                if (Util.isNotHotKey(e))
                    cutNotUrlChars($nameInput);
            });

            $nameInput.keyup(function (e) {
                if (Util.isNotHotKey(e))
                    cutNotUrlChars($nameInput);
            });
        };

        //HTTP Basic / NTLM authentication group
        var initAuthCredentialsGroup = function (group, link) {
            var $group = $(group),
                $groupHeader = $group.find('.' + GROUP_HEADER_CLASS),
                $authCredentialsTable = $group.find('table'),
                $showCredentialsLink = $(link),
                $expandCollapseButton = $showCredentialsLink.next();

            var showAuthCredentials = function () {
                $authCredentialsTable.css('display', '');
                $expandCollapseButton.addClass(BTN_COLLAPSE_CLASS);
                $expandCollapseButton.removeClass(BTN_EXPAND_CLASS);
            };

            var hideAuthCredentials = function () {
                $authCredentialsTable.css('display', 'none');
                $expandCollapseButton.removeClass(BTN_COLLAPSE_CLASS);
                $expandCollapseButton.addClass(BTN_EXPAND_CLASS);
            };

            var onShowCredentialsClick = function () {
                if ($authCredentialsTable.is(':visible'))
                    hideAuthCredentials();
                else
                    showAuthCredentials();
            };

            //NOTE: check is group was inited before
            if (!($expandCollapseButton.hasClass(BTN_COLLAPSE_CLASS) || $expandCollapseButton.hasClass(BTN_EXPAND_CLASS)))
                $groupHeader.bind('click', onShowCredentialsClick);

            hideAuthCredentials();
        };

        //Initialization
        var initFixtureButtons = function () {
            $(FIXTURE_BUTTON).click(function () {
                var $fixtureTr = $(this).parents(BROWSER_ROW).first(),
                    $btn = $fixtureTr.find(FIXTURE_ICON),
                    fixtureUid = $fixtureTr.attr(FIXTURE_UID_ATTR);

                $btn.toggleClass(BTN_COLLAPSE_CLASS);
                $fixtureTr.toggleClass(BROWSER_ROW_EXPANDED_CLASS);

                var $fixtureInfoTr = $(FIXTURE_INFO).filter(function () {
                    return $(this).attr(FIXTURE_UID_ATTR) === fixtureUid;
                });

                $fixtureInfoTr.find(ANIM_CONTAINER).toggle('fast', function () {
                    if ($fixtureInfoTr.find(ANIM_CONTAINER).css('display') === 'none')
                        $fixtureTr.removeClass(HOVER_EMULATION_CLASS);
                });

                saveStateToCookie();
            });

            var fixtureButtonsTouchHandle = function ($elm) {
                $elm.parents(BROWSER_ROW).first().addClass(HOVER_EMULATION_CLASS);
            };

            var animationSpeed = 'fast';

            $(EXPAND_TESTS_GROUP_BTN).click(function () {
                var $groupHeader = $(this).closest('tr'),
                    $groupTests = $groupHeader.next(),
                    $groupTestsContent = $groupTests.find(TESTS_GROUP_CONTENT),
                    groupHeaderHeight = $groupHeader.height();

                //NOTE: Animation step 1: show one row and hide group header
                $groupTests.css('display', 'block');

                $groupTestsContent.css({
                    height: groupHeaderHeight,
                    overflow: 'hidden'
                });

                $groupHeader.hide();

                //NOTE: Animation step 2: expand div
                var contentHeight = $groupTestsContent[0].scrollHeight;

                $groupTestsContent.scrollTop((contentHeight - groupHeaderHeight) / 2);

                $groupTestsContent.animate({
                    height: contentHeight,
                    scrollTop: 0
                }, animationSpeed, function () {
                    $groupTestsContent.css({
                        overflow: 'auto',
                        height: 'auto'
                    });
                });

                //NOTE: Animation step 3: run window offset animate in parallel for tests list.
                $body.animate({
                    scrollTop: Util.getBodyScrollTop() + contentHeight / 2
                }, animationSpeed);

                saveStateToCookie();
            });

            $(COLLAPSE_TESTS_GROUP_BTN).click(function () {
                var $groupTests = $(this).closest('tr'),
                    $groupTestsContent = $groupTests.find(TESTS_GROUP_CONTENT),
                    $groupHeader = $groupTests.prev(),
                    groupHeaderHeight = $groupHeader.height(),
                    contentHeight = $groupTestsContent[0].scrollHeight;

                //NOTE: Animation step 1: collapse tests list and show group header
                $groupTestsContent.css('overflow', 'hidden');

                $groupTestsContent.animate({
                    height: groupHeaderHeight,
                    scrollTop: (contentHeight - groupHeaderHeight) / 2
                }, animationSpeed, function () {
                    $groupHeader.show();
                    $groupTests.hide();

                    $groupTestsContent.css('overflow', 'auto');

                    saveStateToCookie();
                });

                //NOTE: Animation step 2:  run window offset animate in parallel for tests list.
                $body.animate({
                    scrollTop: Util.getBodyScrollTop() - contentHeight / 2
                }, animationSpeed);
            });

            //NOTE: touch device support
            $(FIXTURE_BUTTON).on('touchstart', function () {
                fixtureButtonsTouchHandle($(this));
            });

            $(FIXTURE_BUTTON).on('MSPointerDown', function () {
                if (window.navigator.msMaxTouchPoints)
                    fixtureButtonsTouchHandle($(this));
            });
        };

        var restoreRunDialogState = function () {
            var cookieValue = $.cookie(RUN_DIALOG_STATE_COOKIE),
                dialogState = {};

            if (cookieValue) {
                try{
                    dialogState = JSON.parse(cookieValue);

                    if(!dialogState || typeof dialogState !== 'object' || $.isArray(dialogState))
                        dialogState = {};
                } catch(e) {
                    //NOTE: fallback if format of dialog state is incorrect
                }
            }

            for(var checkboxId in dialogState) {
                if(dialogState.hasOwnProperty(checkboxId)) {
                    var $checkbox = $('input[id="' + checkboxId + '"]');

                    if(!$checkbox.attr('disabled') && !$checkbox.attr('readonly'))
                        $checkbox.prop('checked', dialogState[checkboxId]);
                }
            }
        };

        var saveRunDialogState = function () {
            var checkedElms = {};

            $(WORKER_LIST_DIALOG).find('input[type="checkbox"]').each(function (index, value) {
                var $item = $(value),
                    id = $item.attr('id');

                checkedElms[id] = $item.prop('checked');
            });

            $.cookie(RUN_DIALOG_STATE_COOKIE, JSON.stringify(checkedElms), { expires: COOKIE_EXPIRED_DAYS});
        };

        var initRunButtons = function () {
            $(RUN_BUTTON).click(function () {
                var $btn = $(this);

                $(WORKER_LIST_DIALOG_WRAPPER).load(LIST_WORKERS_URL, function () {
                    $runDialog = $(WORKER_LIST_DIALOG).dialog({
                        width: WORKER_LIST_DIALOG_WIDTH,
                        close: function () {
                            $runDialog
                                .dialog('destroy');
                        }
                    });

                    var $workersList = $(RUN_DIALOG_WORKERS_LIST),
                        $browsersList = $(RUN_DIALOG_BROWSERS_LIST);

                    if ($workersList.length)
                        $workersList.scrollBar({
                            scrollClass: RUN_DIALOG_WORKERS_SCROLLBAR_CLASS
                        });

                    if ($browsersList.length)
                        $browsersList.scrollBar({
                            scrollClass: RUN_DIALOG_BROWSERS_SCROLLBAR_CLASS
                        });

                    restoreRunDialogState();

                    var $dialogBtn = $runDialog.find(DIALOG_RUN_BUTTON);

                    $(SELECT_ALL_BROWSERS_BTN).click(function () {
                        $(BROWSER_SELECT).prop('checked', true);
                        saveRunDialogState();
                    });

                    $(DESELECT_ALL_BROWSERS_BTN).click(function () {
                        $(BROWSER_SELECT).prop('checked', false);
                        saveRunDialogState();
                    });

                    $(SELECT_ALL_WORKERS_BTN).click(function () {
                        $(WORKER_SELECT).prop('checked', true);
                        saveRunDialogState();
                    });

                    $(DESELECT_ALL_WORKERS_BTN).click(function () {
                        $(WORKER_SELECT).prop('checked', false);
                        saveRunDialogState();
                    });

                    $runDialog.find('input[type="checkbox"]').click(function () {
                        saveRunDialogState();
                    });

                    $dialogBtn.click(function () {
                        //NOTE: encodeURI for 'source'-parameter, because it may contain dir path
                        var opt = {
                            sourceType: $btn.attr(SOURCE_TYPE_ATTR),
                            source: encodeURI($btn.attr(SOURCE_ATTR)),
                            workers: [],
                            browsers: [],
                            location: location.href,
                            takeScreenshotOnFails: $runDialog.find(TAKE_SCREENSHOT_OPTION).is(':checked') ? 'true' : 'false',
                            failOnJsErrors: $runDialog.find(FAIL_ON_JS_ERROR_OPTION).is(':checked') ? 'true' : 'false'
                        };

                        if ($btn.attr(SOURCE_TYPE_ATTR) === 'group') {
                            opt.source = [];

                            $btn.closest('tr').next().find(RUN_BUTTON).each(function () {
                                opt.source.push($(this).attr(SOURCE_ATTR));
                            });

                            opt.groupName = $btn.attr(GROUP_NAME_ATTR);
                        }

                        $runDialog.find(SELECTED_BROWSER).each(function () {
                            opt.browsers.push($(this).attr(BROWSER_NAME_ATTR));
                        });

                        $runDialog.find(SELECTED_WORKER).each(function () {
                            opt.workers.push($(this).attr(WORKER_NAME_ATTR));
                        });

                        UXLog.write('Project: run task ec:' +
                            ' br:' + opt.browsers.length +
                            ' cw:' + opt.workers.length);

                        runTask(opt);

                        $runDialog.dialog('close');

                        return false;
                    });
                });
            });
        };

        var initRecordButtons = function () {
            $(RECORD_BUTTON).click(function () {
                UXLog.write('Project: recorder started');

                createLoadingMsg();

                var $btn = $(this),
                    fixtureUid = $btn.attr(FIXTURE_UID_ATTR);

                var jqxhr = $.ajax({
                    type: 'POST',
                    url: START_RECORDING_URL,
                    data: {
                        fixtureUid: fixtureUid,
                        returnUrl: window.location.href.split('?')[0]
                    }
                });


                jqxhr.done(function (res) {
                    if (socket)
                        socket.disconnect();

                    window.location.href = res;
                });

                jqxhr.fail(function () {
                    var $statusContent = $('<span></span>').text(START_RECORDING_FAILED_MSG);

                    restoreContent();
                    $notificationContainer.showPermanentErrorMsg(jqxhr.responseText);
                    $notificationContainer.showStatus(FAIL_STATUS_CLASS, $statusContent);
                });

            });
        };

        //Dialog utils
        var createInputSelectHandler = function ($input) {
            var inputFocus = false,
                inputStartCursorPositionX = 0;

            $input.focusout(function () {
                inputFocus = false;
            });

            $input.mousedown(function (e) {
                inputStartCursorPositionX = e.clientX;
            });

            $input.mouseup(function (e) {
                if (Math.abs(e.clientX - inputStartCursorPositionX) < DIFFERENCE_BETWEEN_SELECT_AND_CLICK && !inputFocus)
                    $(this).select();

                inputFocus = true;
            });
        };

        var initEnterBtnHandler = function () {
            $(UI_DIALOG).find('input').keypress(function (e) {
                if (e.which === ENTER_BTN_CODE) {
                    $(this).closest(UI_DIALOG).find(UI_DIALOG_BTN).last().click();
                    e.stopImmediatePropagation();
                }
            });
        };

        //Init dialogs

        var initOpenExampleProject = function () {
            UXLog.write('Project: Example project opened');
            $document.on('click', EXAMPLE_PROJECT_BTN, function () {
                clearNavigationState();
                createLoadingMsg();

                $.post(OPEN_EXAMPLE_PROJECT_URL,function () {
                }).error(function (data) {
                        $notificationContainer.showPermanentErrorMsg(data.responseText);
                    });
            });
        };

        var initCloseProject = function () {
            UXLog.write('Project: Project closed');
            $(CLOSE_PROJECT_BTN).click(function () {
                $notificationContainer.clearList();
                hideTopPanel();
                clearNavigationState();
                createLoadingMsg();

                $.post(CLOSE_PROJECT_URL,function () {
                    var restore = function () {
                        clearDialogData();

                        $.cookie(PROJECT_NAVIGATION_STATE_COOKIE, '', { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });
                        $.cookie(PROJECT_PAGE_STATE_COOKIE, '', { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });
                        $notificationContainer.hide();

                        if (History.enabled) {
                            if (window.location.pathname === PROJECT_URL) {
                                slideCallback = initStartPageMsg;
                                slide(window.location.href, PROJECT_URL, true);
                            }
                            else
                                History.replaceState(null, pageTitle, PROJECT_URL);
                        }
                        else
                            window.location.href = PROJECT_URL;
                    };

                    execActionAfterAnimationTimeout(restore);
                }, 'text').error(function (data) {
                        restoreContent();
                        $notificationContainer.showPermanentErrorMsg(data.responseText);
                    });
            });
        };

        var initRenameTestDialog = function () {
            var $renameDialog = null,
                $btn = null,
                $cont = $(RENAME_TEST_DIALOG_CONT),
                $testNameInput = $(RENAME_TEST_INPUT),
                filename = '',
                oldName = '';

            $renameDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: RENAME_TEST_CLASS,
                width: FS_OPERATION_DIALOG_WIDTH
            });

            $(RENAME_TEST_BUTTON).click(function () {
                $btn = $(this);
                filename = $btn.closest(FIXTURE_INFO).parent().prev().attr(FIXTURE_FILE_ATTR);
                oldName = $btn.closest('tr').attr(TEST_NAME_ATTR);

                var title = Util.formatText(RENAME_TEST_DIALOG_TITLE_PATTERN, oldName);

                UXLog.write('Project: rename test dialog opened');

                $cont.parent().find(UI_DIALOG_TITLE).html(title);
                $renameDialog.dialog('open');
                Util.removeInputMsgs($cont);
                $cont.find('input').val(oldName).select();
            });

            $(RENAME_TEST_BTN).click(function (event) {
                var newName = $testNameInput.val();

                Util.removeInputMsgs($cont);

                if (!$.trim(newName) || newName === oldName) {
                    if (!$.trim(newName))
                        Util.showInputMsg($testNameInput, EMPTY_NAME_ERR_MSG);

                    if (newName === oldName) {
                        $cont.find('input').focusout();
                        $renameDialog.dialog('close');
                    }
                } else {
                    $cont.find('input').focusout();
                    $renameDialog.dialog('close');

                    renameTest(filename, oldName, newName, function (renameErr) {
                        if (!renameErr)
                            $cont.find('input').val('');
                        else
                            $renameDialog.dialog('open');
                        Util.showInputMsg($testNameInput, renameErr);
                    });
                }

                event.stopImmediatePropagation();
            });
        };

        var initDeleteTestDialog = function () {
            var $deleteDialog = null,
                $btn = null,
                filename = '',
                testName = '',
                $cont = $(DELETE_TEST_DIALOG_CONT);

            $deleteDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: DELETE_TEST_CLASS,
                width: FS_OPERATION_DIALOG_WIDTH
            });

            $(DELETE_TEST_BUTTON).click(function () {
                $btn = $(this);
                filename = $btn.closest(FIXTURE_INFO).parent().prev().attr(FIXTURE_FILE_ATTR);
                testName = $btn.closest('tr').attr(TEST_NAME_ATTR);

                UXLog.write('Project: delete test dialog opened');
                $cont.find(DIALOG_TEXT).text(Util.formatText(DELETE_DIALOG_TEXT_PATTERN, testName));
                $deleteDialog.dialog('open');
            });

            $(DELETE_TEST_YES_BTN).click(function () {
                deleteTest(filename, testName);
                $deleteDialog.dialog('close');
            });

            $(DELETE_TEST_NO_BTN).click(function () {
                $deleteDialog.dialog('close');
            });
        };

        var initAddFixtureDialog = function () {
            var $cont = $(ADD_FIXTURE_DIALOG_CONT),
                $nameInput = $(ADD_FIXTURE_INPUT_NAME),
                $filenameInput = $(ADD_FIXTURE_INPUT_FILENAME),
                $pageInput = $(ADD_FIXTURE_INPUT_PAGE),
                $usernameInput = $(ADD_FIXTURE_AUTH_USERNAME_INPUT),
                $passwordInput = $(ADD_FIXTURE_AUTH_PASSWORD_INPUT);

            $(ADD_FIXTURE_BTN).click(function () {
                initAuthCredentialsGroup(ADD_FIXTURE_AUTH_CREDENTIALS_GROUP, ADD_FIXTURE_SHOW_CREDENTIALS_LINK);
                $addFixtureDialog.dialog('open');
                $cont.find('input').val('');
                Util.removeInputMsgs($cont);
                filenameInputHandler($nameInput, $filenameInput);
            });

            $document.on('click', ADD_FIXTURE_OK_BTN, function (event) {
                var fixtureName = $nameInput.val(),
                    fixtureFilename = $filenameInput.val(),
                    fixturePage = $pageInput.val(),
                    username = $usernameInput.val(),
                    password = $passwordInput.val();

                Util.removeInputMsgs($cont);

                var fixtureAlreadyExists = getFixtureRow(fixtureName).length;

                if (!$.trim(fixtureName) || fixtureAlreadyExists || !$.trim(fixturePage) || !$.trim(fixtureFilename) || (!$.trim(username) && $.trim(password))) {
                    if (!$.trim(fixtureName))
                        Util.showInputMsg($nameInput, EMPTY_NAME_ERR_MSG);

                    if (fixtureAlreadyExists)
                        Util.showInputMsg($nameInput, Util.formatText(FIXTURE_ALREADY_EXISTS_ERR_MSG_PATTERN, fixtureName));

                    if (!$.trim(fixturePage))
                        Util.showInputMsg($pageInput, EMPTY_PAGE_ERR_MSG);

                    if (!$.trim(fixtureFilename))
                        Util.showInputMsg($filenameInput, EMPTY_FILENAME_ERR_MSG);

                    if (!$.trim(username) && $.trim(password))
                        Util.showInputMsg($usernameInput, EMPTY_AUTH_USERNAME_ERR_MSG);
                } else {
                    $addFixtureDialog.dialog('close');

                    addFixture(fixtureName, fixtureFilename, $.trim(fixturePage), $.trim(username), password, function (addFixtureErr) {
                        UXLog.write('Project: New fixture created via "New fixture" dialog');

                        if (!addFixtureErr)
                            $cont.find('input').val('');
                        else {
                            $addFixtureDialog.dialog('open');
                            Util.showInputMsg($filenameInput, addFixtureErr);
                        }
                    });
                }

                event.stopImmediatePropagation();
            });
        };

        var initAddDirDialog = function () {
            var $cont = $(ADD_DIR_CONT),
                $dirNameInput = $(ADD_DIR_INPUT_DIR_NAME);

            $(ADD_DIR_BTN).click(function () {
                $addDialog.dialog('open');
                $cont.find('input').val('');
                Util.removeInputMsgs($cont);
            });

            dirnameInputHandler($dirNameInput);

            $(ADD_DIR_OK_BTN).click(function (event) {
                cutNotUrlChars($dirNameInput);

                var dirName = $dirNameInput.val();

                Util.removeInputMsgs($cont);

                if (!$.trim(dirName))
                    Util.showInputMsg($dirNameInput, EMPTY_DIR_NAME_ERR_MSG);
                else {
                    $addDialog.dialog('close');

                    addDir(dirName, function (addDirErr) {
                        if (!addDirErr)
                            $cont.find('input').val('');
                        else {
                            $addDialog.dialog('open');
                            Util.showInputMsg($dirNameInput, addDirErr);
                        }
                    });
                }

                event.stopImmediatePropagation();
            });
        };

        var initCreateProjectDialog = function () {
            var $cont = $(CREATE_PROJECT_DIALOG_CONT),
                $fsTreeView = $(CREATE_PROJECT_FS_TREEVIEW_CONT),
                $fsTreeViewCont = $fsTreeView.parent(),
                $inputPath = $(PROJECT_PATH_INPUT),
                $inputName = $(PROJECT_NAME_INPUT),
                projectName = '',
                cancelCreating = function () {
                    $createProjectDialog.dialog('close');
                    $inputName.val('');
                    Util.removeInputMsgs($cont);
                },
                redrawScrollbar = function () {
                    $fsTreeViewCont.scrollBar({
                        scrollTo: CREATE_PROJECT_FS_TREEVIEW_CONT + ' li.' + TREEVIEW_SELECTED_ITEM_CLASS,
                        scrollClass: CREATE_PROJECT_SCROLLBAR_CLASS
                    });
                },
                $createProjectDialog = $cont.dialog({
                    autoOpen: false,
                    dialogClass: CREATE_PROJECT_CLASS,
                    width: 'auto',
                    close: cancelCreating
                }),
                treeviewOptions = {
                    url: GET_DIR_BY_PATH_URL,
                    open: currentProject,
                    effect: 'animate',
                    time: FS_EXPAND_DURATION,
                    asyncAction: redrawScrollbar
                };

            dirnameInputHandler($inputName);

            $document.on('click', CREATE_PROJECT_BTN, function () {
                UXLog.write('Project: "Create project" button clicked');
                $fsTreeView.html('');
                treeviewOptions.open = currentProject;
                $fsTreeView.treeView(treeviewOptions);
                $createProjectDialog.dialog('open');

                $cont.parent().find(UI_DIALOG_TITLE_CONT).draggable();
                redrawScrollbar();

                $fsTreeViewCont.resizable({
                    minWidth: CREATE_PROJECT_TREEVIEW_WIDTH,
                    minHeight: CREATE_PROJECT_DIALOG_HEIGHT,
                    class: RESIZE_MARKER_CREATE_PROJECT_CLASS,
                    action: function () {
                        $fsTreeViewCont.scrollBar({ scrollClass: CREATE_PROJECT_SCROLLBAR_CLASS });
                    }
                });

                var selectedDir = $fsTreeViewCont.find(' li.' + TREEVIEW_SELECTED_ITEM_CLASS).attr(INPUT_PATH_ATTR);

                selectedDir = selectedDir || currentProject;
                $inputPath.val(decodeURIComponent(selectedDir));
            });

            createInputSelectHandler($inputPath);

            $inputPath.keyup(function (e) {
                treeviewOptions.open = $inputPath.val();
                treeviewOptions.effect = false;
                $fsTreeView.treeView(treeviewOptions);
                treeviewOptions.effect = 'animate';

                if (e.keyCode !== ENTER_BTN_CODE)
                    Util.removeInputMsgs($cont);
            });

            $(CREATE_PROJECT_OK_BTN).click(function () {
                var projectPath = $inputPath.val(),
                    projectName = $inputName.val();

                Util.removeInputMsgs($cont);

                if (!$.trim(projectPath) || !$.trim(projectName)) {
                    if (!$.trim(projectName))
                        Util.showInputMsg($inputName, EMPTY_NAME_ERR_MSG);

                    if (!$.trim(projectPath))
                        Util.showInputMsg($inputPath, EMPTY_NAME_ERR_MSG);

                    return;
                }

                if ($.trim(projectPath))
                    $inputPath.val(decodeURIComponent(projectPath));

                Util.removeInputMsgs($cont);

                createProject(projectPath, projectName, $createProjectDialog, function (err) {
                    UXLog.write('Project: project created');

                    if (!err) {
                        Util.removeInputMsgs($cont);
                        $createProjectDialog.dialog('close');
                        currentProject = Util.pathJoin(projectPath, projectName);
                        treeviewOptions.open = currentProject;
                    }
                    else
                        Util.showInputMsg($inputName, err);
                });
            });

            $(CREATE_PROJECT_CANCEL_BTN).click(cancelCreating);

            $document.on('click', CREATE_PROJECT_FS_TREEVIEW_CONT + ' li', function () {
                projectName = $(this).attr(INPUT_PATH_ATTR);
                $inputPath.val(decodeURIComponent(projectName));
            });

            $document.on('click', [CREATE_PROJECT_DIALOG_CONT, HOME_DIR_BTN].join(' '), function() {
                $inputPath.val(window.HOME_DIR);
                $inputPath.keyup();
            });
        };

        var initOpenProjectDialog = function () {
            var $cont = $(OPEN_PROJECT_CONT),
                $dirsTreeView = $(DIRS_TREEVIEW_CONT),
                $dirsTreeViewCont = $dirsTreeView.parent(),
                $inputDirPath = $(OPEN_PROJECT_PATH_INPUT),

                $openProjectDialog = $cont.dialog({
                    autoOpen: false,
                    dialogClass: OPEN_PROJECT_CLASS,
                    width: 'auto'
                }),
                loadItems = function () {
                    $dirsTreeViewCont.scrollBar({
                        scrollTo: DIRS_TREEVIEW_CONT + ' li.' + TREEVIEW_SELECTED_ITEM_CLASS,
                        scrollClass: OPEN_PROJECT_SCROLLBAR_CLASS
                    });
                },
                setInputValue = function ($el) {
                    $inputDirPath.val(decodeURIComponent($el.attr(INPUT_PATH_ATTR)));
                },
                treeviewOptions = {
                    url: GET_DIR_BY_PATH_URL,
                    open: currentProject,
                    effect: 'animate',
                    time: FS_EXPAND_DURATION,
                    asyncAction: loadItems,
                    selectAction: setInputValue
                };

            $document.on('click', [OPEN_PROJECT_CONT, HOME_DIR_BTN].join(' '), function() {
                $inputDirPath.val(window.HOME_DIR);
                $inputDirPath.keyup();
            });

            $document.on('click', OPEN_PROJECT_BTN, function () {
                UXLog.write('Project: "Open project" button clicked');
                $dirsTreeView.html('');
                treeviewOptions.open = currentProject;
                $dirsTreeView.treeView(treeviewOptions);
                $openProjectDialog.dialog('open');

                $cont.parent().find(UI_DIALOG_TITLE_CONT).draggable();

                $dirsTreeViewCont.resizable({
                    minWidth: OPEN_PROJECT_TREEVIEW_WIDTH,
                    minHeight: OPEN_PROJECT_DIALOG_HEIGHT,
                    resizeMarkerClass: RESIZE_MARKER_OPEN_PROJECT_DIALOG_CLASS,
                    action: function () {
                        $dirsTreeViewCont.scrollBar({
                            scrollClass: OPEN_PROJECT_SCROLLBAR_CLASS
                        });
                    }
                });

                var selectedDir = $(DIRS_TREEVIEW_CONT + ' li.' + TREEVIEW_SELECTED_ITEM_CLASS).attr(INPUT_PATH_ATTR);

                selectedDir = selectedDir || currentProject;
                $inputDirPath.val(decodeURIComponent(selectedDir));
            });

            createInputSelectHandler($inputDirPath);

            $inputDirPath.keyup(function (e) {
                treeviewOptions.open = $inputDirPath.val();
                treeviewOptions.effect = false;
                $dirsTreeView.treeView(treeviewOptions);
                treeviewOptions.effect = 'animate';

                if (e.keyCode !== ENTER_BTN_CODE)
                    Util.removeInputMsgs($cont);
            });

            $(OPEN_PROJECT_OK_BTN).click(function () {
                var inputValue = $inputDirPath.val();

                if (!$.trim(inputValue))
                    $inputDirPath.val(decodeURIComponent(currentProject));

                Util.removeInputMsgs($cont);

                if(PROJECT_URL !== location.pathname)
                    stopObservationForRevision = true;

                openProject(inputValue, $openProjectDialog, function (err) {
                    UXLog.write('Project: project opened');

                    if (!err) {
                        if(History.enabled) {
                            History.replaceState(null, pageTitle, PROJECT_URL);
                            stopObservationForRevision = false;
                            currentProject = inputValue;
                            treeviewOptions.open = currentProject;
                        }
                        else
                            location.href = [location.protocol, '//', location.host, PROJECT_URL].join('');

                        Util.removeInputMsgs($cont);
                        $notificationContainer.show();
                    } else {
                        Util.showInputMsg($inputDirPath, err);
                        stopObservationForRevision = false;
                    }

                    showTopPanel();
                });
            });

            $(OPEN_PROJECT_CANCEL_BTN).click(function () {
                $openProjectDialog.dialog('close');
            });
        };

        var initRenameDialog = function () {
            var $renameDialog = null,
                $btn = null,
                $cont = $(RENAME_DIR_DIALOG_CONT),
                $dirNameInput = $(RENAME_DIR_INPUT),
                oldName = '';

            $renameDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: RENAME_DIR_CLASS,
                width: FS_OPERATION_DIALOG_WIDTH
            });

            dirnameInputHandler($dirNameInput);

            $(RENAME_DIR_BUTTON).click(function () {
                $btn = $(this);
                oldName = $btn.closest(BROWSER_ROW).attr(DIR_NAME_ATTR);

                var title = Util.formatText(RENAME_DIR_DIALOG_TITLE_PATTERN, oldName);

                $cont.parent().find(UI_DIALOG_TITLE).html(title);
                $renameDialog.dialog('open');
                Util.removeInputMsgs($cont);
                $cont.find('input').val(oldName).select();
            });

            $(RENAME_DIR_BTN).click(function (event) {
                cutNotUrlChars($dirNameInput);

                var dirName = $dirNameInput.val();

                Util.removeInputMsgs($cont);

                if (!$.trim(dirName) || dirName === oldName) {
                    if (!$.trim(dirName))
                        Util.showInputMsg($dirNameInput, EMPTY_NAME_ERR_MSG);

                    if (dirName === oldName) {
                        //NOTE: Ipad bug: does not hide the virtual keyboard if there is no focusout event for the input element.
                        $cont.find('input').focusout();
                        $renameDialog.dialog('close');
                    }
                } else {
                    $cont.find('input').focusout();
                    $renameDialog.dialog('close');

                    renameDir(oldName, dirName, function (renameErr) {
                        if (!renameErr)
                            $cont.find('input').val('');
                        else {
                            $renameDialog.dialog('open');
                            Util.showInputMsg($dirNameInput, renameErr);
                        }
                    });
                }

                event.stopImmediatePropagation();
            });
        };

        var initDeleteDialog = function () {
            var $deleteDialog = null,
                $btn = null,
                dirName = '',
                $cont = $(DELETE_DIR_DIALOG_CONT);

            $deleteDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: DELETE_DIR_CLASS,
                width: FS_OPERATION_DIALOG_WIDTH
            });

            $(DELETE_DIR_BUTTON).click(function () {
                $btn = $(this);
                dirName = $btn.closest(BROWSER_ROW).attr(DIR_NAME_ATTR);

                $cont.find(DIALOG_TEXT).text(Util.formatText(DELETE_DIALOG_TEXT_PATTERN, dirName));
                $deleteDialog.dialog('open');
            });

            $(DELETE_DIR_YES_BTN).click(function () {
                deleteDir(dirName);
                $deleteDialog.dialog('close');
            });

            $(DELETE_DIR_NO_BTN).click(function () {
                $deleteDialog.dialog('close');
            });
        };

        var initEditFixtureDialog = function () {
            var $editFixtureDialog = null,
                $btn = null,
                $cont = $(EDIT_FIXTURE_DIALOG_CONT),
                $fixtureNameInput = $(EDIT_FIXTURE_NAME_INPUT),
                $filenameInput = $(EDIT_FILENAME_INPUT),
                $fixturePageInput = $(EDIT_FIXTURE_PAGE_INPUT),
                $fixtureUsernameInput = $(EDIT_FIXTURE_AUTH_USERNAME_INPUT),
                $fixturePasswordInput = $(EDIT_FIXTURE_AUTH_PASSWORD_INPUT),
                oldName = '',
                oldWebPage,
                oldFilename = '',
                oldUsername = '',
                oldPassword = '';

            $editFixtureDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: EDIT_FIXTURE_CLASS,
                width: FS_OPERATION_DIALOG_WIDTH
            });

            $(EDIT_FIXTURE_BUTTON).click(function () {
                $btn = $(this);

                UXLog.write('Project: edit fixture dialog opened');
                var $browserRow = $btn.closest(BROWSER_ROW);

                oldName = $browserRow.attr(FIXTURE_NAME_ATTR);
                oldFilename = $browserRow.attr(FIXTURE_FILE_ATTR);
                oldWebPage = $browserRow.attr(FIXTURE_PAGE_ATTR);
                oldUsername = $browserRow.attr(FIXTURE_USERNAME_ATTR);
                oldPassword = $browserRow.attr(FIXTURE_PASSWORD_ATTR);

                var title = Util.formatText(EDIT_FIXTURE_DIALOG_TITLE_PATTERN, oldName);

                $cont.parent().find(UI_DIALOG_TITLE).html(title);
                $fixtureNameInput.val(oldName);
                $filenameInput.val(oldFilename);
                $fixturePageInput.val(oldWebPage);
                $fixtureUsernameInput.val(oldUsername);
                $fixturePasswordInput.val(oldPassword);

                initAuthCredentialsGroup(EDIT_FIXTURE_AUTH_CREDENTIALS_GROUP, EDIT_FIXTURE_SHOW_CREDENTIALS_LINK);

                $editFixtureDialog.dialog('open');

                Util.removeInputMsgs($cont);
                filenameInputHandler($fixtureNameInput, $filenameInput);
            });

            $(EDIT_FIXTURE_OK_BTN).click(function (event) {
                var fixtureName = $fixtureNameInput.val(),
                    fixturePage = $fixturePageInput.val(),
                    filename = $filenameInput.val(),
                    username = $fixtureUsernameInput.val(),
                    password = $fixturePasswordInput.val();

                Util.removeInputMsgs($cont);

                if (!$.trim(fixtureName) || !$.trim(fixturePage) || (!$.trim(username) && $.trim(password))) {
                    if (!$.trim(fixtureName))
                        Util.showInputMsg($fixtureNameInput, EMPTY_NAME_ERR_MSG);

                    if (!$.trim(fixturePage))
                        Util.showInputMsg($fixturePageInput, EMPTY_PAGE_ERR_MSG);

                    if (!$.trim(username) && $.trim(password))
                        Util.showInputMsg($fixtureUsernameInput, EMPTY_AUTH_USERNAME_ERR_MSG);
                } else if (oldName === fixtureName && oldFilename === filename && $.trim(oldWebPage) === $.trim(fixturePage) && username === oldUsername && password === oldPassword) {
                    $cont.find('input').val('').focusout();
                    $editFixtureDialog.dialog('close');
                } else {
                    $cont.find('input').focusout();
                    $editFixtureDialog.dialog('close');

                    editFixture(oldFilename, filename, oldName, fixtureName, oldWebPage, $.trim(fixturePage), $.trim(username), password, function (editErr) {
                        if (!editErr)
                            $cont.find('input').val('');
                        else {
                            $editFixtureDialog.dialog('open');
                            Util.showInputMsg($filenameInput, editErr);
                        }
                    });
                }

                event.stopImmediatePropagation();
            });
        };

        var initDeleteFixtureDialog = function () {
            var $deleteFixtureDialog = null,
                $btn = null,
                fixtureName = '',
                fixtureFilename = '',
                $cont = $(DELETE_FIXTURE_DIALOG_CONT);

            $deleteFixtureDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: DELETE_FIXTURE_CLASS,
                width: FS_OPERATION_DIALOG_WIDTH
            });

            $(DELETE_FIXTURE_BUTTON).click(function () {
                $btn = $(this);

                UXLog.write('Project: delete fixture dialog opened');
                fixtureFilename = $btn.closest(BROWSER_ROW).attr(FIXTURE_FILE_ATTR);
                fixtureName = $btn.closest(BROWSER_ROW).attr(FIXTURE_NAME_ATTR);

                $cont.find(DIALOG_TEXT).text(Util.formatText(DELETE_DIALOG_TEXT_PATTERN, fixtureName));
                $deleteFixtureDialog.dialog('open');
            });

            $(DELETE_FIXTURE_YES_BTN).click(function () {
                deleteFixture(fixtureFilename, fixtureName);
                $deleteFixtureDialog.dialog('close');
            });

            $(DELETE_FIXTURE_NO_BTN).click(function () {
                $deleteFixtureDialog.dialog('close');
            });
        };

        var initCodeEditorDialog = function() {
            FixtureEditor.init({
                openBtn: EDIT_FIXTURE_CODE_BTN,
                onSaveClick: function() {
                    fsOperationFlag = true;
                    saveState();
                    createLoadingMsg();
                },
                onSaveSuccess: function(fixtureFile) {
                    fsOperation = {text: Util.formatText(SAVE_FIXTURE_CODE_SUCCESS_MSG_PATTERN, fixtureFile), status: 'success'};
                },
                onSaveFail: function(data) {
                    fsOperationFlag = false;
                    restoreContent();
                    $notificationContainer.showPermanentErrorMsg(data.responseText);
                },
                onOpenFail: function(data) {
                    $notificationContainer.showPermanentErrorMsg(data.responseText);
                },
                getCurrentTestName: function($btn) {
                    return $btn.closest('tr').attr(TEST_NAME_ATTR) || '';
                },
                getFileName: function($btn) {
                    var curTestName = $btn.closest('tr').attr(TEST_NAME_ATTR) || '';

                    return curTestName ?
                        $btn.closest(FIXTURE_INFO).attr(FIXTURE_FILE_ATTR) :
                        $btn.closest(BROWSER_ROW).attr(FIXTURE_FILE_ATTR);
                },
                getCurrentPath: function() {
                    return Util.getCurPath();
                }
            });
        };

        var initHyperlinks = function () {
            $(BROWSER).find(NAV_LINK).click(function () {
                var href = decodeURI($(this).attr('href'));

                $.cookie(PROJECT_NAVIGATION_STATE_COOKIE, href, { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });

                if (History.enabled)
                    History.pushState(null, pageTitle, href);
                else
                    window.location.href = href;

                return false;
            });
        };

        var initNavButtons = function () {
            $(BROWSER).find(NAV_BUTTON).click(function () {
                var $navLink = $(this).parents(BROWSER_ROW).first().find(NAV_LINK);
                var href     = decodeURI($navLink.attr('href'));

                $.cookie(PROJECT_NAVIGATION_STATE_COOKIE, href, { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });

                if (History.enabled) {
                    History.pushState(null, pageTitle, href);
                }
                else
                    window.location.href = href;

                return false;
            });
        };

        var initStatusContainer = function () {
            $notificationContainer = NotificationsList({
                selector: MSG_CONTAINER,
                topOffset: $(HEADER).height() + $(TOP_BUTTONS).height(),
                leftOffset: ($('body').width() + $(PROJECT_VIEW).width()) / 2,
                bottomOffset: $(FOOTER).height()
            });
        };

        var initContent = function () {
            initNoTestsMsg();

            //Buttons
            initFixtureButtons();
            initRunButtons();
            initRecordButtons();
            initNavButtons();
            //Dialogs
            initRenameDialog();
            initDeleteDialog();
            initEditFixtureDialog();
            initDeleteFixtureDialog();
            initAddFixtureDialog();
            initAddDirDialog();
            initRenameTestDialog();
            initDeleteTestDialog();

            initEnterBtnHandler();
            initHyperlinks();

            setStartPageOffset();
        };

        var showTopPanel = function () {
            var $topBtns = $(TOP_BUTTONS);

            $topBtns.show();
            $topBtns.prev().show();
        };

        var hideTopPanel = function () {
            var $topBtns = $(TOP_BUTTONS);

            $topBtns.hide();
            $topBtns.prev().hide();
        };

        var initRevisionWatcher = function () {
            var socketDisconnected = false;

            socket = io.connect(window.location.protocol + '//' + window.location.host);

            var suiteChanged = function () {
                if(stopObservationForRevision)
                    return;

                if (!fsOperationFlag)
                    saveState();

                if ($runDialog && $runDialog.dialog)
                    $runDialog.dialog('close');

                //NOTE: Fix the current path for the opera
                $.get(window.location.pathname + REBUILD_REVISION_PARAM,function (data) {
                    clearDialogData();

                    $(REVISION_CONTAINER).html(data);
                    showTopPanel();
                    restoreState(true);

                    if (!$(FIXTURE_TOP_BTNS).is(':visible'))
                        $(FIXTURE_TOP_BTNS).show();

                    $(REVISION_CONTAINER).effect('pulsate', {times: 1}, 'slow', function () {
                        var $statusContent = $('<span></span>'),
                            statusClassName = null;

                        if (fsOperationFlag) {
                            statusClassName = fsOperation.status === 'success' ? SUCCESS_STATUS_CLASS : FAIL_STATUS_CLASS;
                            $statusContent.html(fsOperation.text);
                            fsOperationFlag = false;
                        } else {
                            $statusContent.html(VFS_REBUILDED_MSG);
                            statusClassName = UPDATE_STATUS_CLASS;
                        }

                        $notificationContainer.showStatus(statusClassName, $statusContent);
                    });

                    //NOTE reload content script
                    initBuildErrs();
                    initContent();
                    $notificationContainer.updateTopOffset();
                    $notificationContainer.updatePosition();
                }).error(function () {
                        slideToUpperLevel();
                    });
            };

            var updateRevision = function () {
                if (isStartPage())
                    return;

                execActionAfterAnimationTimeout(suiteChanged);
            };

            socket.on('vfsRevisionChanged', function () {
                updateRevision();
            });

            socket.on('vfsRevisionStartUpdate', function () {
                createLoadingMsg();
            });

            socket.on('vfsProjectRemoved', function () {
                location.href = [window.location.protocol, '//', window.location.host, PROJECT_URL].join('');
            });

            //NOTE: show update message for reconnect
            socket.on('connect', function () {
                if (socketDisconnected) {
                    updateRevision();
                    socketDisconnected = false;
                }
            });

            socket.on('taskComplete', function (data) {
                $notificationContainer.showTaskCompleteMsg(data);
            });

            socket.on('disconnect', function () {
                socketDisconnected = true;
            });
        };

        var initBuildErrs = function () {
            var $buildErrs = $(BUILD_ERRS),
                $title = $buildErrs.find(BUILD_ERRS_TITLE);

            $title.click(function () {
                $buildErrs.find(BUILD_ERRS_EXP_COL_BUTTON).toggleClass(BTN_COLLAPSE_CLASS);
                $buildErrs.find(BUILD_ERRS_LIST).slideToggle('fast', function () {
                    saveStateToCookie();
                    $notificationContainer.updateTopOffset();
                });
            });

            $(FIXTURE_FILENAME).click(function () {
                var fixturePath = $(this).attr(FIXTURE_FILE_ATTR).split('\\'),
                    fileName = fixturePath.pop(),
                    to = '/project' + fixturePath.join('/');

                slideCallback = function () {
                    expandedBrowserRow(fileName, true, true, function() {
                        saveStateToCookie();
                        slideCallback = null;
                    });
                };

                if (to !== location.pathname) {
                    if (History.enabled)
                        History.pushState(null, pageTitle, to);
                    else
                        location.href = to + EXPAND_ROW_PARAM + fileName;
                } else
                    slideCallback();
            });
        };

        var initHistorySlider = function () {
            if (!History.enabled)
                return;

            var from = window.location.href;

            History.Adapter.bind(window, 'statechange', function () {
                var to = History.getState().url;

                slide(from, to);

                from = to;
            });
        };

        var initPageEffect = function () {
            if (History.enabled)
                History.replaceState(null, pageTitle, window.location.href.split('?')[0]);

            $(REVISION_CONTAINER).fadeIn('slow', function () {
                //NOTE: Expand row mechanism for browsers, that not supported history API.
                if (window.EXPAND_ROW) {
                    expandedBrowserRow(window.EXPAND_ROW, true, true, function () {
                        saveStateToCookie();
                        window.EXPAND_ROW = '';
                    });
                }

                if(window.OPEN_TEST_TARGET) {
                    openTestTarget(window.OPEN_TEST_TARGET, window.OPEN_TEST_NAME, function($row) {
                        Util.scrollTo($row);
                        saveStateToCookie();

                        window.OPEN_TEST_TARGET = '';
                        window.OPEN_TEST_NAME= '';
                    });
                }

                var completedTask = window.COMPLETED_TASK;

                if (completedTask) {
                    $notificationContainer.showTaskCompleteMsg(completedTask);
                    window.COMPLETED_TASK = '';
                }
            });
        };

        var initNoTestsMsg = function () {
            var $noTests = $(NO_TESTS);

            if ($noTests) {
                setInCenter($noTests);
                $noTests.css('visibility', 'visible');
            }

            $window.resize(function () {
                if ($noTests)
                    setInCenter($noTests);
            });
        };

        var initNavigationState = function () {
            var $projectNavigationState = $.cookie(PROJECT_NAVIGATION_STATE_COOKIE),
                $container = $(BROWSER).parent();

            if (!!$projectNavigationState && $projectNavigationState !== window.location.pathname && window.location.pathname === PROJECT_URL) {
                if (History.enabled)
                    History.replaceState(null, pageTitle, $projectNavigationState);

                $.get([$projectNavigationState, SLIDE_QUERY_PARAM].join(''),function (res) {
                    if (History.enabled) {
                        clearDialogData();
                        $container.html(res);
                        initContent();
                        initPageStateFromCookie();
                    } else
                        window.location.href = $projectNavigationState;
                }).error(function () {
                        if (History.enabled)
                            slideToUpperLevel();
                    });
            }
            else {
                $.cookie(PROJECT_NAVIGATION_STATE_COOKIE, window.location.pathname, { expires: COOKIE_EXPIRED_DAYS, path: PROJECT_URL });
                initPageStateFromCookie();
            }
        };

        var initStartPageMsg = function () {
            setStartPageOffset();

            $window.scroll(function (e) {
                setStartPageOffset();
                e.stopImmediatePropagation();
            });

            $window.resize(function (e) {
                setStartPageOffset();
                e.stopImmediatePropagation();
            });
        };

        var initHorizontalScrollHandler = function () {
            $(window).scroll(function () {
                var scrollLeft = $(this).scrollLeft();

                $(HEADER_BACK).css('left', -scrollLeft);
                $(FIXTURE_TOP_BTNS).css('left', -scrollLeft);
                $(FOOTER).css('left', -scrollLeft);
            });
        };

        $document.ready(function () {
            initHorizontalScrollHandler();

            createAddFixtureDialog();
            createDirDialog();
            initOpenProjectDialog();
            initCreateProjectDialog();
            initOpenExampleProject();
            initCloseProject();
            initCodeEditorDialog();

            initNavigationState();
            initPageEffect();
            initHistorySlider();
            initBuildErrs();
            initRevisionWatcher();
            initStatusContainer();
            initStartPageMsg();
            initContent();
        });
    };
});