/* global History: true */
/* global io: true */
/* global ace: true */

ControlPanel.define('ResultsView', function (require, exports) {
    require('Widgets.Dialog');
    require('Widgets.Draggable');
    require('Widgets.Resizable');
    require('Widgets.ScrollBar');

    var $ = require('jQuery'),
        FixtureEditor = require('Widgets.FixtureEditor'),
        Util = require('Util'),
        UXLog = require('UXLog');

    exports.init = function () {

        //Constants
        var
        //Selectors
            ALERT_DIALOG = '#alert-dialog',
            ALERT_DIALOG_OK_BTN = '#alert-dialog-ok-btn',
            CLEAR_REPORTS_DIALOG_CONT = '#clear-reports-dialog',
            CLEAR_REPORTS_NO_BTN = '#clear-reports-no-btn',
            CLEAR_REPORTS_YES_BTN = '#clear-reports-yes-btn',
            CLEAR_RESULTS = '#clear-reports',
            CONTENT = '.content',
            DETAIL_LINK = '.detail-link',
            DETAIL_VIEW = '.task-detail',
            DIALOG_TEXT = '.dialog-text',
            DIALOG_HEADER = '.dialog-text h2',
            ERROR_ROW = '.error-row',
            EXPORT_REPORT = '.export-report-btn',
            FILE_DOWNLOADING = '.file-downloading',
            FIXTURE_NAME = '.fixture-name',
            FOOTER = '.footer',
            GO_TO_CODE_BTN = '.go-to-code-btn',
            GRID_CONTAINER = '#gridContainer',
            GRID_ROW = 'tr.dx-row',
            HEADER_BACK = '.header .back',
            HIGHLIGHT_CODE = '.language-javascript',
            NAV_LINK = 'a.nav-link',
            NO_MSG = '.no-msg',
            NO_TASKS_RESULTS = '.no-tasks-results',
            REMOVE_REPORT_BTN = '.remove-report-btn',
            REMOVE_REPORT_DIALOG_CONT = '#remove-report-dialog',
            REMOVE_REPORT_NO_BTN = '#remove-report-no-btn',
            REMOVE_REPORT_YES_BTN = '#remove-report-yes-btn',
            REPORT_HEADER = '.report-head',
            RESTART_TASK_BTN = '.restart-task',
            RESTART_TASK_LINKS = '.restart-task li',
            SCREENSHOTS_GALLERY = '#screenshots-gallery',
            SCREENSHOTS_GALLERY_ALL_BTN = '.screenshot-cell',
            SCREENSHOTS_LIST = '.screenshots-list',
            TASK_ITEM = '.task-results-item',
            TASKS_RESULTS_CONTAINER = '#tasks-results-container',
            TEST_NAME = '.test-name',
            TOP_BUTTONS = '.results-top-buttons',
            TOP_BUTTONS_CONT = '.results-top-buttons-cont',
            UI_DIALOG_TITLE = '.dialog-header span',
            UI_DIALOG_TITLE_CONT = '.dialog-header',

        //Classes
            ALERT_DIALOG_CLASS = 'alert-dialog',
            CLEAR_REPORTS_CLASS = 'clear-reports',
            FAILED_VIEW_CLASS = 'failed-view',
            FILE_DOWNLOADING_CLASS = 'file-downloading',
            FIXTURE_NAME_CLASS = 'fixture-name',
            HIDDEN_CLASS = 'hidden',
            LOADING_MSG_CLASS = 'loading-img',
            REMOVE_REPORT_CLASS = 'remove-report',
            RESIZE_SCREENSHOT_GALLERY_CLASS = 'resize-screenshot-gallery',
            SCREENSHOTS_GALLERY_CLASS = 'screenshots-gallery',
            SCREENSHOTS_GALLERY_SCROLLBAR_CLASS = 'screenshots-gallery-scrollbar',
            STEP_NAME_CLASS = 'step-name',
            SUCCESS_VIEW_CLASS = 'succeeded-view',
            TEST_NAME_CLASS = 'test-name',

        //Urls
            ACE_LIB_URL = '/ace',
            CLEAR_REPORTS_URL = '/clear_results/',
            DETAIL_QUERY_PARAM = '?detail=1',
            IDLE_WORKER_URL = '/worker/idle/',
            REMOVE_REPORT_URL = '/remove_report/',
            RETURN_URL_QUERY_PARAM = '?returnUrl=',
            REVISION_REQ_URL = '/results_rev/',
            RUN_TASK_URL = '/tests_run/',
            SLIDE_QUERY_PARAM = '?partial=1',
            TASKS_RESULTS_URL = '/results/',
            GET_REPORT_JSON_URL = [REVISION_REQ_URL, '{0}/?json=1'].join(''),
            PROJECT_URL = [location.protocol, '//', location.host, '/project'].join(''),
            RESULTS_URL = [location.protocol, '//', location.host, '/results/'].join(''),
            GET_SCREEN_URL = '/get_screen/?path=',
            TO_SCREEN_URL_PATTERN = '/to_screen/?path={0}&title={1}',
            GET_SCREEN_THUMBNAIL_URL_PATTERN = GET_SCREEN_URL + '{0}&thumbnail=1',

        //Attr
            FIXTURE_FILE_ATTR = 'data-fixture-file',
            FIXTURE_NAME_ATTR = 'data-fixture-name',
            REPORT_FORMAT_ATTR = 'data-format',
            RUN_FAILED_TESTS_ATTR = 'data-run-failed-tests',
            RUN_OPTIONS_ATTR = 'data-run-options',
            TASK_NAME_ATTR = 'data-task-name',
            TASK_UID_ATTR = 'data-taskid',
            TEST_NAME_ATTR = 'data-test-name',
            TEST_UID_ATTR = 'data-test-uid',

        //Texts
            REMOVE_REPORT_DIALOG_TEXT_PATTERN = 'Are you sure that you wish to delete the {0} report?',
            TASK_FAILED_TEXT = 'fail',
            TASK_PASSED_TEXT = 'passed',
            NO_SCREENSHOTS_TEXT = 'No screenshots',
            SCREENSHOTS_AVAILABLE_TEXT = 'Screenshots available',
            NO_ERRORS_TEXT = 'No errors',

            EXPORT_URL_PATTERN = '/export_results/?format={0}&taskID={1}',
            SCREENSHOTS_DIALOG_TEXT_PATTERN = '"{0}" - Screenshots',
            WORKER_VERSION_PATTERN = ' (v. {0})',

        //Tmpl
            ERROR_ROW_PATTERN = '<div class="error-row"><pre>{0}</pre>{1}</div>',
            BROWSER_HEADER_PATTERN = '<span class="browser-header">{0}</span>',
            STEP_HEADER_PATTERN = '<div class="step-separator"></div><p><span></span>Step: "{0}"</p>',
            THUMBNAIL_PATTERN = '<a href="{0}" class="thumbnail" target="_blank">' +
                '<div class="overlay"></div><div class="zoom-icon"></div>' +
                '<img src="{1}" alt=""/><span>{2}</span></a>',
            SCREEN_PREVIEW_PATTERN = '<p class="screen-title">Screenshot: </p>{0}',

        //Metrics
            TASK_MAX_ANIMATION_COUNT = 2,
            SLIDE_DURATION = 200,
            CONFIRM_DIALOG_WIDTH = 610,
            ALERT_DIALOG_WIDTH = 610,
            SCREENSHOTS_GALLERY_WIDTH = 520,
            SCREENSHOTS_GALLERY_MIN_HEIGHT = 400,
            REMOVE_DOWNLOADING_IFRAME_TIMEOUT = 1000,
            LOADING_MSG_DURATION = 750,
            GRID_UPDATE_LOCK_TIMEOUT = 300,

        //Grid options
            GRID_WIDTH = 960,
            PAGE_SIZE = 100,
            SEARCH_PANEL_WIDTH = 240,
            PAGINATION_VALUES = [200, 500, 1000],
            DURATION_COLUMN = 'Duration',
            ERRORS_COLUMN = 'Errors',
            FIXTURE_NAME_COLUMN = 'Fixture',
            SCREENSHOT_COLUMN = 'Screenshots',
            STATUS_COLUMN = 'Status',
            TEST_NAME_COLUMN = 'Test',
            DURATION_COLUMN_WIDTH = 100,
            ERRORS_COLUMN_WIDTH = 60,
            SCREENSHOT_COLUMN_WIDTH = 90,
            STATUS_COLUMN_WIDTH = 90;

        var $document = $(document),
            isPage404 = false,
            pageTitle = document.title,
            scrollTop = 0,
            lastContHeight = 0,
            loadAnimationStartTime = 0,
            taskUpdateList = [],
            lastTaskUpdateData = [],
            animationBlocked = false,
            fullUpdateFlag = false,
            stopUpdateFlag = false,
            taskContent = null,
            $alertDialog = null,
            $screenGallery = null,
            removeTaskReportUid = null,
            screenshots = {},
            isFirstGridInit = true,
            gridState = null,
            highlight = function() {};

        var setInCenter = function ($elm) {
            var topMargin = (Util.getContentHeight() - $elm.height()) / 2;

            if (topMargin > 0)
                $elm.css('margin-top', topMargin);

            var leftMargin = ($(TASKS_RESULTS_CONTAINER).outerWidth(false) - $elm.width()) / 2;

            if (leftMargin > 0)
                $elm.css('margin-left', leftMargin);
        };

        var showTopPanel = function () {
            var $topBtns = $(TOP_BUTTONS);

            $topBtns.removeClass(HIDDEN_CLASS);
            $topBtns.prev().removeClass(HIDDEN_CLASS);
        };

        var hideTopPanel = function () {
            var $topBtns = $(TOP_BUTTONS);

            $topBtns.addClass(HIDDEN_CLASS);
            $topBtns.prev().addClass(HIDDEN_CLASS);
        };

        var slide = function (from, to, disableAnimation) {
            var $container = $(TASKS_RESULTS_CONTAINER),
                forward = Util.getPathDepth(to) > Util.getPathDepth(from);

            var showTopPanelForList = function () {
                if (!isDetailView() && !$(NO_MSG).length)
                    showTopPanel();
                else
                    hideTopPanel();
            };

            $.get([to, SLIDE_QUERY_PARAM].join(''), function (res) {
                if (!disableAnimation) {
                    $container.hide('drop', { direction: forward ? 'left' : 'right' }, SLIDE_DURATION, function () {
                        $container.html(res);

                        $container.show('drop', { direction: forward ? 'right' : 'left' }, SLIDE_DURATION, function () {
                            showTopPanelForList();
                            initContent();
                        });
                    });
                } else {
                    $container.html(res);
                    showTopPanelForList();
                    initContent();
                }
            });
        };

        var isDetailView = function () {
            return $(TASKS_RESULTS_CONTAINER).find(DETAIL_VIEW).length;
        };

        var saveState = function () {
            scrollTop = $(window).scrollTop();
            lastContHeight = $(TASKS_RESULTS_CONTAINER).height();
        };

        var restoreState = function () {
            var $resultsCont = $(TASKS_RESULTS_CONTAINER),
                curContHeight = $resultsCont.height();

            //NOTE: Calculate the difference in height if it is not detailed view.
            if (!$resultsCont.find(DETAIL_VIEW).length)
                scrollTop += curContHeight - lastContHeight;

            $('html, body').scrollTop(scrollTop);
        };

        var createLoadingMsg = function () {
            var $revisionContainer = $(TASKS_RESULTS_CONTAINER);

            stopUpdateFlag = true;
            taskContent = $revisionContainer.html();

            var $loadingMsgContent = $('<div></div>')
                .addClass(LOADING_MSG_CLASS);

            $(TASKS_RESULTS_CONTAINER).html($loadingMsgContent);
            setInCenter($loadingMsgContent);

            loadAnimationStartTime = (new Date()).getTime();
        };

        var clearLoadingMsg = function(returnToList) {
            var animationDuration = (new Date()).getTime() - loadAnimationStartTime;

            var restore = function () {
                if(returnToList && RESULTS_URL !== location.href) {
                    if(History)
                        History.pushState(null, pageTitle, RESULTS_URL);
                    else
                        location.href = RESULTS_URL;
                } else
                    slide(location.href, location.href, true);

                isFirstGridInit = true;
                loadAnimationStartTime = 0;
                stopUpdateFlag = false;
            };

            if (animationDuration < LOADING_MSG_DURATION)
                setTimeout(restore, LOADING_MSG_DURATION - animationDuration);
            else
                restore();
        };

        var initHyperlinks = function () {
            $(TASKS_RESULTS_CONTAINER).find(NAV_LINK).click(function () {
                var href = $(this).attr('href');

                showTopPanel();

                if (History.enabled)
                    History.pushState(null, pageTitle, href);
                else
                    window.location.href = href;

                return false;
            });
        };

        var initAlertDialog = function () {
            $alertDialog = $(ALERT_DIALOG).dialog({
                width: ALERT_DIALOG_WIDTH,
                autoOpen: false,
                dialogClass: ALERT_DIALOG_CLASS
            });

            $(ALERT_DIALOG_OK_BTN).click(function () {
                $alertDialog.dialog('close');
            });
        };

        var restartTests = function(options) {
            var runXhr = $.ajax({
                type: 'POST',
                url: RUN_TASK_URL,
                data: {opt: options}
            });

            runXhr.always(function () {
                var res = JSON.parse(runXhr.responseText);

                if (res.errs.length) {
                    var errsHtml = res.errs.reduce(function (res, elm) {
                        return res + '<p>' + elm + '</p>';
                    }, '');

                    $alertDialog.find(DIALOG_TEXT).html(errsHtml);
                    $alertDialog.dialog('open');
                } else if (res.workerName)
                    location.href = IDLE_WORKER_URL + res.workerName + RETURN_URL_QUERY_PARAM + options.location;
                else
                    location.href = RESULTS_URL + res.taskUid;
            });
        };

        var initRestartTaskButtons = function () {
            var $restartLink = $(TASKS_RESULTS_CONTAINER).find(RESTART_TASK_LINKS);

            $(RESTART_TASK_BTN).click(function(e) {
                e.stopImmediatePropagation();
            });

            $restartLink.click(function (e) {
                var $taskItem = $(this).closest(TASK_ITEM),
                    opt = JSON.parse($taskItem.attr(RUN_OPTIONS_ATTR)),
                    taskId = $taskItem.attr(TASK_UID_ATTR),
                    returnUrl = isDetailView() ? RESULTS_URL : location.href;

                var runTests = function() {
                    restartTests({
                        sourceType: opt.sourceType,
                        source: opt.source,
                        workers: opt.workers,
                        browsers: opt.browsers,
                        groupName: opt.groupName,
                        takeScreenshotOnFails: opt.takeScreenshotOnFails,
                        failOnJsErrors: opt.failOnJsErrors,
                        location: returnUrl,
                        returnToReport: true
                    });
                };

                if (opt._currentWindowWorker)
                    opt.workers = [];

                if($(this).attr(RUN_FAILED_TESTS_ATTR)) {
                    opt.groupName = (typeof opt.source === 'string' ? opt.source : opt.groupName);
                    opt.sourceType = 'group';
                    opt.source = [];

                    $.get(Util.formatText(GET_REPORT_JSON_URL, taskId), function (data) {
                        for(var testUid in data.testErrReports) {
                            if(data.testErrReports.hasOwnProperty(testUid)) {
                                var err = data.testErrReports[testUid],
                                    testPath = err.fixturePath.length ?
                                        Util.sourceJoin(err.fixturePath, err.fixtureName, err.name) :
                                        Util.sourceJoin(err.fixtureName, err.name);

                                if(opt.source.indexOf(testPath) === -1)
                                    opt.source.push(testPath);
                            }
                        }

                        runTests();
                        UXLog.write('Tasks: Restart failed tests clicked');
                    });
                } else {
                    runTests();
                    UXLog.write('Tasks: Restart task clicked');
                }

                e.stopImmediatePropagation();
            });
        };

        var initExportReports = function () {
            $(EXPORT_REPORT).click(function (e) {
                e.stopImmediatePropagation();
            });

            var downLoadFile = function(url) {
                $('body').append("<iframe class=" + FILE_DOWNLOADING_CLASS + " src='" + url + "' style='display: none;' ></iframe>");

                setTimeout(function () {
                    $(FILE_DOWNLOADING).remove();
                }, REMOVE_DOWNLOADING_IFRAME_TIMEOUT);
            };

            $(EXPORT_REPORT).find('li').click(function (e) {
                var $el = $(this),
                    format = $el.attr(REPORT_FORMAT_ATTR);

                UXLog.write('Tasks: export task clicked, format: ' + format);
                downLoadFile(Util.formatText(EXPORT_URL_PATTERN, format, $el.attr(TASK_UID_ATTR)));
                e.stopImmediatePropagation();
            });
        };

        var initNavButtons = function () {
            $(TASKS_RESULTS_CONTAINER).find(DETAIL_LINK).click(function () {
                var taskID = $(this).attr('data-taskID'),
                    navLink = TASKS_RESULTS_URL + taskID;

                UXLog.write('Tasks: task detail opened');

                hideTopPanel();
                isFirstGridInit = true;

                if (History.enabled)
                    History.pushState(null, pageTitle, navLink);
                else
                    window.location.href = navLink;

                return false;
            });
        };

        var openFixtureInTestsView = function(fixturePath, fixtureName) {
            var path = ['', fixturePath].join('/');

            window.open([PROJECT_URL, path, '?open=', encodeURIComponent(fixtureName)].join(''));
        };

        var openTestInTestsView = function(fixturePath, fixtureName, testName) {
            var path = ['', fixturePath].join('/');

            window.open([PROJECT_URL, path,
                '?open=', encodeURIComponent(fixtureName),
                '&testName=', encodeURIComponent(testName)
            ].join(''));
        };

        var initResultsGrid = function(data) {
            var $gridContainer = $(GRID_CONTAINER);

            $gridContainer.css('top', $(HEADER_BACK).height() +
                parseInt($(CONTENT).css('margin-top')) + $(REPORT_HEADER).height() + 'px');

            $gridContainer.dxDataGrid({
                allowColumnResizing: true,
                allowColumnReordering: true,
                dataSource: data,
                hoverStateEnabled: false,
                showBorders: false,
                showColumnLines: true,
                showRowLines: false,
                cellHintEnabled: false,
                scrolling: {
                    mode: 'infinity'
                },
                width: GRID_WIDTH,
                masterDetail: {
                    enabled: true,
                    template: function(container, options) {
                        var errsByWorker = options.key.errMsg,
                            content = '';

                        var $content = $('<div>');

                        for(var worker in errsByWorker) {
                            if(errsByWorker.hasOwnProperty(worker)) {
                                content = content ? content + '\n\n': '';
                                content += Util.formatText(BROWSER_HEADER_PATTERN, worker);
                                content += errsByWorker[worker].errMsg;
                            }
                        }

                        $content.append(content);
                        $content.appendTo(container);

                        highlight(container.find(HIGHLIGHT_CODE).not(GO_TO_CODE_BTN), false);
                        highlight(container.find([HIGHLIGHT_CODE, GO_TO_CODE_BTN].join('')), true);
                        //NOTE: hack for current version
                        $(GRID_CONTAINER).dxDataGrid('instance').getView('rowsView')._highlightSearchText(container);
                    }
                },
                loadPanel:{
                    enabled: false
                },
                paging: {
                    pageSize: PAGE_SIZE
                },
                selection: {
                    mode: 'none'
                },
                pager: {
                    showPageSizeSelector: true,
                    allowedPageSizes: PAGINATION_VALUES
                },
                columns: [
                    {dataField: STATUS_COLUMN, width: STATUS_COLUMN_WIDTH, alignment: 'left', cellTemplate: function (container, options) {
                        $('<p></p>')
                            .text(options.value)
                            .attr('class', options.value)
                            .prepend('<span class="status-marker"></span>')
                            .appendTo(container);
                    }},
                    {dataField: TEST_NAME_COLUMN, cellTemplate: function (container, options) {
                        $('<span></span>')
                            .text(options.value)
                            .attr('class', TEST_NAME_CLASS)
                            .attr('title', options.value)
                            .attr(TEST_NAME_ATTR, options.value)
                            .appendTo(container);
                    }},
                    {dataField: FIXTURE_NAME_COLUMN, cellTemplate: function (container, options) {
                        $('<span></span>')
                            .text(options.value)
                            .attr('class', FIXTURE_NAME_CLASS)
                            .attr('title', options.value)
                            .attr(FIXTURE_FILE_ATTR, options.key.fixtureFileName)
                            .attr(FIXTURE_NAME_ATTR, options.value)
                            .appendTo(container);
                    },
                        groupCellTemplate: function (container, options) {
                            $('<span></span>')
                                .text(options.value)
                                .attr('class', FIXTURE_NAME_CLASS)
                                .attr(FIXTURE_NAME_ATTR, options.value)
                                .click(function () {
                                    UXLog.write('Tasks: Fixture name clicked (Grouping)');
                                    openFixtureInTestsView(options.data.items[0].fixturePath, options.data.items[0].fixtureName);
                                }).appendTo(container);
                        }
                    },
                    {dataField: DURATION_COLUMN, width: DURATION_COLUMN_WIDTH, alignment: 'center', customizeText: function(options) {
                        return Util.getFormattedTime(options.value);
                    }},
                    {dataField: SCREENSHOT_COLUMN, width: SCREENSHOT_COLUMN_WIDTH, alignment: 'center', cellTemplate: function (container, options) {
                        container.html(options.value);
                    }, groupCellTemplate: function(container, options) {
                        if(!options.value)
                            container.html(NO_SCREENSHOTS_TEXT);
                        else
                            container.html(SCREENSHOTS_AVAILABLE_TEXT);
                    }, calculateGroupValue: function(data) {
                        if(data[SCREENSHOT_COLUMN] === '-')
                            return 0;
                        else
                            return 1;

                    }},
                    {dataField: ERRORS_COLUMN, width: ERRORS_COLUMN_WIDTH, alignment: 'center',  dataType:'string', groupCellTemplate: function(container, options) {
                        if(options.value === '-')
                            container.html(NO_ERRORS_TEXT);
                        else
                            container.html(options.column.caption + ': ' +  options.value);
                    }},
                    {calculateCellValue: function(data) {
                        var errMsg = data.errMsg,
                            searchContent = '';

                        for(var worker in errMsg) {
                            if(errMsg.hasOwnProperty(worker)) {
                                searchContent += worker;
                                searchContent += errMsg[worker].errMsg;
                            }
                        }

                        return searchContent;
                    }, allowFiltering: true, dataType: 'string', visible: false, calculateFilterExpression: function(text) {
                        return [this.calculateCellValue, 'contains', text];
                    } }
                ],
                grouping: {
                    autoExpandAll: true
                },
                searchPanel: {
                    visible: true,
                    width: SEARCH_PANEL_WIDTH
                },
                groupPanel: {
                    visible: true
                },
                onCellPrepared: function(e){
                    //NOTE: Not expand master detail if it's empty
                    if (!e.key.errMsg && e.column.command === 'expand')
                        e.cellElement.removeClass('dx-command-expand dx-datagrid-group-closed dx-datagrid-expand');
                },
                onRowPrepared: function(e) {
                    if(e && e.key && e.key[STATUS_COLUMN] !== TASK_PASSED_TEXT)
                        e.rowElement.css('cursor', 'pointer');
                },
                onEditorPrepared: function() {
                    if(gridState) {
                        var dataGrid = $(GRID_CONTAINER).dxDataGrid('instance');
                        dataGrid.state(gridState);

                        gridState = null;
                    }

                    if(isFirstGridInit)
                        isFirstGridInit = false;
                },
                onCellClick: function(clickedCell) {
                    //NOTE: we go to fixture and test only if we clicked on link, not cell
                    var testElement = document.elementFromPoint(clickedCell.jQueryEvent.clientX, clickedCell.jQueryEvent.clientY);
                    if(testElement.tagName.toLowerCase() === 'td' && clickedCell.columnIndex) {
                        if(clickedCell.key.errMsg) {
                            if(clickedCell.component.isRowExpanded(clickedCell.key))
                                clickedCell.component.collapseRow(clickedCell.key);
                            else
                                clickedCell.component.expandRow(clickedCell.key);
                        }

                        return;
                    }

                    if (clickedCell.cellElement.find(TEST_NAME).length) {
                        UXLog.write('Tasks: Test name clicked');
                        openTestInTestsView(clickedCell.data.fixturePath, clickedCell.data.fixtureName, clickedCell.value);
                    } else if (clickedCell.cellElement.find(FIXTURE_NAME).length) {
                        UXLog.write('Tasks: Fixture name clicked');
                        openFixtureInTestsView(clickedCell.data.fixturePath, clickedCell.data.fixtureName);
                    }
                },
                onContentReady: function() {
                    var $scrollView = $(GRID_CONTAINER).find('.dx-scrollable');

                    //NOTE: T154989
                    $scrollView.dxScrollable({ showScrollbar: 'always'});
                 }
            });
        };

        var initDetailPage = function() {
            var locArr = location.href.split('/'),
                taskID = locArr[locArr.length - 1],
                formattedData = [];

            $.get(Util.formatText(GET_REPORT_JSON_URL, taskID), function (data) {
                var getScreenBtn = function(uid, stepName, workerName) {
                    var testScreenshots = data._screenshots[uid];

                    if(testScreenshots) {
                        for (var index = 0, length = testScreenshots.length; index < length; index++) {
                            var screen = testScreenshots[index],
                                getScreenUrl = Util.formatText(TO_SCREEN_URL_PATTERN, encodeURIComponent(screen.filePath), [screen.stepName, screen.workerName].join(' - ')),
                                getScreenThumbnailUrl = Util.formatText(GET_SCREEN_THUMBNAIL_URL_PATTERN, encodeURIComponent(screen.filePath));

                            if(screen.stepName === stepName && screen.workerName === workerName && screen.isFailedStep) {
                                var thumbNail = Util.formatText(THUMBNAIL_PATTERN, getScreenUrl, getScreenThumbnailUrl, '');
                                return Util.formatText(SCREEN_PREVIEW_PATTERN, thumbNail);
                            }
                        }
                    }

                    return '';
                };

                var browserVersions = data.browserVersions;
                screenshots = data._screenshots;

                function buildGridDateHandler(obj) {
                    for (var key in obj) {
                        if (obj.hasOwnProperty(key)) {
                            var test = obj[key],
                                data = {
                                    uid: key
                                },
                                testsDictionary = {};

                            var fixture = test.fixturePath ?
                                ['', test.fixturePath, test.fixtureName].join('/') :
                                [test.fixturePath, test.fixtureName].join('/');

                            data[TEST_NAME_COLUMN] = test.name;
                            data[FIXTURE_NAME_COLUMN] = fixture;
                            data[DURATION_COLUMN] = test.time;
                            data.fixtureName = test.fixtureName;
                            data.fixturePath = test.fixturePath;
                            data.fixtureFileName = test.fixtureFileName;

                            if (screenshots[key] && screenshots[key].length) {
                                var screens = screenshots[key];

                                for(var screenIndex = 0; screenIndex < screens.length; screenIndex++) {
                                    if(!screens[screenIndex].isFailedStep)
                                        data[SCREENSHOT_COLUMN] =
                                            '<a class="screenshot screenshot-cell" title="View screenshots" ' + TEST_UID_ATTR + '="' + key + '"></a>';
                                }

                                if(!data[SCREENSHOT_COLUMN])
                                    data[SCREENSHOT_COLUMN] = '-';
                            } else
                                data[SCREENSHOT_COLUMN] = '-';

                            if (test.errs && test.errs.length) {
                                for (var index = 0, length = test.errs.length; index < length; index++) {
                                    var err = test.errs[index],
                                        browserVersion = browserVersions[err.workerName],
                                        version = browserVersion ? Util.formatText(WORKER_VERSION_PATTERN, browserVersion) : '',
                                        workerName = err.workerName + version,
                                        screenHtml = getScreenBtn(key, err._originalErr.stepName, err.workerName),
                                        errRow = Util.formatText(ERROR_ROW_PATTERN, err._msgMarkdown, screenHtml);

                                    //NOTE: we're grouping several errors for one test;
                                    if(testsDictionary.hasOwnProperty(fixture) && testsDictionary[fixture].hasOwnProperty(test.name)) {
                                        var testData = formattedData[testsDictionary[fixture][test.name]];

                                        if(!testData.errMsg[workerName])
                                            testData.errMsg[workerName] = {
                                                errMsg: errRow,
                                                stepName: err._originalErr.stepName
                                            };
                                        else
                                            testData.errMsg[workerName].errMsg += '\n' + errRow;

                                        continue;
                                    } else {
                                        testsDictionary[fixture] = {};
                                        testsDictionary[fixture][test.name] = formattedData.length;

                                        data[STATUS_COLUMN] = TASK_FAILED_TEXT;
                                        data.errMsg = {};
                                        data.errMsg[workerName] = {
                                            errMsg: errRow,
                                            stepName: err._originalErr.stepName
                                        };
                                        data[ERRORS_COLUMN] = test.errs.length;
                                    }

                                    formattedData.push(data);
                                }
                            }
                            else {
                                data[STATUS_COLUMN] = TASK_PASSED_TEXT;
                                data[ERRORS_COLUMN] = '-';
                                data.errMsg = '';

                                formattedData.push(data);
                            }
                        }
                    }
                }

                buildGridDateHandler(data.testErrReports);
                buildGridDateHandler(data.passedTests);

                if(isFirstGridInit)
                    initResultsGrid(formattedData);
                else {
                    var $gridCont = $(GRID_CONTAINER);

                    gridState = $gridCont.dxDataGrid('instance').state();
                    $gridCont.dxDataGrid({dataSource: formattedData});
                }
            });
        };

        var updateGalleryAndOpen = function(uid, testName, stepName) {
            var $content = $(SCREENSHOTS_GALLERY).find(SCREENSHOTS_LIST),
                screenTarget = stepName ? stepName : testName;

            $content.html('');
            $screenGallery.parent().find(UI_DIALOG_TITLE).html(Util.formatText(SCREENSHOTS_DIALOG_TEXT_PATTERN, screenTarget));

            var testScreens = screenshots[uid] || [],
                curStepName = '';

            for(var index = 0, length = testScreens.length; index < length; index++) {
                var screen = testScreens[index],
                    getScreenUrl = Util.formatText(TO_SCREEN_URL_PATTERN, encodeURIComponent(screen.filePath), [screen.stepName, screen.workerName].join(' - ')),
                    getScreenThumbnailUrl = Util.formatText(GET_SCREEN_THUMBNAIL_URL_PATTERN, encodeURIComponent(screen.filePath)),
                    screenHtml = Util.formatText(THUMBNAIL_PATTERN, getScreenUrl, getScreenThumbnailUrl, screen.workerName);

                if(stepName) {
                    if(stepName === screen.stepName && screen.isFailedStep) {
                        if (screen.stepName !== curStepName) {
                            $content.append(Util.formatText(STEP_HEADER_PATTERN, screen.stepName));
                            curStepName = screen.stepName;
                        }
                        $content.append(screenHtml);
                    }
                } else if (!screen.isFailedStep) {
                    if (screen.stepName !== curStepName) {
                        $content.append(Util.formatText(STEP_HEADER_PATTERN, screen.stepName));
                        curStepName = screen.stepName;
                    }
                    $content.append(screenHtml);
                }
            }

            $screenGallery.parent().find(UI_DIALOG_TITLE_CONT).draggable();
            $content.resizable({
                minWidth: SCREENSHOTS_GALLERY_WIDTH,
                minHeight: SCREENSHOTS_GALLERY_MIN_HEIGHT,
                resizeMarkerClass: RESIZE_SCREENSHOT_GALLERY_CLASS,
                action: function () {
                    $content.scrollBar({ scrollClass: SCREENSHOTS_GALLERY_SCROLLBAR_CLASS });
                }
            });
            $screenGallery.dialog('open');
            $content.scrollBar({ scrollClass: SCREENSHOTS_GALLERY_SCROLLBAR_CLASS });

            UXLog.write('Tasks: screenshots gallery opened');
        };

        var initScreenshotsGallery = function() {
            $screenGallery = $(SCREENSHOTS_GALLERY).dialog({
                width: 'auto',
                autoOpen: false,
                dialogClass: SCREENSHOTS_GALLERY_CLASS
            });

            $document.on('click', SCREENSHOTS_GALLERY_ALL_BTN, function(e) {
                var $btn = $(this),
                    $row = $btn.closest(GRID_ROW),
                    testName = $row.find(TEST_NAME).attr(TEST_NAME_ATTR);

                updateGalleryAndOpen($btn.attr(TEST_UID_ATTR), testName, '');

                e.stopImmediatePropagation();
            });
        };

        var initContent = function () {
            initNoTaskResultsMsg();
            initNavButtons();
            initRestartTaskButtons();
            initHyperlinks();
            initRemoveReportBtn();
            initExportReports();
            initCodeEditorDialog();
            initScreenshotsGallery();

            if(isDetailView())
                initDetailPage();
        };

        var initNoTaskResultsMsg = function () {
            var setNoResultsMsgMargin = function () {
                var $noTasksResultsMsg = $(NO_TASKS_RESULTS),
                    topMargin = Math.ceil((Util.getContentHeight() - $noTasksResultsMsg.outerHeight(false)) / 2);

                if (topMargin > 0)
                    $noTasksResultsMsg.css('margin-top', topMargin);
            };

            if ($(NO_TASKS_RESULTS))
                setNoResultsMsgMargin();

            $(window).resize(function () {
                if ($(NO_TASKS_RESULTS))
                    setNoResultsMsgMargin();
            });
        };

        var initHistorySlider = function () {
            if (!History.enabled)
                return;

            var from = window.location.href;

            History.Adapter.bind(window, 'statechange', function () {

                //NOTE: Fix for return from 404 error page. Reload page for delete 404 page from browser history.
                if (isPage404) {
                    window.location.reload();
                    return;
                }

                var to = History.getState().url;

                isFirstGridInit = true;

                slide(from, to);

                from = to;
            });
        };

        var taskUpdate = function (taskID, callback) {
            if (stopUpdateFlag)
                return;

            //NOTE: push in updateList array count active animation, need it for limit the animation.
            if (!taskUpdateList[taskID])
                taskUpdateList[taskID] = 1;

            var $taskResultsCont = $(TASKS_RESULTS_CONTAINER),
                $taskItem = $taskResultsCont.find(TASK_ITEM + '[data-taskID = "' + taskID + '"]'),
                queryPath = REVISION_REQ_URL + taskID;

            if (isDetailView()) {
                queryPath += DETAIL_QUERY_PARAM;

                if(!$taskItem.length)
                    return;
            }

            $.get(queryPath, function (data) {
                lastTaskUpdateData[taskID] = data;
                $taskItem = $taskResultsCont.find(TASK_ITEM + '[data-taskID = "' + taskID + '"]');

                var insertTaskContent = function () {
                    saveState();

                    if ($taskItem.length) {
                        if (!isDetailView()) {
                            $taskItem.replaceWith(data);
                            showTopPanel();
                        }
                        else
                            $taskItem.find(REPORT_HEADER).replaceWith(data);

                    }
                    else if (!isDetailView()) {
                        $taskResultsCont.prepend(data);
                        $(NO_MSG).remove();
                    }

                    $taskItem = $taskResultsCont.find(TASK_ITEM + '[data-taskID = "' + taskID + '"]');

                    restoreState();
                    //NOTE reload content script
                    initContent();
                };

                if (!isDetailView() && taskUpdateList[taskID] < TASK_MAX_ANIMATION_COUNT) {
                    animationBlocked = true;
                    insertTaskContent();

                    $($taskItem).show('fast', function () {
                        animationBlocked = false;
                        saveState();
                        //NOTE: Insert task data, which are updated during the animation.
                        $taskItem.replaceWith(lastTaskUpdateData[taskID]);
                        restoreState();
                        initContent();
                        taskUpdateList[taskID] = taskUpdateList[taskID] ? --taskUpdateList[taskID] : 0;
                    });
                } else {
                    //NOTE: Flag animationBlocked stops insert the updated data, to the end of the animation.
                    if (!animationBlocked) {
                        insertTaskContent();

                        $($taskItem).show();
                        taskUpdateList[taskID] = taskUpdateList[taskID] ? --taskUpdateList[taskID] : 0;
                    } else {
                        taskUpdateList[taskID] = taskUpdateList[taskID] ? --taskUpdateList[taskID] : 0;
                    }
                }

                //NOTE: animation limit
                if (taskUpdateList[taskID] > TASK_MAX_ANIMATION_COUNT) {
                    fullUpdateFlag = true;
                } else {
                    taskUpdateList[taskID]++;
                }

                if(typeof callback === 'function')
                    callback();
            });
        };

        var initRevisionWatcher = function () {
            var socket = io.connect(window.location.protocol + '//' + window.location.host),
                socketDisconnected = false,
                updateLock = false,
                updateDelayTimeout = null;

            if (fullUpdateFlag) {
                slide(location.href, location.href);
                fullUpdateFlag = false;
                taskUpdateList = [];
            }

            socket.on('taskUpdated', function (data) {
                if (!isDetailView()) {
                    taskUpdate(data.taskUid);
                }
                else {
                    if (!updateLock) {
                        updateLock = true;
                        taskUpdate(data.taskUid, function() {
                            updateLock = false;
                        });
                    } else if (!updateDelayTimeout) {
                        updateDelayTimeout = setTimeout(function () {
                            var $reportHeader = $(REPORT_HEADER);

                            if (!$reportHeader.hasClass(FAILED_VIEW_CLASS) && !$reportHeader.hasClass(SUCCESS_VIEW_CLASS)) {
                                taskUpdate(data.taskUid, function() {
                                    window.clearTimeout(updateDelayTimeout);
                                    updateDelayTimeout = null;
                                });
                            }
                        }, GRID_UPDATE_LOCK_TIMEOUT);
                    }
                }
            });

            socket.on('taskComplete', function (data) {
                taskUpdate(data.taskUid);
            });

            //NOTE: show update message for reconnect
            socket.on('connect', function () {
                if (socketDisconnected) {
                    window.location.reload();
                    socketDisconnected = false;
                }
            });

            socket.on('disconnect', function () {
                socketDisconnected = true;
            });
        };

        var initPageEffect = function () {
            $(TASKS_RESULTS_CONTAINER).fadeIn();
        };

        var initRemoveReportBtn = function () {
            var $cont = $(REMOVE_REPORT_DIALOG_CONT),
                $removeReportDialog = $cont.dialog({
                    autoOpen: false,
                    dialogClass: REMOVE_REPORT_CLASS,
                    width: CONFIRM_DIALOG_WIDTH
                }),
                $report = null,
                $detailView = $(DETAIL_VIEW);

            $(REMOVE_REPORT_BTN).click(function (e) {
                $report = $(this).closest(DETAIL_LINK);
                removeTaskReportUid = isDetailView() ? $detailView.attr(TASK_UID_ATTR) : $report.attr(TASK_UID_ATTR);

                var reportName = isDetailView() ? $detailView.attr(TASK_NAME_ATTR) : $report.attr(TASK_NAME_ATTR);

                $cont.find(DIALOG_HEADER).text(Util.formatText(REMOVE_REPORT_DIALOG_TEXT_PATTERN, reportName));

                $removeReportDialog.dialog('open');
                UXLog.write('Tasks: remove task report opened');

                e.stopImmediatePropagation();
            });

            $(REMOVE_REPORT_NO_BTN).click(function (e) {
                removeTaskReportUid = null;
                $removeReportDialog.dialog('close');
                e.stopImmediatePropagation();
            });

            $(REMOVE_REPORT_YES_BTN).click(function (e) {
                if (!removeTaskReportUid)
                    return;

                $removeReportDialog.dialog('close');
                hideTopPanel();
                createLoadingMsg();

                $.post(REMOVE_REPORT_URL, {uid: removeTaskReportUid}, function () {
                    clearLoadingMsg(true);
                });

                e.stopImmediatePropagation();
            });
        };

        var initClearResultsBtn = function () {
            var $cont = $(CLEAR_REPORTS_DIALOG_CONT),
                $clearReportsDialog = $cont.dialog({
                    autoOpen: false,
                    dialogClass: CLEAR_REPORTS_CLASS,
                    width: CONFIRM_DIALOG_WIDTH
                });

            $(CLEAR_RESULTS).click(function (e) {
                $clearReportsDialog.dialog('open');

                e.stopImmediatePropagation();
            });

            $(CLEAR_REPORTS_NO_BTN).click(function (e) {
                $clearReportsDialog.dialog('close');

                e.stopImmediatePropagation();
            });

            $(CLEAR_REPORTS_YES_BTN).click(function (e) {
                $clearReportsDialog.dialog('close');
                hideTopPanel();
                createLoadingMsg();

                $.post(CLEAR_REPORTS_URL, function () {
                   clearLoadingMsg(false);
                });

                e.stopImmediatePropagation();
            });
        };

        var initHorizontalScrollHandler = function () {
            $(window).scroll(function () {
                var scrollLeft = $(this).scrollLeft();

                $(HEADER_BACK).css('left', -scrollLeft);
                $(TOP_BUTTONS).css('left', -scrollLeft);
                $(TOP_BUTTONS_CONT).css('left', -scrollLeft);
                $(FOOTER).css('left', -scrollLeft);
            });
        };

        var initCodeHighlighter = function (callback) {
            var baseLocation = [location.protocol, location.host].join('//'),
                aceBaseLocation = baseLocation + ACE_LIB_URL;

            ace.config.set('modePath', aceBaseLocation);
            ace.config.set('workerPath', aceBaseLocation);
            ace.config.set('themePath', aceBaseLocation);

            Util.loadAceEditorModules(ace,
                [
                    'ace/theme/textmate',
                    'ace/mode/javascript',
                    'ace/mode/html',
                    'ace/ext/language_tools',
                    'ace/ext/static_highlight'
                ],
                function () {
                    var dom = ace.require('ace/lib/dom'),
                        highlighter = ace.require('ace/ext/static_highlight'),
                        JavaScriptParser = new (ace.require('ace/mode/javascript').Mode)(),
                        HTMLParser = new (ace.require('ace/mode/html').Mode)(),
                        theme = ace.require('ace/theme/textmate');

                    var highlight = function ($codeElms, wrapElm) {
                        $.each($codeElms, function (index, el) {
                            var $highlightedEl = wrapElm ? $(el).wrap('<div class="multiline_editor"></div>')[0] : el;
                            var data = $highlightedEl.textContent,
                                mode = /(^<)|(^\[<)/.test(data) ? HTMLParser : JavaScriptParser,
                                highlighted = highlighter.render(data, mode, theme, 0, false);

                            dom.importCssString(highlighted.css, 'ace_highlight');
                            $highlightedEl.innerHTML = highlighted.html;

                            if (el.offsetWidth < el.scrollWidth) {
                                var $el = $(el);
                                $el.width(el.scrollWidth - parseInt($el.css('padding-left')) + 'px');
                            }
                        });
                    };

                    callback(highlight);
                });
        };

        var initCodeEditorDialog = function() {
            FixtureEditor.init({
                openBtn: GO_TO_CODE_BTN,
                onSaveClick: function() {
                    hideTopPanel();
                    createLoadingMsg();
                },
                onSaveSuccess: function() {
                    clearLoadingMsg(false);
                },
                onSaveFail: function(data) {
                    clearLoadingMsg(false);
                    $alertDialog.find(DIALOG_TEXT).html(data.responseText);
                    $alertDialog.dialog('open');
                },
                onOpenFail: function(data) {
                    $alertDialog.find(DIALOG_TEXT).html(data.responseText);
                    $alertDialog.dialog('open');
                },
                getCurrentTestName: function($btn) {
                    return $btn.closest(GRID_ROW).prev().find(TEST_NAME).attr(TEST_NAME_ATTR) || '';
                },
                getCurrentStep: function ($btn) {
                    if ($btn.hasClass(STEP_NAME_CLASS))
                        return $btn.text();
                    else
                        return $btn.closest(ERROR_ROW).find(GO_TO_CODE_BTN).eq(0).text();
                },
                getCurrentAssert: function ($btn) {
                    if ($btn.hasClass(STEP_NAME_CLASS))
                        return '';
                    else
                        return $btn.text();
                },
                getFileName: function ($btn) {
                    return $btn.closest(GRID_ROW).prev().find(FIXTURE_NAME).attr(FIXTURE_FILE_ATTR);
                },
                getCurrentPath: function ($btn) {
                    var fixtureName = $btn.closest(GRID_ROW).prev().find(FIXTURE_NAME).attr(FIXTURE_NAME_ATTR),
                        fixtureNameArr = fixtureName.indexOf('/') > -1 ? fixtureName.split('/') : [''];

                    return fixtureNameArr.length > 1 ? fixtureNameArr.slice(1, fixtureNameArr.length - 1).join('/') : '';
                }
            });
        };

        $document.ready(function () {
            UXLog.write('Tasks tab opened');
            initHorizontalScrollHandler();

            initPageEffect();
            initHistorySlider();
            initRevisionWatcher();
            initContent();
            initClearResultsBtn();
            initAlertDialog();
            initCodeHighlighter(function(func) {
                highlight = func;
            });
        });
    };
});