ControlPanel.define('Util', function (require, exports) {
    var $ = require('jQuery'),
        UXLog = require('UXLog');

    //Constants
    var
    //Selectors
        HEADER = '.header',
        FOOTER = '.footer',
        TESTS_VIEW_TOP_PANEL = '.fixtures-top-buttons',

        ERR_TEXT = '.err-text',
        INPUT_ERR_MSG = '.input-err',
        ERR_VALUE_CLASS = 'err-value',
        WARN_VALUE_CLASS = 'warn-value',

        ERR_MSG_TYPE = 'error',
        WARN_MSG_TYPE = 'warning';

    exports.getPathDepth = function (url) {
        var a = document.createElement('a');

        a.href = url;

        var path = a.pathname,
            depth = 0;

        if (path) {
            $.each(path.split('/'), function () {
                if (this.length)
                    depth++;
            });
        }

        return depth;
    };

    exports.scrollTo = function ($elem) {
        var offset = $elem.offset(),
            offsetTop = 0,
            $topPanel = $(TESTS_VIEW_TOP_PANEL),
            topPanelHeight = $topPanel.height();

        if (offset) {
            offsetTop = offset.top - $(HEADER).height();

            if ($topPanel.is(':visible'))
                offsetTop -= topPanelHeight;

            $('html, body').animate({
                scrollTop: offsetTop
            }, 'fast');
        }
    };

    exports.formatText = function (string) {
        var args = [].slice.call(arguments, 1);

        return string.replace(/{(\d+)}/g, function (match, number) {
            return typeof args[number] !== 'undefined' ? args[number] : match;
        });
    };

    exports.getContentHeight = function () {
        return $('body').height() - $(HEADER).height() - $(FOOTER).height();
    };

    exports.pathJoin = function () {
        var separator = '/',
            result = arguments[0] || '';

        if (arguments.length > 1) {
            if (/\\/.test(arguments[0]))
                separator = '\\';

            result = Array.prototype.slice.call(arguments).join('/')
                .replace(/[\/\\]/g, separator)
                .replace(new RegExp('\\' + separator + '{2,}', 'g'), separator);
        }

        return result;
    };

    exports.sourceJoin = function () {
        var res = arguments[0] || '';

        if (arguments.length > 1)
            res = Array.prototype.slice.call(arguments).join('/');

        return res;
    };

    exports.isNotHotKey = function (event) {
        return !event.ctrlKey && !event.altKey;
    };

    exports.cutQueryString = function (url) {
        return url.replace(/(\?.*)*$/, '');
    };

    //Fix for cross browser behavior jquery 2.1.3
    //We use scrollTop instead offset, because $('html,body').offset().top and $('body').offset().top always return 0
    exports.getBodyScrollTop = function () {
        return Math.max($('html,body').scrollTop(), $('body').scrollTop());
    };

    exports.getFormattedTime = function (time) {
        var duration = '0';

        if (time > 0) {
            duration = '';

            if (time >= 3600) {
                duration += Math.floor(time / 3600) + 'h ';
                time = time % 3600;
            }

            if (time >= 60) {
                duration += Math.floor(time / 60) + 'm ';
                time = time % 60;
            }

            if (time > 0)
                duration += time + 's';
        }

        if (duration[duration.length - 1] === ' ')
            duration = duration.substr(0, duration.length - 1);

        return duration;
    };

    exports.prepareTextForRegExp = function (str) {
        return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&").replace(/[\s]{1}/g, '[\\s]{1}');
    };

    exports.getCurPath = function () {
        var curPath = location.pathname.split('/').slice(2);

        if (!curPath[curPath.length - 1])
            curPath.pop();

        return curPath.join('/');
    };

    exports.removeInputMsgs = function ($cont) {
        var $dialogErrMsg = $cont.find(INPUT_ERR_MSG),
            $dialogErrMsgTitle = $dialogErrMsg.find(ERR_TEXT);

        $cont.find('input').removeClass(ERR_VALUE_CLASS).removeClass(WARN_VALUE_CLASS);
        $dialogErrMsg.hide();
        $dialogErrMsgTitle.html('');
    };

    exports.showInputMsg = function ($input, text, type) {
        exports.removeInputMsgs($input);
        type = type ? type : ERR_MSG_TYPE;

        if (type === WARN_MSG_TYPE) {
            UXLog.write('Settings: warning msg was shown for input #' + $input.attr('id'));
            $input.addClass(WARN_VALUE_CLASS);
        } else if (type === ERR_MSG_TYPE) {
            UXLog.write('Settings: error msg was shown for input #' + $input.attr('id'));
            $input.addClass(ERR_VALUE_CLASS);
        }

        $input.next().show().find(ERR_TEXT).text(text);
    };

    exports.loadAceEditorModules = function(ace, modules, cb) {
        var module = modules.shift();

        ace.config.loadModule(module, function() {
            if(modules.length)
                exports.loadAceEditorModules(ace, modules, cb);
            else
                cb();
        });
    };
});