/* global ace: true */

ControlPanel.define('Widgets.CodeEditor', function (require) {
    var $ = require('jQuery');

    //Consts
    var CODE_EDITOR = '#code-editor',
        CODE_EDITOR_ID = 'code-editor',
        ACE_LIB_URL = '/ace',
        EDITOR_THEME = 'ace/theme/textmate',
        EDITOR_MODE = 'ace/mode/javascript';

    var codeEditor = function ($el, options) {
        var editor = null,
            session = null,
            baseLocation = [location.protocol, location.host].join('//'),
            aceBaseLocation = baseLocation + ACE_LIB_URL;

        $el.html('<div id="' + CODE_EDITOR_ID + '"></div>');

        var $codeEditor = $(CODE_EDITOR);

        //Init ace base paths
        ace.config.set("modePath", aceBaseLocation);
        ace.config.set("workerPath", aceBaseLocation);
        ace.config.set("themePath", aceBaseLocation);

        //Init editor container
        editor = ace.edit($codeEditor.attr('id'));
        session = editor.getSession();

        //NOTE: warning in console ace.js:1 Automatically scrolling cursor into view after selection change this will be disabled in the next version set editor.$blockScrolling = Infinity to disable this message
        editor.$blockScrolling = Infinity;

        ace.config.loadModule("ace/ext/language_tools", function () {
            //Init basic autocomplete and snippets
            editor.setOptions({
                enableBasicAutocompletion: true,
                enableSnippets: true
            });

            //Init editor highlight
            editor.setTheme(EDITOR_THEME);
            session.setMode(EDITOR_MODE);

            //Delete info annotations
            session.on("changeAnnotation", function (e, sessionData) {
                var modifiedAnnotations = [],
                    hasChanged = false;

                if (sessionData.$annotations && sessionData.$annotations.length) {
                    for (var index = 0, length = sessionData.$annotations.length; index < length; index++) {
                        var annotation = sessionData.$annotations[index];

                        if (annotation.type !== 'info')
                            modifiedAnnotations.push(annotation);
                        else
                            hasChanged = true;
                    }

                    if (hasChanged)
                        session.setAnnotations(modifiedAnnotations);
                }
            });

            //Init wrap text
            session.setUseWrapMode(false);

            //Init fixture code
            editor.setValue(options.code, 1);

            //NOTE: ace editor bug. markup of editor  is yet not ready after call setValue
            //After call resize markup of editor is visible.
            editor.resize(true);

            editor.gotoLine(options.line);
            editor.scrollToLine(options.line - 1);

            //Set editor focus
            editor.focus();
        });

        var getCode = function () {
            return editor && session ? session.getValue() : null;
        };

        var destroyEditor = function () {
            if (editor) {
                session.$stopWorker();
                editor.destroy();
                $el.html('');
            }
        };

        var resizeEditor = function () {
            editor.resize();
        };

        return {
            getCode: getCode,
            destroyEditor: destroyEditor,
            resizeEditor: resizeEditor
        };
    };

    this.exports = codeEditor;
});