ControlPanel.define('Widgets.Dialog', function(require) {
    var $ = require('jQuery');

    // Consts
    var DIALOG_CLASS_NAME = 'dialog',
        DIALOG_HEADER_CLASS_NAME = 'dialog-header',
        DIALOG_CLOSE_BUTTON_CLASS_NAME = 'dialog-close-btn',
        DIALOG_CONTENT_CLASS_NAME = 'dialog-content',
        DIALOG_MODAL_OVERLAY_CLASS_NAME = 'dialog-modal-overlay',

        STORED_DATA_KEY = 'stored-data',
        STORED_TAB_INDEX_KEY = 'stored-tab-index';

    // Dialogs
    var getDlgInfo = function($el, options) {
            var dialogs = $.fn.testCafeDialogs.dialogs,
                dlgInfo = null,
                el = $el[0];

            $.each(dialogs, function() {
                if(this.elem === el)
                    dlgInfo = this.dlgInfo;
            });

            if(!dlgInfo) {
                dlgInfo = createDialog($el, options);

                dialogs.push({
                    elem : el,
                    dlgInfo : dlgInfo
                });
            }

            return dlgInfo;
        },

        createDialog = function($el, options) {
            var dlgInfo = {},
                dlgTitle = $el.attr('data-title');

            dlgInfo.dialogEl = $('<div>').appendTo($('body'));
            dlgInfo.headerEl = $('<div>').appendTo(dlgInfo.dialogEl);
            dlgInfo.contentEl = $el.detach();

            if(dlgTitle)
                dlgInfo.titleEl = $('<span>', { text : dlgTitle }).appendTo(dlgInfo.headerEl);

            dlgInfo.closeBtnEl = $('<div>', {title : 'close'}).appendTo(dlgInfo.headerEl);
            dlgInfo.dialogEl.append(dlgInfo.contentEl);
            dlgInfo.options = options;

            prepareDialog(dlgInfo);

            return dlgInfo;
        },

        prepareDialog = function(dlgInfo) {
            var options = dlgInfo.options;

            dlgInfo.dialogEl.addClass(DIALOG_CLASS_NAME);
            dlgInfo.headerEl.addClass(DIALOG_HEADER_CLASS_NAME);
            dlgInfo.closeBtnEl.addClass(DIALOG_CLOSE_BUTTON_CLASS_NAME);
            dlgInfo.contentEl.addClass(DIALOG_CONTENT_CLASS_NAME);

            if(options) {
                if(options.width)
                    dlgInfo.dialogEl.css( { width : options.width });
                if(options.dialogClass)
                    dlgInfo.dialogEl.addClass(options.dialogClass);
            }

            $(dlgInfo.closeBtnEl).click(function() {
                $(dlgInfo.contentEl).dialog('close');
            });
        },

        updateDlgPos = function() {
            var currDlgInfo = $.fn.testCafeDialogs.currDlgInfo;

            if(currDlgInfo) {
                var $dialogEl = currDlgInfo.dialogEl,
                    $doc = $(document),
                    $overlay = getOverlay(),
                    $window = $(window);

                $dialogEl.css({
                    top : Math.max(0, (($window.height() - $dialogEl.outerHeight(false)) / 2) + $window.scrollTop()),
                    left : Math.max(0, (($window.width() - $dialogEl.outerWidth(false)) / 2) + $window.scrollLeft())
                });

                $overlay.css({
                    width: $doc.width(),
                    height: $doc.height()
                });
            }
        },

        openDialog = function(dlgInfo) {
            if(dlgInfo !== $.fn.testCafeDialogs.currDlgInfo) {
                showOverlay();
                dlgInfo.dialogEl.css({ display : 'block' });
                $.fn.testCafeDialogs.currDlgInfo = dlgInfo;
                updateDlgPos();

                changeTabIndices();
                focusFirstInput(dlgInfo.contentEl);
            }
        },

        closeDialog = function(dlgInfo) {
            if(dlgInfo === $.fn.testCafeDialogs.currDlgInfo) {
                dlgInfo.dialogEl.css({ display : 'none' });
                hideOverlay();
                $.fn.testCafeDialogs.currDlgInfo = null;

                if(dlgInfo.options && dlgInfo.options.close)
                    dlgInfo.options.close();

                restoreTabIndices();
            }
        },

        destroy = function(dlgInfo) {
            var dialogs = $.fn.testCafeDialogs.dialogs;

            closeDialog(dlgInfo);

            dlgInfo.dialogEl.remove();

            for(var i = 0; i < dialogs.length; i++) {
                if(dialogs[i].dlgInfo === dlgInfo) {
                    dialogs.splice(i, 1);
                    return;
                }
            }
        },

        // Modal cover
        getOverlay = function() {
            $.fn.modalOverlay = $.fn.modalOverlay || createOverlay();

            return $.fn.modalOverlay;
        },

        createOverlay = function() {
            return $('<div>')
                .addClass(DIALOG_MODAL_OVERLAY_CLASS_NAME)
                .appendTo($('body'));
        },

        showOverlay = function() {
            var $html = $('html'),
                $overlay = getOverlay(),
                $window = $(window),
                storedWindowWidth = $window.width(),
                storedData = {
                    scrollLeft : $window.scrollLeft(),
                    scrollTop : $window.scrollTop(),
                    overflow : $html.css('overflow')
                };

            $html.css({ overflow : 'hidden' });
            $window.scrollLeft(storedData.scrollLeft).scrollTop(storedData.scrollTop);

            var widthDelta = $window.width() - storedWindowWidth;

            storedData.paddingRight = widthDelta > 0 ? $html.css('padding-right') : null;

            if(widthDelta > 0)
                $html.css({ 'padding-right' : widthDelta });

            $overlay.css({ display : 'block' });
            $html.data(STORED_DATA_KEY, storedData);
        },

        hideOverlay = function() {
            var $html = $('html'),
                $overlay = getOverlay(),
                storedData = $html.data(STORED_DATA_KEY);

            if(storedData.paddingRight)
                $html.css({ 'padding-right' : storedData.paddingRight });

            $html.css({ overflow : storedData.overflow});
            $(window).scrollLeft(storedData.scrollLeft).scrollTop(storedData.scrollTop);
            $overlay.css({ display : 'none' });
        },

        // Tab indeces

        changeTabIndices = function() {
            var currDlgInfo = $.fn.testCafeDialogs.currDlgInfo;

            $('*').each(function() {
                if(typeof this.focus === 'function') {
                    var $this = $(this),
                        isCurrDlgChild = currDlgInfo.dialogEl.has($this).length > 0;

                    if(!isCurrDlgChild) {
                        $this.data(STORED_TAB_INDEX_KEY, $this.attr('tabindex'));
                        $this.attr('tabindex', -1);
                    }
                }
            });
        },

        focusFirstInput = function($contentEl) {
            var $inputs = $contentEl.find('input, button');

            if($inputs.length)
                $inputs[0].focus();
        },

        restoreTabIndices = function() {
            $('[tabindex="-1"]').each(function() {
                var $this = $(this),
                    storedTabIndex = $this.data(STORED_TAB_INDEX_KEY);

                if(storedTabIndex)
                    $this.attr('tabindex', storedTabIndex);
                else
                    $this.removeAttr('tabindex');
            });
        },

        dialog = function(options){
            if(this.length > 1) {
                this.each(function() {
                    $(this).dialog(options);
                });
            } else {
                var dlgInfo = getDlgInfo(this, options);

                if(!options)
                    openDialog(dlgInfo);
                else if(typeof options === 'string') {
                    if(options === 'open')
                        openDialog(dlgInfo);
                    else if(options === 'close')
                        closeDialog(dlgInfo);
                    else if(options === 'destroy')
                        destroy(dlgInfo);
                } else {
                    if(options.autoOpen || typeof options.autoOpen === 'undefined')
                        openDialog(dlgInfo);
                }
            }

            return this;
        },

        keyupHandler = function(e) {
            var currDlgInfo = $.fn.testCafeDialogs.currDlgInfo;

            if(currDlgInfo && !currDlgInfo.options.disableKeysHandler) {
                if (e.keyCode === 27) // Esc
                    currDlgInfo.contentEl.dialog('close');
            }
        },

        scroll = function(e) {
            updateDlgPos();

            if($.fn.testCafeDialogs.currDlgInfo)
                return e.preventDefault();
        };

    $.fn.testCafeDialogs = {
        dialogs : [],
        currDlgInfo : null
    };

    $.fn.dialog = dialog;

    $(document).keyup(keyupHandler);

    $(window).bind({
        resize : updateDlgPos,
        scroll : scroll
    });
});