ControlPanel.define('Widgets.FixtureEditor', function (require) {
    require('Widgets.Dialog');
    require('Widgets.Draggable');
    require('Widgets.Resizable');

    var $ = require('jQuery'),
        CodeEditor = require('Widgets.CodeEditor'),
        Util = require('Util'),
        UXLog = require('UXLog');

    var CODE_EDITOR_DIALOG = '#code-editor-dialog',
        CODE_EDITOR_CONT = '#code-editor-cont',
        CODE_EDITOR_CANCEL_BTN = '#cancel-fixture-code-btn',
        UI_DIALOG_TITLE_CONT = '.dialog-header',
        CODE_EDITOR_SAVE_BTN = '#save-fixture-code-btn',
        UI_DIALOG_CLOSE_BTN = '.dialog-close-btn',
        RESIZE_CODE_EDITOR_DIALOG_CLASS = 'resize-code-editor',

        CODE_EDITOR_CLASS = 'code-editor',

        GET_FIXTURE_CODE_URL = '/get_fixture_code/',
        SAVE_FIXTURE_CODE_URL = '/save_fixture_code/',

        FIXTURE_CODE_PATTERN_TEXT = '^.*[\'"]@test[\'"]\\[[\'"]{0}[\'"]\\].*$',

        CODE_EDITOR_WIDTH = 980,
        CODE_EDITOR_HEIGHT = 500;

    var saveFixtureCode = function (options, curPath, fixtureFile, code) {
        options.onSaveClick();

        $.post(SAVE_FIXTURE_CODE_URL,
            {
                curPath: curPath,
                filename: fixtureFile,
                code: code
            },
            function () {
                options.onSaveSuccess(fixtureFile);
            }).error(function (data) {
                options.onSaveFail(data);
            }
        );
    };

    var findIndexOfStrAfterLineIndex = function(str, text, lineNumber) {
        var findStr = new RegExp(Util.prepareTextForRegExp(str), 'gm'),
            matchArr = text.match(findStr),
            lines = text.split('\n');

        for(var index = 0, length = matchArr.length; index < length; index++) {
            for(var strIndex = lineNumber, strLength = lines.length; strIndex < strLength; strIndex++) {
                if(lines[strIndex].indexOf(matchArr[index]) > -1)
                    return strIndex + 1;
            }
        }
    };

    this.exports.init = function (options) {
        var $cont = $(CODE_EDITOR_DIALOG),
            $codeEditorCont = $(CODE_EDITOR_CONT),
            curPath = null,
            editorApi = null,
            fixtureFile = '',
            $codeEditorDialog = $cont.dialog({
                autoOpen: false,
                dialogClass: CODE_EDITOR_CLASS,
                width: 'auto',
                disableKeysHandler: true
            });

        $(document).on('click', options.openBtn, function (e) {
            var $btn = $(this),
                curTestName = options.getCurrentTestName($btn),
                curStep = options.hasOwnProperty('getCurrentStep') ? options.getCurrentStep($btn) : '',
                curAssert = options.hasOwnProperty('getCurrentAssert') ? options.getCurrentAssert($btn) : '';

            curPath = options.getCurrentPath($btn);
            fixtureFile = options.getFileName($btn);

            UXLog.write('Project: Fixture code editor opened');

            $.post(GET_FIXTURE_CODE_URL, {
                curPath: curPath,
                fixtureFile: fixtureFile
            },function (fixtureCode) {
                $codeEditorDialog.dialog('open');

                //NOTE: fixtureCodeMod used only for finding a line number for testname and contains modified source code
                // convert new line symbol \r\n -> \n
                // remove escaping for quotes. It will not cause a conflict the test names,
                // because for code parser test name 'test\'' is equivalent "test'"
                // cut comment /*...*/, it need if testname string contains comment
                var fixtureCodeMod = fixtureCode.replace(/\r/g, '')
                        .replace(/\\"/g, '"')
                        .replace(/\\'/g, '\'')
                        .replace(/\/\*.*?\*\//g, ''),
                    lineNumber = 1;

                //NOTE: escaping symbols used in regexp.
                curTestName = Util.prepareTextForRegExp(curTestName);

                if (curTestName) {
                    try {
                        var findLineExp = new RegExp(Util.formatText(FIXTURE_CODE_PATTERN_TEXT, curTestName), 'gm'),
                            lineCode = fixtureCodeMod.match(findLineExp)[0];

                        lineNumber = fixtureCodeMod.split('\n').indexOf(lineCode) + 1;

                        if(curStep) {
                            var stepLine = findIndexOfStrAfterLineIndex(curStep, fixtureCodeMod, lineNumber);
                            lineNumber = stepLine ? stepLine : lineNumber;

                            if(curAssert) {
                                var curAssertArr = curAssert.split('\n');

                                curAssert = curAssertArr.length > 1 ? curAssertArr[0].replace(/\r/g, '') : curAssert;

                                var assertLine = findIndexOfStrAfterLineIndex(curAssert, fixtureCodeMod, lineNumber);
                                lineNumber = assertLine ? assertLine : lineNumber;
                            }
                        }

                    } catch (e) {
                        //NOTE: fallback if test name wasn't found. Open editor on first line.
                    }
                }

                editorApi = CodeEditor($codeEditorCont, {
                    code: fixtureCode,
                    line: lineNumber
                });
            }).error(function (res) {
                    $codeEditorDialog.dialog('close');
                    options.onOpenFail(res);
                });

            e.stopImmediatePropagation();
        });

        $(CODE_EDITOR_CONT).css({
            width: CODE_EDITOR_WIDTH,
            height: CODE_EDITOR_HEIGHT
        });

        $cont.parent().find(UI_DIALOG_TITLE_CONT).draggable();
        $(CODE_EDITOR_CONT).resizable({
            minWidth: CODE_EDITOR_WIDTH,
            minHeight: CODE_EDITOR_HEIGHT,
            resizeMarkerClass: RESIZE_CODE_EDITOR_DIALOG_CLASS,
            action: function () {
                editorApi.resizeEditor();
            }
        });

        $(document).on('click', CODE_EDITOR_CANCEL_BTN, function () {
            $codeEditorDialog.dialog('close');
            editorApi.destroyEditor();
        });

        $codeEditorDialog.find(UI_DIALOG_CLOSE_BTN).click(function () {
            editorApi.destroyEditor();
        });

        $(document).on('click', CODE_EDITOR_SAVE_BTN, function () {
            var fixtureCode = editorApi.getCode();

            saveFixtureCode(options, curPath, fixtureFile, fixtureCode);
            $codeEditorDialog.dialog('close');
            editorApi.destroyEditor();
        });
    };
});