ControlPanel.define('Widgets.NotificationsList', function(require) {
    var $ = require('jQuery'),
        Util = require('Util'),
        UXLog = require('UXLog');

    var
        //Classes
        MSGS_CONT_FIXED_CLASS = 'fixed',
        ERR_FRAME_CLASS = 'err hidden',
        TITLE_CLASS = 'title',
        CLOSE_BUTTON_CLASS = 'close-button',
        TASK_RESULTS_CLASS = 'task-results',
        TASK_NAME_CLASS = 'task-name',
        HIDDEN_CLASS = 'hidden',

        //Metrics
        STATUS_FRAME_FADE_DURATION = 3000,
        STATUS_FRAME_FREEZE_DURATION = 5000,

        //Url
        TRACK_TASK_URL = '/results/',

        //Templates
        TASK_RESULTS_TMPL = '' +
            '<a href="' + TRACK_TASK_URL + '{0}" target="_blank"><div class="stats {1}">' +
            '<ul>' +
            '<li><p>{2}</p><p>Passed</p></li>' +
            '<li><p>{3}</p><p>Failed</p></li>' +
            '<li><p>{4}</p><p>Total</p></li>' +
            '</ul></div>' +
            '</a>';

    var NotificationsList = function(options) {
        this.statusContainerOffset = 0;
        this.el = options.selector;
        this.topOffset = options.topOffset;
        this.leftOffset = options.leftOffset;
        this.bottomOffset = options.bottomOffset;

        this._initNotificationList();
    };

    NotificationsList.prototype.showStatus = function (className, $content, link) {
        var nl = this,
            $container = $(this.el),
            $status = $('<div></div>')
                .addClass(className)
                .prependTo($container);

        var $title = !link ? $('<p></p>') : $('<a href="' + link + '" target="_blank"></a>');

        $title.addClass(TITLE_CLASS)
            .appendTo($status);

        if (link) {
            $title.html('<a></a>')
                .on('click', function () {
                    //TODO check uxlog
                    UXLog.write('Project: track task link clicked');
                });
        }

        $content.appendTo($title);
        $status.show();

        $status.effect('pulsate', { times: 3 }, 'fast', function () {
            nl._removeNotificationByTimeout($status);
        });
    };

    NotificationsList.prototype.showPermanentErrorMsg = function (text) {
        var $container = $(this.el);

        var $err = $('<div></div>')
            .addClass(ERR_FRAME_CLASS)
            .prependTo($container);

        var $errTitle = $('<p></p>')
            .addClass(TITLE_CLASS)
            .text(text)
            .appendTo($err);

        $('<span></span>')
            .addClass(CLOSE_BUTTON_CLASS)
            .appendTo($errTitle)
            .click(function () {
                $err.remove();
            });

        $err.effect('pulsate', {times: 3}, 'fast');
    };

    NotificationsList.prototype.showTaskCompleteMsg = function (data) {
        var nl = this;

        if (data.taskUid) {
            var $report = $('<div></div>').addClass(TASK_RESULTS_CLASS);

            $report.append(Util.formatText(TASK_RESULTS_TMPL, data.taskUid, data.status, data.passed, data.failed, data.total));

            $('<span></span>')
                .text(data.taskName)
                .addClass(TASK_NAME_CLASS)
                .prependTo($report.find('div').eq(0))
                .on('click', function () {
                    UXLog.write('Project: track completed task link clicked');
                });

            $report.prependTo($(nl.el));
            $report.effect('pulsate', { times: 3 }, 'fast', function () {
                nl._removeNotificationByTimeout($report);
            });
        }
    };

    NotificationsList.prototype.updateTopOffset = function () {
        var $msgCont = $(this.el);

        this.statusContainerOffset = $msgCont.offset().top - parseInt($msgCont.css('margin-top')) || 0;
    };

    NotificationsList.prototype.removeFixedState = function() {
        $(this.el).removeClass(MSGS_CONT_FIXED_CLASS).css('left', 'auto');
    };

    NotificationsList.prototype.updatePosition = function() {
        this._setPositionMsgsCont();
    };

    NotificationsList.prototype.clearList = function() {
        $(this.el).html('');
    };

    NotificationsList.prototype.hide = function() {
        $(this.el).addClass(HIDDEN_CLASS);
    };

    NotificationsList.prototype.show = function() {
        $(this.el).removeClass(HIDDEN_CLASS);
    };

    NotificationsList.prototype._removeNotificationByTimeout = function($msg) {
        var createTimeout = function () {
            return window.setTimeout(function () {
                $msg.fadeOut(STATUS_FRAME_FADE_DURATION, function () {
                    $msg.remove();
                });
            }, STATUS_FRAME_FREEZE_DURATION);
        };

        var timeout = createTimeout();

        $msg.hover(function () {
            window.clearTimeout(timeout);
            $msg.stop(true, false);
            $msg.css('opacity', '1');
        }, function () {
            timeout = createTimeout();
        });
    };

    NotificationsList.prototype._setPositionMsgsCont = function () {
        var $msgCont = $(this.el);

        if ($(document).scrollTop() + this.topOffset >= this.statusContainerOffset) {
            $msgCont.addClass(MSGS_CONT_FIXED_CLASS);
            $msgCont.css('left', this.leftOffset - $msgCont.width() - parseInt($msgCont.css('padding-right')));

            var maxAvailableStackSize = 0,
                curStackSize = 0,
                msgContMarginTop = parseInt($msgCont.css('margin-top')) || 0,
                $msgs = $msgCont.children('div');

            maxAvailableStackSize = $(window).height() - msgContMarginTop - this.topOffset - this.bottomOffset;

            $msgs.each(function (index) {
                var $el = $msgs.eq(index);

                curStackSize += $el.height() + parseInt($el.css('margin-top'));

                if (curStackSize > maxAvailableStackSize)
                    $el.hide();
                else
                    $el.show();
            });
        }
        else {
            $msgCont.removeClass(MSGS_CONT_FIXED_CLASS).css('left', 'auto');
            $msgCont.find('div').show();
        }
    };

    NotificationsList.prototype._initNotificationList = function() {
        var nl = this;

        nl.updateTopOffset();

        $(window).scroll(function () {
            nl._setPositionMsgsCont();
        });

        $(window).resize(function () {
            nl._setPositionMsgsCont();
        });
    };

    this.exports = function(options) {
        return new NotificationsList(options);
    };
});