ControlPanel.define('Widgets.Resizable', function(require) {
    var $ = require('jQuery');

    var DIALOG_CONTENT = '.dialog-content',
        RESIZE_MARKER_STYLE_CLASS = 'resize-marker';

    var Resizable = function($el, options) {
        this.$el = $el;
        this.isResize = false;
        this.mouseX = 0;
        this.mouseY = 0;

        this.settings = {
            minHeight: 0,
            minWidth: 0,
            action: null,
            resizeMarkerClass: RESIZE_MARKER_STYLE_CLASS
        };

        $.extend(this.settings, options);
        this._initResizable();
    };

    Resizable.prototype._createResizeMarker = function () {
        var $el = this.$el,
            settings = this.settings,
            resizeMarkerClass = this.settings.resizeMarkerClass,
            resizeMarkerSel = '.' + resizeMarkerClass,
            $existingResizeMarker = $(resizeMarkerSel);

        if($existingResizeMarker.length)
            $existingResizeMarker.remove();

        $el.closest(DIALOG_CONTENT).prepend('<div class="' + [resizeMarkerClass, RESIZE_MARKER_STYLE_CLASS].join(' ') + '"></div>');

        var $resizeMarker = $(resizeMarkerSel);

        $resizeMarker.css({
            position: 'absolute',
            cursor: 'nw-resize'
        });

        if(settings.minWidth && $el.width() < settings.minWidth)
            $el.css('width', settings.minWidth);

        if(settings.minHeight && $el.height() < settings.minHeight)
            $el.css('height', settings.minHeight);

        return $resizeMarker;
    };

    Resizable.prototype._resizeEl = function (x, y) {
        var $el = this.$el,
            settings = this.settings,
            widthDiff = x - this.mouseX,
            heightDiff = y - this.mouseY,
            elHeight = $el.height(),
            elWidth = $el.width(),
            newHeight = settings.minHeight,
            newWidth = settings.minWidth;

        if (elHeight + heightDiff > settings.minHeight) {
            newHeight = elHeight + heightDiff;
            this.mouseY = y;
        }

        if (widthDiff + elWidth > settings.minWidth) {
            this.mouseX = x;
            newWidth = widthDiff + elWidth;
        }

        $el.css({
            'height': newHeight + 'px',
            'width': newWidth + 'px'
        });

        if (typeof settings.action === 'function')
            settings.action();
    };

    Resizable.prototype._initResizable = function () {
        var resizeMarker = this._createResizeMarker(),
            resizable = this;

        resizeMarker.bind('mousedown', function (e) {
            resizable.isResize = true;
            resizable.mouseX = e.pageX;
            resizable.mouseY = e.pageY;

            e.preventDefault();
        });

        $(document).bind('mouseup', function () {
            resizable.isResize = false;
        });

        $(document).bind('mousemove', function (e) {
            if (resizable.isResize) {
                resizable._resizeEl(e.pageX, e.pageY);
                e.preventDefault();
            }
        });
    };

    $.fn.resizable = function(options) {
        return new Resizable(this, options);
    };
});