ControlPanel.define('Widgets.ScrollBar', function(require) {
    var $ = require('jQuery');

    var SCROLL_MIN_HEIGHT = 10,
        SCROLL_RIGHT_MARGIN = '5px',
        SCROLL_MIN_INNER_WIDTH = '5px',
        SCROLL_MIN_OUTER_WIDTH = '15px',
        SCROLL_BAR_COLOR = '#bebebe',
    //Classes
        SCROLL_BAR_CLASS = 'scrollbar';

    var onScrollDrag = function (e) {
        var diff = e.pageY - this.mouseYCoord,
            scrollTop = parseInt(this.$scrollBar.css('top')),
            scrollHeight = this.$scrollBar.outerHeight(false);

        if (diff > 0)
            scrollTop = Math.min(scrollTop + diff, this.elHeight - scrollHeight);
        else
            scrollTop = Math.max(scrollTop + diff, 0);

        var offset = Math.floor(scrollTop * this.contentHeight / this.elHeight);
        this.$el.scrollTop(offset);

        this.mouseYCoord = e.pageY;
        this.$scrollBar.css('top', scrollTop + 'px');
    };

    var onWheel = function (e) {
        var diff = e.originalEvent.wheelDelta || -e.originalEvent.deltaY * 8,
            elOffsetTop = this.$el.scrollTop(),
            scrollHeight = this.$scrollBar.outerHeight(false),
            scrollTop = parseInt(this.$scrollBar.css('top')),
            scrollBar = this;

        var getScrollOffset = function() {
            return Math.floor(scrollBar.$el.scrollTop() / scrollBar.contentHeight * scrollBar.elHeight);
        };

        if (diff > 0) {
            this.$el.scrollTop(Math.max(elOffsetTop - diff, 0));
            scrollTop = Math.max(getScrollOffset(), 0);
        } else {
            this.$el.scrollTop(Math.min(elOffsetTop - diff, this.contentHeight - this.elHeight));
            scrollTop = Math.min(getScrollOffset(), this.elHeight - scrollHeight);
        }

        this.$scrollBar.css('top', scrollTop + 'px');

        e.preventDefault();
        e.stopImmediatePropagation();
    };

    var ScrollBar = function($el, options) {
        this.$el = $el;
        this.contentHeight = 0;
        this.isScrollDragging = false;
        this.mouseYCoord = 0;
        this.elHeight = 0;
        this.$scrollBar = null;
        this.settings = {
            scrollTo: null,
            scrollClass: null
        };

        $.extend(this.settings, options);
        this._initScrollBar();
    };

    ScrollBar.prototype._getScrollHeight = function() {
        if (this.elHeight >= this.contentHeight)
            return 0;

        var height = Math.floor(this.elHeight / this.contentHeight * this.elHeight);

        return height > SCROLL_MIN_HEIGHT ? height : SCROLL_MIN_HEIGHT;
    };

    ScrollBar.prototype._getContentHeight = function() {
        return this.$el[0].scrollHeight - parseInt(this.$el.css('padding-top')) - parseInt(this.$el.css('padding-bottom'));
    };

    ScrollBar.prototype._isVisible = function($item) {
        var offset = $item.offset().top - this.$el.offset().top;

        return (offset > 0 && offset < this.elHeight);
    };

    ScrollBar.prototype._paintScrollBar = function () {
        var scrollBarClass = this.settings.scrollClass || SCROLL_BAR_CLASS,
            scrollBarSelector = '.' + scrollBarClass,
            $existsScrollBar = this.$el.parent().find(scrollBarSelector);

        if ($existsScrollBar.length)
             $existsScrollBar.remove();

        this.$el.before('<div class="' + scrollBarClass + '"></div>');

        var $scrollBar = $(scrollBarSelector),
            scrollHeight = this._getScrollHeight(),
            elScrollTop = this.$el.scrollTop(),
            top = Math.ceil(Math.min(elScrollTop / this.contentHeight * this.elHeight, this.elHeight - scrollHeight));

        $scrollBar.css({
            float: 'right',
            position: 'absolute',
            height: scrollHeight,
            width: SCROLL_MIN_OUTER_WIDTH,
            right: SCROLL_RIGHT_MARGIN,
            cursor: 'pointer',
            top: top + 'px'
        }).html('<div></div>');

        $scrollBar.find('div').css({
            height: scrollHeight,
            width: SCROLL_MIN_INNER_WIDTH,
            'background-color': SCROLL_BAR_COLOR
        });

        return $scrollBar;
    };

    ScrollBar.prototype._initScrollBar = function() {
        var scrollBar = this,
            $scrollTo = $(this.settings.scrollTo);

        this.$el.css({
            'overflow-y': 'hidden'
        });

        this.contentHeight = this._getContentHeight();
        this.elHeight = this.$el.height();

        if($scrollTo && $scrollTo.length && !this._isVisible($scrollTo)) {
            var scrollingOffset = $scrollTo.offset().top - this.$el.children().eq(0).offset().top;

            this.$el.scrollTop(scrollingOffset);
            this.settings.scrollTo = null;
        }

        this.$scrollBar = this._paintScrollBar();

        var wheelEvent = typeof document.onwheel !== 'undefined' ? 'wheel' : 'mousewheel';

        this.$el.off(wheelEvent);
        this.$el.bind(wheelEvent, function(e) {
            onWheel.call(scrollBar, e);
        });

        this.$scrollBar.off('mousedown');
        this.$scrollBar.bind('mousedown', function (e) {
            scrollBar.isScrollDragging = true;
            scrollBar.mouseYCoord = e.pageY;

            e.preventDefault();
        });

        $(document).bind('mouseup', function () {
            scrollBar.isScrollDragging = false;
        });

        $(document).bind('mousemove', function (e) {
            if (scrollBar.isScrollDragging) {
                onScrollDrag.call(scrollBar, e);
                e.preventDefault();
            }
        });
    };

    $.fn.scrollBar = function(options){
        return new ScrollBar(this, options);
    };
});