ControlPanel.define('Widgets.TreeView', function(require) {
    var $ = require('jQuery');

    //Consts
    var //Classes
        SELECTED_CLASS = 'selected',
        EXPANDED_CLASS = 'expanded',
        DIR_NAME_LABEL_CLASS = 'dir-name-label',
        EXPAND_BTN_OPENED_CLASS = 'expand-btn-opened',
        PRELOADER_CLASS = 'preloader',
        EXPAND_ICON_CLASS = 'expand-icon',
        FOLDER_ICON_CLASS = 'folder-icon',
        PROJECT_FOLDER_ICON_CLASS = 'project-folder-icon',
        FILE_ICON_CLASS = 'file-icon',
        FOLDER_CLASS = 'folder',
    //Selectors
        PRELOADER = '.preloader',
        EXPAND_ICON = '.expand-icon',
    //Attr
        PATH_ATTR = 'data-path',

        PATH_SEP = '/';

    var $document = $(document);

    var formatPath = function (path) {
        return path.replace(/\\/g, '/').replace(/\/{2}/g, '/');
    };

    var TreeView = function ($el, options) {
        this.$el = $el;
        this.settings = {
            url: null,
            open: null,
            effect: null,
            time: 0,
            asyncAction: null,
            selectAction: null
        };

        $.extend(this.settings, options);
        this._initTreeview();
    };

    TreeView.prototype._callAsyncAction = function () {
        if (typeof this.settings.asyncAction === 'function')
            this.settings.asyncAction();
    };

    TreeView.prototype._animateEl = function ($item, collapse, callback) {
        var time = this.settings.time,
            effect = this.settings.effect;

        if ($item.is(":visible")) {
            var $content = $item.children('ul'),
                height = $content.outerHeight(false);

            if (!collapse) {
                switch (effect) {
                    case 'animate':
                        $content.css({
                            height: 0,
                            opacity: 0,
                            overflow: 'hidden'
                        });

                        $content.animate({
                            height: height,
                            opacity: 1
                        }, time, function () {
                            $content.css('height', 'auto');
                            callback();
                        });

                        break;

                    case 'fadeIn':
                        $content.css('display', 'none');
                        $content.fadeIn(time, callback);

                        break;

                    default:
                        callback();
                        break;
                }
            }
            else {
                switch (effect) {
                    case 'animate':
                        $content.animate({
                            height: 0,
                            opacity: 0
                        }, time, callback);

                        break;

                    case 'fadeIn':
                        $content.fadeOut(time, callback);

                        break;

                    default:
                        callback();
                        break;
                }
            }
        }
        else
            callback();
    };

    TreeView.prototype._expandElm = function ($item, callback) {
        var treeview = this,
            url = treeview.settings.url,
            path = $item ? $item.attr(PATH_ATTR) : '';

        if (($item && $item.hasClass(EXPANDED_CLASS)) || (!$item && treeview.$el.find('li').length)) {
            if ($item)
                treeview._setSelected($item);

            callback();
            return;
        }

        var callbackFn = function () {
            treeview._callAsyncAction();
            callback();
        };

        if ($item)
            $item.append('<span class="' + PRELOADER_CLASS + '"></span>');

        $.get(url, {path: path}, function (itemsData) {
            var items = '';

            for (var i = 0, length = itemsData.length; i < length; i++) {
                if (itemsData[i].type === 'dir') {
                    items += '<li ' + PATH_ATTR + '=' + itemsData[i].id + ' class="' + FOLDER_CLASS + '">' +
                        '<span class=' + EXPAND_ICON_CLASS + '></span>';

                    if (itemsData[i].isProjectDir)
                        items += '<span class="' + PROJECT_FOLDER_ICON_CLASS + '"></span>';
                    else
                        items += '<span class="' + FOLDER_ICON_CLASS + '"></span>';

                    items += '<span class="' + DIR_NAME_LABEL_CLASS + '">' + itemsData[i].text + '</span></li>';
                } else {
                    items += '<li ' + PATH_ATTR + '=' + itemsData[i].id + '>' +
                        '<span></span>' +
                        '<span class="' + FILE_ICON_CLASS + '"></span>' +
                        '<span class="' + DIR_NAME_LABEL_CLASS + '">' + itemsData[i].text + '</span></li>';
                }
            }

            if ($item) {
                $item.children(PRELOADER).remove();
                items = '<ul>' + items + '</ul>';

                $item.children('span').addClass(EXPAND_BTN_OPENED_CLASS);
                $item.addClass(EXPANDED_CLASS).html($item.html() + items);

                treeview._setSelected($item);

                if (treeview.settings.effect)
                    treeview._animateEl($item, false, function () {
                        callbackFn();
                    });
                else
                    callbackFn();
            }
            else {
                treeview.$el.html(items);
                callbackFn();
            }
        });
    };

    TreeView.prototype._setSelected = function ($item) {
        if ($item && $item.length) {
            this.$el.find('li').removeClass(SELECTED_CLASS);
            $item.addClass(SELECTED_CLASS);
        }
    };

    TreeView.prototype._openNode = function (path) {
        var treeview = this,
            $el = treeview.$el,
            pathArr = formatPath(path).split(PATH_SEP),
            currPath = '',
            $currItem = null,
            tmpEffect = null;

        if (pathArr.length > 1 && !pathArr[pathArr.length - 1])
            pathArr.pop();

        var tmp = treeview.settings.asyncAction,
            effectTmp = treeview.settings.effect;

        treeview.settings.asyncAction = function () {
        };
        treeview.settings.effect = false;

        var expandNode = function () {
            if (pathArr.length) {
                var pathEl = pathArr.shift();

                if (currPath)
                    currPath += (currPath[currPath.length - 1] === PATH_SEP) ? pathEl : PATH_SEP + pathEl;
                else
                    currPath = pathEl + PATH_SEP;

                var encodePathDirectSlash = encodeURIComponent(currPath),
                    encodePathBackSlash = encodeURIComponent(currPath.replace(/\//g, '\\')),
                    $item = $el.find('li[' + PATH_ATTR + '="' + encodePathDirectSlash + '"]');

                if (!$item.length)
                    $item = $el.find('li[' + PATH_ATTR + '="' + encodePathBackSlash + '"]');

                //NOTE: live search support. Will find '\path\name_' in '\path\name_of_program'.
                if (!$item.length) {
                    var items = treeview.$el.find('li').map(function () {
                        return $(this).attr('data-path');
                    });

                    for (var index = 0, length = items.length; index < length; index++) {
                        if (items[index].indexOf(encodePathDirectSlash) === 0 ||
                            items[index].indexOf(encodePathBackSlash) === 0) {

                            $item = $el.find('li[' + PATH_ATTR + '="' + items[index] + '"]');

                            tmpEffect = treeview.settings.effect;
                            treeview.settings.effect = '';

                            break;
                        }
                    }
                }

                treeview._expandElm($item, function () {
                    if (tmpEffect)
                        treeview.settings.effect = tmpEffect;

                    expandNode();
                });

                $currItem = $item;

            } else {
                treeview.settings.asyncAction = tmp;
                treeview.settings.asyncAction();
                treeview.settings.effect = effectTmp;
            }
        };
        expandNode();
    };

    TreeView.prototype._initTreeview = function () {
        var treeview = this;

        treeview._expandElm(null, function () {
            if (treeview.settings.open)
                treeview._openNode(treeview.settings.open);
        });

        var expandHandler = function ($item, e) {
            if (treeview.settings.url && !$item.hasClass(EXPANDED_CLASS))
                treeview._expandElm($item, function () {
                });
            else {
                treeview._animateEl($item, true, function () {
                    $item.children('span').removeClass(EXPAND_BTN_OPENED_CLASS);
                    $item.removeClass(EXPANDED_CLASS).find('ul').remove();

                    treeview._callAsyncAction();
                });
            }

            e.stopPropagation();
        };

        $document.on('dblclick', treeview.$el.selector + ' li', function (e) {
            var $item = $(this);

            if ($item.hasClass(FOLDER_CLASS))
                expandHandler($item, e);

            e.stopImmediatePropagation();
        });

        $document.on('click', treeview.$el.selector + ' li', function (e) {
            treeview._setSelected($(this));

            if(treeview.settings.selectAction)
                treeview.settings.selectAction($(this));

            e.stopImmediatePropagation();
        });

        $document.on('click', treeview.$el.selector + ' ' + EXPAND_ICON, function (e) {
            var $item = $(this).parent();

            if ($item.hasClass(EXPANDED_CLASS)) {
                treeview._setSelected($item);
                $item.dblclick();
            }

            e.stopImmediatePropagation();
        });
    };

    $.fn.treeView = function (options) {
        return new TreeView(this, options);
    };
});