/* global io: true */

ControlPanel.define('WorkerIdle', function (require, exports) {
    var $ = require('jQuery');

    exports.init = function () {
        var
            IDLE_HEARTBEAT_DELAY = window.WebSocket ? 30000 : 2000,
            IDLE_HEARTBEAT_URL = '/worker/heartbeat',
            FORCE_WORKER_CLOSE_URL = '/worker/force-close',
        //Selector
            IDLE_MSG_CONT = '.idle-msg-wrap',
            IDLE_MSG = '.idle-msg',
        //Status messages
            WORKER_IS_UTILIZED = 'Worker has been utilized';

        window.startWorkerIdleHeartbeat = function (workerName) {
            var disconnectSocket = function () {
                if (window.WebSocket)
                    socket.disconnect();
            };

            var heartbeat = function () {
                var jqxhr = $.ajax({
                    type: 'POST',
                    url: IDLE_HEARTBEAT_URL,
                    data: { name: workerName }
                });

                jqxhr.done(function (res) {
                    if (res && res.status === 'run') {
                        //NOTE: we force socket disconnection here (and in all other cases when we change location)
                        //because seems like that there is a bug in socket.io in Firefox which lead to the following issue:
                        //We receive 'taskAdded' event, change page location, socket is disconnected and trying to reconnect
                        //again. On connection it performs heartbeat once again and all this leads to infinite loop
                        //(see B234283).
                        disconnectSocket();
                        location.href = decodeURI(res.testRunUrl);
                    } else if (res && res.status === 'utilized' && res.returnUrl) {
                        disconnectSocket();
                        location.href = (!res.lastTask || !/project/.test(res.returnUrl)) ?
                            res.returnUrl : [res.returnUrl, '?taskUid=', res.lastTask].join('');
                    } else if (res && res.status === 'utilized') {
                        disconnectSocket();
                        //NOTE: Try to close browser from js code first
                        window.open('', '_self', '');
                        window.close();

                        //FATALITY: if we fail to close browser window using previous methods, setup targetWindowMark as
                        //window.title, so it will be identified and killed by Windows taskkill
                        document.title = res.targetWindowMark;

                        $.ajax({
                            type: 'POST',
                            url: FORCE_WORKER_CLOSE_URL,
                            data: {targetWindowMark: res.targetWindowMark}
                        });

                        //NOTE: If we fail to close window then just show info message
                        $(IDLE_MSG).find('h2').text(WORKER_IS_UTILIZED).addClass('utilizedMsg');
                        $(IDLE_MSG).find('p').remove();
                    } else {
                        window.setTimeout(function () {
                            heartbeat();
                        }, IDLE_HEARTBEAT_DELAY);
                    }
                });
            };

            //NOTE: if WebSockets is not supported, use heartbeat with small interval
            if (window.WebSocket) {
                var socket = io.connect(window.location.protocol + '//' + window.location.host);

                socket.on('connect', function () {
                    socket.emit('registerWorker', workerName);
                    heartbeat();
                });

                socket.on('taskAdded', function () {
                    heartbeat();
                });
            } else
                heartbeat();
        };

        var setWaitMsgMargin = function () {
            var $idleMsg = $(IDLE_MSG),
                $idleMsgCont = $(IDLE_MSG_CONT);

            var topMargin = Math.ceil(($idleMsgCont.height() - $idleMsg.outerHeight(false)) / 2);

            if (topMargin > 0)
                $idleMsg.css('margin-top', topMargin);
        };

        $(document).ready(function () {
            setWaitMsgMargin();
        });

        $(window).resize(function () {
            setWaitMsgMargin();
        });
    };
});