/* global io: true */
/* global QRCode: true */

ControlPanel.define('WorkersView', function (require, exports) {
    var $ = require('jQuery'),
        Util = require('Util'),
        UXLog = require('UXLog');

    exports.init = function () {

        var
        //Selector
            HEADER_BACK = '.header .back',
            FOOTER = '.footer',
            ADD_WORKER_FORM_CONT = '.add-worker-form-wrap',
            NO_WORKERS = '.no_workers',
            ADD_WORKER_BTN = '#add-worker-btn',
            ADD_WORKER_ERR = '#add-worker-err',
            WORKER_NAME = '#worker-name',
            WORKERS_CONTAINER = '.workers-cont',
            NAME_SORT_BTN = '.name-sort',
            TYPE_SORT_BTN = '.type-sort',
            ERR_TITLE = '.title',
            WORKER_ROW = '.workers-dashboard > table tbody tr',
            WORKER_TBODY = '.workers-dashboard > table tbody',
            WORKER_NAME_VALUE = '.worker-name',
            WORKER_TYPE_VALUE = '.worker-type p',
            CURRENT_TASK_LINK = '.workers-dashboard .task-name',
            CURRENT_TEST_LINK = '.workers-dashboard .current-test',
            WORKER_QR_CODE = '#worker-qr-code',
        //Classes
            SORTING_ASC_ICON_CLASS = 'ascSort',
            SORTING_DESC_ICON_CLASS = 'descSort',
        //ID
            WORKER_QR_CODE_ID = 'worker-qr-code',
        //Url
            ADD_WORKER_URL = '/worker/add/',
            REVISION_REQ_URL = '/workers/',
            NO_REDIRECT_QUERY_PARAM = '?noRedirect=1',
            WORKER_REVISION_PARAM = '?workersUpdate=1',
            CREATE_WORKER_BY_QR_CODE_URL = '/worker/add/qrcode?create_by_qr_code=1',
        //String
            ERR_EMPTY_WORKER_NAME = 'Worker name is empty. The name should contain at least one character which cannot be a blank space.',
        //Metrics
            QR_CODE_HEIGHT = 80,
            QR_CODE_WIDTH = 80;

        var isNameSortedAsc = true,
            isTypeSortedAsc = true,
            lastGridSortResult = null,
            socketDisconnected = false;

        var sortGrid = function (sortType, asc) {
            var data = [],
                sortResult = '';

            $(WORKER_ROW).each(function (i, row) {
                data.push({
                    key: $(row).find(sortType).text(),
                    html: row.outerHTML
                });
            });

            data = data.sort(function (descr1, descr2) {
                if (descr1.key < descr2.key)
                    return -1;
                if (descr1.key > descr2.key)
                    return 1;

                return 0;
            });

            if (!asc)
                data = data.reverse();

            for (var i = 0; i < data.length; i++)
                sortResult += data[i].html;

            $(WORKER_TBODY).html(sortResult);
        };

        var scrollToChangedWorker = function () {
            if (window.WORKER) {
                var workerRow = $('tr[data-workername="' + window.WORKER + '"]');

                if (workerRow) {
                    Util.scrollTo(workerRow);
                    window.WORKER = '';
                }
            }
        };

        var initNoWorkersMsg = function () {
            var setNoWorkersMsgMargin = function () {
                var $noWorkersMsg = $(NO_WORKERS),
                    topMargin = Math.ceil((Util.getContentHeight() - $(ADD_WORKER_FORM_CONT).height() - $noWorkersMsg.outerHeight(false)) / 2);

                if (topMargin > 0)
                    $noWorkersMsg.css('margin-top', topMargin);
            };

            if ($(NO_WORKERS))
                setNoWorkersMsgMargin();

            $(window).resize(function () {
                if ($(NO_WORKERS))
                    setNoWorkersMsgMargin();
            });
        };

        var initQRCode = function () {
            new QRCode(WORKER_QR_CODE_ID, {
                text: [window.HOSTNAME, CREATE_WORKER_BY_QR_CODE_URL].join(''),
                width: QR_CODE_WIDTH,
                height: QR_CODE_HEIGHT,
                correctLevel: QRCode.CorrectLevel.L
            });

            //NOTE: qrcodejs haven't options for disabling the code title
            $(WORKER_QR_CODE).removeAttr('title');
        };

        var initContent = function () {
            initNoWorkersMsg();
            scrollToChangedWorker();

            $(CURRENT_TASK_LINK).on('click', function () {
                UXLog.write('Workers: current task link clicked');
            });

            $(CURRENT_TEST_LINK).on('click', function () {
                UXLog.write('Workers: current test link clicked');
            });

            $(ADD_WORKER_BTN).click(function () {
                var $err = $(ADD_WORKER_ERR),
                    $errTitle = $err.find(ERR_TITLE),
                    workerName = $(WORKER_NAME).val();

                UXLog.write('Workers: worker connected');

                $err.hide();

                if (!workerName) {
                    $errTitle.text(ERR_EMPTY_WORKER_NAME);
                    $err.show('fast');

                    return false;
                }

                $.ajax({url: [ADD_WORKER_URL, encodeURIComponent(workerName), NO_REDIRECT_QUERY_PARAM].join('')})
                    .done(function (res) {
                        window.location.href = res;
                    })
                    .fail(function (jqxhr) {
                        $errTitle.text(jqxhr.responseText);
                        $err.show('fast');
                    });

                return false;
            });

            $(NAME_SORT_BTN).click(function () {
                $(this).addClass(isNameSortedAsc ? SORTING_ASC_ICON_CLASS : SORTING_DESC_ICON_CLASS)
                    .removeClass(!isNameSortedAsc ? SORTING_ASC_ICON_CLASS : SORTING_DESC_ICON_CLASS);
                $(TYPE_SORT_BTN).removeClass(!isTypeSortedAsc ? SORTING_ASC_ICON_CLASS : SORTING_DESC_ICON_CLASS);

                UXLog.write('Workers: Sorting by name used');

                sortGrid(WORKER_NAME_VALUE, isNameSortedAsc);
                lastGridSortResult = {field: WORKER_NAME_VALUE, asc: isNameSortedAsc, sortBtn: NAME_SORT_BTN};
                isNameSortedAsc = !isNameSortedAsc;
            });

            $(TYPE_SORT_BTN).click(function () {
                $(this).addClass(isTypeSortedAsc ? SORTING_ASC_ICON_CLASS : SORTING_DESC_ICON_CLASS)
                    .removeClass(!isTypeSortedAsc ? SORTING_ASC_ICON_CLASS : SORTING_DESC_ICON_CLASS);
                $(NAME_SORT_BTN).removeClass(!isNameSortedAsc ? SORTING_ASC_ICON_CLASS : SORTING_DESC_ICON_CLASS);

                UXLog.write('Workers: Sorting by type used');

                sortGrid(WORKER_TYPE_VALUE, isTypeSortedAsc);
                lastGridSortResult = {field: WORKER_TYPE_VALUE, asc: isTypeSortedAsc, sortBtn: TYPE_SORT_BTN};
                isTypeSortedAsc = !isTypeSortedAsc;
            });
        };

        var updateState = function () {
            var queryPath = REVISION_REQ_URL + WORKER_REVISION_PARAM;

            $.get(queryPath, function (data) {
                var scrollTop = $(window).scrollTop();

                $(WORKERS_CONTAINER).html(data);

                $('html, body').animate({scrollTop: scrollTop}, 'fast');

                if (lastGridSortResult) {
                    sortGrid(lastGridSortResult.field, lastGridSortResult.asc);
                    $(lastGridSortResult.sortBtn).addClass(lastGridSortResult.asc ? SORTING_ASC_ICON_CLASS : SORTING_DESC_ICON_CLASS);
                }

                initContent();
            });
        };

        var initSocketEvent = function () {
            var socket = io.connect(window.location.protocol + '//' + window.location.host);

            socket.on('connect', function () {
                socket.emit('registerSeveralWorkers');

                if (socketDisconnected)
                    updateState();
            });

            socket.on('taskAdded', function () {
                updateState();
            });

            socket.on('taskUpdated', function () {
                updateState();
            });

            socket.on('workerAdded', function () {
                updateState();
            });

            socket.on('workerDisconnected', function () {
                updateState();
            });

            socket.on('disconnect', function () {
                socketDisconnected = true;
            });
        };

        var initPageEffect = function () {
            $(WORKERS_CONTAINER).fadeIn();
        };

        var initHorizontalScrollHandler = function () {
            $(window).scroll(function () {
                var scrollLeft = $(this).scrollLeft();

                $(HEADER_BACK).css('left', -scrollLeft);
                $(ADD_WORKER_FORM_CONT).css('left', -scrollLeft);
                $(FOOTER).css('left', -scrollLeft);
            });
        };

        $(document).ready(function () {
            UXLog.write('Workers tab opened');

            initHorizontalScrollHandler();
            initPageEffect();
            initSocketEvent();
            initQRCode();
            initContent();
        });
    };
});