HammerheadClient.define('DOMSandbox.ActiveWindowTracker', function (require, exports) {
    var EventSandbox = require('DOMSandbox.Event'),
        MessageSandbox = require('DOMSandbox.Message');

    // Consts
    var WINDOW_ACTIVATED_EVENT = 'windowActivated',
        WINDOW_DEACTIVATED_EVENT = 'windowDeactivated';

    var currentWindow = null,
        isIFrameWindow = false,
        activeWindow = null,
        isActive = false;

    function notifyPrevActiveWindow () {
        try {
            if (activeWindow.top && activeWindow !== activeWindow.top) {
                MessageSandbox.sendServiceMsg({
                    cmd: WINDOW_DEACTIVATED_EVENT
                }, activeWindow);
            }
        }
        catch (err) {
            //NOTE: the error appears in IE when the corresponding iframe is removed
            return void 0;
        }
    }

    function setActiveState (activeState) {
        isActive = activeState;
        EventSandbox.setCurrentWindowActiveState(activeState);
    }

    exports.init = function (win) {
        currentWindow = win;
        isIFrameWindow = currentWindow !== currentWindow.top;
        activeWindow = !isIFrameWindow ? currentWindow.top : null;
        setActiveState(!isIFrameWindow);

        MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, function (e) {
            if (e.message.cmd === WINDOW_ACTIVATED_EVENT) {
                notifyPrevActiveWindow();

                setActiveState(false);
                activeWindow = e.source;
            }
            else if (e.message.cmd === WINDOW_DEACTIVATED_EVENT)
                setActiveState(false);
        });

        EventSandbox.on(EventSandbox.WINDOW_ACTIVATED_EVENT, function () {
            setActiveState(true);

            if (!isIFrameWindow) {
                notifyPrevActiveWindow();

                activeWindow = currentWindow;
            }
            else {
                MessageSandbox.sendServiceMsg({
                    cmd: WINDOW_ACTIVATED_EVENT
                }, currentWindow.top);
            }
        });
    };

    exports.isCurrentWindowActive = function () {
        return isActive;
    };
});