/* global initHammerheadClient:true, isIFrameWithoutSrc:true */

HammerheadClient.define('DOMSandbox', function (require, exports) {
    var $ = require('jQuery'),
        DomAccessorWrappers = require('DOMSandbox.DomAccessorWrappers'),
        EventSandbox = require('DOMSandbox.Event'),
        ActiveWindowTracker = require('DOMSandbox.ActiveWindowTracker'),
        HiddenInfo = require('DOMSandbox.Upload.HiddenInfo'),
        IFrameSandbox = require('DOMSandbox.IFrame'),
        InfoManager = require('DOMSandbox.Upload.InfoManager'),
        JSProcessor = require('Shared.JSProcessor'),
        MessageSandbox = require('DOMSandbox.Message'),
        NativeMethods = require('DOMSandbox.NativeMethods'),
        PageProc = require('Shared.PageProc'),
        SandboxedJQuery = require('SandboxedJQuery'),
        SandboxUtil = require('DOMSandbox.Util'),
        ShadowUI = require('DOMSandbox.ShadowUI'),
        SharedConst = require('Shared.Const'),
        Transport = require('Transport'),
        UploadSandbox = require('DOMSandbox.Upload'),
        UrlUtil = require('UrlUtil'),
        Util = require('Util'),
        XhrSandbox = require('DOMSandbox.Xhr');

    // Consts
    var BEFORE_DOCUMENT_CLEANED = 'beforeDocumentCleaned',
        BODY_CONTENT_CHANGED = 'bodyContentChanged',
        BODY_CREATED = 'bodyCreated',
        DOCUMENT_CLEANED = 'documentCleaned',
        DOCUMENT_CLOSED = 'documentClosed',
        IFRAME_ADDED = 'iframeAdded',
        UNCAUGHT_JS_ERROR = 'uncaughtJSError';

    var IFRAME_DOM_SANDBOXES_STORE = 'dom_sandboxes_store_5d9138e9';

    var ALLOWED_SERVICE_WORKER_PROTOCOLS  = ['https:', 'wss:', 'file:'];
    var ALLOWED_SERVICE_WORKER_HOST_NAMES = ['localhost', '127.0.0.1'];

    exports.BEFORE_DOCUMENT_CLEARED = BEFORE_DOCUMENT_CLEANED;
    exports.BODY_CREATED = BODY_CREATED;
    exports.DOCUMENT_CLEARED = DOCUMENT_CLEANED;
    exports.DOCUMENT_CLOSED = DOCUMENT_CLOSED;
    exports.IFRAME_ADDED = IFRAME_ADDED;
    exports.UNCAUGHT_JS_ERROR = UNCAUGHT_JS_ERROR;

    var eventEmitter = new Util.EventEmitter();

    exports.on = eventEmitter.on.bind(eventEmitter);
    exports.off = eventEmitter.off.bind(eventEmitter);

    function onIFrameAddedToDOM(iframe) {
        if (!Util.isCrossDomainIframe(iframe, true)) {
            eventEmitter.emit(IFRAME_ADDED, {
                iframe: iframe
            });

            IFrameSandbox.iframeAddedToDom(iframe);
        }
    }

    function onBodyContentChanged() {
        ShadowUI.onBodyContentChanged();
    }

    function onBodyCreated() {
        EventSandbox.initDocumentBodyListening(document);
        eventEmitter.emit(BODY_CREATED, {
            body: document.body
        });
    }

    function onBodyElementMutation() {
        ShadowUI.onBodyElementMutation();
    }

    IFrameSandbox.on(IFrameSandbox.IFRAME_READY_TO_INIT_INTERNAL, function (e) {
        // Eval Hammerhead code script
        initHammerheadClient(e.iframe.contentWindow, true);
    });

    IFrameSandbox.on(IFrameSandbox.IFRAME_DOCUMENT_CREATED, function (e) {
        // Override only document (In fact, we only need 'write' and 'writeln' methods)
        overrideDocument(e.iframe.contentWindow, e.iframe.contentDocument);
    });

    IFrameSandbox.on(IFrameSandbox.IFRAME_DOCUMENT_RECREATED, function (e) {
        // We should informs iframe DomSandbox so that it restore communication with the recreated document
        exports.rebindDomSandboxToIframe(e.iframe);
    });

    function overrideElementDomMethods(el, window) {
        if (el[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT] !== window) {
            el[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT] = window;

            overrideElement(el, true);
            EventSandbox.overrideElement(el, true);
            ShadowUI.overrideElement(el, true);
        }
    }

    exports.raiseBodyCreatedEvent = onBodyCreated;

    exports.rebind = function (iframe) {
        // Assign exists DomSandbox to cleared document
        onDocumentCleaned(iframe.contentWindow, iframe.contentDocument);
    };

    exports.rebindDomSandboxToIframe = function (iframe) {
        if (iframe) {
            var topSameDomainWindow = Util.getTopSameDomainWindow(window),
                domSandboxesStore = topSameDomainWindow[IFRAME_DOM_SANDBOXES_STORE];

            // Find iframe DomSandbox
            for (var i = 0; i < domSandboxesStore.length; i++) {
                if (domSandboxesStore[i].iframe === iframe) {
                    // Inform the DomSandbox so that it restore communication with the recreated document
                    domSandboxesStore[i].domSandbox.rebind(iframe);

                    return;
                }
            }

            // If the iframe DomSandbox is not found, this means that iframe not initialized,
            // in this case we should inject Hammerhead

            // Hack: IE10 clean up overrided methods after document.write calling
            NativeMethods.restoreNativeDocumentMeth(iframe.contentDocument);

            // DomSandbox for this iframe not found (iframe not yet initialized).
            // Inform the IFrameSandbox about it, and it inject Hammerhead
            IFrameSandbox.onIframeBeganToRun(iframe);
        }
    };

    exports.overrideDomMethods = function (el, doc, win) {
        doc = doc || document;
        win = win || window;

        if (!el) {
            EventSandbox.overrideElement(doc);

            if(doc.documentElement)
                exports.overrideDomMethods(doc.documentElement);
        } else {
            //OPTIMIZATION: use querySelectorAll to iterate over descendant nodes
            if (el.querySelectorAll) {
                overrideElementDomMethods(el, win);

                var children = el.querySelectorAll('*');

                for (var i = 0; i < children.length; i++)
                    overrideElementDomMethods(children[i], win);
            }

            //NOTE: if querySelectorAll is not available fallback to recursive algorithm
            else if (el.nodeType === 1 || el.nodeType === 11) {
                overrideElementDomMethods(el, win);

                var cnLength = el.childNodes.length;

                if (cnLength) {
                    for (var j = 0; j < cnLength; j++)
                        exports.overrideDomMethods(el.childNodes[j], doc, win);
                }
            }
        }
    };

    // Overrided methods
    function getIframes(el) {
        if (el.tagName && el.tagName.toLowerCase() === 'iframe')
            return [el];
        else
            return el.querySelectorAll('iframe');
    }

    function onElementAdded(el) {
        if ((el.nodeType === 1 || el.nodeType === 9) && Util.isElementInDocument(el)) {
            var iframes = getIframes(el);

            if (iframes.length) {
                for (var i = 0; i < iframes.length; i++)
                    onIFrameAddedToDOM(iframes[i]);
            } else if (el.tagName && el.tagName.toLowerCase() === 'body')
                onBodyElementMutation();
        }

        var $el = $(el);

        $el.add($el.find('*')).each(function () {
            if (Util.isFileInput(this))
                HiddenInfo.addInputInfo(this, InfoManager.getFiles(this), InfoManager.getValue(this));
        });
    }

    function onElementRemoved(el) {
        if (el.nodeType === 1 && el.tagName && el.tagName.toLowerCase() === 'body')
            onBodyElementMutation();
    }

    function overridedInsertBefore(newNode, refNode) {
        newNode = prepareNodeForInsertion(newNode, this);

        exports.overrideDomMethods(newNode);

        var result = NativeMethods.insertBefore.call(this, newNode, refNode);

        onElementAdded(newNode);

        return result;
    }

    function overridedAppendChild(child) {
        child = prepareNodeForInsertion(child, this);

        exports.overrideDomMethods(child);

        var result = NativeMethods.appendChild.call(this, child);

        onElementAdded(child);

        return result;
    }

    function overridedRemoveChild(child) {
        var $el = $(child);

        $el.add($el.find('*')).each(function () {
            if (Util.isFileInput(this))
                HiddenInfo.removeInputInfo(this);
        });

        var result = NativeMethods.removeChild.call(this, child);

        onElementRemoved(child);

        return result;
    }

    function overridedCloneNode() {
        var clone = NativeMethods.cloneNode.apply(this, arguments);

        exports.overrideDomMethods(clone);

        return clone;
    }

    function overridedGetAttribute(attr) {
        return overridedGetAttributeCore(this, attr);
    }

    function overridedGetAttributeNS(ns, attr) {
        return overridedGetAttributeCore(this, attr, ns);
    }

    function overridedGetAttributeCore(el, attr, ns) {
        var getAttrMeth = ns ? NativeMethods.getAttributeNS : NativeMethods.getAttribute;

        // Optimization: hasAttribute meth is very slow
        if (isUrlAttr(el, attr) || attr === 'sandbox' || PageProc.EVENTS.indexOf(attr) !== -1 ||
            attr === 'autocomplete') {
            var storedAttr = PageProc.getStoredAttrName(attr);

            if (attr === 'autocomplete' && getAttrMeth.apply(el, ns ? [ns, storedAttr] : [storedAttr]) === 'none')
                return null;
            else if (el.hasAttribute(storedAttr))
                attr = storedAttr;
        }

        return getAttrMeth.apply(el, ns ? [ns, attr] : [attr]);
    }

    function overridedSetAttribute(attr, value) {
        return overridedSetAttributeCore(this, attr, value);
    }

    function overridedSetAttributeNS(ns, attr, value) {
        return overridedSetAttributeCore(this, attr, value, ns);
    }

    function overridedSetAttributeCore(el, attr, value, ns) {
        var setAttrMeth = ns ? NativeMethods.setAttributeNS : NativeMethods.setAttribute,
            tagName = el.tagName.toLowerCase(),
            isSupportedProtocol = UrlUtil.isSupportedProtocol(value),
            urlAttr = isUrlAttr(el, attr),
            isEventAttr = PageProc.EVENTS.indexOf(attr) !== -1;

        value = value + '';

        if ((urlAttr && !isSupportedProtocol) || isEventAttr) {
            var isJsProtocol = PageProc.JAVASCRIPT_PROTOCOL_REG_EX.test(value),
                storedJsAttr = PageProc.getStoredAttrName(attr);

            if ((urlAttr && isJsProtocol) || isEventAttr) {
                /* jshint ignore:start */
                var valueWithoutProtocol = value.replace(PageProc.JAVASCRIPT_PROTOCOL_REG_EX, ''),
                    matches = valueWithoutProtocol.match(PageProc.HTML_STRING_REG_EX),
                    processedValue = '';

                if (matches && isJsProtocol) {
                    var html = matches[2];
                    var stringQuote = matches[1];

                    if(!SandboxUtil.isPageHtml(html))
                        html = '<html><body>' + html + '</body></html>'

                    html = html.replace(new RegExp('\\\\' + stringQuote, 'g'), stringQuote);
                    html = SandboxUtil.processHtml(html);
                    html = html.replace(new RegExp(stringQuote, 'g'), '\\' + stringQuote);
                    processedValue = 'javascript:' + stringQuote + html + stringQuote;
                }
                else
                    processedValue = (isJsProtocol ? 'javascript:' : '') + PageProc.processScript(valueWithoutProtocol, true);

                if (processedValue !== value) {
                    setAttrMeth.apply(el, ns ? [ns, storedJsAttr, value] : [storedJsAttr, value]);
                    value = processedValue;
                }
                /* jshint ignore:end */
            } else
                setAttrMeth.apply(el, ns ? [ns, storedJsAttr, value] : [storedJsAttr, value]);
        } else if (urlAttr && isSupportedProtocol) {
            var storedUrlAttr = PageProc.getStoredAttrName(attr);

            setAttrMeth.apply(el, ns ? [ns, storedUrlAttr, value] : [storedUrlAttr, value]);

            if (tagName !== 'img') {
                if (value !== '') {
                    var isIframe = tagName === 'iframe',
                        isScript = tagName === 'script',
                        isCrossDomainUrl = isSupportedProtocol && !UrlUtil.sameOriginCheck(location.toString(), value),
                        resourceType = null;

                    if (isScript)
                        resourceType = UrlUtil.SCRIPT;
                    else if (isIframe || PageProc.isOpenLinkInIFrame(el))
                        resourceType = UrlUtil.IFRAME;

                    value = (isIframe && isCrossDomainUrl) ? UrlUtil.getCrossDomainIframeProxyUrl(value) :
                        UrlUtil.getProxyUrl(value, null, null, null, null, resourceType);
                }
            } else {
                if (value && !UrlUtil.parseProxyUrl(value))
                    value = UrlUtil.resolveUrlAsOrigin(value);
            }
        } else if (attr === 'autocomplete') {
            var storedAutocompleteAttr = PageProc.getStoredAttrName(attr);

            setAttrMeth.apply(el, ns ? [ns, storedAutocompleteAttr, value] : [storedAutocompleteAttr, value]);
            value = 'off';
        }
        else if (attr === 'target' && value === '_blank' && PageProc.TARGET_ATTR_TAGS[tagName])
            return;
        else if (attr === 'sandbox') {
            var storedSandboxAttr = PageProc.getStoredAttrName(attr);
            var allowSameOrigin   = value.indexOf('allow-same-origin') !== -1;
            var allowScripts      = value.indexOf('allow-scripts') !== -1;

            setAttrMeth.apply(el, ns ? [ns, storedSandboxAttr, value] : [storedSandboxAttr, value]);

            if (!allowSameOrigin || !allowScripts) {
                value += !allowSameOrigin ? ' allow-same-origin' : '';
                value += !allowScripts ? ' allow-scripts' : '';
            }
        }
        else if (tagName === 'meta' && attr === 'name' && value === 'referrer')
            return;

        return setAttrMeth.apply(el, ns ? [ns, attr, value] : [attr, value]);
    }

    function overridedRemoveAttribute() {
        overridedRemoveAttributeCore.call(this, false, arguments);
    }

    function overridedRemoveAttributeNS() {
        overridedRemoveAttributeCore.call(this, true, arguments);
    }

    function overridedRemoveAttributeCore(ns, arg) {
        var attr = ns ? arg[1] : arg[0],
            removeAttrFunc = ns ? NativeMethods.removeAttributeNS : NativeMethods.removeAttribute;

        if (isUrlAttr(this, attr) || attr === 'sandbox' || attr === 'autocomplete' ||
            PageProc.EVENTS.indexOf(attr) !== -1) {
            var storedAttr = PageProc.getStoredAttrName(attr);

            if (attr === 'autocomplete')
                NativeMethods.setAttribute.call(this, storedAttr, 'none');
            else
                removeAttrFunc.apply(this, ns ? [arg[0], storedAttr] : [storedAttr]);
        }

        if (attr !== 'autocomplete')
            return removeAttrFunc.apply(this, arg);
    }

    function overridedInsertRow() {
        var tagName = this.tagName.toLowerCase(),
            nativeMeth = tagName === 'table' ? NativeMethods.insertTableRow : NativeMethods.insertTBodyRow,
            row = nativeMeth.apply(this, arguments);

        exports.overrideDomMethods(row);

        return row;
    }

    function overridedInsertCell() {
        var cell = NativeMethods.insertCell.apply(this, arguments);

        exports.overrideDomMethods(cell);

        return cell;
    }

    function overridedInsertAdjacentHTML(pos, html) {
        if (html !== null)
            html = SandboxUtil.processHtml('' + html, this.parentNode && this.parentNode.tagName);

        NativeMethods.insertAdjacentHTML.call(this, pos, html);
        exports.overrideDomMethods(this.parentNode || this);
    }

    function overridedFormSubmit() {
        var form = this;

        addDescriptorToForm({target: form});

        Transport.waitCookieMsg(function () {
            NativeMethods.formSubmit.apply(form, arguments);
        });
    }

    // Utils
    function isUninitializedIframeWithoutSrc(window) {
        try {
            return window !== window.top && Util.isIframeWithoutSrc(window.frameElement) && !IFrameSandbox.isIframeInitialized(window.frameElement);
        } catch (e) {
            return false;
        }
    }

    function isSecureOrigin (url) {
        // NOTE: https://www.chromium.org/Home/chromium-security/prefer-secure-origins-for-powerful-new-features
        var parsedUrl = UrlUtil.parseUrl(UrlUtil.resolveUrlAsOrigin(url));

        return ALLOWED_SERVICE_WORKER_PROTOCOLS.indexOf(parsedUrl.protocol) === -1 &&
               ALLOWED_SERVICE_WORKER_HOST_NAMES.indexOf(parsedUrl.hostname) === -1;
    }

    function isUrlAttr(el, attr) {
        var tagName = el.tagName.toLowerCase();

        return PageProc.URL_ATTR_TAGS[attr] && PageProc.URL_ATTR_TAGS[attr].indexOf(tagName) !== -1;
    }

    function overrideElement(el, overridePrototypeMeths) {
        var isDocFragment = el.nodeType === 11,
            elTagName = el.tagName && el.tagName.toLowerCase(),
            isForm = elTagName === 'form',
            isIframe = elTagName === 'iframe';

        if (!isDocFragment)
            PageProc.processElement(el, UrlUtil.convertToProxyUrl);

        if (elTagName === 'img') {
            el.addEventListener('error', function (e) {
                var storedAttr = NativeMethods.getAttribute.call(el, PageProc.getStoredAttrName('src'));

                if (storedAttr && !UrlUtil.parseProxyUrl(el.src) && UrlUtil.isSupportedProtocol(el.src)) {
                    NativeMethods.setAttribute.call(el, 'src', UrlUtil.getProxyUrl(storedAttr));
                    Util.stopPropagation(e);
                }
            }, false);
        }

        if (isIframe && !Util.isCrossDomainIframe(el, true))
            IFrameSandbox.overrideIframe($(el));

        if ('insertAdjacentHTML' in el)
            el.insertAdjacentHTML = overridedInsertAdjacentHTML;

        if (isForm)
            el.addEventListener('submit', addDescriptorToForm, false);

        if (!SandboxUtil.BROWSER_HAS_ELEMENT_PROTOTYPE || overridePrototypeMeths) {
            el.insertBefore = overridedInsertBefore;
            el.appendChild = overridedAppendChild;
            el.removeChild = overridedRemoveChild;
            el.cloneNode = overridedCloneNode;

            if (!isDocFragment) {
                el.setAttribute = overridedSetAttribute;
                el.setAttributeNS = overridedSetAttributeNS;
                el.getAttribute = overridedGetAttribute;
                el.getAttributeNS = overridedGetAttributeNS;
                el.removeAttribute = overridedRemoveAttribute;
                el.removeAttributeNS = overridedRemoveAttributeNS;
            }

            if ('insertRow' in el)
                el.insertRow = overridedInsertRow;

            if ('insertCell' in el)
                el.insertCell = overridedInsertCell;

            if (isForm)
                el.submit = overridedFormSubmit;
        }
    }

    // NOTE: The browser ignores the query string, if the action method is get.
    function addDescriptorToForm(e) {
        var target = e.target || e.srcElement,
            action = NativeMethods.getAttribute.call(target, 'action'),
            descName = UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY,
            desc = UrlUtil.parseUrl(action || location.toString()).query[descName];

        if (target.method === 'get') {
            if (target[descName])
                target.removeChild(target[descName]);

            $('<input>', {
                type: 'hidden',
                name: descName,
                value: desc
            }).appendTo(target);
        }
    }

    function raiseUncaughtJsErrorEvent(msg, window, pageUrl) {
        if (Util.isCrossDomainWindows(window, window.top))
            return;

        var sendToTopWindow = window !== window.top;

        if (!pageUrl) {
            var originUrlObj = UrlUtil.getOriginUrlObj();
            pageUrl = UrlUtil.formatUrl(originUrlObj);
        }

        if (sendToTopWindow) {
            eventEmitter.emit(UNCAUGHT_JS_ERROR, {
                msg: msg,
                pageUrl: pageUrl,
                inIFrame: true
            });

            MessageSandbox.sendServiceMsg({
                cmd: UNCAUGHT_JS_ERROR,
                pageUrl: pageUrl,
                msg: msg
            }, window.top);
        } else {
            eventEmitter.emit(UNCAUGHT_JS_ERROR, {
                msg: msg,
                pageUrl: pageUrl
            });
        }
    }

    //NOTE: DOM sandbox hides evidence of the content proxying from page-native script. Proxy replaces URLs for
    //resources. Our goal is to make native script think that all resources are fetched from origin resource not
    //from proxy and also provide proxying for dynamicly created elements.
    exports.init = function (window, document) {
        onInit(window);

        ShadowUI.init(window, document);
        EventSandbox.init(window, document);
        ActiveWindowTracker.init(window);
        XhrSandbox.init(window, document);
        MessageSandbox.init(window, document);
        UploadSandbox.init(window, document);
        DomAccessorWrappers.init(window, document);

        DomAccessorWrappers.on(DomAccessorWrappers.BODY_CONTENT_CHANGED, function (el) {
            var elContextWindow = el[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT];

            if (elContextWindow !== window) {
                MessageSandbox.sendServiceMsg({
                    cmd: BODY_CONTENT_CHANGED
                }, elContextWindow);
            } else
                onBodyContentChanged(el);
        });

        initInternal(window, document);

        SandboxedJQuery.init(window, undefined);

        MessageSandbox.on(MessageSandbox.SERVICE_MSG_RECEIVED, function (e) {
            var message = e.message;

            if (message.cmd === UNCAUGHT_JS_ERROR)
                raiseUncaughtJsErrorEvent(message.msg, window, message.pageUrl);

            if (message.cmd === BODY_CONTENT_CHANGED)
                onBodyContentChanged();
        });
    };

    function initInternal(window, document) {
        // NOTE: Iframe loses its contentWindow after reinserting in the DOM (in the FF).
        if (Util.isMozilla) {
            exports.on(IFRAME_ADDED, function (e) {
                IFrameSandbox.overrideIframe($(e.iframe));
            });
        }

        // NOTE: in some browsers (for example Firefox) 'window.document' are different objects when iframe is created
        // just now and on document ready event. Therefore we should update 'document' object to override its methods (Q527555).
        $(document).ready(function () {
            exports.overrideDomMethods(null, document);
        });

        overrideDocument(window, document);
        overrideWindow(window);
    }

    function onDocumentCleaned(window, document) {
        if(Util.isIE)  {
            var needToUpdateNativeDomMeths = false,
                needToUpdateNativeElementMeths = false;

            try {
                needToUpdateNativeDomMeths = !document.createElement ||
                    (NativeMethods.createElement.toString() === document.createElement.toString());
            } catch (e) {
                needToUpdateNativeDomMeths = true;
            }

            try {
                var nativeElement = NativeMethods.createElement.call(document, 'div');

                needToUpdateNativeElementMeths = nativeElement.getAttribute.toString() === NativeMethods.getAttribute.toString();
            } catch (e) {
                needToUpdateNativeElementMeths = true;
            }

            // T173709
            if (needToUpdateNativeDomMeths)
                NativeMethods.refreshDocument(document);

            if(needToUpdateNativeElementMeths)
                NativeMethods.refreshElementMeths(document, window);
        }

        EventSandbox.initDocumentListening();

        if(Util.isWebKit)
            EventSandbox.restartElementListening(window);

        ShadowUI.init(window, document);
        DomAccessorWrappers.init(window, document); // T182337

        eventEmitter.emit(DOCUMENT_CLEANED, {
            document: document,
            isIFrameWithoutSrc: isIFrameWithoutSrc
        });

        overrideDocument(window, document);
    }

    function onInit(window) {
        var topSameDomainWindow = Util.getTopSameDomainWindow(window);

        if (isIFrameWithoutSrc) {
            topSameDomainWindow[IFRAME_DOM_SANDBOXES_STORE].push({
                iframe: window.frameElement,
                domSandbox: exports
            });
        } else if (window === topSameDomainWindow) {
            window[IFRAME_DOM_SANDBOXES_STORE] = [];
        }
    }

    function overrideDocument(window, document) {
        var storedDocumentWriteContent = '',
            writeBlockCounter = 0;

        function beforeDocumentCleaned() {
            eventEmitter.emit(BEFORE_DOCUMENT_CLEANED, {
                document: document,
                isIFrameWithoutSrc: isIFrameWithoutSrc
            });
        }

        function onDocumentClosed() {
            eventEmitter.emit(DOCUMENT_CLOSED, {
                document: document,
                isIFrameWithoutSrc: isIFrameWithoutSrc
            });
        }

        function overridedDocumentWrite(args, ln) {
            args = Array.prototype.slice.call(args);

            var separator = ln ? '\n' : '',
                lastArg = args.length ? args[args.length - 1] : '',
                isBegin = lastArg === JSProcessor.DOCUMENT_WRITE_BEGIN_PARAM,
                isEnd = lastArg === JSProcessor.DOCUMENT_WRITE_END_PARAM;

            if (isBegin)
                writeBlockCounter++;
            else if (isEnd)
                writeBlockCounter--;

            if (isBegin || isEnd)
                args.pop();

            var str = separator + args.join(separator);

            if ((isEnd && !writeBlockCounter) || SandboxUtil.isPageHtml(str)) {
                writeBlockCounter = 0;
                str = storedDocumentWriteContent + str;
                storedDocumentWriteContent = '';
            } else if (isBegin || storedDocumentWriteContent) {
                storedDocumentWriteContent += str;

                return;
            }

            var isUninitializedIframe = isUninitializedIframeWithoutSrc(window);

            str = SandboxUtil.processHtml('' + str);

            if (!isUninitializedIframe)
                beforeDocumentCleaned();

            // FireFox, IE recreate window instance during the document.write function execution T213930
            if ((Util.isMozilla || Util.isIE) && !SandboxUtil.isPageHtml(str))
                str = SandboxUtil.INIT_SCRIPT_FOR_IFRAME_TEMPLATE + str;

            var result = NativeMethods.documentWrite.call(document, str);

            if (!isUninitializedIframe) {
                onDocumentCleaned(window, document);
                exports.overrideDomMethods(null, document, window); // B234357
            }

            return result;
        }

        document.open = function () {
            var isUninitializedIframe = isUninitializedIframeWithoutSrc(window);

            if (!isUninitializedIframe)
                beforeDocumentCleaned();

            var result = NativeMethods.documentOpen.call(document);

            if (!isUninitializedIframe)
                onDocumentCleaned(window, document);
            else
            // If iframe initialization in progress, we should once again override document.write and document.open meths
            // because they were cleaned after native document.open meth calling
                overrideDocument(window, document);

            return result;
        };

        document.close = function () {
            // IE10 and IE9 rise "load" event only when document.close meth called.
            // We should restore overrided document.open and document.write meths before Hammerhead injection
            // if window not initialized
            if (Util.isIE && !IFrameSandbox.isWindowInited(window))
                NativeMethods.restoreNativeDocumentMeth(document);

            var result = NativeMethods.documentClose.call(document);

            if (!isUninitializedIframeWithoutSrc(window))
                onDocumentClosed();

            return result;
        };

        document.createElement = function (tagName) {
            var el = NativeMethods.createElement.call(document, tagName);

            exports.overrideDomMethods(el, document, window);

            return el;
        };

        document.createElementNS = function (ns, tagName) {
            var el = NativeMethods.createElementNS.call(document, ns, tagName);

            exports.overrideDomMethods(el, document, window);

            return el;
        };

        document.write = function () {
            return overridedDocumentWrite(arguments);
        };

        document.writeln = function () {
            return overridedDocumentWrite(arguments, true);
        };

        document.createDocumentFragment = function () {
            var fragment = NativeMethods.createDocumentFragment.apply(document, arguments);

            exports.overrideDomMethods(fragment, document, window);

            return fragment;
        };
    }

    function overrideWindow(window) {
        // Additional methods for the DOM
        window[SharedConst.DOM_SANDBOX_OVERRIDE_DOM_METHOD_NAME] = exports.overrideDomMethods;

        window.CanvasRenderingContext2D.prototype.drawImage = function () {
            var image = arguments[0];

            if (!Util.isCanvas(image)) {
                var args = Array.prototype.slice.call(arguments, 0),
                    src = image.src;

                if (UrlUtil.sameOriginCheck(location.toString(), src)) {
                    args[0] = NativeMethods.createElement.call(window.document, 'img');
                    args[0].src = UrlUtil.getProxyUrl(src);
                }

                return NativeMethods.canvasContextDrawImage.apply(this, args);
            } else
                return NativeMethods.canvasContextDrawImage.apply(this, arguments);
        };

        // Override uncaught error handling
        window.onerror = function (msg, url, line, col, errObj) {
            // FireFox raises NS_ERROR_NOT_INITIALIZED exception after widnow has been removed from the dom
            if(msg.indexOf('NS_ERROR_NOT_INITIALIZED') !== -1)
                return true;

            var originalOnErrorHandler = window[DomAccessorWrappers.ORIGINAL_WINDOW_ON_ERROR_HANDLER_KEY],
                caught = originalOnErrorHandler &&
                    originalOnErrorHandler.call(window, msg, url, line, col, errObj) === true;

            if (caught)
                return true;

            raiseUncaughtJsErrorEvent(msg, window);

            return false;
        };

        window.open = function () {
            var newArgs = [];

            newArgs.push(UrlUtil.getProxyUrl(arguments[0]));
            newArgs.push('_self');

            if (arguments.length > 2)
                newArgs.push(arguments[2]);
            if (arguments.length > 3)
                newArgs.push(arguments[3]);

            return NativeMethods.windowOpen.apply(window, newArgs);
        };

        window.Worker = function (scriptURL) {
            scriptURL = UrlUtil.getProxyUrl(scriptURL);

            return new NativeMethods.workerCtor(scriptURL);
        };

        if (window.Blob) {
            window.Blob = function (parts, opts) {
                switch (arguments.length) {
                    case 0:
                        return new NativeMethods.blobCtor();
                    case 1:
                        return new NativeMethods.blobCtor(parts);
                    default:
                        var type = opts && opts.type && opts.type.toString().toLowerCase();

                        if (type === 'text/javascript' || type === 'application/javascript' ||
                            type === 'application/x-javascript') {
                            parts = [PageProc.processScript(parts.join(''))];
                        }

                        return new NativeMethods.blobCtor(parts, opts);
                }

            };
        }

        window.EventSource = function (url) {
            url = UrlUtil.getProxyUrl(url);

            return new NativeMethods.eventSourceCtor(url);
        };

        if (window.MutationObserver) {
            window.MutationObserver = function (callback) {
                var wrapper = function (mutations) {
                    var result = [];

                    for (var i = 0; i < mutations.length; i++) {
                        if (!ShadowUI.isShadowUIMutation(mutations[i]))
                            result.push(mutations[i]);
                    }

                    if (result.length)
                        callback(result);
                };

                return new NativeMethods.mutationObserverCtor(wrapper);
            };
        }

        window.Image = function () {
            var image = null;

            if (!arguments.length)
                image = new NativeMethods.imageCtor();
            else if (arguments.length === 1)
                image = new NativeMethods.imageCtor(arguments[0]);
            else
                image = new NativeMethods.imageCtor(arguments[0], arguments[1]);

            exports.overrideDomMethods(image);

            return image;
        };

         window.Function = function () {
             var length = arguments.length;
             var lastArgIndex = length - 1;
             var args = [];

             for (var i = 0; i < length; i++)
                args.push(arguments[i]);

             if (typeof args[lastArgIndex] === 'string')
                 args[lastArgIndex] = JSProcessor.process(args[lastArgIndex]);

             return NativeMethods.Function.apply(this, args);
         };
         window.Function.prototype = NativeMethods.Function.prototype;
         window.Function.prototype.constructor = window.Function;

        // Override native DOM methods
        if (SandboxUtil.BROWSER_HAS_ELEMENT_PROTOTYPE) {
            window.Element.prototype.insertBefore = overridedInsertBefore;
            window.Element.prototype.appendChild = overridedAppendChild;
            window.Element.prototype.removeChild = overridedRemoveChild;
            window.Element.prototype.setAttribute = overridedSetAttribute;
            window.Element.prototype.setAttributeNS = overridedSetAttributeNS;
            window.Element.prototype.getAttribute = overridedGetAttribute;
            window.Element.prototype.getAttributeNS = overridedGetAttributeNS;
            window.Element.prototype.removeAttribute = overridedRemoveAttribute;
            window.Element.prototype.removeAttributeNS = overridedRemoveAttributeNS;
            window.Element.prototype.cloneNode = overridedCloneNode;
            window.Node.prototype.cloneNode = overridedCloneNode;
            window.Node.prototype.appendChild = overridedAppendChild;
            window.Node.prototype.removeChild = overridedRemoveChild;
            window.Node.prototype.insertBefore = overridedInsertBefore;
            window.HTMLTableElement.prototype.insertRow = overridedInsertRow;
            window.HTMLTableSectionElement.prototype.insertRow = overridedInsertRow;
            window.HTMLTableRowElement.prototype.insertCell = overridedInsertCell;
            window.HTMLFormElement.prototype.submit = overridedFormSubmit;
        }

        if (typeof window.history.pushState === 'function' && typeof window.history.replaceState === 'function') {
            window.history.pushState = function (data, title, url) {
                var args = [data, title];

                if (arguments.length > 2)
                    args.push(url ? UrlUtil.getProxyUrl(url) : url);

                return NativeMethods.historyPushState.apply(history, args);
            };

            window.history.replaceState = function (data, title, url) {
                var args = [data, title];

                if (arguments.length > 2)
                    args.push(url ? UrlUtil.getProxyUrl(url) : url);

                return NativeMethods.historyReplaceState.apply(history, args);
            };
        }

        if (window.Range.prototype.createContextualFragment) {
            window.Range.prototype.createContextualFragment = function () {
                /* jshint ignore:start */
                if (typeof arguments[0] === 'string')
                    arguments[0] = SandboxUtil.processHtml(arguments[0]);
                /* jshint ignore:end */

                var fragment = NativeMethods.createContextualFragment.apply(this, arguments);

                exports.overrideDomMethods(fragment, document, window);

                return fragment;
            };
        }

        if (window.navigator.registerProtocolHandler) {
            window.navigator.registerProtocolHandler = function () {
                var args = Array.prototype.slice.call(arguments),
                    urlIndex = 1,
                    originUrlObj = UrlUtil.getOriginUrlObj(),
                    isOriginUrl = $.browser.mozilla ?
                        UrlUtil.isSubDomain(originUrlObj.hostname, UrlUtil.parseUrl(args[urlIndex]).hostname) :
                        UrlUtil.sameOriginCheck(UrlUtil.formatUrl(originUrlObj), args[urlIndex]);

                if (isOriginUrl)
                    args[urlIndex] = UrlUtil.getProxyUrl(args[urlIndex]);

                return NativeMethods.registerProtocolHandler.apply(navigator, args);
            };
        }

        if (window.FormData) {
            window.FormData.prototype.append = function (name, value) {
                // NOTE: We should not send our hidden input's value along with the file info,
                // because our input may have incorrect value if the input with the file has been removed from DOM.
                if (name === SharedConst.UPLOAD_SANDBOX_HIDDEN_INPUT_NAME)
                    return;

                // NOTE: If we append our file wrapper to FormData, we will lose the file name.
                // This happens because the file wrapper is an instance of Blob
                // and a browser thinks that Blob does not contain the "name" property.
                if (arguments.length === 2 && Util.isBlobInstance(value) && 'name' in value)
                    NativeMethods.formDataAppend.call(this, name, value, value.name);
                else
                    NativeMethods.formDataAppend.apply(this, arguments);
            };
        }

        if (NativeMethods.registerServiceWorker) {
            window.navigator.serviceWorker.register = function () {
                var args = [];

                for (var i = 0; i < arguments.length; i++)
                    args.push(arguments[i]);

                var url = args[0];

                if (typeof url === 'string') {
                    if (isSecureOrigin(url))
                        return window.Promise.reject(new window.DOMException('Only secure origins are allowed.', 'SecurityError'));

                    args[0] = UrlUtil.getProxyUrl(url, null, null, null, null, UrlUtil.SCRIPT);
                }

                if (args[1] && typeof args[1].scope === 'string')
                    args[1].scope = UrlUtil.getProxyUrl(args[1].scope, null, null, null, null, UrlUtil.SCRIPT);

                return NativeMethods.registerServiceWorker.apply(this, args);
            };
        }
    }

    function prepareNodeForInsertion (node, parentNode) {
        //NOTE: we should process a TextNode as a script if it is appended to a script element (B254284)
        if (node.nodeType === 3 && parentNode.tagName && parentNode.tagName.toLowerCase() === 'script')
            node.data = PageProc.processScript(node.data);
        else if (node.nodeType === 3 && Util.isStyleElement(parentNode))
            node.data = PageProc.processStylesheet(node.data, UrlUtil.getProxyUrl);

        return node;
    }
});
