/* global Document:true*/
HammerheadClient.define('DOMSandbox.NativeMethods', function (require, exports) {
    function refreshDocumentMeths(doc) {
        doc = doc || document;

        // Dom
        exports.createDocumentFragment = doc.createDocumentFragment || Document.prototype.createDocumentFragment;
        exports.createElement = doc.createElement || Document.prototype.createElement;
        exports.createElementNS = doc.createElementNS || Document.prototype.createElementNS;
        exports.documentOpen = doc.open || Document.prototype.open;
        exports.documentClose = doc.close || Document.prototype.close;
        exports.documentWrite = doc.write || Document.prototype.write;
        exports.documentWriteLn = doc.writeln || Document.prototype.writeln;
        exports.elementFromPoint = doc.elementFromPoint || Document.prototype.elementFromPoint;
        exports.getElementById = doc.getElementById || Document.prototype.getElementById;
        exports.getElementsByClassName = doc.getElementsByClassName || Document.prototype.getElementsByClassName;
        exports.getElementsByName = doc.getElementsByName || Document.prototype.getElementsByName;

        exports.getElementsByTagName = doc.getElementsByTagName || Document.prototype.getElementsByTagName;
        exports.querySelector = doc.querySelector || Document.prototype.querySelector;
        exports.querySelectorAll = doc.querySelectorAll || Document.prototype.querySelectorAll;

        // Event
        exports.documentAddEventListener = doc.addEventListener || Document.prototype.addEventListener;
        exports.documentRemoveEventListener = doc.removeEventListener || Document.prototype.removeEventListener;
        exports.documentCreateEvent = doc.createEvent || Document.prototype.createEvent;
        exports.documentCreateTouch = doc.createTouch || Document.prototype.createTouch;
        exports.documentCreateTouchList = doc.createTouchList || Document.prototype.createTouchList;
    }

    function refreshElementMeths(doc, win) {
        win = win || window;

        var createElement = function (tagName) {
                return exports.createElement.call(doc || document, tagName);
            },
            nativeElement = createElement('div');

        // Dom
        exports.appendChild = nativeElement.appendChild;
        exports.cloneNode = nativeElement.cloneNode;
        exports.elementGetElementsByClassName = nativeElement.getElementsByClassName;
        exports.elementGetElementsByTagName = nativeElement.getElementsByTagName;
        exports.elementQuerySelector = nativeElement.querySelector;
        exports.elementQuerySelectorAll = nativeElement.querySelectorAll;
        exports.getAttribute = nativeElement.getAttribute;
        exports.getAttributeNS = nativeElement.getAttributeNS;
        exports.insertAdjacentHTML = nativeElement.insertAdjacentHTML;
        exports.insertBefore = nativeElement.insertBefore;
        exports.insertCell = createElement('tr').insertCell;
        exports.insertTableRow = createElement('table').insertRow;
        exports.insertTBodyRow = createElement('tbody').insertRow;
        exports.removeAttribute = nativeElement.removeAttribute;
        exports.removeAttributeNS = nativeElement.removeAttributeNS;
        exports.removeChild = nativeElement.removeChild;
        exports.setAttribute = nativeElement.setAttribute;
        exports.setAttributeNS = nativeElement.setAttributeNS;

        // Event
        exports.addEventListener = nativeElement.addEventListener;
        exports.attachEvent = nativeElement.attachEvent;
        exports.detachEvent = nativeElement.detachEvent;
        exports.blur = nativeElement.blur;
        exports.click = nativeElement.click;
        exports.dispatchEvent = nativeElement.dispatchEvent;
        exports.fireEvent = nativeElement.fireEvent;
        exports.focus = nativeElement.focus;
        exports.removeEventListener = nativeElement.removeEventListener;
        exports.select = window.TextRange ? createElement('body').createTextRange().select : null;
        exports.setSelectionRange = createElement('input').setSelectionRange;
        exports.textAreaSetSelectionRange = createElement('textarea').setSelectionRange;

        exports.svgFocus = win.SVGElement ? win.SVGElement.prototype.focus : exports.focus;
        exports.svgBlur  = win.SVGElement ? win.SVGElement.prototype.blur : exports.blur;
    }

    function refreshClasses(win) {
        function mock() { }

        exports.windowClass = win.Window || mock;
        exports.documentClass = win.Document || mock;
        exports.locationClass = win.Location || mock;
        exports.elementClass = win.Element || mock;
        exports.styleClass = win.CSSStyleDeclaration || win.CSS2Properties || win.MSStyleCSSProperties || mock;
        exports.styleSheetClass = win.CSSStyleSheet || mock;
    }

    function refreshWindowMeths() {
        // Dom
        exports.eval = window.eval;
        exports.eventSourceCtor = window.EventSource;
        exports.formSubmit = window.HTMLFormElement.prototype.submit;
        exports.historyPushState = window.history ? window.history.pushState : null;
        exports.historyReplaceState = window.history ? window.history.replaceState : null;
        exports.imageCtor = window.Image;
        exports.Function = window.Function;
        exports.mutationObserverCtor = window.MutationObserver;
        exports.postMessage = window.postMessage;
        exports.windowOpen = window.open;
        exports.workerCtor = window.Worker;
        exports.blobCtor = window.Blob;
        exports.setTimeout = window.setTimeout;
        exports.setInterval = window.setInterval;
        exports.clearTimeout = window.clearTimeout;
        exports.clearInterval = window.clearInterval;
        exports.XMLHttpRequest = window.XMLHttpRequest;
        exports.registerProtocolHandler = window.navigator.registerProtocolHandler;
        exports.createContextualFragment = window.Range.prototype.createContextualFragment;

        // Event
        exports.windowAddEventListener = window.addEventListener;
        exports.windowRemoveEventListener = window.removeEventListener;
        exports.WindowPointerEvent = window.PointerEvent;
        exports.WindowMSPointerEvent = window.MSPointerEvent;
        exports.WindowTouch = window.Touch;
        exports.WindowTouchEvent = window.TouchEvent;

        // Canvas
        exports.canvasContextDrawImage = window.CanvasRenderingContext2D.prototype.drawImage;

        // FormData
        if (window.FormData)
            exports.formDataAppend = window.FormData.prototype.append;

        // XHR
        exports.xhrAbort                 = window.XMLHttpRequest.prototype.abort;
        exports.xhrOpen                  = window.XMLHttpRequest.prototype.open;
        exports.xhrSend                  = window.XMLHttpRequest.prototype.send;
        exports.xhrAddEventListener      = window.XMLHttpRequest.prototype.addEventListener;
        exports.xhrRemoveEventListener   = window.XMLHttpRequest.prototype.removeEventListener;
        exports.xhrGetResponseHeader     = window.XMLHttpRequest.prototype.getResponseHeader;
        exports.xhrGetAllResponseHeaders = window.XMLHttpRequest.prototype.getAllResponseHeaders;
        exports.xhrSetRequestHeader      = window.XMLHttpRequest.prototype.setRequestHeader;

        try {
            exports.registerServiceWorker = window.navigator.serviceWorker.register;
        }
        catch (e) {
            exports.registerServiceWorker = null;
        }

        exports.objectToString = window.Object.prototype.toString;

        // Setters
        var inputValueDescriptor    = window.Object.getOwnPropertyDescriptor(window.HTMLInputElement.prototype, 'value');
        var textAreaValueDescriptor = window.Object.getOwnPropertyDescriptor(window.HTMLTextAreaElement.prototype, 'value');

        if (inputValueDescriptor && typeof inputValueDescriptor.set === 'function')
            exports.inputValueSetter = inputValueDescriptor.set;

        if (textAreaValueDescriptor && typeof textAreaValueDescriptor.set === 'function')
            exports.textAreaValueSetter = textAreaValueDescriptor.set;

        refreshClasses(window);
    }

    exports.refreshDocument = refreshDocumentMeths;
    exports.refreshElementMeths = refreshElementMeths;

    function isNativeMeth(meth) {
        return meth.toString().indexOf('[native code]') !== -1;
    }

    refreshDocumentMeths(isNativeMeth(document.createElement) ? document : Document.prototype);
    refreshElementMeths();
    refreshWindowMeths();

    exports.restoreNativeDocumentMeth = function (document) {
        document.createDocumentFragment = exports.createDocumentFragment;
        document.createElement = exports.createElement;
        document.createElementNS = exports.createElementNS;
        document.open = exports.documentOpen;
        document.close = exports.documentClose;
        document.write = exports.documentWrite;
        document.writeln = exports.documentWriteLn;
        document.elementFromPoint = exports.elementFromPoint;
        document.getElementById = exports.getElementById;
        document.getElementsByClassName = exports.getElementsByClassName;
        document.getElementsByName = exports.getElementsByName;
        document.getElementsByTagName = exports.getElementsByTagName;
        document.querySelector = exports.querySelector;
        document.querySelectorAll = exports.querySelectorAll;

        // Event
        document.addEventListener = exports.documentAddEventListener;
        document.removeEventListener = exports.documentRemoveEventListener;
        document.createEvent = exports.documentCreateEvent;
        document.createTouch = exports.documentCreateTouch;
        document.createTouchList = exports.createTouchList;
    };
});