HammerheadClient.define('DOMSandbox.Util', function (require, exports) {
    var $             = require('jQuery'),
        NativeMethods = require('DOMSandbox.NativeMethods'),
        PageProc      = require('Shared.PageProc'),
        SharedConst   = require('Shared.Const'),
        UrlUtil       = require('UrlUtil'),
        Util          = require('Util');

    var FAKE_TAG_NAME_PREFIX = 'faketagname_',
        FAKE_ATTR_WITH_TAG_NAME = 'hh_fake_attr',
        FAKE_TAG_NAME_RE = new RegExp('(<\\/?)' + FAKE_TAG_NAME_PREFIX, 'ig'),
        WRAPPED_TAGS_RE = /(<\/?)(table|tbody|tfoot|thead|tr|td|th|caption|colgroup)([^>]*>)/ig,
        WRAPPED_COL_NOSCRIPT_TAG_RE = /<(\/?(?:col|noscript))(\s[^>]*?)?(\s?\/)?>/ig,
        WRAPPED_COL_NOSCRIPT_TAG_TEMPLATE = '<br ' + FAKE_ATTR_WITH_TAG_NAME + '="$1|$3"$2>',
        FAKE_COL_NOSCRIPT_TAG_RE = new RegExp('<br([^>]*?) ' + FAKE_ATTR_WITH_TAG_NAME + '="([^|]+)\\|([^"]*)"([^>]*)', 'ig'),
        INIT_SCRIPT_FOR_IFRAME_TEMPLATE =
            '<script class="' + SharedConst.TEST_CAFE_SCRIPT_CLASSNAME + '" type="text/javascript">' +
            'var parentHammerhead = null;' +
            'try {' +
            '   parentHammerhead = window.parent.Hammerhead;' +
            '} catch(e) {}' +
            'if (parentHammerhead) parentHammerhead._rebindDomSandboxToIframe(window.frameElement);' +
            'var script = document.currentScript || document.scripts[document.scripts.length - 1];' +
            'script.parentNode.removeChild(script);' +
            '<\/script>';

    exports.BROWSER_HAS_ELEMENT_PROTOTYPE   = window.Element && Element.prototype && !$.browser.mozilla;
    exports.INIT_SCRIPT_FOR_IFRAME_TEMPLATE = INIT_SCRIPT_FOR_IFRAME_TEMPLATE;

    exports.isPageHtml = isPageHtml;

    var htmlDocument = document.implementation.createHTMLDocument('title'),
        htmlParser   = htmlDocument.createDocumentFragment();

    PageProc.on(PageProc.HTML_PROCESSING_REQUIRED, function (html, callback) {
        if (!exports.isPageHtml(html))
            html = '<html><body>' + html + '</body></html>';

        callback(exports.processHtml(html));
    });

    function getHtmlDocument () {
        try {
            // IE bug: access denied
            if (htmlDocument.location)
                htmlDocument.location.toString();
        } catch (e) {
            htmlDocument = document.implementation.createHTMLDocument('title');
            htmlParser   = htmlDocument.createDocumentFragment();
        }

        return htmlDocument;
    }

    function hidePageTag (tagHtml) {
        return tagHtml ? tagHtml.replace(/^(\s*<\s*\/?)(head|body|html)/i, '$1' + FAKE_TAG_NAME_PREFIX + '$2') : '';
    }

    function isPageHtml (html) {
        return /^\s*(<\s*(!doctype|html|head|body)[^>]*>)/i.test(html);
    }

    function preparePageHtml (html) {
        var doctypeRegEx     = /^(\s*<\s*!doctype[^>]*>)([\s\S]*)$/i;
        var headBodyRegEx    = /^(\s*<\s*(head|body)[^>]*>)([\s\S]*?)(<\s*\/(head|body)\s*>\s*)?$/i;
        var htmlContentRegEx = /^(\s*<\s*head[^>]*>)([\s\S]*?)(<\s*\/head\s*>\s*)(<\s*body[^>]*>)([\s\S]*?)(<\s*\/body\s*>\s*)?$/i;
        var htmlRegEx        = /^(\s*<\s*html[^>]*>)([\s\S]*?)(<\s*\/html\s*>\s*)?$/i;
        var doctype          = '';
        var prefix           = '';
        var postfix          = '';

        var doctypeMatches = html.match(doctypeRegEx);

        if (doctypeMatches) {
            doctype = doctypeMatches[1];
            html    = doctypeMatches[2];
        }

        var htmlMatches = html.match(htmlRegEx);

        if (htmlMatches) {
            prefix += hidePageTag(htmlMatches[1]);
            html    = htmlMatches[2];
            postfix = hidePageTag(htmlMatches[3]) + postfix;
        }

        var htmlContentMatches = html.match(htmlContentRegEx);

        if (htmlContentMatches) {
            prefix += hidePageTag(htmlContentMatches[1]) + htmlContentMatches[2] + hidePageTag(htmlContentMatches[3]);
            postfix = hidePageTag(htmlContentMatches[4]) + htmlContentMatches[5] + hidePageTag(htmlContentMatches[6]) +
                      postfix;
            html    = '';
        }
        else {
            var headBodyMatches = html.match(headBodyRegEx);

            if (headBodyMatches) {
                prefix += hidePageTag(headBodyMatches[1]);
                html    = headBodyMatches[3];
                postfix = hidePageTag(headBodyMatches[4]) + postfix;
            }
        }

        html = prefix + html + postfix;

        return { doctype: doctype, html: html };
    }

    function processHtml (html, parentTag, process) {
        var isPage  = isPageHtml(html);
        var doctype = '';

        if (isPage) {
            var preparedHtml = preparePageHtml(html);

            doctype = preparedHtml.doctype;
            html    = preparedHtml.html;
        }

        html = html
            .replace(WRAPPED_TAGS_RE, '$1' + FAKE_TAG_NAME_PREFIX + '$2$3')
            .replace(WRAPPED_COL_NOSCRIPT_TAG_RE, WRAPPED_COL_NOSCRIPT_TAG_TEMPLATE);

        var container = getHtmlDocument().createElement('div');

        htmlParser.innerHTML = '';
        NativeMethods.appendChild.call(htmlParser, container);

        parentTag = parentTag ? parentTag.toLowerCase() : '';

        var isScript = parentTag === 'script';

        if (isScript)
            html = '<script>' + html + '</script>';

        container.innerHTML = html;

        if (process(container))
            html = container.innerHTML;

        if (isScript)
            html = html.replace(/^<script>|<\/script>$/ig, '');

        html = html
            .replace(FAKE_COL_NOSCRIPT_TAG_RE, '<$2$1$4$3')
            .replace(FAKE_TAG_NAME_RE, '$1');

        if (isPage)
            html = doctype + html;

        return html;
    }

    function AttributesWrapper (attributes) {
        var length = 0;

        for (var i = 0; i < attributes.length; i++) {
            var attr = attributes[i];

            if (!Util.isHammerheadAttr(attr.name)) {
                var storedAttrName = attributes[PageProc.getStoredAttrName(attr.name)];

                if (storedAttrName) {
                    attr       = attr.cloneNode();
                    attr.value = storedAttrName.value;
                    Object.defineProperty(this, attr.name, { value: attr });
                }

                Object.defineProperty(this, length, { value: attr });
                length++;
            }
        }

        Object.defineProperty(this, 'length', { value: length });

        this.item = function (index) {
            return this[index];
        };

        for (var funcName in attributes) {
            if (typeof this[funcName] === 'function' && funcName !== 'item')
                this[funcName] = attributes[funcName].bind(attributes);
        }
    }

    exports.cleanUpHtml = function (html, parentTag) {
        return processHtml(html, parentTag, function (container) {
            var changed = false;

            for (var i = 0; i < PageProc.URL_ATTRS.length; i++) {
                var attr       = PageProc.URL_ATTRS[i],
                    storedAttr = PageProc.getStoredAttrName(attr),
                    $els       = $(container).find('[' + storedAttr + ']');

                for (var j = 0; j < $els.length; j++) {
                    var el = $els[j];

                    if (el.hasAttribute(attr)) {
                        NativeMethods.setAttribute.call(el, attr, NativeMethods.getAttribute.call(el, storedAttr));
                        NativeMethods.removeAttribute.call(el, storedAttr);

                        changed = true;
                    }
                }
            }

            var $container = $(container);

            $container.find('[class*="' + SharedConst.TEST_CAFE_UI_CLASSNAME_POSTFIX + '"]').each(function () {
                $(this).remove();

                changed = true;
            });

            $container.find('script').each(function () {
                var innerHTML        = this.innerHTML,
                    cleanedInnerHTML = PageProc.cleanUpScriptHeader(innerHTML);

                if (innerHTML !== cleanedInnerHTML) {
                    this.innerHTML = cleanedInnerHTML;

                    changed = true;
                }
            });

            $container.find('[' + SharedConst.TEST_CAFE_HOVER_PSEUDO_CLASS_ATTR + ']').each(function () {
                NativeMethods.removeAttribute.call(this, SharedConst.TEST_CAFE_HOVER_PSEUDO_CLASS_ATTR);

                changed = true;
            });

            $container.find(FAKE_TAG_NAME_PREFIX + 'head, ' + FAKE_TAG_NAME_PREFIX + 'body').each(function () {
                if (this.innerHTML.indexOf(INIT_SCRIPT_FOR_IFRAME_TEMPLATE) !== -1) {
                    this.innerHTML = this.innerHTML.replace(INIT_SCRIPT_FOR_IFRAME_TEMPLATE, '');

                    changed = true;
                }
            });

            return changed;
        });
    };

    exports.processHtml = function (html, parentTag) {
        return processHtml(html, parentTag, function (container) {
            var htmlElements = [];

            var processElement = function (el) {
                PageProc.processElement(el, UrlUtil.convertToProxyUrl);

                var elTagName = el.tagName && el.tagName.toLowerCase();

                if (elTagName === FAKE_TAG_NAME_PREFIX + 'head' || elTagName === FAKE_TAG_NAME_PREFIX + 'body')
                    htmlElements.push(el);
            };


            //NOTE: we check this condition to avoid unnecessary calling the querySelectorAll function
            if (container.children.length === 1 && container.children[0].children &&
                !container.children[0].children.length)
                processElement(container.children[0]);
            else {
                var children = container.querySelectorAll('*');

                for (var i = 0; i < children.length; i++)
                    processElement(children[i]);
            }

            if (!parentTag) {
                for (var j = 0; j < htmlElements.length; j++)
                    htmlElements[j].innerHTML = INIT_SCRIPT_FOR_IFRAME_TEMPLATE + htmlElements[j].innerHTML;
            }

            return true;
        });
    };

    exports.createPropertyDesc = function (descBase) {
        descBase.configurable = true;
        descBase.enumerable   = true;
        return descBase;
    };

    exports.getAttributesProperty = function (el) {
        for (var i = 0; i < el.attributes.length; i++) {
            if (Util.isHammerheadAttr(el.attributes[i].name)) {
                AttributesWrapper.prototype = el.attributes;

                return new AttributesWrapper(el.attributes);
            }
        }

        return el.attributes;
    };
});
