HammerheadClient.define('DOMSandbox.Xhr', function (require, exports) {
    var NativeMethods = require('DOMSandbox.NativeMethods'),
        SharedConst   = require('Shared.Const'),
        UrlUtil       = require('UrlUtil'),
        Util          = require('Util');

    var IS_OPENED_XHR = 'hammerhead|xhr|is-opened-xhr';

    var xhr           = new NativeMethods.XMLHttpRequest();
    var corsSupported = typeof xhr.withCredentials !== 'undefined';

    //Event
    exports.XHR_COMPLETED = 'xhrCompleted';
    exports.XHR_ERROR     = 'xhrError';
    exports.XHR_SEND      = 'xhrSend';

    exports.eventEmitter = exports.eventEmitter || new Util.EventEmitter();

    exports.on = exports.eventEmitter.on.bind(exports.eventEmitter);

    exports.isOpenedXhr = function (obj) {
        return obj[IS_OPENED_XHR];
    };

    exports.createNativeXHR = function () {
        var xhr = new NativeMethods.XMLHttpRequest();

        xhr.open                = NativeMethods.xhrOpen;
        xhr.abort               = NativeMethods.xhrAbort;
        xhr.send                = NativeMethods.xhrSend;
        xhr.addEventListener    = NativeMethods.xhrAddEventListener;
        xhr.removeEventListener = NativeMethods.xhrRemoveEventListener;

        return xhr;
    };

    exports.init = function (window) {
        var xmlHttpRequestProto = window.XMLHttpRequest.prototype;

        xmlHttpRequestProto.abort = function () {
            NativeMethods.xhrAbort.apply(this, arguments);
            exports.eventEmitter.emit(exports.XHR_ERROR, {
                err: new Error('XHR aborted'),
                xhr: this
            });
        };

        //NOTE: redirect all requests to TestCafe proxy and ensure that request don't violate Same Origin Policy
        xmlHttpRequestProto.open = function () {
            var args = arguments;

            this[IS_OPENED_XHR] = true;

            //NOTE: emulate CORS, so 3rd party libs (e.g. jQuery) will allow requests with proxy host and
            //origin page host as well
            if (!corsSupported)
                this.withCredentials = false;

            if (typeof arguments[1] === 'string') {
                try {
                    args[1] = UrlUtil.getProxyUrl(arguments[1]);
                } catch (err) {
                    exports.eventEmitter.emit(exports.XHR_ERROR, {
                        err: err,
                        xhr: xhr
                    });

                    return;
                }
            }

            NativeMethods.xhrOpen.apply(this, args);
        };

        xmlHttpRequestProto.send = function () {
            var xhr = this;

            exports.eventEmitter.emit(exports.XHR_SEND, {
                xhr: xhr
            });

            var orscHandler = function () {
                if (xhr.readyState === 4)
                    exports.eventEmitter.emit(exports.XHR_COMPLETED, { xhr: xhr });
            };

            //NOTE: if we're in sync mode or it's in cache and has been retrieved directly (IE6 & IE7)
            //we need to manually fire the callback
            if (xhr.readyState === 4)
                orscHandler();
            else {
                //NOTE: get out of current execution tick and when proxy onreadystatechange.
                //Because e.g. jQuery assigns handler after send() was called.
                NativeMethods.setTimeout.call(window, function () {
                    //NOTE: if state already changed we just call handler without onreadystatechange proxying
                    var originalOnReadyStateChangeHandler = null;

                    try {
                        originalOnReadyStateChangeHandler = this.onreadystatechange;
                    }
                    catch (e) {
                        // NOTE: we should use another approaches to call orscHandler and not call
                        // xhr.onreadystatechange, if it's unable to be getted or setted (T468796)
                    }

                    if (xhr.readyState === 4)
                        orscHandler();

                    else if (typeof originalOnReadyStateChangeHandler === 'function') {
                        xhr.onreadystatechange = function (progress) {
                            orscHandler();
                            originalOnReadyStateChangeHandler.call(xhr, progress);
                        };
                    }
                    else if (xhr.addEventListener)
                        xhr.addEventListener('readystatechange', orscHandler, false);
                    else
                        xhr.attachEvent('onreadystatechange', orscHandler);

                }, 0);
            }

            /*jshint bitwise: false*/
            //NOTE: add XHR request mark, so proxy can recognize it as XHR request.
            //Due to the fact that all requests are passed to the proxy we need to perform all Same Origin Policy
            //compliance checks on server side. So we pass CORS support flag as well to inform proxy that it can
            //analyze Access-Control_Allow_Origin flag and skip "preflight" requests.
            NativeMethods.xhrSetRequestHeader.call(xhr, SharedConst.XHR_REQUEST_MARKER_HEADER,
                (corsSupported ? SharedConst.XHR_CORS_SUPPORTED_FLAG : 0) |
                (!!xhr.withCredentials ? SharedConst.XHR_WITH_CREDENTIALS_FLAG : 0)
            );
            /*jshint bitwise: true*/

            NativeMethods.xhrSend.apply(xhr, arguments);
        };
    };
});
