/* global isIFrameWithoutSrc:true */
HammerheadClient.define('UrlUtil', function (require) {
    var Settings = require('Settings'),
        SharedUrlUtil = require('Shared.UrlUtil');

    var SLASHED_PROTOCOLS = {
            'http:': true,
            'https:': true,
            'ftp:': true,
            'gopher:': true,
            'file:': true
        },
        URL_REGEXP = /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?(([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(#.*)?)/;

    var UrlUtil = {};

    //URL processing funcs
    UrlUtil.DOCUMENT_URL_RESOLVER = 'doc_url_resolver_8ff20d5e';
    UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY = SharedUrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY;
    UrlUtil.REQUEST_DESCRIPTOR_VALUES_SEPARATOR = SharedUrlUtil.REQUEST_DESCRIPTOR_VALUES_SEPARATOR;

    UrlUtil.IFRAME = SharedUrlUtil.IFRAME;
    UrlUtil.SCRIPT = SharedUrlUtil.SCRIPT;

    document[UrlUtil.DOCUMENT_URL_RESOLVER] = document.createElement('a');

    function getResolver(doc) {
        // IE clean up document after document.open call
        if(!doc[UrlUtil.DOCUMENT_URL_RESOLVER])
            doc[UrlUtil.DOCUMENT_URL_RESOLVER] = doc.createElement('a');

        return doc[UrlUtil.DOCUMENT_URL_RESOLVER];
    }

    UrlUtil.getProxyUrl = function (url, proxyHostname, proxyPort, jobUid, jobOwnerToken, resourceType) {
        if (!UrlUtil.isSupportedProtocol(url))
            return url;

        url = UrlUtil.resolveUrl(url);

        var parsedAsProxy   = UrlUtil.parseProxyUrl(url),
            isCompletedProxyUrl = !!parsedAsProxy;

        if (isCompletedProxyUrl) {
              if(resourceType && parsedAsProxy.resourceType === resourceType)
                  return url;
              else {
                  // NOTE: we need to change proxy url resource type
                  var destUrl = UrlUtil.formatUrl(parsedAsProxy.originResourceInfo);

                  return UrlUtil.getProxyUrl(destUrl, proxyHostname, proxyPort, jobUid, jobOwnerToken, resourceType);
              }
        }

        proxyHostname = proxyHostname || location.hostname;
        proxyPort = proxyPort || location.port.toString();
        jobUid = jobUid || Settings.JOB_UID;
        jobOwnerToken = jobOwnerToken || Settings.JOB_OWNER_TOKEN;

        var parsedUrl = UrlUtil.parseUrl(url),
            proxyHost = proxyHostname + ':' + proxyPort;

        if (parsedUrl.host === proxyHost && parsedUrl.protocol === location.protocol) {
            var originUrlObj = UrlUtil.getOriginUrlObj();

            var query = [
                originUrlObj.protocol,
                originUrlObj.host,
                jobUid,
                jobOwnerToken
            ];

            if (resourceType)
                query.push(resourceType);

            parsedUrl.query[UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY] = query.join(UrlUtil.REQUEST_DESCRIPTOR_VALUES_SEPARATOR);

            return UrlUtil.formatUrl(parsedUrl);
        }

        return SharedUrlUtil.getProxyUrl(url, proxyHostname, proxyPort, jobUid, jobOwnerToken, resourceType, UrlUtil.parseUrl, UrlUtil.formatUrl);
    };

    UrlUtil.getOriginUrlObj = function (proxyUrl) {
        proxyUrl = proxyUrl || location.toString();

        var parsedProxyUrl = UrlUtil.parseProxyUrl(proxyUrl),
            parsedUrl = UrlUtil.parseUrl(proxyUrl),
            query = proxyUrl ? parsedUrl.query : SharedUrlUtil.parseQueryString(location.search),
            result = {};

        result.protocol = parsedProxyUrl ? parsedProxyUrl.originResourceInfo.protocol : Settings.ORIGIN_URL_OBJ.protocol;
        result.port = parsedProxyUrl ? parsedProxyUrl.originResourceInfo.port : Settings.ORIGIN_URL_OBJ.port;
        result.hostname = parsedProxyUrl ? parsedProxyUrl.originResourceInfo.hostname : Settings.ORIGIN_URL_OBJ.hostname;
        result.host = parsedProxyUrl ? parsedProxyUrl.originResourceInfo.host : Settings.ORIGIN_URL_OBJ.host;
        result.pathname = parsedUrl.pathname;
        result.hash = parsedUrl.hash;
        result.query = query;

        if (result.query[UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY])
            delete result.query[UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY];

        result.search = SharedUrlUtil.formatQuery(result.query);

        return result;
    };

    UrlUtil.getCrossDomainIframeProxyUrl = function (url) {
        return UrlUtil.getProxyUrl(url, null, Settings.CROSS_DOMAIN_PROXY_PORT, null, null, UrlUtil.IFRAME);
    };

    UrlUtil.getCrossDomainProxyUrl = function () {
        return location.protocol + '//' + location.hostname + ':' + Settings.CROSS_DOMAIN_PROXY_PORT + '/';
    };

    UrlUtil.resolveUrl = function (url, doc) {
        url = SharedUrlUtil.prepareUrl(url);

        if (url && url.indexOf('//') === 0)
            url = UrlUtil.getOriginUrlObj().protocol + url;

        var urlResolver = getResolver(doc || document);

        if (url === null)
            urlResolver.removeAttribute('href');
        else {
            urlResolver.href = url;

            //NOTE: it looks like a chrome bug: in nested iframe without src (when iframe is placed in another iframe) you
            //cannot set relative link href for some time while the iframe loading is not completed. So, we'll do it with
            //parent's urlResolver
            if (url && !urlResolver.href && isIFrameWithoutSrc && window.parent && window.parent.document)
                return UrlUtil.resolveUrl(url, window.parent.document);
        }

        return urlResolver.href;
    };

    UrlUtil.resolveUrlAsOrigin = function (url) {
        return SharedUrlUtil.resolveUrlAsOrigin(url, UrlUtil.formatUrl, UrlUtil.getProxyUrl, UrlUtil.parseProxyUrl);
    };

    UrlUtil.parseUrl = function (urlStr) {
        urlStr = SharedUrlUtil.prepareUrl(urlStr);

        if (urlStr) {
            var isRelativePath = !/(^(\s)*(\w)+\:)|(^(\s)*\/\/)/.test(urlStr);

            if (isRelativePath && urlStr.indexOf('/') !== 0)
                urlStr = '/' + urlStr;
        }

        var urlParts = URL_REGEXP.exec(urlStr);

        return {
            protocol: urlParts[1] ? urlParts[1] + ':' : '',
            host: urlParts[2] || '',
            hostname: urlParts[3] || '',
            port: urlParts[4] || '',
            pathname: urlParts[6] || '',
            query: SharedUrlUtil.parseQueryString(typeof urlParts[9] === 'string' ? '?' + urlParts[9] : ''),
            hash: urlParts[10] || ''
        };
    };

    UrlUtil.convertToProxyUrl = function (url, resourceType) {
        return UrlUtil.getProxyUrl(url, null, null, null, null, resourceType);
    };

    UrlUtil.changeProxyUrl = function (url, prop, value, resourceType) {
        var parsedUrl = UrlUtil.parseUrl(url),
            parsedProxyUrl = UrlUtil.parseProxyUrl(url),
            newQuery = null;

        if (parsedProxyUrl) {
            var descr = [];

            if (prop === 'search') {
                newQuery = SharedUrlUtil.parseQueryString(value);

                newQuery[UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY] = parsedUrl.query[UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY];
                parsedUrl.query = newQuery;
            }

            descr.push(prop === 'protocol' ? value : parsedProxyUrl.originResourceInfo.protocol);

            if (prop === 'host')
                descr.push(value);
            else if (prop === 'hostname')
                descr.push(value + ':' + parsedProxyUrl.originResourceInfo.port);
            else if (prop === 'port')
                descr.push(parsedProxyUrl.originResourceInfo.hostname + ':' + value);
            else
                descr.push(parsedProxyUrl.originResourceInfo.host);

            descr.push(parsedProxyUrl.jobInfo.uid);
            descr.push(parsedProxyUrl.jobInfo.ownerToken);

            if (resourceType)
                descr.push(resourceType);

            newQuery = descr.join(UrlUtil.REQUEST_DESCRIPTOR_VALUES_SEPARATOR);

            parsedUrl.query[UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY] = newQuery;

            if (prop === 'hash')
                parsedUrl.hash = value;
            else if (prop === 'pathname')
                parsedUrl.pathname = value;

            return UrlUtil.formatUrl(parsedUrl);
        }

        return url;
    };

    UrlUtil.formatUrl = function (urlObj) {
        var protocol = urlObj.protocol || '',
            host = urlObj.host || '',
            search = urlObj.search || '',
            hash = urlObj.hash || '',
            pathname = urlObj.pathname ? urlObj.pathname.replace(/[?#]/g, function (match) {
                return encodeURIComponent(match);
            }) : '';

        if (protocol && protocol.charAt(protocol.length - 1) !== ':')
            protocol += ':';

        if (!host && urlObj.hostname) {
            host = urlObj.hostname;

            if (urlObj.port)
                host += ':' + urlObj.port;
        }

        if (host && (!protocol || SLASHED_PROTOCOLS[protocol] > -1)) {
            host = '//' + host;

            if (pathname && pathname.charAt(0) !== '/')
                pathname = '/' + pathname;
        }

        if (!search && urlObj.query)
            search = SharedUrlUtil.formatQuery(urlObj.query);

        search = search.replace('#', '%23');

        if (hash && hash.charAt(0) !== '#')
            hash = '#' + hash;

        return protocol + host + pathname + search + hash;
    };

    UrlUtil.isSubDomain = function(domain, subDomain) {
        return SharedUrlUtil.isSubDomain(domain, subDomain);
    };

    UrlUtil.sameOriginCheck = function (location, checkedUrl) {
        return SharedUrlUtil.sameOriginCheck(location, checkedUrl, UrlUtil.parseUrl, UrlUtil.resolveUrl);
    };

    UrlUtil.parseProxyUrl = function (proxyUrl) {
        return SharedUrlUtil.parseProxyUrl(proxyUrl, UrlUtil.parseUrl);
    };

    UrlUtil.isSupportedProtocol = function (url) {
        return SharedUrlUtil.isSupportedProtocol(url);
    };

    this.exports = UrlUtil;
});
