﻿#region DEMO_REMOVE
using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Text;
using Microsoft.Win32;
using System.Threading;
using System.Security;
#if !DXCommon
using DevExpress.Internal.Utils.About;
#endif
namespace DevExpress.Utils.About {
#if !DXALGO_PARSER_ONLY
internal class CRC32 {
        uint[] tab;
        uint poly = 0;
        public CRC32() : this(0x04c11db7) { }
        public CRC32(uint poly) {
            this.poly = poly;
        }
        void Init() {
            if(tab != null) return; 
            tab = new uint[256];
            for(uint i = 0; i < 256; i++) {
                uint t = i;
                for(int j = 0; j < 8; j++)
                    if((t & 1) == 0)
                        t >>= 1;
                    else
                        t = (t >> 1) ^ poly;
                tab[i] = t;
            }
        }
        static CRC32 _default;
        public static CRC32 Default {
            get {
                if(_default == null) _default = new CRC32();
                return _default;
            }
        }
        public uint ComputeHash(string text) {
            return ComputeHash(System.Text.Encoding.UTF8.GetBytes(text));
        }
        public uint ComputeHash(byte[] data) {
            return ComputeHash(data, 0, data.Length);
        }
        public virtual uint ComputeHash(byte[] data, int start, int length) {
            return ComputeHash<byte[]>(data, start, length);
        }
        public virtual uint ComputeHash<T>(T data, int start, int length) where T : System.Collections.IList {
            Init();
            uint hash = 0xFFFFFFFF;
            for(int n = 0; n < length; n++) {
                byte b = (byte)data[n + start];
                hash = (hash << 8)
                    ^ tab[b ^ (hash >> 24)];
            }
            return ~hash;
        }
    }
    internal class AlgoProvider {
        static Guid? id;
        internal static Guid? Id {
            get {
                if(id == null) id = GetId();
                return id;
            }
        }

        static Guid? GetId() {
            try {
                Guid res = Guid.NewGuid(); 
                string path = @"Licenses\5FCB5B35-D11E-4374-AD58-19A608D8228C";
                RegistryKey key = Registry.ClassesRoot.OpenSubKey(path);
                if(key == null) {
                    key = Registry.ClassesRoot.CreateSubKey(path);
                    key.SetValue(null, res.ToString());
                }
                else {
                    res = new Guid((string)key.GetValue(null));
                }
                return res;
            }
            catch {
            }
            return null;
        }
    }
    public class UAlgo {
        static byte? status = null;
        static bool? enabled = null;
        public static bool Enabled {
            get {
                if(status == null) return false;
                if(enabled == null) {
                    enabled = CheckEnabled();
                }
                return enabled.Value;
            }
            set {
                enabled = value;
            }
        }
        static byte? defaultPlatform = 0;
        public static void Wpf() {
            defaultPlatform = pWPF;
        }
        public static void WinForms() {
            defaultPlatform = pWinForms;
        }
        public static void Web() {
            defaultPlatform = pWeb;
        }
        public static void DemoMode() {
            status = kDemo;
        }
        public static void DesignMode() {
            status = kDesignTime;
        }
        public static void ASPDesigner() {
            status = kDesignTime;
            defaultPlatform = pWeb;
            if(Enabled) return;
        }
        public static void WinFormsDesigner() {
            status = kDesignTime;
            defaultPlatform = pWinForms;
            if(Enabled) return;
        }
        [SecuritySafeCritical]
        static bool CheckEnabled() {
            try {
                if(!CheckValidRegistry()) return false;
                if(AlgoProvider.Id == null) return false;
                AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
                return true;
            }
            catch {
                return false;
            }
        }
        static RegistryKey GetKey(bool writable) {
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\DevExpress", writable);
            return key;
        }
        static bool CheckValidRegistry() {
            RegistryKey key = GetKey(false);
            if(key == null) return false;
            object value = key.GetValue("TrialSendInfo");
            if(value == null) return false;
            if(string.IsNullOrEmpty(value.ToString())) return false;
            return true;
        }

        static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e) {
            Default.DoEventException(e);
        }
        byte LastPlatform = 0;
        public const byte pUnknown = UAlgoConstants.pUnknown, pWinForms = UAlgoConstants.pWinForms, pWeb = UAlgoConstants.pWeb, pWPF = UAlgoConstants.pWPF, pSilverlight = UAlgoConstants.pSilverlight, pWinRT = UAlgoConstants.pWinRT, pXAF = UAlgoConstants.pXAF, pCodeRush = UAlgoConstants.pCodeRush;
        public const byte kDemo = UAlgoConstants.kDemo, kDemoMain = UAlgoConstants.kDemoMain, kDemoModule = UAlgoConstants.kDemoModule, kDesignTime = UAlgoConstants.kDesignTime, kDesignTimeFrame = UAlgoConstants.kDesignTimeFrame, kDesignTimeWizard = UAlgoConstants.kDesignTimeWizard, kDesignTimeTemplate = UAlgoConstants.kDesignTimeTemplate, kStack = UAlgoConstants.kStack, kExceptionDemo = UAlgoConstants.kExceptionDemo, kExceptionVisualStudio = UAlgoConstants.kExceptionVisualStudio, kInstall = UAlgoConstants.kInstall, kUnInstall = UAlgoConstants.kUnInstall,
            kExceptionPaintDesign = UAlgoConstants.kExceptionPaintDesign, kExceptionPaint = UAlgoConstants.kExceptionPaint;
        [MethodImpl(MethodImplOptions.NoInlining)]
        public void DoStackEvent(byte platform) {
            System.Diagnostics.StackTrace st = new System.Diagnostics.StackTrace();
            System.Diagnostics.StackFrame sf = st.GetFrame(1);
            MethodBase mb = sf.GetMethod();
            DoEvent(kStack, platform, string.Format("{0}.{1}", mb.DeclaringType.FullName, mb.Name));
        }
        public void DoEventTemplate(byte platform, object instance, object[] customParams) {
            try {
                string action = instance.GetType().FullName;
                if(customParams != null && customParams.Length > 0) {
                    action = System.IO.Path.GetFileName(customParams[0].ToString());
                }
                DoEvent(kDesignTimeTemplate, platform, action);
            }
            catch {
            }
        }
        public void DoEventException(UnhandledExceptionEventArgs e) {
            try {
                Exception eo = e.ExceptionObject as Exception;
                DoEvent(kExceptionVisualStudio, LastPlatform, GetText(eo));
            }
            catch { }
        }
        string GetText(Exception eo) {
            return GetInfo() + GenerateExceptionReportCore(eo);
            //MethodBase mb = eo == null ? null : eo.TargetSite;
            //string res = string.Empty;
            //if(mb != null) {
            //    res += string.Format("{0}.{1};", mb.DeclaringType.FullName, mb.Name);
            //}
            //res += eo.Message + ";" + eo.StackTrace;
            //return res;
        }
        string GetInfo() {
            try {
                string proc = System.Diagnostics.Process.GetCurrentProcess().ProcessName;
                string res = string.Format("OS {0}, x64:{1},{2}", Environment.OSVersion.ToString(), Environment.Is64BitOperatingSystem, Environment.Is64BitProcess);
                string memory = (System.Diagnostics.Process.GetCurrentProcess().WorkingSet64 / (1024 * 1024)).ToString() + "mb";
                string display = string.Format("{0}x{1}", System.Windows.Forms.SystemInformation.WorkingArea.Size.Width, System.Windows.Forms.SystemInformation.WorkingArea.Size.Height);
                return string.Concat(proc, "\r\n", res, "\r\n", memory, "\r\n", display, "\r\n");
            }
            catch {
            }
            return "";
        }
        static string GetExceptionType(Exception e) {
            Type type = e.GetType();
            if(type.FullName != null && type.FullName.StartsWith("System.")) return type.Name;
            return type.FullName;
        }
        string GenerateExceptionReportCore(Exception e) {
            if(e == null) return "**** Exception is null *** ";
            try {
                return GenerateExceptionReport(e);
            }
            catch {
                return "*** XR ***";
            }
        }
        static string GenerateExceptionReport(Exception e) {
            StringBuilder sb = new StringBuilder();
            Exception current = e;
            string indent = "";
            int currentStack = 0;
            while(current != null && currentStack < 4) {
                sb.AppendFormat("{0}---- {1}", indent, GetExceptionType(current));
                sb.AppendLine();
                sb.AppendFormat("{0}{1}", indent, current.Message);
                sb.AppendLine();
                sb.AppendFormat("{0}Stack:{1}", indent, current.StackTrace);
                sb.AppendLine();
                if(current.Data != null) {
                    foreach(var key in current.Data.Keys) {
                        var value = current.Data[key];
                        sb.AppendFormat("{0}  #{1} == {2}", indent, key, value == null ? "#null" : value);
                        sb.AppendLine();
                    }
                }
                if(current.InnerException != null) {
                    indent += new string(' ', 4);
                    sb.AppendFormat("{0} *** Inner Exception", indent);
                    sb.AppendLine();
                }
                current = current.InnerException;
                currentStack++;
            }
            return sb.ToString();
        }


        string Dump(Exception e) {
            if(e == null) return string.Empty;
            try {
                string exception = "1." + e.GetType().FullName;
                string message = "2." + e.Message;
                string stack = "3." + e.StackTrace;
                return string.Join("\r\n", exception, message, stack);
            }
            catch {
                return "*** XR";
            }
        }
        public void DoEventInstall(byte kind) {
            DoEvent(kind, pWinForms, kind == kInstall ? "Install" : "UnInstall");
        }
        public void DoEventObject(byte kind, object instance) { DoEventObject(kind, pUnknown, instance); }
        public void DoEventObject(byte kind, byte platform, object instance) {
            if(instance == null) return;
            try {
                string fullName = instance.GetType().FullName;
                if(IsDemo(kind) || fullName.ToLowerInvariant().StartsWith("devexpress."))
                    DoEvent(kind, platform, instance.GetType());
            }
            catch {
            }
        }
        public void DoEvent(byte kind, Type action) { DoEvent(kind, pUnknown, action); }
        public void DoEvent(byte kind, byte platform, Type action) {
            try {
                DoEvent(kind, platform, action.FullName);
            }
            catch { }
        }
        public void DoEvent(byte kind, string action) { DoEvent(kind, pUnknown, action); }
        public void DoEvent(byte kind, byte platform, string action) {
            DoEvent(kind, platform, action, CRC32.Default.ComputeHash(action));
        }
        [ThreadStatic]
        static UAlgo _default;
        public static UAlgo Default {
            get {
                if(_default == null) _default = new UAlgo();
                return _default;
            }
            
        }
        bool CheckFolder(string path) {
            if(string.IsNullOrEmpty(path)) return false;
            try {
                if(Directory.Exists(path)) return true;
                Directory.CreateDirectory(path);
                return true;
            }
            catch {
                return false;
            }
        }
        List<byte> list = new List<byte>();
        void DoEvent(byte kind, byte platform, string action, uint actionNumber) {
            try {
                if(string.IsNullOrEmpty(action)) return;
                CheckUpdateStatus(kind);
                platform = CheckPlatform(platform);
                LastPlatform = platform;
                if(!Enabled) return;
                long time = DateTime.UtcNow.ToFileTime();
                int index = list.Count;
                list.AddRange(BitConverter.GetBytes(time));
                list.Add(kind);
                list.Add(platform);
                list.AddRange(BitConverter.GetBytes(actionNumber));
                byte[] textUTF = UTF8Encoding.Default.GetBytes(action.Substring(0, Math.Min(action.Length, GetTextLength(kind))));
                list.AddRange(BitConverter.GetBytes((short)textUTF.Length)); //custom record -> may be will use in future
                list.AddRange(textUTF);
#if DEBUGTEST
                if(LogRecord != null) LogRecord(this, EventArgs.Empty);
                DoEventDebug(time, kind, platform, action, actionNumber);
#endif
                CheckFlush();
            }
            catch {
            }
        }

         int GetTextLength(byte kind) {
             if(kind == kExceptionDemo || kind == kExceptionPaint || kind == kExceptionPaintDesign || kind == kExceptionVisualStudio) return 2048;
             return 80;
        }
        void CheckUpdateStatus(byte kind) {
            if(status != null) return;
            if(IsDemo(kind)) status = kDemo;
            if(kind == kDesignTime || kind == kDesignTimeFrame || kind == kDesignTimeWizard || kind == kDesignTimeTemplate) status = kind;
            if(kind == kInstall || kind == kUnInstall) status = kind;
        }
        byte CheckPlatform(byte platform) {
            if(platform == pUnknown) platform = defaultPlatform.HasValue ? defaultPlatform.Value : LastPlatform;
            return platform;
        }
        bool IsDemo(byte kind) {
            if(kind == kDemo || kind == kDemoModule || kind == kDemoMain) return true;
            return false;
        }
#if DEBUGTEST
        public event EventHandler LogRecord;
        void DoEventDebug(long time, byte kind, byte platform, string action, uint actionNumber) {
            try {
                string text = string.Format("{0}: {1} - {2} {3:x4} {4}", DateTime.FromFileTimeUtc(time), kind, platform, actionNumber, action);
                System.Diagnostics.Debug.WriteLine(text);

                string path = System.Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\DevExpress";
                if(!CheckFolder(path)) return;
                using(StreamWriter sw = new StreamWriter(path + @"\udx" + Version + ".log.txt", true)) {
                    sw.WriteLine(text);
                }
                
            }
            catch {
            }
        }
#endif
#if !DXCommon
        string Version { get { return DevExpress.Internal.AssemblyInfo.VersionShort; } } //"12.2"
        int VersionId { get { return DevExpress.Internal.AssemblyInfo.VersionIdMinor; } } //"12.2"
#else
        int VersionId { get { return AssemblyInfo.VersionIdMinor; } } //"12.2"
        string Version { get { return AssemblyInfo.VersionShort; } } //"12.2"
#endif
        void CheckFlush() {
            FlushList();
        }
        string GetPath() {
            string res = System.Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\DevExpress";
            if(!CheckFolder(res)) return null;
            return path = res + @"\udx" + Version + ".log";
        }
        string path = null;
        string Path {
            get {
                if(path == null) {
                    path = GetPath();
                }
                return path;
            }
        }
        void FlushList() {
            if(list == null || list.Count == 0) return;
            try {
                if(!Enabled) return;
                if(Path == null) return;
                using(FileStream fs = new FileStream(path, FileMode.Append, FileAccess.Write)) {
                    byte[] bytes = list.ToArray();
                    fs.Write(bytes, 0, bytes.Length);
                }
                bool post = CheckPost();
                if(post) PostInfo.Post();
            }
            catch {
            }
            this.list.Clear();
        }
        void DoPost(int wait) {
            if(!Enabled || Path == null) return;
            if(PostInfo.Post()) {
                DateTime now = DateTime.Now;
                while(PostInfo.IsWorking) {
                    Thread.Sleep(100);
                    if(now.Subtract(DateTime.Now).TotalMilliseconds > wait) break;
                }
            }
        }
        UAlgoPost postInfo;
        UAlgoPost PostInfo {
            get {
                if(postInfo == null) postInfo = new UAlgoPost(path, AlgoProvider.Id.Value, VersionId);
                return postInfo;
            }
        }
        bool CheckPost() {
            if(!AlgoProvider.Id.HasValue) return false;
            if(PostInfo.disabled) {
                Disable();
                return false;
            }
            if(!File.Exists(path)) return false;
            FileInfo fi = new FileInfo(path);
            if(fi.Length > 0) return true;
            return false;
        }

        void Disable() {
            enabled = false;
            RegistryKey key = GetKey(true);
            if(key == null) return;
            key.SetValue("TrialSendInfo", "");
            key.Close();
            return;
        }
    }
#endif
#if DXALGO
    internal class UAlgoParser {
        public List<byte> list = new List<byte>();
        public void DoEvent(byte kind, byte platform, string action, uint actionNumber) {
            if(string.IsNullOrEmpty(action)) return;
            try {
                long time = DateTime.UtcNow.ToFileTime();
                list.AddRange(BitConverter.GetBytes(time));
                list.Add(kind);
                list.Add(platform);
                list.AddRange(BitConverter.GetBytes(actionNumber));
                byte[] textUTF = UTF8Encoding.Default.GetBytes(action.Substring(0, Math.Min(action.Length, 80)));
                list.AddRange(BitConverter.GetBytes((short)textUTF.Length)); //custom record -> may be will use in future
                list.AddRange(textUTF);
            }
            catch {
            }
        }
        const int MinRecordLength = 16;
        public List<UAlgoRecord> Parse(byte[] bytes) { return UAlgoParser.DoParse(bytes); }
        public static List<UAlgoRecord> DoParse(byte[] bytes) {
            List<UAlgoRecord> res = new List<UAlgoRecord>();

            int pos = 0;
            while(pos < bytes.Length) {
                UAlgoRecord rec = new UAlgoRecord();
                if(bytes.Length - pos < MinRecordLength) break; //
                long fileTime = BitConverter.ToInt64(bytes, pos);
                rec.Kind = bytes[pos + 8];
                rec.Platform = bytes[pos + 9];
                rec.CRC32 = BitConverter.ToUInt32(bytes, pos + 10);
                rec.CustomRecordLength = BitConverter.ToInt16(bytes, pos + 14);
                if(rec.CustomRecordLength > 0) {
                    try {
                        rec.Text = UTF8Encoding.Default.GetString(bytes, pos + 16, rec.CustomRecordLength);
                    }
                    catch { return null; }
                }
                pos += MinRecordLength + rec.CustomRecordLength;
                try {
                    rec.Date = DateTime.FromFileTimeUtc(fileTime);
                    if(rec.Date.Year < 2013 || rec.Date.Year > DateTime.UtcNow.Year) return null;
                }
                catch {
                    return null; //error
                }
                res.Add(rec);

            }

            return res;
        }
    }
    internal class UAlgoRecord {
        public int CustomRecordLength { get; set; }
        public DateTime Date { get; set; }
        public byte Platform { get; set; }
        public byte Kind { get; set; }
        public uint CRC32 { get; set; }
        public string Text { get; set; }

        //DB Only
        public Guid Computer { get; set; }
        public int Version { get; set; }
    }
#endif
}
#if DXALGO && DEBUGTEST
namespace DevExpress.Utils.About.Tests {
    using NUnit.Framework;
    [TestFixture]
    public class UAlgoParseTests {
        [Test]
        public void TestSimple() {
            UAlgoParser p = new UAlgoParser();
            DateTime dt = DateTime.UtcNow;
            p.DoEvent((byte)(1), (byte)(2), "hello world", (uint)(123456789));
            List<UAlgoRecord> records = p.Parse(p.list.ToArray());
            Assert.AreEqual(1, records.Count);
            UAlgoRecord r = records[0];
            Assert.AreEqual(1, r.Kind);
            Assert.AreEqual(2, r.Platform);
            Assert.AreEqual(11, r.CustomRecordLength);
            Assert.AreEqual((uint)(123456789), r.CRC32);
            Assert.LessOrEqual(r.Date.Subtract(dt).TotalSeconds, 10);
            Assert.AreEqual("hello world", r.Text);
        }
        [Test]
        public void Test() {
            UAlgoParser p = new UAlgoParser();
            for(int n = 0; n < 20; n++) {
                p.DoEvent((byte)(1 + n), (byte)(2 + n), "hello world", (uint)(123456789 + n));
            }
            List<UAlgoRecord> records = p.Parse(p.list.ToArray());
            Assert.AreEqual(20, records.Count);
            Assert.AreEqual(1, records[0].Kind);
            Assert.AreEqual(20, records[19].Kind);
        }
    }
}
#endif
#endregion DEMO_REMOVE