﻿#region DEMO_REMOVE
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;

namespace DevExpress.Utils.About {
    internal class UAlgoPost {
        Thread postThread = null;
        string path;
        Guid id;
        int version;
        static string url = "http://uxlog.devexpress.com?id={0}&version={1}";
        public UAlgoPost(string path, Guid id, int version) {
            this.path = path;
            this.id = id;
            this.version = version;
        }
        public bool Post() {
            if(postThread != null) return false;

            this.postThread = new Thread(OnStart);
            this.postThread.Start();
            return true;
        }
        protected void OnStart() {
            try {
                ProcessData();
            }
            catch {
            }
            this.postThread = null;

        }
        public bool IsWorking { get { return postThread != null; } }
        void ProcessData() {
            if(!File.Exists(path)) return;
            byte[] data = null;
            using(FileStream fs = new FileStream(path, FileMode.Open, FileAccess.Read)) {
                data = new byte[fs.Length];
                if(fs.Read(data, 0, data.Length) != data.Length) return;
            }
            var response = XmlHttpRequest.Submit(
                string.Format(url, id.ToString().Replace("-", ""), version),
                "POST",
                null,
                data);
            if(response.Content == null)
                File.Delete(path);
            if(response.ContentText == "DISABLED") disabled = true;
        }
        internal bool disabled = false;
    }
    public static class XmlHttpRequest {
        public static WebRequest Create(Uri uri) {
            if(uri == null) {
                throw new ArgumentNullException("uri");
            }
            if(!uri.IsAbsoluteUri) {
                throw new ArgumentOutOfRangeException("uri",
                    string.Format("The Request URI must be absolute. URI: {0}", uri));
            }
            return WebRequest.Create(uri.ToString());
        }
        public static void GetResponseBytes(Stream source, ref byte[] buffer) {
            const int BufferSize = 0x1000;
            byte[] bytes = new byte[BufferSize];
            int bytesRead;
            while((bytesRead = source.Read(bytes, 0, bytes.Length)) > 0) {
                byte[] responseBytes2 = new byte[bytesRead +
                    (buffer != null && buffer.Length > 0 ? buffer.Length : 0)];
                if(buffer != null && buffer.Length > 0) {
                    System.Buffer.BlockCopy(
                        buffer,
                        0,
                        responseBytes2,
                        0,
                        buffer.Length);
                }
                System.Buffer.BlockCopy(
                    bytes,
                    0,
                    responseBytes2,
                    responseBytes2.Length - bytesRead,
                    bytesRead);
                buffer = responseBytes2;
            }
        }
        public static byte[] GetResponseBytes(WebResponse response) {
            using(Stream source = response.GetResponseStream()) {
                byte[] buffer = null;
                GetResponseBytes(source, ref buffer);
                return buffer;
            }
        }
        public static string GetRequestUriString(Uri baseUri, string relativeUri, params object[] args) {
            return GetRequestUriString(baseUri, string.Format(relativeUri, args));
        }
        public static string GetRequestUriString(Uri baseUri, string relativeUri) {
            if(baseUri == null) {
                throw new ArgumentNullException("baseUri");
            }
            if(string.IsNullOrEmpty(relativeUri)) {
                throw new ArgumentNullException("relativeUri");
            }
            int start = 0;
            while(start < relativeUri.Length && (relativeUri[start] == '/' || relativeUri[start] == ' ')) {
                start++;
            }
            string baseUriString = baseUri.ToString();
            int baseUriLen = baseUriString.Length;
            while(baseUriLen > 0 &&
                        (baseUriString[baseUriLen - 1] == '/' || baseUriString[baseUriLen - 1] == ' ')) {
                baseUriLen--;
            }
            return string.Format("{0}/{1}",
                baseUriString.Substring(0, baseUriLen),
                relativeUri.Substring(start));
        }
        public static XmlHttpResponse Submit(Uri uri, string method, string authorization) {
            return Submit(uri.ToString(), method, null);
        }
        public static XmlHttpResponse Submit(string uriString, string method, string authorization) {
            return Submit(uriString, method, authorization, null);
        }
        public static XmlHttpResponse Submit(Uri uri, string method, string authorization, byte[] content) {
            return Submit(uri.ToString(), method, authorization, content);
        }
        public static XmlHttpResponse Submit(Uri baseUri, string relativeUri, string method, string authorization, byte[] content, params object[] args) {
            return Submit(
                GetRequestUriString(baseUri, relativeUri, args),
                method,
                authorization,
                content);
        }
        public static XmlHttpResponse Submit(string uriString, string method, string authorization, byte[] content) {
            HttpWebRequest req = (HttpWebRequest)XmlHttpRequest.Create(new Uri(uriString, UriKind.RelativeOrAbsolute));
            req.Method = method;
            req.Headers["X-HTTP-Method-Override"] = method;
            if(!string.IsNullOrEmpty(authorization)) {
                req.Headers["X-Authorization"] = authorization;
            }
            req.ContentType = "app/x-www-form-urlencoded";
            if(content != null && content.Length > 0) {
                using(var stream = req.GetRequestStream()) {
                    stream.Write(
                        content,
                        0,
                        content.Length);
                }
            }
            using(WebResponse response = req.GetResponse()) {
                return new XmlHttpResponse(
                    GetResponseBytes(response),
                    response.ContentType);
            }
        }
        public struct XmlHttpResponse {
            public static readonly XmlHttpResponse Empty = new XmlHttpResponse();
            public XmlHttpResponse(byte[] content, string contentType) {
                _content = content;
                _contentType = contentType;
            }
            byte[] _content;
            public byte[] Content {
                get {
                    return _content;
                }
            }
            string _contentType;
            public string ContentType {
                get {
                    return _contentType;
                }
            }
            public string ContentText {
                get {
                    if(!string.IsNullOrEmpty(_contentType) && _content != null) {
                        if(_contentType.ToLower().StartsWith("text/")) {
                                return Encoding.UTF8.GetString(_content, 0, _content.Length);
                        }
                    }
                    return string.Empty;

                }
            }
            public override string ToString() {
                return ContentText;
            }
        }
    }
}
#endregion