var async = require('async'),
    fs = require('fs'),
    path = require('path'),
    uuid = require('node-uuid'),
    conn = new (require('ssh2'));

var DEBUG_FLAG = false;

var BUILD_UUID = uuid.v4(),
    BUILD_FS_TIMEOUT = 1e4,
    BUILD_INSTALLER_NAME = "TestCafe",
    BUILD_SERVER_LOGIN = 'admin',
    BUILD_SERVER_PASSWORD = '123',
    BUILD_SERVER_HOST = DEBUG_FLAG ? 'admins-mac' : 'osx-testcafe',
    BUILD_SERVER_LOCATION = ['./Desktop/TestCafe', BUILD_UUID].join('/'),
    BUILD_TGZ_NAME = 'mac_dist.tgz',
    BUILD_TGZ_LOCATION = [BUILD_SERVER_LOCATION, BUILD_TGZ_NAME].join('/'),
    BUILD_SCRIPT_NAME = 'build_installer.sh',
    BUILD_SCRIPT_LOCATION = [BUILD_SERVER_LOCATION, BUILD_SCRIPT_NAME].join('/'),
    BUILD_DMG_FILE = [BUILD_SERVER_LOCATION, 'DevExpress-TestCafe.dmg'].join('/'),
    FORMATTED_SCRIPT_NAME = 'build.sh';

exports.buildMacBySSH = function (sourceTgz, outputDir, buildName, success, fail) {
    var options = {
        host: BUILD_SERVER_HOST,
        port: 22,
        tryKeyboard: true,
        username: BUILD_SERVER_LOGIN,
        password: BUILD_SERVER_PASSWORD,
        readyTimeout: 1e7
    };

    conn.on('keyboard-interactive', function () {
        arguments[4]([BUILD_SERVER_PASSWORD]);
    });

    conn.on('ready', function () {
        var sftp = null;

        async.series({
            'Create sftp connection': function (done) {
                conn.sftp(function (err, sftpConn) {
                    if (err)
                        done(err);

                    sftp = sftpConn;
                    done();
                });
            },
            'Make base dir': function (done) {
                sftp.mkdir(BUILD_SERVER_LOCATION, ['-m', '777', '-p'], function (err) {
                    done(err);
                });
            },
            'Upload dist': function (done) {
                sftp.fastPut(sourceTgz, BUILD_TGZ_LOCATION, function (err) {
                    done(err);
                });
            },
            'Untar dist': function (done) {
                conn.exec('cd ' + BUILD_SERVER_LOCATION + ' && tar xf ./' + BUILD_TGZ_NAME, function (err, stream) {
                    stream.stderr.on('data', done);
                    stream.on('close', done);
                });
            },
            'Set permission': function (done) {
                sftp.chmod(BUILD_SERVER_LOCATION, '777', function () {
                    sftp.chmod(BUILD_SCRIPT_LOCATION, '777', done)
                });
            },
            'Convert windows new line symbols': function (done) {
                conn.exec(['cd ', BUILD_SERVER_LOCATION, ' && cat ' + BUILD_SCRIPT_NAME, ' | col -b > ', FORMATTED_SCRIPT_NAME].join(''), function (err, stream) {
                    stream.stderr.on('data', function (err) {
                        done('1: ' + err);
                    });
                    stream.on('close', done);
                });
            },
            'Build dist': function (done) {
                conn.exec(['cd', BUILD_SERVER_LOCATION, '&& chmod 777 ./' + FORMATTED_SCRIPT_NAME + '&& ./' + FORMATTED_SCRIPT_NAME, BUILD_INSTALLER_NAME].join(' '), function (err, stream) {
                    stream.stderr.on('data', function (err) {
                        done(err);
                    });
                    stream.on('close', done);
                });
            },
            'Download dmg installer': function (done) {
                sftp.fastGet(BUILD_DMG_FILE, path.join(outputDir, buildName), function (err) {
                    done(err);
                });
            }
        }, function (buildErr) {
            //NOTE: clear resources
            conn.exec('hdiutil detach -force"' + BUILD_INSTALLER_NAME + '"', function (err, stream) {
                stream.stderr.on('data', function (err) {
                    //NOTE: we ignore detach errors, because in the case of success disk is already ejected
                });
                stream.on('close', function () {
                    setTimeout(function () {
                        conn.exec('rm -rf ' + BUILD_SERVER_LOCATION, function (err, stream) {
                            stream.stderr.on('data', function (err) {
                                buildErr += '\n' + err.toString();
                            });

                            stream.on('close', function () {
                                conn.end();

                                if (!buildErr)
                                    success();
                                else
                                    fail(buildErr);
                            });
                        });
                    }, BUILD_FS_TIMEOUT);
                });
            });
        });
    }).connect(options);
};
