#!/usr/bin/env bash
#Constants
installerTitle=$1
appTitle='TestCafe'
appIcon='./icon.icns'
backgroundFileSrc='./back.png'
backgroundFileSrcX2='./back_x2.png'
backgroundFile='back.tiff'
distFileName='./testcafe.dmg'
licenseIcon='./license.icns'
installerLocation="$(pwd)/${installerTitle}"

#Disable path bar. If path bar is enable, height for .dmg calculates incorrect
defaults write com.apple.finder ShowPathbar -bool false
killall Finder

#Copy build files
mkdir -p ./build
mkdir -p ./build/.background

#Create multi resolution tiff for support retina
tiffutil -cathidpicheck ${backgroundFileSrc} ${backgroundFileSrcX2} -out "${backgroundFile}" &> /dev/null
cp ./back.tiff ./build/.background/back.tiff
chmod -Rf go-w ./build

#Set icon for license file
sips -i ${licenseIcon}
DeRez -only icns ${licenseIcon} > ./license.rsrc

#Create stubs of build files
touch ./build/EULA.txt
mkdir ./build/TestCafe.app
ln -s /Applications "./build/Applications"

Rez -append ./license.rsrc -o ./build/EULA.txt
SetFile -a C ./build/EULA.txt

#Create .dmg image
hdiutil create -srcfolder ./build -volname "${installerTitle}" -fs HFS+ -fsargs "-c c=64,a=16,e=16" -format UDRW -size 100m ${distFileName}
device=$(hdiutil attach -mountpoint "${installerTitle}" -readwrite -noverify -noautoopen "${distFileName}" | egrep '^/dev/' | sed 1q | awk '{print $1}')

#Set icon for disk image
cp ${appIcon} "$installerLocation/.VolumeIcon.icns"
SetFile -a C "$installerLocation"

sleep 5

#Customize installer layout
echo '
    tell application "Finder"
        tell disk "'${installerTitle}'"
            open
                delay 5
                set current view of container window to icon view
                set toolbar visible of container window to false
                set statusbar visible of container window to false
                set the bounds of container window to {400, 100, 945, 505}
                set theViewOptions to the icon view options of container window
                set arrangement of theViewOptions to not arranged
                set icon size of theViewOptions to 112
                set background picture of theViewOptions to file ".background:'${backgroundFile}'"
                set the position of item "EULA.txt" to {410, 85}
                set the position of item "'${appTitle}'" to {120, 245}
                set the position of item "Applications"  to {410, 245}
                update without registering applications
                delay 5
        end tell
    end tell
' | osascript

sync
hdiutil detach -force ${device}
zip -r testcafe-tmpl.zip ${distFileName}