//
//  AppDelegate.m
//  TestCafe
//
//  Copyright (c) 2017 DevExpress. All rights reserved.
//

#import "AppDelegate.h"
#import <ServiceManagement/ServiceManagement.h>
#import <Security/Authorization.h>

@implementation AppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    [self createDeafaultDirectoryForProjects];
    
    basePath = [NSString stringWithFormat: @"%@/testcafe", [[NSBundle mainBundle] resourcePath]];
    nodePath = [NSString stringWithFormat: @"%@/node", basePath];
    controlPanelHost = [[NSString stringWithFormat: @"%@", [self getControlPanelHost]] retain];

    [self runNode];
}

- (void)menuWillOpen:(NSMenu *)statusMenu
{
    NSImage *statusItemImage = [NSImage imageNamed:@"testcafe-white"];
    [statusItem setImage:statusItemImage];
}

- (void)menuDidClose:(NSMenu *)statusMenu
{
    NSImage *statusItemImage = [NSImage imageNamed:@"testcafe-black"];
    [statusItem setImage:statusItemImage];
}

- (void)awakeFromNib{
    statusItem = [[[NSStatusBar systemStatusBar] statusItemWithLength:NSVariableStatusItemLength] retain];
    NSImage *statusItemImage = [NSImage imageNamed:@"testcafe-black"];
    
    [statusItem setMenu:statusMenu];    
    [statusItem setImage:statusItemImage];
    [statusItem setHighlightMode:YES];
    
    [statusMenu setDelegate:self];
}

- (void)showNotification{
    SInt32 OSXVersionMinor;
    
    Gestalt(gestaltSystemVersionMinor, &OSXVersionMinor);
    
    if(OSXVersionMinor > 7)
    {
        NSUserNotification *notification = [[NSUserNotification alloc] init];
    
        notification.title = @"TestCafe";
        notification.informativeText = @"Control Panel URL has been copied to clipboard.";
        notification.soundName = NSUserDefaultsDidChangeNotification;
    
        [[NSUserNotificationCenter defaultUserNotificationCenter] deliverNotification: notification];
    
        [notification release];
    }
}

- (NSString *)getControlPanelHost{
    NSTask *task;
    task = [[NSTask alloc] init];
    NSArray *arguments;
    arguments = [NSArray arrayWithObjects: @"-e", @"var Config=require('./lib/config'); var config=new Config(); config._initFromFile(function() {console.log(config.controlPanelHost);});", nil];
    
    [task setLaunchPath: nodePath];
    [task setCurrentDirectoryPath: basePath];
    [task setArguments: arguments];
    
    NSPipe *pipe;
    pipe = [NSPipe pipe];
    [task setStandardOutput: pipe];
    
    NSFileHandle *file;
    file = [pipe fileHandleForReading];
    
    [task launch];
    
    NSData * data;
    data = [file readDataToEndOfFile];

    NSString *formattedHost = [[NSString alloc] initWithData: data encoding: NSUTF8StringEncoding];
    formattedHost = [formattedHost stringByTrimmingCharactersInSet:[NSCharacterSet newlineCharacterSet]];
    formattedHost = [NSString stringWithFormat: @"http://%@/", formattedHost];
    formattedHost = [formattedHost stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    
    [task release];
  
    return formattedHost;
}

- (void)createDeafaultDirectoryForProjects{
    NSFileManager *fileManager = [[NSFileManager alloc] init];
    BOOL isDir;
    
    if(![fileManager fileExistsAtPath:@"/Library/Application Support/TestCafe" isDirectory:&isDir] || !isDir){
        OSStatus status;
        AuthorizationRef authRef;
        FILE *pipe = NULL;
        char *command = "/bin/mkdir";
        char *args[] = {"-m", "777", "/Library/Application Support/TestCafe", NULL};

        status = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authRef);
        
        if(status != errAuthorizationSuccess) {
            [self terminateTestCafe];
        }
        
        AuthorizationItem authRightsItem = {kAuthorizationRightExecute, 0, NULL, 0};
        AuthorizationRights authRights = {1, &authRightsItem};
        
        AuthorizationFlags authFlags = kAuthorizationFlagDefaults | kAuthorizationFlagInteractionAllowed | kAuthorizationFlagPreAuthorize | kAuthorizationFlagExtendRights;
        status = AuthorizationCopyRights(authRef, &authRights, kAuthorizationEmptyEnvironment, authFlags, NULL);
        if(status != errAuthorizationSuccess) {
           [self terminateTestCafe];
        }
        
        status = AuthorizationExecuteWithPrivileges(authRef, command, kAuthorizationFlagDefaults, args, &pipe);
        if(status != errAuthorizationSuccess) {
            [self terminateTestCafe];
        }
        
        AuthorizationFree(authRef, kAuthorizationFlagDestroyRights);
    }
    
    [fileManager release];
}

- (void)runNode{
    nodeTask = [[NSTask alloc] init];
    NSArray *arguments;
    arguments = [NSArray arrayWithObjects: @"--max-old-space-size=1024", @"-e", @"new (require('./index').TestCafe)(null, true)", nil];
    
    [nodeTask setLaunchPath: nodePath];
    [nodeTask setCurrentDirectoryPath:basePath];
    [nodeTask setArguments: arguments];
    [nodeTask launch];
}

- (void)terminateTestCafe{
    [controlPanelHost release];
    [nodeTask terminate];
    [nodeTask release];
    [[NSApplication sharedApplication] terminate:nil];
}

- (IBAction)openControlPanel:(id)sender{
    [[NSWorkspace sharedWorkspace] openURL: [NSURL URLWithString:controlPanelHost]];
}

- (IBAction)copyURLToClipboard:(id)sender{
    NSPasteboard *pasteBoard = [NSPasteboard generalPasteboard];
    [pasteBoard declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:nil];
    [pasteBoard setString:[NSString stringWithFormat: @"%@", controlPanelHost] forType:NSStringPboardType];
    
    [self showNotification];
}

- (IBAction)terminateTestCafe:(id)sender{
    [self terminateTestCafe];
}

@end
