﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Drawing.Imaging;
using System.Threading;
using System.Text;
using System.Windows.Forms;

namespace ShotWindow {


    class Program {
        const int GW_CHILD = 5;
        const int GW_HWNDNEXT = 2;
        const int SW_FORCEMINIMIZE = 11;
        const int SW_RESTORE = 9;

        const int MINIMIZE_MAXIMIZE_DELAY = 20;
        const int BRING_TO_FOREGROUND_DELAY = 500;

        [DllImport("user32.dll")]
        static extern bool IsZoomed(IntPtr hWnd);

        [DllImport("user32.dll")]
        public static extern IntPtr GetForegroundWindow();

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool ShowWindowAsync(IntPtr hWnd, uint nCmdShow);

        [DllImport("user32.dll")]
        private static extern bool PrintWindow(IntPtr hWnd, IntPtr hdcBlt, uint nFlags);

        [DllImport("user32.dll", SetLastError = true)]
        static extern IntPtr GetWindowInfo(IntPtr hWnd, ref WindowInfo pwi);

        [DllImport("user32.dll", SetLastError = true)]
        internal static extern bool MoveWindow(IntPtr hWnd, int X, int Y, int nWidth, int nHeight, bool bRepaint);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern IntPtr FindWindowEx(IntPtr parent /*HWND*/, IntPtr next /*HWND*/, string sClassName, IntPtr sWindowTitle);

        [DllImport("user32.dll")]
        public static extern IntPtr GetWindowRect(IntPtr hWnd, ref Rect rect);

        public delegate bool EnumThreadDelegate(IntPtr hWnd, IntPtr lParam);
        [DllImport("user32.dll")]
        public static extern bool EnumThreadWindows(int dwThreadId, EnumThreadDelegate lpfn, IntPtr lParam);

        [DllImport("user32.dll", EntryPoint = "GetWindowTextLength", SetLastError = true)]
        public static extern int GetWindowTextLength(IntPtr hwnd);

        [DllImport("user32.dll", EntryPoint = "GetWindowText", SetLastError = true)]
        public static extern int GetWindowText(IntPtr hWnd, StringBuilder lpString, int nMaxCount);

        [DllImport("user32.dll", ExactSpelling = true, CharSet = CharSet.Auto)]
        public static extern IntPtr GetWindow(IntPtr hWnd, int uCmd);

        [DllImport("user32.Dll")]
        public static extern void GetClassName(int h, StringBuilder s, int nMaxCount);

        [StructLayout(LayoutKind.Sequential)]
        public struct Rect {
            public int left;
            public int top;
            public int right;
            public int bottom;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct WindowInfo {
            public uint cbSize;
            public Rect rcWindow;
            public Rect rcClient;
            public uint dwStyle;
            public uint dwExStyle;
            public uint dwWindowStatus;
            public uint cxWindowBorders;
            public uint cyWindowBorders;
            public ushort atomWindowType;
            public ushort wCreatorVersion;

            public WindowInfo(Boolean? filler)
                : this()   // Allows automatic initialization of "cbSize" with "new WINDOWINFO(null/true/false)".
            {
                cbSize = (UInt32)(Marshal.SizeOf(typeof(WindowInfo)));
            }

        }

        private static bool windowFound = false;

        static Bitmap getResizedBitmap(Bitmap btm, int width, int height) {
            Bitmap result = new Bitmap(width, height);

            using(var g = Graphics.FromImage(result)) {
                g.DrawImage(btm, 0, 0, width, height);
                g.Dispose();
            }
            return result;
        }

        static void FitWindowOnScreen(IntPtr hWnd) {
            
            WindowInfo wi = new WindowInfo();
            GetWindowInfo(hWnd, ref wi);
            
            int maxArea = 0;
            Rectangle resultWindowRect = new Rectangle(
                wi.rcWindow.left,
                wi.rcWindow.top,
                wi.rcWindow.right - wi.rcWindow.left,
                wi.rcWindow.bottom - wi.rcWindow.top
            );

            bool found = false;

            foreach(var s in Screen.AllScreens) {
                bool leftBorderOnScreen = (wi.rcWindow.left >= s.WorkingArea.Left) && (wi.rcWindow.left < s.WorkingArea.Right),
                     rightBorderOnScreen = (wi.rcWindow.right > s.WorkingArea.Left) && (wi.rcWindow.right <= s.WorkingArea.Right),
                     topBorderOnScreen = (wi.rcWindow.top >= s.WorkingArea.Top) && (wi.rcWindow.top < s.WorkingArea.Bottom),
                     bottomBorderOnScreen = (wi.rcWindow.bottom > s.WorkingArea.Top) && (wi.rcWindow.bottom <= s.WorkingArea.Bottom);

                int widthOnScreen = 0,
                    heightOnScreen = 0;

                if(leftBorderOnScreen)
                    widthOnScreen = rightBorderOnScreen ? wi.rcWindow.right - wi.rcWindow.left : s.WorkingArea.Right - wi.rcWindow.left;
                else if(rightBorderOnScreen)
                    widthOnScreen = wi.rcWindow.right - s.WorkingArea.Left;

                if(topBorderOnScreen)
                    heightOnScreen = bottomBorderOnScreen ? wi.rcWindow.bottom - wi.rcWindow.top : s.WorkingArea.Bottom - wi.rcWindow.top;
                else if(bottomBorderOnScreen)
                    heightOnScreen = wi.rcWindow.bottom - s.WorkingArea.Top;

                int area = widthOnScreen * heightOnScreen;

                if(area > maxArea) {
                    maxArea = area;

                    if(!leftBorderOnScreen)
                        resultWindowRect.X = s.WorkingArea.X;
                    else {
                        resultWindowRect.X = !rightBorderOnScreen ?
                            s.WorkingArea.Right - wi.rcWindow.right + wi.rcWindow.left :
                            wi.rcWindow.left;
                    }

                    if(!topBorderOnScreen)
                        resultWindowRect.Y = s.WorkingArea.Y;
                    else {
                        resultWindowRect.Y = !bottomBorderOnScreen ?
                            s.WorkingArea.Bottom - wi.rcWindow.bottom + wi.rcWindow.top :
                            wi.rcWindow.top;
                    }

                    found = true;
                }
            }

            if(found)
                MoveWindow(hWnd, resultWindowRect.X, resultWindowRect.Y, resultWindowRect.Width, resultWindowRect.Height, true);
        }

        static Bitmap CaptureFromScreen(IntPtr hWnd) {

            IntPtr fgHWnd = GetForegroundWindow();
            
            if(fgHWnd != hWnd) {
                ShowWindowAsync(hWnd, SW_FORCEMINIMIZE);
                Thread.Sleep(MINIMIZE_MAXIMIZE_DELAY);
                ShowWindowAsync(hWnd, SW_RESTORE);
                Thread.Sleep(BRING_TO_FOREGROUND_DELAY);
            }

            WindowInfo wi = new WindowInfo();
            GetWindowInfo(hWnd, ref wi);

            Bitmap windowBitmap = new Bitmap(wi.rcWindow.right - wi.rcWindow.left, wi.rcWindow.bottom - wi.rcWindow.top, System.Drawing.Imaging.PixelFormat.Format32bppRgb);
            Graphics graphicsWindow = Graphics.FromImage(windowBitmap);

            graphicsWindow.CopyFromScreen(new Point(wi.rcWindow.left, wi.rcWindow.top), Point.Empty, windowBitmap.Size, CopyPixelOperation.SourceCopy);

            if(fgHWnd != hWnd) {
                ShowWindowAsync(fgHWnd, SW_FORCEMINIMIZE);
                Thread.Sleep(MINIMIZE_MAXIMIZE_DELAY);
                ShowWindowAsync(fgHWnd, SW_RESTORE);
                Thread.Sleep(BRING_TO_FOREGROUND_DELAY);
            }

            return windowBitmap;
        }

        static void ShotWindow(IntPtr hWnd, WindowInfo wi, string browserName, string path, string fileName, string previewFolder, int previewWidth, int previewHeight) {
            
            Bitmap windowBitmap = null;

            if(browserName == "MSEDGE") {
                windowBitmap = CaptureFromScreen(hWnd);
            } else {
                // Copy browser window
                windowBitmap = new Bitmap(wi.rcWindow.right - wi.rcWindow.left, wi.rcWindow.bottom - wi.rcWindow.top, System.Drawing.Imaging.PixelFormat.Format32bppRgb);
                Graphics graphicsWindow = Graphics.FromImage(windowBitmap);
                IntPtr hdc = graphicsWindow.GetHdc();

                PrintWindow(hWnd, hdc, 0);
                if(browserName == "IE") // HACK for IE
                    PrintWindow(hWnd, hdc, 0);

                graphicsWindow.ReleaseHdc(hdc);
                graphicsWindow.Flush();
            }
            

            // Copy browser client area
            Bitmap clientAreaBitmap = windowBitmap.Clone(new Rectangle(
                new Point(wi.rcClient.left - wi.rcWindow.left, wi.rcClient.top - wi.rcWindow.top),
                new Size(wi.rcClient.right - wi.rcClient.left, wi.rcClient.bottom - wi.rcClient.top)), PixelFormat.Format32bppRgb);

            if(clientAreaBitmap.Size.Height == 1 && clientAreaBitmap.Size.Width == 1) {
                Console.Out.Write("Can't find the browser window");
                Environment.Exit(-1);
            }

            try {
                if(!Directory.Exists(path))
                    Directory.CreateDirectory(path);
            }
            catch(Exception e) {
                Console.Out.Write(e.Message);
                Environment.Exit(-1);
            }

            try {
                clientAreaBitmap.Save(path + fileName);
            }
            catch(Exception e) {
                Console.Out.Write(e.Message);
                Environment.Exit(-1);
            }

            try {
                //Create screenshot preview
                string pathPreviews = path + previewFolder + @"\";

                if(!Directory.Exists(pathPreviews))
                    Directory.CreateDirectory(pathPreviews);

                Bitmap previewBitmap = getResizedBitmap((Bitmap)clientAreaBitmap.Clone(), previewWidth, previewHeight);
                previewBitmap.Save(pathPreviews + fileName);
            }
            catch(Exception e) {
                Console.Out.Write(e.Message);
                Environment.Exit(-1);
            }

            Environment.Exit(0);
        }

        static IEnumerable<IntPtr> GetAllWindowsOfProcess(int processId) {
            var hWnds = new List<IntPtr>();
            foreach(ProcessThread thread in Process.GetProcessById(processId).Threads)
                EnumThreadWindows(thread.Id, (hWnd, lParam) => { hWnds.Add(hWnd); return true; }, IntPtr.Zero);
            return hWnds;
        }

        static string GetCaptionText(IntPtr hWnd) {
            // Allocate correct string length first
            int length = GetWindowTextLength(hWnd);
            StringBuilder sb = new StringBuilder(length + 1);
            GetWindowText(hWnd, sb, sb.Capacity);
            return sb.ToString();
        }

        static IntPtr GetBrowserWindowHandle(string windowMark, string browserName, string pageUrl) {
            IntPtr hWnd = IntPtr.Zero;

            if(browserName == "IE") {
                SHDocVw.ShellWindows shellWindows = new SHDocVw.ShellWindows();
                foreach(SHDocVw.WebBrowser ie in shellWindows) {
                    if(Path.GetFileNameWithoutExtension(ie.FullName).ToLower().Equals("iexplore")) {
                        if(ie.LocationURL.ToLower().Contains(pageUrl.ToLower())) {
                            hWnd = ((IntPtr)ie.HWND);
                            break;
                        }
                    }
                }
            }
            else {
                var procs = browserName == "MSEDGE" ? Process.GetProcessesByName("applicationframehost") : Process.GetProcessesByName(browserName);
                foreach(var p in procs) {
                    IEnumerable<IntPtr> windows = GetAllWindowsOfProcess(p.Id);
                    foreach(IntPtr window in windows) {
                        if(GetCaptionText(window).ToLower().Contains(windowMark.ToLower())) {
                            hWnd = window;
                            break;
                        }
                    }
                }
            }

            return hWnd;
        }

        static void CaptureBrowser(string windowMark, string browserName, string pageUrl, string path, string fileName, string previewFolder, int previewWidth, int previewHeight) {
            IntPtr hWnd = GetBrowserWindowHandle(windowMark, browserName, pageUrl);

            if(hWnd != IntPtr.Zero) {
                windowFound = true;

                if(browserName == "MSEDGE" && !IsZoomed(hWnd))
                    FitWindowOnScreen(hWnd);

                WindowInfo wi = new WindowInfo();
                GetWindowInfo(hWnd, ref wi);

                if(browserName == "CHROME") {
                    IntPtr hostHWND = FindWindowEx(hWnd, IntPtr.Zero, "Chrome_RenderWidgetHostHWND", IntPtr.Zero);
                    if(hostHWND != IntPtr.Zero)
                        GetWindowRect(hostHWND, ref wi.rcClient);
                }
                else if(browserName == "FIREFOX") {
                    // Window client area has border
                    wi.rcClient.top += 1;      // For client area in FireFox v28 and lower this border is absent.
                    wi.rcClient.left += 1;
                    wi.rcClient.bottom -= 1;
                    wi.rcClient.right -= 1;
                }
                else if(browserName == "IE") {
                    IntPtr hChildWnd = GetWindow(hWnd, GW_CHILD);
                    StringBuilder sb = new StringBuilder(256);

                    IntPtr tempHWnd = hWnd;

                    while(tempHWnd != IntPtr.Zero) {
                        GetClassName(hWnd.ToInt32(), sb, 256);

                        if(sb.ToString().IndexOf("Shell DocObject View", 0) > -1) {
                            tempHWnd = FindWindowEx(tempHWnd, IntPtr.Zero, "Internet Explorer_Server", IntPtr.Zero);
                            break;
                        }
                        if(sb.ToString().IndexOf("TabWindowClass", 0) > -1) { //IE7
                            tempHWnd = FindWindowEx(tempHWnd, IntPtr.Zero, "Shell DocObject View", IntPtr.Zero);
                            tempHWnd = FindWindowEx(tempHWnd, IntPtr.Zero, "Internet Explorer_Server", IntPtr.Zero);
                            break;
                        }
                        if(sb.ToString().IndexOf("Frame Tab", 0) > -1) { // IE8
                            tempHWnd = FindWindowEx(tempHWnd, IntPtr.Zero, "TabWindowClass", IntPtr.Zero);
                            tempHWnd = FindWindowEx(tempHWnd, IntPtr.Zero, "Shell DocObject View", IntPtr.Zero);
                            tempHWnd = FindWindowEx(tempHWnd, IntPtr.Zero, "Internet Explorer_Server", IntPtr.Zero);
                            break;
                        }
                        tempHWnd = GetWindow(tempHWnd, GW_HWNDNEXT);
                    }

                    GetWindowRect(tempHWnd, ref wi.rcClient);
                }

                ShotWindow(hWnd, wi, browserName, path, fileName, previewFolder, previewWidth, previewHeight);
            }
        }

        static void Main(string[] args) {
            if(args.Length < 7) {
                Console.Out.Write("Incorrect arguments");
                Environment.Exit(-1);
                return;
            }

            string windowMark = args[0];    //Search by window Title
            string browserName = args[1];   //"CHROME" || "IE" || "FIREFOX" || "OPERA"
            string pageUrl = args[2];   //Search by URL (for IE)
            string path = args[3];   //Path to screenshots folder
            string fileName = args[4];   //Screenshot file name
            string previewFolder = args[5];   //Screenshot previews folder name
            int previewWidth = Convert.ToInt32(args[6]);   //Screenshot preview size
            int previewHeight = Convert.ToInt32(args[7]);

            CaptureBrowser(windowMark, browserName, pageUrl, path, fileName, previewFolder, previewWidth, previewHeight);

            if(!windowFound) {
                Console.Out.Write("Browser window is not found");
                Environment.Exit(-1);
            }
        }
    }
}