"@fixture Project Navigation";

"@page http://127.0.0.1:1335/project";

"@require :util";

"@require :fs";
"@require :tc";
"@require :mixin";

"@test"["404 handler"] = {
    '1': '@mixin Create dir',
    "2.Go to test dir": function() {
        var div = $('.browser-cell a').filter(function() {
            return $(this).attr('title') === 'test';
        });

        act.click(div);
    },
    '3': '@mixin Delete dir',
    "4.Check page": function() {
        eq(tc.fixture('Root Marker').length, 1);
    }
};

"@test"["Dir navigation"] = {
    "1.At level 1": function() {
        act.click(tc.dir('level1'), {
            offsetX: 206,
            offsetY: 19
        });
    },
    "2.Check page": function() {
        eq(tc.fixture('level1').length, 1);
    },
    "3.At level 2": function() {
        act.click(tc.dir('level2'), {
            offsetX: 205,
            offsetY: 24
        });
    },
    "4.Check page": function() {
        eq(tc.fixture('level2').length, 1);
    },
    "5.Click table cell": function() {
        var tableCell = $("#project-browser").find(".browser-cell.nav-button").eq(1);
        act.click(tableCell, {
            offsetX: 184,
            offsetY: 24
        });
    },
    "6. Check page": function() {
        eq(tc.fixture('level1').length, 1);
    },
    "7.Click table cell": function() {
        var tableCell = $("#project-browser").find(".browser-cell.nav-button").eq(1);
        act.click(tableCell, {
            offsetX: 184,
            offsetY: 24
        });
    },
    "8. Check page": function() {
        eq(tc.fixture('Root Marker').length, 1);
    }
};

"@test"["Expand/Collapse fixture"] = {
    "1.Click div": function() {
        var div = $("#project-browser").find(".fixture-name").filter(function() {
            return $(this).text() === "Root Marker";
        });
        act.click(div, {
            offsetX: 117,
            offsetY: 5
        });
    },
    "2. Check anim-container": function() {
        ok(tc.fixture('Root Marker').isExpanded());
    },
    "3.Click div": function() {
        act.click(tc.fixture('Root Marker'), {
            offsetX: 125,
            offsetY: 5
        });
    },
    "4. Check anim-container": function() {
        notOk(tc.fixture('Root Marker').isExpanded());
    }
};

"@test"["Breadcrumbs"] = {
    "1.At level 1": function() {
        act.click(tc.dir('level1'), {
            offsetX: 102,
            offsetY: 27
        });
    },
    "2.At level 2": function() {
        act.click(tc.dir('level2'), {
            offsetX: 111,
            offsetY: 21
        });
    },
    '3.Click span "level2"': function() {
        var span = $("#project-browser").find(".final");
        act.click(span, {
            offsetX: 23,
            offsetY: 9
        });
    },
    "4. Check level 2": function() {
        eq(tc.fixture('level2').length, 1);
    },
    '5.Click link "level1"': function() {
        var link = $(".breadcrumbs").find(":containsExcludeChildren(level1)");
        act.click(link, {
            offsetX: 22,
            offsetY: 12
        });
    },
    "6. Check level 1": function() {
        eq(tc.fixture('level1').length, 1);
    },
    '7.Click project name': function() {
        var link = $(".breadcrumbs").find(".root");
        act.click(link, {
            offsetX: 15,
            offsetY: 12
        });
    },
    "8. Check Root Marker": function() {
        eq(tc.fixture('Root Marker').length, 1);
    }
};

"@test"["Saving state of expanded elements"] = {
    '1': '@mixin Delete fixture',
    "2.Click table cell": function() {
        act.click(tc.fixture('Root Marker'));
    },
    "3.Click table cell": function() {
        act.click(tc.fixture('Fixture with errors.test.js'));
    },
    "4.Emulate file operation": function() {
        var xhr = fs.createFixture("fsOperation", 'fsOperation.test.js', '/');

        act.waitFor(xhr, util.WAIT_FOR_TIMEOUT);
    },
    "5.Check expanded elements": function() {
        ok(tc.fixture('Root Marker').isExpanded());
        ok(tc.fixture('fixture with errors.test.js').isExpanded());
    },
    "6.Click div": function() {
        var link = tc.fixture('Root Marker'),
            offsetTop = link.offset().top;

        $(window).scrollTop(offsetTop);

        act.click(link, {});
    },
    "7.Emulate file operation": function() {
        var xhr = fs.deleteFixture('/', 'fsOperation.test.js');

        act.waitFor(xhr, util.WAIT_FOR_TIMEOUT);
    },
    "8.Check expanded elements": function() {
        notOk(tc.fixture('Root Marker').isExpanded());
        ok(tc.fixture('fixture with errors.test.js').isExpanded());
    }
};

"@test"["Expand/Collapse Data Driving"] = {
    '1': '@mixin Create fixture',
    '2.Create fixture with tests': function() {
        var xhr = fs.createDataDrivingTest("test.test.js", "test", "testName");
        act.waitFor(xhr, util.WAIT_FOR_TIMEOUT);
    },
    '3.Click div': function() {
        $('.footer-line').hide();

        act.click(tc.fixture('test'));
    },
    '5.Expand': function() {
        act.click(tc.fixture('test').test('testName').find('.expand-test-group'));
    },
    '6.Wait for animation':function() {
        act.wait(300);
    },
    '7.Check elements': function() {
        var $fixtureNextElm = tc.fixture('test').next(),
            $header = $fixtureNextElm.find('.group-root'),
            $list =  $fixtureNextElm.find('.group-tests');

        eq($header.css('display'), 'none');
        eq($list.css('display'), 'block');
    },
    '8.Collapse': function() {
        var $btn = tc.fixture('test').test('testName').next().find('.collapse-test-group');

        act.click($btn);
    },
    '9.Wait for animation':function() {
        act.wait(300);
    },
    '10.Check elements': function() {
        var $fixtureNextElm = tc.fixture('test').next(),
            $header = $fixtureNextElm.find('.group-root'),
            $list =  $fixtureNextElm.find('.group-tests');

        eq($header.css('display'), 'block');
        eq($list.css('display'), 'none');
    },
    '11': '@mixin Delete fixture'
};

"@test"["Data Driving Regression"] = {
    "1.Expand fixture": function() {
        act.click(tc.fixture('Data driving test'));
    },
    "2.Check template": function() {
        var $fixture = tc.fixture('Data driving test');

        eq($fixture.test('My first test').length, 1);
        eq($fixture.test('Test1').length, 1);
    },
    "3.Expand elements": function() {
        act.click(tc.fixture('Data driving test').test('My first test').find('.expand-test-group'));
    },
    "4.Expand elements": function() {
        act.click(tc.fixture('Data driving test').test('Test1').find('.expand-test-group'));
    },
    '6.Wait for animation':function() {
        act.wait(300);
    },
    '7.Check elements': function() {
        var $fixtureNextElm = tc.fixture('Data driving test').next(),
            $header = $fixtureNextElm.find('.group-root'),
            $list =  $fixtureNextElm.find('.group-tests');

        eq($header.eq(0).css('display'), 'none');
        eq($header.eq(1).css('display'), 'none');
        eq($list.eq(0).css('display'), 'block');
        eq($list.eq(1).css('display'), 'block');
    }
};

"@test"['T292235 TestCafe Control Panel cannot open a file - nav buttons'] = {
    "1.Click table cell": function() {
        var actionTarget = function() {
            return $(".browser-cell.nav-button").eq(1);
        };
        act.click(actionTarget);
    },
    "2.Click table cell": function() {
        var actionTarget = function() {
            return $(".browser-cell.nav-button").eq(5);
        };
        act.click(actionTarget);
    },
    "3.Click table cell": function() {
        var actionTarget = function() {
            return $(".browser-cell.nav-button").eq(3);
        };
        act.click(actionTarget);
    },
    "4.Click table cell": function() {
        var actionTarget = function() {
            return $(".browser-cell.nav-button").eq(3);
        };
        act.click(actionTarget);
    },
    "5.Click table cell": function() {
        this.currUrl = location.href;

        var actionTarget = function() {
            return $(".browser-cell.nav-button").eq(1);
        };
        act.click(actionTarget);
    },
    '6.Click link "spaces level 2"': function() {
        act.click(":containsExcludeChildren(spaces level 2)");
    },
    '7.Check url': function() {
        eq(this.currUrl, location.href);
    }
};

"@test"['T292235 TestCafe Control Panel cannot open a file - breadcrumbs'] = {
    "1.Click table cell": function() {
        var actionTarget = function() {
            return $(".browser-cell.nav-button").eq(1);
        };
        act.click(actionTarget);
    },
    "2.Click table cell": function() {
        var actionTarget = function() {
            return $(".browser-cell.nav-button").eq(5);
        };
        act.click(actionTarget);
    },
    "3.Click table cell": function() {
        this.currUrl1 = location.href;

        var actionTarget = function() {
            return $(".browser-cell.nav-button").eq(3);
        };
        act.click(actionTarget);
    },
    "4.Click table cell": function() {
        this.currUrl2 = location.href;

        var actionTarget = function() {
            return $(".browser-cell.nav-button").eq(3);
        };
        act.click(actionTarget);
    },
    '5.Click link "spaces level 1"': function() {
        act.click(":containsExcludeChildren(spaces level 1)");
    },
    '6.Check url': function() {
        eq(this.currUrl2, location.href.replace(/\/$/, ''));
    },
    '7.Click link "spaces in name"': function() {
        act.click(":containsExcludeChildren(spaces in name)");
    },
    '8.Check url': function () {
        eq(this.currUrl1, location.href.replace(/\/$/, ''));
    }
};

