var path = require('path'),
    fs = require('fs'),
    ncp = require('ncp').ncp,
    async = require('async'),
    testCafePatcher = require('./tools/patch_testcafe.js'),
    exec = require('child_process').exec,
    rmdir = require('rimraf');

var BASE_PATH = __dirname,
    DEEP_TEST_CAFE_MODULE = path.join(BASE_PATH, './node_modules/patched_testcafe'),
    TEST_CAFE_DIR = path.join(BASE_PATH, './../../../'),
    TEMP_DIR = path.join(BASE_PATH, './temp');

//Utils
function dirExists(path) {
    try {
        return fs.statSync(path).isDirectory();
    } catch (x) {
        return false;
    }
}

function isDir(path) {
    return fs.statSync(path).isDirectory();
}

exports.createPatchedTestCafe = function (callback) {
    if (dirExists(DEEP_TEST_CAFE_MODULE)) {
        callback();
        return;
    }

    async.series({
        '1.Copy testcafe to the temp dir': function (seriesCallback) {
            console.log('1.Copying testcafe to the temp dir...');

            fs.mkdirSync(TEMP_DIR);

            async.eachSeries(fs.readdirSync(TEST_CAFE_DIR), function (entry, eachCallback) {
                if (entry !== 'test' && /^[^\._]/.test(entry)) {
                    console.log('- copying ' + entry);
                    ncp(path.join(TEST_CAFE_DIR, entry), path.join(TEMP_DIR, entry), function (err) {
                        if (err) {
                            console.log(err);
                            return;
                        }

                        eachCallback();
                    });
                }
                else
                    eachCallback();
            }, seriesCallback);
        },

        '2.Patch testcafe': function (seriesCallback) {
            console.log('2.Patching testcafe...');

            try {
                testCafePatcher.patch(TEMP_DIR, seriesCallback);
            }
            catch (err) {
                console.log('TestCafePatcher error: ' + err);
            }
        },

        '3.Create patched distr': function (seriesCallback) {
            console.log('3.Creating patched TestCafe distr...');

            exec(path.join(TEMP_DIR, './bin/win/node') + ' ' + path.join(TEMP_DIR, './gulp') + ' Distributive', {
                cwd: TEMP_DIR
            }, function () {
                seriesCallback();
            });
        },

        '4.Move patched distr to modules': function (seriesCallback) {
            console.log('4.Moving patched TestCafe to node_modules...');

            fs.mkdirSync(DEEP_TEST_CAFE_MODULE);
            ncp(path.join(TEMP_DIR, './_dist_/win/'), DEEP_TEST_CAFE_MODULE, function (name) {
                return name !== 'node_modules';
            }, seriesCallback);
        },

        '5.Remove temp dir': function () {
            console.log('5.Removing temp dir...');
            rmdir(TEMP_DIR, callback);
        }
    });


};