var async = require('async'),
    path = require('path'),
    exec = require('child_process').exec,
    fs = require('fs');

var BASE_PATH = __dirname,
    TEST_CAFE_DIR = path.join(BASE_PATH, './../../../'),
    TESTED_TESTCAFE_TESTS_DIR = path.join(BASE_PATH, './tests_env/test_directory'),
    CURRENT_PROJECT = path.join(BASE_PATH, './tests');

var TESTED_CONTROL_PANEL_PORT = 1341,
    TESTED_SERVICE_PORT = 1342,
    TESTING_CONTROL_PANEL_PORT = 1343,
    TESTING_SERVICE_PORT = 1344;

var patchedTestCafe = null,
    availableBrowsers = null;

function setupTestEnvironment(callback) {
    var testedConfigPath = path.join(TEST_CAFE_DIR, 'config.json'),
        testedConfig = fs.existsSync(testedConfigPath) ? JSON.parse(fs.readFileSync(testedConfigPath)) : null;

    availableBrowsers = testedConfig ? testedConfig.browsers : { "Internet Explorer": { "path": 'C:\\Program Files\\Internet Explorer\\iexplore.exe' } };

    var testSite = require('./tests_env/site/app.js');

    testSite.run(callback);
}

function runTestedTestCafe() {
    var TestedTestCafe = require(path.join(TEST_CAFE_DIR, './index.js')).TestCafe;

    return new TestedTestCafe({
        controlPanelPort: TESTED_CONTROL_PANEL_PORT,
        servicePort: TESTED_SERVICE_PORT,
        testsDir: TESTED_TESTCAFE_TESTS_DIR,
        browsers: availableBrowsers,
        reportsPath: path.join(__dirname, '../../../reports')
    });
}

exports.runTestRunnerTests = function (runCallback) {
    async.series({
        '1.Setup test environment': function (callback) {
            console.log('I.Setup test environment...');

            setupTestEnvironment(callback);
        },

        '2.Run tests': function () {
            console.log('II.Run tests TestCafe');

            var testCafe = runTestedTestCafe();

            testCafe.runTests({
                sourceType: 'dir',
                source: 'Tests',
                browsers: testCafe.listAvailableBrowsers(),
                quarantineMode: false
            });

            testCafe.on('taskComplete', function (report) {
                if (report.failed == 0)
                    runCallback("All tests passed successfully!");
                else
                    runCallback(report);
            });
        }
    });
};

exports.run = function (runTests, runCallback) {
    async.series({
        '1.Setup test environment': function (callback) {
            console.log('I.Setup test environment...');

            setupTestEnvironment(callback);
        },

        '2.Run tested TestCafe': function (callback) {
            console.log('II.Run tested TestCafe');

            runTestedTestCafe();
            callback();
        },

        '3.Run patched TestCafe': function (callback) {
            console.log('III.Run patched TestCafe');

            var testsConfig = JSON.parse(fs.readFileSync(path.join(CURRENT_PROJECT, 'test_config.json')));

            testsConfig.baseUrl = 'http://localhost:' + TESTED_CONTROL_PANEL_PORT;

            fs.writeFileSync(path.join(CURRENT_PROJECT, 'test_config.json'), JSON.stringify(testsConfig));

            var PatchedTestCafe = require('patched_testcafe').TestCafe;

            patchedTestCafe = new PatchedTestCafe({
                controlPanelPort: TESTING_CONTROL_PANEL_PORT,
                servicePort: TESTING_SERVICE_PORT,
                testsDir: CURRENT_PROJECT,
                browsers: availableBrowsers
            });

            callback();
        },

        '4.Finish': function () {
            if (runTests) {
                patchedTestCafe.runTests({
                    browsers: ['Internet Explorer']
                });

                patchedTestCafe.on('taskComplete', function (report) {
                    if (report.failed == 0)
                        runCallback("All tests passed successfully!");
                    else
                        runCallback(report);
                });
            }
            else {
                exec('start http://localhost:' + TESTING_CONTROL_PANEL_PORT);
                console.log('Control Panel URL - http://localhost:' + TESTING_CONTROL_PANEL_PORT);
            }

        }
    });
};