"@fixture Uncaught Js Errors during playback";
"@page ./project/UncaughtJsErrors";

"@require :recorderHelper";
"@require ../../mixins/recorder.js";

var PLAYBACK_TIMEOUT = 20000;

"@test"["Error on page start"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('errors page'));
    },

    "1.Click html": function () {
        act.click("html", {
            offsetY: 1,
            offsetX: 1
        });
    },

    "2.Start playback": function () {
        act.click(recorderHelper.toolbar.getRunPlaybackSelector);
    },

    "3.Wait for page error message is appeared": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.popup.isPageErrorMessageAppeared);
    },

    "4.Check errors and resume playback": function () {
        var errors = recorderHelper.popup.getPageErrorMessageErrors();

        ok(errors.length, 1);
        ok(errors[0].indexOf('ERR1') > -1);

        act.click(recorderHelper.getPopupButtonSelector('Resume playback'));
    },

    "5.Finish playback": "@mixin Finish playback",

    "6.Check the step is marked as a failed": function () {
        ok(recorderHelper.stepsPanel.isStepFailed(0));
    }
};

"@test"["Error on page start with redirect"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('errors page'));
    },

    "1.Click link": function () {
        act.click($(":containsExcludeChildren(no errors link)"));
    },

    "2.Start playback": function () {
        act.click(recorderHelper.toolbar.getRunPlaybackSelector);
    },

    "3.Wait for page error message is appeared": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.popup.isPageErrorMessageAppeared);
    },

    "4.Check errors and resume playback": function () {
        var errors = recorderHelper.popup.getPageErrorMessageErrors();

        ok(errors.length, 1);
        ok(errors[0].indexOf('ERR1') > -1);

        act.click(recorderHelper.getPopupButtonSelector('Resume playback'));
    },

    "5.Finish playback": "@mixin Finish playback",

    "6.Check the step is marked as a failed": function () {
        ok(recorderHelper.stepsPanel.isStepFailed(0));
    }
};

"@test"["Errors during steps"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('errors page'));
    },
    '1.Click link "errors link"': function () {
        var elementSelector = function () {
            return $(":containsExcludeChildren(errors link)").eq(0);
        };
        act.click(elementSelector);
    },
    '2.Click submit button "Error button"': function () {
        act.click("#errBtn");
    },
    '3.Click submit button "Cyclic error button"': function () {
        act.click("#cyclicErrBtn");
    },
    '4.Click link "no errors link"': function () {
        act.click(":containsExcludeChildren(no errors link)");
    },
    '5.Click submit button "No errors button"': function () {
        act.click("#btn");
    },
    "6.Start playback": function () {
        act.click(recorderHelper.toolbar.getRunPlaybackSelector);
    },
    "7.Wait for page error message is appeared": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.popup.isPageErrorMessageAppeared);
    },

    "8.Check errors and resume playback": function () {
        var errors = recorderHelper.popup.getPageErrorMessageErrors();

        eq(errors.length, 1);
        ok(errors[0].indexOf('ERR1') > -1);

        act.click(recorderHelper.getPopupButtonSelector('Resume playback'));
    },
    "10.Wait for page error message is appeared": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.popup.isPageErrorMessageAppeared);
    },
    "11.Check errors and resume playback": function () {
        var errors = recorderHelper.popup.getPageErrorMessageErrors();

        eq(errors.length, 1);
        ok(errors[0].indexOf('ERR1') > -1);

        act.click(recorderHelper.getPopupButtonSelector('Resume playback'));
    },
    "12.Wait for page error message is appeared": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.popup.isPageErrorMessageAppeared);
    },
    "13.Check errors and resume playback": function () {
        var errors = recorderHelper.popup.getPageErrorMessageErrors();

        eq(errors.length, 2);
        ok(errors[0].indexOf('ERR2') > -1);
        ok(errors[1].indexOf('ERR3') > -1);

        act.click(recorderHelper.getPopupButtonSelector('Resume playback'));
    },
    "14.Wait for page error message is appeared": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.popup.isPageErrorMessageAppeared);
    },
    "15.Check errors and resume playback": function () {
        var errors = recorderHelper.popup.getPageErrorMessageErrors();

        eq(errors.length, 6);

        act.click(recorderHelper.getPopupButtonSelector('Resume playback'));
    },
    "17.Finish playback": "@mixin Finish playback",
    "18.Check steps are failed": function () {
        ok(recorderHelper.stepsPanel.isStepFailed(0));
        ok(recorderHelper.stepsPanel.isStepFailed(1));
        ok(recorderHelper.stepsPanel.isStepFailed(2));
        notOk(recorderHelper.stepsPanel.isStepFailed(3));
        notOk(recorderHelper.stepsPanel.isStepFailed(4));
    }
};

