"@fixture In iframe assertions";
"@page ./tests";

"@require :recorderHelper";

var PLAYBACK_TIMEOUT = 20000;

"@test"["Record an action after an assertion"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('CrossDomainPage'));
    },

    "1.Click on the black element": inIFrame('#iframe', function () {
        act.click('.black');
    }),

    "2.Start wizard": function () {
        act.click([
            recorderHelper.toolbar.getAddAssertionsSelector,
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getOkPopupItem,
            recorderHelper.editAssertionDialog.getOpenWizardButton
        ]);
    },

    "3.Select on the black element": inIFrame('#iframe', function () {
        act.click('.black');
    }),

    "4.Click the 'Next' button": function () {
        act.click(recorderHelper.getPopupButtonSelector('Next'));
    },

    "5.Select the 'length' property": function () {
        act.click(recorderHelper.selectUIElementByText('length'));
    },

    "6.Confirm assertion": function () {
        function generateButtonSelector(text) {
            return function () {
                return recorderHelper.getPopupButtonSelector(text);
            }
        }

        act.click([
            generateButtonSelector('Next'),
            recorderHelper.editAssertionDialog.getConfirmButton,
            generateButtonSelector('OK')
        ]);
    },

    "7.Click on the black element": inIFrame('#iframe', function () {
        act.click('.black');
    }),

    "8.Check recorded steps length": function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 3);
    }
};

var ASSERTION_IN_THE_MAIN_IFRAME_TITLE = 'Assertions in the main window',
    ASSERTION_IN_AN_IFRAME_TITLE = 'Assertions in IFrame iframe';

"@test"["Combined assertions"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('CrossDomainPage'));
    },
    "1.Start wizard": function () {
        act.click([
            recorderHelper.toolbar.getAddAssertionsSelector,
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getEqPopupItem,
            recorderHelper.editAssertionDialog.getOpenWizardButton
        ]);
    },
    "2.Click body": function () {
        act.click(['body', 'body'], { offsetY: 1 });
    },
    '3.Click span "Next"': function () {
        act.click(":containsExcludeChildren(Next)");
    },
    '4.Click div "width"': function () {
        act.click(":containsExcludeChildren(width)");
    },
    '5.Click span "Next"': function () {
        var elementSelector = function () {
            return $(":containsExcludeChildren(Next)").eq(1);
        };
        act.click(elementSelector);
    },
    "6.Copy value": function () {
        act.click(recorderHelper.editAssertionDialog.getCopyButton);
    },
    '7.Click submit button "OK"': function () {
        act.click(recorderHelper.editAssertionDialog.getConfirmButton);
    },
    "8.Assert": function () {
        notOk($(":containsExcludeChildren(" + ASSERTION_IN_THE_MAIN_IFRAME_TITLE + ")").is(":visible"), 'default frame title is not visible');
    },
    "9.Start wizard": function () {
        act.click([
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getEqPopupItem,
            recorderHelper.editAssertionDialog.getOpenWizardButton
        ]);
    },
    '10.Click div "black"': inIFrame($("#iframe"), function () {
        act.click(":containsExcludeChildren(black)");
    }),
    '11.Click span "Next"': function () {
        act.click(":containsExcludeChildren(Next)");
    },
    '12.Click div "text"': function () {
        act.click(":containsExcludeChildren(text)");
    },
    '13.Click span "Next"': function () {
        act.click(":containsExcludeChildren(Next)");
    },
    "14.Copy value": function () {
        act.click(recorderHelper.editAssertionDialog.getCopyButton);
    },
    '15.Click submit button "OK"': function () {
        act.click(recorderHelper.editAssertionDialog.getConfirmButton);
    },
    "16.Assert": function () {
        ok($(":containsExcludeChildren(" + ASSERTION_IN_THE_MAIN_IFRAME_TITLE + ")").is(":visible"), 'default frame message is visible');
        ok($(":containsExcludeChildren(" + ASSERTION_IN_AN_IFRAME_TITLE + ")").is(":visible"), 'iframe message is visible');
    },
    "17.Remove assertion": function () {
        act.click(recorderHelper.assertionsStepDialog.getRemoveAssertionButton(0));
    },
    "18.Assert": function () {
        notOk($(":containsExcludeChildren(" + ASSERTION_IN_THE_MAIN_IFRAME_TITLE + ")").is(":visible"), 'default frame message is not visible');
        ok($(":containsExcludeChildren(" + ASSERTION_IN_AN_IFRAME_TITLE + ")").is(":visible"), 'iframe message is visible');
    },
    '19.Click submit button "OK"': function () {
        act.click(":containsExcludeChildren(OK)");
    },
    '20.Click div "Assert"': function () {
        act.click(":containsExcludeChildren(Assert)");
    },
    "21.Open assertion editor": function () {
        act.click(recorderHelper.assertionsStepDialog.getEditAssertionSelector(0));
    },
    "22.Click on editor": function () {
        act.click(recorderHelper.editAssertionDialog.getExpectedCodeEditor);
    },
    "23.Change value": function () {
        act.press("ctrl+a 1");
    },
    '24.Click submit button "OK"': function () {
        act.click(recorderHelper.editAssertionDialog.getConfirmButton);
    },
    "25.Create new assertion": function () {
        act.click([
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getOkPopupItem
        ]);
    },
    "26.Click on editor": function () {
        act.click(recorderHelper.editAssertionDialog.getActualCodeEditor);
    },
    "27.Type 'false'": function () {
        act.press("f a l s e");
    },
    '28.Click submit button "OK"': function () {
        act.click(recorderHelper.editAssertionDialog.getConfirmButton);
    },
    "29.Run playback": function () {
        act.click(recorderHelper.toolbar.getRunPlaybackSelector);
    },
    "30.Wait for playback failed": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.isPopupShown);
    },
    '31.Click submit button "Skip step"': function () {
        act.click(recorderHelper.stepsPanel.getSkipStepButtonSelector);
    },
    '32.Click submit button "Return to recording"': function () {
        act.click(":containsExcludeChildren(Return to recording)");
    },
    '33.Click div "Assert"': function () {
        act.click(":containsExcludeChildren(Assert)");
    },
    "34.Assert": function () {
        ok(recorderHelper.assertionsStepDialog.isAssertionFailed(0));
        ok(recorderHelper.assertionsStepDialog.isAssertionFailed(1));
    },
    "35.Run playback": function () {
        act.click(recorderHelper.toolbar.getRunPlaybackSelector);
    },
    "36.Wait for playback failed": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.isPopupShown);
    },
    "37.Assert": function () {
        ok(recorderHelper.assertionsStepDialog.isAssertionFailed(0));
        ok(recorderHelper.assertionsStepDialog.isAssertionFailed(1));
    },
    "38.Open the second assertion editor": function () {
        act.click(recorderHelper.assertionsStepDialog.getEditAssertionSelector(1));
    },
    "39.Click on editor": function () {
        act.click(recorderHelper.editAssertionDialog.getActualCodeEditor);
    },
    "40.Change value": function () {
        act.press("ctrl+a delete t r u e");
    },
    '41.Click submit button "OK"': function () {
        act.click(recorderHelper.editAssertionDialog.getConfirmButton);
    },
    '42.Click submit button "Replay step"': function () {
        act.click(recorderHelper.stepsPanel.getResumePlaybackButtonSelector);
    },
    "43.Assert": function () {
        ok(recorderHelper.assertionsStepDialog.isAssertionFailed(0));
        notOk(recorderHelper.assertionsStepDialog.isAssertionFailed(1));
    },
    "44.Edit failed assertion": function () {
        act.click(recorderHelper.assertionsStepDialog.getEditAssertionSelector(0));
    },
    "45.Copy value": function () {
        act.click(recorderHelper.editAssertionDialog.getCopyButton);
    },
    '46.Click submit button "OK"': function () {
        act.click(recorderHelper.editAssertionDialog.getConfirmButton);
    },
    '47.Click submit button "Replay step"': function () {
        act.click(recorderHelper.stepsPanel.getResumePlaybackButtonSelector);
    },
    "48.Assert": function () {
        ok($(".playback-result-TC2b9a6d.success-TC2b9a6d").length);
    },
    '49.Click submit button "Return to recording"': function () {
        act.click(":containsExcludeChildren(Return to recording)");
    },
    '50.Click div "Assert"': function () {
        act.click(":containsExcludeChildren(Assert)");
    },
    "51.Assert": function () {
        notOk(recorderHelper.assertionsStepDialog.isAssertionFailed(0));
        notOk(recorderHelper.assertionsStepDialog.isAssertionFailed(1));
    }
};

"@test"["Regression - correct assertions status"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('CrossDomainPage'));
    },
    "1.Start wizard": function () {
        act.click([
            recorderHelper.toolbar.getAddAssertionsSelector,
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getOkPopupItem
        ]);
    },
    "2.Click on editor": function () {
        act.click(recorderHelper.editAssertionDialog.getActualCodeEditor);
    },
    "3.Type 'false'": function () {
        act.press("f a l s e");
    },
    '4.Click submit button "OK"': function () {
        act.click(recorderHelper.editAssertionDialog.getConfirmButton);
    },
    "5.Start wizard": function () {
        act.click([
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getEqPopupItem,
            recorderHelper.editAssertionDialog.getOpenWizardButton
        ]);
    },
    '6.Click div "black"': inIFrame($("#iframe"), function () {
        act.click(":containsExcludeChildren(black)");
    }),
    '7.Click span "Next"': function () {
        act.click(":containsExcludeChildren(Next)");
    },
    '8.Click div "text"': function () {
        act.click(":containsExcludeChildren(text)");
    },
    '9.Click span "Next"': function () {
        act.click(":containsExcludeChildren(Next)");
    },
    "10.Copy value": function () {
        act.click(recorderHelper.editAssertionDialog.getCopyButton);
    },
    '11.Click submit button "OK"': function () {
        act.click(recorderHelper.editAssertionDialog.getConfirmButton);
    },
    '12.Click OK': function () {
        act.click(":containsExcludeChildren(OK)");
    },
    "13.Run playback": function () {
        act.click(recorderHelper.toolbar.getRunPlaybackSelector);
    },
    "14.Wait for playback failed": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.isPopupShown);
    },
    "14.1.Wait for playback failed": function () {
        act.wait(100);
    },
    "15.Assert": function () {
        ok(recorderHelper.assertionsStepDialog.isAssertionFailed(0));
        ok(recorderHelper.assertionsStepDialog.isAssertionSuccessful(1));
    },
    '16.Click submit button "Skip step"': function () {
        act.click(recorderHelper.stepsPanel.getSkipStepButtonSelector);
    },
    '17.Click submit button "Return to recording"': function () {
        act.click(":containsExcludeChildren(Return to recording)");
    },
    '18.Click div "Assert"': function () {
        act.click(":containsExcludeChildren(Assert)");
    },
    '19.Assert': function () {
        ok(recorderHelper.assertionsStepDialog.isAssertionFailed(0));
        notOk(recorderHelper.assertionsStepDialog.isAssertionFailed(1));
    }
};

