"@fixture Shortcuts";
"@page ./tests";

"@require :recorderHelper";
"@require ../../../mixins/recorder.js";

var iframeSelector = '#iframe',
    PLAYBACK_TIMEOUT = 20000;

"@test"["Check shortcuts in iFrame"] = {
    "@testCases": [
        { '@name': 'Cross-domain iframe', title: 'CrossDomainPage' },
        { '@name': 'Same-domain iframe', title: 'SameDomainPage' },
        { '@name': 'Inline iframe', title: 'InlineIframePage' }
    ],

    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(this.title));
    },

    "1.Focus iFrame": inIFrame(iframeSelector, function () {
        $('#input')[0].focus();
    }),

    "2.Press hover shortcut": inIFrame(iframeSelector, function () {
        act.press(recorderHelper.HOVER_SHORTCUT);
    }),

    "3.Click input": inIFrame(iframeSelector, function () {
        act.click('#input');
    }),

    "4.Check action is saved": function () {
        var addHoverButtonTitle = recorderHelper.toolbar.getRecordHoverActionSelector().parent().attr('title');
        ok(addHoverButtonTitle, 'toolbar "Add hover action" button has title');

        if (addHoverButtonTitle)
            eq(addHoverButtonTitle.substring(addHoverButtonTitle.indexOf('(') + 1, addHoverButtonTitle.indexOf(')')), 'Ctrl+Space');

        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        this.currentIndex = recorderHelper.stepsPanel.getCurrentStepIndex();
        act.click(recorderHelper.stepsPanel.getStepListItem(this.currentIndex));
    },

    "4.1.Check saved action": function () {
        eq(recorderHelper.stepsPanel.getCurrentStepType(), 'hover');
        eq(recorderHelper.stepsPanel.getCurrentSelector(), '$("#input")');
        act.click(recorderHelper.stepsPanel.getStepListItem(this.currentIndex));
    },

    "5.Press wait shortcut": inIFrame(iframeSelector, function () {
        //NOTE: active element in cross-domain iframe doesn't restore
        $('#input')[0].focus();
        act.press(recorderHelper.WAIT_SHORTCUT);
    }),

    "6.Check dialog and confirm action": function () {
        ok(recorderHelper.isPopupShown());
        act.click(recorderHelper.addActionDialog.getAddActionButton);
    },

    "7.Check action is saved": function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 2);
        this.currentIndex = recorderHelper.stepsPanel.getCurrentStepIndex();
        act.click(recorderHelper.stepsPanel.getStepListItem(this.currentIndex));
    },

    "7.1.Check saved action": function () {
        eq(recorderHelper.stepsPanel.getCurrentStepType(), 'wait');
        act.click(recorderHelper.stepsPanel.getStepListItem(this.currentIndex));
    },

    "8. Press add assertion shortcut": inIFrame(iframeSelector, function () {
        //NOTE: active element in cross-domain iframe doesn't restore
        $('#input')[0].focus();

        act.press(recorderHelper.ASSERTION_SHORTCUT);
    }),

    "9.Check dialog and cancel addition assertion": function () {
        ok(recorderHelper.isPopupShown());
        act.click(recorderHelper.dialog.getCancelButton);
    },

    "10.Select first step": function () {
        act.click([recorderHelper.stepsPanel.getStepListItem(0), recorderHelper.stepsPanel.getStepListItem(0)]);
    },

    "11.Press run playback shortcut": function () {
        eq(recorderHelper.stepsPanel.getCurrentStepIndex(), 0);
        act.press(recorderHelper.PLAYBACK_SHORTCUT);
    },

    "12.Finish playback": "@mixin Finish playback",

    "13.Check steps panel after playback": function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 2);
        eq(recorderHelper.stepsPanel.getCurrentStepIndex(), 1);
    },

    "14. Press save test shortcut": inIFrame(iframeSelector, function () {
        //NOTE: active element in cross-domain iframe doesn't restore
        $('#input')[0].focus();

        act.press(recorderHelper.SAVE_TEST_SHORTCUT);
    }),

    "15.Check dialog and press cancel button": function () {
        ok(recorderHelper.isPopupShown());
        ok(recorderHelper.setTestNameDialog.getTestNameInput().length, 'test name input is shown');
        act.click(recorderHelper.dialog.getCancelButton);
    },

    "16.Press end test shortcut": inIFrame(iframeSelector, function () {
        //NOTE: active element in cross-domain iframe doesn't restore
        $('#input')[0].focus();

        act.press(recorderHelper.END_TEST_SHORTCUT);
    }),

    "17.Check dialog and press cancel button": function () {
        ok(recorderHelper.isPopupShown());
        act.click(recorderHelper.dialog.getCancelButton);
    },

    "18.Click on input": inIFrame(iframeSelector, function () {
        act.type('#input', '123');
    }),

    "19.Press complete action shortcut": inIFrame(iframeSelector, function () {
        act.press(recorderHelper.COMPLETE_ACTION_SHORTCUT);
    }),

    "20.Check action is saved": function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 3);
        this.currentIndex = recorderHelper.stepsPanel.getCurrentStepIndex();
        act.click(recorderHelper.stepsPanel.getStepListItem(this.currentIndex));
    },

    "21.1.Check saved action": function () {
        eq(recorderHelper.stepsPanel.getCurrentStepType(), 'type');
        eq(recorderHelper.stepsPanel.getCurrentSelector(), '$("#input")');
    },

    "22.Press screenshow shortcut": inIFrame(iframeSelector, function () {
        act.press(recorderHelper.SCREENSHOT_SHORTCUT);
    }),

    "23.Check action is saved": function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 4);
        this.currentIndex = recorderHelper.stepsPanel.getCurrentStepIndex();
        act.click(recorderHelper.stepsPanel.getStepListItem(this.currentIndex));
    },

    "23.1.Check saved action": function () {
        eq(recorderHelper.stepsPanel.getCurrentStepType(), 'screenshot');
        act.click(recorderHelper.stepsPanel.getStepListItem(this.currentIndex));
    }
};
