"@fixture Saving test behavior";
"@page ./tests";

"@require :recorderHelper";
"@require ../../../mixins/recorder.js";

var FILE_SYSTEM_UPDATING_TIME = 5000;

var fixtureName = 'BaseRecorderPage',
    fileName = 'baserecorderpage.test.js';

"@test"["Exit recording before until it was not written a single step"] = {
    '-1.Start recording': function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    '1.Exit recording': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '2.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, isNoStepsTitleVisible);
    },

    '3.Check that no test was not created': function () {
        notOk(recorderHelper.controlPanelHelper.getTestCount('baserecorderpage.test.js'));
    }
};

"@test"["Exit without saving"] = {
    '-1.Start recording': function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    '1.Record click action': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click('body');
    },

    '2.Exit recording': function () {
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '2.1.Click Exit button in Unsaved changes dialog': function () {
        act.click(recorderHelper.dialog.getNoButton);
    },

    '3.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, isNoStepsTitleVisible);
    },

    '4.Check that no test was not created': function () {
        notOk(recorderHelper.controlPanelHelper.getTestCount('baserecorderpage.test.js'));
    }
};

"@test"["Save test and exit recording"] = {
    '-1.Start recording': function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    '1.Record click action': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click('body');
    },

    '2.Check that test has unsaved changes and save': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());

        this.testName = '1';
        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '2.1.Set empty test name': function () {
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, '  ');
    },

    '2.2.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '2.3.Check that error was raised and set correct test name': function () {
        ok(recorderHelper.setTestNameDialog.hasError('Test name is empty.'));

        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.testName, {
            replace: true
        });
    },

    '2.4.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '3.Check that test has not unsaved changes and exit recording': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '4.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, function () {
            return recorderHelper.controlPanelHelper.getTest(fileName, this.testName).is(':visible');
        });
    },

    '5.Check that test was saved and delete it': function () {
        ok(recorderHelper.controlPanelHelper.hasFixtureTest(this.testName, fileName));
        act.click(recorderHelper.controlPanelHelper.getDeleteTestButtonSelector(this.testName, fileName));
    }
};

"@test"["Exit recording and save unsaved changes"] = {
    //NOTE: create simple test
    '-1.Start recording': function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    '1.Record click action': function () {
        act.click('body');
    },

    '2.Save test': function () {
        this.usedTestName = '1';
        this.testName = '2';
        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '2.1.Set test name': function () {
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.usedTestName);
    },

    '2.2.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '3.Exit recording': function () {
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '4.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, function () {
            return recorderHelper.controlPanelHelper.getTest(fileName, this.usedTestName).is(':visible');
        });
    },

    '5.Check that test was saved and start recording again': function () {
        ok(recorderHelper.controlPanelHelper.hasFixtureTest(this.usedTestName, fileName));
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    //NOTE: create new test
    '6.Record click action': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click('body');
    },

    '7.Check that test has unsaved changes and exit recording': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '7.1.Click Cancel button in Unsaved changes dialog': function () {
        act.click(recorderHelper.dialog.getCancelButton);
    },

    '7.2.Check cancel saving and exit recording': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '7.3.Click Save button in Unsaved changes dialog': function () {
        act.click(recorderHelper.dialog.getYesButton);
    },

    '7.4.Set already used test name': function () {
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.usedTestName);
    },

    '7.5.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '7.6.Check that error was raised and set correct test name': function () {
        ok(recorderHelper.setTestNameDialog.hasError('Test with name "' + this.usedTestName + '" is already exists.'));

        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.testName, {
            replace: true
        });
    },

    '7.7.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '8.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, function () {
            return recorderHelper.controlPanelHelper.getTest(fileName, this.testName).is(':visible');
        });
    },

    '9.Check that test was saved and delete first test': function () {
        ok(recorderHelper.controlPanelHelper.hasFixtureTest(this.testName, fileName));
        act.click(recorderHelper.controlPanelHelper.getDeleteTestButtonSelector(this.usedTestName, fileName));
    },

    '10.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, function () {
            return recorderHelper.controlPanelHelper.getTest(fileName, this.testName).is(':visible');
        });
    },

    '11.Delete test': function () {
        act.click(recorderHelper.controlPanelHelper.getDeleteTestButtonSelector(this.testName, fileName));
    }
};

"@test"["Save test, exit recording and save unsaved changes"] = {
    '-1.Start recording': function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    '1.Record click action': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click('body');
    },

    '2.Check that test has unsaved changes and exit recording': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());

        this.testName = '3';
        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '2.1.Set test name': function () {
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.testName);
    },

    '2.2.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '3.Check that test has not unsaved changes and record action': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.dblclick('body');
    },

    '4.Check that action was saved and exit recording': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 2);
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '4.1.Click Save button in Unsaved changes dialog': function () {
        act.click(recorderHelper.dialog.getYesButton);
    },

    '5.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, function () {
            return recorderHelper.controlPanelHelper.getTest(fileName, this.testName).is(':visible');
        });
    },

    '6.Check that test was saved': function () {
        ok(recorderHelper.controlPanelHelper.hasFixtureTest(this.testName, fileName));
        act.click(recorderHelper.controlPanelHelper.getDeleteTestButtonSelector(this.testName, fileName));
    }
};

"@test"["Add assertion in existed assertion step"] = {
    '-1.Start recording': function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    "1.Add assertion": function () {
        act.click([
            recorderHelper.toolbar.getAddAssertionsSelector,
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getOkPopupItem
        ]);
    },

    "1.1.Fill actual argument": function () {
        act.click(recorderHelper.getCodeEditorSelector().eq(0));
    },

    "1.2.Fill actual argument": function () {
        act.press('t r u e esc enter');
    },

    "1.3.Add an assertion step": function () {
        act.click(recorderHelper.getPopupButtonSelector('OK'));
    },

    '2.Save step': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());

        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '2.1.Set test name': function () {
        this.testName = '4';
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.testName);
    },

    '2.2.Click Cancel button': function () {
        act.click(recorderHelper.dialog.getCancelButton);
    },

    '2.3.Check canceling saving and save step': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());

        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '2.4.Set test name': function () {
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.testName);
    },

    '2.5.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '3.Check that test has not unsaved changes and add new assertion': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.toolbar.getAddAssertionsSelector);
    },

    "3.1.Cancel of assertion addition": function () {
        act.click(recorderHelper.getPopupButtonSelector('Cancel'));
    },

    '4.Check that test has not unsaved changes and add new assertion': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.toolbar.getAddAssertionsSelector);
    },

    "4.1.Add an assertion step - click Ok button": function () {
        act.click(recorderHelper.getPopupButtonSelector('OK'));
    },

    '5.Check that test has unsaved changes and save': function () {
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '6.Exit recording': function () {
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '7.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, function () {
            return recorderHelper.controlPanelHelper.getTest(fileName, this.testName).is(':visible');
        });
    },

    '8.Check that test was saved': function () {
        ok(recorderHelper.controlPanelHelper.hasFixtureTest(this.testName, fileName));
        act.click(recorderHelper.controlPanelHelper.getDeleteTestButtonSelector(this.testName, fileName));
    }
};

"@test"["Check 'back to start recording page' popup"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    "1.Record action without moving to another page": function () {
        act.click($('body'));
    },

    "2.Delete last step": function () {
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "3.Check that popup 'Go back to initial page' is visible and click on its button": function () {
        ok(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        notOk(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
        ok(!recorderHelper.tooltip.isTooltipVisible());
        act.click(recorderHelper.popup.getPopupButtonSelector('Go back to initial page'));
    },

    "4.1.Record some actions with moving to another page": function () {
        act.click($('body'));
    },

    "4.2.Record some actions with moving to another page": function () {
        act.click($('#linkToAnotherPage'));
    },

    "5.Delete last step": function () {
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "6.Check last step deletion warning tooltip is visible and delete last step again": function () {
        ok(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
        notOk(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "7.Check that popup 'Go back to initial page' is visible and click on its button": function () {
        ok(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        notOk(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
        ok(!recorderHelper.tooltip.isTooltipVisible());
        act.click(recorderHelper.popup.getPopupButtonSelector('Go back to initial page'));
    },

    "8.Check recording was restarted": function () {
        ok(!recorderHelper.popup.isPopupShown());
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        notOk(recorderHelper.toolbar.isRunPlaybackButtonEnabled());
    },

    "9.Add assertion step": "@mixin Add assertion step",

    "10.Delete last step": function () {
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "11.Check that popup 'Go back to initial page' and last step deletion tooltip are not visible": function () {
        notOk(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        notOk(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
        notOk(recorderHelper.tooltip.isTooltipVisible());
    },

    "12.Add assertion step": "@mixin Add assertion step",

    "13.Record action without moving to another page": function () {
        act.click($('body'));
    },

    "10.Delete last step": function () {
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "11.Check last step deletion warning tooltip is visible and delete last step again": function () {
        ok(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
        notOk(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "12.Check that popup 'Go back to initial page' is visible and click on its button": function () {
        ok(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        act.click(recorderHelper.popup.getPopupButtonSelector('Go back to initial page'));
    },

    "13.Check recording was restarted": function () {
        notOk(recorderHelper.popup.isPopupShown());
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        notOk(recorderHelper.toolbar.isRunPlaybackButtonEnabled());
    }
};

"@test"["Check 'back to start recording page' popup after save test"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    "1.Record action without moving to another page": function () {
        act.click($('body'));
    },

    "2.1.Save test": function () {
        this.testName = '5';
        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '2.2.Save test': function () {
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.testName);
    },

    '2.3.Save test': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '3.Check that test has not unsaved changes and delete last step': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "4.Check that popup 'Go back to initial page' is visible and click on its button": function () {
        ok(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        notOk(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
        ok(!recorderHelper.tooltip.isTooltipVisible());
        act.click(recorderHelper.popup.getPopupButtonSelector('Go back to initial page'));
    },

    "5.1.Record some actions with moving to another page": function () {
        act.click($('body'));
    },

    "5.2.Record some actions with moving to another page": function () {
        act.click($('#linkToAnotherPage'));
    },

    "6.Delete last step": function () {
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "7.Check last step deletion warning tooltip is visible and delete last step again": function () {
        ok(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
        notOk(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "8.Check that popup 'Go back to initial page' is visible and click on its button": function () {
        ok(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        notOk(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
        act.click(recorderHelper.popup.getPopupButtonSelector('Go back to initial page'));
    },

    "9.Check recording was restarted and exit recording": function () {
        ok(!recorderHelper.popup.isPopupShown());
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        notOk(recorderHelper.toolbar.isRunPlaybackButtonEnabled());
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '10.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, function () {
            return recorderHelper.controlPanelHelper.getTest(fileName, this.testName).is(':visible');
        });
    },

    '11.Check that test was saved and delete it': function () {
        ok(recorderHelper.controlPanelHelper.hasFixtureTest(this.testName, fileName));
        act.click(recorderHelper.controlPanelHelper.getDeleteTestButtonSelector(this.testName, fileName));
    }
};

"@test"["Check save deferred actions before exit recording (confirm saving)"] = {
    '-1.Start recording': function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    '1.Record click action': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click('body');
    },

    '2.Check that test has unsaved changes and click on input': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());

        act.click('#input');
    },

    '3.Check recorded actions count and that we have deferred action click': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        eq(recorderHelper.getCompleteClickIcon().length, 1);
    },

    '4.Save test': function () {
        this.testName = 'temp_test';
        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '2.1.Set test name': function () {
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.testName);
    },

    '2.2.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '3.Check that test has not unsaved changes, deferred click action and exit recording': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        eq(recorderHelper.getCompleteClickIcon().length, 1);
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '4.Click Save button in Unsaved changes dialog': function () {
        ok(recorderHelper.isPopupShown());
        act.click(recorderHelper.dialog.getYesButton);
    },

    '4.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, function () {
            return recorderHelper.controlPanelHelper.getTest(fileName, this.testName).is(':visible');
        });
    },

    '5.1.Check that test was saved and saved test code': function () {
        ok(recorderHelper.controlPanelHelper.hasFixtureTest(this.testName, fileName));

        act.click(recorderHelper.controlPanelHelper.getEditTestButtonSelector(this.testName));
    },

    '5.2.Check that test was saved and saved test code': function () {
        var testCode = $(".ace_content").text().replace(/\s/g, "");

        notEq(testCode.indexOf('{"1.Clickbody":function(){act.click("body");},"2.Clickinput":function(){act.click("#input");}};'), -1);
    },

    '6.Close edit dialog and delete test': '@mixin Close Edit Dialog And Delete Test'
};

"@test"["Check save deferred actions before exit recording (cancel saving)"] = {
    '-1.Start recording': function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    '1.Record click action': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        act.click('body');
    },

    '2.Check that test has unsaved changes and click on input': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        ok(recorderHelper.toolbar.isSaveTestButtonEnabled());

        act.click('#input');
    },

    '3.Check recorded actions count and that we have deferred action click': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        eq(recorderHelper.getCompleteClickIcon().length, 1);
    },

    '4.Save test': function () {
        this.testName = 'temp_test';
        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '2.1.Set test name': function () {
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.testName);
    },

    '2.2.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    '3.Check that test has not unsaved changes, deferred click action and exit recording': function () {
        notOk(recorderHelper.toolbar.isSaveTestButtonEnabled());
        eq(recorderHelper.getCompleteClickIcon().length, 1);
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '4.Click Save button in Unsaved changes dialog': function () {
        ok(recorderHelper.isPopupShown());
        act.click(recorderHelper.dialog.getNoButton);
    },

    '4.Wait for update file system': function () {
        act.wait(FILE_SYSTEM_UPDATING_TIME, function () {
            return recorderHelper.controlPanelHelper.getTest(fileName, this.testName).is(':visible');
        });
    },

    '5.1.Check that changes were not saved': function () {
        ok(recorderHelper.controlPanelHelper.hasFixtureTest(this.testName, fileName));

        act.click(recorderHelper.controlPanelHelper.getEditTestButtonSelector(this.testName));
    },

    '5.2.Check that changes were not saved': function () {
        var testCode = $(".ace_content").text().replace(/\s/g, "");

        notEq(testCode.indexOf('{"1.Clickbody":function(){act.click("body");}'), -1);
    },

    '6.Close edit dialog and delete test': '@mixin Close Edit Dialog And Delete Test'
};

"@test"["T183278 - Cancel exit and continue recording"] = {
    '-1.Start recording': function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector(fixtureName));
    },

    '1.Record click action': function () {
        act.click('body');
    },

    '2.Click toolbar Exit recording button': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    '3.Click Cancel button in Unsaved changes dialog': function () {
        act.click(recorderHelper.dialog.getCancelButton);
    },

    '4.Record another click action': function () {
        act.click('body');
    },

    '5.Check recorded steps': function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 2);
    }
};

var isNoStepsTitleVisible = function () {
    return recorderHelper.controlPanelHelper.getNoTestTitle().is(':visible');
};