"@fixture Editing";
"@page ./tests";

"@require :recorderHelper";
"@require ../../../mixins/recorder.js";

var PLAYBACK_TIMEOUT = 20000;

"@test"['Edit step info'] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.Record the click action": function () {
        act.click('.black', { offsetX: 1, offsetY: 1});
    },

    "2.Open step info": function () {
        eq(window.blackClick, 1, 'Click on the black element is successful');

        act.click(recorderHelper.stepsPanel.getStepListItem(0));
    },

    "3.Check popup and click outside the step info panel": function () {
        ok(recorderHelper.stepsPanel.isStepInfoPanelShown(), 'Step Info panel is shown');

        act.click('.black', { offsetX: 1, offsetY: 1 });
    },

    "4.Check click was prevented and step info panel is closed": function () {
        eq(window.blackClick, 1, 'Click on the black element was prevented');
        eq(recorderHelper.stepsPanel.getStepCount(), 1, 'Click on the black element was not recorded');
        notOk(recorderHelper.stepsPanel.isStepInfoPanelShown(), 'Step Info panel is not shown');
    },

    "5.Record the click action": function () {
        act.click('.black', { offsetX: 1, offsetY: 1});
    },

    "6.Check action is recorded and open step info": function () {
        eq(window.blackClick, 2, 'Click on the black element was prevented');
        eq(recorderHelper.stepsPanel.getStepCount(), 2, 'Click on the black element was not recorded');

        act.click(recorderHelper.stepsPanel.getStepListItem(0));
    },

    "7.Click on the toolbar": function () {
        act.click(recorderHelper.toolbar.getDragArea);
    },

    "8.Check step info panel is closed": function () {
        notOk(recorderHelper.stepsPanel.isStepInfoPanelShown(), 'Step Info panel is not shown');
    }
};

"@test"["Delete step button behavior"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.1.Check delete step button is not visible and record an action": function () {
        notOk(recorderHelper.stepsPanel.isDeleteStepButtonVisible());
        act.click(recorderHelper.toolbar.getRecordWaitActionSelector);
    },
    "1.2.Check delete step button is not visible and record an action": function () {
        act.type(document.activeElement, 2000, { replace: true });
    },
    "1.3.Check delete step button is not visible and record an action": function () {
        act.press('enter');
    },
    "2.Check delete step button is visible and start playback": function () {
        ok(recorderHelper.stepsPanel.isDeleteStepButtonVisible());
        act.click(recorderHelper.toolbar.getRunPlaybackSelector);
    },
    "3.Wait for playback started": function () {
        act.wait(5000, function () {
            return recorderHelper.toolbar.isShown();
        });
    },
    "4.Check delete step button is not visible": function () {
        notOk(recorderHelper.stepsPanel.isDeleteStepButtonVisible());
    },
    "5.Finish playback": "@mixin Finish playback",
    "6.Check delete step button is enabled and click it": function () {
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },
    "7.Check step is removed and delete step button is hidden": function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 0);
        notOk(recorderHelper.stepsPanel.isDeleteStepButtonVisible());
    }
};

"@test"["Warning tooltip after deleting last step"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.Record three actions": function () {
        act.click(["input:first", "div:contains(black)", "div:contains(red)"]);
    },
    "2.Check tooltip is not visible": function () {
        ok(!recorderHelper.tooltip.isTooltipVisible());
    },
    "3.Delete last action step": function () {
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },
    "4.Check tooltip is visible and close tooltip": function () {
        ok(recorderHelper.tooltip.isLastStepDeletionWarningVisible());

        //HACK: while T184792 is not fixed
        recorderHelper.getRoot().css('position', 'fixed');

        act.click(recorderHelper.tooltip.getCloseButton);
    },
    "5.Check tooltip is not visible": function () {
        ok(!recorderHelper.tooltip.isTooltipVisible());
    },
    "6.Delete first action step": function () {
        act.click([recorderHelper.stepsPanel.getStepListItem(0), recorderHelper.stepsPanel.getDeleteStepButton()]);
    },
    "7.Check tooltip is not visible": function () {
        ok(!recorderHelper.tooltip.isTooltipVisible());
    },
    "8.Delete the last remaining action step": function () {
        recorderHelper.getPageRoot().remove();  //T239765
        recorderHelper.getPageRoot();
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },
    "9.Check popup 'Go back to initial page' is visible and click his button": function () {
        ok(recorderHelper.isDivBackgroundShown());  //T239765

        ok(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        notOk(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
        ok(!recorderHelper.tooltip.isTooltipVisible());
        act.click(recorderHelper.popup.getPopupButtonSelector('Go back to initial page'));
    },
    "10.Check tooltip and popup are not visible": function () {
        ok(!recorderHelper.tooltip.isTooltipVisible());
        notOk(recorderHelper.popup.isAllStepsDeletionWarningVisible());
        notOk(recorderHelper.tooltip.isLastStepDeletionWarningVisible());
    },
    "11.Open edit assertion dialog": function () {
        act.click([
            recorderHelper.toolbar.getAddAssertionsSelector,
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getOkPopupItem
        ]);
    },
    "12.Fill actual argument": function () {
        act.type(recorderHelper.getCodeEditorSelector().eq(0), 'true');
    },
    "13.Add assertion step": function () {
        act.click([
            recorderHelper.editAssertionDialog.getConfirmButton,
            function () {
                return recorderHelper.getPopupButtonSelector('OK');
            }
        ]);
    },
    "14.Delete assertion step": function () {
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },
    "15.Check tooltip is not visible": function () {
        ok(!recorderHelper.tooltip.isTooltipVisible());
    }
};

"@test"['Edit assertion'] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.1.Add an assertion": function () {
        act.click([
            recorderHelper.toolbar.getAddAssertionsSelector,
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getOkPopupItem
        ]);
    },

    "1.2.Add an assertion": function () {
        act.click(recorderHelper.getCodeEditorSelector().eq(0));
    },

    "1.3.Add an assertion": function () {
        act.press('t r u e esc enter');
    },

    "1.4.Add an assertion": function () {
        act.click(recorderHelper.getPopupButtonSelector('OK'));
    },

    "2.Open the assertion": function () {
        act.click([
            recorderHelper.stepsPanel.getStepListItem(0),
            function () {
                return recorderHelper.assertionsStepDialog.getEditAssertionSelector(0);
            }
        ]);
    },

    "3.Check toolbar is hidden and modal background is shown and close the dialog": function () {
        notOk(recorderHelper.toolbar.isShown());
        ok(recorderHelper.isDivBackgroundShown());

        act.click(recorderHelper.getPopupButtonSelector('OK'));
    },

    "4.Check press action is not recorded and reopen assertion": function () {
        ok(recorderHelper.toolbar.isShown());
        eq(recorderHelper.stepsPanel.getStepCount(), 1);

        act.click(recorderHelper.assertionsStepDialog.getEditAssertionSelector(0));
    },

    "5.1.Change assertion state to failed": function () {
        act.click(recorderHelper.getCodeEditorSelector().eq(0));
    },

    "5.2.Change assertion state to failed": function () {
        act.press('ctrl+a f a l s e esc');
    },

    "5.3.Close dialog": function () {
        act.click(recorderHelper.editAssertionDialog.getConfirmButton);
    },

    "5.4.Close step info": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(0));
    },

    "6.Run playback": function () {
        act.click(recorderHelper.toolbar.getRunPlaybackSelector);
    },

    "7.Wait for playback is failed": function () {
        act.wait(PLAYBACK_TIMEOUT, recorderHelper.isPopupShown);
    },

    "8.Check dialog": function () {
        eq(recorderHelper.stepsPanel.getCurrentStepIndex(), 0);
    },

    "9.Resume playback": function () {
        act.click(recorderHelper.getPopupButtonSelector('Skip step'));
    },

    "10.Wait for playback is finished": "@mixin Finish playback"
};

"@test"['Action are not recorded if step info panel is opened after an assertion is changed'] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.1.Add an assertion": function () {
        act.click([
            recorderHelper.toolbar.getAddAssertionsSelector,
            recorderHelper.assertionsStepDialog.getAddAssertionButton,
            recorderHelper.assertionsStepDialog.getOkPopupItem
        ]);
    },

    "1.2.Add an assertion": function () {
        act.click(recorderHelper.getCodeEditorSelector().eq(0));
    },

    "1.3.Add an assertion": function () {
        act.press('t r u e esc enter');
    },

    "1.4.Add an assertion": function () {
        act.click(recorderHelper.getPopupButtonSelector('OK'));
    },

    "2.Open the assertion and open assertion argument wizard": function () {
        act.click([
            recorderHelper.stepsPanel.getStepListItem(0),
            function () {
                return recorderHelper.assertionsStepDialog.getEditAssertionSelector(0);
            },
            recorderHelper.editAssertionDialog.getOpenWizardButton
        ]);
    },

    "3.Close wizard and press any button": function () {
        act.press('esc a');
    },

    "4.Close the dialog": function () {
        act.click(recorderHelper.getPopupButtonSelector('Cancel'));
    },

    "5.Check the second 'esc' button press is not recorded and click on the link on the page": function () {
        ok(recorderHelper.toolbar.isShown());
        eq(recorderHelper.stepsPanel.getStepCount(), 1);

        act.click('#link');
    },

    "6.Check click action is not recorded": function () {
        eq(recorderHelper.stepsPanel.getStepCount(), 1);
    }
};

"@test"['Steps selection'] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.Record ten actions": "@mixin Record ten action",

    "2.Select second item with ctrl": function () {
        checkSelectedItems([9], 10);

        act.click(recorderHelper.stepsPanel.getStepListItem(1), {
            ctrl: true
        });
    },

    "3.Select seventh item with ctrl": function () {
        checkSelectedItems([1, 9]);

        act.click(recorderHelper.stepsPanel.getStepListItem(6), {
            ctrl: true
        });
    },

    "4.Deselect last item with ctrl": function () {
        checkSelectedItems([1, 6, 9]);

        act.click(recorderHelper.stepsPanel.getLastItem(), {
            ctrl: true
        });
    },

    "5.Delete selected items": function () {
        checkSelectedItems([1, 6]);

        act.click(recorderHelper.stepsPanel.getDeleteStepButton());
    },

    "6.Try deselect single selected item with ctrl": function () {
        checkSelectedItems([5], 8);

        act.click(recorderHelper.stepsPanel.getStepListItem(5), {
            ctrl: true
        });
    },

    "7.Reset selection by recording click action": function () {
        checkSelectedItems([5]);

        act.click($('body'));
    },

    "8.Open last recorded step": function () {
        checkSelectedItems([8], 9);

        act.click(recorderHelper.stepsPanel.getLastItem());
    },

    "9.Try select opened step with ctrl": function () {
        checkSelectedItems([]);
        eq(recorderHelper.stepsPanel.getCurrentStepIndex(), 8, 'last item is opened');

        act.click(recorderHelper.stepsPanel.getLastItem(), {
            ctrl: true
        });
    },

    "10.Try select opened step with shift": function () {
        checkSelectedItems([]);
        eq(recorderHelper.stepsPanel.getCurrentStepIndex(), 8, 'last item is opened');

        act.click(recorderHelper.stepsPanel.getLastItem(), {
            shift: true
        });
    },

    "11.Close opened step": function () {
        checkSelectedItems([]);
        eq(recorderHelper.stepsPanel.getCurrentStepIndex(), 8, 'last item is opened');

        act.click(recorderHelper.stepsPanel.getLastItem());
    },

    "12.Try deselect single selected item with shift": function () {
        checkSelectedItems([8]);

        act.click(recorderHelper.stepsPanel.getLastItem(), {
            shift: true
        });
    },

    "13.Create multi selection from selected start item": function () {
        checkSelectedItems([8]);

        act.click(recorderHelper.stepsPanel.getStepListItem(3), {
            shift: true
        });
    },

    "14.Open third item": function () {
        checkSelectedItems([3, 4, 5, 6, 7, 8]);

        act.click(recorderHelper.stepsPanel.getStepListItem(2));
    },

    "15.Create multi selection from opened start item": function () {
        checkSelectedItems([]);
        eq(recorderHelper.stepsPanel.getCurrentStepIndex(), 2, 'third item is opened');

        act.click(recorderHelper.stepsPanel.getStepListItem(4), {
            shift: true
        });
    },

    "16.Delete selected items": function () {
        checkSelectedItems([2, 3, 4]);

        act.click(recorderHelper.stepsPanel.getDeleteStepButton());
    },

    "17.Make selected only second item": function () {
        checkSelectedItems([2], 6);

        act.click([recorderHelper.stepsPanel.getStepListItem(1), recorderHelper.stepsPanel.getStepListItem(1)]);
    },

    "18.Create multi selection from up to down": function () {
        checkSelectedItems([1]);

        act.click(recorderHelper.stepsPanel.getStepListItem(3), {
            shift: true
        });
    },

    "19.Click item below end of selection with shift": function () {
        checkSelectedItems([1, 2, 3]);

        act.click(recorderHelper.stepsPanel.getStepListItem(5), {
            shift: true
        });
    },

    "20.Click item inside selection with shift": function () {
        checkSelectedItems([1, 2, 3, 4, 5]);

        act.click(recorderHelper.stepsPanel.getStepListItem(4), {
            shift: true
        });
    },

    "21.Click item above start of selection with shift": function () {
        checkSelectedItems([1, 2, 3, 4]);

        act.click(recorderHelper.stepsPanel.getStepListItem(0), {
            shift: true
        });
    },

    "22.Reset selection by recording click action": function () {
        checkSelectedItems([0, 1]);

        act.click($('body'));
    },

    "23.Make selected only forth item": function () {
        checkSelectedItems([6], 7);

        act.click([recorderHelper.stepsPanel.getStepListItem(3), recorderHelper.stepsPanel.getStepListItem(3)]);
    },

    "24.Create multi selection from down to up": function () {
        checkSelectedItems([3]);

        act.click(recorderHelper.stepsPanel.getStepListItem(1), {
            shift: true
        });
    },

    "25.Click item above start of selection with shift": function () {
        checkSelectedItems([1, 2, 3]);

        act.click(recorderHelper.stepsPanel.getStepListItem(0), {
            shift: true
        });
    },

    "26.Click item inside selection with shift": function () {
        checkSelectedItems([0, 1, 2, 3]);

        act.click(recorderHelper.stepsPanel.getStepListItem(2), {
            shift: true
        });
    },

    "27.Click item below end of selection with shift": function () {
        checkSelectedItems([2, 3]);

        act.click(recorderHelper.stepsPanel.getStepListItem(6), {
            shift: true
        });
    },

    "28.Deselect one of selected items": function () {
        checkSelectedItems([3, 4, 5, 6]);

        act.click(recorderHelper.stepsPanel.getStepListItem(5), {
            ctrl: true
        });
    },

    "29.Create multi selection from new start point": function () {
        checkSelectedItems([3, 4, 6]);

        act.click(recorderHelper.stepsPanel.getStepListItem(2), {
            shift: true
        });
    },

    "30.Check selected items": function () {
        checkSelectedItems([2, 3, 4, 5]);
    }
};

"@test"['"keysCommand" action argument label text must be equal to "Keys"'] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.Record the press action": function () {
        act.press('t');
    },

    "2.Open step info": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(0));
    },

    "3.Check popup has \"Keys\" label instead of \"KeysCommand\"": function () {
        ok(recorderHelper.stepsPanel.hasCurrentStepActionArgumentProperty(/keys/i), 'Step Info panel is shown');
        ok(!recorderHelper.stepsPanel.hasCurrentStepActionArgumentProperty(/keyscommand/i), 'Step Info panel is shown');
    }
};

"@test"['Prevent redundant scrolling and page selection in IE'] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.Record the click action": function () {
        act.click('.black', { offsetX: 1, offsetY: 1});
    },

    "2.Scroll page and open step info": function () {
        eq(window.blackClick, 1, 'Click on the black element is successful');

        $('body').height(5000);
        this.scrollTopValue = 1500;
        $(window).scrollTop(this.scrollTopValue);

        act.click(recorderHelper.stepsPanel.getStepListItem(0));
    },

    "3.Check scroll and close opened step": function () {
        eq($(window).scrollTop(), this.scrollTopValue);
        act.click(recorderHelper.stepsPanel.getStepListItem(0));
    },

    "4.Check stored scroll value and scroll to top": function () {
        eq($(window).scrollTop(), this.scrollTopValue);
        $(window).scrollTop(0);
    },

    "5.Click delete button": function () {
        act.click(recorderHelper.stepsPanel.getDeleteStepButton);
    },

    "6.Restart recording": function () {
        act.click(recorderHelper.popup.getPopupButtonSelector('Go back to initial page'));
    },

    "7.Record ten actions": "@mixin Record ten action",

    "8.Open first step info": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(0));
    },

    "9.Select all step": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(recorderHelper.stepsPanel.getStepCount() - 1), {
            shift: true
        })
    },

    "10.Check that page doesn't contain selection region": function () {
        var currentSelection = window.getSelection();
        eq(currentSelection.anchorNode, currentSelection.focusNode);
    }
};

"@test"['Press action - incorrect value validation'] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.Record click action": function () {
        act.click('.black');
    },

    '2.1.Save test': function () {
        this.fileName = 'baserecorderpage.test.js';
        this.testName = 'temp_test';
        act.click(recorderHelper.toolbar.getSaveTestButtonSelector);
    },

    '2.2.Set test name': function () {
        act.type(recorderHelper.setTestNameDialog.getTestNameInput, this.testName);
    },

    '2.3.Click Save button': function () {
        act.click(recorderHelper.dialog.getSaveButton);
    },

    "3.Record the press action": function () {
        act.press('t');
    },

    "4.Open step info": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(1));
    },

    "5.1.Make 'Keys' argument incorrect": function () {
        act.type(recorderHelper.stepsPanel.getStepActionArgumentProperty(/keys/i), '12345');
    },

    "5.2.Close stepInfo": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(1));
    },

    "6.Record another press action": function () {
        act.press('g');
    },

    "7.Open step info": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(2));
    },

    "8.1.Make 'Keys' argument incorrect, remove input value": function () {
        act.type(recorderHelper.stepsPanel.getStepActionArgumentProperty(/keys/i), ' ', {replace: true});
    },

    "8.2.Close stepInfo": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(2));
    },

    "9.1.Record wait action": function () {
        act.click(recorderHelper.toolbar.getRecordWaitActionSelector)
    },

    "9.2.Record wait action": function () {
        act.press('enter');
    },

    "10.Open step info": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(3));
    },

    "11.1.Make 'Ms' argument incorrect, remove input value": function () {
        act.type(recorderHelper.stepsPanel.getStepActionArgumentProperty(/ms/i), ' ', {replace: true});
    },

    "11.2.Close stepInfo": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(3));
    },

    "12.Move to another page to reload stepsInfo": function () {
        act.click($('#linkToAnotherPage'));
    },

    "13.Check failed steps": function () {
        notOk(recorderHelper.stepsPanel.isStepFailed(0));
        ok(recorderHelper.stepsPanel.isStepFailed(1));
        ok(recorderHelper.stepsPanel.isStepFailed(2));
        ok(recorderHelper.stepsPanel.isStepFailed(3));
        notOk(recorderHelper.stepsPanel.isStepFailed(4));
    },

    "14.1.Check first press action argument validation message": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(1));
    },

    "14.2.Check first press action argument validation message": function () {
        eq(recorderHelper.stepsPanel.getStepActionArgumentErrorText(/keys/i), 'Incorrect keys');
    },

    "15.1.Check first press action argument validation message": function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(2));
    },

    "15.2.Check first press action argument validation message": function () {
        eq(recorderHelper.stepsPanel.getStepActionArgumentErrorText(/keys/i), 'Incorrect keys');
    },

    "16.Exit recording": function () {
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    "17.Check popup text and click 'No' to cancel quit": function () {
        ok(recorderHelper.isPopupShown());
        ok(recorderHelper.popup.exitWithoutSavingPopupVisible());

        act.click(recorderHelper.dialog.getNoButton);
    },

    "18. Check failed steps": function () {
        notOk(recorderHelper.stepsPanel.isStepFailed(0));
        ok(recorderHelper.stepsPanel.isStepFailed(1));
        ok(recorderHelper.stepsPanel.isStepFailed(2));
        ok(recorderHelper.stepsPanel.isStepFailed(3));
        notOk(recorderHelper.stepsPanel.isStepFailed(4));
    },

    "19.Exit recording": function () {
        act.click(recorderHelper.toolbar.getExitRecordingButtonSelector);
    },

    "20.Check popup text and click 'Yes'": function () {
        ok(recorderHelper.isPopupShown());
        ok(recorderHelper.popup.exitWithoutSavingPopupVisible());

        act.click(recorderHelper.dialog.getYesButton);
    },

    '21.Wait for update file system': function () {
        act.wait(5000, function () {
            return recorderHelper.controlPanelHelper.getTest(this.fileName, this.testName).is(':visible');
        });
    },

    '22.1.Check that changes were not saved': function () {
        ok(recorderHelper.controlPanelHelper.hasFixtureTest(this.testName, this.fileName));

        act.click(recorderHelper.controlPanelHelper.getEditTestButtonSelector(this.testName));
    },

    '22.2.Check that changes were not saved': function () {
        var testCode = $(".ace_content").text().replace(/\s/g, "");

        notEq(testCode.indexOf('\'1.Clickdiv"black"\':function(){act.click(":containsExcludeChildren(black)");}}'), -1);
    },

    '23.Close edit dialog and delete test': '@mixin Close Edit Dialog And Delete Test'
};

"@test"['Sorting behavior'] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.Record ten actions": "@mixin Record ten action",

    "2.Click on item with some mousemove events": function () {
        act.drag(recorderHelper.stepsPanel.getStepListItemName(1), 4, 4);
    },

    "3.Check opened step and close it": function () {
        ok(recorderHelper.stepsPanel.isStepInfoPanelShown(), 'Step Info panel is shown');
        eq(recorderHelper.stepsPanel.getOpenedStepIndex(), 1, 'Second step is opened');
        eq(recorderHelper.stepsPanel.getStepListItemName(1).text(), 'Press key 2', 'Check non dragged item');

        act.click(recorderHelper.stepsPanel.getStepListItem(1));
    },

    "4.Try to turn sorting mode via only horizontally dragging item": function () {
        act.drag(recorderHelper.stepsPanel.getStepListItemName(2), 40, 0);
    },

    "5.Check opened step and close it": function () {
        ok(recorderHelper.stepsPanel.isStepInfoPanelShown(), 'Step Info panel is shown');
        eq(recorderHelper.stepsPanel.getOpenedStepIndex(), 2, 'Second step is opened');
        eq(recorderHelper.stepsPanel.getStepListItemName(2).text(), 'Press key 3', 'Check non dragged item');

        act.click(recorderHelper.stepsPanel.getStepListItem(2));
    },

    "6.Click on item sortable area with some mousemove events": function () {
        //NOTE: We can't test item sorting because we don't simulate mousemove event and don't replace place holder during dragging
        //that's why 'mousedown' and 'mouseup' event target elements are the same and we raise 'click' which open item

        act.drag(recorderHelper.stepsPanel.getItemSortableArea(3), 4, 4);
    },

    "7.Check selected step after transfet stepPanel to sorting mode": function () {
        notOk(recorderHelper.stepsPanel.isStepInfoPanelShown(), 'Step Info panel is not shown');
        eq(recorderHelper.stepsPanel.getOpenedStepIndex(), null, 'Second step is opened');
        eq(recorderHelper.stepsPanel.getSelectedItemIndexes(), [3], 'Fourth step is opened');
    }
};

function checkSelectedItems(selectedIndexes, itemCount) {
    eq(recorderHelper.stepsPanel.getSelectedItemIndexes(), selectedIndexes);

    if (typeof itemCount !== 'undefined')
        eq(recorderHelper.stepsPanel.getStepCount(), itemCount);
}