"@fixture Record actions via toolbar";
"@page ./tests";

"@require :recorderHelper";
"@require ../../../mixins/recorder.js";

"@test"["Record hover action"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.Focus the input": function () {
        act.click(['#input', recorderHelper.getCompleteClickIcon]);
    },

    "2.Start hover action recording via toolbar": function () {
        act.click(recorderHelper.toolbar.getRecordHoverActionSelector);
    },

    "3. Bind handlers for check that we don't raise actions for element frame (T199100)": function () {
        window.wasElementFrameActions = false;
        var elementFrame = null;

        document.onmousemove = function (e) {
            if (!elementFrame)
                elementFrame = recorderHelper.getElementPickerFrame()[0];

            if (e.target === elementFrame)
                window.wasElementFrameActions = true;
        };

        document.onmouseout = function (e) {
            if (!elementFrame)
                elementFrame = recorderHelper.getElementPickerFrame()[0];

            if (e.target === elementFrame)
                window.wasElementFrameActions = true;
        };

        document.onmouseover = function (e) {
            if (!elementFrame)
                elementFrame = recorderHelper.getElementPickerFrame()[0];

            if (e.target === elementFrame)
                window.wasElementFrameActions = true;
        };
    },

    "4.Hover over the action target": function () {
        act.hover('#hoverDiv');
    },

    "5.Select the action target": function () {
        var $el = $('#hoverDiv');
        ok(recorderHelper.isElementFrameShown($el[0]));

        act.click($el);
    },

    "6.Check element state and open the second recorded step": function () {
        //T199100 - check that we don't raise actions for element frame
        notOk(window.wasElementFrameActions);

        var $hoverDiv = $('#hoverDiv');
        ok($hoverDiv.html().indexOf('mouseover') > -1);
        notOk($hoverDiv.html().indexOf('click') > -1);
        eq(recorderHelper.stepsPanel.getStepCount(), 2);

        act.click(recorderHelper.stepsPanel.getStepListItem(1));
    },

    "7.Check the step": function () {
        eq(recorderHelper.stepsPanel.getCurrentStepType(), 'hover');
        eq($.trim(recorderHelper.stepsPanel.getCurrentSelector()), '$("#hoverDiv")');
    }
};

function getWaitActionStepName(ms) {
    return 'Wait ' + ms + ' milliseconds';
}

"@test"["Record wait action"] = {
    "-1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('BaseRecorderPage'));
    },

    "1.Focus the input": function () {
        act.click(['#input', recorderHelper.getCompleteClickIcon]);
    },

    "2.Start wait action recording via toolbar": function () {
        act.click(recorderHelper.toolbar.getRecordWaitActionSelector);
    },

    "3.Check dialog and change value to 3000": function () {
        ok(recorderHelper.isPopupShown());
        eq($(document.activeElement).parent().prev().text().toLowerCase(), 'ms:');
        eq(recorderHelper.selectUIElementByText('Name').parent().find('div input')[0].value, getWaitActionStepName(1000));
        eq(recorderHelper.popup.getPopupTitle(), getWaitActionStepName(1000));

        act.type(document.activeElement, '3000');
    },

    "4.Check messages": function () {
        eq(recorderHelper.selectUIElementByText('Name').parent().find('div input')[0].value, getWaitActionStepName(3000));
        eq(recorderHelper.popup.getPopupTitle(), getWaitActionStepName(3000));
    },

    "5.Check Add Action button is enabled": function () {
        ok(!recorderHelper.getPopupButtonSelector('OK').attr('disabled'));
    },

    "6.Change value to 0": function () {
        act.press('backspace backspace backspace backspace 0');
    },

    "7.Wait for validation": function () {
        act.wait(500);
    },

    "8.Check Add Action button is disabled": function () {
        ok(recorderHelper.getPopupButtonSelector('OK').attr('disabled'));
    }
};

