"@fixture Upload";
"@page ./tests/Recorder/Upload";

"@require :recorderHelper";
"@require ./../../../mixins/recorder.js";

'@test'['Uploading'] = {
	 '@testCases': [{ 
		'@name': 'Upload', 
			filesToUpload: ['./../../../uploads/img1.jpg'], 
			uploadedFiles: ['./uploads/img1.jpg'],		
			stepName: 'Upload "img1.jpg" file',
			generatedAction: 'act.upload("#file", "./uploads/img1.jpg");'
		}, { 
		'@name': 'Multiupload', 
			filesToUpload: ['./../../../uploads/img1.jpg', './../../../uploads/img2.jpg'], 
			uploadedFiles: ['./uploads/img1.jpg', './uploads/img2.jpg'],
			stepName: 'Upload "img1.jpg", "img2.jpg" files',
			generatedAction: 'act.upload("#file", [ "./uploads/img1.jpg", "./uploads/img2.jpg" ]);'
		}
    ],
	
    "1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('UploadPage'));
    },
	
	'2.Upload file(s)': function () {
        act.upload('#file', this.filesToUpload);
    },
	
	'3.Wait uploding': '@mixin Wait file uploding',
	
	'4.Check tooltip': function () {
		var tooltipText = recorderHelper.tooltip.getToolTipText();
		
		for(var i = 0; i < this.uploadedFiles.length; i++)
			notEq(tooltipText.indexOf(this.uploadedFiles[i]), -1);
			
		if(!this.uploadedFiles.length)
		    eq(tooltipText, 'The file input was cleared.');
    },
	
	'5.Open upload step dialog': function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(0));
    },

    '6.Check step params': function () {
		var files = recorderHelper.stepsPanel.getCurrentUploadStepFiles();

        if(this.uploadedFiles.length > 0)
            eq(files, this.uploadedFiles.join(''));
        else
            notOk(files);
        
		eq(recorderHelper.stepsPanel.getCurrentStepType(), 'upload');
		eq(recorderHelper.stepsPanel.getCurrentSelector(), '$("#file")');
		notEq(recorderHelper.stepsPanel.getStepListItem(0).text().indexOf(this.stepName), -1);
    },
	
	'7.Save Test': '@mixin Save Test And Open Edit Dialog',
	
	'8.Check generated test code': function () {
        var testCode = $(".ace_content").text().replace(/\s/g, " ");

		notEq(testCode.indexOf(this.generatedAction), -1);
		notEq(testCode.indexOf(this.stepName), -1);
    },
	
	'9.Close edit dialog and delete test': '@mixin Close Edit Dialog And Delete Test'
};

"@test"["Clear"] = {
    "1.Start recording": function () {
        act.click(recorderHelper.controlPanelHelper.getStartRecordingSelector('UploadPage'));
    },

    '2.Upload "img1.jpg" file': function() {
        act.upload('#file', './../../../uploads/img1.jpg');
    },
	
    '3.Wait uploding': '@mixin Wait file uploding',
	
    '4.Clean up file input': function () {
        act.upload('#file');
    },
	
    '5.Wait uploding': '@mixin Wait file uploding',
	
    '6.Check tooltip': function () {
        var tooltipText = recorderHelper.tooltip.getLastTooltipText();
		
        eq(tooltipText, 'The file input was cleared.');
    },
	
    '7.Open upload step dialog': function () {
        act.click(recorderHelper.stepsPanel.getStepListItem(1));
    },

    '8.Check step params': function () {
        var files = recorderHelper.stepsPanel.getCurrentUploadStepFiles();

        notOk(files);
        
        eq(recorderHelper.stepsPanel.getCurrentStepType(), 'upload');
        eq(recorderHelper.stepsPanel.getCurrentSelector(), '$("#file")');
        notEq(recorderHelper.stepsPanel.getStepListItem(1).text().indexOf('Clean up file input'), -1);
    },
	
    '9.Save Test': '@mixin Save Test And Open Edit Dialog',
	
    '10.Check generated test code': function () {
        var testCode = $(".ace_content").text().replace(/\s/g, " ");

        notEq(testCode.indexOf('act.upload("#file");'), -1);
        notEq(testCode.indexOf('Clean up file input'), -1);
    },
    
    '11.Close edit dialog and delete test': '@mixin Close Edit Dialog And Delete Test'
};

