"@fixture Native dialogs handling";
"@page ./project/Native%20dialogs";

'@require :recorderHelper';
"@require ./../../../mixins/testrunner.js";

'@test'['Native dialogs - succeed tests'] = {
    '@testCases': [
        {
            '@name': 'Confirm (false) after redirect',
            fixture: 'basePage',
            testName: 'Confirm false after redirect',
            succeed: true
        },
        {
            '@name': 'Confirm (true) after redirect',
            fixture: 'basePage',
            testName: 'Confirm true after redirect',
            succeed: true
        },
        {
            '@name': 'Confirm after an action succeed',
            fixture: 'basePage',
            testName: 'Confirm after an action should succeed',
            succeed: true
        },
        {
            '@name': 'Confirm dialog on page load',
            fixture: 'confirmPage',
            testName: 'Confirm dialog on page load',
            succeed: true
        },
        {
            '@name': 'Before unload dialog',
            fixture: 'basePage',
            testName: 'Before unload dialog should succeed',
            succeed: true
        },
        {
            '@name': 'Expected native dialog after the click on the link without redirect',
            fixture: 'basePage',
            testName: 'Expected native dialog after the click on the link without redirect',
            succeed: true
        }
    ],

    '1.Run test and wait for results': '@mixin Run test and go to results',

    '2.Check test results': function () {
        eq(recorderHelper.testResults.getFailIndicator().length, this.succeed ? 0 : 1);
    }
};

'@test'['Native dialogs - failed tests'] = {
    '@testCases': [
        {
            '@name': 'Confirm after an action failed',
            fixture: 'basePage',
            testName: 'Confirm after an action should be failed',
            succeed: false,
            errorText: 'Error on step "1.Click submit button'
        },
        {
            '@name': 'Confirm after redirect',
            fixture: 'basePage',
            testName: 'Confirm after redirect',
            succeed: false,
            errorText: 'Error on step "1.Click link'
        },
        {
            '@name': 'No expected confirm after an action',
            fixture: 'basePage',
            testName: 'No expected confirm after an action',
            succeed: false,
            errorText: 'Error on step "1.Click submit button "Button"'
        },
        {
            '@name': 'No expected confirm after redirect',
            fixture: 'basePage',
            testName: 'No expected confirm after redirect',
            succeed: false,
            errorText: 'Error on step "1.Click link'
        },
        {
            '@name': 'No expected before unload dialog',
            fixture: 'basePage',
            testName: 'No expected before unload dialog',
            succeed: false,
            errorText: 'Error on step "1.Click link'
        },
        {
            '@name': 'Unexpected before unload dialog',
            fixture: 'basePage',
            testName: 'Unexpected before unload dialog',
            succeed: false,
            errorText: 'Error on step "2.Click link'
        }
    ],

    '1.Run test and wait for results': '@mixin Run test and go to results',

    '2.Check test results': function () {
        eq(recorderHelper.testResults.getFailIndicator().length, this.succeed ? 0 : 1);
    },

    '3.Expand failed test': '@mixin Expand failed test',

    '4.Check error message': function () {
        ok(recorderHelper.testResults.getErrorMessageText().indexOf(this.errorText) > -1);
    }
};

'@test'['Native dialogs in an iFrame - succeed tests'] = {
    '@testCases': [
        {
            '@name': 'Confirm (false) after redirect',
            fixture: 'iframePage',
            testName: 'Confirm false after redirect in iframe',
            succeed: true
        },
        {
            '@name': 'Confirm (true) after redirect',
            fixture: 'iframePage',
            testName: 'Confirm true after redirect in iframe',
            succeed: true
        },
        {
            '@name': 'Confirm after an action succeed',
            fixture: 'iframePage',
            testName: 'Confirm after an action in iframe should succeed',
            succeed: true
        },
        {
            '@name': 'Before unload dialog',
            fixture: 'basePage',
            testName: 'Before unload dialog should succeed',
            succeed: true
        }
    ],

    '1.Run test and wait for results': '@mixin Run test and go to results',

    '2.Check test results': function () {
        eq(recorderHelper.testResults.getFailIndicator().length, this.succeed ? 0 : 1);
    }
};

'@test'['Native dialogs in an iFrame - failed tests'] = {
    '@testCases': [
        {
            '@name': 'Confirm after an action failed',
            fixture: 'iframePage',
            testName: 'Confirm after an action in iframe should be failed',
            succeed: false,
            errorText: 'Error on step "1.Click submit button'
        },
        {
            '@name': 'Confirm after redirect',
            fixture: 'iframePage',
            testName: 'Confirm after redirect in iframe',
            succeed: false,
            errorText: 'Error on step "1.Click link'
        },
        {
            '@name': 'No expected confirm after an action in iframe',
            fixture: 'iframePage',
            testName: 'No expected confirm after an action in iframe',
            succeed: false,
            errorText: 'Error on step "1.Click submit button "Button"'
        },
        {
            '@name': 'No expected confirm after redirect in iframe',
            fixture: 'iframePage',
            testName: 'No expected confirm after redirect in iframe',
            succeed: false,
            errorText: 'Error on step "1.Click link'
        },
        {
            '@name': 'No expected before unload dialog',
            fixture: 'basePage',
            testName: 'No expected before unload dialog',
            succeed: false,
            errorText: 'Error on step "1.Click link'
        },
        {
            '@name': 'Unexpected before unload dialog',
            fixture: 'basePage',
            testName: 'Unexpected before unload dialog',
            succeed: false,
            errorText: 'Error on step "2.Click link'
        }
    ],

    '1.Run test and wait for results': '@mixin Run test and go to results',

    '2.Check test results': function () {
        eq(recorderHelper.testResults.getFailIndicator().length, this.succeed ? 0 : 1);
    },

    '3.Expand failed test': '@mixin Expand failed test',

    '4.Check error message': function () {
        ok(recorderHelper.testResults.getErrorMessageText().indexOf(this.errorText) > -1);
    }
};