var FIXTURE_INFO_SELECTOR = '.fixture-info',
    DELETE_TEST_BUTTON_SELECTOR = '.delete-test-btn',
    EDIT_TEST_BUTTON_SELECTOR = '.edit-fixture-code-btn',
    CONFIRM_DELETE_TEST_BUTTON_SELECTOR = '#delete-test-yes-btn',

    REPORT_ERROR_ROW_SELECTOR = '.error-row',
    REPORT_SCREENSHOT_THUMBNAIL_SELECTOR = '.thumbnail';

function getFixtureInfo(filename) {
    return recorderHelper.select(FIXTURE_INFO_SELECTOR).filter(function () {
        return $(this).attr('data-fixture-file') === filename;
    });
}

function getTest(fileName, testName) {
    var source = fileName ? getFixtureInfo(fileName) : $('*');

    return source.find('table:.tests').find('tr').filter(function () {
        return $(this).attr('data-test-name') === testName;
    });
}

function hideFixedControlPanelElements () {
    $('.header, .footer, .top-buttons-push, .fixtures-top-buttons').css('visibility', 'hidden');
}

var controlPanelHelper = {
    getStartRecordingSelector: function (testName) {
        var testNameSpan = controlPanelHelper.getFixtureElementSelector(testName);

        testNameSpan[0].scrollIntoView();
        hideFixedControlPanelElements();
        return [testNameSpan, testNameSpan.parents('tr').find('.record-button')];
    },

    getFixtureElementSelector: function (testName) {
        var testNameSpan = $(":containsExcludeChildren(" + testName + ")").eq(0);

        testNameSpan.parents('tr').find('.record-button').css('visibility', 'visible');
        hideFixedControlPanelElements();
        return testNameSpan;
    },

    getRunTestSelector: function (fixtureName, testName) {
        var testNameSpan = controlPanelHelper.getFixtureElementSelector(fixtureName),
            testNameDiv = $(":containsExcludeChildren(" + testName + ")").eq(0);

        hideFixedControlPanelElements();
        return [ function () {
            testNameSpan[0].scrollIntoView();
            return testNameSpan;
        }, function () {
            var $runButton = testNameDiv.parent().parent().find('.run-button-cell');

            if ($runButton.length) {
                $runButton[0].scrollIntoView();

                //NOTE: we should do this because of errors scrollIntoView method
                var top = $runButton[0].getBoundingClientRect().top,
                    fixedHeight = $('.header').height() + $('.fixtures-top-buttons').height() + $('.header-line').height();

                if ($runButton.is(':visible') && top < fixedHeight)
                    $(window).scrollTop(Math.max(0, $(window).scrollTop() - fixedHeight + top));
            }

            return $runButton;
        }, '#dialog-run-btn' ]
    },

    takeScreenshotSelector: function () {
        return '#take-screenshot-option';
    },

    failOnJsErrorsOptSelector: function () {
        return '#fail-on-js-errors-option';
    },

    getTestCount: function (fileName) {
        return getFixtureInfo(fileName).find('table:.tests').find('tr').filter(function () {
            return $(this).attr('data-test-name')
        }).length;
    },

    getTest: getTest,

    hasFixtureTest: function (testName, fileName) {
        return getTest(fileName, testName).length;
    },

    getNoTestTitle: function () {
        return getFixtureInfo(fileName).find('table:.tests').find('tr:.no-tests-in-fixture');
    },

    getEditTestButtonSelector: function (testName, fileName) {
        var $buttons = getTest(fileName, testName).find('.button');

        $buttons.css('visibility', 'visible');

        return $buttons.filter(function () {
            return $(this).is(EDIT_TEST_BUTTON_SELECTOR);
        });
    },

    getDeleteTestButtonSelector: function (testName, fileName) {
        var $buttons = getTest(fileName, testName).find('.button');

        $buttons.css('visibility', 'visible');

        var $deleteBtn = $buttons.filter(function () {
            return $(this).is(DELETE_TEST_BUTTON_SELECTOR);
        });

        return [$deleteBtn, CONFIRM_DELETE_TEST_BUTTON_SELECTOR];
    },

    getReportErrorRows: function () {
        return $(REPORT_ERROR_ROW_SELECTOR);
    },

    getReportScreenshotCount: function () {
        return $(REPORT_SCREENSHOT_THUMBNAIL_SELECTOR);
    }
};

recorderHelper.controlPanelHelper = controlPanelHelper;