var POPUP_DIALOG_CLASS = 'popup',
    CLOSE_BUTTON_CLASS = 'close-button',
    CURSOR_CLASS = 'cursor',
    SELECTOR_EDITOR_SELECTOR = '.selector-editor .editable',
    CODE_EDITOR_SELECTOR = '.codeEditor',
    ELEMENT_FRAME_CLASS = 'element-frame',
    ELEMENT_FRAME_PART_CLASS = 'element-frame-part',
    ELEMENT_FRAME_WIDTH = 2;

var cssPostfix = '',

    isIE11 = !!(navigator.appCodeName === 'Mozilla' && /Trident\/7.0/.test(navigator.userAgent)),
    isIE = !!$.browser.msie || isIE11,
    isMozilla = !!$.browser.mozilla && !isIE11;

function fillCssPostfix() {
    var $el = $('[id^=root-]');

    if ($el.length)
        cssPostfix = $el[0].className.split(' ')[0].replace('root', '');
}

function select(classSelector, $parent) {
    if ($parent)
        return $parent.find(patchClassSelector(classSelector));

    return $(patchClassSelector(classSelector));
}

function patchClassSelector(classSelector) {
    if (!cssPostfix)
        fillCssPostfix();

    var classes = classSelector.split(' ');
    for (var i = 0; i < classes.length; i++)
        classes[i] += cssPostfix;

    return classes.join(' ');
}

function isCursorShown() {
    return isElementVisible('.' + CURSOR_CLASS);
}

function getCompleteClickIcon() {
    if (!cssPostfix)
        fillCssPostfix();

    return $('.complete-icon' + cssPostfix + '.click' + cssPostfix);
}

function getCompleteTypingIcon() {
    if (!cssPostfix)
        fillCssPostfix();

    return $('.complete-icon' + cssPostfix + '.typing' + cssPostfix);
}

function isElementFrameShown(el, inFrame) {
    var $elementFrame = select('.' + ELEMENT_FRAME_CLASS);

    if (isIE && inFrame) {
        $elementFrame = select('.' + ELEMENT_FRAME_PART_CLASS).filter(function (index) {
            return $(this).is(':visible');
        });

        return $elementFrame.length === 4;
    }

    if (!el)
        return !!$elementFrame.length && $elementFrame.is(':visible');

    if (!$elementFrame.length)
        return false;

    return $elementFrame[0].offsetLeft === el.offsetLeft - ELEMENT_FRAME_WIDTH &&
        $elementFrame[0].offsetTop === el.offsetTop - ELEMENT_FRAME_WIDTH &&
        $elementFrame[0].offsetWidth === el.offsetWidth + 2 * ELEMENT_FRAME_WIDTH &&
        $elementFrame[0].offsetHeight === el.offsetHeight + 2 * ELEMENT_FRAME_WIDTH;
}

function getElementPickerFrame() {
    return select('.' + ELEMENT_FRAME_CLASS);
}

function getIFrameSelectorEditor() {
    var $selectorEditors = select(SELECTOR_EDITOR_SELECTOR);

    return $selectorEditors.length < 2 ? null : $selectorEditors.eq(0);
}

function getElementSelectorEditor() {
    var $selectorEditors = select(SELECTOR_EDITOR_SELECTOR);

    return $selectorEditors.eq($selectorEditors.length - 1);
}

function isElementVisible(selector) {
    var $element = select(selector);
    return $element.length && $element.is(':visible') && $element.css('visibility') !== 'hidden';
}

var addActionDialog = (function () {
    return {
        getAddActionButton: function () {
            return $('button:containsExcludeChildren("OK")');
        },
        getCancelButton: function () {
            return $('button:containsExcludeChildren("Cancel")');
        },
        getElementSelectorEditor: getElementSelectorEditor
    }
})();

var stepsPanel = (function () {
    var STEP_LIST_PANEL_SELECTOR = '.step-list-panel',
        STEP_INFO_PANEL_SELECTOR = '.stepInfo-panel',
        STEP_LIST_ITEM_SELECTOR = '.item',
        STEP_LIST_ITEM_NAME_SELECTOR = '.name',
        ACTION_TYPE_FIELD_SELECTOR = '.action-type',
        DELETE_STEP_BUTTON_AREA_SELECTOR = '.delete-step-button-area',
        DELETE_STEP_BUTTON_SELECTOR = '.delete-step-button',
        ITEM_SELECTED_SELECTOR = '.selected',
        ITEM_OPENED_SELECTOR = '.opened',
        ACTION_STEP_SELECTOR = '.action-step',
        OPTIONS_SELECTOR = '.options',
        DIALOG_PROPERTY_SELECTOR = '.dialog-property',
        FIELD_SELECTOR = '.field',
        VALIDATION_SELECTOR = '.validation';

    return {
        getStepListPanel: function () {
            return select(STEP_LIST_PANEL_SELECTOR);
        },
        getStepCount: function () {
            return select(STEP_LIST_PANEL_SELECTOR).find(STEP_LIST_ITEM_SELECTOR + cssPostfix).length;
        },
        getStepListItem: function (index) {
            return select(STEP_LIST_PANEL_SELECTOR).find(STEP_LIST_ITEM_SELECTOR + cssPostfix).eq(index);
        },
        getStepListItemName: function (index) {
            return select(STEP_LIST_PANEL_SELECTOR).find(STEP_LIST_ITEM_SELECTOR + cssPostfix).find(STEP_LIST_ITEM_NAME_SELECTOR + cssPostfix).eq(index);
        },
        getCurrentStepType: function () {
            return select(STEP_INFO_PANEL_SELECTOR).find(ACTION_TYPE_FIELD_SELECTOR + cssPostfix).text();
        },
        hasCurrentStepActionArgumentProperty: function (labelTextRegExp) {
            var $argumentPropertyEditorLabels = select([STEP_INFO_PANEL_SELECTOR, ACTION_STEP_SELECTOR, OPTIONS_SELECTOR,
                DIALOG_PROPERTY_SELECTOR, FIELD_SELECTOR].join(' '));
            for (var i = 0; i < $argumentPropertyEditorLabels.length; i++)
                if (labelTextRegExp.test($argumentPropertyEditorLabels.text()))
                    return true;
            return false;
        },
        getStepActionArgumentProperty: function (labelTextRegExp) {
            var $argumentPropertyEditorProperties = select([STEP_INFO_PANEL_SELECTOR, ACTION_STEP_SELECTOR, OPTIONS_SELECTOR,
                    DIALOG_PROPERTY_SELECTOR].join(' ')),
                $currentLabel = null;

            for (var i = 0; i < $argumentPropertyEditorProperties.length; i++) {
                $currentLabel = select(FIELD_SELECTOR, $argumentPropertyEditorProperties.eq(i));
                if (labelTextRegExp.test($currentLabel.text()))
                    return $argumentPropertyEditorProperties.eq(i);
            }
        },
        getCurrentStepIndex: function () {
            var $selected = select(ITEM_SELECTED_SELECTOR);

            if (!$selected.length)
                $selected = select(ITEM_OPENED_SELECTOR);

            return $selected.eq(0).index();
        },
        getOpenedStepIndex: function () {
            return select(ITEM_OPENED_SELECTOR).length ? select(ITEM_OPENED_SELECTOR).eq(0).index() : null;
        },
        getLastItem: function () {
            return select(STEP_LIST_PANEL_SELECTOR).find(STEP_LIST_ITEM_SELECTOR + cssPostfix).last();
        },
        getSelectedItemIndexes: function () {
            var $items = select(STEP_LIST_PANEL_SELECTOR).find(STEP_LIST_ITEM_SELECTOR + cssPostfix),
                $selected = select(ITEM_SELECTED_SELECTOR),
                indexes = [];

            $.each($selected, function (i, item) {
                indexes.push($.inArray(item, $items));

            });
            return indexes;
        },
        getCurrentSelector: function () {
            var elementSelector = getElementSelectorEditor();

            return elementSelector.text();
        },
        getCurrentUploadStepFiles: function () {
            return select('.readonly_property .value').text();
        },
        getCurrentIFrameSelector: function () {
            var selectorEditor = getIFrameSelectorEditor();

            return selectorEditor ? selectorEditor.text() : null;
        },
        isResumePlaybackButtonEnabled: function () {
            return !popup.getPopupButtonSelector('Replay step').attr('disabled');
        },
        isSkipStepButtonEnabled: function () {
            return !popup.getPopupButtonSelector('Skip step').attr('disabled');
        },
        getResumePlaybackButtonSelector: function () {
            return popup.getPopupButtonSelector('Replay step');
        },
        getSkipStepButtonSelector: function () {
            return popup.getPopupButtonSelector('Skip step');
        },
        isStepInfoPanelShown: function () {
            return !!select(STEP_INFO_PANEL_SELECTOR).length;
        },
        isDeleteStepButtonVisible: function () {
            return isElementVisible(DELETE_STEP_BUTTON_AREA_SELECTOR);
        },
        isDeleteStepButtonEnabled: function () {
            var $button = select(DELETE_STEP_BUTTON_SELECTOR);

            return $button[0].className.indexOf('disabled') === -1;
        },
        getDeleteStepButton: function () {
            return select(DELETE_STEP_BUTTON_SELECTOR);
        },
        isStepFailed: function (index) {
            return stepsPanel.getStepListItem(index).is('.failed' + cssPostfix);
        },
        getStepActionArgumentErrorText: function (labelTextRegExp) {
            var $property = stepsPanel.getStepActionArgumentProperty(labelTextRegExp),
                $validationMessage = select(VALIDATION_SELECTOR, $property);

            if (!$validationMessage.is('.error-message' + cssPostfix))
                return '';

            return $validationMessage.text();
        },
        getItemSortableIcon: function (index) {
            return select('.sortable-icon').eq(index);
        },
        getItemSortableArea: function (index) {
            return select('.sortable-area').eq(index);
        }
    }
})();

var toolbar = (function () {
    var TOOLBAR_AREA_SELECTOR = '.toolbar-area',
        TOOLBAR_SELECTOR = '.drag-area',
        RECORD_HOVER_BUTTON_SELECTOR = '.hover',
        RECORD_WAIT_BUTTON_SELECTOR = '.wait',
        ADD_ASSERTIONS_BUTTON_SELECTOR = '.add-assertions',
        PLAYBACK_BUTTON_SELECTOR = '.playback',
        SAVE_TEST_BUTTON_SELECTOR = '.save',
        EXIT_RECORDING_BUTTON_SELECTOR = '.exit',
        DISABLE_BUTTON_AREA_CLASS = 'disable',
        HIDE_STEPS_BUTTON_AREA_SELECTOR = '.hide-steps-buttons-area',
        SHOW_STEPS_BUTTON_AREA_SELECTOR = '.show-steps-buttons-area';

    return {
        getDragArea: function () {
            return select(TOOLBAR_SELECTOR);
        },

        getToolbarArea: function () {
            return select(TOOLBAR_AREA_SELECTOR);
        },

        getRecordHoverActionSelector: function () {
            return select(RECORD_HOVER_BUTTON_SELECTOR);
        },

        getRecordWaitActionSelector: function () {
            return select(RECORD_WAIT_BUTTON_SELECTOR);
        },

        getAddAssertionsSelector: function () {
            return select(ADD_ASSERTIONS_BUTTON_SELECTOR);
        },

        getSaveTestButtonSelector: function () {
            return select(SAVE_TEST_BUTTON_SELECTOR);
        },

        isSaveTestButtonEnabled: function () {
            return !select(SAVE_TEST_BUTTON_SELECTOR).parent().hasClass(patchClassSelector('disable'));
        },

        isRunPlaybackButtonEnabled: function () {
            return !select(PLAYBACK_BUTTON_SELECTOR).parent().hasClass(patchClassSelector('disable'));
        },

        getExitRecordingButtonSelector: function () {
            return select(EXIT_RECORDING_BUTTON_SELECTOR);
        },

        getRunPlaybackSelector: function () {
            return select(PLAYBACK_BUTTON_SELECTOR);
        },

        isShown: function () {
            return !!(select(TOOLBAR_SELECTOR).length && select(TOOLBAR_SELECTOR).is(':visible'));
        },

        getHideStepsButton: function () {
            return select(HIDE_STEPS_BUTTON_AREA_SELECTOR);
        },

        getShowStepsButton: function () {
            return select(SHOW_STEPS_BUTTON_AREA_SELECTOR);
        },

        isExpandStepsButtonEnabled: function () {
            var $button = select(HIDE_STEPS_BUTTON_AREA_SELECTOR).length ? select(HIDE_STEPS_BUTTON_AREA_SELECTOR) : select(SHOW_STEPS_BUTTON_AREA_SELECTOR);
            return !$button.children().hasClass(patchClassSelector(DISABLE_BUTTON_AREA_CLASS));
        }
    }

})();

var popup = (function () {
    var POPUP_HEADER_SELECTOR = '.header',
        POPUP_TITLE_SELECTOR = POPUP_HEADER_SELECTOR + ' .title',
        CONFIRM_MESSAGE_CONTAINER_SELECTOR = '.confirm-message',
        BACK_TO_START_PAGE_POPUP_TEXT = 'return to the initial test page',
        EXIT_WITHOUT_SAVE_CHANGES_POPUP_TEXT = 'There are changes that can be saved after all incorrect steps are corrected. Do you want to quit losing all unsaved changes?',
        DIALOGS_EDITOR_SELECTOR = '.action-dialogs-editor',
        CHECKBOX_SELECTOR = '.checkbox',
        CHECKBOX_ICON_SELECTOR = '.checkbox-icon',
        FAILED_CLASS = 'failed',
        PRESSED_CLASS = 'pressed',
        SCROLLBAR_SELECTOR = '.scroll';

    return {
        getPopup: function () {
            return select('.' + POPUP_DIALOG_CLASS);
        },

        getPopupCloseButton: function () {
            return select('.' + POPUP_DIALOG_CLASS).find(patchClassSelector('.' + CLOSE_BUTTON_CLASS));
        },

        getPopupButtonSelector: function (text) {
            var $button = select('.' + POPUP_DIALOG_CLASS).find('button:containsExcludeChildren("' + text + '")');

            if ($button.length)
                return $button;


            return select('.' + POPUP_DIALOG_CLASS).find('button>span:containsExcludeChildren("' + text + '")');
        },

        getPopupHeader: function () {
            return select(POPUP_HEADER_SELECTOR);
        },

        getPopupTitle: function () {
            return select(POPUP_TITLE_SELECTOR).text();
        },

        isPopupShown: function () {
            return !!select('.' + POPUP_DIALOG_CLASS).length;
        },

        isPageErrorMessageAppeared: function () {
            return !!select('.' + POPUP_DIALOG_CLASS).find('.error-message' + cssPostfix).length;
        },

        getPageErrorMessageErrors: function () {
            return $.map(select('.' + POPUP_DIALOG_CLASS).find('li'), function (obj) {
                return obj.textContent;
            });
        },

        isAllStepsDeletionWarningVisible: function () {
            return !!select('.' + POPUP_DIALOG_CLASS).length && select(CONFIRM_MESSAGE_CONTAINER_SELECTOR).text().indexOf(BACK_TO_START_PAGE_POPUP_TEXT) >= 0;
        },

        getDialogsEditor: function () {
            return select(DIALOGS_EDITOR_SELECTOR);
        },

        hasAlertHandler: function () {
            var $alertCheckboxIcon = select(CHECKBOX_ICON_SELECTOR).first();
            return $alertCheckboxIcon.hasClass(patchClassSelector(PRESSED_CLASS));
        },

        isAlertHandlerFailed: function () {
            var $alertCheckbox = select(CHECKBOX_SELECTOR).first();
            return $alertCheckbox.hasClass(patchClassSelector(FAILED_CLASS));
        },

        getAlertHandlerCheckbox: function () {
            return select(CHECKBOX_ICON_SELECTOR).first();
        },

        exitWithoutSavingPopupVisible: function () {
            return !!select('.' + POPUP_DIALOG_CLASS).length && select(CONFIRM_MESSAGE_CONTAINER_SELECTOR).text().indexOf(EXIT_WITHOUT_SAVE_CHANGES_POPUP_TEXT) >= 0;
        },

        hasScroll: function () {
            var $popup = select('.' + POPUP_DIALOG_CLASS),
                $scrollbar = select(SCROLLBAR_SELECTOR, $popup);
            return  $popup.length && $scrollbar.length ? !!$scrollbar.height() : false;
        }
    };
})();

var testResults = (function () {
    var BUTTON_EXPAND_SELECTOR = '.dx-datagrid-expand',
        FAIL_INDICATOR_SELECTOR = 'tr p.fail',
        MASTER_DETAIL_CELL_SELECTOR = '.dx-master-detail-cell';

    return {
        getExpandButtonSelector: function () {
            return BUTTON_EXPAND_SELECTOR;
        },
        getFailIndicator: function () {
            return $(FAIL_INDICATOR_SELECTOR);
        },
        getErrorMessageText: function ($failIndicator) {
            var $indicator = $failIndicator && $failIndicator.length ? $failIndicator : $(FAIL_INDICATOR_SELECTOR);

            return $indicator.length ? $indicator.closest('tr').next().text() : '';
        },
        isFailedTestExpanded: function () {
            return !!$(MASTER_DETAIL_CELL_SELECTOR).length;
        }
    }
})();

var dialog = (function () {
    return {
        getCancelButton: function () {
            return $('button:containsExcludeChildren("Cancel")');
        },

        getSaveButton: function () {
            return $('button:containsExcludeChildren("Save")');
        },

        getYesButton: function () {
            return $('button:containsExcludeChildren("Yes")');
        },

        getNoButton: function () {
            return $('button:containsExcludeChildren("No")');
        }
    };
})();

var saveTestDialog = (function () {
    var ID_PREFIX = '989b5ac6-476f-4c86-a53b-37c9decad66b-',
        TEST_NAME_INPUT_ID = ID_PREFIX + 'test-name';

    return {
        getTestNameInputSelector: function () {
            return $('#' + TEST_NAME_INPUT_ID);
        }
    };
})();

var assertionsStepDialog = (function () {
    var ADD_ASSERTION_BUTTON_SELECTOR = '.add-assertion-button',
        ADD_ASSERTION_BUTTON_POPUP_ITEM_SELECTOR = '.add-assertion-button-popup-item',
        EQ_ICON_SELECTOR = '.eq-icon',
        NOT_EQ_ICON_SELECTOR = '.not-eq-icon',
        OK_ICON_SELECTOR = '.ok-icon',
        NOT_OK_ICON_SELECTOR = '.not-ok-icon',
        EQ_POPUP_ITEM_SELECTOR = ADD_ASSERTION_BUTTON_POPUP_ITEM_SELECTOR + ' ' + EQ_ICON_SELECTOR,
        NOT_EQ_POPUP_ITEM_SELECTOR = ADD_ASSERTION_BUTTON_POPUP_ITEM_SELECTOR + ' ' + NOT_EQ_ICON_SELECTOR,
        OK_POPUP_ITEM_SELECTOR = ADD_ASSERTION_BUTTON_POPUP_ITEM_SELECTOR + ' ' + OK_ICON_SELECTOR,
        NOT_OK_POPUP_ITEM_SELECTOR = ADD_ASSERTION_BUTTON_POPUP_ITEM_SELECTOR + ' ' + NOT_OK_ICON_SELECTOR,
        ASSERTION_SELECTOR = '.step .assertion',
        OPERATOR_AREA_SELECTOR = '.operator-area';

    return {
        getAddAssertionButton: function () {
            return select(ADD_ASSERTION_BUTTON_SELECTOR);
        },
        getEqPopupItem: function () {
            return select(EQ_POPUP_ITEM_SELECTOR);
        },
        getNotEqPopupItem: function () {
            return select(NOT_EQ_POPUP_ITEM_SELECTOR);
        },
        getOkPopupItem: function () {
            return select(OK_POPUP_ITEM_SELECTOR);
        },
        getNotOkPopupItem: function () {
            return select(NOT_OK_POPUP_ITEM_SELECTOR);
        },

        getEqIconSelector: function () {
            return patchClassSelector(EQ_ICON_SELECTOR);
        },
        getNotEqIconSelector: function () {
            return patchClassSelector(NOT_EQ_ICON_SELECTOR);
        },
        getOkIconSelector: function () {
            return patchClassSelector(OK_ICON_SELECTOR);
        },
        getNotOkIconSelector: function () {
            return patchClassSelector(NOT_OK_ICON_SELECTOR);
        },

        getAssertions: function () {
            return select(ASSERTION_SELECTOR);
        },

        getAddAssertionsButton: function () {
            return $('button:containsExcludeChildren("OK")');
        },

        getRemoveAssertionButton: function (index) {
            return select('.remove-assertion-button-area').find('button').eq(index);
        },

        getEditAssertionSelector: function (index) {
            return select(ASSERTION_SELECTOR).eq(index);
        },

        isAssertionFailed: function (index) {
            return select(OPERATOR_AREA_SELECTOR).eq(index).is('.failed' + cssPostfix);
        },

        isAssertionSuccessful: function (index) {
            return select(OPERATOR_AREA_SELECTOR).eq(index).is('.successful' + cssPostfix);
        }
    };
})();

var editAssertionDialog = (function () {
    var ACTUAL_VALUE_SELECTOR = '.argument-value',
        STATE_INDICATOR_SELECTOR = '.state-indicator',
        ARGUMENT_SELECTOR = '.argument',
        COPY_BUTTON_SELECTOR = '.copy-button',
        DIALOG_SELECTOR = '.edit-assertion-dialog';

    return {
        isDialogVisible: function () {
            return isElementVisible(DIALOG_SELECTOR);
        },

        getConfirmButton: function () {
            return select(DIALOG_SELECTOR).parent().find('button:containsExcludeChildren("OK")');
        },

        getActualCodeEditor: function () {
            return select(CODE_EDITOR_SELECTOR).eq(0);
        },

        getExpectedCodeEditor: function () {
            return select(CODE_EDITOR_SELECTOR).eq(1);
        },

        getActualValue: function () {
            return select(ACTUAL_VALUE_SELECTOR).eq(0).text();
        },

        getExpectedValue: function () {
            return select(ACTUAL_VALUE_SELECTOR).eq(1).text();
        },

        getCopyButton: function () {
            return select(COPY_BUTTON_SELECTOR);
        },

        isStateIndicatorVisible: function () {
            return isElementVisible(STATE_INDICATOR_SELECTOR);
        },

        isAssertionSuccessful: function () {
            var $stateIndicator = select(STATE_INDICATOR_SELECTOR);

            return $stateIndicator.text().toLowerCase() === 'passed';
        },

        isCopyButtonEnabled: function () {
            return isElementVisible(COPY_BUTTON_SELECTOR) && !select(COPY_BUTTON_SELECTOR).attr('disabled');
        },

        getMessageInput: function () {
            return select('.edit-assertion-dialog').find('input');
        },

        getOpenWizardButton: function (index) {
            return select(ARGUMENT_SELECTOR).eq(index || 0).find('button');
        }
    };
})();

var chooseElementSelectorDialog = (function () {
    var DIALOG_SELECTOR = '.choose-selector-dialog';

    return {
        isDialogVisible: function () {
            return isElementVisible(DIALOG_SELECTOR);
        },

        getElementSelector: function () {
            var dialog = select(DIALOG_SELECTOR)[0];

            return select(CODE_EDITOR_SELECTOR).filter(function () {
                return $.contains(dialog, this);
            });
        }
    };
})();

var chooseAssertionArgumentPropertyDialog = (function () {
    var RESULT_SELECTOR = '.result',
        TABS_PANEL_SELECTOR = '.tabs-panel',
        DIALOG_SELECTOR = 'choose-property-dialog';

    return {
        isDialogVisible: function () {
            return isElementVisible(DIALOG_SELECTOR);
        },

        getResult: function () {
            return select(RESULT_SELECTOR).text();
        },

        getSelectedPropertyName: function () {
            var propertyList = select('.property-list-items')[0],
                $selectedProperty = select('.selected').filter(function () {
                    return $(this).is(':visible') && $.contains(propertyList, this);
                }),
                $selectedPropertyName = $selectedProperty.length ? $selectedProperty.children().eq(0) : null;

            return $selectedPropertyName ? $selectedPropertyName.text() : null;
        },

        getTabByIndex: function (index) {
            var $tabPanel = select(TABS_PANEL_SELECTOR);

            return $tabPanel.children().eq(index);
        }
    }
})();

var setTestNameDialog = (function () {
    var TEST_NAME_INPUT_SELECTOR = '.test-name',
        VALIDATION_DIV_SELECTOR = '.validation';
    return {
        getTestNameInput: function () {
            return select(TEST_NAME_INPUT_SELECTOR);
        },

        hasError: function (errorText) {
            return select(VALIDATION_DIV_SELECTOR).text() === errorText;
        }
    }
})();

var tooltip = (function () {
    var TOOLTIP_SELECTOR = '.tooltip',
        CLOSE_BUTTON_SELECTOR = '.close-button',
        LAST_STEP_TEXT = 'last step',
        isTooltipVisible = function () {
            return isElementVisible(TOOLTIP_SELECTOR);
        },
        getToolTip = function () {
            return select(TOOLTIP_SELECTOR);
        },
        getToolTipText = function () {
            return getToolTip().text();
        },
        getLastTooltipText = function () {
            var toolTips = getToolTip();

            return $(toolTips[toolTips.length - 1]).text();
        };

    return{
        getCloseButton: function () {
            return getToolTip().find(patchClassSelector(CLOSE_BUTTON_SELECTOR));
        },

        getToolTip: getToolTip,

        getToolTipText: getToolTipText,

        getLastTooltipText: getLastTooltipText,

        isTooltipVisible: isTooltipVisible,

        isLastStepDeletionWarningVisible: function () {
            return isTooltipVisible() && getToolTipText().indexOf(LAST_STEP_TEXT) >= 0;
        }
    }
})();

var selectorSwitcher = (function () {
    var SELECTOR_SWITCHER_SELECTOR = '.selector-switcher',
        BAR_SELECTOR = '.bar',
        ITEMS_SELECTOR = '.items',
        SCROLL_BAR_SELECTOR = '.scroll';

    return{
        getBar: function (selectorSwitcherIndex) {
            return select(SELECTOR_SWITCHER_SELECTOR + ' ' + BAR_SELECTOR).eq(selectorSwitcherIndex || 0);
        },

        getScrollableContentContainer: function (selectorSwitcherIndex) {
            return select(SELECTOR_SWITCHER_SELECTOR + ' ' + ITEMS_SELECTOR).eq(selectorSwitcherIndex || 0);
        },

        getScrollBar: function (selectorSwitcherIndex) {
            return select(SELECTOR_SWITCHER_SELECTOR + ' ' + SCROLL_BAR_SELECTOR).eq(selectorSwitcherIndex || 0);
        }
    }
})();

function getCursor() {
    return window.Hammerhead__.ShadowUI.getRoot().find('*').filter(function () {
        return this.className.indexOf(CURSOR_CLASS) > -1;
    });
}

function hideCursor() {
    getCursor().css('display', 'none');
}

function showCursor() {
    getCursor().css('display', '');
}

function getCodeEditorSelector() {
    return select(CODE_EDITOR_SELECTOR);
}

function getSelectorEditorError() {
    return select('.error-message').text();
}

function isSvgBackgroundShown() {
    return select('.background').length === 1;
}

function isDivBackgroundShown() {
    return select('.modal-background').length === 1 && select('.modal-background').is(':visible');
}

function isLoadingIconShown() {
    var $icon = select('.loading-icon');

    return $icon.length === 1 && (window.getComputedStyle($icon[0]).visibility !== 'hidden');
}

function selectUIElementByText(text) {
    return $(':containsExcludeChildren("' + text + '")');
}

function getRoot() {
    return window.Hammerhead__.ShadowUI.getRoot();
}

function getPageRoot() {
    return window.Hammerhead.ShadowUI.getRoot();
}

var recorderHelper = {
    ASSERTION_SHORTCUT: 'ctrl+r',
    HOVER_SHORTCUT: 'ctrl+space',
    WAIT_SHORTCUT: 'ctrl+q',
    SCREENSHOT_SHORTCUT: 'ctrl+m',
    PLAYBACK_SHORTCUT: 'ctrl+d',
    SAVE_TEST_SHORTCUT: 'ctrl+s',
    END_TEST_SHORTCUT: 'ctrl+e',
    COMPLETE_ACTION_SHORTCUT: 'ctrl+g',

    CSS_LINK_CLASS: 'ui-stylesheet',

    select: select,
    isElementFrameShown: isElementFrameShown,
    getElementPickerFrame: getElementPickerFrame,
    isCursorShown: isCursorShown,
    getSelectorEditorError: getSelectorEditorError,
    getCompleteClickIcon: getCompleteClickIcon,
    getCompleteTypingIcon: getCompleteTypingIcon,
    addActionDialog: addActionDialog,
    stepsPanel: stepsPanel,
    toolbar: toolbar,
    tooltip: tooltip,
    dialog: dialog,
    popup: popup,
    testResults: testResults,
    getPopupButtonSelector: popup.getPopupButtonSelector,
    isPopupShown: popup.isPopupShown,
    assertionsStepDialog: assertionsStepDialog,
    editAssertionDialog: editAssertionDialog,
    chooseElementSelectorDialog: chooseElementSelectorDialog,
    chooseAssertionArgumentPropertyDialog: chooseAssertionArgumentPropertyDialog,
    setTestNameDialog: setTestNameDialog,
    hideCursor: hideCursor,
    showCursor: showCursor,
    getCodeEditorSelector: getCodeEditorSelector,
    selectUIElementByText: selectUIElementByText,
    saveTestDialog: saveTestDialog,
    selectorSwitcher: selectorSwitcher,
    getRoot: getRoot,
    getPageRoot: getPageRoot,

    isSvgBackgroundShown: isSvgBackgroundShown,
    isDivBackgroundShown: isDivBackgroundShown,
    isLoadingIconShown: isLoadingIconShown,

    isIE: isIE,
    isMozilla: isMozilla
};