/**
 * Module dependencies.
 */

var express = require('express'),
    routes = require('./routes'),
    http = require('http'),
    fs = require('fs'),
    rimraf = require('rimraf'),
    path = require('path');

//Const
var SCREENSHOT_EXTENSION = 'png';

function getFilesCount (dir) {
    var results = 0,
        list = fs.readdirSync(dir),
        screenshotRegExp = new RegExp('\\.' + SCREENSHOT_EXTENSION + '$');

    list.forEach(function (file) {
        file = dir + '/' + file;
        var stat = fs.statSync(file);
        if (stat && stat.isDirectory())
            results += getFilesCount(file);
        else if (screenshotRegExp.test(file))
            results++;
    });
    return results;
}

function createApp (port) {
    var app = express();

    app.set('port', port);
    app.set('views', __dirname + '/views');
    app.set('view engine', 'ejs');
    app.use(express.favicon());
    app.use(express.bodyParser());
    app.use(app.router);
    app.use(express.static(path.join(__dirname, 'public')));

    app.get('/index', routes.index);

    app.get('/jquery', function (req, res) {
        res.set('Content-Type', 'application/javascript');
        res.send(fs.readFileSync(path.join(__dirname, '../../../../../vendor/jquery-1.7.2.js')).toString());
    });
    app.get('/jquery/:delay', function (req, res) {
        var delay = req.params.delay || 0;

        res.set('Content-Type', 'application/javascript');

        setTimeout(function () {
            res.send(fs.readFileSync(path.join(__dirname, '../../../../../vendor/jquery-1.7.2.js')).toString());
        }, delay);
    });
    app.get('/', function (req, res) {
        res.set('Location', '/index');
        res.send(302);
    });

    app.get('/xhr-request/:delay', function (req, res) {
        var delay = req.params.delay || 0;

        setTimeout(function () {
            res.send(req.url);
        }, delay);
    });

    app.get('/download', function (req, res) {
        var file = __dirname + '/package.json';
        res.download(file);
    });

    app.get('/download/xml', function (req, res) {
        var file = __dirname + '/reports.xml';
        res.download(file);
    });

    app.get('/404', function (req, res) {
        if (req.query.text)
            res.write(req.query.text);

        res.statusCode = 404;
        res.end();
    });

    app.get('/screenshot-count', function (req, res) {
        res.set('Content-Type', 'application/javascript');

        var TESTED_TESTCAFE_REPORTS_DIR = path.join(__dirname, './../../../../.././reports');

        fs.exists(TESTED_TESTCAFE_REPORTS_DIR, function (exists) {
            if (exists) {
                var resultDirs = fs.readdirSync(TESTED_TESTCAFE_REPORTS_DIR).filter(function (file) {
                    return fs.statSync(path.join(TESTED_TESTCAFE_REPORTS_DIR, file)).isDirectory();
                });
                var hasReportFile = false;
                var hasScreenshotDir = false;
                var screenshotCount = 0;

                if (resultDirs.length === 1) {
                    var resultPath = path.join(TESTED_TESTCAFE_REPORTS_DIR, resultDirs[0]);
                    var resultFiles = fs.readdirSync(resultPath);

                    resultFiles.forEach(function (file) {
                        if (file === 'report.json')
                            hasReportFile = true;
                        else if (file === 'screenshots' && fs.statSync(resultPath + '/' + file).isDirectory())
                            hasScreenshotDir = true;
                    });

                    screenshotCount = getFilesCount(path.join(TESTED_TESTCAFE_REPORTS_DIR, resultDirs[0]));
                }

                rimraf(TESTED_TESTCAFE_REPORTS_DIR, function () {
                    res.send('cb({reportSaved: ' + hasReportFile + ', screenshotDirCreated: ' + hasScreenshotDir +
                             ', screenshotCount: ' + screenshotCount + '})');
                });
            }
            else
                res.send('cb({reportSaved: false, screenshotDirCreated: false, screenshotCount: 0})');
        });
    });

    app.get('/remove-reports', function (req, res) {
        res.set('Content-Type', 'application/javascript');

        var TESTED_TESTCAFE_REPORTS_DIR = path.join(__dirname, './../../../../.././reports');

        fs.exists(TESTED_TESTCAFE_REPORTS_DIR, function (exists) {
            if (exists) {
                rimraf(TESTED_TESTCAFE_REPORTS_DIR, function () {
                    res.send();
                });
            }
            else
                res.send();
        });
    });

    app.get('/form-confirm', function (req, res) {
        setTimeout(function () {
            res.send('<html><body><script>confirm("confirm form server");</script><div id="result">OK</div></body></html>');
        }, 300)
    });

    app.get('/*', routes.tests);

    http.createServer(app).listen(app.get('port'), function () {
        console.log('Express server listening on port ' + app.get('port'));
    });

}

exports.run = function (callback) {
    createApp(3000);
    createApp(3001);
    callback();
};