(function () {

    var Path = require('path'),
        Fs = require('fs');

    function isDir(path) {
        return Fs.statSync(path).isDirectory();
    }

    var REPLACE_DIRS = ['./client', './hammerhead/client', './hammerhead/shared', './lib/shared'],
        EXCLUDED_DIRS = [],//'./client/ui_recorder'],
        REPLACE_FILES = ['./Gulpfile.js', './vendor/mods.js', './vendor/jquery-1.7.2.js', './control_panel/client/init.js'],
        REPLACE_POSTFIX = '_',
        REPLACE_TARGETS = [
            'TestCafeClient',
            'HammerheadClient',
            'initTestCafeClientCore',
            'initTestCafeRecorder',
            'initTestCafeRecorderUI',
            'initTestCafeUI',
            '__set\\$',
            '__get\\$',
            '__call\\$',
            '__proc\\$Script',
            'window\\.Hammerhead',
            'Mods',
            '__w\\$undef_'
        ];

    var testCafeDir = '',
        eventSandboxPath = '',
        iframeSandboxPath = '',
        messageSandboxPath = '',
        xhrSandboxPath = '',
        domAccessorWrappersPath = '';

    var filesToCommonReplace = [];

    function fillFilesToCommonReplace() {
        var i = 0;

        function readDir(path) {
            for (var j = 0; j < EXCLUDED_DIRS.length; j++) {
                if (path === Path.join(testCafeDir, EXCLUDED_DIRS[j]))
                    return;
            }

            Fs.readdirSync(path).forEach(function (entry) {
                var subpath = Path.join(path, entry);

                if (isDir(subpath))
                    readDir(subpath);
                else if (subpath.indexOf('.png') < 0)
                    filesToCommonReplace.push(subpath);
            });
        }

        for (i = 0; i < REPLACE_DIRS.length; i++) {
            readDir(Path.join(testCafeDir, REPLACE_DIRS[i]));
        }

        for (i = 0; i < REPLACE_FILES.length; i++) {
            filesToCommonReplace.push(Path.join(testCafeDir, REPLACE_FILES[i]));
        }
    }

    function replaceInFile(filepath, searchValue, newValue, strict, once) {
        var text = Fs.readFileSync(filepath).toString();

        if (strict && !new RegExp(searchValue).test(text))
            throw 'Can not find value to replace: filepath: ' + filepath + '; searchValue: ' + searchValue;

        text = text.replace(new RegExp(searchValue, once ? '' : 'g'), newValue);

        Fs.writeFileSync(filepath, text);
    }

    function replaceInFileR(filepath, searchRegExp, fn) {
        var text = Fs.readFileSync(filepath).toString();

        if (!searchRegExp.test(text))
            throw 'Can not find value to replace';

        text = text.replace(searchRegExp, fn);

        Fs.writeFileSync(filepath, text);
    }

    function replaceCommonThings() {
        console.log('TestCafePatcher: 1.Replacing common things...');
        for (var i = 0; i < filesToCommonReplace.length; i++) {
            for (var j = 0; j < REPLACE_TARGETS.length; j++) {
                replaceInFile(filesToCommonReplace[i], REPLACE_TARGETS[j], REPLACE_TARGETS[j].replace('\\', '') + REPLACE_POSTFIX);
            }
            replaceInFile(filesToCommonReplace[i], 'window.Hammerhead' + REPLACE_POSTFIX + 'Client', 'window.HammerheadClient');
        }
    }

    function replaceDomSandboxThings() {
        console.log('TestCafePatcher: 2.Replacing dom-sandbox things...');
        var DOM_SANDBOX_PATH = Path.join(testCafeDir, './hammerhead/client/sandboxes/dom.js'),
            methods = [];

        replaceInFile(DOM_SANDBOX_PATH, '(el\\.\\S*)(\\s=\\s[^;]*;)', function (str, p1, p2) {
            methods.push(p1.replace('el.', ''));

            return '(function(){' + str + ' ' + p1 + REPLACE_POSTFIX + p2 + '})();';
        });

        replaceInFile(DOM_SANDBOX_PATH, '(window\\.Element\\.prototype\\.\\S*)(\\s=\\s[^;]*;)', function (str, p1, p2) {
            methods.push(p1.replace('window.Element.prototype.', ''));

            return ' (function(){' + str + p1 + REPLACE_POSTFIX + p2 + '})();';
        });

        replaceInFile(DOM_SANDBOX_PATH, "IFRAME_DOM_SANDBOXES_STORE = 'dom_sandboxes_store_5d9138e9'", "IFRAME_DOM_SANDBOXES_STORE = 'dom_sandboxes_store_5d9138e9'" + REPLACE_POSTFIX);

        for (var i = 0; i < filesToCommonReplace.length; i++) {
            for (var j = 0; j < methods.length; j++) {
                replaceInFile(filesToCommonReplace[i], '([^\\s)(!\'\"]+)\\.' + '(' + methods[j] + ')\\(([^\\)]*)',
                    function (str, owner, meth, args) {
                        return '(' + owner + '.' + meth + '_ || ' + owner + '.' + meth + ').call(' + owner + (args.length ? '\, ' + args : '');
                    });
            }
        }
    }

    function replaceEventSandboxThings() {
        console.log('TestCafePatcher: 3.Replacing event sandbox things...');
        var methods = [];

        replaceInFile(eventSandboxPath, 'function overrideElementOrHTMLElementMethod\\(methodName, overridedMethod\\) {', 'function overrideElementOrHTMLElementMethod(methodName, overridedMethod, force) {', true);
        replaceInFile(eventSandboxPath, 'if \\(window.Element && methodName in window.Element.prototype\\)', 'if (window.Element && (methodName in window.Element.prototype || force))', true);
        replaceInFile(eventSandboxPath, 'else if \\(window.HTMLElement && methodName in window.HTMLElement.prototype\\)', 'else if (window.HTMLElement && (methodName in window.HTMLElement.prototype || force))', true);
        replaceInFile(eventSandboxPath, 'if \\(window.Document && methodName in window.Document.prototype\\)', 'if (window.Document && (methodName in window.Document.prototype || force))', true);

        replaceInFile(eventSandboxPath, '(overrideElementOrHTMLElementMethod\\(\')([^,]+)(\',\\s[^\\)]+)(\\);)', function (str, p1, meth, p2, p3) {
            methods.push(meth);

            return '(function(){' + str + ' ' + p1 + meth + REPLACE_POSTFIX + p2 + ', true' + p3 + '})();';
        }, true);

        replaceInFile(eventSandboxPath, '(el\\.\\S*)(\\s=\\s[^f][^;]*;)', function (str, p1, p2) {
            methods.push(p1.replace('el.', ''));

            return '(function(){' + str + ' ' + p1 + REPLACE_POSTFIX + p2 + '})();';
        }, true);

        for (var i = 0; i < filesToCommonReplace.length; i++) {
            for (var j = 0; j < methods.length; j++) {
                replaceInFile(filesToCommonReplace[i], '([^\\s)(!\'\"]+)\\.' + '(' + methods[j] + ')\\(([^\\)]*)',
                    function (str, owner, meth, args) {
                        return '(' + owner + '.' + meth + '_ || ' + owner + '.' + meth + ').call(' + owner + (args.length ? '\, ' + args : '');
                    });
            }
        }
    }

    function overrideConsts() {
        console.log('TestCafePatcher: 4.Overriding consts...');
        var _FLAG_REG_EXP = /(\S+_FLAG\s=\s[^,;]+)([,;])/g,
            _PROP_REG_EXP = /(\S+_PROP\s=\s[^,;]+)([,;])/g;

        replaceInFileR(eventSandboxPath, _FLAG_REG_EXP, function (str, r1, r2) {
            return r1 + " + '" + REPLACE_POSTFIX + "'" + r2;
        });
        replaceInFileR(eventSandboxPath, _PROP_REG_EXP, function (str, r1, r2) {
            return r1 + " + '" + REPLACE_POSTFIX + "'" + r2;
        });
        replaceInFile(iframeSandboxPath, "IFRAME_WINDOW_INITED = 'hh_iwi_5d9138e9", "IFRAME_WINDOW_INITED = 'hh_iwi_5d9138e9" + REPLACE_POSTFIX, true);

        replaceInFile(messageSandboxPath, "USER: 'qWip2'", "USER: 'qWip2_'", true);
        replaceInFile(messageSandboxPath, "SERVICE: '5Gtrb'", "SERVICE: '5Gtrb_'", true);

        var HAMMERHEAD_SHARED_CONSTS_PATH = Path.join(testCafeDir, './hammerhead/shared/const.js'),
            TC_SHARED_CONSTS_PATH = Path.join(testCafeDir, './lib/shared/const.js'),
            HAMMERHEAD_SHARED_URL_UTIL_PATH = Path.join(testCafeDir, './hammerhead/shared/url_util.js'),
            SHARED_CONSTS_REGEXP = /(Const.\S+\s=[^;]+)(;)/g;

        replaceInFileR(HAMMERHEAD_SHARED_CONSTS_PATH, SHARED_CONSTS_REGEXP, function (str, r1, r2) {
            return r1 + " + '" + REPLACE_POSTFIX + "'" + r2;
        });

        replaceInFileR(TC_SHARED_CONSTS_PATH, SHARED_CONSTS_REGEXP, function (str, r1, r2) {
            return r1 + " + '" + REPLACE_POSTFIX + "'" + r2;
        });

        replaceInFile(HAMMERHEAD_SHARED_URL_UTIL_PATH, "UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY = '7929ba6d39aa4465';", "UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY = '7929ba6d39aa4465_';", true);
        replaceInFile(HAMMERHEAD_SHARED_URL_UTIL_PATH, "UrlUtil.REQUEST_DESCRIPTOR_VALUES_SEPARATOR = '\\|';", "UrlUtil.REQUEST_DESCRIPTOR_VALUES_SEPARATOR = '{';", true);
        replaceInFile(domAccessorWrappersPath, "LOCATION_WRAPPER = 'location_1b082a6cec", "LOCATION_WRAPPER = 'location_1b082a6cec" + REPLACE_POSTFIX, true);
        replaceInFile(domAccessorWrappersPath, "ORIGINAL_WINDOW_ON_ERROR_HANDLER_KEY = 'onerror_23ad9304921s", "ORIGINAL_WINDOW_ON_ERROR_HANDLER_KEY = 'onerror_23ad9304921s" + REPLACE_POSTFIX, true);
    }

    function fixHTTPRequest() {
        console.log('TestCafePatcher: 5.Fixing http request...');
        var REGEXP = /(window\.XMLHttpRequest)(\s=\sfunction\s\(\)\s{[^}]+};)/m;

        replaceInFileR(xhrSandboxPath, REGEXP, function (str, r1, r2) {
            return str + '\n\r    ' + r1 + REPLACE_POSTFIX + r2;
        });

        var VENDOR_PATH = Path.join(testCafeDir, './vendor/');

        Fs.readdirSync(VENDOR_PATH).forEach(function (entry) {
            var subpath = Path.join(VENDOR_PATH, entry);

            if (!isDir(subpath))
                replaceInFile(subpath, 'window.XMLHttpRequest', ('(window.XMLHttpRequest' + REPLACE_POSTFIX + ' || window.XMLHttpRequest)'));
        });
    }

    function fixShadowUI() {
        console.log('TestCafePatcher: 6.Fixing shadowUI...');
        var SHADOW_UI_PATH = Path.join(testCafeDir, './hammerhead/client/sandboxes/shadow_ui.js'),
            HOOKED_FUNCTIONS = [
                'elementFromPoint',
                'getElementById',
                'getElementsByClassName',
                'getElementsByName',
                'getElementsByTagName',
                'querySelector',
                'querySelectorAll',
                'childNodes',
                'children',
                'firstChild',
                'firstElementChild',
                'lastChild',
                'lastElementChild'
            ];

        replaceInFile(SHADOW_UI_PATH, 'element\\[propName\\] = function', 'element[propName] = element[propName + "' + REPLACE_POSTFIX + '"] = function');

        replaceInFile(SHADOW_UI_PATH, 'document.elementFromPoint = function', 'document.elementFromPoint = document.elementFromPoint' + REPLACE_POSTFIX + ' = function');
        replaceInFile(SHADOW_UI_PATH, 'document.getElementById = function', 'document.getElementById = document.getElementById' + REPLACE_POSTFIX + ' = function');
        replaceInFile(SHADOW_UI_PATH, 'document.getElementsByClassName = function', 'document.getElementsByClassName = document.getElementsByClassName' + REPLACE_POSTFIX + ' = function');
        replaceInFile(SHADOW_UI_PATH, 'document.getElementsByName = function', 'document.getElementsByName = document.getElementsByName' + REPLACE_POSTFIX + ' = function');
        replaceInFile(SHADOW_UI_PATH, 'document.getElementsByTagName = function', 'document.getElementsByTagName = document.getElementsByTagName' + REPLACE_POSTFIX + ' = function');
        replaceInFile(SHADOW_UI_PATH, 'document.querySelector = function', 'document.querySelector = document.querySelector' + REPLACE_POSTFIX + ' = function');
        replaceInFile(SHADOW_UI_PATH, 'document.querySelectorAll = function', 'document.querySelectorAll = document.querySelectorAll' + REPLACE_POSTFIX + ' = function');

        replaceInFile(SHADOW_UI_PATH, 'el.getElementsByClassName = function', 'el.getElementsByClassName = el.getElementsByClassName' + REPLACE_POSTFIX + ' = function');
        replaceInFile(SHADOW_UI_PATH, 'el.getElementsByTagName = function', 'el.getElementsByTagName = el.getElementsByTagName' + REPLACE_POSTFIX + ' = function');
        replaceInFile(SHADOW_UI_PATH, 'el.querySelector = function', 'el.querySelector = el.querySelector' + REPLACE_POSTFIX + ' = function');
        replaceInFile(SHADOW_UI_PATH, 'el.querySelectorAll = function', 'el.querySelectorAll = el.querySelectorAll' + REPLACE_POSTFIX + ' = function');

        for (var i = 0; i < filesToCommonReplace.length; i++) {
            for (var j = 0; j < HOOKED_FUNCTIONS.length; j++) {
                replaceInFile(filesToCommonReplace[i], '([^\\s)(!\'\"]+)\\.' + '(' + HOOKED_FUNCTIONS[j] + ')\\(([^\\)]*)',
                    function (str, owner, meth, args) {
                        return '(' + owner + '.' + meth + '_ || ' + owner + '.' + meth + ').call(' + owner + (args.length ? '\, ' + args : '');
                    });

                replaceInFile(filesToCommonReplace[i], '([^\\s)(!\'\"]+)\\.' + '(' + HOOKED_FUNCTIONS[j] + ')(\\)?\\.call)',
                    function (str, owner, meth, args) {
                        return '(' + owner + '.' + meth + '_ || ' + owner + '.' + meth + ')' + args;
                    });
            }
        }
    }

    function fixCrossDomainPort() {
        console.log('TestCafePatcher: 7.Fixing cross domaint port...');
        replaceInFile(Path.join(testCafeDir, './lib/config/index.js'), '5555', '5556', true);
    }

    var steps = [
        {
            num: -1,
            name: 'fill files to common replace',
            action: fillFilesToCommonReplace
        },
        {
            num: 0,
            name: 'HACK: fix removeChild issue in sandboxed_jquery',
            action: function () {
                console.log('0. HACK: fixing removeChild issue in sandboxed_jquery...');

                var SANDBOXED_JQUERY_FILE = Path.join(testCafeDir, './hammerhead/client/sandboxed_jquery.js'),
                    JQUERY_FILE = Path.join(testCafeDir, './vendor/jquery-1.7.2.js');

                replaceInFile(SANDBOXED_JQUERY_FILE, '__get\\$\\(tbody\\, j\\)\\.parentNode\\.removeChild\\(__get\\$\\(tbody\\, j\\)\\)', 'var el = __get$(tbody, j); el.parentNode.removeChild(el);', true);
                replaceInFile(JQUERY_FILE, 'tbody\\[ j \\]\\.parentNode\\.removeChild\\( tbody\\[ j \\] \\);', 'var el = tbody[j]; el.parentNode.removeChild( el[j] );', true);


                //__get$_(tbody, j)(.parentNode.removeChild_ || .parentNode.removeChild).call(.parentNode, __get$_(tbody, j));
                //replaceInFile(SANDBOXED_JQUERY_FILE, '__get\\$_\\(tbody, j\\)\\(\\.parentNode\\.removeChild_ \\|\\| \\.parentNode\\.removeChild\\)\\.call\\(\\.parentNode, __get\\$_\\(tbody, j\\)\\);',
                //'(__get$_(tbody, j).parentNode.removeChild_ || __get$_(tbody, j).parentNode.removeChild).call(__get$_(tbody, j).parentNode, __get$_(tbody, j));', true);

                /*replaceInFile(JQUERY_FILE, '__get\\$_\\(tbody, j\\)\\(\\.parentNode\\.removeChild_ \\|\\| \\.parentNode\\.removeChild\\)\\.call\\(\\.parentNode, __get\\$_\\(tbody, j\\)\\);',
                 '(__get$_(tbody, j).parentNode.removeChild_ || __get$_(tbody, j).parentNode.removeChild).call(__get$_(tbody, j).parentNode, __get$_(tbody, j));', true);*/
            }
        },
        {
            num: 1,
            name: 'patch hammerhead to prevent stop test on uncaught page errors',
            action: function () {
                var DOM_SANDBOX_PATH = Path.join(testCafeDir, './hammerhead/client/sandboxes/dom.js');
                replaceInFile(DOM_SANDBOX_PATH, 'window.onerror = function \\(msg, url, line, col, errObj\\) \\{', 'window.onerror___ = function (msg, url, line, col, errObj) {', true);

                replaceInFile(domAccessorWrappersPath, 'onerror: \\{', 'onerror_: {', true);
            }
        },
        {
            num: 2,
            name: 'replace common things',
            action: replaceCommonThings
        },
        {
            num: 3,
            name: 'replace dom sandbox overridden functions',
            action: replaceDomSandboxThings
        },
        {
            num: 4,
            name: 'replace event sandbox overridden functions',
            action: replaceEventSandboxThings
        },
        {
            num: 5,
            name: 'override consts',
            action: overrideConsts
        },
        {
            num: 6,
            name: 'fix window.XMLHTTPRequest',
            action: fixHTTPRequest
        },
        {
            num: 7,
            name: 'fix shadowUI',
            action: fixShadowUI
        },
        {
            num: 8,
            name: 'fix cross-domain port',
            action: fixCrossDomainPort
        },
        {
            num: 9,
            name: 'fix message sandbox',
            action: function () {
                console.log('8.Fix message sandbox');
                replaceInFile(messageSandboxPath, 'return originListener.call\\(window, resultEvt\\);', 'return originListener.call(window, e);', true);
                replaceInFile(messageSandboxPath, "RECEIVE_MSG_FN = 'tc_rmf_375fb9e7'", "RECEIVE_MSG_FN = 'tc_rmf_375fb9e7_'", true);
            }
        },
        {
            num: 10,
            name: 'fix native dialogs',
            action: function () {
                console.log('9.Fixing native dialogs...');

                var IFRAME_TEST_RUN_TEMPLATE_PATH = Path.join(testCafeDir, './client/task_templates/iframe_test_run.jstmpl'),
                    NATIVE_DIALOG_FILE = Path.join(testCafeDir, './client/core/test_runner/api/native_dialogs.js');

                var nativeDialogsProcessing = '\n\r    window.alert__=function (){};window.alert=function(){window.alert__.apply(window, arguments);};\n\r    ' +
                    'window.confirm__=function (){};window.confirm=function(){window.confirm__.apply(window, arguments);};\n\r    ' +
                    'window.prompt__=function (){};window.prompt=function(){window.prompt__.apply(window, arguments);};';

                replaceInFile(IFRAME_TEST_RUN_TEMPLATE_PATH, '\\(function \\(\\) {', '(function () {' + nativeDialogsProcessing, true, true);
                replaceInFile(NATIVE_DIALOG_FILE, 'window\\.alert', 'window.alert__', true);
                replaceInFile(NATIVE_DIALOG_FILE, 'window\\.confirm', 'window.confirm__', true);
                replaceInFile(NATIVE_DIALOG_FILE, 'window\\.prompt', 'window.prompt__', true);
            }
        },

        {
            num: 11,
            name: 'patch gulp file',
            action: function () {
                console.log('10.Path gulp file');

                var gulpFile = Path.join(testCafeDir, './Gulpfile.js');

                replaceInFile(gulpFile, "if \\(isFarmTask\\(\\) \\|\\| env\\._\\.indexOf\\('Distributive'\\) > -1\\)", 'if (isFarmTask())', true);
                replaceInFile(gulpFile, "HgPrecommitHook.setup\\(\\)", '"1"', true);
            }
        }

    ];

    exports.patch = function (dir, callback) {
        testCafeDir = dir;

        eventSandboxPath = Path.join(testCafeDir, './hammerhead/client/sandboxes/event.js');
        iframeSandboxPath = Path.join(testCafeDir, './hammerhead/client/sandboxes/iframe.js');
        messageSandboxPath = Path.join(testCafeDir, './hammerhead/client/sandboxes/message.js');
        xhrSandboxPath = Path.join(testCafeDir, './hammerhead/client/sandboxes/xhr.js');
        domAccessorWrappersPath = Path.join(testCafeDir, './hammerhead/client/sandboxes/dom_accessor_wrappers.js');

        function runStep(num, stepCallback) {
            var step = steps[num];

            if (typeof step.action === 'function') {
                step.action();
            }

            setTimeout(stepCallback, 0);
        }

        function runStepRec(num, recCallback) {
            if (num === steps.length)
                recCallback();
            else
                runStep(num, function () {
                    runStepRec(num + 1, recCallback);
                });
        }

        runStepRec(0, callback);
    };
})();