var url = require('url');

function AnalysisUtil() {
    this._hammerhead = null;
    this._requestDescriptorQueryKey = null;
}

AnalysisUtil.prototype.SERVICE_URLS_REGEXP = /\/(hammerhead.js|task.js|tccservicemsg)$/;

AnalysisUtil.prototype.init = function(hammerhead) {
    this._hammerhead = hammerhead;
    this._requestDescriptorQueryKey = Object.keys(url.parse(this.getProxyUrl('http://a.ru/'), true).query)[0];
};

AnalysisUtil.prototype.isServiceUrl = function(url) {
    return this.SERVICE_URLS_REGEXP.test(url);
};

AnalysisUtil.prototype.isProxyUrl = function(url) {
    return url.indexOf(this._requestDescriptorQueryKey) !== -1;
};

AnalysisUtil.prototype.getOriginUrl = function(proxyUrl) {
    try {
        return this._hammerhead.getOriginUrl(proxyUrl);
    } catch (e) {
        return "";
    }
};

AnalysisUtil.prototype.getProxyUrl = function(url) {
    try {
        return this._hammerhead.getProxyUrl(url);
    } catch (e) {
        return "";
    }
};

AnalysisUtil.prototype.testSimilarity = function(str1, str2, percent) {
    var dis = this._computeDistance(str1, str2),
        maxLen = Math.max(str2.length, str1.length);

    return (!maxLen ? 1 : 1 - dis / maxLen) >= percent;
};

AnalysisUtil.prototype.calcMedian = function(array) {
    var arrMedian = array.length % 2 ? Math.floor(array.length / 2) : Math.round(array.length / 2);

    array.sort(function(a, b) { return a - b; });

    return array.length % 2 ? array[arrMedian] : (array[arrMedian] + array[arrMedian - 1]) / 2;
};


AnalysisUtil.prototype._computeDistance = function(str1, str2) {
    var str1Len = str1.length,
        str2Len = str2.length,
        distance = [],
        cost = 0,
        i, j;

    if (str1Len === 0 || str2Len === 0)
        return str1Len || str2Len;

    for (i = 0; i <= str1Len; i++) {
        distance.push([]);

        if (i)
            distance[i][0] = i;
        else
            for (j = 0; j <= str2Len; distance[0][j] = j++);
    }

    for (i = 1; i <= str1Len; i++) {
        for (j = 1; j <= str2Len; j++) {
            cost = str2.charAt(j - 1) === str1.charAt(i - 1) ? 0 : 1;
            distance[i][j] = Math.min(Math.min(distance[i - 1][j] + 1, distance[i][j - 1] + 1), distance[i - 1][j - 1] + cost);
        }
    }

    return distance[str1Len][str2Len];
};


module.exports = new AnalysisUtil();
