var http = require('http'),
    ejs = require('ejs'),
    fs = require('fs'),
    auth = require('http-auth'),
    urlLib = require('url'),
    healthMonitor = require('./health_monitor');

var EJS_TEMPLATE = fs.readFileSync(__dirname + '/index.ejs', 'utf8'),
    CONTROL_PANEL_PORT = 1350,
    HAMMERHEAD_PORT1 = 1351,
    HAMMERHEAD_PORT2 = 1352;

var done = false,
    basic = auth.basic({
        realm: "Private area",
        file: __dirname + "/htpasswd"
    }),
    monitorConfig = {
        port1: HAMMERHEAD_PORT1,
        port2: HAMMERHEAD_PORT2,
        startUrl: 'http://localhost:' + CONTROL_PANEL_PORT,
        runInspector: false,
        buildInfo: ''
    };

create(CONTROL_PANEL_PORT);

function searchSiteIndexByUrl(data, url) {
    var index = 0;

    if (data && url) {
        for (var i = 0; i < data.length; i++) {
            if (data[i].url === url)
                return i;
        }
    }

    return index;
}

function history(url, res) {
    res.writeHead(200, {'Content-Type': 'text/html'});

    var parsedUrl = urlLib.parse(url, true),
        timestamp = +parsedUrl.query.timestamp,
        findDataFunction = timestamp ? 'findDataByTimestamp' : 'findLastData',
        args = [];

    if(timestamp)
        args.push(timestamp);

    args.push(function(doc) {
        healthMonitor.db.getRunList(null, function(list) {
            res.end(ejs.render(EJS_TEMPLATE, {
                result: doc,
                runList: list,
                currentSiteIndex: +parsedUrl.query.result || searchSiteIndexByUrl(doc && doc.data, parsedUrl.query.url),
                path: '/history',
                filename: __dirname + '/index.ejs'
            }));
        });
    });

    healthMonitor.db[findDataFunction].apply(healthMonitor.db, args);
}

function create(port) {
    http.createServer(basic, function (req, res) {
        if (!done)
            return res.end();

        if (req.url === '/') {
            statistics(res);
        } else if (/^\/history($|\?)/.test(req.url)) {
            history(req.url, res);
        } else if (req.url ===  '/options') {
            options(res);
        } else if (/^\/xhr\//.test(req.url)) {
            getXhr(req, res);
        } else {
            try {
                if (/\.js$/.test(req.url))
                    res.setHeader('content-type', 'text/javascript');
                else if (/\.css$/.test(req.url))
                    res.setHeader('content-type', 'text/css');
                else if (/\.svg$/.test(req.url))
                    res.setHeader('content-type', 'image/svg+xml');

                res.end(fs.readFileSync(__dirname + req.url, 'utf8'));
            } catch(e) {
                res.writeHead(404);
                res.end();
            }
        }
    }).listen(port);

    healthMonitor.start(monitorConfig, function() {
        done = true;
    });
}

function statistics(res) {
    res.writeHead(200, {'Content-Type': 'text/html'});

    healthMonitor.db.getStatistics(0, function(data) {
        res.end(ejs.render(EJS_TEMPLATE, {
            data: data,
            path: '/',
            filename: __dirname + '/index.ejs'
        }));
    });
}

function options(res) {
    res.writeHead(200, {'Content-Type': 'text/html'});
    res.end(ejs.render(EJS_TEMPLATE, {
        path: '/options',
        filename: __dirname + '/index.ejs'
    }));
}

function getXhr(req, res) {
    var chunks = [];

    req.on('data', function (chunk) {
        chunks.push(chunk);
    });

    req.on('end', function () {
        replyXhr(req.url.substring('/xhr/'.length), chunks.join(''), res);
    });
}

function replyXhr(action, data, res) {
    switch (action) {
        case 'get_urls':
            healthMonitor.db.getUrlList(function(urlList) {
                res.end(JSON.stringify(urlList));
            });

            break;
        case 'add_urls':
            healthMonitor.db.insertUrls(JSON.parse(data), function(docs) {
                res.end(JSON.stringify(docs));
            });

            break;
        case 'change_url':
            healthMonitor.db.changeUrl(JSON.parse(data), function(url) {
                res.end(JSON.stringify(url));
            });

            break;
        case 'remove_url':
            healthMonitor.db.removeUrl(JSON.parse(data), function() {
                res.end(JSON.stringify({}));
            });

            break;
        case 'clear_data':
            healthMonitor.db.clearStatisticsAndHistory(function() {
                res.end(JSON.stringify({}));
            });

            break;
        default:
            res.writeHead(404);
            res.end();

            break;
    }
}