function UrlGroupButtons() {
    this._urlList = null;
}

UrlGroupButtons.prototype.ADD_URLS_BUTTON_TEXT = 'Add urls';
UrlGroupButtons.prototype.ENABLE_ALL_URLS_BUTTON_TEXT = 'Enable all urls';
UrlGroupButtons.prototype.DISABLE_ALL_URLS_BUTTON_TEXT = 'Disable all urls';
UrlGroupButtons.prototype.REMOVE_ALL_URLS_BUTTON_TEXT = 'Remove all urls';
UrlGroupButtons.prototype.GROUP_BUTTON_CLASS_NAME = 'group-button';
UrlGroupButtons.prototype.ADD_URLS_BUTTON_CLASS_NAME = 'add';
UrlGroupButtons.prototype.ENABLE_ALL_URLS_BUTTON_CLASS_NAME = 'enable';
UrlGroupButtons.prototype.DISABLE_ALL_URLS_BUTTON_CLASS_NAME = 'disable';
UrlGroupButtons.prototype.REMOVE_ALL_URLS_BUTTON_CLASS_NAME = 'remove';
UrlGroupButtons.prototype.ADD_URLS_WRITING_BUTTON_STATE_CLASS_NAME = 'writing';

UrlGroupButtons.prototype.renderTo = function(parentElem) {
    parentElem.innerHTML = '';

    parentElem.appendChild(
        this._renderButton(this.ADD_URLS_BUTTON_TEXT, this.ADD_URLS_BUTTON_CLASS_NAME,
            this._onClickAddButton, this._prepareAddButton));
    parentElem.appendChild(
        this._renderButton(this.ENABLE_ALL_URLS_BUTTON_TEXT, this.ENABLE_ALL_URLS_BUTTON_CLASS_NAME,
            this._onClickEnableButton));
    parentElem.appendChild(
        this._renderButton(this.DISABLE_ALL_URLS_BUTTON_TEXT, this.DISABLE_ALL_URLS_BUTTON_CLASS_NAME,
            this._onClickDisableButton));
    parentElem.appendChild(
        this._renderButton(this.REMOVE_ALL_URLS_BUTTON_TEXT, this.REMOVE_ALL_URLS_BUTTON_CLASS_NAME,
            this._onClickRemoveButton));

    return this;
};

UrlGroupButtons.prototype.setUrlList = function(urlList) {
    this._urlList = urlList;
};


UrlGroupButtons.prototype._renderButton = function(text, className, clickHandler, prepare) {
    var button = document.createElement('li');

    button.innerHTML = text;
    button.classList.add(this.GROUP_BUTTON_CLASS_NAME);
    button.classList.add(className);

    if (typeof clickHandler === 'function')
        button.addEventListener('click', clickHandler.bind(this), false);

    if (typeof prepare === 'function')
        prepare.call(this, button);

    return button;
};

UrlGroupButtons.prototype._prepareAddButton = function(button) {
    var textarea = document.createElement('textarea');

    button.appendChild(textarea);
};

UrlGroupButtons.prototype._onClickAddButton = function(e) {
    var button = e.target,
        textarea = button.querySelector('textarea');

    if (button.classList.contains(this.GROUP_BUTTON_CLASS_NAME)) {
        if (button.classList.contains(this.ADD_URLS_WRITING_BUTTON_STATE_CLASS_NAME)) {
            button.classList.remove(this.ADD_URLS_WRITING_BUTTON_STATE_CLASS_NAME);

            this._urlList.addUrls(this._processUrlsFromText(textarea.value));
            textarea.value = '';
        } else {
            button.classList.add(this.ADD_URLS_WRITING_BUTTON_STATE_CLASS_NAME);
            textarea.focus();
        }
    }
};

UrlGroupButtons.prototype._onClickEnableButton = function() {
    this._urlList.changeStatusAllItems(true);
};

UrlGroupButtons.prototype._onClickDisableButton = function() {
    this._urlList.changeStatusAllItems(false);
};

UrlGroupButtons.prototype._onClickRemoveButton = function() {
    this._urlList.removeAll();
};

UrlGroupButtons.prototype._processUrlsFromText = function(text) {
    var urls = text.split('\n'),
        index = 0;

    while (index < urls.length) {
        urls[index] = urls[index].trim();

        if (!urls[index].length)
            urls.splice(index, 1);
        else
            index++;
    }

    return urls;
};
