var async = require('async'),
    exec = require('child_process').exec,
    fs = require('fs'),
    path = require('path'),
    server = require('./server');

//Consts
var BUILDS_PATH_PUBLIC_FIX = '\\\\corp\\builds\\release\\Build.Release.TESTCafe\\TESTCafe.Release',
    BUILDS_PATH = '\\\\corp\\builds\\release\\Build.Release.TESTCafe.Label\\TESTCafe.Release',
    TEST_USER_EMAIL = 'dxtestTestCafe@devexpress.com',
    TEST_USER_PASS = 'HGABSFDE',
    DEFAULT_INSTALL_PATH = fs.existsSync('C:\\Program Files (x86)') ?
        'C:\\Program Files (x86)\\DevExpress\\TestCafe\\Tools':
        'C:\\Program Files\\DevExpress\\TestCafe\\Tools',
    TESTCAFE_INDEX_PATH = path.join(DEFAULT_INSTALL_PATH, 'index.js'),
    TESTCAFE_DOCUMENTS_PATH = path.join(process.env['USERPROFILE'], 'Documents', 'TestCafe'),
    EXAMPLES_DIR_PATH = path.join(process.env['USERPROFILE'], 'Documents', 'TestCafe', 'Tests'),
    INSTALL_DATE_REG_KEY = 'HKEY_CLASSES_ROOT\\Licenses\\6F0F8269-1516-44C6-BD30-0E90BE27871C',
    LICENSE_DATE_REG_KEY = 'HKEY_CLASSES_ROOT\\Licenses\\0378852D-D597-4A32-B6D9-680A16A3CDA6',
    FS_WAITING_TIMEOUT = 5000;

var packageJSON = fs.readFileSync(path.join(__dirname, '../../package.json'), 'UTF8'),
    versionArr = JSON.parse(packageJSON.replace('\ufeff', '')).version.split('.'),
    registryField = [versionArr[0], versionArr[1]].join('');

//Utils
var getRegistryKeyValue = function(key, field, callback) {
    var cmd = 'reg query "' + key + '" /v ' + field,
        fieldString = "";

    exec(cmd, function(err, stdout) {
        try {
            fieldString = stdout.split('\r\n')[2].split('   ')[3];
        } catch(e) {
        }

        callback(fieldString);
    });
};

var removeRegistryKey = function(key, callback) {
    var cmd = 'reg delete ' + key + ' /f';

    exec(cmd, function() {
        callback();
    });
};

var getLatestBuildPath = function() {
    var buildsPath = (process.argv[3] === '--public-fix' ? BUILDS_PATH_PUBLIC_FIX : BUILDS_PATH),
        files = fs.readdirSync(buildsPath);

    var latestBuildDir = path.join(buildsPath, files[files.length - 1]),
        installerDir = path.join(latestBuildDir, 'Installations'),
        installerExe = fs.readdirSync(installerDir)[0];

    return path.join(installerDir, installerExe);
};

var setExpiryTrialDate = function(callback) {
    var cmd = ['reg add', INSTALL_DATE_REG_KEY, '/v', registryField, '/t REG_DWORD /d 1 /f'].join(' ');

    exec(cmd, function() {
        callback();
    });
};

var fatalError = function(err) {
    //NOTE: process.exit not waiting for console output. Sometimes app terminated before the outputting in console.
    var waitingIO = false;
    process.stdout.on('drain', function () {
        if (waitingIO)
            process.exit(-1);
    });

    if (process.stdout.write(err))
        process.exit(-1);
    else
        waitingIO = true;
};

var installerPath = getLatestBuildPath(),
    installCmd = [installerPath, ' /Q /INSTALLPATH:"%ProgramFiles%\\Devexpress\\TestCafe" /EULA:accept'].join(''),
    licensedInstallCmd = [installCmd, ' /EMAIL:', TEST_USER_EMAIL, ' /PASSWORD:', TEST_USER_PASS].join(''),
    uninstallCmd = [installerPath, '/Q -U'].join(' ');

//Test steps
async.series({
    'Clear registry keys - Trial': function(done) {
        console.log('Clear registry keys');

        removeRegistryKey(INSTALL_DATE_REG_KEY, function() {
            removeRegistryKey(LICENSE_DATE_REG_KEY, done);
        });
    },

    'Try uninstall previous installation': function(done) {
        console.log('Try uninstall previous installation');

        exec(uninstallCmd, function() {
            setTimeout(done, FS_WAITING_TIMEOUT);
        });
    },

    'Create mock samples directory if not exists': function(done) {
        console.log('Create mock samples directory if not exists');

        if(!fs.existsSync(EXAMPLES_DIR_PATH)) {
            if(!fs.existsSync(TESTCAFE_DOCUMENTS_PATH))
                fs.mkdirSync(TESTCAFE_DOCUMENTS_PATH);

            fs.mkdirSync(EXAMPLES_DIR_PATH);
        }

        done();
    },

    'Trial mode - installation': function(done){
        console.log('Trial mode - installation');

        exec(installCmd, function(err) {
            if(err)
                fatalError('INSTALLATION_ERR_TRIAL_MODE: ' + err);
            else {
                getRegistryKeyValue(LICENSE_DATE_REG_KEY, registryField, function(val) {
                    console.assert(!val, 'INSTALLER_ERR_TRIAL_MODE: License key has been installed in Trial Mode');
                    getRegistryKeyValue(INSTALL_DATE_REG_KEY, registryField, function(val) {
                        console.assert(parseInt(val, 16) > 0, 'INSTALLER_ERR_TRIAL_MODE: Install date hasn\'t been written in registry');
                        console.assert(fs.existsSync(path.join(DEFAULT_INSTALL_PATH, 'TestCafe.exe')), 'INSTALLER_ERR_TRIAL_MODE: TestCafe.exe hasn\'t been installed');

                        setTimeout(done, FS_WAITING_TIMEOUT);
                    });
                });
            }
        });
    },

    'Trial mode - Functional - trial isn\'t ending': function(done) {
        console.log('Trial mode - Functional - trial isn\'t ending');

        server.run(server.TRIAL_DIR, server.TRIAL_NOT_EXPIRE_PORT, TESTCAFE_INDEX_PATH, function(err) {
            if(err)
                fatalError('INSTALLATION_ERR_TRIAL_MODE: ' + JSON.stringify(err));
            else
                setTimeout(done, FS_WAITING_TIMEOUT);
        });
    },

    'Trial mode - Functional - trial is ending': function(done) {
        console.log('Trial mode - Functional - trial is ending');
        setExpiryTrialDate(function() {
            server.run(server.TRIAL_EXPIRED_DIR, server.TRIAL_EXPIRED_PORT, TESTCAFE_INDEX_PATH, function(err) {
                if(err)
                    fatalError('INSTALLATION_ERR_TRIAL_MODE: ' + JSON.stringify(err));
                else
                    setTimeout(done, FS_WAITING_TIMEOUT);
            });
        });
    },

    'Trial mode - uninstalling': function(done) {
        console.log('Trial mode - uninstalling');

        exec(uninstallCmd, function(err) {
            if(err)
                fatalError('UNINSTALLING_ERR_TRIAL_MODE: ' + err);
            else {
                console.assert(!fs.existsSync(DEFAULT_INSTALL_PATH), 'INSTALLER_ERR_TRIAL_MODE: TestCafe directory hasn\'t been uninstalled');
                console.assert(fs.existsSync(EXAMPLES_DIR_PATH), 'INSTALLER_ERR_TRIAL_MODE: Examples files have deleted');

                setTimeout(done, FS_WAITING_TIMEOUT);
            }
        });
    },

    'Clear registry keys - License': function(done) {
        console.log('Clear registry keys');

        removeRegistryKey(INSTALL_DATE_REG_KEY, function() {
            removeRegistryKey(LICENSE_DATE_REG_KEY, done);
        });
    },

    'License mode - installation': function(done) {
        console.log('License mode - installation');

        exec(licensedInstallCmd, function(err) {
            if(err)
                fatalError('INSTALLATION_ERR_LICENSE_MODE: ' + err);
            else {
                getRegistryKeyValue(LICENSE_DATE_REG_KEY, registryField, function(val) {
                    console.assert(val, "INSTALLER_ERR_LICENSE_MODE: License key hasn't been installed in License Mode");
                    getRegistryKeyValue(INSTALL_DATE_REG_KEY, registryField, function(val) {
                        console.assert(!parseInt(val, 16), "INSTALLER_ERR_LICENSE_MODE: Trial date has been written in registry");
                        console.assert(fs.existsSync(path.join(DEFAULT_INSTALL_PATH, 'TestCafe.exe')), 'INSTALLER_ERR_LICENSE_MODE: TestCafe.exe hasn\'t been installed');

                        setTimeout(done, FS_WAITING_TIMEOUT);
                    });
                });
            }
        });
    },

    'License mode - Functional': function(done) {
        console.log('License mode - Functional');

        server.run(server.LICENSE_DIR, server.LICENSE_PORT, TESTCAFE_INDEX_PATH, function(err) {
            if(err)
                fatalError('INSTALLATION_ERR_LICENSE_MODE: ' + JSON.stringify(err));
            else
                setTimeout(done, FS_WAITING_TIMEOUT);
        });
    },

    'License mode - uninstalling': function(done) {
        console.log('License mode - uninstalling');

        exec(uninstallCmd, function(err) {
            if(err)
                fatalError('UNINSTALLING_ERR_LICENSE_MODE: ' + err);
            else {
                console.assert(!fs.existsSync(DEFAULT_INSTALL_PATH), 'INSTALLER_ERR_LICENSE_MODE: TestCafe directory hasn\'t been uninstalled');
                console.assert(fs.existsSync(EXAMPLES_DIR_PATH), 'INSTALLER_ERR_LICENSE_MODE: Examples files have deleted');

                setTimeout(done, FS_WAITING_TIMEOUT);
            }
        });
    }
}, function() {
    console.log('___________________________________\nInstall tests successfully completed.');

    process.nextTick(function() {
        process.exit(0);
    });

});