var markdownFormatter = require('../../../../../lib/err_msg_builder/markdown_formatter'),
    msgBuilderUtil = require('../../../../../lib/err_msg_builder/util');

exports['Markdown formatter'] = {
    'escapeHtml': function(t) {
        var html = '<div><p>&</p></div>';

        t.strictEqual(1,  markdownFormatter.escapeHtml(1));
        t.strictEqual('&lt;div&gt;&lt;p&gt;&amp;&lt;/p&gt;&lt;/div&gt;', markdownFormatter.escapeHtml(html));
        t.done();
    },

    'wrapStepName': function(t) {
        t.strictEqual('<code class="go-to-code-btn step-name">stepName</code>', markdownFormatter.wrapStepName('stepName'));
        t.done();
    },

    'wrapLink':function(t) {
        t.strictEqual('<a href="http://host:1234/test?a=b">http://host:1234/test?a=b</a>', markdownFormatter.wrapLink('http://host:1234/test?a=b'));
        t.done();
    },

    'notOk - markdown': function(t) {
        var err =  {
            stepName: 'Step',
            relatedSourceCode: 'notOk("test")',
            actual: '"test"'
        };

        t.strictEqual('<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;: <code class="go-to-code-btn language-javascript">notOk("test")</code>\n' +
            '<strong>Expected:</strong> <code class="language-javascript">null</code>, <code class="language-javascript">undefined</code>, <code class="language-javascript">false</code>, <code class="language-javascript">NaN</code> or <code class="language-javascript">&#39;&#39;</code>\n' +
            '<strong>Actual:</strong> <code class="language-javascript">"test"</code></p>',
            markdownFormatter.buildNotOkAssert(err));
        t.done();

    },

    'ok - markdown': function(t) {
        var err =  {
            stepName: 'Step',
            relatedSourceCode: 'ok(false)',
            actual: 'false'
        };

        t.strictEqual('<p><strong class=\"assert-icon\"></strong>Assertion failed at step &quot;<code class=\"go-to-code-btn step-name\">Step</code>&quot;: <code class=\"go-to-code-btn language-javascript\">ok(false)</code>\n' +
            '<strong>Expected:</strong> not <code class=\"language-javascript\">null</code>, not <code class=\"language-javascript\">undefined</code>, not <code class=\"language-javascript\">false</code>, not <code class=\"language-javascript\">NaN</code> and not <code class=\"language-javascript\">&#39;&#39;</code>\n' +
            '<strong>Actual:</strong> <code class=\"language-javascript\">false</code></p>',
            markdownFormatter.buildOkAssert(err));
        t.done();
    },

    'notEq - markdown': function(t) {
        var err = {
            stepName: 'Step',
            relatedSourceCode: 'notEq("test", "test")',
            actual: '"test"',
            expected: '"test"'
        };

        t.strictEqual('<p><strong class=\"assert-icon\"></strong>Assertion failed at step &quot;<code class=\"go-to-code-btn step-name\">Step</code>&quot;: <code class=\"go-to-code-btn language-javascript\">notEq(\"test\", \"test\")</code>\n' +
            '<strong>Expected:</strong> <code class=\"language-javascript\">not "test"</code>\n' +
            '<strong>Actual:</strong> <code class=\"language-javascript\">"test"</code></p>',
            markdownFormatter.buildNotEqAssert(err));
        t.done();
    },

    'eq - markdown': function(t) {
        msgBuilderUtil.MAX_STRING_LENGTH = 10;

        var errs = {
            'different types': {
                stepName: 'Step',
                relatedSourceCode: 'eq("test", 123)',
                actual: 123,
                expected: '"test"',
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;: <code class="go-to-code-btn language-javascript">eq("test", 123)</code>\n' +
                    '<strong>Expected:</strong> <code class="language-javascript">"test"</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">123</code></p>'
            },
            'numbers': {
                stepName: 'Step',
                relatedSourceCode: 'eq(12345, 54321)',
                actual: '54321',
                expected: '12345',
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;: <code class="go-to-code-btn language-javascript">eq(12345, 54321)</code>\n' +
                    '<strong>Expected:</strong> <code class="language-javascript">12345</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">54321</code></p>'
            },
            'string - string length less than max output length': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00001000", "0000000")',
                actual: '"0000000"',
                expected: '"00001000"',
                isStrings: true,
                key: 4,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq("00001000", "0000000")</code></p>\n\n<p>Strings differ at index <code class="language-javascript">4</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <code class="language-javascript">"00001000"</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">"0000000"</code>\n<code class="diff-marker">     ^</code></p>'
            },
            'string - string more than max output length: diff in the begin': {
                stepName: 'Step',
                relatedSourceCode: 'eq("12345678901", "00000000000")',
                actual: '"00000000000"',
                expected: '"12345678901"',
                isStrings: true,
                key: 0,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq("12345678901", "00000000000")</code></p>\n\n<p>Strings differ at index <code class="language-javascript">0</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <code class="language-javascript">"1234567..."</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">"0000000..."</code>\n<code class="diff-marker"> ^</code></p>'
            },
            'string - string more than max output length: diff in the end': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00000000000", "00000000001")',
                actual: '"00000000001"',
                expected: '"00000000000"',
                isStrings: true,
                key: 10,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq("00000000000", "00000000001")</code></p>\n\n<p>Strings differ at index <code class="language-javascript">10</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <code class="language-javascript">"...0000000"</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">"...0000001"</code>\n<code class="diff-marker">          ^</code></p>'
            },
            'string - string more than max output length: diff in middle': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00000000000", "0000001000000")',
                actual: '"0000001000000"',
                expected: '"0000000000000"',
                isStrings: true,
                key: 6,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq("00000000000", "0000001000000")</code></p>\n\n<p>Strings differ at index <code class="language-javascript">6</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <code class="language-javascript">"...0000..."</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">"...0100..."</code>\n<code class="diff-marker">     ^</code></p>'
            },
            'Regression - string - string with new line': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00\n0001\n0", "00\n000000")',
                actual: '"00\n0000100\n0"',
                expected: '"00\n00000000"',
                isStrings: true,
                key: 6,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq("00\n0001\n0", "00\n000000")</code></p>\n\n<p>Strings differ at index <code class="language-javascript">6</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <code class="language-javascript">"...0000000"</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">"...0100\\n0"</code>\n<code class="diff-marker">     ^</code></p>'
            },
            'arrays - different types': {
                stepName: 'Step',
                relatedSourceCode: 'eq([1, "test"], [1, 123])',
                isArrays: true,
                key: 1,
                actual: 123,
                expected: '"test"',
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq([1, "test"], [1, 123])</code></p>\n\n<p>Arrays differ at index <code class="language-javascript">1</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <strong>[1]: </strong><code class="language-javascript">"test"</code>\n' +
                    '<strong>Actual:</strong> <strong>[1]: </strong><code class="language-javascript">123</code></p>'
            },
            'arrays - numbers': {
                stepName: 'Step',
                relatedSourceCode: 'eq([12345], [54321])',
                isArrays: true,
                key: 0,
                actual: '54321',
                expected: '12345',
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq([12345], [54321])</code></p>\n\n<p>Arrays differ at index <code class="language-javascript">0</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <strong>[0]: </strong><code class="language-javascript">12345</code>\n' +
                    '<strong>Actual:</strong> <strong>[0]: </strong><code class="language-javascript">54321</code></p>'
            },
            'arrays - two strings - string length less than max output length': {
                stepName: 'Step',
                relatedSourceCode: 'eq(["01000"], ["0000"])',
                actual: '"0000"',
                expected: '"01000"',
                isArrays: true,
                diffType: {
                    isStrings: true,
                    diffIndex: 1
                },
                key: 0,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq(["01000"], ["0000"])</code></p>\n\n<p>Arrays differ at index <code class="language-javascript">0</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <strong>[0]: </strong><code class="language-javascript">"01000"</code>\n' +
                    '<strong>Actual:</strong> <strong>[0]: </strong><code class="language-javascript">"0000"</code>\n<code class="diff-marker">       ^</code></p>'
            },
            'arrays - two strings - string - string more than max output length: diff in the begin': {
                stepName: 'Step',
                relatedSourceCode: 'eq("12345678901", "00000000000")',
                actual: '"00000000000"',
                expected: '"12345678901"',
                isArrays: true,
                diffType: {
                    isStrings: true,
                    diffIndex: 0
                },
                key: 0,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq("12345678901", "00000000000")</code></p>\n\n<p>Arrays differ at index <code class="language-javascript">0</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <strong>[0]: </strong><code class="language-javascript">"12..."</code>\n' +
                    '<strong>Actual:</strong> <strong>[0]: </strong><code class="language-javascript">"00..."</code>\n<code class="diff-marker">      ^</code></p>'
            },
            'arrays - two strings - string - string more than max output length: diff in the end': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00000000000", "00000000001")',
                actual: '"00000000001"',
                expected: '"00000000000"',
                isArrays: true,
                diffType: {
                    isStrings: true,
                    diffIndex: 10
                },
                key: 0,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq("00000000000", "00000000001")</code></p>\n\n<p>Arrays differ at index <code class="language-javascript">0</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <strong>[0]: </strong><code class="language-javascript">"...00"</code>\n' +
                    '<strong>Actual:</strong> <strong>[0]: </strong><code class="language-javascript">"...01"</code>\n<code class="diff-marker">          ^</code></p>'
            },
            'arrays - two objects': {
                stepName: 'Step',
                relatedSourceCode: 'eq([1, {t:1}], [1, {t:2}])',
                actual: '{t:2}',
                expected: '{t:1}',
                isArrays: true,
                key: 1,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq([1, {t:1}], [1, {t:2}])</code></p>\n\n<p>Arrays differ at index <code class="language-javascript">1</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <strong>[1]: </strong><code class="language-javascript">{t:1}</code>\n' +
                    '<strong>Actual:</strong> <strong>[1]: </strong><code class="language-javascript">{t:2}</code></p>'
            },
            'arrays - comparing values more than max string length': {
                stepName: 'Step',
                relatedSourceCode: 'eq([1, {t:"1111111111"}], [1, {t:"222222222222"}])',
                actual: '{t:"222222222222"}',
                expected: '{t:"1111111111"}',
                isArrays: true,
                key: 0,
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq([1, {t:"1111111111"}], [1, {t:"222222222222"}])</code></p>\n\n<p>Arrays differ at index <code class="language-javascript">0</code>:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <strong>[0]: </strong><code class="language-javascript">{t:"111...</code>\n' +
                    '<strong>Actual:</strong> <strong>[0]: </strong><code class="language-javascript">{t:"222...</code></p>'
            },
            'objects - nested objects - diff in elemental values': {
                stepName: 'Step',
                relatedSourceCode: 'eq({t:{a:1}}, {t:{a:2}})',
                actual: '2',
                expected: '1',
                isObjects: true,
                key: 't.a',
                res: '<p><strong class=\"assert-icon\"></strong>Assertion failed at step &quot;<code class=\"go-to-code-btn step-name\">Step</code>&quot;:<code class=\"go-to-code-btn language-javascript\">eq({t:{a:1}}, {t:{a:2}})</code></p>\n\n<p>Objects differ at the <code class=\"language-javascript\">t.a</code> field:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <code class=\"language-javascript\">1</code>\n' +
                    '<strong>Actual:</strong> <code class=\"language-javascript\">2</code></p>'
            },
            'objects - nested object - diff in objects structure': {
                stepName: 'Step',
                relatedSourceCode: 'eq({t:{a:{b:1}}}, {t:{a:{c:2}}})',
                actual: '{c:2}',
                expected: '{b:1}',
                isObjects: true,
                key: 't.a',
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq({t:{a:{b:1}}}, {t:{a:{c:2}}})</code></p>\n\n<p>Objects differ at the <code class="language-javascript">t.a</code> field:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <code class="language-javascript">{b:1}</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">{c:2}</code></p>'
            },
            'objects - diff in arrays': {
                stepName: 'Step',
                relatedSourceCode: 'eq({t: [1, 2, 3]}}, {t: [1, 2, 4]})',
                actual: '[1, 2, 4]',
                expected: '[1, 2, 3]',
                isObjects: true,
                key: 't',
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq({t: [1, 2, 3]}}, {t: [1, 2, 4]})</code></p>\n\n<p>Objects differ at the <code class="language-javascript">t</code> field:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <code class="language-javascript">[1, 2, 3]</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">[1, 2, 4]</code></p>'
            },
            'objects - comparing values more than max string length': {
                stepName: 'Step',
                relatedSourceCode: 'eq({a: {b:"1111111111111111"}, {a: {c:"222222222222"}})',
                actual: '{c:"222222222222"}',
                expected: '{b:"1111111111111111"}',
                isObjects: true,
                key: 'a.b',
                res: '<p><strong class="assert-icon"></strong>Assertion failed at step &quot;<code class="go-to-code-btn step-name">Step</code>&quot;:<code class="go-to-code-btn language-javascript">eq({a: {b:"1111111111111111"}, {a: {c:"222222222222"}})</code></p>\n\n<p>Objects differ at the <code class="language-javascript">a.b</code> field:</p>\n\n<p>' +
                    '<strong>Expected:</strong> <code class="language-javascript">{b:"111...</code>\n' +
                    '<strong>Actual:</strong> <code class="language-javascript">{c:"222...</code></p>'
            }
        };

        Object.keys(errs).forEach(function(key) {
            var err = errs[key];

            t.strictEqual(err.res, markdownFormatter.buildEqAssert(err));
        });

        t.done();
    }
};