var plainTextFormatter = require('../../../../../lib/err_msg_builder/plain_text_formatter'),
    msgBuilderUtil = require('../../../../../lib/err_msg_builder/util');

exports['Plain text formatter'] = {
    'notOk - plain text': function(t) {
        var err =  {
            stepName: 'Step',
            relatedSourceCode: 'notOk("test")',
            actual: '"test"'
        };

        t.strictEqual('Assertion failed at step Step: notOk("test")\n' +
            'Expected: null, undefined, false, NaN or \'\'\n' +
            'Actual:   "test"\n',
            plainTextFormatter.buildNotOkAssert(err));
        t.done();
    },

    'ok - plain text': function(t) {
        var err =  {
            stepName: 'Step',
            relatedSourceCode: 'ok(false)',
            actual: 'false'
        };

        t.strictEqual('Assertion failed at step Step: ok(false)\n' +
            'Expected: null, undefined, false, NaN or \'\'\n' +
            'Actual:   false\n',
            plainTextFormatter.buildNotOkAssert(err));
        t.done();
    },

    'notEq - plain text': function(t) {
        var err = {
            stepName: 'Step',
            relatedSourceCode: 'notEq("test", "test")',
            actual: '"test"',
            expected: '"test"'
        };

        t.strictEqual('Assertion failed at step Step: notEq("test", "test")\n' +
            'Expected: not "test"\n' +
            'Actual:   "test"\n',
            plainTextFormatter.buildNotEqAssert(err));
        t.done();
    },

    'eq - plain text': function(t) {
        msgBuilderUtil.MAX_STRING_LENGTH = 10;

        var errs = {
            'different types': {
                stepName: 'Step',
                relatedSourceCode: 'eq("test", 123)',
                actual: 123,
                expected: '"test"',
                res: 'Assertion failed at step "Step": eq("test", 123)\n' +
                    'Expected: "test"\n' +
                    'Actual:   123\n'
            },
            'numbers': {
                stepName: 'Step',
                relatedSourceCode: 'eq(12345, 54321)',
                actual: '54321',
                expected: '12345',
                res: 'Assertion failed at step "Step": eq(12345, 54321)\n' +
                    'Expected: 12345\n' +
                    'Actual:   54321\n'
            },
            'string - string length less than max output length': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00001000", "0000000")',
                actual: '"0000000"',
                expected: '"00001000"',
                isStrings: true,
                key: 4,
                res: 'Assertion failed at step "Step": eq("00001000", "0000000")\n' +
                    'Strings differ at index "4":\n' +
                    'Expected: "00001000"\n' +
                    'Actual:   "0000000"\n' +
                    '               ^'
            },
            'string - string more than max output length: diff in the begin': {
                stepName: 'Step',
                relatedSourceCode: 'eq("12345678901", "00000000000")',
                actual: '"00000000000"',
                expected: '"12345678901"',
                isStrings: true,
                key: 0,
                res: 'Assertion failed at step "Step": eq("12345678901", "00000000000")\n' +
                    'Strings differ at index "0":\n' +
                    'Expected: "1234567..."\n' +
                    'Actual:   "0000000..."\n' +
                    '           ^'
            },
            'string - string more than max output length: diff in the end': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00000000000", "00000000001")',
                actual: '"00000000001"',
                expected: '"00000000000"',
                isStrings: true,
                key: 10,
                res: 'Assertion failed at step "Step": eq("00000000000", "00000000001")\n' +
                    'Strings differ at index "10":\n' +
                    'Expected: "...0000000"\n' +
                    'Actual:   "...0000001"\n' +
                    '                    ^'
            },
            'string - string more than max output length: diff in middle': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00000000000", "00000000001")',
                actual: '"0000001000000"',
                expected: '"0000000000000"',
                isStrings: true,
                key: 6,
                res: 'Assertion failed at step "Step": eq("00000000000", "00000000001")\n' +
                    'Strings differ at index "6":\n' +
                    'Expected: "...0000..."\n' +
                    'Actual:   "...0100..."\n' +
                    '               ^'
            },
            'Regression - string - string with new line': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00\n0001\n0", "00\n000000")',
                actual: '"00\n0000100\n0"',
                expected: '"00\n00000000"',
                isStrings: true,
                key: 6,
                res: 'Assertion failed at step "Step": eq("00\\n0001\\n0", "00\\n000000")\n' +
                    'Strings differ at index "6":\n' +
                    'Expected: "...0000000"\n' +
                    'Actual:   "...0100\\n0"\n' +
                    '               ^'
            },
            'arrays - different types': {
                stepName: 'Step',
                relatedSourceCode: 'eq([1, "test"], [1, 123])',
                isArrays: true,
                key: 1,
                actual: 123,
                expected: '"test"',
                res: 'Assertion failed at step Step: eq([1, "test"], [1, 123])\n' +
                    'Arrays differ at index "1":\n' +
                    'Expected: [1]: "test"\n' +
                    'Actual:   [1]: 123\n'
            },
            'arrays - numbers': {
                stepName: 'Step',
                relatedSourceCode: 'eq([12345], [54321])',
                isArrays: true,
                key: 0,
                actual: '54321',
                expected: '12345',
                res: 'Assertion failed at step Step: eq([12345], [54321])\n' +
                    'Arrays differ at index "0":\n' +
                    'Expected: [0]: 12345\n' +
                    'Actual:   [0]: 54321\n'
            },
            'arrays - two strings - string length less than max output length': {
                stepName: 'Step',
                relatedSourceCode: 'eq(["01000"], ["0000"])',
                actual: '"0000"',
                expected: '"01000"',
                isArrays: true,
                diffType: {
                    isStrings: true,
                    diffIndex: 1
                },
                key: 0,
                res: 'Assertion failed at step Step: eq(["01000"], ["0000"])\n' +
                    'Arrays differ at index "0":\n' +
                    'Expected: [0]: "01000"\n' +
                    'Actual:   [0]: "0000"\n' +
                    '                 ^'
            },
            'arrays - two strings - string - string more than max output length: diff in the begin': {
                stepName: 'Step',
                relatedSourceCode: 'eq("12345678901", "00000000000")',
                actual: '"00000000000"',
                expected: '"12345678901"',
                isArrays: true,
                diffType: {
                    isStrings: true,
                    diffIndex: 0
                },
                key: 0,
                res: 'Assertion failed at step Step: eq("12345678901", "00000000000")\n' +
                    'Arrays differ at index "0":\n' +
                    'Expected: [0]: "12..."\n' +
                    'Actual:   [0]: "00..."\n' +
                    '                ^'
            },
            'arrays - two strings - string - string more than max output length: diff in the end': {
                stepName: 'Step',
                relatedSourceCode: 'eq("00000000000", "00000000001")',
                actual: '"00000000001"',
                expected: '"00000000000"',
                isArrays: true,
                diffType: {
                    isStrings: true,
                    diffIndex: 10
                },
                key: 0,
                res: 'Assertion failed at step Step: eq("00000000000", "00000000001")\n' +
                    'Arrays differ at index "0":\n' +
                    'Expected: [0]: "...00"\n' +
                    'Actual:   [0]: "...01"\n' +
                    '                    ^'
            },
            'arrays - two objects': {
                stepName: 'Step',
                relatedSourceCode: 'eq([1, {t:1}], [1, {t:2}])',
                actual: '{t:2}',
                expected: '{t:1}',
                isArrays: true,
                key: 1,
                res: 'Assertion failed at step Step: eq([1, {t:1}], [1, {t:2}])\n' +
                    'Arrays differ at index \"1\":\n' +
                    'Expected: [1]: {t:1}\n' +
                    'Actual:   [1]: {t:2}\n'
            },
            'arrays - comparing values more than max string length': {
                stepName: 'Step',
                relatedSourceCode: 'eq([1, {t:"1111111111"}], [1, {t:"222222222222"}])',
                actual: '{t:"222222222222"}',
                expected: '{t:"1111111111"}',
                isArrays: true,
                key: 0,
                res: 'Assertion failed at step Step: eq([1, {t:"1111111111"}], [1, {t:"222222222222"}])\n' +
                    'Arrays differ at index "0":\n' +
                    'Expected: [0]: {t:"111...\n' +
                    'Actual:   [0]: {t:"222...\n'
            },
            'objects - nested objects - diff in elemental values': {
                stepName: 'Step',
                relatedSourceCode: 'eq({t:{a:1}}, {t:{a:2}})',
                actual: '2',
                expected: '1',
                isObjects: true,
                key: 't.a',
                res: 'Assertion failed at step "Step": eq({t:{a:1}}, {t:{a:2}})\n' +
                    'Objects differ at the "t.a" field:\n' +
                    'Expected: 1\n' +
                    'Actual:   2\n'
            },
            'objects - nested object - diff in objects structure': {
                stepName: 'Step',
                relatedSourceCode: 'eq({t:{a:{b:1}}}, {t:{a:{c:2}}})',
                actual: '{c:2}',
                expected: '{b:1}',
                isObjects: true,
                key: 't.a',
                res: 'Assertion failed at step "Step": eq({t:{a:{b:1}}}, {t:{a:{c:2}}})\n' +
                    'Objects differ at the "t.a" field:\n' +
                    'Expected: {b:1}\n' +
                    'Actual:   {c:2}\n'
            },
            'objects - diff in arrays': {
                stepName: 'Step',
                relatedSourceCode: 'eq({t: [1, 2, 3]}}, {t: [1, 2, 4]})',
                actual: '[1, 2, 4]',
                expected: '[1, 2, 3]',
                isObjects: true,
                key: 't',
                res: 'Assertion failed at step "Step": eq({t: [1, 2, 3]}}, {t: [1, 2, 4]})\n' +
                    'Objects differ at the "t" field:\n' +
                    'Expected: [1, 2, 3]\n' +
                    'Actual:   [1, 2, 4]\n'
            },
            'objects - comparing values more than max string length': {
                stepName: 'Step',
                relatedSourceCode: 'eq({a: {b:"1111111111111111"}, {a: {c:"222222222222"}})',
                actual: '{c:"222222222222"}',
                expected: '{b:"1111111111111111"}',
                isObjects: true,
                key: 'a.b',
                res: 'Assertion failed at step "Step": eq({a: {b:"1111111111111111"}, {a: {c:"222222222222"}})\n' +
                    'Objects differ at the "a.b" field:\n' +
                    'Expected: {b:"111...\n' +
                    'Actual:   {c:"222...\n'
            }
        };

        Object.keys(errs).forEach(function(key) {
            var err = errs[key];

            t.strictEqual(err.res, plainTextFormatter.buildEqAssert(err));
        });

        t.done();
    }
};