var fs = require('fs'),
    testUtils = require('../../../test_utils'),
    tmp = require('tmp'),
    path = require('path'),
    async = require('async'),
    FixtureCode = require('../../../../../lib/fixture_code'),
    ERR = require('../../../../../lib/server/server_errs');

var rmdirRecursive = testUtils.rmdirRecursive,
    np = testUtils.normalizePath,
    isNumber = testUtils.isNumber,
    watchdog = null,
    basePath = null;

var TRIM_REG_EXP = /\n|\r|\s|'|"/g;

function copyFile(src, dest) {
    fs.writeFileSync(dest, fs.readFileSync(src).toString());
}

exports['FixtureCode.CodeGenerator.getTestStepBodies()'] = {
    'Create action steps functions': function (t) {
        var stepsInfo = [
            {
                name: 'Click element 1',
                action: 'click',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\")"}
                ],
                actionArgs: [
                    {
                        options: {}
                    }
                ]
            },

            {
                name: 'RClick element2',
                action: 'rclick',
                currentSelectorIndex: 0,
                currentIFrameSelectorIndex: 1,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\")"}
                ],
                iFrameSelectors: [
                    { id: 'id', selector: "$(\"#iframe\")"},
                    { id: 'className', selector: "$(\".iframe\")"}
                ],
                actionArgs: [
                    {
                        options: {}
                    }
                ]
            }
        ];

        var expectedCodeWithoutIFrameWrapping = [
                '[',
                'function(){act.click("#titleLink6",{})},',
                'inIFrame($(".iframe"), function(){act.rclick("#titleLink6",{})})',
                ']'
            ].join(''),
            expectedCodeWithIFrameWrapping = [
                '[',
                'function(){act.click("#titleLink6",{})},',
                'inIFrame(function () { return $(".iframe") }, function(){act.rclick("#titleLink6",{})})',
                ']'
            ].join(''),
            resWithoutIFrameWrapping = FixtureCode.CodeGenerator.getTestStepBodies(stepsInfo),
            resWithIFrameWrapping = FixtureCode.CodeGenerator.getTestStepBodies(stepsInfo, true);

        t.ok(testUtils.compareCode(expectedCodeWithoutIFrameWrapping, resWithoutIFrameWrapping));
        t.ok(testUtils.compareCode(expectedCodeWithIFrameWrapping, resWithIFrameWrapping));
        t.done();
    },

    'Create assertions steps functions': function (t) {
        var stepsInfo = [
            {
                name: 'check input1',
                isAssertion: true,
                blocks: [
                    {
                        assertions: [
                            {
                                operator: 'eq',
                                arguments: [ '$("#input1").val()', '"test"' ],
                                message: 'check input1 value'
                            },
                            {
                                operator: 'ok',
                                arguments: [ '$("#input1").is(":visible")' ],
                                message: 'check input1 visibility'
                            }
                        ]
                    }
                ]
            },

            {
                name: 'check input2',
                isAssertion: true,
                blocks: [
                    {
                        assertions: [
                            {
                                operator: 'notEq',
                                arguments: [ '$("input")[1].valueAsNumber', '11' ],
                                message: undefined
                            },
                            {
                                operator: 'notOk',
                                arguments: [ '$("input")[1].offsetWidth > 150 && $("input")[1].offsetHeight < 150' ],
                                message: undefined
                            }
                        ]
                    }
                ]
            },

            {
                name: 'empty arguments',
                isAssertion: true,
                blocks: [
                    {
                        assertions: [
                            {
                                operator: 'eq',
                                arguments: [undefined, undefined],
                                message: undefined
                            }
                        ]
                    }
                ]
            },

            {
                name: 'inIFrame assertion',
                isAssertion: true,
                blocks: [
                    {
                        assertions: [
                            {
                                operator: 'ok',
                                arguments: ['true'],
                                message: undefined
                            }
                        ],
                        iFrameSelector: "$(\".iframe\")"
                    }
                ]
            },

            {
                name: 'Assert',
                isAssertion: true,
                blocks: [
                    {
                        iFrameSelector: "$(\".iframe\")",
                        assertions: [
                            {
                                operator: 'ok',
                                arguments: ['true'],
                                message: 'message1'
                            }
                        ]
                    },

                    {
                        assertions: [
                            {
                                operator: 'notOk',
                                arguments: ['false'],
                                message: 'message2'
                            },
                            {
                                operator: 'ok',
                                arguments: ['true'],
                                message: 'message3'
                            }
                        ]
                    }
                ]
            }
        ];

        var expectedCode = [
                '[',
                'function(){eq($("#input1").val(),"test","check input1 value");ok($("#input1").is(":visible"),"check input1 visibility")},',
                'function(){notEq($("input")[1].valueAsNumber,11);notOk($("input")[1].offsetWidth>150&&$("input")[1].offsetHeight<150)},',
                'function(){eq(undefined,undefined)},',
                'inIFrame($(".iframe"),function(){ok(true)}),',
                'inIFrame($(".iframe"),function(){ok(true,"message1")}),',
                'function(){notOk(false,"message2");ok(true,"message3")}',
                ']'
            ].join(''),
            res = FixtureCode.CodeGenerator.getTestStepBodies(stepsInfo);

        t.ok(testUtils.compareCode(expectedCode, res));
        t.done();
    },

    'Converting selectors to make them compatible with animation waiting mechanism': function (t) {
        var stepsInfo = [
            {
                name: '\$ function call',
                action: 'click',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\")"}
                ]
            },
            {
                name: 'jQuery function call',
                action: 'click',
                currentSelectorIndex: 0,
                currentIFrameSelectorIndex: 1,
                selectors: [
                    { id: 'custom', selector: "jQuery(':contains(text)')"}
                ]
            },
            {
                name: 'jQuery finction with eq function',
                action: 'click',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'custom', selector: "$(\"#titleLink6\").eq(0)"}
                ]
            },
            {
                name: 'jQuery finction with two parameters',
                action: 'click',
                currentSelectorIndex: 0,
                currentIFrameSelectorIndex: 1,
                selectors: [
                    { id: 'custom', selector: "jQuery(':contains(text)', '#id1')"}
                ]
            },
            {
                name: 'string selector',
                action: 'click',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'custom', selector: '".className1"'}
                ]
            },
            {
                name: 'function',
                action: 'click',
                currentSelectorIndex: 0,
                currentIFrameSelectorIndex: 1,
                selectors: [
                    { id: 'custom', selector: "(function(){ return $('#id1'); })"}
                ]
            },
            {
                name: 'some other selector expression',
                action: 'click',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'custom', selector: 'document.getElementById("id1")'}
                ]
            }
        ];

        var expectedCode = [
                '[',
                'function(){act.click("#titleLink6")},',
                'function(){act.click(":contains(text)")},',
                'function(){var actionTarget=function(){return $("#titleLink6").eq(0)};',
                'act.click(actionTarget)},',
                'function(){var actionTarget=function(){return jQuery(":contains(text)","#id1")};',
                'act.click(actionTarget)},',
                'function(){act.click(".className1")},',
                'function(){var actionTarget=function(){return $("#id1")};',
                'act.click(actionTarget)},',
                'function(){var actionTarget=function(){return document.getElementById("id1")};',
                'act.click(actionTarget)}',
                ']'
            ].join(''),
            actualCode = FixtureCode.CodeGenerator.getTestStepBodies(stepsInfo);

        t.ok(testUtils.compareCode(expectedCode, actualCode));
        t.done();
    }
};

exports['FixtureCode.CodeGenerator.createFixture()'] = {
    setUp: function (done) {
        tmp.dir(function (err, tmpDir) {
            basePath = tmpDir;
            watchdog = new testUtils.Watchdog();

            done();
        });
    },

    tearDown: function (done) {
        watchdog.shrink();
        rmdirRecursive(basePath);

        done();
    },

    'Create fixture': function (t) {
        var fixtureDescrs = [
            {name: 'test1', page: 'testPage1', expectedCode: '"@fixture test1";"@page http://testPage1";'},
            {name: 'test2', page: '/testPage2', expectedCode: '"@fixture test2";"@page /testPage2";'},
            {name: 'test3', page: './testPage3', expectedCode: '"@fixture test3";"@page ./testPage3";'},
            {name: 'test4', page: '../testPage4', expectedCode: '"@fixture test4";"@page ../testPage4";'}
        ];

        t.expect(8);

        async.forEach(fixtureDescrs, function (descr, callback) {
            var fileName = [basePath, '/', descr.name, '.test.js'].join('');

            FixtureCode.CodeGenerator.createFixture(fileName, descr.name, descr.page, function (err) {
                t.ok(!err);

                var fixtureCode = fs.readFileSync(fileName).toString();
                t.ok(testUtils.compareCode(descr.expectedCode, fixtureCode));

                callback();
            });
        }, function () {
            t.done();
        });
    },

    'Error fs IO operation': function (t) {
        t.expect(2);

        var fixtureName = "testFixtureName",
            fixturePage = "testFixturePage",
            dirName = "testDir";

        FixtureCode.CodeGenerator.createFixture(path.join(basePath, dirName, fixtureName + '.test.js'), fixtureName, fixturePage, function (err) {
            t.strictEqual(err.code, FixtureCode.ErrCodes.WRITE_FILE_FAILED);
            t.ok(err.filename);

            t.done();
        });
    }
};

exports['FixtureCode.CodeGenerator.editDirectives()'] = {
    setUp: function (done) {
        tmp.dir(function (err, tmpDir) {
            basePath = tmpDir;

            done();
        });
    },

    tearDown: function (done) {
        rmdirRecursive(basePath);

        done();
    },


    'Change fixture directives': function (t) {
        var fixtureFilePath = path.join(basePath, 'fixture.test.js'),
            initialFixtureCode = '//Yo\r\n /*Test*/ \r\n\r\n"@fixture oldname"\r\n\r\n' +
                '"@page oldpage"; ',
            expectedFixtureCode = '//Yo\r\n /*Test*/ "@fixture new\\"name" ' +
                '"@page http://newpage";\r\n' +
                '"@auth newLogin:newPass";\r\n  ';

        fs.writeFile(fixtureFilePath, initialFixtureCode, function () {
            var newDirectvies = {
                fixture: 'new"name',
                page: 'newpage',
                auth: 'newLogin:newPass'
            };

            FixtureCode.CodeGenerator.editDirectives(fixtureFilePath, newDirectvies, function (err) {
                t.ok(!err);

                var actualJs = fs.readFileSync(fixtureFilePath).toString();

                t.ok(testUtils.compareCode(expectedFixtureCode, actualJs));

                t.done();
            });
        });
    },

    'Edit existing @auth directive': function (t) {
        var fixtureFilePath = path.join(basePath, 'fixture.test.js'),
            initialFixtureCode = '"@fixture name";' +
                '"@page page";' +
                '"@auth myLogin:myPass";',
            expectedFixtureCode = '"@fixture name";' +
                '"@page page";' +
                '"@auth newLogin:newPass";';

        fs.writeFile(fixtureFilePath, initialFixtureCode, function () {
            FixtureCode.CodeGenerator.editDirectives(fixtureFilePath, {auth: 'newLogin:newPass'}, function (err) {
                t.ok(!err);

                var actualJs = fs.readFileSync(fixtureFilePath).toString();

                t.ok(testUtils.compareCode(expectedFixtureCode, actualJs));

                t.done();
            });
        });
    },

    'Create new @auth directive': function (t) {
        var fixtureFilePath = path.join(basePath, 'fixture.test.js'),
            initialFixtureCode = '//Test\n"@fixture name";' +
                '"@page page";',
            expectedFixtureCode = '//Test\n"@fixture name";' +
                '"@page page";\n' +
                '"@auth new\\"Login:newPass";\n';

        fs.writeFile(fixtureFilePath, initialFixtureCode, function () {
            FixtureCode.CodeGenerator.editDirectives(fixtureFilePath, {auth: 'new"Login:newPass'}, function (err) {
                t.ok(!err);

                var actualJs = fs.readFileSync(fixtureFilePath).toString();

                t.ok(testUtils.compareCode(expectedFixtureCode, actualJs));

                t.done();
            });
        });
    },

    'Remove existing @auth directive': function (t) {
        var fixtureFilePath = path.join(basePath, 'fixture.test.js'),
            initialFixtureCode = '"@fixture name";' +
                '"@page page";' +
                '"@auth newLogin:newPass";';
        var expectedFixtureCode = '"@fixture name";' +
            '"@page page";';

        fs.writeFile(fixtureFilePath, initialFixtureCode, function () {
            FixtureCode.CodeGenerator.editDirectives(fixtureFilePath, {auth: null}, function (err) {
                t.ok(!err);

                var actualJs = fs.readFileSync(fixtureFilePath).toString();

                t.ok(testUtils.compareCode(expectedFixtureCode, actualJs));

                t.done();
            });
        });
    },

    'Invalid fixture': function (t) {
        var fixtureFilePath = path.join(basePath, 'fixture.test.js'),
            fixtureCode = '"@fixtu123re oldname";' +
                '"@p1age oldpage";';

        fs.writeFile(fixtureFilePath, fixtureCode, function () {
            var newDirectvies = {
                fixture: 'newname',
                page: 'newpage'
            };

            FixtureCode.CodeGenerator.editDirectives(fixtureFilePath, newDirectvies, function (err) {
                t.strictEqual(err.code, FixtureCode.ErrCodes.INVALID_FILE_FORMAT);
                t.strictEqual(np(err.filename), np(fixtureFilePath));

                t.done();
            });
        });
    },

    'Invalid Js': function (t) {
        var fixtureFilePath = path.join(basePath, 'fixture.test.js'),
            fixtureCode = '"@fixtu123re oldname;' +
                '"@p1age oldpage"; ' +
                'flsd;ldfk@';

        fs.writeFile(fixtureFilePath, fixtureCode, function () {
            var newDirectvies = {
                fixture: 'newname',
                page: 'newpage'
            };

            FixtureCode.CodeGenerator.editDirectives(fixtureFilePath, newDirectvies, function (err) {
                t.strictEqual(err.code, FixtureCode.ErrCodes.JAVASCRIPT_PARSING_FAILED);
                t.strictEqual(np(err.filename), np(fixtureFilePath));

                t.done();
            });
        });
    },

    'Error fs IO operation': function (t) {
        var newFixtureName = "testNewFixtureName",
            dirName = "testDir",
            invalidPath = path.join(basePath, dirName, newFixtureName + '.test.js');

        FixtureCode.CodeGenerator.editDirectives(invalidPath, {fixture: newFixtureName}, function (err) {
            t.strictEqual(err.code, FixtureCode.ErrCodes.READ_FILE_FAILED);
            t.done();
        });
    }
};

exports['FixtureCode.CodeGenerator.addTest()'] = {
    setUp: function (done) {
        tmp.dir(function (err, tmpDir) {
            basePath = tmpDir;

            done();
        });
    },

    tearDown: function (done) {
        rmdirRecursive(basePath);

        done();
    },

    'Add test code': function (t) {
        var stepsInfo = [
            {
                name: 'Click element 1',
                action: 'click',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\")"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -103,
                            offsetY: -2
                        }
                    }
                ]
            },

            {
                name: 'RClick element2',
                action: 'rclick',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\")"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -103,
                            offsetY: -2
                        }
                    }
                ]
            },

            {
                name: 'DblClick element3',
                action: 'dblclick',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "(function(){\n   var ancestorWithId = $(\"#DemoArea\");\n   return ancestorWithId.find(\"*\").filter(function(){ \n      var ownText = $(this).clone().children().remove().end().text();\n      ownText = $.trim(ownText.replace(/[^a-z 0-9]+/gi,\"\"));\n      return ownText === \"Description\";\n   });\n})"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -258,
                            offsetY: -15
                        }
                    }
                ]
            },

            {
                name: 'Drag element4',
                action: 'drag',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#elemId\")"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -8,
                            offsetY: 3
                        }
                    },
                    {x: 883},
                    {y: 383}
                ]
            },

            {
                name: 'Type text into element5',
                action: 'type',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#someTestElemId\")"}
                ],
                actionArgs: [
                    { text: 'test' }
                ]
            },

            {
                name: 'Check element5 value',
                isAssertion: true,
                blocks: [
                    {
                        assertions: [
                            {
                                operator: 'eq',
                                arguments: [ '$(\"#someTestElemId\").get(0).value', '\'test\'' ],
                                message: 'check element5 value'
                            }
                        ]
                    }
                ]
            },

            {
                name: 'Press button',
                action: 'press',
                actionArgs: [
                    { keys: 'tab' }
                ]
            },

            {
                name: 'Hover element 6',
                action: 'hover',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "(function(){\n   var ancestorWithId = $(\"#someId\");\n   return ancestorWithId.find(\"*\").filter(function(){ \n      var ownText = $(this).clone().children().remove().end().text();\n      ownText = $.trim(ownText.replace(/[^a-z 0-9]+/gi,\"\"));\n      return ownText === \"Documents\";\n   });\n})"}
                ],
                actionArgs: []
            },

            {
                name: 'Check element6 borderWidth',
                isAssertion: true,
                blocks: [
                    {
                        assertions: [
                            {
                                operator: 'ok',
                                arguments: [ "(function(){\n   var ancestorWithId = $(\"#someId\");\n   return ancestorWithId.find(\"*\").filter(function(){ \n      var ownText = $(this).clone().children().remove().end().text();\n      ownText = $.trim(ownText.replace(/[^a-z 0-9]+/gi,\"\"));\n      return ownText === \"Documents\";\n   });\n})().eq(0).css('borderWidth') === \"5px\"" ]
                            }
                        ]
                    }
                ]
            },

            {
                name: 'Step name',
                action: 'wait',
                actionArgs: [
                    { ms: 407 }
                ]
            },

            {
                name: 'Click element in iFrame',
                action: 'click',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\")"}
                ],
                currentIFrameSelectorIndex: 0,
                iFrameSelectors: [
                    { id: 'id', selector: "$(\"#iframe1\")"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -103,
                            offsetY: -2
                        }
                    }
                ]
            },

            {
                name: 'Assert',
                isAssertion: true,
                blocks: [
                    {
                        iFrameSelector: "$(\".iframe\")",
                        assertions: [
                            {
                                operator: 'ok',
                                arguments: ['true'],
                                message: 'message1'
                            }
                        ]
                    },

                    {
                        assertions: [
                            {
                                operator: 'notOk',
                                arguments: ['false'],
                                message: 'message2'
                            },
                            {
                                operator: 'ok',
                                arguments: ['true'],
                                message: 'message3'
                            }
                        ]
                    }
                ]
            }
        ];

        var srcPath = path.join(basePath, 'src.test.js');
        copyFile('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/src.test.js', srcPath);

        FixtureCode.CodeGenerator.addTest(srcPath, 'Testy test =)', stepsInfo, function (err) {
            t.ok(!err);

            var actualCode = fs.readFileSync(srcPath).toString(),
                expectedCode = fs.readFileSync('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/expected.test.js').toString();

            t.ok(testUtils.compareCode(actualCode, expectedCode));
            t.done();
        });
    },

    'Add test with duplicate test name': function (t) {
        var stepsInfo = [
            {
                name: 'Click element 1',
                action: 'click',
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\")"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -103,
                            offsetY: -2
                        }
                    }
                ]
            }
        ];
        var srcPath = path.join(basePath, 'src.test.js');
        copyFile('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/src.test.js', srcPath);

        var expectedJs = fs.readFileSync('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/src.test.js', 'UTF8');
        expectedJs = expectedJs.replace(TRIM_REG_EXP, '');

        FixtureCode.CodeGenerator.addTest(srcPath, 'My first test', stepsInfo, function (err) {
            var res = fs.readFileSync(srcPath, 'UTF8');
            res = res.replace(TRIM_REG_EXP, '');
            t.strictEqual(res, expectedJs);
            t.ok(err.code, FixtureCode.ErrCodes.TEST_NAME_CHANGED_TO_ALREADY_USED);
            t.done();
        });
    },

    'Corrupted fixture': function (t) {
        var fixtureFiles = [
            path.join(basePath, '1.test.js'),
            path.join(basePath, '2.test.js'),
            path.join(basePath, '3.test.js')
        ];

        copyFile('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/invalid_fixture1.test.js', fixtureFiles[0]);
        copyFile('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/invalid_fixture2.test.js', fixtureFiles[1]);
        copyFile('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/invalid_fixture3.test.js', fixtureFiles[2]);

        async.forEach(fixtureFiles, function (filePath, callback) {
            FixtureCode.CodeGenerator.addTest(filePath, 'Testy test =)', [], function (err) {
                t.strictEqual(err.code, FixtureCode.ErrCodes.INVALID_FILE_FORMAT);
                t.strictEqual(np(err.filename), np(filePath));

                callback();
            });
        }, function () {
            t.done();
        });
    },

    'Invalid JS': function (t) {
        var srcPath = path.join(basePath, 'invalid_js.test.js');

        copyFile('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/invalid_js.test.js', srcPath);

        FixtureCode.CodeGenerator.addTest(srcPath, 'Testy test =)', [], function (err) {
            t.strictEqual(err.code, FixtureCode.ErrCodes.JAVASCRIPT_PARSING_FAILED);
            t.strictEqual(np(err.filename), np(srcPath));

            t.done();
        });
    },

    'Corrupted selector': function (t) {
        var stepsInfo = [
            {
                name: 'Click element 1',
                action: 'click',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\"; @dklkfl cx||"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -103,
                            offsetY: -2
                        }
                    }
                ]
            }
        ];

        var srcPath = path.join(basePath, 'src.test.js');
        copyFile('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/src.test.js', srcPath);

        FixtureCode.CodeGenerator.addTest(srcPath, 'Testy test =)', stepsInfo, function (err) {
            t.strictEqual(err.code, FixtureCode.ErrCodes.ELEMENT_SELECTOR_PARSING_FAILED);
            t.ok(err.parserErr);

            t.done();
        });
    },

    'Corrupted assertion argument': function (t) {
        var stepsInfo = [
            {
                name: 'Check element1 value',
                isAssertion: true,
                blocks: [
                    {
                        assertions: [
                            {
                                operator: 'eq',
                                arguments: [
                                    'document.getElementById("someId).value', '"value"'
                                ]
                            }
                        ]
                    }
                ]
            }
        ];

        var srcPath = path.join(basePath, 'src.test.js');
        copyFile('test/nodeunit/core/data/fixture_code_code_generator/add_test_code/src.test.js', srcPath);

        FixtureCode.CodeGenerator.addTest(srcPath, 'Testy test =)', stepsInfo, function (err) {
            t.strictEqual(err.code, FixtureCode.ErrCodes.ASSERTION_ARGUMENT_PARSING_FAILED);
            t.ok(err.parserErr);

            t.done();
        });
    },

    'T113094 - Comment is broken after adding new test, if it\'s standing after test definition': function (t) {
        var stepsInfo = [
            {
                name: 'Click element 1',
                action: 'click',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\")"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -103,
                            offsetY: -2
                        }
                    }
                ]
            }
        ];

        var srcPath = path.join(basePath, 'src.test.js');
        copyFile('test/nodeunit/core/data/fixture_code_code_generator/T113094/src.test.js', srcPath);

        FixtureCode.CodeGenerator.addTest(srcPath, 'Testy test =)', stepsInfo, function (err) {
            t.ok(!err);

            var actualCode = fs.readFileSync(srcPath).toString(),
                expectedCode = fs.readFileSync('test/nodeunit/core/data/fixture_code_code_generator/T113094/expected.test.js').toString();

            t.ok(testUtils.compareCode(actualCode, expectedCode));
            t.done();
        });
    }
};

exports['FixtureCode.CodeGenerator.overwriteTest()'] = {
    setUp: function (done) {
        tmp.dir(function (err, tmpDir) {
            basePath = tmpDir;

            done();
        });
    },

    tearDown: function (done) {
        rmdirRecursive(basePath);

        done();
    },

    'Overwrite test': function (t) {
        var stepsInfo = [
            {
                name: 'Click element 1',
                action: 'click',
                elemIdentifier: 'element1',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink6\")"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -103,
                            offsetY: -2
                        }
                    }
                ]
            }
        ];

        var srcPath = path.join(basePath, 'src.test.js');
        copyFile('test/nodeunit/core/data/fixture_code_code_generator/overwrite_test/src.test.js', srcPath);

        FixtureCode.CodeGenerator.overwriteTest(srcPath, 'My first test', stepsInfo, function (err) {
            t.ok(!err);

            var actualCode = fs.readFileSync(srcPath).toString(),
                expectedCode = fs.readFileSync('test/nodeunit/core/data/fixture_code_code_generator/overwrite_test/expected.test.js').toString();

            t.ok(testUtils.compareCode(actualCode, expectedCode));
            t.done();
        });
    },

    'Nonexistent test': function (t) {
        var stepsInfo = [
            {
                name: 'Click element 2',
                action: 'click',
                elemIdentifier: 'element1',
                currentSelectorIndex: 0,
                selectors: [
                    { id: 'id', selector: "$(\"#titleLink2\")"}
                ],
                actionArgs: [
                    {
                        options: {
                            ctrl: false,
                            alt: false,
                            shift: false,
                            meta: false,
                            offsetX: -103,
                            offsetY: -2
                        }
                    }
                ]
            }
        ];

        var srcPath = path.join(basePath, 'src.test.js');
        copyFile('test/nodeunit/core/data/fixture_code_code_generator/overwrite_test/src.test.js', srcPath);

        FixtureCode.CodeGenerator.overwriteTest(srcPath, 'YoYo', stepsInfo, function (err) {
            t.ok(!err);

            var actualCode = fs.readFileSync(srcPath).toString(),
                expectedCode = fs.readFileSync('test/nodeunit/core/data/fixture_code_code_generator/overwrite_test/nonexistent_expected.test.js').toString();

            t.ok(testUtils.compareCode(actualCode, expectedCode));
            t.done();
        });
    }
};

exports['FixtureCode.CodeGenerator.deleteTest()'] = {
    setUp: function (done) {
        tmp.dir(function (err, tmpDir) {
            basePath = tmpDir;

            done();
        });
    },

    tearDown: function (done) {
        rmdirRecursive(basePath);

        done();
    },

    'Delete test': function (t) {
        var testName = 'First test',
            srcPath = path.join(basePath, 'src.test.js');

        copyFile('test/nodeunit/core/data/fixture_code_code_generator/delete_test/src.test.js', srcPath);
        var expectedJs = fs.readFileSync('test/nodeunit/core/data/fixture_code_code_generator/delete_test/success.test.js', 'UTF8');
        expectedJs = expectedJs.replace(TRIM_REG_EXP, '');

        FixtureCode.CodeGenerator.deleteTest(srcPath, testName, function (err) {
            var res = fs.readFileSync(srcPath, 'UTF8');

            res = res.replace(TRIM_REG_EXP, '');

            t.ok(!err);
            t.strictEqual(res, expectedJs);

            t.done();
        });
    },

    'Path is not exists': function (t) {
        var testName = 'First test',
            srcPath = path.join(basePath, 'src.test.js');

        FixtureCode.CodeGenerator.deleteTest(srcPath, testName, function (err) {
            t.strictEqual(err.code, FixtureCode.ErrCodes.READ_FILE_FAILED);
            t.strictEqual(err.filename, srcPath);

            t.done();
        });
    },

    'Test is not found': function (t) {
        var testName = 'I\'m not exists',
            srcPath = path.join(basePath, 'src.test.js');

        copyFile('test/nodeunit/core/data/fixture_code_code_generator/delete_test/src.test.js', srcPath);

        FixtureCode.CodeGenerator.deleteTest(srcPath, testName, function (err) {
            t.strictEqual(err.code, FixtureCode.ErrCodes.TEST_IS_NOT_FOUND);
            t.strictEqual(err.testName, testName);

            t.done();
        });
    }
};

exports['FixtureCode.CodeGenerator.renameTest()'] = {
    setUp: function (done) {
        tmp.dir(function (err, tmpDir) {
            basePath = tmpDir;

            done();
        });
    },

    tearDown: function (done) {
        rmdirRecursive(basePath);

        done();
    },

    'Test renamed': function (t) {
        var testName = 'Second test',
            newName = 'Test with new" name',
            srcPath = path.join(basePath, 'src.test.js');

        copyFile('test/nodeunit/core/data/fixture_code_code_generator/rename_test/src.test.js', srcPath);
        var expectedJs = fs.readFileSync('test/nodeunit/core/data/fixture_code_code_generator/rename_test/success.test.js', 'UTF8');
        expectedJs = expectedJs.replace(TRIM_REG_EXP, '');

        FixtureCode.CodeGenerator.renameTest(srcPath, testName, newName, function (err) {
            var res = fs.readFileSync(srcPath, 'UTF8');

            res = res.replace(TRIM_REG_EXP, '');

            t.ok(!err);
            t.strictEqual(res, expectedJs);

            t.done();
        });
    },

    'Path is not exists': function (t) {
        var testName = 'First test',
            newName = '',
            srcPath = path.join(basePath, 'src.test.js');

        FixtureCode.CodeGenerator.renameTest(srcPath, testName, newName, function (err) {
            t.strictEqual(err.code, FixtureCode.ErrCodes.READ_FILE_FAILED);
            t.strictEqual(err.filename, srcPath);

            t.done();
        });
    },

    'Test is not found': function (t) {
        var testName = 'I\'m not exists',
            newName = '',
            srcPath = path.join(basePath, 'src.test.js');

        copyFile('test/nodeunit/core/data/fixture_code_code_generator/rename_test/src.test.js', srcPath);

        FixtureCode.CodeGenerator.renameTest(srcPath, testName, newName, function (err) {
            t.strictEqual(err.code, FixtureCode.ErrCodes.TEST_IS_NOT_FOUND);
            t.strictEqual(err.testName, testName);

            t.done();
        });
    },

    'Test already exists': function (t) {
        var testName = 'First test',
            newName = 'Second test',
            srcPath = path.join(basePath, 'src.test.js');

        copyFile('test/nodeunit/core/data/fixture_code_code_generator/rename_test/src.test.js', srcPath);
        var expectedJs = fs.readFileSync('test/nodeunit/core/data/fixture_code_code_generator/rename_test/src.test.js', 'UTF8');
        expectedJs = expectedJs.replace(TRIM_REG_EXP, '');

        FixtureCode.CodeGenerator.renameTest(srcPath, testName, newName, function (err) {
            var res = fs.readFileSync(srcPath, 'UTF8');

            res = res.replace(TRIM_REG_EXP, '');

            t.ok(err.code, FixtureCode.ErrCodes.TEST_NAME_CHANGED_TO_ALREADY_USED);
            t.strictEqual(res, expectedJs);

            t.done();
        });
    }
};