var fs = require('fs'),
    fsUtil = require('../../../../lib/fs_util'),
    path = require('path'),
    tmp = require('tmp'),
    testUtils = require('../../test_utils');

var basePath = null,
    rmdirRecursive = testUtils.rmdirRecursive,
    copyDirRecursiveSync = testUtils.copyDirRecursiveSync;

exports['fsUtil'] = {
    setUp: function (done) {
        tmp.dir(function (err, tmpDir) {
            basePath = tmpDir;

            done();
        });
    },

    tearDown: function (done) {
        rmdirRecursive(basePath);

        done();
    },

    'getFSListRecursive - get list': function(t) {
        copyDirRecursiveSync(path.join(__dirname, '../data/fs_util/testDir'), basePath);

        var expectedList = [
            '',
            '\\test.js',
            '\\test1',
            '\\test1\\test3',
            '\\test1\\test3\\test.js',
            '\\test1\\test4',
            '\\test1\\test4\\test.js',
            '\\test1\\test_config.json',
            '\\test2',
            '\\test2\\test.js'
        ];

        fsUtil.getFSListRecursive(basePath, function(err, list) {
            var actualList = [];

            list.forEach(function(elmPath){
                actualList.push(elmPath.replace(basePath, ''));
            });

            t.ok(!err);
            t.strictEqual(JSON.stringify(expectedList), JSON.stringify(actualList));
            t.done();
        });
    },

    'getFSListRecursive - directory not exists': function(t) {
        fsUtil.getFSListRecursive('notExists', function(err, list) {
            t.ok(err);
            t.ok(err.toString().indexOf('Error: ENOENT, readdir') >= -1);
            t.ok(!list);
            t.done();
        });
    },

    'removeFSList - remove files': function(t) {
        var testDir = path.join(basePath, 'test');

        fs.mkdirSync(testDir);
        copyDirRecursiveSync(path.join(__dirname, '../data/fs_util/testDir'), testDir);
        fsUtil.rmdirRecursive(testDir, function(err, elm){
            t.ok(!err);
            t.ok(!elm);
            t.ok(!fs.existsSync(testDir));
            t.done();
        });
    },

    'removeFSList - directory not exists': function(t) {
        var testDir = path.join(basePath, 'notExists');

        fsUtil.rmdirRecursive(testDir, function(err, elm){
            t.ok(err);
            t.ok(err.toString().indexOf('Error: ENOENT, readdir') >= -1);
            t.strictEqual(elm, testDir);
            t.done();
        });
    },

    'copyDirRecursiveSync - copy dir': function(t) {
        fsUtil.copyDirRecursiveSync(path.join(__dirname, '../data/fs_util/testDir'), basePath);

        var fsList = [
            '\\test.js',
            '\\test1',
            '\\test1\\test3',
            '\\test1\\test3\\test.js',
            '\\test1\\test4',
            '\\test1\\test4\\test.js',
            '\\test1\\test_config.json',
            '\\test2',
            '\\test2\\test.js'
        ];

        fsList.forEach(function(elm){
            t.ok(fs.existsSync(path.join(basePath, elm)));
        });

        t.done();
    },

    'mkdirRecursiveSync - make dir': function(t) {
        var dirPath1 = path.join(basePath, 'test'),
            dirPath2 = path.join(basePath, 'level1', 'level2', 'level3');

        fsUtil.mkdirRecursiveSync(dirPath1);
        fsUtil.mkdirRecursiveSync(dirPath2);

        t.ok(fs.existsSync(dirPath1));
        t.ok(fs.existsSync(dirPath2));

        t.done();
    }
};