var util = require('util'),
    fs = require('fs'),
    path = require('path'),
    tmp = require('tmp'),
    Reporter = require('../../../../../lib/test_runner/reporter'),
    testUtils = require('../../../test_utils'),
    ERR = require('../../../../../lib/server/server_errs');

var basePath = null;

exports['Reporter.taskAdded'] = function (t) {
    var reporter = new Reporter('', false),
        workerNames = ['wrkr1', 'wrkr2'],
        browserVersions = {
            'wrkr1': '0.0.0',
            'wrkr2': '0.0.0'
        },
        taskUid = 'testTaskUid',
        taskName = 'Test task name';

    reporter.taskAdded(taskUid, taskName, 3, workerNames, browserVersions, false, {});

    t.strictEqual(reporter.taskReports.length, 1);

    var report = reporter.getTaskReportByUid('testTaskUid');

    t.strictEqual(report.uid, taskUid);
    t.strictEqual(report.name, taskName);
    t.strictEqual(report.status, 'pending');
    t.ok(!report.startedAt);
    t.ok(!report.completedAt);
    t.strictEqual(report.time, 0);
    t.deepEqual(report.workerNames, workerNames);
    t.strictEqual(report.testCount, 3);
    t.strictEqual(report.failed, 0);
    t.ok(report.testErrReports);
    t.ok(report.passedTests);
    t.done();
};

exports['Reporter.taskStarted'] = function (t) {
    var reporter = new Reporter('', false),
        workerNames = ['wrkr1', 'wrkr2'],
        browserVersions = {
            'wrkr1': '0.0.0',
            'wrkr2': '0.0.0'
        },
        taskUid = 'testTaskUid',
        taskName = 'Test task name';

    reporter.taskAdded(taskUid, taskName, 3, workerNames, browserVersions, false, {});

    var report = reporter.getTaskReportByUid('testTaskUid');

    reporter.taskStarted(taskUid);

    t.strictEqual(report.uid, taskUid);
    t.strictEqual(report.name, taskName);
    t.strictEqual(report.status, 'running');
    t.ok(report.startedAt);
    t.ok(!report.completedAt);
    t.strictEqual(report.time, 0);
    t.deepEqual(report.workerNames, workerNames);
    t.strictEqual(report.testCount, 3);
    t.strictEqual(report.failed, 0);
    t.ok(report.testErrReports);
    t.ok(report.passedTests);
    t.done();
};

exports['Reporter.testRunComplete'] = function (t) {
    var reporter = new Reporter('', false),
        testRun = {
            fixture: {
                name: "test",
                path: []
            },
            taskUid: 'task1',
            test: { uid: 'someTest'},
            errs: []
        },
        taskUpdated = false;

    reporter.on('taskUpdated', function () {
        taskUpdated = true;
    });

    reporter.taskAdded('task1', 'task1', 1, ['wrkr1', 'wrkr2'], {'wrkr1':'0.0.0', 'wrkr2':'0.0.0'}, false, {});

    var report = reporter.getTaskReportByUid('task1');

    reporter.testRunCompleted(testRun);

    t.strictEqual(report.passed, 0);

    reporter.testRunCompleted(testRun);

    t.strictEqual(report.passed, 1);

    t.ok(taskUpdated);
    t.done();

    taskUpdated = false;
};

exports['Reporter.testRunComplete - report unstable'] = function (t) {
    var reporter = new Reporter('', false),
        testRun1 = {
            taskUid: 'task1',
            test: {
                uid: 'testUid1',
                name: 'testName1'
            },
            fixture: {
                name: 'fixtureName',
                path: ['fixturePath']
            },
            quarantine: {
                succeeded: 2,
                failed: 1,
                runErrs: {
                    testRun1Errs: [
                        {
                            err: {code: ERR.WORKER_POOL_FAILED_TO_CREATE_WORKER_IN_BROWSER_WINDOW, browserName: 'browser'},
                            possibleCauses: [
                                {err: {code: ERR.WORKER_POOL_FAILED_TO_CREATE_WORKER_IN_BROWSER_WINDOW, browserName: 'browser'}, type: 'causeType'}
                            ]
                        }
                    ]}
            },
            errs: [],
            workerName: 'wrkr1'
        },
        testRun2 = {
            taskUid: 'task1',
            test: {
                uid: 'testUid2',
                name: 'testName2'
            },
            fixture: {
                name: 'fixtureName',
                path: ['fixturePath']
            },
            quarantine: {
                succeeded: 1,
                failed: 0
            },
            errs: [],
            workerName: 'wrkr2'
        },
        testRun3 = {
            taskUid: 'task1',
            test: {
                uid: 'testUid3',
                name: 'testName3'
            },
            fixture: {
                name: 'fixtureName',
                path: ['fixturePath']
            },
            quarantine: {
                succeeded: 0,
                failed: 3
            },
            errs: [],
            workerName: 'wrkr2'
        },
        testRun4 = {
            taskUid: 'task1',
            test: {
                uid: 'testUid1',
                name: 'testName1'
            },
            fixture: {
                name: 'fixtureName',
                path: ['fixturePath']
            },
            quarantine: {
                succeeded: 2,
                failed: 1,
                runErrs: {
                    testRun1Errs: [
                        {
                            err: {code: ERR.WORKER_POOL_FAILED_TO_CREATE_WORKER_IN_BROWSER_WINDOW, browserName: 'browser'},
                            possibleCauses: [
                                {err: {code: ERR.WORKER_POOL_FAILED_TO_CREATE_WORKER_IN_BROWSER_WINDOW, browserName: 'browser'}, type: 'causeType'}
                            ]
                        }
                    ]}
            },
            errs: [],
            workerName: 'wrkr2'
        };

    reporter.taskAdded('task1', 'task1', 1, ['wrkr1', 'wrkr2'], {'wrkr1': '0.0.0', 'wrkr2':'0.0.0'}, true, {});

    reporter.testRunCompleted(testRun1);
    reporter.testRunCompleted(testRun2);
    reporter.testRunCompleted(testRun3);
    reporter.testRunCompleted(testRun4);

    var report = reporter.getTaskReportByUid('task1');
    t.strictEqual(Object.keys(report.unstableTests).length, 1);

    var unstableReport = report.unstableTests['testUid1'];

    t.strictEqual(unstableReport.name, 'testName1');
    t.strictEqual(unstableReport.fixtureName, 'fixtureName');
    t.strictEqual(unstableReport.fixturePath, 'fixturePath');
    t.strictEqual(unstableReport.workerNames.length, 2);
    t.ok(unstableReport.workerNames.indexOf('wrkr1') > -1);
    t.ok(unstableReport.workerNames.indexOf('wrkr2') > -1);

    t.done();
};

exports['Reporter.testRunComplete - with errors'] = function (t) {
    var reporter = new Reporter('', false),
        testRun = {
            taskUid: 'task1',
            workerName: 'wrkr1',
            test: {
                uid: 'testUid',
                name: 'testName'
            },
            fixture: {
                name: 'fixtureName',
                path: ['fixturePath']
            },
            errs: []
        }, taskUpdated = false;

    reporter.on('taskUpdated', function () {
        taskUpdated = true;
    });

    reporter.taskAdded('task1', 'task1', 1, ['wrkr1', 'wrkr2'], {'wrkr1': '0.0.0', 'wrkr2':'0.0.0'}, false, {});
    testRun.errs.push({
        err: {code: ERR.WORKER_POOL_FAILED_TO_CREATE_WORKER_IN_BROWSER_WINDOW, browserName: 'browser'},
        possibleCauses: [
            {err: {code: ERR.WORKER_POOL_FAILED_TO_CREATE_WORKER_IN_BROWSER_WINDOW, browserName: 'browser'}, type: 'causeType'}
        ]
    });
    reporter.testRunCompleted(testRun);

    var report = reporter.getTaskReportByUid('task1');

    t.strictEqual(report.passed, 0);

    reporter.testRunCompleted(testRun);

    t.strictEqual(report.passed, 0);

    t.ok(taskUpdated);
    t.done();
};

exports['Reporter.taskComplete'] = function (t) {
    var reporter = new Reporter('', false),
        workerNames = ['wrkr1', 'wrkr2'],
        browserVersions = {
            'wrkr1': '0.0.0',
            'wrkr2': '0.0.0'
        },
        taskUid = 'testTaskUid',
        taskName = 'Test task name';

    reporter.taskAdded(taskUid, taskName, 3, workerNames, browserVersions, false, {});
    reporter.taskStarted(taskUid);

    var report = reporter.getTaskReportByUid('testTaskUid');

    reporter.taskCompleted(taskUid);

    t.strictEqual(report.uid, taskUid);
    t.strictEqual(report.name, taskName);
    t.strictEqual(report.status, 'succeeded');
    t.ok(report.startedAt);
    t.ok(report.completedAt);
    t.deepEqual(report.workerNames, workerNames);
    t.strictEqual(report.testCount, 3);
    t.strictEqual(report.failed, 0);
    t.ok(report.testErrReports);
    t.ok(report.passedTests);
    t.done();
};

exports['Reporter.getTaskReportByUid'] = function (t) {
    var reporter = new Reporter('', false);

    reporter.taskAdded('task1', 'task1', 1, ['wrkr'], {'wrkr': '0.0.0'}, false, {});
    reporter.taskAdded('task2', 'task2', 1, ['wrkr'], {'wrkr': '0.0.0'}, false, {});

    var report = reporter.getTaskReportByUid('task2');

    t.strictEqual(report.name, 'task2');

    report = reporter.getTaskReportByUid('unknownUid');

    t.ok(!report);

    t.done();
};

exports['Reporter - test run error'] = function (t) {
    var reporter = new Reporter('', false),
        testRun1 = {
            taskUid: 'task1',
            workerName: 'wrkr1',
            browserVersions: {
                'wrk1': '0.0.0'
            },
            test: {
                uid: 'testUid',
                name: 'testName'
            },
            fixture: {
                name: 'fixtureName',
                path: ['fixturePath']
            },
            errs: []
        },
        testRun2 = {
            taskUid: 'task1',
            workerName: 'wrkr2',
            browserVersions: {
                'wrk2': '0.0.0'
            },
            test: {
                uid: 'testUid',
                name: 'testName'
            },
            fixture: {
                name: 'fixtureName',
                path: ['fixturePath']
            },
            errs: []
        };

    reporter.taskAdded('task1', 'task1', 1, ['wrkr1', 'wrkr2'], {'wrk1': '0.0.0'}, false, {});

    var report = reporter.getTaskReportByUid('task1');

    testRun1.errs.push({code: ERR.WORKER_POOL_FAILED_TO_CREATE_WORKER_IN_BROWSER_WINDOW, browserName: 'browser'});
    testRun2.errs.push({code: ERR.WORKER_POOL_FAILED_TO_CREATE_WORKER_IN_BROWSER_WINDOW, browserName: 'browser'});

    reporter.testRunCompleted(testRun1);
    t.strictEqual(report.failed, 1);

    reporter.testRunCompleted(testRun2);
    t.strictEqual(report.failed, 1);

    var testErrReport = report.testErrReports['testUid'];

    t.strictEqual(testErrReport.name, 'testName');
    t.strictEqual(testErrReport.fixtureName, 'fixtureName');
    t.strictEqual(testErrReport.fixturePath, 'fixturePath');

    var errs = testErrReport.errs;

    t.strictEqual(errs.length, 2);
    t.strictEqual(errs[0].workerName, 'wrkr1');
    t.strictEqual(errs[1].workerName, 'wrkr2');
    t.ok(errs[0].msg);
    t.ok(errs[1].msg);

    t.done();
};

exports['Regression - B239267 - Does not match the number of passed and failed tests with the total number of tests'] = function (t) {
    var reporter = new Reporter('', false),
        testRun1 = {
            taskUid: 'task1',
            workerName: 'wrkr1',
            browserVersions: {
                'wrk1': '0.0.0'
            },
            test: {
                uid: 'testUid',
                name: 'testName'
            },
            fixture: {
                name: 'fixtureName',
                path: ['fixturePath']
            },
            errs: []
        },
        testRun2 = {
            taskUid: 'task2',
            workerName: 'wrkr2',
            browserVersions: {
                'wrk2': '0.0.0'
            },
            test: {
                uid: 'testUid',
                name: 'testName'
            },
            fixture: {
                name: 'fixtureName',
                path: ['fixturePath']
            },
            errs: []
        };

    reporter.taskAdded('task1', 'task1', 1, ['wrkr1'], { 'wrk1': '0.0.0'}, false, {});
    reporter.taskAdded('task2', 'task2', 1, ['wrkr2'], { 'wrk2': '0.0.0'}, false, {});

    reporter.testRunCompleted(testRun1);
    reporter.testRunCompleted(testRun2);

    var report1 = reporter.getTaskReportByUid('task1'),
        report2 = reporter.getTaskReportByUid('task2');

    t.strictEqual(report1.failed + report1.passed, report1.testCount);
    t.strictEqual(report2.failed + report2.passed, report2.testCount);

    t.done();
};

exports['Reporter - get completed taskReports'] = function(t){
    var reporter = new Reporter('', false),
        workerNames = ['wrkr1', 'wrkr2'],
        browserVersions = {
            'wrkr1': '0.0.0',
            'wrkr2': '0.0.0'
        },
        task1Uid = 'testTask1Uid',
        task1Name = 'Test task 1 name',
        task2Name = 'Test task 2 name',
        task2Uid = 'testTask2Uid';

    reporter.taskAdded(task1Uid, task1Name, 3, workerNames, browserVersions, false, {});
    reporter.taskStarted(task1Uid);

    reporter.taskAdded(task2Uid, task2Name, 3, workerNames, browserVersions, false, {});
    reporter.taskStarted(task2Uid);

    reporter.taskCompleted(task1Uid);
    var taskReports = reporter.getCompletedTaskReports(),
        expectedReport = reporter.getTaskReportByUid('testTask1Uid');

    t.strictEqual(taskReports.length, 1);
    t.strictEqual(expectedReport.uid, taskReports[0].uid);

    t.done();
};


exports['Reporter - remove completed taskReports'] = function(t){
    var reporter = new Reporter('', false),
        workerNames = ['wrkr1', 'wrkr2'],
        browserVersions = {
            'wrkr1': '0.0.0',
            'wrkr2': '0.0.0'
        },
        task1Uid = 'testTask1Uid',
        task1Name = 'Test task 1 name',
        task2Name = 'Test task 2 name',
        task2Uid = 'testTask2Uid';

    reporter.taskAdded(task1Uid, task1Name, 3, workerNames, browserVersions, false, {});
    reporter.taskStarted(task1Uid);

    reporter.taskAdded(task2Uid, task2Name, 3, workerNames, browserVersions, false, {});
    reporter.taskStarted(task2Uid);

    reporter.taskCompleted(task1Uid);
    reporter.removeCompletedTaskReport(task1Uid);

    var report1 = reporter.getTaskReportByUid('testTask1Uid'),
        report2 = reporter.getTaskReportByUid('testTask2Uid');

    t.ok(!report1);

    t.strictEqual(report2.uid, task2Uid);
    t.strictEqual(report2.name, task2Name);
    t.strictEqual(report2.status, 'running');
    t.ok(report2.startedAt);
    t.ok(!report2.completedAt);
    t.deepEqual(report2.workerNames, workerNames);
    t.strictEqual(report2.testCount, 3);
    t.strictEqual(report2.failed, 0);
    t.ok(report2.testErrReports);
    t.ok(report2.passedTests);

    t.done();
};

exports['Reporter - remove all completed taskReports'] = function(t){
    var reporter = new Reporter('', false),
        workerNames = ['wrkr1', 'wrkr2'],
        browserVersions = {
            'wrkr1': '0.0.0',
            'wrkr2': '0.0.0'
        },
        task1Uid = 'testTask1Uid',
        task1Name = 'Test task 1 name',
        task2Name = 'Test task 2 name',
        task2Uid = 'testTask2Uid';

    reporter.taskAdded(task1Uid, task1Name, 3, workerNames, browserVersions, false, {});
    reporter.taskStarted(task1Uid);

    reporter.taskAdded(task2Uid, task2Name, 3, workerNames, browserVersions, false, {});
    reporter.taskStarted(task2Uid);

    reporter.taskCompleted(task1Uid);
    reporter.removeAllCompletedTaskReports();

    var report1 = reporter.getTaskReportByUid('testTask1Uid'),
        report2 = reporter.getTaskReportByUid('testTask2Uid');

    t.ok(!report1);

    t.strictEqual(report2.uid, task2Uid);
    t.strictEqual(report2.name, task2Name);
    t.strictEqual(report2.status, 'running');
    t.ok(report2.startedAt);
    t.ok(!report2.completedAt);
    t.deepEqual(report2.workerNames, workerNames);
    t.strictEqual(report2.testCount, 3);
    t.strictEqual(report2.failed, 0);
    t.ok(report2.testErrReports);
    t.ok(report2.passedTests);

    t.done();
};

exports['Reporter - format report'] = {
    'nunit/junit/json format': function (t) {
        var reporter = new Reporter('', false),
            report = {
                uid: '842a36e2-a12c-48d8-a946-f878896ad53c',
                name: '"Root" directory tests',
                status: 'succeeded',
                startedAt: [ '07/29/2013', '12:36', 'PM' ],
                completedAt: [ '07/29/2013', '12:37', 'PM' ],
                workerNames: [ 'workerName', '2' ],
                testCount: 10,
                failed: 0,
                passed: 10,
                time: 123,
                browserVersions: {
                    'workerName': '0.0.0',
                    '2': '0.0.0'
                },
                testErrReports: {
                    'f60f7573-fa42-4f2e-b760-33883eff9a05': {
                        name: 'Click a label that does not exist',
                        fixtureName: 'My Page',
                        fixturePath: '',
                        time: 234,
                        errs: [
                            {
                                msg: 'Error on step "Click label": "click" action target does not contain DOM-elements.',
                                workerName: 'workerName',
                                possibleCauses: [
                                    { type: 'JS_ERROR',
                                        msg: 'Uncaught JavaScript error - "\'undefined_variable\' is undefined"' }
                                ]
                            }
                        ],
                        screenshots: [
                            {
                                workerName: "workerName",
                                isFailedStep: true,
                                filePath: "C:\\filename.txt",
                                stepName: "2.Assert text changed"
                            }
                        ]
                    }},
                unstableTests: {},
                passedTests: {
                    'f60f7573-fa42-4f2e-b760-33883eff9a06': {
                        fixtureName: 'My Fixture',
                        fixturePath: 'test/path/to/fixture',
                        name: 'someTest',
                        time: 345
                    }
                }
            },
            expectedDataNUnit = fs.readFileSync('./test/nodeunit/core/data/test_runner/reporter/nunit.xml', 'UTF8'),
            expectedDataJUnit = fs.readFileSync('./test/nodeunit/core/data/test_runner/reporter/junit.xml', 'UTF8'),
            expectedJSON = JSON.stringify(report);

        reporter.formatTaskReport(report, 'nunit', function(err, result) {
            t.ok(!err);
            t.ok(testUtils.compareCode(expectedDataNUnit, result));

            reporter.formatTaskReport(report, 'junit', function(err, result) {
                t.ok(!err);
                t.ok(testUtils.compareCode(expectedDataJUnit, result));

                reporter.formatTaskReport(report, 'json', function(err, result) {
                    t.ok(!err);
                    t.ok(testUtils.compareCode(expectedJSON, JSON.stringify(result)));

                    reporter.formatTaskReport(report, 'someValue', function(err, result) {
                        t.ok(!err);
                        t.ok(testUtils.compareCode(expectedJSON, JSON.stringify(result)));

                        t.done();
                    });
                });
            });
        });
    }
};

exports['Write report'] = {
    'Successful writing': function(t) {
        var reporter = new Reporter('', false);

        tmp.dir(function (err, tmpDir) {
            var reportPath = path.join(tmpDir, 'report.xml');

            reporter.saveReport(reportPath, "", function(err) {
                t.ok(!err);
                t.ok(fs.existsSync(reportPath));

                testUtils.rmdirRecursive(tmpDir);

                t.done();
            });
        });
    },

    'Report\'s directory for doesn\'t exists': function(t) {
        var reporter = new Reporter('', false);

        tmp.dir(function (err, tmpDir) {
            var reportPath = path.join(tmpDir, 'nonExistingPath', 'report.xml');

            reporter.saveReport(reportPath, "", function(err) {
                t.strictEqual(err.code, ERR.REPORTER_WRITE_REPORT_FAILED);
                t.ok(!fs.existsSync(reportPath));

                t.done();
            });
        });
    }
};

exports['Reports storage'] = {
    setUp: function (done) {
        tmp.dir(function (err, tmpDir) {
            basePath = tmpDir;

            done();
        });
    },

    tearDown: function (done) {
        testUtils.rmdirRecursive(basePath);

        done();
    },

    'Save reporter state': function (t) {
        var reportsPath = path.join(basePath, 'reports'),
            reporter = new Reporter(reportsPath, true),
            report =
            {
                uid: "1234",
                t: 123
            };

            fs.mkdirSync(reportsPath);

            var expectedPath = path.join(reportsPath, report.uid, Reporter.REPORT_FILENAME);

            reporter.saveCompletedReport(report.uid, report, function () {
                var data = fs.readFileSync(expectedPath, 'UTF8');

                t.strictEqual(JSON.stringify(report), data);
                t.done();
            });
    },

    'Regression - Save reporter state - xml format': function(t) {
        var reportsPath = path.join(basePath, 'reports'),
            reporter = new Reporter(reportsPath, true),
            report = fs.readFileSync(path.join(__dirname, '../../data/test_runner/reporter/junit.xml')),
            uid = '12345';

        fs.mkdirSync(reportsPath);

        var expectedPath = path.join(reportsPath, uid, Reporter.REPORT_FILENAME);

        reporter.saveCompletedReport(uid, report, function () {
            var data = fs.readFileSync(expectedPath, 'UTF8');

            t.strictEqual(JSON.stringify(report), data);
            t.done();
        });
    },

    'Restore reporter state': function (t) {
        var reportsPath = path.join(basePath, 'reports');

        fs.mkdirSync(reportsPath);
        testUtils.copyDirRecursiveSync(path.join(__dirname, '../../data/test_runner/reporter/reports'), reportsPath);

        var reporter = new Reporter(reportsPath, true),
            expectedReport1 = JSON.parse(fs.readFileSync(path.join(reportsPath, 'uid1', Reporter.REPORT_FILENAME), 'UTF8')),
            expectedReport2 = JSON.parse(fs.readFileSync(path.join(reportsPath, 'uid2', Reporter.REPORT_FILENAME), 'UTF8')),
            expectedState = [expectedReport1, expectedReport2];

        t.deepEqual(expectedState, reporter.taskReports);
        t.done();
    },

    'removeReportFilesByUids - remove report': function(t) {
        var reportsPath = path.join(basePath, 'reports');

        fs.mkdirSync(reportsPath);
        testUtils.copyDirRecursiveSync(path.join(__dirname, '../../data/test_runner/reporter/reports'), reportsPath);

        var reporter = new Reporter(reportsPath, true);

        reporter.removeReportFilesByUids(['uid1'], function(err) {
            t.ok(!fs.existsSync(path.join(reportsPath, 'uid1', Reporter.REPORT_FILENAME)));
            t.ok(fs.existsSync(path.join(reportsPath, 'uid2', Reporter.REPORT_FILENAME)));
            t.done();
        });
    }
};