var fs = require('fs'),
    path = require('path'),
    tmp = require('tmp'),
    testUtils = require('../../../test_utils'),
    StatusIndication = require('../../../../../lib/vfs/status_indication');

//Utils
function loadJSON(filePath) {
    return JSON.parse(fs.readFileSync(filePath).toString());
}

var indication = null;

module.exports = {
    setUp: function (done) {
        indication = new StatusIndication();
        done();
    },

    'Update indication': function (t) {
        var latestUpdateInfos = null,
            testData = loadJSON('test/nodeunit/core/data/vfs/status_indication/update.json');

        indication.on('update', function (updates) {
            latestUpdateInfos = updates;
        });

        testData.forEach(function (data) {
            latestUpdateInfos = [];

            indication.update(data.testRun);

            t.strictEqual(latestUpdateInfos.length, data.expectedUpdates.length);

            latestUpdateInfos.forEach(function (info, idx) {
                t.deepEqual(info, data.expectedUpdates[idx], data.name);
            });
        });

        t.done();
    },

    'Sync with VFS': function (t) {
        var testData = loadJSON('test/nodeunit/core/data/vfs/status_indication/sync_with_vfs.json');

        testData.testRuns.forEach(function (testRun) {
            indication.update(testRun);
        });

        indication.syncWithVfs(testData.suite);

        var treeRoot = indication.root,
            deleteParentRecursive = function (node) {
                delete node.parent;

                if (node.children) {
                    Object.keys(node.children).forEach(function (childKey) {
                        deleteParentRecursive(node.children[childKey]);
                    })
                }
            };

        deleteParentRecursive(treeRoot);

        t.deepEqual(treeRoot, testData.expectedStateTree);
        t.done();
    },

    'Get dir status info': function (t) {
        var testData = loadJSON('test/nodeunit/core/data/vfs/status_indication/get_dir_status.json'),
            resultSorter = function (a, b) {
                if (a.name === b.name)
                    return 0;

                return a.name > b.name ? 1 : -1;
            };

        testData.testRuns.forEach(function (testRun) {
            indication.update(testRun);
        });

        testData.queries.forEach(function (query) {
            var actual = indication.getDirStatusInfo(query.dirPath),
                expected = query.expectedResult;

            actual = actual && actual.sort(resultSorter);
            expected = expected && expected.sort(resultSorter);

            t.deepEqual(actual, expected);
        });

        t.done();
    },

    'Load/Save': function (t) {
        tmp.dir(function (tmpErr, basePath) {
            var filePath = path.join(basePath, 'status_indication.json'),
                testData = loadJSON('test/nodeunit/core/data/vfs/status_indication/update.json'),
                prepareNodeForComparisonRecursive = function (node) {
                    //NOTE: remove toJSON function if we have one
                    if (node.toJSON)
                        delete node.toJSON;

                    //NOTE: get rid of the circular parent property so we will be able to perform
                    //test for the deep equality
                    if (node.children) {
                        Object.keys(node.children).forEach(function (childKey) {
                            var child = node.children[childKey];

                            child.parent = child.parent ? child.parent.name : 'root';
                            prepareNodeForComparisonRecursive(child);
                        });
                    }
                };

            testData.forEach(function (data) {
                indication.update(data.testRun);
            });

            var prevRoot = indication.root;

            indication.dumpToFile(filePath);
            indication.loadFromFile(filePath);


            prepareNodeForComparisonRecursive(indication.root);
            prepareNodeForComparisonRecursive(prevRoot);

            t.deepEqual(indication.root, prevRoot);

            testUtils.rmdirRecursive(basePath);
            t.done();
        });
    }
};