var fs = require('fs'),
    testUtils = require('../../../test_utils'),
    TestSuiteFactory = require('../../../../../lib/vfs/test_suite_factory'),
    FixtureCode = require('../../../../../lib/fixture_code'),
    ERR = require('../../../../../lib/server/server_errs');

var watchdog = null;

exports['TestSuiteFactory'] = {
    setUp: function (done) {
        watchdog = new testUtils.Watchdog();
        done();
    },

    tearDown: function (done) {
        watchdog.shrink();
        done();
    },

    'Create suite': function (t) {
        var fsEntries = [];

        TestSuiteFactory.createSuite('test/nodeunit/core/data/suite/build', function (res) {
            t.ok(!res.errs);

            var fixtureCount = 0,
                expectedFixtures = {
                    'Top level fixture': {
                        page: 'http://some_random.domain/index',
                        fileName: 'top.test.js',
                        path: [],
                        testName: 'Top level test',
                        tested: false
                    },
                    'Level1 no cfg fixture': {
                        page: 'http://some_random.domain/index1',
                        fileName: 'level1_no_cfg.test.js',
                        path: ['level1_no_cfg'],
                        testName: 'Level1 no cfg fixture test',
                        tested: false
                    },
                    'Level1 fixture1': {
                        page: 'http://some_random.domain/level1/index1',
                        fileName: 'level1_1.test.js',
                        path: ['level1'],
                        testName: 'Level1 fixture1 test',
                        tested: false
                    },
                    'Level1 fixture2': {
                        page: 'http://some_random.domain/level1/index2',
                        fileName: 'level1_2.test.js',
                        path: ['level1'],
                        testName: 'Level1 fixture2 test',
                        tested: false
                    },
                    'Level2 fixture': {
                        page: 'http://some_random.domain/level1/level2/index',
                        fileName: 'level2.test.js',
                        path: ['level1', 'level2'],
                        testName: 'Level2 fixture test',
                        tested: false
                    }
                };


            Object.keys(res.fixtures).forEach(function (fixtureUid) {
                var fixture = res.fixtures[fixtureUid],
                    expected = expectedFixtures[fixture.name];

                t.ok(!expected.tested);

                t.strictEqual(fixture.page, expected.page);
                t.strictEqual(fixture.fileName, expected.fileName);
                t.strictEqual(fixture.path.length, expected.path.length);

                for (var i = 0; i < fixture.path.length; i++)
                    t.strictEqual(fixture.path[i], expected.path[i]);

                t.strictEqual(fixture.tests.length, 1);

                var test = res.tests[fixture.tests[0]];

                t.strictEqual(test.uid, fixture.tests[0]);
                t.strictEqual(test.name, expected.testName);
                t.strictEqual(test.fixtureUid, fixtureUid);

                expected.tested = true;
                fixtureCount++;
            });

            t.strictEqual(fixtureCount, 5);
            t.strictEqual(fsEntries.length, 9);

            var assertEntryExists = function (entryPath, isDir) {
                var found = false;

                for (var i = 0; i < fsEntries.length; i++) {
                    if (fsEntries[i].path === entryPath && fsEntries[i].isDir === isDir) {
                        found = true;
                        break;
                    }
                }

                t.ok(found);
            };

            assertEntryExists('test/nodeunit/core/data/suite/build', true);
            assertEntryExists('test/nodeunit/core/data/suite/build/level1', true);
            assertEntryExists('test/nodeunit/core/data/suite/build/level1/level2', true);
            assertEntryExists('test/nodeunit/core/data/suite/build', true);
            assertEntryExists('test/nodeunit/core/data/suite/build/level1_no_cfg', true);

            assertEntryExists('test/nodeunit/core/data/suite/build/top.test.js', false);
            assertEntryExists('test/nodeunit/core/data/suite/build/level1/level1_1.test.js', false);
            assertEntryExists('test/nodeunit/core/data/suite/build/level1/level1_2.test.js', false);
            assertEntryExists('test/nodeunit/core/data/suite/build/level1/level2/level2.test.js', false);
            assertEntryExists('test/nodeunit/core/data/suite/build/level1_no_cfg/level1_no_cfg.test.js', false);

            t.done();
        }, function (entryPath, isDir) {
            fsEntries.push({path: testUtils.normalizePath(entryPath), isDir: isDir});
        });
    },

    'Create suite - with errors': function (t) {
        TestSuiteFactory.createSuite('test/nodeunit/core/data/suite/build_with_errors', function (res) {
            var fixtureCount = 0,
                testCount = 0,
                fixtureErrs = [];

            for (var i = 0, length = res.buildErrs.length; i < length; i++) {
                var uid = res.buildErrs[i].fixtureUid;

                if (uid && !fixtureErrs.hasOwnProperty(uid))
                    fixtureErrs[uid] = "";
            }

            Object.keys(res.fixtures).forEach(function (fixtureUid) {
                if (!fixtureErrs.hasOwnProperty(fixtureUid))
                    fixtureCount++;
            });

            Object.keys(res.tests).forEach(function (testUid) {
                testCount++;
            });

            t.strictEqual(fixtureCount, 1);
            t.strictEqual(testCount, 1);
            t.strictEqual(res.buildErrs.length, 3);
            t.strictEqual(res.buildErrs[0].code, ERR.SUITE_CFG_FILE_PARSING_FAILED); // TODO
            t.strictEqual(res.buildErrs[1].code, ERR.SUITE_CFG_MODULE_IS_NOT_ARRAY);
            t.strictEqual(res.buildErrs[2].code, FixtureCode.ErrCodes.TEST_STEP_IS_EMPTY);

            t.done();
        });
    },

    'Cfg validation': function (t) {
        TestSuiteFactory.createSuite('test/nodeunit/core/data/suite/cfg_validation', function (res) {
            t.strictEqual(res.buildErrs.length, 4);
            t.strictEqual(res.buildErrs[0].code, ERR.SUITE_CFG_PROPERTY_IS_NOT_STRING);
            t.strictEqual(res.buildErrs[1].code, ERR.SUITE_CFG_PROPERTY_IS_NOT_STRING);
            t.strictEqual(res.buildErrs[2].code, ERR.SUITE_CFG_MODULE_IS_NOT_ARRAY);
            t.strictEqual(res.buildErrs[3].code, ERR.SUITE_CFG_UNSUPPORTED_PROPERTY);

            t.done();
        });
    },

    'Create suite - with modules': function (t) {
        TestSuiteFactory.createSuite('test/nodeunit/core/data/suite/build_with_modules', function (suite) {
            var fixtures = [],
                testCount = 0;

            Object.keys(suite.fixtures).forEach(function (fixtureUid) {
                fixtures.push(suite.fixtures[fixtureUid]);
            });

            Object.keys(suite.tests).forEach(function (testUid) {
                testCount++;
            });

            t.strictEqual(fixtures.length, 1);
            t.strictEqual(testCount, 1);
            t.ok(!suite.buildErrs.length);


            var expectedSharedJs = fs.readFileSync('test/nodeunit/core/data/suite/build_with_modules/expected_require_js.js').toString(),
                requireJs = suite.requireJsMap[fixtures[0].requireJsMapKey];
            t.ok(testUtils.compareCode(requireJs, expectedSharedJs));

            t.done();
        });
    }
};

exports['Regression'] = {
    setUp: function (done) {
        watchdog = new testUtils.Watchdog();
        done();
    },

    tearDown: function (done) {
        watchdog.shrink();
        done();
    },

    "B236409 - Config files in UTF-8 encoding don't work": function (t) {
        TestSuiteFactory.createSuite('test/nodeunit/core/data/suite/B236409', function (res) {
            var fixtures = [];

            Object.keys(res.fixtures).forEach(function (fixtureUid) {
                fixtures.push(res.fixtures[fixtureUid]);
            });

            t.strictEqual(fixtures.length, 1);
            t.ok(!res.buildErrs.length);

            t.strictEqual(fixtures[0].page, 'Accessibility/B236409');

            t.done();
        });
    }
};