var DOMSandbox     = HammerheadClient.get('DOMSandbox'),
    IFrameSandbox  = HammerheadClient.get('DOMSandbox.IFrame'),
    InfoManager    = HammerheadClient.get('DOMSandbox.Upload.InfoManager'),
    Hammerhead     = HammerheadClient.get('Hammerhead'),
    PageProc       = HammerheadClient.get('Shared.PageProc'),
    SandboxUtil    = HammerheadClient.get('DOMSandbox.Util'),
    Settings       = HammerheadClient.get('Settings'),
    SharedConst    = TestCafeClient.get('Shared.Const'),
    SharedConstHH  = HammerheadClient.get('Shared.Const'),
    UrlUtil        = HammerheadClient.get('UrlUtil'),

    $              = Hammerhead.$,
    Util           = Hammerhead.Util,
    EventSimulator = Hammerhead.EventSandbox.Simulator,
    JSProcessor    = Hammerhead.JSProcessor,
    NativeMethods  = Hammerhead.NativeMethods,
    Transport      = Hammerhead.Transport;

Hammerhead.init();

var overrideDomMeth = window[Hammerhead.DOM_SANDBOX_OVERRIDE_DOM_METHOD_NAME];

window[Hammerhead.DOM_SANDBOX_OVERRIDE_DOM_METHOD_NAME] = function (el) {
    if (el)
        overrideDomMeth(el);
};

var handler = function (e) {
    if (e.iframe.id.indexOf('test') !== -1) {
        e.iframe.contentWindow.eval.call(e.iframe.contentWindow, [
            'HammerheadClient.define(\'Settings\', function (require, exports) {',
            '    exports.ORIGIN_URL_OBJ = { protocol: "https:", host: "example.com" };',
            '});',
            'HammerheadClient.get(\'Hammerhead\').init();'
        ].join(''));
    }
};

QUnit.testStart = function () {
    // 'window.open' method uses in the QUnit
    window.open       = NativeMethods.windowOpen;
    window.setTimeout = NativeMethods.setTimeout;
    IFrameSandbox.on(IFrameSandbox.IFRAME_READY_TO_INIT, handler);
    IFrameSandbox.off(IFrameSandbox.IFRAME_READY_TO_INIT, IFrameSandbox.iframeReadyToInitHandler);
};

QUnit.testDone = function () {
    IFrameSandbox.off(IFrameSandbox.IFRAME_READY_TO_INIT, handler);
};

asyncTest('Prevent "error" event during image reloading', function () {
    var storedGetOriginUrlObj = UrlUtil.getProxyUrl,
        errorEventRised       = false,
        realImageUrl          = '/testcafe_client/uisprite.png',
        fakeIamgeUrl          = 'fakeIamge.gif';

    UrlUtil.getProxyUrl = function () {
        return storedGetOriginUrlObj(realImageUrl);
    };

    var img = document.createElement('img');

    img.onerror = function () {
        errorEventRised = true;
    };

    img.onload = function () {
        ok(!errorEventRised);
        equal(img.src, storedGetOriginUrlObj(realImageUrl));

        $(img).remove();
        UrlUtil.getProxyUrl = storedGetOriginUrlObj;

        start();
    };

    eval(window[JSProcessor.PROCESS_SCRIPT_METH_NAME]('img.src="' + fakeIamgeUrl + '";'));
    document.body.appendChild(img);
});

asyncTest('Reload image through the proxy', function () {
    var storedGetOriginUrlObj = UrlUtil.getOriginUrlObj;

    UrlUtil.getOriginUrlObj = function () {
        return {
            protocol: location.protocol,
            host:     location.host
        };
    };

    var realImageUrl  = '/testcafe_client/uisprite.png',
        fakeIamgeUrl  = 'fakeIamge.gif',
        proxyIamgeUrl = UrlUtil.getProxyUrl(fakeIamgeUrl);

    var $img1 = $('<img src="' + realImageUrl + '">').appendTo('body'),
        $img2 = $('<img>').appendTo('body');

    $img2[0].onload = function () {
        $img2[0].onload = null;

        ok($img1[0].src.indexOf(realImageUrl) !== -1);
        ok($img2[0].src.indexOf(realImageUrl) !== -1);

        $img1.remove();
        $img2.remove();

        $img1 = $('<img src="' + fakeIamgeUrl + '">').appendTo('body');
        $img2 = $('<img>').appendTo('body');

        $img2[0].onerror = function () {
            $img2[0].onerror = null;

            equal($img1[0].src, proxyIamgeUrl);
            equal($img2[0].src, proxyIamgeUrl);

            $img1.remove();
            $img2.remove();

            $img1 = $('<img src="">').appendTo('body');

            $img1[0].onerror = function () {
                $img1[0].onerror = null;

                equal($img1[0].src, 'about:blank');

                $img1.remove();

                UrlUtil.getOriginUrlObj = storedGetOriginUrlObj;

                start();
            };

            eval(window[JSProcessor.PROCESS_SCRIPT_METH_NAME]('$img1[0].src="about:blank";'));
        };

        eval(window[JSProcessor.PROCESS_SCRIPT_METH_NAME]('$img2[0].src="' + fakeIamgeUrl + '";'));
    };

    eval(window[JSProcessor.PROCESS_SCRIPT_METH_NAME]('$img2[0].src="' + realImageUrl + '";'));
});

// IE9 does not support insertAdjacentHTML for the 'tr'
if (!$.browser.msie || $.browser.version !== '9.0') {
    test('Html fragment', function () {
        var table = $('<table><tr></tr></table>')[0],
            tbody = table.childNodes[0];

        tbody.childNodes[0].insertAdjacentHTML('beforebegin', '<tr><td><div></div></td></tr>');

        equal(tbody.childNodes.length, 2);
        equal(tbody.childNodes[0].tagName.toLowerCase(), 'tr');
        equal(tbody.childNodes[1].tagName.toLowerCase(), 'tr');
        equal(tbody.childNodes[0].childNodes[0].tagName.toLowerCase(), 'td');
        equal(tbody.childNodes[0].childNodes[0].childNodes[0].tagName.toLowerCase(), 'div');
    });
}

test('Process a text node when it is appended to script', function () {
    var originGetProxyUrl = UrlUtil.getProxyUrl,
        proxyUrl          = 'http://example.proxy.com/';

    UrlUtil.getProxyUrl = function () {
        return proxyUrl;
    };

    var url      = 'http://example.com',
        scriptEl = document.createElement('script'),
        textNode = document.createTextNode('window.testLink = document.createElement("a"); window.testLink.href="' +
                                           url + '";');

    scriptEl.appendChild(textNode);
    $('head')[0].appendChild(scriptEl);

    ok(window.testLink);
    ok(window.testLink.tagName && window.testLink.tagName.toLowerCase() === 'a');
    equal(window.testLink.href, proxyUrl);

    UrlUtil.getProxyUrl = originGetProxyUrl;
});

test('Iframe added to dom event', function () {
    var firstIframe  = null,
        secondIframe = null,
        count        = 0;

    DOMSandbox.on(DOMSandbox.IFRAME_ADDED, function (e) {
        if (e.iframe === firstIframe || e.iframe === secondIframe)
            count++;
    });

    firstIframe  = $('<iframe id="test101">')[0];
    secondIframe = $('<iframe id="test102">')[0];

    var container = document.createElement('div');

    container.appendChild(firstIframe);
    container.appendChild(secondIframe);

    equal(count, 0);

    var body = $('body')[0];

    body.appendChild(container);
    $(container).remove();

    equal(count, 2);

    body.insertBefore(container, body.childNodes[0]);
    $(container).remove();

    equal(count, 4);
});

asyncTest('Body created event', function () {
    expect(1);

    $('<iframe src="/data/dom_sandbox/body_created_event.html">').appendTo('body').load(function () {
        ok(this.contentWindow.testOk);
        start();
    });
});


module('Override DOM methods');

(function () {
    function checkInnerHtmlOverrided (el) {
        el.innerHtml = '<div></div>'

        ok(el.insertBefore !== NativeMethods.insertBefore &&
           el.appendChild !== NativeMethods.appendChild, 'innerHtml overrided');
    }

    function checkInsertBeforeOverrided (el) {
        ok(el.insertBefore !== NativeMethods.insertBefore, 'insertBefore overrided');
    }

    function checkAppendChildOverrided (el) {
        ok(el.appendChild !== NativeMethods.appendChild, 'appendChild overrided');
    }

    test('document.createElement', function () {
        var el = document.createElement('DIV');

        checkInsertBeforeOverrided(el);
        checkAppendChildOverrided(el);
        checkInnerHtmlOverrided(el);
    });

    test('element.insertAdjacentHTML', function () {
        var el = document.createElement('DIV');

        el.insertAdjacentHTML('afterbegin', '<div></div>');

        checkInsertBeforeOverrided(el.childNodes[0]);
        checkAppendChildOverrided(el.childNodes[0]);
        checkInnerHtmlOverrided(el.childNodes[0]);

        el.childNodes[0].insertAdjacentHTML('beforebegin', '<span></span>');

        equal(el.childNodes[0].tagName, 'SPAN');
        equal(el.childNodes.length, 2);

        checkInsertBeforeOverrided(el.childNodes[0]);
        checkAppendChildOverrided(el.childNodes[0]);
        checkInnerHtmlOverrided(el.childNodes[0]);
    });

    test('element.insertBefore', function () {
        var el          = document.createElement('DIV'),
            firstChild  = NativeMethods.createElement.call(document, 'DIV'),
            secondChild = NativeMethods.createElement.call(document, 'DIV');

        ok(el.appendChild(secondChild), 'appended');
        ok(el.insertBefore(firstChild, secondChild), 'inserted');

        checkInsertBeforeOverrided(firstChild);
        checkAppendChildOverrided(firstChild);
        checkInnerHtmlOverrided(firstChild);
    });

    test('element.appendChild', function () {
        var el = document.createElement('DIV');

        ok(el.appendChild(NativeMethods.createElement.call(document, 'DIV')), 'appended');

        var child = el.childNodes[0];

        checkInsertBeforeOverrided(child);
        checkAppendChildOverrided(child);
        checkInnerHtmlOverrided(child);
    });

    test('element.cloneNode (B234291)', function () {
        var el    = document.createElement('DIV'),
            clone = el.cloneNode();

        checkAppendChildOverrided(clone);
        checkInnerHtmlOverrided(clone);
    });

    test('element.removeAttribute, element.removeAttributeNS', function () {
        var el         = document.createElement('a'),
            attr       = 'href',
            storedAttr = PageProc.getStoredAttrName(attr),
            namespace  = 'http://www.w3.org/1999/xhtml',
            urlExample = '/test.html';

        el.setAttribute(attr, urlExample);
        el.setAttributeNS(namespace, attr, urlExample);

        ok(NativeMethods.getAttribute.call(el, attr) && NativeMethods.getAttribute.call(el, storedAttr) &&
           NativeMethods.getAttributeNS.call(el, namespace, attr) &&
           NativeMethods.getAttributeNS.call(el, namespace, storedAttr));

        el.removeAttributeNS(namespace, attr);

        ok(NativeMethods.getAttribute.call(el, attr) && NativeMethods.getAttribute.call(el, storedAttr) &&
           !NativeMethods.getAttributeNS.call(el, namespace, attr) &&
           !NativeMethods.getAttributeNS.call(el, namespace, storedAttr));

        el.removeAttribute(attr);

        ok(!NativeMethods.getAttribute.call(el, attr) && !NativeMethods.getAttribute.call(el, storedAttr) &&
           !NativeMethods.getAttributeNS.call(el, namespace, attr) &&
           !NativeMethods.getAttributeNS.call(el, namespace, storedAttr));
    });

    test('element.getAttributeNS, element.setAttributeNS', function () {
        var savedGetProxyUrl = UrlUtil.getProxyUrl;

        var elTagName  = 'image',
            attr       = 'href',
            storedAttr = PageProc.getStoredAttrName(attr);

        UrlUtil.getProxyUrl = function () {
            return 'replaced';
        };

        var el = document.createElementNS('xlink', elTagName);

        equal(el[window.Hammerhead.DOM_SANDBOX_PROCESSED_CONTEXT], window);

        el.setAttributeNS('xlink', attr, 'image.png');
        equal(NativeMethods.getAttributeNS.call(el, 'xlink', attr), 'replaced');
        equal(NativeMethods.getAttributeNS.call(el, 'xlink', storedAttr), 'image.png');
        equal(el.getAttributeNS('xlink', attr), 'image.png');

        UrlUtil.getProxyUrl = savedGetProxyUrl;
    });

    test('table.insertRow, table.insertCell', function () {
        var table    = document.createElement('table'),
            tbody    = document.createElement('tbody'),
            tableRow = table.insertRow(0),
            tbodyRow = tbody.insertRow(0),
            cell     = tableRow.insertCell(0);

        ok(tableRow.appendChild !== NativeMethods.appendChild);
        ok(tbodyRow.appendChild !== NativeMethods.appendChild);
        ok(cell.appendChild !== NativeMethods.appendChild);
    });

    test('document.createDocumentFragment (B237717)', function () {
        var fragment = document.createDocumentFragment();

        fragment.appendChild(document.createElement('div'));

        var clone = fragment.cloneNode(true);

        ok(fragment.firstChild.getAttribute !== NativeMethods.getAttribute);
        ok(clone.firstChild.getAttribute !== NativeMethods.getAttribute);
    });

    test('setAttribute: img src', function () {
        var $img = $('<img>');

        window[window.Hammerhead.DOM_SANDBOX_OVERRIDE_DOM_METHOD_NAME]($img[0]);

        $img[0].setAttribute('src', '/image.gif?param=value');

        equal(NativeMethods.getAttribute.call($img[0], 'src'), UrlUtil.resolveUrlAsOrigin('/image.gif?param=value'));
        $img.remove();
    });

    test('CanvasRenderingContext2D.drawImage', function () {
        var storedNativeMethod    = NativeMethods.canvasContextDrawImage,
            storedOriginUrlObject = Settings.ORIGIN_URL_OBJ,
            url1                  = 'http://crossdomain.com/image.png',
            url2                  = 'http://' + location.host + '/';

        expect(6);

        var img           = NativeMethods.createElement.call(document, 'img'),
            convasContext = $('<canvas>')[0].getContext('2d'),
            otherCanvas   = $('<canvas>')[0];

        Settings.ORIGIN_URL_OBJ = {
            protocol: 'http:',
            host:     'samedomain.com'
        };

        NativeMethods.canvasContextDrawImage = function (img, p1, p2) {
            ok(p1 === 2 && p2 === 3);
            equal(img.src, url1);
        };
        img.src                              = url1;
        convasContext.drawImage(img, 2, 3);

        NativeMethods.canvasContextDrawImage = function (img, p1, p2) {
            ok(p1 === 4 && p2 === 5);
            equal(img.src, UrlUtil.getProxyUrl(url2));
        };
        img.src                              = url2;
        convasContext.drawImage(img, 4, 5);

        NativeMethods.canvasContextDrawImage = function (img, p1, p2) {
            ok(p1 === 23 && p2 === 52);
            equal(img, otherCanvas);
        };
        convasContext.drawImage(otherCanvas, 23, 52);

        NativeMethods.canvasContextDrawImage = storedNativeMethod;
        Settings.ORIGIN_URL_OBJ              = storedOriginUrlObject;
    });

    if (!Util.isMozilla) {
        test('document.write exception', function () {
            var $iframe = $('<iframe id="test10">').appendTo('body');

            eval(window[JSProcessor.PROCESS_SCRIPT_METH_NAME]([
                '$iframe[0].contentDocument.write("<html><body><div id=\'div1\'></div></body></html>");',
                '$iframe[0].contentDocument.write("<script>window.test = true;<\/script>");'
            ].join('')));

            ok($iframe[0].contentWindow.test);
            ok($iframe[0].contentDocument.getElementById('div1'));

            $iframe.remove();
        });
    }

    if (window.FormData) {
        asyncTest('FormData must be sent correctly with our file wrappers (T320300)', function () {
            var formData = new FormData();
            var xhr      = new NativeMethods.XMLHttpRequest();

            InfoManager.prepareFileListWrapper([{
                info: {
                    size: 4,
                    type: 'text/plain',
                    name: 'correctName.txt'
                },
                blob: new Blob(['text'], { type: 'text/plain' })
            }], function (err, fileList) {
                var fileWraper = fileList[0];

                formData.append('file', fileWraper);
                formData.append(SharedConstHH.UPLOAD_SANDBOX_HIDDEN_INPUT_NAME, '[]');

                xhr.open('post', '/form-data-test', true);
                xhr.addEventListener('readystatechange', function () {
                    if (this.readyState !== this.DONE)
                        return;

                    var result = JSON.parse(this.responseText);

                    strictEqual(Object.keys(result.body).length, 0);
                    strictEqual(Object.keys(result.files).length, 1);
                    strictEqual(typeof result.files.file, 'object');
                    strictEqual(result.files.file.name, 'correctName.txt');

                    start();
                });
                xhr.send(formData);
            });
        });
    }
})();

module('Override DOM classes');

(function () {
    test('window.Image', function () {
        ok(window.Image !== NativeMethods.imageCtor);
    });

    test('window.Worker', function () {
        ok(window.Worker !== NativeMethods.workerCtor);
    });

    test('window.EventSource', function () {
        ok(window.EventSource !== NativeMethods.eventSourceCtor);
    });

    test('window.Function (T423261)', function () {
        var removeOuterFunctionRe = /^[^{]+\{\s*|\s*}$/g;
        var fnCtor                = Function;
        var codeStr               = 'location.href="/page.htm";';
        var processedCodeStr      = JSProcessor.process(codeStr).trim();

        strictEqual(new Function('arg1', 'arg2', codeStr).toString().replace(removeOuterFunctionRe, ''), processedCodeStr);
        strictEqual(Function('arg1', 'arg2', codeStr).toString().replace(removeOuterFunctionRe, ''), processedCodeStr);
        strictEqual(fnCtor('arg1', 'arg2', 'arg3', codeStr).toString().replace(removeOuterFunctionRe, ''), processedCodeStr);
        strictEqual(Function.apply(null, ['arg1', codeStr]).toString().replace(removeOuterFunctionRe, ''), processedCodeStr);
        strictEqual(Function.call(null, 'arg1', codeStr).toString().replace(removeOuterFunctionRe, ''), processedCodeStr);

        ok(Function() instanceof Function);
        strictEqual(Function().constructor, Function);
    });
})();

module('Override attributes');

(function () {
    test('url attributes', function () {
        var testUrlAttr = function (tagName, attr) {
            var el         = NativeMethods.createElement.call(document, tagName),
                storedAttr = PageProc.getStoredAttrName(attr),
                namespace  = 'http://www.w3.org/1999/xhtml';

            var getAttr = function () {
                return NativeMethods.getAttribute.call(el, attr);
            };

            var getWrapAttr    = function () {
                return NativeMethods.getAttribute.call(el, storedAttr);
            };

            window[window.Hammerhead.DOM_SANDBOX_OVERRIDE_DOM_METHOD_NAME](el);

            el.setAttribute(attr, '');
            var emptyAttrValue = el[attr];

            var origin = 'http://origin.com/',
                proxy  = UrlUtil.getProxyUrl(origin, null, null, null, null, tagName === 'script' ? 'script' : null);

            window[JSProcessor.SET_PROPERTY_METH_NAME](el, attr, origin);
            equal(el[attr], proxy);
            equal(window[JSProcessor.GET_PROPERTY_METH_NAME](el, attr), origin);
            equal(getAttr(), proxy);
            equal(getWrapAttr(), origin);

            var newUrl      = '/image',
                proxyNewUrl = UrlUtil.getProxyUrl('/image', null, null, null, null, tagName ===
                                                                                    'script' ? 'script' : null);

            el.setAttribute(attr, newUrl);
            equal(el[attr], proxyNewUrl);
            equal(window[JSProcessor.GET_PROPERTY_METH_NAME](el, attr), UrlUtil.formatUrl(UrlUtil.parseProxyUrl(proxyNewUrl).originResourceInfo));
            equal(getAttr(), proxyNewUrl);
            equal(getWrapAttr(), newUrl);

            window[JSProcessor.SET_PROPERTY_METH_NAME](el, attr, '');
            ok(getWrapAttr() === '' && getAttr() === '');
            strictEqual(el[attr], emptyAttrValue);

            el.removeAttribute(attr);
            ok(getWrapAttr() === null && getAttr() === null);

            if (attr === 'action' && el[attr] === window.location.toString())
                strictEqual(window[JSProcessor.GET_PROPERTY_METH_NAME](el, attr), UrlUtil.formatUrl(UrlUtil.getOriginUrlObj()));
            else
                strictEqual(window[JSProcessor.GET_PROPERTY_METH_NAME](el, attr), '');

            el.setAttributeNS(namespace, attr, origin);
            equal(NativeMethods.getAttributeNS.call(el, namespace, attr), proxy);
            equal(NativeMethods.getAttributeNS.call(el, namespace, storedAttr), origin);
        };

        var testData = [
            { tagName: 'a', attr: 'href' },
            { tagName: 'script', attr: 'src' },
            { tagName: 'link', attr: 'href' },
            { tagName: 'form', attr: 'action' },
            { tagName: 'embed', attr: 'src' },
            { tagName: 'object', attr: 'data' }
        ];

        for (var i = 0; i < testData.length; i++)
            testUrlAttr(testData[i].tagName, testData[i].attr);
    });

    test('script src', function () {
        var storedJobUid     = Settings.JOB_UID,
            storedOwnerToken = Settings.JOB_OWNER_TOKEN;

        Settings.JOB_UID         = 'uid';
        Settings.JOB_OWNER_TOKEN = 'token';

        var script = document.createElement('script');

        overrideDomMeth(script);

        script.setAttribute('src', 'http://google.com');

        equal(UrlUtil.parseProxyUrl(script.src).resourceType, UrlUtil.SCRIPT);

        Settings.JOB_UID         = storedJobUid;
        Settings.JOB_OWNER_TOKEN = storedOwnerToken;
    });

    asyncTest('inject HH in the iframe with "javascript: <html>...</html>" src', function () {
        var iframe = document.createElement('iframe'),
            src    = 'javascript:"<script>var d = {}; d.src = 1; window.test = true;<\/script>"';

        iframe.id = 'test55';
        overrideDomMeth(iframe);

        iframe.setAttribute('src', src);

        iframe.onload = function () {
            if (this.contentWindow.test) {
                ok(this.contentWindow.test);
                document.body.removeChild(this);

                start();
            }
        };

        document.body.appendChild(iframe);
    });

    asyncTest('iframe html src', function () {
        var iframe = document.createElement('iframe'),
            src    = 'javascript:\'<html><body><a id=\\\'test\\\' data-attr=\"123\">link</a></body></html>\'';

        iframe.id = 'test56';
        overrideDomMeth(iframe);

        iframe.setAttribute('src', src);

        var srcAttr       = NativeMethods.getAttribute.call(iframe, 'src'),
            storedSrcAttr = NativeMethods.getAttribute.call(iframe, 'src' + Hammerhead.DOM_SANDBOX_STORED_ATTR_POSTFIX);

        notEqual(srcAttr, src);
        equal(srcAttr, 'javascript:\'' +
                       SandboxUtil.processHtml('<html><body><a id=\'test\' data-attr=\"123\">link</a></body></html>') +
                       '\'');
        equal(storedSrcAttr, src);
        equal(iframe.getAttribute('src'), src);

        iframe.onload = function () {
            if (this.contentDocument.getElementById('test')) {
                ok(this.contentDocument.getElementById('test'));
                equal(this.contentDocument.getElementById('test').getAttribute('data-attr'), '123');
                document.body.removeChild(this);

                start();
            }
        };

        document.body.appendChild(iframe);
    });

    test('target attribute', function () {
        var tagNames = ['a', 'form', 'area', 'base'],
            tag      = null;

        for (var i = 0; i < tagNames.length; i++) {
            tag = document.createElement(tagNames[i]);

            tag.setAttribute('target', '_blank');
            ok(!tag.getAttribute('target'));

            tag.setAttribute('target', '_self');
            ok(tag.getAttribute('target'));

            window[JSProcessor.SET_PROPERTY_METH_NAME](tag, 'target', '_blank');
            ok(tag.target !== '_blank');
        }
    });

    test('event attributes', function () {
        var link      = document.createElement('a'),
            attrValue = 'location.href="managers.aspx";';

        link.setAttribute('onclick', attrValue);

        var attr       = NativeMethods.getAttribute.call(link, 'onclick'),
            storedAttr = NativeMethods.getAttribute.call(link, PageProc.getStoredAttrName('onclick'));

        notEqual(attr, storedAttr);
        strictEqual(storedAttr, attrValue);
        strictEqual(attr, JSProcessor.process(attrValue));
        strictEqual(link.getAttribute('onclick'), attrValue);
    });

    test('event attributes: javascript protocol', function () {
        var link      = document.createElement('a'),
            attrValue = 'javascript:location.href="managers.aspx";';

        link.setAttribute('onclick', attrValue);

        var attr       = NativeMethods.getAttribute.call(link, 'onclick'),
            storedAttr = NativeMethods.getAttribute.call(link, PageProc.getStoredAttrName('onclick'));

        notEqual(attr, storedAttr);
        strictEqual(storedAttr, attrValue);
        strictEqual(attr, 'javascript:' + JSProcessor.process(attrValue.replace('javascript:', '')));
        strictEqual(link.getAttribute('onclick'), attrValue);
    });

    test('url attribute with "javascript:" protocol', function () {
        var link      = document.createElement('a'),
            hrefValue = 'javascript:location.href="managers.aspx?Delete=814";';

        link.setAttribute('href', hrefValue);

        var href       = NativeMethods.getAttribute.call(link, 'href'),
            storedHref = NativeMethods.getAttribute.call(link, PageProc.getStoredAttrName('href'));

        notEqual(href, storedHref);
        strictEqual(storedHref, hrefValue);
        strictEqual(href, 'javascript:' + JSProcessor.process(hrefValue.replace('javascript:', '')));
        strictEqual(link.getAttribute('href'), hrefValue);
    });

    test('iframe.sandbox', function () {
        var attr       = 'sandbox',
            storedAttr = PageProc.getStoredAttrName(attr);

        var el = $('<ifram>')[0];

        window[window.Hammerhead.DOM_SANDBOX_OVERRIDE_DOM_METHOD_NAME](el);

        ok(!NativeMethods.getAttribute.call(el, attr));
        ok(!NativeMethods.getAttribute.call(el, storedAttr));
        ok(!el.getAttribute(attr));

        el.setAttribute(attr, 'allow-scripts allow-forms');

        equal(NativeMethods.getAttribute.call(el, attr), 'allow-scripts allow-forms allow-same-origin');
        equal(NativeMethods.getAttribute.call(el, storedAttr), 'allow-scripts allow-forms');
        equal(el.getAttribute(attr), 'allow-scripts allow-forms');

        el.setAttribute(attr, 'allow-forms');

        equal(NativeMethods.getAttribute.call(el, attr), 'allow-forms allow-same-origin allow-scripts');
        equal(NativeMethods.getAttribute.call(el, storedAttr), 'allow-forms');
        equal(el.getAttribute(attr), 'allow-forms');
    });

    test('crossdomain_iframe.src', function () {
        var el            = document.createElement('IFRAME'),
            storedSrcAttr = PageProc.getStoredAttrName('src');

        var savedGetProxyUrl                  = UrlUtil.getProxyUrl,
            savedGetCrossDomainIframeProxyUrl = UrlUtil.getCrossDomainIframeProxyUrl;

        var crossDomainUrl      = 'http://cross.domain.com/',
            crossDomainProxyUrl = 'http://proxy.cross.domain.com/',
            proxyUrl            = 'http://proxy.com/';


        UrlUtil.getProxyUrl = function () {
            return proxyUrl;
        };

        UrlUtil.getCrossDomainIframeProxyUrl = function () {
            return crossDomainProxyUrl;
        };

        window[JSProcessor.SET_PROPERTY_METH_NAME](el, 'src', crossDomainUrl);
        equal(el.src, crossDomainProxyUrl);
        equal(NativeMethods.getAttribute.call(el, 'src'), crossDomainProxyUrl);
        equal(NativeMethods.getAttribute.call(el, storedSrcAttr), crossDomainUrl);

        window[JSProcessor.SET_PROPERTY_METH_NAME](el, 'src', location.toString() + '?param');
        equal(el.src, proxyUrl);
        equal(NativeMethods.getAttribute.call(el, 'src'), proxyUrl);
        equal(el.getAttribute('src'), location.toString() + '?param');

        UrlUtil.getProxyUrl                  = savedGetProxyUrl;
        UrlUtil.getCrossDomainIframeProxyUrl = savedGetCrossDomainIframeProxyUrl;
    });

    test('input.autocomplete', function () {
        var input      = document.createElement('input'),
            etalon     = NativeMethods.createElement.call(document, 'input'),
            storedAttr = PageProc.getStoredAttrName('autocomplete');

        strictEqual(input.getAttribute('autocomplete'), NativeMethods.getAttribute.call(etalon, 'autocomplete'));
        strictEqual(NativeMethods.getAttribute.call(input, 'autocomplete'), 'off');
        strictEqual(NativeMethods.getAttribute.call(input, storedAttr), 'none');

        input.setAttribute('autocomplete', 'off');
        NativeMethods.setAttribute.call(etalon, 'autocomplete', 'off');
        strictEqual(input.getAttribute('autocomplete'), NativeMethods.getAttribute.call(etalon, 'autocomplete'));
        strictEqual(NativeMethods.getAttribute.call(input, 'autocomplete'), 'off');
        strictEqual(NativeMethods.getAttribute.call(input, storedAttr), 'off');

        input.setAttribute('autocomplete', 'on');
        NativeMethods.setAttribute.call(etalon, 'autocomplete', 'on');
        strictEqual(input.getAttribute('autocomplete'), NativeMethods.getAttribute.call(etalon, 'autocomplete'));
        strictEqual(NativeMethods.getAttribute.call(input, 'autocomplete'), 'off');
        strictEqual(NativeMethods.getAttribute.call(input, storedAttr), 'on');

        input.setAttribute('autocomplete', '');
        NativeMethods.setAttribute.call(etalon, 'autocomplete', '');
        strictEqual(input.getAttribute('autocomplete'), NativeMethods.getAttribute.call(etalon, 'autocomplete'));
        strictEqual(NativeMethods.getAttribute.call(input, 'autocomplete'), 'off');
        strictEqual(NativeMethods.getAttribute.call(input, storedAttr), '');

        input.removeAttribute('autocomplete');
        NativeMethods.removeAttribute.call(etalon, 'autocomplete');
        strictEqual(input.getAttribute('autocomplete'), NativeMethods.getAttribute.call(etalon, 'autocomplete'));
        strictEqual(NativeMethods.getAttribute.call(input, 'autocomplete'), 'off');
        strictEqual(NativeMethods.getAttribute.call(input, storedAttr), 'none');
    });

    test('window.onbeforeunload', function () {
        //NOTE: if the test will be failed, system dialog blocks page before the page unloading
        var handler = function () {
            return 'value';
        };

        strictEqual(window.onbeforeunload, null);

        window[JSProcessor.SET_PROPERTY_METH_NAME](window, 'onbeforeunload', handler);

        ok(window.onbeforeunload);
    });

    test('element.innerHTML', function () {
        Settings.JOB_UID         = 'job';
        Settings.JOB_OWNER_TOKEN = 'token';

        var ENCODED_DESCRIPTOR_VALUES_SEPARATOR = encodeURI(UrlUtil.REQUEST_DESCRIPTOR_VALUES_SEPARATOR);

        var container    = document.createElement('DIV'),
            checkElement = function (el, attr) {
                var storedAttr   = PageProc.getStoredAttrName(attr),
                    resourceType = '';

                if (el.tagName.toLowerCase() === 'script' && attr === 'src')
                    resourceType = ENCODED_DESCRIPTOR_VALUES_SEPARATOR + 'script';

                equal(NativeMethods.getAttribute.call(el, storedAttr), '/Images/1.png', 'original url stored');
                equal(NativeMethods.getAttribute.call(el, attr), 'http://' + location.host + '/Images/1.png?' +
                                                                 UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY +
                                                                 '=https%3A' +
                                                                 ENCODED_DESCRIPTOR_VALUES_SEPARATOR +
                                                                 'example.com' +
                                                                 ENCODED_DESCRIPTOR_VALUES_SEPARATOR +
                                                                 'job' +
                                                                 ENCODED_DESCRIPTOR_VALUES_SEPARATOR +
                                                                 'token' + resourceType, 'url replaced');
            };

        var html = ' <A iD = " fakeId1 " hRef = "/Images/1.png" attr="test" ></A>' +
                   ' <form iD="fakeId3 " action="/Images/1.png"    attr="test" ></form>' +
                   ' <link  iD = " fakeId5" href="/Images/1.png" attr="test" ></link>' +
                   ' <' + 'script  iD = "  fakeId6  " sRc=  "/Images/1.png" attr= "test"' + '>' + '</' + 'scriPt' + '>';

        window[JSProcessor.SET_PROPERTY_METH_NAME](container, 'innerHTML', html);

        checkElement($(container).find('a')[0], 'href');
        checkElement($(container).find('form')[0], 'action');
        checkElement($(container).find('link')[0], 'href');
        checkElement($(container).find('script')[0], 'src');
    });

    test('anchor with target attribute', function () {
        var anchor   = document.createElement('a'),
            url      = 'http://url.com/',
            proxyUrl = UrlUtil.getProxyUrl(url, null, null, null, null, 'iframe');

        anchor.setAttribute('target', 'iframeName');

        equal(NativeMethods.getAttribute.call(anchor, 'target'), 'iframeName');

        anchor.setAttribute('href', url);

        var nativeHref = NativeMethods.getAttribute.call(anchor, 'href');

        equal(nativeHref, proxyUrl);
        equal(NativeMethods.getAttribute.call(anchor, PageProc.getStoredAttrName('href')), url);
        equal(anchor.getAttribute('href'), url);
        equal(UrlUtil.parseProxyUrl(nativeHref).resourceType, 'iframe');
    });

    test('meta with name=referrer', function () {
        var meta   = document.createElement('meta');

        meta.setAttribute('name', 'referrer');

        ok(!NativeMethods.getAttribute.call(meta, 'name'));
    });
})();

module('Regression tests');

(function () {
    test('B237015 - Some tests fail in IE9 with unexpected alert message', function () {
        var testPropertyName = 'testProperty',
            el               = document.createElement('div'),
            body             = document.getElementsByTagName('body')[0],
            script           = "<script>window." + testPropertyName + " = true;\<\/script>";

        body.appendChild(el);

        el.innerHTML = script;

        ok(!window[testPropertyName]);
    });

    test('B234340 - An infinite response waiting on Amazon.com', function () {
        var img = new Image();

        window[JSProcessor.SET_PROPERTY_METH_NAME](img, 'src', 'data/image.png');

        equal(NativeMethods.getAttribute.call(img, 'src'), UrlUtil.resolveUrlAsOrigin('data/image.png'));
        equal(NativeMethods.getAttribute.call(img, PageProc.getStoredAttrName('src')), 'data/image.png');

        var nativeImageConstr = NativeMethods.imageCtor;

        equal((new Image()).outerHTML, new nativeImageConstr().outerHTML);

        equal((new Image(15)).outerHTML, new nativeImageConstr(15).outerHTML);
        equal((new Image(15, 15)).outerHTML, new nativeImageConstr(15, 15).outerHTML);

        equal((new Image(undefined)).outerHTML, new nativeImageConstr(undefined).outerHTML);
        equal((new Image(undefined, undefined)).outerHTML, new nativeImageConstr(undefined, undefined).outerHTML);
    });

    asyncTest('B237231 - Scripts are not executed when added into a header in IE9', function () {
        var scriptText = 'window.top.testField = true;',
            head       = document.getElementsByTagName('head')[0],
            script     = document.createElement('script');

        script.src = '/get-script/' + scriptText;

        ok(!window.top.testField);
        head.appendChild(script);

        window.setTimeout(function () {
            ok(window.top.testField);
            $(script).remove();
            start();
        }, 500);
    });

    test('Q519748 - TestCafe breaks jQuery Tabs widget', function () {
        var link        = $('<a href=""></a>').appendTo('body'),
            resolvedUrl = window[JSProcessor.GET_PROPERTY_METH_NAME](link[0], 'href');

        strictEqual(resolvedUrl.indexOf(Settings.ORIGIN_URL_OBJ.protocol), 0);
        ok(resolvedUrl.indexOf(Settings.ORIGIN_URL_OBJ.host) > -1);

        link.remove();
    });

    test('B237402 - overrideEventListeners: one listener for several events', function () {
        var $div       = $('<div></div>').appendTo('body'),
            mousemoved = false,
            mouseuped  = false,
            handler    = function (e) {
                if (e.type === 'mousemove')
                    mousemoved = true;
                if (e.type === 'mouseup')
                    mouseuped = true;
            };
        $div[0].addEventListener('mousemove', handler, false);
        $div[0].addEventListener('mouseup', handler, false);
        EventSimulator['mousemove']($div[0], {});
        EventSimulator['mouseup']($div[0], {});
        ok(mousemoved, 'mousemove has been handled');
        ok(mouseuped, 'mouseup has been handled');
        mousemoved     = mouseuped = false;
        $div[0].removeEventListener('mousemove', handler, false);
        $div[0].removeEventListener('mouseup', handler, false);
        EventSimulator['mousemove']($div[0], {});
        EventSimulator['mouseup']($div[0], {});
        ok(!mousemoved, 'mousemove has not been handled');
        ok(!mouseuped, 'mouseup has not been handled');
        $div.remove();
    });

    test('B238830 - Js error occurs on the http://www.tripadvisor.ru/ website.', function () {
        var error     = false,
            windowObj = window.Window;

        window.onerror = function () {
            error = true;
        };

        window.Window = function () {
        };

        window.addEventListener('click', function () {
        });

        window.Window = windowObj;

        ok(!error);
    });

    test('B238838 - TestCafe cant switch ASPxPageControl tabs in AutoPostBack mode', function () {
        var url              = 'http://www.test.com/',
            $linkWithHref    = $('<a href="' + url + '"></a>').appendTo('body'),
            $linkWithoutHref = $('<a></a>');

        equal(window[JSProcessor.GET_PROPERTY_METH_NAME]($linkWithHref[0], 'href'), url);
        equal(window[JSProcessor.GET_PROPERTY_METH_NAME]($linkWithoutHref[0], 'href'), '');
    });

    asyncTest('Q527555 - Can not click on button in DIV', function () {
        var $iframe   = $('<iframe id="test">').appendTo('body'),
            innerHTML = 'text';

        window.setTimeout(function () {
            var iframeBody = $iframe[0].contentWindow.document.body,
                html       = '<a href="url" ' + PageProc.getStoredAttrName('src') + '="url1" />';

            iframeBody.innerHTML = html;

            ok(window[JSProcessor.GET_PROPERTY_METH_NAME](iframeBody, 'innerHTML') !== html);
            start();
        }, 100);
    });

    asyncTest('B239643 - The test on the http://kinopoisk.ru page doesn\'t continue after the first step', function () {
        var $container               = $('<div><iframe id="test1"></iframe></div>').appendTo('body'),
            iframeLoadingEventRaised = false;

        var onIframeLoading = function () {
            iframeLoadingEventRaised = true;
        };

        //iframe loading waiting
        window.setTimeout(function () {
            IFrameSandbox.on(IFrameSandbox.IFRAME_READY_TO_INIT, onIframeLoading);

            var dummy = $container[0].innerHTML;

            ok(!iframeLoadingEventRaised);
            $container.remove();
            start();
        }, 100);
    });

    if (!Util.isMozilla) {
        test('Override document after document.write calling', function () {
            var $div    = $('<div>').appendTo('body'),
                $sdiv   = $('<div>').appendTo('body'),
                $iframe = $('<iframe id="test11" src="about:blank">"'),
                iframe  = $iframe[0];

            var checkIframeDocumentOverrided = function () {
                    var document = iframe.contentDocument,
                        result   = true;

                    if (document) {
                        if (document.write.toString() === NativeMethods.documentWrite.toString())
                            result = false;
                    }

                    // Stack overflow check
                    ok(!document || document.getElementsByTagName('body'));
                    ok(window.top.document.getElementsByTagName('body'));

                    ok(result);
                },

                checkWriteFunction           = function () {
                    checkIframeDocumentOverrided();
                    iframe.contentDocument.open();
                    checkIframeDocumentOverrided();
                    iframe.contentDocument.write('<div></div>');
                    checkIframeDocumentOverrided();
                    iframe.contentDocument.close();
                    checkIframeDocumentOverrided();

                    iframe.contentDocument.open();
                    checkIframeDocumentOverrided();
                    iframe.contentDocument.write('<html><body><a href="http://google.com/"></body></html>');
                    checkIframeDocumentOverrided();
                    iframe.contentDocument.close();
                    checkIframeDocumentOverrided();
                };

            $iframe.ready(checkIframeDocumentOverrided);
            $iframe.load(checkIframeDocumentOverrided);

            // After append to DOM
            $div[0].appendChild(iframe);
            checkWriteFunction();

            // After reinsert to dom
            $sdiv[0].appendChild(iframe);
            checkWriteFunction();

            $iframe.remove();
            $sdiv.remove();
            $div.remove();
        });
    }

    if (!Util.isIE || Util.browserVersion > 9) {
        test('Mousedown does not processed after replace element using Range.createContextualFragment (in IE and FF) (T367352)', function () {
            var tagString = '<a href="http://some.domain.com/index.html"></a>';
            var range     = document.createRange();
            var container = document.createElement('div');

                document.body.appendChild(container);

                range.selectNode(container);

                var fragment = range.createContextualFragment(tagString);

                strictEqual(fragment.childNodes[0].href, UrlUtil.getProxyUrl('http://some.domain.com/index.html'));
        });
    }

    asyncTest('Inject HH in the iframe with empty url', function () {
        var $div   = $('<div>').appendTo('body'),
            cheked = false;

        DOMSandbox.overrideDomMethods($div[0]);

        var check = function () {
            var document = $iframe[0].contentDocument;

            if (document)
                ok(document.write.toString() !== NativeMethods.documentWrite.toString());
        };

        var $iframe = $('<iframe id="test3" src="about:blank">"');

        check();

        $iframe.ready(check);
        $iframe.load(function () {
            check();

            var id = setInterval(function () {
                if (cheked) {
                    clearInterval(id);
                    $iframe.remove();
                    start();
                }
            }, 10);
        });

        $div[0].appendChild($iframe[0]);
        check();
        cheked      = true;
    });

    asyncTest('Inject HH in the iframe with empty url: "javascript:" src', function () {
        var $div = $('<div>').appendTo('body');

        DOMSandbox.overrideDomMethods($div[0]);

        var $iframe = $('<iframe id="test4" src="javascript:&quot;<html><body><a id=\'link\' href=\'http://google.com/\'></body></html>&quot;"></iframe>"');

        $div[0].appendChild($iframe[0]);

        ok($iframe[0].contentDocument.write.toString() !== NativeMethods.documentWrite.toString());
        $iframe.remove();
        start();
    });

    test('T190753 - A JavaScript error occurs during test running on msn.com', function () {
        var $div            = $('<div>').appendTo('body'),
            $iframe         = $('<iframe id="test5">'),
            script          = 'var a = [1,2], b = 0; window.test = a[b];',
            processedScript = PageProc.processScript(script).replace(/\s*/g, '');

        overrideDomMeth($div[0]);
        $div[0].appendChild($iframe[0]);

        ok(script.replace(/\s*/g, '') !== processedScript);

        $iframe[0].contentDocument.write('<html><head><script>' + script + '<\/script><head><body></body></html>');

        strictEqual($iframe[0].contentWindow.test, 1);

        var scripts = $iframe[0].contentDocument.getElementsByTagName('script');

        strictEqual(scripts.length, 1);
        strictEqual(scripts[0].text.replace(/\s*/g, ''), processedScript);

        //$iframe.remove();
        //$div.remove();
    });

    asyncTest('B239854 - Google Images - reload page after first mouse click on image', function () {
        var form = document.createElement('form'),
            url  = 'http://example.com';

        expect(2);
        notEqual(form.submit, NativeMethods.formSubmit);

        form.action = url + '/?' + UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY + '=value';

        form.addEventListener('submit', function (event) {
            strictEqual(this[UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY], this[this.length - 1]);

            event.preventDefault();

            start();
        }, false);

        var ev = document.createEvent('Events');
        ev.initEvent('submit', true, true);
        form.dispatchEvent(ev);
    });

    if (!$.browser.mozilla) {
        asyncTest('T239131 - TD15.1 - error on image click (wikipedia.org)', function () {
            var $iframe     = $('<iframe id="test04">'),
                loadHandler = function () {
                    $iframe.unbind('load', loadHandler);

                    // Some browsers remove they documentElement after "write([])" calling.
                    // Previously, if the documentElement was null, "overrideDomMethods"" 
                    // failed with the 'Maximum call stack size exceeded' error                    
                    $iframe[0].contentDocument.write([]);
                    ok(true);
                    $iframe[0].contentDocument.close();
                    $iframe.remove();
                    start();
                };

            $iframe.bind('load', loadHandler).appendTo('body');
        });
    }

    test('window.onerror setter/getter', function () {
        strictEqual(window[JSProcessor.GET_PROPERTY_METH_NAME](window, 'onerror'), null);

        window[JSProcessor.SET_PROPERTY_METH_NAME](window, 'onerror', 123);
        strictEqual(window[JSProcessor.GET_PROPERTY_METH_NAME](window, 'onerror'), null);

        var handler = function () {
        };

        window[JSProcessor.SET_PROPERTY_METH_NAME](window, 'onerror', handler);
        strictEqual(window[JSProcessor.GET_PROPERTY_METH_NAME](window, 'onerror'), handler);
    });

    asyncTest('T215136 - Not overrided all cases of "document.write" function (kakaku.com)', function () {
        expect(2);

        $('<iframe src="/data/dom_sandbox/iframe_with_doc_write.html">').appendTo('body').load(function () {
            var iframe = this,
                div    = iframe.contentDocument.querySelector('#parent');

            strictEqual(div.children.length, 3);
            strictEqual(div.parentNode.lastElementChild, div);

            iframe.parentNode.removeChild(iframe);
            start();
        });
    });

    test('T230764: TD15.1 - Firefox - value.replace is not a function, js errors on lastfm.ru', function () {
        var a     = document.createElement('a'),
            error = false;

        try {
            a.setAttribute('href', function () { /* */
            });
        } catch (e) {
            error = true;
        } finally {
            ok(!error);
        }
    });

    if (!Util.isIE || Util.isIE11) {
        asyncTest('T259367 - The TestCafe recorder cannot load jsTree bound via Ajax', function () {
            var script = ['self.onmessage = function() { var t = {};', '__set$(t, "blobTest", true); postMessage(t.blobTest); };'];
            var blob   = new window.Blob(script, { type: "texT/javascript" });
            var url    = window.URL.createObjectURL(blob);
            var worker = new window.Worker(url);

            worker.onmessage = function (e) {
                strictEqual(e.data, true);
                start();
            };

            worker.postMessage('');
        });
    }

    if (navigator.registerProtocolHandler) {
        test('T185853 - Gmail doesn\'t load in Chrome', function () {
            var originGetOriginUrlObjMeth = UrlUtil.getOriginUrlObj,
                testUrl                   = function (url, result, description) {
                    var exception = false;

                    try {
                        navigator.registerProtocolHandler('web+testprotocol', url, 'Title');
                    } catch (e) {
                        exception = true;
                    } finally {
                        strictEqual(result, exception, description);
                    }
                };

            UrlUtil.getOriginUrlObj = function () {
                return {
                    protocol: 'https:',
                    host:     'example.com:233',
                    hostname: 'example.com',
                    port:     '233'
                };
            };

            testUrl('https://example.com:233/?url=%s', false, 'Origin url');
            testUrl('http://example.com:233/?url=%s', $.browser.mozilla ? false : true, 'Another protocol');
            testUrl('https://xample.com:233/?url=%s', true, 'Another hostname');
            testUrl('https://example.com:934/?url=%s', $.browser.mozilla ? false : true, 'Another port');
            testUrl('https://subdomain.example.com:233/?url=%s', false, 'Sub domain');

            UrlUtil.getOriginUrlObj = originGetOriginUrlObjMeth;
        });
    }

    if (window.Blob) {
        test('T279106 - IE11 throws an error when the second parameter of the Blob function is undefined', function() {
            expect(1);

            try {
                var result = new window.Blob(['text']);
            } catch(e) {
                ok(false);
            } finally {
                ok(!!result);
            }
        });
    }
})();

module('Iframe sandbox');

(function () {
    test('Should not raise event before iframe is appended to DOM', function () {
        var eventRaised = false;

        IFrameSandbox.on(IFrameSandbox.IFRAME_READY_TO_INIT, function () {
            eventRaised = true;
        });

        document.createElement('iframe');

        ok(!eventRaised);
    });

    test('Should not raise event if a cross-domain iframe is appended', function () {
        var eventRaised = false;

        IFrameSandbox.on(IFrameSandbox.IFRAME_READY_TO_INIT, function () {
            eventRaised = true;
        });

        var $iframe = $('<iframe id="test7" src="http://cross.domain.com">').appendTo('body');

        ok(!eventRaised);

        $iframe.remove();
    });

    test('document.write calling', function () {
        var $iframe = $('<iframe id="test10">').appendTo('body');

        $iframe[0].contentDocument.write("<script>window.tempTestValue = !!__call$;<\/script>");

        ok($iframe[0].contentWindow.tempTestValue);
    });

    test('T470972 - text nodes should be processed while are appended', function () {
        var styleTextNode  = ".test-class:hover {background-color:'red'}";
        var scriptText     = 'someElement.attributes';
        var style1         = document.createElement('style');
        var style2         = document.createElement('style');
        var script1        = document.createElement('script');
        var script2        = document.createElement('script');

        style1.appendChild(document.createTextNode(styleTextNode));
        script1.appendChild(document.createTextNode(scriptText));

        style2.insertBefore(document.createTextNode(styleTextNode), null);
        script2.insertBefore(document.createTextNode(scriptText), null);

        ok(style1.innerHTML.indexOf(SharedConstHH.TEST_CAFE_HOVER_PSEUDO_CLASS_ATTR) > -1, 'Style1 is processed');
        ok(style2.innerHTML.indexOf(SharedConstHH.TEST_CAFE_HOVER_PSEUDO_CLASS_ATTR) > -1, 'Style2 is processed');

        notEqual(script1.innerHTML, scriptText, 'Script1 is processed');
        notEqual(script2.innerHTML, scriptText, 'Script2 is processed');
    });

    // NOTE: This test must be last (IE11 hack)
    asyncTest('setAttribute: elements in iframe', function () {
        var src = Util.isMozilla ? ' src="javascript:&quot;<html><body></body></html>&quot;"' : '';

        expect(12);

        $('<iframe id="test20"' + src + '>').load(function () {
            var iFrame     = this,
                iFrameBody = iFrame.contentDocument.body;

            iFrame.contentWindow.HammerheadClient.get('DOMSandbox.IFrame').on(IFrameSandbox.IFRAME_READY_TO_INIT, function (e) {
                e.iframe.contentWindow.eval.call(e.iframe.contentWindow, [
                    'HammerheadClient.define(\'Settings\', function (require, exports) {',
                    '    exports.ORIGIN_URL_OBJ = { protocol: "https:", host: "example.com" };',
                    '});',
                    'HammerheadClient.get(\'Hammerhead\').init();'
                ].join(''));
            });

            $('<iframe id="test21">').load(function () {
                var subIFrameBody     = this.contentDocument.body,
                    testData          = [
                        [document.body, 'a', 'href', null, null],
                        [iFrameBody, 'a', 'href', null, 'iframe'],
                        [document.body, 'form', 'action', null, null],
                        [iFrameBody, 'form', 'action', null, 'iframe'],
                        [document.body, 'area', 'href', null, null],
                        [iFrameBody, 'area', 'href', null, null],
                        [document.body, 'a', 'href', '_top', null],
                        [iFrameBody, 'a', 'href', '_top', null],
                        [subIFrameBody, 'a', 'href', '_top', null],
                        [document.body, 'a', 'href', '_parent', null],
                        [iFrameBody, 'a', 'href', '_parent', null],
                        [subIFrameBody, 'a', 'href', '_parent', 'iframe']
                    ], testIFrameFlag = function (body, tag, urlAttr, target, resultFlag) {
                        var element = document.createElement(tag);

                        body.appendChild(element);
                        target && element.setAttribute('target', target);
                        element.setAttribute(urlAttr, '/index.html');

                        equal(UrlUtil.parseProxyUrl(element[urlAttr]).resourceType, resultFlag);

                        body.removeChild(element);
                    };

                testData.forEach(function (arguments) {
                    testIFrameFlag.apply(null, arguments);
                });

                start();
                $(iFrame).remove();
            }).appendTo(iFrameBody);
        }).appendTo('body');
    });

    if (NativeMethods.registerServiceWorker) {
        module('ServiceWorker');

        test('window.navigator.serviceWorker.register (GH-797)', function () {
            var storedNative = NativeMethods.registerServiceWorker;
            var scriptUrl    = '/serviceWorker.js';
            var scopeUrl     = '/';

            NativeMethods.registerServiceWorker = function (url, options) {
                strictEqual(url, UrlUtil.getProxyUrl(scriptUrl, null, null, null, null, UrlUtil.SCRIPT));
                strictEqual(options.scope, UrlUtil.getProxyUrl(scopeUrl, null, null, null, null, UrlUtil.SCRIPT));

                NativeMethods.registerServiceWorker = storedNative;
            };

            window.navigator.serviceWorker.register(scriptUrl, { scope: scopeUrl });
        });

        asyncTest('should reject a Promise for unsecure url (GH-1411)', function () {
            window.navigator.serviceWorker.register('http://example.com/worker.js')
                .then(function () {
                    ok(false);
                })
                .catch(function () {
                    ok(true);
                })
                .then(function () {
                    start();
                });
        });

        // NOTE: Service workers are only accessible via https. The easiest way around it is
        // to go to http://localhost instead of the IP address of the computer you are running.
        // https://www.chromium.org/Home/chromium-security/prefer-secure-origins-for-powerful-new-features
        // This condition works only for running on the local machine only. On Saucelabs url with a domain name is opened.
        if (location.hostname === 'localhost') {
            asyncTest('should correctly process the "scope" option into the serviceWorker.register (GH-1233)', function () {
                window.navigator.serviceWorker.register('/data/serviceWorker.js', { scope: '/data/' })
                    .then(function () {
                        ok(true);
                    })
                    .catch(function (err) {
                        ok(false, err);
                    })
                    .then(function () {
                        start();
                    });
            });
        }
    }
})();
