var DomSandboxUtil = HammerheadClient.get('DOMSandbox.Util'),
    Hammerhead = HammerheadClient.get('Hammerhead'),
    PageProc = HammerheadClient.get('Shared.PageProc'),
    UrlUtil = HammerheadClient.get('UrlUtil'),
    IFrameSandbox = HammerheadClient.get('DOMSandbox.IFrame'),

    NativeMethods = Hammerhead.NativeMethods,
    JSProcessor = Hammerhead.JSProcessor,
    ShadowUI = Hammerhead.ShadowUI;

Hammerhead.init();

test('Clean up html', function () {
    var storedGetProxyUrl = UrlUtil.getProxyUrl;

    UrlUtil.getProxyUrl = function (url) {
        return url + '_proxy';
    };

    var html = '<a href="http://domain.com"></a>' +
        '<div ' + window.Hammerhead.TEST_CAFE_HOVER_PSEUDO_CLASS_ATTR + '></div>' +
        '<div ' + window.Hammerhead.TEST_CAFE_HOVER_PSEUDO_CLASS_ATTR + '=""></div>';

    var expexted = '<a href="http://domain.com"></a><div></div><div></div>',
        pocessedHtml = DomSandboxUtil.processHtml(html);

    equal(DomSandboxUtil.cleanUpHtml(pocessedHtml), expexted);

    UrlUtil.getProxyUrl = storedGetProxyUrl;
});

test('Clean up html: shadow ui elements', function () {
    var $uiElem = $('<div>').attr('id', 'uiElem');
    ShadowUI.addClass($uiElem, 'ui-elem-class');

    var el = document.createElement('div');

    el.appendChild($uiElem[0]);

    var html = '<head>' + el.innerHTML + '</head><body>' + el.innerHTML + '</body>';

    strictEqual(DomSandboxUtil.cleanUpHtml(html), '<head></head><body></body>');
});

test('Clean up html: form', function () {
    var storedGetProxyUrl = UrlUtil.getProxyUrl;

    UrlUtil.getProxyUrl = function (url) {
        return url + '_proxy';
    };

    var url = 'http://domain.com',
        pocessedHtml = '<form action="' + UrlUtil.getProxyUrl(url) + '" ' +
            PageProc.getStoredAttrName('action') + '="' + url + '"></form>',
        expexted = '<form action="http://domain.com"></form>';

    equal(DomSandboxUtil.cleanUpHtml(pocessedHtml), expexted);

    UrlUtil.getProxyUrl = storedGetProxyUrl;
});

test('Porocess iframe', function () {
    var originConvertToProxyUrl = UrlUtil.convertToProxyUrl,
        originParseProxyUrl = UrlUtil.parseProxyUrl;

    UrlUtil.convertToProxyUrl = function (url, isIFrame) {
        return 'http://example.proxy.com/' + (isIFrame ? 'iframe' : '');
    };
    UrlUtil.parseProxyUrl = function () {
        var result = {};

        result.originResourceInfo = UrlUtil.parseUrl('http://example.com/');

        return result;
    };

    ok(DomSandboxUtil.processHtml('<iframe src="http://example.com/">')
        .indexOf('http://example.proxy.com/iframe') !== -1);

    UrlUtil.convertToProxyUrl = originConvertToProxyUrl;
    UrlUtil.parseProxyUrl = originParseProxyUrl;
});

test('Process element with error in attribute', function () {
    var originalUrl = 'http://example.com/',
        src = '<script data-src="' + originalUrl + '"></script\>',
        processedSrc = src;

    src = DomSandboxUtil.processHtml(src);

    strictEqual(src, processedSrc);
});

test('Process html with encoded symbols', function () {
    var div = document.createElement('div'),
        tag = 'a',
        attr = 'href',
        storedAttr = PageProc.getStoredAttrName(attr),
        urlEncoded = 'http://example.com/?x=&lt;&y=5',
        urlDecoded = 'http://example.com/?x=<&y=5',
        proxyEncoded = $('<div/>').text(UrlUtil.getProxyUrl(urlDecoded)).html(),
        html = '<' + tag + ' ' + attr + '="' + urlEncoded + '"></' + tag + '>',
        processedHTML = '<' + tag + ' ' + attr + '="' + proxyEncoded + '" ' + storedAttr + '="' + urlEncoded + '"></' + tag + '>';

    div.innerHTML = DomSandboxUtil.processHtml(html) + processedHTML;

    equal(NativeMethods.getAttribute.call(div.firstChild, attr), NativeMethods.getAttribute.call(div.lastChild, attr));
    equal(NativeMethods.getAttribute.call(div.firstChild, storedAttr), NativeMethods.getAttribute.call(div.lastChild, storedAttr));
});

test('Process html with text node', function () {
    var error = false;

    window.onerror = function () {
        error = true;
    };

    DomSandboxUtil.processHtml('some text', 'div');

    ok(!error);
});

test('Process script inner html', function () {
    var html = DomSandboxUtil.processHtml('var v = a && b;', 'script');

    equal(html.replace(/\s/g,''), (PageProc.SCRIPT_HEADER + 'var v = a && b;').replace(/\s/g,''));
});

test('Process html fragment', function () {
    var storedGetProxyUrl = UrlUtil.getProxyUrl,
        htmlToProcess = $('<a href="www.google.com">Link</a>')[0].innerHTML,
        processedHTML = $('<a href="replaced" ' + PageProc.getStoredAttrName('href') + '="www.google.com">Link</a>')[0].innerHTML;

    UrlUtil.getProxyUrl = function () {
        return 'replaced';
    };

    var checkFragment = function (html, parentTag) {
        var src = html.replace('%s', htmlToProcess),
            expected = html.replace('%s', processedHTML);

        equal(DomSandboxUtil.processHtml(src, parentTag).replace(/\s/g, ''), expected.replace(/\s/g, ''));
    };

    checkFragment('<td>%s</td><td>Content</td>', 'tr');
    checkFragment('<td>Content1</td><td>Content2</td>', 'tr');
    checkFragment('<tr><td>%s</td><td>Content</td></tr>', 'table');
    checkFragment('<tr><td>Content1</td><td>Content2</td></tr>', 'table');
    checkFragment('%s', 'html');
    checkFragment('<div>Content</div>', 'html');

    UrlUtil.getProxyUrl = storedGetProxyUrl;
});

test('Text nodes', function () {
    var hoverAttr = window.Hammerhead.TEST_CAFE_HOVER_PSEUDO_CLASS_ATTR;

    var check = function (html) {
        var processedHtml = DomSandboxUtil.cleanUpHtml('<div ' + hoverAttr + '="">' + html + '</div>');

        equal(processedHtml, '<div>' + html + '</div>');
    };

    check('<table> \n\t <tfoot>textNode</tfoot><tbody> \n\t <tr> \n\t </tr></tbody></table>');
    check('<table> textNode' +
        '<thead>textNode</thead>textNode' +
        '<tfoot>textNode</tfoot>textNode' +
        '<tbody>textNode' +
        '<tr>textNode' +
        '<td>textNode</td>textNode' +
        '</tr>textNode' +
        '</tbody>textNode' +
        '</table>');
    check('<!--<h4>New Job</h4>-->' +
          '<!--<table>-->' +
          '<!--    <tr>-->' +
          '<!--        <td>Customer:</td>-->' +
          '<!--        <td>-->');
    check('<table>' +
          '  <colgroup>' +
          '    <col></col>' +
          '    <col/>' +
          '  </colgroup>' +
          '  <tr><td></td></tr>' +
          '</table>');
});

test('Process page html', function () {
    var storedGetProxyUrl = UrlUtil.getProxyUrl;

    UrlUtil.getProxyUrl = function (url) {
        return 'replaced' + url;
    };

    var check = function (html) {
        var processedHtml = DomSandboxUtil.processHtml(html);

        ok(processedHtml.indexOf('replacedBodyScript.js') !== -1);
        ok(processedHtml.indexOf('replacedHeadScript.js') !== -1);
        equal(html, DomSandboxUtil.cleanUpHtml(processedHtml));
    };

    check('<!DOCTYPE html><html><head><script src="HeadScript.js"><\/script></head><body><script src="BodyScript.js"><\/script></body></html>');
    check('<html><head><script src="HeadScript.js"><\/script></head><body><script src="BodyScript.js"><\/script></body></html>');
    check('<head><script src="HeadScript.js"><\/script></head><body><script src="BodyScript.js"><\/script></body>');
    check('<head><script src="HeadScript.js"><\/script><script src="BodyScript.js"><\/script></head>');
    check('<body><script src="HeadScript.js"><\/script><script src="BodyScript.js"><\/script></body>');
    check('<!DOCTYPE html><html><head><script src="HeadScript.js"><\/script></head><body>\u2028\u2029<script src="BodyScript.js"><\/script></body></html>');
    check('<html><head><script src="HeadScript.js"><\/script></head><body>\u2028\u2029<script src="BodyScript.js"><\/script></body></html>');
    check('<head param="value"><script src="HeadScript.js"><\/script></head><body>\u2028\u2029<script src="BodyScript.js"><\/script></body>');
    check('<head>\u2028\u2029<script src="HeadScript.js"><\/script></head><body param="value"><script src="BodyScript.js"><\/script></body>');
    check('<body><script src="HeadScript.js"><\/script>\u2028\u2029<script src="BodyScript.js"><\/script></body>');
    check('<body><script src="HeadScript.js"><\/script>\u2028\u2029<script src="BodyScript.js"><\/script></body>');

    UrlUtil.getProxyUrl = storedGetProxyUrl;
});


test('partial page html', function () {
    strictEqual(DomSandboxUtil.cleanUpHtml(DomSandboxUtil.processHtml('<!DOCTYPE html><html><head></head><body>')),
        '<!DOCTYPE html><html><head></head><body></body></html>');

    strictEqual(DomSandboxUtil.cleanUpHtml(DomSandboxUtil.processHtml('<html><head></head><body>')),
        '<html><head></head><body></body></html>');
});

test('Inject init script', function () {
    var pageHtmlTemplates = [
        '<!doctype><html><head>{0}</head><body>{0}</body></html>',
        '<html><head>{0}</head><body>{0}</body></html>',
        '<head>{0}</head>',
        '<body>{0}</body>'
    ];

    var check = function (template) {
        var html = template.replace(/\{0\}/g, ''),
            expectedProcessedHtml = template.replace(/\{0\}/g, DomSandboxUtil.INIT_SCRIPT_FOR_IFRAME_TEMPLATE);

        var processedHtml = DomSandboxUtil.processHtml(html);

        equal(processedHtml, expectedProcessedHtml);
        equal(DomSandboxUtil.cleanUpHtml(processedHtml), html);
    };

    for (var i = 0; i < pageHtmlTemplates.length; i++)
        check(pageHtmlTemplates[i]);
});

module('Regression tests');

test('T112153 - Click (Touch) events being swallowed when using a combination of TestCafé 14.1.1 + KendoUI Mobile + iOS', function () {
    var html =
        // <script type="script/x-kendo-template" id="inboxItem">
        "<h3 class=\"time\">#: Time#</h3><h3>#: From #</h3>\n" +
            "<a class=\"reply\"\n" +
            "data-role=\"button\"\n" +
            "data-rel=\"actionsheet\"\n" +
            "href=\"\\\\#inboxActions\"\n" +
            "data-actionsheet-context=\"#:ID#\">Reply</a>\n" +
            "<h2>#: Subject#</h2>\n" +
            "<p>#: Text#</p>";
    // </script\>

    equal(DomSandboxUtil.cleanUpHtml(html).replace(/\s/g, ''), html.replace(/\s/g, ''));
});

test('T226655: 15.1 Testing - Hammerhead DomSandboxUtil.processHtml function does not process body/head/html attributes', function () {
    var attrValue = 'var js = document.createElement(\'script\');js.src = \'http://google.com\'; document.body.appendChild(js);',
        expectedAttrValue = PageProc.processScript(attrValue, true).replace(/\s/g, ''),
        htmlWithBody = '<body onload="' + attrValue + '">',
        htmlWithHeadAndBody = '<head></head><body onload="' + attrValue + '"></body>';

    ok(DomSandboxUtil.processHtml(htmlWithBody).replace(/\s/g, '').replace(/&quot;/ig, '"').indexOf(expectedAttrValue) !== -1);
    ok(DomSandboxUtil.processHtml(htmlWithHeadAndBody).replace(/\s/g, '').replace(/&quot;/ig, '"').indexOf(expectedAttrValue) !== -1);
});

test('T226885: Hammerhead breaks Xenarius Designer', function () {
    var code = 'var t = 1;',
        $script = $('<script>' + code + '<\/script>');

    notEqual($script[0].innerHTML.replace(/^\s*|\s*$/g, ''), code);
    equal(eval(window[JSProcessor.PROCESS_SCRIPT_METH_NAME]('$script[0].innerHTML')).replace(/^\s*|\s*$/g, ''), code);

    $script.remove();
});

test('T459565: The Yahoo Mail login page appears incomplete during recording', function () {
    var html = '<noscript><div></noscript> <span></span> <noscript></div></noscript>';

    strictEqual(DomSandboxUtil.processHtml(html), html);
});
