var Hammerhead              = HammerheadClient.get('Hammerhead'),
    SharedConst             = TestCafeClient.get('Shared.Const'),
    Automation              = TestCafeClient.get('Automation'),
    IFrameSandbox           = HammerheadClient.get('DOMSandbox.IFrame'),
    HoverPlaybackAutomation = TestCafeClient.get('Automation.Hover.Playback'),
    CursorWidget            = TestCafeClient.get('UI.Cursor'),

    $                       = Hammerhead.$,
    Util                    = Hammerhead.Util,
    EventSandbox            = Hammerhead.EventSandbox,
    NativeMethods           = Hammerhead.NativeMethods,
    JSProcessor             = Hammerhead.JSProcessor,
    EventSimulator          = EventSandbox.Simulator;

Hammerhead.init();
Automation.init();
CursorWidget.init();

var handler = function (e) {
    if (e.iframe.id.indexOf('test') !== -1) {
        e.iframe.contentWindow.eval.call(e.iframe.contentWindow, [
            'HammerheadClient.define(\'Settings\', function (require, exports) {',
            '    exports.ORIGIN_URL_OBJ = { protocol: "https:", host: "example.com" };',
            '});',
            'HammerheadClient.get(\'Hammerhead\').init();'
        ].join(''));
    }
};

QUnit.testStart = function () {
    IFrameSandbox.on(IFrameSandbox.IFRAME_READY_TO_INIT, handler);
    IFrameSandbox.off(IFrameSandbox.IFRAME_READY_TO_INIT, IFrameSandbox.iframeReadyToInitHandler);
};

QUnit.testDone = function () {
    IFrameSandbox.off(IFrameSandbox.IFRAME_READY_TO_INIT, handler);
};

if (!Util.isTouchDevice) {
    asyncTest('Hover pseudo class', function () {
        var $parent   = $('<div style="width:100px; height:100px; background-color: Red">').appendTo($('body')),
            $child    = $('<div style="width:50px; height:50px; background-color: Blue">').appendTo($parent),

            isHovered = function ($el) {
                return $el.attr(window.Hammerhead.TEST_CAFE_HOVER_PSEUDO_CLASS_ATTR) === '';
            };

        ok(!isHovered($parent));
        ok(!isHovered($child));

        HoverPlaybackAutomation.run($parent[0], {}, function () {
            ok(isHovered($parent));
            ok(!isHovered($child));

            HoverPlaybackAutomation.run($child[0], {}, function () {
                ok(isHovered($parent));
                ok(isHovered($child));

                HoverPlaybackAutomation.run($('body')[0], {}, function () {
                    ok(!isHovered($parent));
                    ok(!isHovered($child));

                    $parent.remove();
                    $child.remove();

                    start();
                });
            });
        });
    });
}

module('Regression tests');

if (document.attachEvent) {
    test('Q532574 - TestCafe cannot proceed a third-party drop-down menu in IE', function () {
        var $div                = $('<div>').appendTo('body'),
            clickRaisedCount    = 0,
            docClickRaisedCount = 0;

        $div[0].attachEvent('onmousedown', function (e) {
            clickRaisedCount++;
        });

        document.attachEvent('onmousedown', function (e) {
            docClickRaisedCount++;
        });

        EventSimulator.mousedown($div[0]);

        equal(clickRaisedCount, 1);
        equal(docClickRaisedCount, 1);

        $div.remove();
    });
}

asyncTest('B253685 - Google Disk - page is crashed when a text document is opening', function () {
    var $iframe = $('<iframe id="test1">').appendTo('body');

    window.setTimeout(function () {
        var iframeDocument         = $iframe[0].contentWindow.document,
            iframeBody             = iframeDocument.body,
            blurOnIframeBodyRaised = false;

        equal(iframeDocument.activeElement, iframeBody);

        iframeBody.addEventListener('blur', function () {
            blurOnIframeBodyRaised = true;
        });

        $iframe[0].focus();

        EventSandbox.focus($iframe[0], function () {
            ok(!blurOnIframeBodyRaised, 'a blur event on the input must not be raised');

            start();
        });

    }, 500);
});

test('Q532574 - Check document addEventListener overriding', function () {
    var docClickRaisedCount = 0;

    document.addEventListener('mousedown', function (e) {
        docClickRaisedCount++;
    });

    EventSimulator.mousedown(document);

    equal(docClickRaisedCount, 1);
});

test('Q532574 - Fire and dispatch events from code, events created by different ways', function () {
    var $div                      = $('<div>').appendTo('body'),
        attachedHandlerCount      = 0,
        addedHandlerCount         = 0,
        inlineHandlerClickedCount = 0,
        jQueryHandlerClickedCount = 0,
        event                     = null;

    if ($div[0].attachEvent) {
        $div[0].attachEvent('onclick', function () {
            attachedHandlerCount++;
        });
    }
    $div[0].addEventListener('click', function () {
        addedHandlerCount++;
    });
    $div[0].onclick               = function () {
        inlineHandlerClickedCount++;
    };
    $div.click(function () {
        jQueryHandlerClickedCount++;
    });

    //createEvent
    event = document.createEvent('MouseEvents');
    event.initEvent('click', true, false);

    $div[0].dispatchEvent(event);
    equal(attachedHandlerCount, Util.isIE && Util.browserVersion < 11 ? 1 : 0);
    equal(addedHandlerCount, 1);
    equal(inlineHandlerClickedCount, 1);
    equal(jQueryHandlerClickedCount, 1);

    if ($div[0].fireEvent) {
        $div[0].fireEvent('onclick', event);
        equal(attachedHandlerCount, 2);
        equal(addedHandlerCount, 2);
        equal(inlineHandlerClickedCount, 2);
        equal(jQueryHandlerClickedCount, 2);
    }

    //createEventObject (we CAN NOT call dispatchEvent, only fire event)
    if (document.createEventObject) {
        event = document.createEventObject('MouseEvents');
        $div[0].fireEvent('onclick', event);

        equal(attachedHandlerCount, 3);
        equal(addedHandlerCount, 3);
        equal(inlineHandlerClickedCount, 3);
        equal(jQueryHandlerClickedCount, 3);
    }

    //new MouseEvent (this way not for IE and fireEvent)
    var error = false;

    if (!Util.isIE) {
        try {
            event = new MouseEvent('click', {
                'view':       window,
                'bubbles':    true,
                'cancelable': true
            });
        }
        catch (e) {
            //browser doesn't support this action
            error = true;
        }

        if (!error) {
            $div[0].dispatchEvent(event);
            equal(attachedHandlerCount, 0);
            equal(addedHandlerCount, 2);
            equal(inlineHandlerClickedCount, 2);
            equal(jQueryHandlerClickedCount, 2);
        }
    }
    $div.remove();
});

if (!Util.isTouchDevice) {
    asyncTest('B254111 - Recorder - hover does not freeze', function () {
        var $parent   = $('<div style="width:100px; height:100px; background-color: Red">').appendTo($('body')),
            $child    = $('<div style="width:50px; height:50px; background-color: Blue">').appendTo($parent),

            isHovered = function ($el) {
                return $el.attr(window.Hammerhead.TEST_CAFE_HOVER_PSEUDO_CLASS_ATTR) === '';
            };

        ok(!isHovered($parent));
        ok(!isHovered($child));

        EventSandbox.fixHoveredElement();

        HoverPlaybackAutomation.run($parent[0], {}, function () {
            ok(!isHovered($parent));
            ok(!isHovered($child));

            EventSandbox.freeHoveredElement();

            HoverPlaybackAutomation.run($child[0], {}, function () {
                ok(isHovered($parent));
                ok(isHovered($child));

                HoverPlaybackAutomation.run($('body')[0], {}, function () {
                    ok(!isHovered($parent));
                    ok(!isHovered($child));

                    $parent.remove();
                    $child.remove();

                    start();
                });
            });
        });
    });
}

test('T239606 - TD15.1 - Errors raise on page http://demos111.mootools.net/Drag.Absolutely', function () {
    var el = NativeMethods.createElement.call(document, 'A');

    var attachEventExist = !!el.attachEvent,
        fireEventExist   = !!el.fireEvent,
        detachEventExist = !!el.detachEvent;

    if (attachEventExist || fireEventExist || detachEventExist)
        ok(NativeMethods.attachEvent && NativeMethods.fireEvent && NativeMethods.detachEvent);
    else {
        EventSandbox.overrideElement(el);

        ok(!el.attachEvent);
        ok(!el.fireEvent);
        ok(!el.detachEvent);
    }
});

asyncTest('T137892 - Health monitor - incorrectly makes copy message event (taobao.com)', function () {
    window.addEventListener('message', function (e) {
        try {
            e.stopPropagation();
            ok(true);
        } catch (e) {
            ok(false);
        } finally {
            start();
        }
    });

    eval(window[JSProcessor.PROCESS_SCRIPT_METH_NAME]('window.postMessage("hello", "*")'));
});

if (!$.browser.msie || $.browser.version !== '9.0') {
    asyncTest('T203986 - TestCafe - A test failed in IE9, IE11 and Safari with different error messages', function () {
        var str = 'success';

        setTimeout(function (msg) {
            strictEqual(msg, str);
            start();
        }, 10, str)
    });
}

asyncTest('T261234 - All cases of the listener parameter aren\'t overrided in addEventListener function', function () {
    var divEl          = document.body.appendChild(document.createElement('div')),
        eventObjOrigin = {
            clickCount:  0,
            handleEvent: function () {
                this.clickCount++;
            }
        },
        eventObjWrap   = {
            clickCount:  0,
            handleEvent: function () {
                this.clickCount++;

                test();
            }
        },
        test           = function () {
            strictEqual(eventObjOrigin.clickCount, 1);
            strictEqual(eventObjWrap.clickCount, 1);

            document.body.removeChild(divEl);
            start();
        };

    EventSandbox.initElementListening(divEl);

    NativeMethods.addEventListener.call(divEl, 'click', eventObjOrigin);
    divEl.addEventListener('click', eventObjWrap);

    expect(2);
    divEl.click();
});

if (Util.isWebKit) {
    asyncTest('The "Illegal invocation" error after svg element focused (#82)', function () {
        var $svgElement = $(
            '<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1">' +
            '<rect id="rect" width="300" height="300" fill="red" tabIndex="1"></rect>' +
            '</svg>').appendTo('body');

        window[Hammerhead.DOM_SANDBOX_OVERRIDE_DOM_METHOD_NAME]($svgElement[0]);

        var rectElement = document.getElementById('rect');

        rectElement.onfocus = function () {
            rectElement.onblur = function () {
                ok(true);
                $svgElement.remove();
                start();
            };
            rectElement.blur();
        };
        rectElement.focus();
    });
}
