var Hammerhead   = HammerheadClient.get('Hammerhead');
var EventSandbox = Hammerhead.EventSandbox;
var Util         = Hammerhead.Util;

Hammerhead.init();

var inputTestValue  = 'test@host.net';
var isIE            = Util.isIE;
var isMobileBrowser = Util.isIOS || Util.isAndroid;

var CHROME_VERSION_RE = /chrom(e|ium)\/([0-9]+)\./;

function getChromeVersion () {
    var match = navigator.userAgent.toLowerCase().match(CHROME_VERSION_RE);

    return match ? parseInt(match[2], 10) : 0;
}


var createTestInput = function () {
    var input = document.createElement('input');

    input.setAttribute('type', 'email');

    document.body.appendChild(input);

    input.value = inputTestValue;

    return input;
};

module('Selection');

test('Set and get selection on input with "email" type', function () {
    var input = createTestInput();

    var testSelection = {
        start:     2,
        end:       9,
        direction: 'backward'
    };

    input.setSelectionRange(testSelection.start, testSelection.end, testSelection.direction);

    var selection = EventSandbox.getSelection(input);

    strictEqual(selection.start, testSelection.start);
    strictEqual(selection.end, testSelection.end);

    if (!isIE)
        strictEqual(selection.direction, testSelection.direction);

    document.body.removeChild(input);
});

test('Get selection on input with "email" type', function () {
    var input     = createTestInput();
    var selection = EventSandbox.getSelection(input);

    strictEqual(selection.start, Util.isMozilla && Util.browserVersion < 51 ||
                                 isIE ? 0 : inputTestValue.length);
    strictEqual(selection.end, Util.isMozilla && Util.browserVersion < 51 || isIE ? 0 : inputTestValue.length);

    if (isMobileBrowser || Util.isSafari || Util.isWebKit && (Util.isMacPlatform || getChromeVersion() < 54))
        strictEqual(selection.direction, 'none');
    else if (!isIE)
        strictEqual(selection.direction, 'forward');

    document.body.removeChild(input);
});
