var Hammerhead = HammerheadClient.get('Hammerhead'),
    $ = Hammerhead.$,
    UrlUtil = HammerheadClient.get('UrlUtil'),
    SharedErrors = HammerheadClient.get('Shared.Errors'),
    Settings = HammerheadClient.get('Settings');

var PROXY_PORT = 1337,
    PROXY_HOSTNAME = '127.0.0.1',
    PROXY_HOST = PROXY_HOSTNAME + ':' + PROXY_PORT,
    ENCODED_DESCRIPTOR_VALUES_SEPARATOR = encodeURI(UrlUtil.REQUEST_DESCRIPTOR_VALUES_SEPARATOR);

var createProxyParam = function(protocol, hostname, port, jobUid, jobOwnerToken) {
    return [ UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY + '=' + protocol + '%3A', hostname + (port !== null ? '%3A' + port : ''),
        jobUid, jobOwnerToken ].join(ENCODED_DESCRIPTOR_VALUES_SEPARATOR);
};

test('getCrossDomainProxyUrl method', function () {
    var storedCrossDomainport = Settings.CROSS_DOMAIN_PROXY_PORT;

    Settings.CROSS_DOMAIN_PROXY_PORT = 5555;

    strictEqual(UrlUtil.getCrossDomainProxyUrl(), 'http://' + location.hostname + ':5555/');

    Settings.CROSS_DOMAIN_PROXY_PORT = storedCrossDomainport;
});

test('sameOriginCheck method', function () {
    ok(UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|origin.com%3A111', 'http://origin.com:111/index.php'));
    ok(UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|origin.com%3A111', 'http://sub.origin.com:111/index.php'));
    ok(UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|origin.com%3A111', 'http://sub1.sub2.origin.com:111/index.php'));
    ok(UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|sub.origin.com%3A111', 'http://origin.com:111/index.php'));
    ok(UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|sub.origin.com%3A111', 'http://location/index.php'));
    ok(UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|www.origin.com', 'http://origin.com/'));
    ok(!UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|origin.com%3A111', 'http://origin.com/index.php'));
    ok(!UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|sub.origin.com%3A111', 'http://location:111/index.php'));
    ok(!UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|sub.origin.com%3A111', 'https://location/index.php'));
    ok(!UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|origin.com%3A111', 'http://origin.com:222/index.php'));
    ok(!UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|origin.com%3A111', 'https://origin.com:111/index.php'));
    ok(!UrlUtil.sameOriginCheck('http://location/index.html?7929ba6d39aa4465=http%3A|origin.com%3A111', 'http://origin2.com:111/index.php'));
});

test('resolveUrl', function () {
    var storedGetOriginUrlObj = UrlUtil.getOriginUrlObj;

    UrlUtil.getOriginUrlObj = function () {
        return {
            protocol: 'protocol:'
        };
    };

    equal(UrlUtil.resolveUrl('//domain.com/index.php'), 'protocol://domain.com/index.php');
    equal(UrlUtil.resolveUrl('//dom\n\tain.com/index.php'), 'protocol://domain.com/index.php');

    UrlUtil.getOriginUrlObj = storedGetOriginUrlObj;

    equal(UrlUtil.resolveUrl(location), location.toString());
});

test('resolveUrlAsOrigin', function () {
    var storedGetOriginUrlObj = UrlUtil.getOriginUrlObj;

    UrlUtil.getOriginUrlObj = function () {
        return {
            protocol: 'https:',
            port: 1338,
            host: 'google.com:1338',
            hostname: 'google.com'
        };
    };

    equal(UrlUtil.resolveUrlAsOrigin('/index.html#hash'), 'https://google.com:1338/index.html#hash');
    equal(UrlUtil.resolveUrlAsOrigin('javascript:0;'), 'javascript:0;');
    equal(UrlUtil.resolveUrlAsOrigin('/index.html?param=value#hash'), 'https://google.com:1338/index.html?param=value#hash');
    equal(UrlUtil.resolveUrlAsOrigin('https://twitter.com/index.html?param=value#hash'), 'https://twitter.com/index.html?param=value#hash');
    equal(UrlUtil.resolveUrlAsOrigin('//twitter.com/index.html?param=value#hash'), 'https://twitter.com/index.html?param=value#hash');
    equal(UrlUtil.resolveUrlAsOrigin('http://g.tbcdn.cn/??kissy/k/1.4.2/seed-min.js'), 'http://g.tbcdn.cn/??kissy/k/1.4.2/seed-min.js');

    UrlUtil.getOriginUrlObj = storedGetOriginUrlObj;
});

test('Parse url with newline characters', function() {
    var url = 'http://exa\nmple.com/?par\n=val',
        parsingResult = UrlUtil.parseUrl(url);

    strictEqual(parsingResult.hostname, 'example.com');
    strictEqual(parsingResult.query['par'], 'val');
});

test('Parse url with tabulation characters', function() {
    var url = 'http://exa\tmple.com/?par\t=val',
        parsingResult = UrlUtil.parseUrl(url);

    strictEqual(parsingResult.hostname, 'example.com');
    strictEqual(parsingResult.query['par'], 'val');
});

test('Extract origin url object', function () {
    var pathname = '/path/index.html',
        hash = '#testHash',
        search = '?param1&param2=value&' + createProxyParam('http', 'test.example.com', 53, 'MyUID', 'ownerToken'),
        proxyUrl = 'http://localhost:1338' + pathname + search + hash;


    var originUrlObj = UrlUtil.getOriginUrlObj(proxyUrl);

    strictEqual(originUrlObj.protocol, 'http:', 'protocol');
    strictEqual(originUrlObj.port, '53', 'port');
    strictEqual(originUrlObj.hostname, 'test.example.com', 'hostname');
    strictEqual(originUrlObj.host, 'test.example.com:53', 'host');
    strictEqual(originUrlObj.pathname, pathname, 'pathname');
    strictEqual(originUrlObj.hash, hash, 'hash');
    strictEqual(originUrlObj.search, '?param1&param2=value', 'search');

    ok(originUrlObj.query.hasOwnProperty('param1') && !originUrlObj.query['param1']);
    ok(originUrlObj.query['param2'] === 'value');
});

test('Query string with empty params', function() {
    var url = '//localhost/?&param1&param2=&param3=value',
        parsedUrl = UrlUtil.parseUrl(url);

    strictEqual(parsedUrl.query[''], null);
    strictEqual(parsedUrl.query['param1'], null);
    strictEqual(parsedUrl.query['param2'], '');
    strictEqual(parsedUrl.query['param3'], 'value');
    strictEqual(UrlUtil.formatUrl(parsedUrl), url);
});

module('Get proxy url');

test('Already proxied url', function () {
    var originUrl = 'http://test.example.com/';
    var proxyUrl = UrlUtil.getProxyUrl(originUrl, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken');
    var newUrl = UrlUtil.getProxyUrl(proxyUrl, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken', 'iframe');

    strictEqual(UrlUtil.parseProxyUrl(newUrl).resourceType, 'iframe');

});

test('Origin with query, path, hash and host', function () {
    var originUrl = 'http://test.example.com/pa/th/Page?param1=value&param2=&param3#testHash',
        proxyUrl = UrlUtil.getProxyUrl(originUrl, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken');

    strictEqual(proxyUrl, 'http://' + PROXY_HOST + '/pa/th/Page?param1=value&param2=&param3&' +
        createProxyParam('http', 'test.example.com', null, 'MyUID', 'ownerToken') + '#testHash');
});

test('Origin with empty hash', function () {
    var originUrl = 'http://test.example.com/pa/th/Page#',
        proxyUrl = UrlUtil.getProxyUrl(originUrl, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken');

    strictEqual(proxyUrl, 'http://' + PROXY_HOST + '/pa/th/Page?' +
                          createProxyParam('http', 'test.example.com', null, 'MyUID', 'ownerToken') + '#');
});

test('Origin with host only', function () {
    var originUrl = 'http://test.example.com/',
        proxyUrl = UrlUtil.getProxyUrl(originUrl, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken');

    strictEqual(proxyUrl, 'http://' + PROXY_HOST + '/?' + createProxyParam('http', 'test.example.com', null, 'MyUID', 'ownerToken'));
});

test('Origin with https protocol', function () {
    var originUrl = 'https://test.example.com:53/',
        proxyUrl = UrlUtil.getProxyUrl(originUrl, PROXY_HOSTNAME, PROXY_PORT, 'My UID', 'owner Token');

    strictEqual(proxyUrl, 'http://' + PROXY_HOST + '/?' +
        createProxyParam('https', 'test.example.com', 53, 'My%20UID', 'owner%20Token'));
});

test('Origin with non http or https protocol', function () {
    expect(2);

    var originUrl = 'someProtocol://test.example.com:53/';

    try {
        UrlUtil.getProxyUrl(originUrl, PROXY_HOSTNAME, PROXY_PORT)
    } catch (err) {
        strictEqual(err.code, SharedErrors.URL_UTIL_PROTOCOL_IS_NOT_SUPPORTED);
        strictEqual(err.originUrl.toLowerCase(), originUrl.toLowerCase());
    }
});

test('Relative path', function () {
    var Settings = HammerheadClient.get('Settings');

    Settings.ORIGIN_URL_OBJ = {
        protocol: 'https:',
        host: 'example.com'
    };

    Settings.JOB_UID = 'job';
    Settings.JOB_OWNER_TOKEN = 'token';

    var originUrl = '/Image1.jpg',
        proxyUrl = UrlUtil.getProxyUrl(originUrl);

    strictEqual(proxyUrl, 'http://' + location.host + '/Image1.jpg?' +
        createProxyParam('https', 'example.com', null, 'job', 'token'));

    var relativeUrl = 'share?id=1kjQMWh7IcHdTBbTv6otRvCGYr-p02q206M7aR7dmog0',
        parsedUrl = UrlUtil.parseUrl(relativeUrl);

    ok(!parsedUrl.hostname);
    ok(!parsedUrl.host);
    ok(!parsedUrl.hash);
    ok(!parsedUrl.port);
    ok(!parsedUrl.protocol);
    equal(parsedUrl.query['id'], '1kjQMWh7IcHdTBbTv6otRvCGYr-p02q206M7aR7dmog0');
    equal(parsedUrl.pathname, '/share');
});

test('Url contains successive question marks in query', function () {
    expect(1);

    var originUrl = 'http://test.example.com/??dirs/???files/',
        proxyUrl = UrlUtil.getProxyUrl(originUrl, '127.0.0.1', PROXY_PORT, 'MyUID', 'ownerToken');

    strictEqual(proxyUrl, 'http://' + PROXY_HOST + '/??dirs/???files/&' + createProxyParam('http', 'test.example.com', null, 'MyUID', 'ownerToken'));
});

test('Origin with port', function () {
    var originUrl = 'http://test.example.com:53/',
        proxyUrl = UrlUtil.getProxyUrl(originUrl, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken');

    strictEqual(proxyUrl, 'http://' + PROXY_HOST + '/?' + createProxyParam('http', 'test.example.com', 53, 'MyUID', 'ownerToken'));
});

test('Url with value undefined or null', function() {
    var a = document.createElement('a'),
        proxyUrl = UrlUtil.getProxyUrl(null, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken');

    a.href = null;

    equal(proxyUrl, UrlUtil.getProxyUrl(a.href, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken'), 'null');

    proxyUrl = UrlUtil.getProxyUrl(undefined, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken');

    a.href = undefined;

    equal(proxyUrl, UrlUtil.getProxyUrl(a.href, PROXY_HOSTNAME, PROXY_PORT, 'MyUID', 'ownerToken'), 'undefined');
});

module('Parse proxy url');

test('Parse http URL', function () {
    var proxyUrl = 'http://' + PROXY_HOST + '/PA/TH/?' +
            createProxyParam('http', 'test.example.com', 53, 'My%20UID', 'owner%20Token') + '#testHash',
        parsingResult = UrlUtil.parseProxyUrl(proxyUrl);

    strictEqual(parsingResult.originResourceInfo.protocol, 'http:');
    strictEqual(parsingResult.originResourceInfo.host, 'test.example.com:53');
    strictEqual(parsingResult.originResourceInfo.hostname, 'test.example.com');
    strictEqual(parsingResult.originResourceInfo.port, '53');
    strictEqual(parsingResult.originResourceInfo.pathname, '/PA/TH/');
    strictEqual(parsingResult.originResourceInfo.hash, '#testHash');
    strictEqual(parsingResult.jobInfo.uid, 'My UID');
    strictEqual(parsingResult.jobInfo.ownerToken, 'owner Token');
});

test('Parse https URL', function () {
    var proxyUrl = 'http://' + PROXY_HOST + '/PA/TH/?' +
            createProxyParam('https', 'test.example.com', 53, 'MyUID', 'ownerToken') + '#testHash',
        parsingResult = UrlUtil.parseProxyUrl(proxyUrl);

    strictEqual(parsingResult.originResourceInfo.protocol, 'https:');
    strictEqual(parsingResult.originResourceInfo.host, 'test.example.com:53');
    strictEqual(parsingResult.originResourceInfo.hostname, 'test.example.com');
    strictEqual(parsingResult.originResourceInfo.port, '53');
    strictEqual(parsingResult.originResourceInfo.pathname, '/PA/TH/');
    strictEqual(parsingResult.originResourceInfo.hash, '#testHash');
    strictEqual(parsingResult.jobInfo.uid, 'MyUID');
    strictEqual(parsingResult.jobInfo.ownerToken, 'ownerToken');
});

test('Parse non-proxy URL', function () {
    var proxyUrl = 'http://' + PROXY_HOST + '/PA/TH/?someParam=value',
        originUrlInfo = UrlUtil.parseProxyUrl(proxyUrl);

    ok(!originUrlInfo);
});

test('Parse URL with successive question marks', function () {
    var proxyUrl = 'http://' + PROXY_HOST + '/??dirs/???files/&' +
            createProxyParam('http', 'test.example.com', 53, 'MyUID', 'ownerToken') + '#testHash',
        parsingResult = UrlUtil.parseProxyUrl(proxyUrl);

    strictEqual(parsingResult.originResourceInfo.protocol, 'http:');
    strictEqual(parsingResult.originResourceInfo.host, 'test.example.com:53');
    strictEqual(parsingResult.originResourceInfo.hostname, 'test.example.com');
    strictEqual(parsingResult.originResourceInfo.port, '53');
    strictEqual(parsingResult.originResourceInfo.pathname, '/');
    strictEqual(parsingResult.originResourceInfo.hash, '#testHash');
    strictEqual(parsingResult.originResourceInfo.query['?dirs/???files/'], null);
    strictEqual(parsingResult.jobInfo.uid, 'MyUID');
    strictEqual(parsingResult.jobInfo.ownerToken, 'ownerToken');
});

test('Empty parameter', function () {
    var paresedUrl = UrlUtil.parseUrl('http://google.com/?prarm1=1&');

    strictEqual(Object.keys(paresedUrl.query).length, 2);
});

test('Single question mark', function () {
    var url = 'http://ac-gb.marketgid.com/p/j/2865/11?',
        proxyUtrl = UrlUtil.getProxyUrl(url, 'hostname', 1111, 'MyUID', 'ownerToken');

    equal(url, UrlUtil.formatUrl(UrlUtil.parseProxyUrl(proxyUtrl).originResourceInfo));
});

test('Health monitor - question mark disappears', function() {
    var url = 'http://google.ru:345/path?',
        parsedUrl = UrlUtil.parseUrl(url);

    strictEqual(Object.keys(parsedUrl.query).length, 1);
    strictEqual(parsedUrl.query[''], null);
    strictEqual(UrlUtil.formatUrl(parsedUrl), url);

    url = 'http://yandex.ru:234/path';
    parsedUrl = UrlUtil.parseUrl(url);

    strictEqual(Object.keys(parsedUrl.query).length, 0);
    strictEqual(UrlUtil.formatUrl(parsedUrl), url);
});

module('Change proxy url');

test('Parse and change proxy URL', function () {
    var proxyUrl = 'http://test.com/test.html?' + createProxyParam('http', 'test.example.com', 53, 'MyUID', 'ownerToken') + '#testHash';

    strictEqual(UrlUtil.changeProxyUrl(proxyUrl, 'port', '34'), 'http://test.com/test.html?' +
        createProxyParam('http', 'test.example.com', 34, 'MyUID', 'ownerToken') + '#testHash', 'Change port');

    strictEqual(UrlUtil.changeProxyUrl(proxyUrl, 'host', 'newhost:80'), 'http://test.com/test.html?' +
        createProxyParam('http', 'newhost', 80, 'MyUID', 'ownerToken') + '#testHash', 'Change host');

    strictEqual(UrlUtil.changeProxyUrl(proxyUrl, 'hostname', 'newhostname'), 'http://test.com/test.html?' +
        createProxyParam('http', 'newhostname', 53, 'MyUID', 'ownerToken') + '#testHash', 'Change hostname');

    strictEqual(UrlUtil.changeProxyUrl(proxyUrl, 'protocol', 'https:'), 'http://test.com/test.html?' +
        createProxyParam('https', 'test.example.com', 53, 'MyUID', 'ownerToken') + '#testHash', 'Change protocol');

    strictEqual(UrlUtil.changeProxyUrl(proxyUrl, 'pathname', 'test1.html'), 'http://test.com/test1.html?' +
        createProxyParam('http', 'test.example.com', 53, 'MyUID', 'ownerToken') + '#testHash', 'Change pathname');

    strictEqual(UrlUtil.changeProxyUrl(proxyUrl, 'hash', 'newHash'), 'http://test.com/test.html?' +
        createProxyParam('http', 'test.example.com', 53, 'MyUID', 'ownerToken') + '#newHash', 'Change newHash');

    strictEqual(UrlUtil.changeProxyUrl(proxyUrl, 'search', '?hl=ru&tab=wn'), 'http://test.com/test.html?hl=ru&tab=wn&' +
        createProxyParam('http', 'test.example.com', 53, 'MyUID', 'ownerToken') + '#testHash', 'Change search');
});

module('Encoding / Decoding');

test('Params encoding', function () {
    strictEqual(UrlUtil.formatUrl({ hostname: 'localhost', query: { param: 'simpleValue' }}),
        '//localhost?param=simpleValue');

    strictEqual(UrlUtil.formatUrl({ hostname: 'localhost', query: { param: 'encodable;,Value' }}),
        '//localhost?param=encodable;,Value');

    var urlObjWithDesc = {
        hostname: 'localhost',
        query: { param: 'encodable;,Value' }
    };

    urlObjWithDesc.query[UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY] = 'descEncodable;,Value';

    strictEqual(UrlUtil.formatUrl(urlObjWithDesc),
        '//localhost?param=encodable;,Value&' + UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY + '=descEncodable%3B%2CValue');
});

test('Params decoding', function () {
    var parsedUrl = UrlUtil.parseUrl('//localhost?param=simpleValue');

    ok(parsedUrl.query['param'] === 'simpleValue');

    parsedUrl = UrlUtil.parseUrl('//localhost?param=decodable%3B%2CValue');

    ok(parsedUrl.query['param'] === 'decodable%3B%2CValue');

    parsedUrl = UrlUtil.parseUrl('//localhost?param=decodable%3B%2CValue&' + UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY + '=descDecodable%3B%2CValue');

    ok(parsedUrl.query['param'] === 'decodable%3B%2CValue' &&
        parsedUrl.query[UrlUtil.REQUEST_DESCRIPTOR_QUERY_KEY] === 'descDecodable;,Value'
    );
});

module('Regression tests');

test('B239791 - param value can has "=" symbol', function () {
    var url = 'http://api-maps.yandex.ru/2.0.33/release/combine.xml?' +
        'param1=value=value=value&param2=value=value';

    var paresedUrl = UrlUtil.parseUrl(url),
        paramCount = 0;

    for (var key in paresedUrl.query) {
        if (paresedUrl.query.hasOwnProperty(key))
            paramCount++;
    }

    strictEqual(paramCount, 2);
    strictEqual(paresedUrl.query['param1'], 'value=value=value');
    strictEqual(paresedUrl.query['param2'], 'value=value');
});

test('T106172 - Health monitor - cross-domain errors(rediff.com)', function () {
    ok(UrlUtil.sameOriginCheck('http://www.rediff.com', 'http://money.rediff.com'));
});
