var Hammerhead  = HammerheadClient.get('Hammerhead'),
    SharedConst = HammerheadClient.get('Shared.Const'),
    UrlUtil     = HammerheadClient.get('UrlUtil'),

    $           = Hammerhead.$,
    Util        = Hammerhead.Util;

test('isDomElement', function () {
    ok(Util.isDomElement(document.body));
    ok(Util.isDomElement(document.createElement('span')));
    ok(Util.isDomElement(document.createElement('strong')));
    ok(Util.isDomElement(document.createElement('a')));
    ok(!Util.isDomElement(null));
    ok(!Util.isDomElement(Element.prototype)); // T184805
});

asyncTest('getTopSameDomainWindow', function () {
    var $iframe = $('<iframe>');

    $iframe.load(function () {
        equal(Util.getTopSameDomainWindow(window.top), window.top);
        equal(Util.getTopSameDomainWindow(this.contentWindow), window.top);

        $iframe.remove();
        start();
    });

    $iframe.appendTo('body');
});

test('isWindowInstance', function () {
    ok(Util.isWindowInstance(window));
    ok(!Util.isWindowInstance({ top: '' }));
    ok(!Util.isWindowInstance(
        [{
            toString: function () {
                ok(false);
            }
        }
        ]));

    var storedToString = window.toString;

    window.toString = function () {
        throw 'eid library overrides window.toString() method';
    };

    ok(Util.isWindowInstance(window));

    window.toString = storedToString;
});

asyncTest('isWindow for a cross-domain window', function () {
    var iframe = document.createElement('iframe');

    iframe.id  = 'test' + Date.now();
    iframe.src = 'http://localhost:1336/simple_page.html';

    iframe.addEventListener('load', function () {
        var iframeWindow = iframe.contentWindow;

        ok(Util.isWindowInstance(iframeWindow));

        // NOTE: The firefox does not provide access to the cross-domain location.
        if (!Util.isMozilla)
            ok(!Util.isWindowInstance(iframeWindow.location));

        iframe.parentNode.removeChild(iframe);
        start();
    });

    document.body.appendChild(iframe);
});

test('isNodeList', function () {
    ok(Util.isNodeList(document.body.childNodes));
});

test('isHTMLCollection', function () {
    ok(Util.isHTMLCollection(document.body.children));
});

module('isIframeWithoutSrc');

//NOTE: we've commented the following test because we don't process the following scenario now. When we will meet this case
//in a real site we will process it
/*asyncTest('isIframeWithoutSrc: changed location 1', function () {
    var handler = function () {
        $(this).unbind('load', handler);
        $(this).bind('load', function () {
            this[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT] = window;
            ok(Util.isIframeWithoutSrc(this));
            ok(!Util.isCrossDomainIframe(this));
            $(this).remove();

            start();
        });
        this.contentWindow.location = 'about:blank';
    };

    $('<iframe src="http://' + location.host + '/">').bind('load', handler).appendTo('body')[0];
});*/

asyncTest('isIframeWithoutSrc: changed location 2', function () {
    var handler = function () {
        $(this).unbind('load', handler);
        $(this).bind('load', function () {
            this[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT] = window;
            ok(!Util.isIframeWithoutSrc(this));
            ok(!Util.isCrossDomainIframe(this));
            $(this).remove();

            start();
        });
        this.contentWindow.location = 'http://' + location.host + '/';
    };

    $('<iframe src="about:blank">').bind('load', handler).appendTo('body')[0];
});

asyncTest('isIframeWithoutSrc: crossdomain src', function () {
    $('<iframe src="http://' + location.hostname + ':1336/simple_page.html">').bind('load', function () {
        this[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT] = window;
        ok(!Util.isIframeWithoutSrc(this));
        ok(Util.isCrossDomainIframe(this));
        $(this).remove();

        start();
    }).appendTo('body');
});

asyncTest('isIframeWithoutSrc: samedomain src', function () {
    $('<iframe src="http://' + location.host + '/">').bind('load', function () {
        this[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT] = window;
        ok(!Util.isIframeWithoutSrc(this));
        ok(!Util.isCrossDomainIframe(this));
        $(this).remove();

        start();
    }).appendTo('body');
});

asyncTest('isIframeWithoutSrc: without src attribute', function () {
    $('<iframe>').bind('load', function () {
        this[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT] = window;
        ok(Util.isIframeWithoutSrc(this));
        ok(!Util.isCrossDomainIframe(this));
        $(this).remove();
        start();
    }).appendTo('body');
});

asyncTest('isIframeWithoutSrc: about:blank', function () {
    $('<iframe src="about:blank">').bind('load', function () {
        this[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT] = window;
        ok(Util.isIframeWithoutSrc(this));
        ok(!Util.isCrossDomainIframe(this));
        $(this).remove();
        start();
    }).appendTo('body');
});

module('isCrossDomainIFrame');

asyncTest('isCrossDomainIFrame: location is changed to cross-domain', function () {
    var iteration = 0;
    expect(4);
    $('<iframe src="http://' + location.host + '/">').bind('load', function () {
        if (!iteration) {
            ok(!Util.isCrossDomainIframe(this));
            ok(!Util.isCrossDomainIframe(this, true));

            this.contentDocument.location.href = 'http://' + location.hostname + ':1336/simple_page.html';
            iteration++;
        }
        else {
            ok(Util.isCrossDomainIframe(this));
            ok(!Util.isCrossDomainIframe(this, true));
            $(this).remove();
            start();
        }
    }).appendTo('body');
});

asyncTest('isIframeWithoutSrc: empty src attribute', function () {
    $('<iframe src>').bind('load', function () {
        this[SharedConst.DOM_SANDBOX_PROCESSED_CONTEXT] = window;
        ok(Util.isIframeWithoutSrc(this));
        ok(!Util.isCrossDomainIframe(this));
        $(this).remove();
        start();
    }).appendTo('body');
});

module('isIFrameWindowInDOM');

asyncTest('isIFrameWindowInDOM', function () {
    expect(Util.isIE ? 2 : 1);

    var messageCounter = 0;

    function finishTest () {
        window.removeEventListener('message', onMessage, false);
        start();
    }

    function onMessage (event) {
        if (messageCounter === 0) {
            equal(event.data, true);

            var iFramePostMessage = $iFrame[0].contentWindow.postMessage.bind($iFrame[0].contentWindow);
            $iFrame.remove();

            //NOTE: In WebKit, scripts cannot be executed in a removed iframe. Therefore, the test is finished here.
            if (Util.isIE)
                iFramePostMessage('isIFrameWindowInDOM', '*');
            else
                finishTest();
        }
        else {
            equal(event.data, false);
            finishTest();
        }

        messageCounter++;
    }

    window.addEventListener('message', onMessage, false);

    var $iFrame = $('<iframe>')
        .attr('src', window.getCrossDomainPageUrl('iframe_in_dom.html'))
        .bind('load', function () {
            $iFrame[0].contentWindow.postMessage('isIFrameWindowInDOM', '*');
        }).appendTo('body');

});
