var Hammerhead = HammerheadClient.get('Hammerhead');
var XHRSandbox = HammerheadClient.get('DOMSandbox.Xhr');

var JSProcessor = Hammerhead.JSProcessor,
    NativeMethods  = Hammerhead.NativeMethods;

Hammerhead.init();

module('Regression tests');

asyncTest('T239198: TD15.1 - web page doesnt display correctly (hellomonday.com)', function(){
    var request = new XMLHttpRequest();

    request.addEventListener("progress", function(event) {
        ok(event.target);
    }, true);

    request.addEventListener('load', function(event){
        ok(event.target);
        start();
    }, true);

    request.open("GET", '/xhr-large-response', true);
    request.send(null);
});

test('the internal 222 status code should be replaced to 0 on the client side', function () {
    var xhr = new XMLHttpRequest();

    xhr.open('GET', '/xhr-222/', false);
    xhr.send();

    strictEqual(xhr.response || xhr.responseText, 'true');
    strictEqual(window[JSProcessor.GET_PROPERTY_METH_NAME](xhr, 'status'), 0);
});

test('createNativeXHR', function () {
    window.XMLHttpRequest = function () {};

    var xhr = XHRSandbox.createNativeXHR();

    ok(xhr instanceof NativeMethods.XMLHttpRequest);

    window.XMLHttpRequest = NativeMethods.XMLHttpRequest;
});

asyncTest('T468796 - ignore broken "xhr.onreadystatechange" property while sending xhr', function () {
    var originalWindowOnError = window.onerror;

    var xhr          = new window.XMLHttpRequest();
    var isError      = false;

    var throwError = function () {
        throw new Error('Custom error');
    };

    window.onerror = function () {
        isError = true;

        originalWindowOnError.apply(window, arguments);
    };

    Object.defineProperty(xhr, 'onreadystatechange', { get: throwError, set: throwError });

    xhr.open('GET', '/xhr-test/200', true);
    xhr.send();

    window.setTimeout(function () {
        ok(!isError);

        window.onerror = originalWindowOnError;

        start();
    }, 300);
});
