var Hammerhead = HammerheadClient.get('Hammerhead'),
    $ = Hammerhead.$,
    Automation = TestCafeClient.get('Automation'),
    Util = Hammerhead.Util,
    NativeMethods = Hammerhead.NativeMethods;

Hammerhead.init();
Automation.init();

$(document).ready(function () {
    var $el = null,
        press = TestCafeClient.get('Automation.Press.KeyPressSimulator').press,

    //constants
        TEST_ELEMENT_CLASS = 'testElement',

    //utils
        addInputElement = function (type, id, x, y) {
            var elementString = ['<input type="', type, '" id="', id, '" value="', id, '" />'].join('');
            return $(elementString)
                .css({
                    position: 'absolute',
                    marginLeft: x + 'px',
                    marginTop: y + 'px'
                })
                .addClass(type)
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo('body');
        },

        eventsLog = '',
        eventsSeparator = '-',
        logEvent = function (e) {
            eventsLog = [
                eventsLog,
                getEventLog(e)
            ].join(eventsSeparator);
        },
        createCheckingLog = function (events) {
            var log = '';
            for (var i = 0; i < events.length; i++) {
                log = [
                    log,
                    getEventLog(events[i])
                ].join(eventsSeparator);
            }
            return log;
        },
        getEventLog = function (e) {
            return                    [
                e.type,
                e.keyCode || 0,
                e.charCode || 0,
                e.ctrlKey || false,
                e.shiftKey || false,
                e.altKey || false
            ].join(' ');
        },

    //keyCodes
        KEYCODES = {
            shift: 16,
            ctrl: 17,
            alt: 18,
            delete: 46,

            enter: 13,
            end: 35,

            a: 65,
            b: 66,

            A: 65,

            '1': 49,
            '"': 222
        },
        CHARCODES = {
            a: 97,
            b: 98,

            A: 65,

            '1': 49,
            '!': 33,
            '"': 34
        };

    //tests
    QUnit.testStart = function () {
        $el = addInputElement('button', 'button1', Math.floor(Math.random() * 100),
            Math.floor(Math.random() * 100));

        $el[0].focus();

        $el.keydown(function (e) {
            logEvent(e);
        });
        $el.keypress(function (e) {
            logEvent(e);
        });
        $el.keyup(function (e) {
            logEvent(e);
        });
    };

    QUnit.testDone = function () {
        eventsLog = '';
        $('.' + TEST_ELEMENT_CLASS).remove();
    };

    module('events raising');
    asyncTest('press literal symbol', function () {
            var literal = 'a';
            var expectedEvents = [
                { type: 'keydown', keyCode: KEYCODES[literal] },
                { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES[literal], charCode: CHARCODES[literal] },
                { type: 'keyup', keyCode: KEYCODES[literal] }
            ];

            var callback = function () {
                equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
                start();
            };
            press(literal, callback);
        });

    asyncTest('press literal symbol uppercase', function () {
        var literal = 'A';
        var expectedEvents = [
            { type: 'keydown', keyCode: KEYCODES[literal] },
            { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES[literal], charCode: CHARCODES[literal] },
            { type: 'keyup', keyCode: KEYCODES[literal] }
        ];

        var callback = function () {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        };
        press(literal, callback);
    });

    asyncTest('press two literal symbols', function () {
        var aLiteral = 'a',
            bLiteral = 'b';

        var expectedEvents = [
            { type: 'keydown', keyCode: KEYCODES[aLiteral] },
            { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES[aLiteral], charCode: CHARCODES[aLiteral] },
            { type: 'keydown', keyCode: KEYCODES[bLiteral] },
            { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES[bLiteral], charCode: CHARCODES[bLiteral] },
            { type: 'keyup', keyCode: KEYCODES[bLiteral] },
            { type: 'keyup', keyCode: KEYCODES[aLiteral] }
        ];

        var callback = function () {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        };
        press(aLiteral + '+' + bLiteral, callback);
    });

    asyncTest('press literal with ctrl', function () {
        var literal = 'b';

        var expectedEvents = [
            { type: 'keydown', keyCode: KEYCODES.ctrl, ctrlKey: true },
            { type: 'keydown', keyCode: KEYCODES[literal], ctrlKey: true },
            { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES[literal], charCode: CHARCODES[literal], ctrlKey: true },
            { type: 'keyup', keyCode: KEYCODES[literal], ctrlKey: true },
            { type: 'keyup', keyCode: KEYCODES.ctrl }
        ];

        var callback = function () {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        };
        press('ctrl+b', callback);
    });

    asyncTest('press number key', function () {
        var literal = '1';

        var expectedEvents = [
            { type: 'keydown', keyCode: KEYCODES[literal] },
            { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES[literal], charCode: CHARCODES[literal] },
            { type: 'keyup', keyCode: KEYCODES[literal] }
        ];

        var callback = function () {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        };
        press(literal, callback);
    });

    asyncTest('press special key', function () {
        var specialKey = 'enter',
            specialKeyCode = KEYCODES[specialKey];

        var expectedEvents = [
            { type: 'keydown', keyCode: specialKeyCode },
            { type: 'keypress', keyCode: specialKeyCode, charCode: Util.isMozilla ? 0 : specialKeyCode },
            { type: 'keyup', keyCode: specialKeyCode }
        ];

        var callback = function () {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        };
        press(specialKey, callback);
    });

    asyncTest('press mapped modifier', function () {
        var expectedEvents = [
            { type: 'keydown', keyCode: KEYCODES.alt, altKey: true },
            { type: 'keyup', keyCode: KEYCODES.alt }
        ];

        var callback = function () {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        };
        press('option', callback);
    });

    asyncTest('symbols with incorrect keycode', function () {
        var expectedEvents = [
            { type: 'keydown', keyCode: KEYCODES['shift'], shiftKey: true },
            { type: 'keydown', keyCode: KEYCODES['"'], shiftKey: true  },
            { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES['"'], charCode: CHARCODES['"'] , shiftKey: true  },
            { type: 'keyup', keyCode: KEYCODES['"'], shiftKey: true  },
            { type: 'keyup', keyCode: KEYCODES['shift'] }
        ];

        function callback() {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        }

        press('"', callback);
    });

    module('shift key');
    //press(shift+a)
    asyncTest('shift+a', function () {
        var literal = 'a',
            upperLiteral = literal.toUpperCase();

        var expectedEvents = [
            { type: 'keydown', keyCode: KEYCODES.shift, shiftKey: true },
            { type: 'keydown', keyCode: KEYCODES[literal], shiftKey: true },
            { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES[upperLiteral], charCode: CHARCODES[upperLiteral], shiftKey: true },
            { type: 'keyup', keyCode: KEYCODES[literal], shiftKey: true },
            { type: 'keyup', keyCode: KEYCODES.shift }
        ];

        var callback = function () {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        };
        press('shift+a', callback);
    });
    //press(shift+A)
    asyncTest('shift+A', function () {
        var literal = 'A',
            lowerLiteral = literal.toLowerCase();

        var expectedEvents = [
            { type: 'keydown', keyCode: KEYCODES.shift, shiftKey: true },
            { type: 'keydown', keyCode: KEYCODES[literal], shiftKey: true },
            { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES[lowerLiteral], charCode: CHARCODES[lowerLiteral], shiftKey: true },
            { type: 'keyup', keyCode: KEYCODES[literal], shiftKey: true },
            { type: 'keyup', keyCode: KEYCODES.shift }
        ];

        var callback = function () {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        };
        press('shift+A', callback);
    });

    //'shift+1' press helpers
    var literal = '1',
        modifiedLiteral = '!';
    var expectedEventsShift_1_Pressing = [
        { type: 'keydown', keyCode: KEYCODES.shift, shiftKey: true },
        { type: 'keydown', keyCode: KEYCODES[literal], shiftKey: true },
        { type: 'keypress', keyCode: Util.isMozilla ? 0 : CHARCODES[modifiedLiteral], charCode: CHARCODES[modifiedLiteral], shiftKey: true },
        { type: 'keyup', keyCode: KEYCODES[literal], shiftKey: true },
        { type: 'keyup', keyCode: KEYCODES.shift }
    ];
    var shift_1_pressCallback = function () {
        equal(eventsLog, createCheckingLog(expectedEventsShift_1_Pressing), 'events are correct');
        start();
    };
    //press(shift+1)
    asyncTest('shift+1', function () {
        press('shift+1', shift_1_pressCallback);
    });
    //press(shift+!)
    asyncTest('shift+!', function () {
        press('shift+!', shift_1_pressCallback);
    });
    //press(!)
    asyncTest('!', function () {
        press('!', shift_1_pressCallback);
    });

    module('Regression tests');
    asyncTest('B237817 - ASPxComboBox - pressing "ctrl+end" via act.press does not work', function () {
        var expectedEvents = [
            { type: 'keydown', keyCode: KEYCODES.ctrl, ctrlKey: true },
            { type: 'keydown', keyCode: KEYCODES.end, ctrlKey: true }
        ];

        if (Util.isMozilla)
            expectedEvents.push({ type: 'keypress', keyCode: KEYCODES.end, charCode: 0, ctrlKey: true });

        expectedEvents.push({ type: 'keyup', keyCode: KEYCODES.end, ctrlKey: true });
        expectedEvents.push({ type: 'keyup', keyCode: KEYCODES.ctrl });

        var callback = function () {
            equal(eventsLog, createCheckingLog(expectedEvents), 'events are correct');
            start();
        };

        press('ctrl+end', callback);
    });

    asyncTest('B237084 - Client instance works incorrect after "enter" key has been pressed on the focused control', function () {
        var clickRaisedCount = 0,
            $input = $('<input type="button">').addClass(TEST_ELEMENT_CLASS)
                .click(function () {
                    clickRaisedCount++;
                })
                .appendTo('body');

        $input[0].focus();
        window.setTimeout(function () {
            press('enter', function () {
                equal(clickRaisedCount, 1);

                press('space', function () {
                    equal(clickRaisedCount, 2);

                    start();
                });
            });
        }, 100);
    });

    asyncTest('B237122 - Client instance works incorrect after "space" key has been pressed on the focused control', function () {
        var input = $('<input>').addClass(TEST_ELEMENT_CLASS).appendTo('body')[0];

        input.addEventListener('keyup', function (ev) {
            equal(ev.keyCode, Util.KEYS_MAPS.SPECIAL_KEYS.space);
        });

        input.focus();
        window.setTimeout(function () {
            press('space', function () {
                expect(1);
                start();
            });
        }, 100);
    });

    asyncTest('B254435 - TestCafe allows act.press with service keys in input in firefox browser if there is call preventDefault() in keypress event handler', function () {
        var value = 'text',
            input = $('<input>').attr('value', value).addClass(TEST_ELEMENT_CLASS).appendTo('body')[0];

        input.addEventListener('keypress', function (ev) {
            Util.preventDefault(ev);
        });

        input.focus();

        press('ctrl+a', function () {
            press('delete', function () {

                equal(input.value, Util.isMozilla ? value : '');

                start();
            });
        });
    });

    asyncTest('T338564 - Wrong "which" property in "keypress" objects during pressing of special symbols in Firefox', function () {
        $el.on({
            keydown: function(e){
                if(e.keyCode === KEYCODES.a)
                    equal(e.which, KEYCODES.a);
                else
                    equal(e.which, KEYCODES.delete);
            },

            keypress: function(e){
                if(e.charCode === CHARCODES.a)
                    equal(e.which, CHARCODES.a);
                else
                    equal(e.which, 0);
            },

            keyup: function(e){
                if(e.keyCode === KEYCODES.a)
                    equal(e.which, KEYCODES.a);
                else
                    equal(e.which, KEYCODES.delete);
            }
        });

        var callback = function () {
            if(Util.isMozilla)
                expect(6);
            else
                expect(5);
            start();
        };

        press('a+delete', callback);
    });

    if (NativeMethods.inputValueSetter) {
        asyncTest('call native setter of the value property (GH-1558)(T547983)', function () {
            var input    = $('<input type="text" />').addClass(TEST_ELEMENT_CLASS).appendTo('body')[0];
            var textArea = $('<textarea/>').addClass(TEST_ELEMENT_CLASS).appendTo('body')[0];

            var testNativeValueSetter = function (element, callback) {
                var valueGetter = Object.getOwnPropertyDescriptor(element.constructor.prototype, 'value').get;

                element.value = '1';
                element.focus();
                element.setSelectionRange(1, 1);

                Object.defineProperty(element, 'value', {
                    get: function () {
                        return valueGetter.call(element);
                    },
                    set: function () {
                        ok(false);
                    }
                });

                press('backspace', function () {
                    equal(element.value, '');
                    callback();
                });
            };

            testNativeValueSetter(input, function () {
                testNativeValueSetter(textArea, function () {
                    start();
                });
            });
        });
    }

});
