var Hammerhead = HammerheadClient.get('Hammerhead'),
    $ = Hammerhead.$,
    async = Hammerhead.async,
    Util = Hammerhead.Util,
    NativeMethods = Hammerhead.NativeMethods,
    ShadowUI = Hammerhead.ShadowUI,

    TextSelection = Hammerhead.TextSelection,
    Automation = TestCafeClient.get('Automation'),
    ClickPlaybackAutomation = TestCafeClient.get('Automation.Click.Playback'),
    DblClickPlaybackAutomation = TestCafeClient.get('Automation.DblClick.Playback'),
    HoverPlaybackAutomation = TestCafeClient.get('Automation.Hover.Playback'),
    TypePlaybackAutomation = TestCafeClient.get('Automation.Type.Playback'),
    SelectPlaybackAutomation = TestCafeClient.get('Automation.Select.Playback'),
    PressPlaybackAutomation = TestCafeClient.get('Automation.Press.Playback'),
    CursorWidget = TestCafeClient.get('UI.Cursor');

Hammerhead.init();
Automation.init();
CursorWidget.init();

$(document).ready(function () {
    //consts
    var TEST_ELEMENT_CLASS = 'testElement',
        OPTION_LIST_CLASS = 'tcOptionList';

    //vars
    var body = $('body')[0];

    //utils
    var createInput = function (type) {
        return $('<input>')
            .attr('type', type || 'text')
            .attr('id', 'input')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo('body');
    };

    var createButton = function () {
        return $('<input type="button">').addClass(TEST_ELEMENT_CLASS).appendTo('body');
    };

    var createSelect = function () {
        var select = $('<select><select/>')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo('body')[0];
        $('<option>one</option>')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo(select);
        $('<option>two</option>')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo(select);
        $('<option>three</option>')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo(select);
        $('<option>four</option>')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo(select);
        $('<option>five</option>')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo(select);
        return $(select);
    };

    $(body).css('height', 1500);
    //NOTE: problem with window.top bodyMargin in IE9 if test 'runAll'
    //because we can't determine that element is in qunit test iframe
    if (Util.isIE && Util.browserVersion === 9)
        $(window.top.document).find('body').css('marginTop', '0px');

    var createDraggable = function (currentWindow, currentDocument, x, y) {
        var curDocument = currentDocument || document;
        currentWindow = currentWindow || window;

        var lastCursorPosition = null,
            $draggable = $('<div></div>')
                .attr('id', 'draggable')
                .addClass(TEST_ELEMENT_CLASS)
                .css({
                    width:           '60px',
                    height:          '60px',
                    position:        'absolute',
                    backgroundColor: 'grey',
                    left:            x ? x + 'px' : '100px',
                    top:             y ? y + 'px' : '850px',
                    zIndex:          5
                }).bind(Util.isTouchDevice ? 'touchstart' : 'mousedown', function (e) {
                    lastCursorPosition = Util.isTouchDevice ? {
                        x: e.originalEvent.targetTouches[0].pageX || e.originalEvent.touches[0].pageX,
                        y: e.originalEvent.targetTouches[0].pageY || e.originalEvent.touches[0].pageY
                    } : {
                        x: e.clientX,
                        y: e.clientY
                    };
                    $(this).data('dragStarted', true);
                })

                .bind(Util.isTouchDevice ? 'touchend' : 'mouseup', function () {
                    lastCursorPosition = null;
                    $(this).data('dragStarted', false);
                })
                .appendTo($(curDocument).find('body'));

        $(curDocument).bind(Util.isTouchDevice ? 'touchmove' : 'mousemove', function (e) {
            var curMousePos = Util.isTouchDevice ? {
                x: e.originalEvent.targetTouches[0].pageX || e.originalEvent.touches[0].pageX,
                y: e.originalEvent.targetTouches[0].pageY || e.originalEvent.touches[0].pageY
            } : {
                x: e.clientX,
                y: e.clientY
            };
            $.each($draggable, function () {
                var $this = $(this);
                if ($(this).data('dragStarted')) {

                    $this.css({
                        left: Math.round($this.position().left) + curMousePos.x - lastCursorPosition.x,
                        top:  Math.round($this.position().top) + curMousePos.y - lastCursorPosition.y
                    });
                    return false;
                }
            });
            lastCursorPosition = curMousePos;
        });

        var $window = $(currentWindow),
            windowScrollX = 0,
            windowScrollY = 0;


        $window.scroll(function () {
            var x = $window.scrollLeft() - windowScrollX,
                y = $window.scrollTop() - windowScrollY;
            windowScrollX = $window.scrollLeft();
            windowScrollY = $window.scrollTop();

            if ($draggable.data('dragStarted')) {
                $draggable.css({
                    left: $draggable.position().left + x,
                    top:  $draggable.position().top + y
                });
            }
        });

        return $draggable;
    };

    var startNext = function () {
        if (Util.isIE) {
            removeTestElements();
            window.setTimeout(start, 30);
        }
        else
            start();
    };

    var removeTestElements = function () {
        $('.' + TEST_ELEMENT_CLASS).remove();
    };



    QUnit.testDone = function () {
        if (!Util.isIE)
            removeTestElements();
    };

    module('regression tests');

    if (Util.isIE)
        asyncTest('click on submit button child (B236676)', function () {
            var $form = $('<form></form>').addClass(TEST_ELEMENT_CLASS).appendTo('body'),
                $button = $('<button></button>').attr('type', 'submit').addClass(TEST_ELEMENT_CLASS).appendTo($form),
                $img = $('<img />').attr('alt', 'img').addClass(TEST_ELEMENT_CLASS).appendTo($button),
                imgClicked = false,
                buttonClicked = false,
                formSubmitted = false;

            $form.submit(function (ev) {
                formSubmitted = true;
                Util.preventDefault(ev);
                return false;
            });

            $button.click(function () {
                buttonClicked = true;
            });

            $img.click(function () {
                imgClicked = true;
            });

            ClickPlaybackAutomation.run($img[0], {}, function () {

                //in IE submit button's children do not receive click event if user clicks on it
                ok(formSubmitted, 'form submit received');
                ok(buttonClicked, 'button click received');
                ok(!imgClicked, 'img click not received');

                formSubmitted = buttonClicked = imgClicked = false;

                ClickPlaybackAutomation.run($button[0], {
                    offsetX: Math.round($button[0].offsetWidth / 2),
                    offsetY: Math.round($button[0].offsetHeight / 2)
                }, function () {
                    ok(formSubmitted, 'form submit received');
                    ok(buttonClicked, 'button click received');
                    ok(!imgClicked, 'img click not received');
                    startNext();
                });
            });
        });

    if (!Util.isTouchDevice) {
        asyncTest('B236966 - TESTCafe - onmouseout event is not called during the execution of the method hover.', function () {
            var $element = createDraggable(window, document, 200, 200),
                firstEvent = null;

            $element.bind('mouseover', function () {
                if (!firstEvent)
                    firstEvent = 'mouseover';
            });

            $element.bind('mousemove', function () {
                if (!firstEvent)
                    firstEvent = 'mousemove';
            });

            HoverPlaybackAutomation.run($element[0], {}, function () {
                equal(firstEvent, Util.isIE ? 'mousemove' : 'mouseover');
                startNext();
            });
        });
    }

    asyncTest('B237084 - Client instance works incorrect after "enter" key has been pressed on the focused control', function () {
        var getSrcElement = function (ev) {
            return ev.srcElement || ev.target;
        };

        var button1 = createButton()[0],
            button2 = createButton()[0],

            documentClickFirstHandlerRaised = false,
            documentClickSecondHandlerRaised = false,
            button2ClickHandlerRaised = false;

        document.addEventListener('click', function (ev) {
            if (getSrcElement(ev) === button1) {
                documentClickFirstHandlerRaised = true;
                button2.click();
            }
        });

        document.addEventListener('click', function (ev) {
            if (getSrcElement(ev) === button1)
                documentClickSecondHandlerRaised = true;
        });

        button2.addEventListener('click', function () {
            button2ClickHandlerRaised = true;
        });

        ClickPlaybackAutomation.run(button1, {}, function () {
            ok(documentClickFirstHandlerRaised);
            ok(documentClickSecondHandlerRaised);
            ok(button2ClickHandlerRaised);

            startNext();
        });
    });

    asyncTest('B237672 - TesCafe throw exception "Access is denied" after trying to get content of iframe in IE browsers', function () {
        var clicked = false,
            $iframe = $('<iframe></iframe>')
                .width(500)
                .height(500)
                .attr('src', 'http://www.cross.domain.com')
                .addClass(TEST_ELEMENT_CLASS)
                .click(function () {
                    clicked = true;
                })
                .appendTo('body');

        window.setTimeout(function () {
            try {
                //NOTE: for not ie
                var iframeBody = $iframe[0].contentWindow.document;
                NativeMethods.addEventListener.call(iframeBody, 'click', function () {
                    clicked = true;
                });
            }
            catch (e) {
            }

            ClickPlaybackAutomation.run($iframe[0], {}, function () {
                ok(clicked, 'click was raised');
                startNext();
            });
        }, 500);
    });

    asyncTest('B237862 - Test runner - the type action does not consider maxLength of the input element.', function () {
        var initText = 'init',
            newText = 'newnewnew',
            $input = createInput().attr('value', initText),
            resultString = initText + newText,
            maxLength = 7;

        $input.attr('maxLength', maxLength);
        equal(parseInt($input.attr('maxLength')), 7);

        TypePlaybackAutomation.run($input[0], newText, {}, function () {
            equal($input[0].value, resultString.substring(0, maxLength));
            startNext();
        });
    });

    if (!Util.isIE) {
        //TODO: IE wrong detection dimension top for element if this element have height more than scrollable container
        //and element's top less than container top
        asyncTest('B237890 - Wrong scroll before second click on big element in scrollable container', function () {
            var clickCount = 0,
                errorScroll = false,

                $scrollableContainer = $('<div />')
                    .css({
                        position: 'absolute',
                        left:     '200px',
                        top:      '250px',
                        border:   '1px solid black',
                        overflow: 'scroll'
                    })
                    .width(250)
                    .height(200)
                    .addClass(TEST_ELEMENT_CLASS)
                    .appendTo($('body')[0]);

            $('<div></div>').addClass(TEST_ELEMENT_CLASS)
                .css({
                    height:          '20px',
                    width:           '20px',
                    marginTop:       2350 + 'px',
                    backgroundColor: '#ffff00'
                })
                .appendTo($scrollableContainer);

            var scrollHandler = function () {
                    if (clickCount === 1)
                        errorScroll = true;
                },
                bindScrollHandlers = function () {
                    $scrollableContainer.bind('scroll', scrollHandler);
                    $(window).bind('scroll', scrollHandler);
                },
                unbindScrollHandlers = function () {
                    $scrollableContainer.unbind('scroll', scrollHandler);
                    $(window).unbind('scroll', scrollHandler);
                };


            var $element = $('<div></div>')
                .addClass(TEST_ELEMENT_CLASS)
                .css({
                    width:           '150px',
                    height:          '400px',
                    position:        'absolute',
                    backgroundColor: 'red',
                    left:            '50px',
                    top:             '350px'
                })
                .appendTo($scrollableContainer)
                .bind('mousedown', function () {
                    unbindScrollHandlers();
                })
                .bind('click', function () {
                    clickCount++;

                });

            bindScrollHandlers();

            ClickPlaybackAutomation.run($element[0], {}, function () {
                equal(clickCount, 1);
                bindScrollHandlers();

                ClickPlaybackAutomation.run($element[0], {}, function () {
                    equal(clickCount, 2);
                    ok(!errorScroll);
                    startNext();
                });
            });
        });
    }

    asyncTest('B237763 - ASPxPageControl - Lite render - Tabs are not clicked in Firefox', function () {
        var clickRaised = false,
            $list = $('<div></div>').addClass(TEST_ELEMENT_CLASS).appendTo('body'),
            $b = $('<b></b>').html('text').appendTo($list);

        $list[0].onclick = function () {
            clickRaised = true;
        };

        ClickPlaybackAutomation.run($b[0], {}, function () {
            ok(clickRaised);
            startNext();
        });
    });

    asyncTest('Click on label with for attribute', function () {
        var $input = $('<input type="checkbox"/>').addClass(TEST_ELEMENT_CLASS).attr('id', 'test123').appendTo('body'),
            $label = $('<label>label</label>').addClass(TEST_ELEMENT_CLASS).attr('for', 'test123').appendTo('body');

        $input[0].checked = false;

        ClickPlaybackAutomation.run($label[0], {}, function () {
            ok($input[0].checked);
            startNext();
        });
    });

    asyncTest('Q518957 - Test is inactive with mouse clicks and date-en-gb.js is included', function () {
        var savedDateNow = window.Date;

        window.Date.now = function () {
            return {};
        };

        var $input = $('<input type="button" />').addClass(TEST_ELEMENT_CLASS).appendTo('body'),
            completed = false;

        HoverPlaybackAutomation.run($input[0], {}, function () {
            if (!completed) {
                completed = true;
                ok('test success');
                window.Date = savedDateNow;
                startNext();
            }
        });

        window.setTimeout(function () {
            if (!completed) {
                completed = true;
                ok(false, 'timeout expired');
                window.Date.now = savedDateNow;
                startNext();
            }
        }, 2000);
    });

    asyncTest('B238560 - Change event is not raised during TestCafe test running', function () {
        var $input = $('<input type="checkbox" />').addClass(TEST_ELEMENT_CLASS).appendTo('body'),
            changeRaised = false;

        $input[0].addEventListener('change', function () {
            changeRaised = true;
        });

        ClickPlaybackAutomation.run($input[0], {}, function () {
            ok(changeRaised);
            startNext();
        });
    });

    asyncTest('B237794 - Select options list doesn\'t close after dblclick in Chrome and Opera (dblclick)', function () {
        var $select = createSelect();

        equal(ShadowUI.select('.' + OPTION_LIST_CLASS).length, 0);

        DblClickPlaybackAutomation.run($select[0], {}, function () {
            equal($select[0], document.activeElement);
            notEqual(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), true);
            equal(ShadowUI.select('.' + OPTION_LIST_CLASS).length, 0);
            equal($select[0].selectedIndex, 0);
            startNext();
        });
    });

    asyncTest('B237794 - Select options list doesn\'t close after dblclick in Chrome and Opera (two successive clicks)', function () {
        var $select = createSelect();

        equal(ShadowUI.select('.' + OPTION_LIST_CLASS).length, 0);

        async.series({
            'First click': function (callback) {
                ClickPlaybackAutomation.run($select[0], {}, callback);
            },

            'Check assertions and set timeout': function (callback) {
                equal($select[0], document.activeElement);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), true);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).length, 1);
                equal($select[0].selectedIndex, 0);

                window.setTimeout(callback, 500);
            },

            'Second click': function (callback) {
                ClickPlaybackAutomation.run($select[0], {}, callback);
            },

            'Check assertions': function () {
                equal($select[0], document.activeElement);
                notEqual(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), true);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).length, 0);
                equal($select[0].selectedIndex, 0);
                startNext();
            }
        });
    });

    asyncTest('B238984 - After click on select element and then click on another first opened options list doesn\'t closed', function () {
        var $firstSelect = createSelect(),
            $secondSelect = createSelect(),
            $secondSelectOption = $secondSelect.find('option').last();

        equal(ShadowUI.select('.' + OPTION_LIST_CLASS).length, 0);

        async.series({
            'Click on the first select': function (callback) {
                ClickPlaybackAutomation.run($firstSelect[0], {}, callback);
            },

            'Check assertions after the first click': function (callback) {
                equal($firstSelect[0], document.activeElement);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), true);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).length, 1);
                equal($firstSelect[0].selectedIndex, 0);
                window.setTimeout(callback, 500);
            },

            'Click on the second select': function (callback) {
                ClickPlaybackAutomation.run($secondSelect[0], {}, callback);
            },

            'Check assertions after the second click': function (callback) {
                equal($secondSelect[0], document.activeElement);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), true);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).length, 1);
                equal($firstSelect[0].selectedIndex, 0);
                window.setTimeout(callback, 500);
            },

            'Click on the second select option': function (callback) {
                ClickPlaybackAutomation.run($secondSelectOption[0], {}, callback);
            },

            'Check assertions': function () {
                equal($secondSelect[0], document.activeElement);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), false);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).length, 0);
                equal($firstSelect[0].selectedIndex, 0);
                equal($secondSelect[0].selectedIndex, $secondSelect.find('option').length - 1);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            }
        });
    });

    asyncTest('B238956 - Event sandbox: svg specific native methods not memorized', function () {
        var $svg = $('<svg xmlns="http://www.w3.org/2000/svg" version="1.1"></svg>')
                .addClass(TEST_ELEMENT_CLASS)
                .width(500)
                .height(300)
                .appendTo('body'),
            clickCount = 0;

        $svg.click(function () {
            clickCount++;
        });

        ClickPlaybackAutomation.run($svg[0], {}, function () {
            equal(clickCount, 1);
            startNext();
        });
    });

    asyncTest('B252929 - Wrong behavior during recording dblclick on input', function () {
        var $input = createInput(),
            dblclickCount = 0,
            clickCount = 0;

        $input[0].value = 'Test cafe';

        $input.dblclick(function () {
            dblclickCount++;
        });

        $input.click(function () {
            clickCount++;
        });

        DblClickPlaybackAutomation.run($input[0], {
            caretPos: 3
        }, function () {
            equal($input[0].selectionStart, 3, 'start selection correct');
            equal($input[0].selectionEnd, 3, 'end selection correct');
            equal(clickCount, 2);
            equal(dblclickCount, 1);
            startNext();
        });
    });

    asyncTest('B253465 - Incorrect behavior when a math function is typed in ASPxSpreadsheet\'s cell', function () {
        var ROUND_BRACKET_KEY_CODE = 57,
            ROUND_BRACKET_CHAR_CODE = 40;

        function checkKeyCode (e) {
            equal(e.keyCode, ROUND_BRACKET_KEY_CODE);
        }

        function checkCharCode (e) {
            equal(Util.isMozilla ? e.charCode : e.keyCode, ROUND_BRACKET_CHAR_CODE);
        }

        var $input = createInput().keydown(checkKeyCode).keypress(checkCharCode).keyup(checkKeyCode);

        TypePlaybackAutomation.run($input[0], '(', {}, function () {
            startNext();
        });

        expect(3);
    });

    asyncTest('B254340 - type in input with type="email"', function () {
        var initText = 'support@devexpress.com',
            newText = 'new',
            $input = createInput('email').attr('value', initText),
            caretPos = 5,
            resultString = initText.substring(0, caretPos) + newText + initText.substring(caretPos);

        TypePlaybackAutomation.run($input[0], newText, {
            caretPos: caretPos
        }, function () {
            equal($input[0].value, resultString);
            equal(TextSelection.getSelectionStart($input[0]), caretPos + newText.length, 'start selection correct');
            equal(TextSelection.getSelectionEnd($input[0]), caretPos + newText.length, 'end selection correct');
            startNext();
        });
    });

    if (Util.isIE) {
        //TODO: fix it for other browsers
        asyncTest('Unexpected focus events are raised during click', function () {
            var input1FocusCount = 0,
                input2FocusCount = 0,
                $input1 = createInput().attr('id', '1').focus(function () {
                    input1FocusCount++;
                }),
                $input2 = createInput().attr('id', '2').focus(function () {
                    input2FocusCount++;
                    $input1[0].focus();
                });

            ClickPlaybackAutomation.run($input2[0], {}, function () {
                equal(input1FocusCount, 1);
                equal(input2FocusCount, 1);

                startNext();
            });
        });

        asyncTest('Unexpected focus events are raised during dblclick', function () {
            var input1FocusCount = 0,
                input2FocusCount = 0,
                $input1 = createInput().attr('id', '1').focus(function () {
                    input1FocusCount++;
                }),
                $input2 = createInput().attr('id', '2').focus(function () {
                    input2FocusCount++;
                    $input1[0].focus();
                });

            DblClickPlaybackAutomation.run($input2[0], {}, function () {
                equal(input1FocusCount, Util.isIE ? 1 : 2);
                equal(input2FocusCount, Util.isIE ? 1 : 2);

                startNext();
            });
        });
    }

    if (Util.isIE11 && Util.browserVersion > 9) {
        asyncTest('T109295 - User action act.click isn\'t raised by click on map', function () {
            var initText = 'click',
                $input = createInput('button').attr('value', initText).css({
                    position: 'absolute',
                    left:     '200px',
                    top:      '200px'
                }),
                log = '',

                events = {
                    mouse:    ['mouseover', 'mouseout', 'mousedown', 'mouseup', 'click'],
                    touch:    ['touchstart', 'touchend'],
                    pointer:  ['pointerover', 'pointerout', 'pointerdown', 'pointerup'],
                    MSevents: ['MSPointerOver', 'MSPointerOut', 'MSPointerDown', 'MSPointerUp']
                };

            var addListeners = function (el, events) {
                $.each(events, function (index, event) {
                    el.addEventListener(event, function (e) {
                        if (log !== '')
                            log = log + ', ';
                        log = log + e.type;
                    });
                });
            };

            addListeners($input[0], events.mouse);

            if (Util.browserVersion > 10)
                addListeners($input[0], events.pointer);
            else
                addListeners($input[0], events.MSevents);


            ClickPlaybackAutomation.run($input[0], {}, function () {
                if (Util.browserVersion > 10)
                    equal(log, 'pointerover, mouseover, pointerdown, mousedown, pointerup, mouseup, click');
                else
                    equal(log, 'MSPointerOver, mouseover, MSPointerDown, mousedown, MSPointerUp, mouseup, click');
                startNext();
            });
        });
    }

    asyncTest('T235186 - Focus event handlers don\'t call for iframe\'s contenteditable body', function () {
        var focusRaised = false,
            $iframe = $('<iframe></iframe>')
                .width(500)
                .height(500)
                .attr('src', '/data/focus_blur_change/test_iframe.html')
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo('body');

        $iframe.load(function () {
            var $iFrameBody = $($iframe[0].contentWindow.document.body);
            $iFrameBody.attr('contenteditable', true);

            NativeMethods.addEventListener.call($iFrameBody[0], 'focus', function () {
                focusRaised = true;
            });

            ClickPlaybackAutomation.run($iFrameBody[0], {}, function () {
                ok(focusRaised, 'focus handler was called');
                focusRaised = false;

                ClickPlaybackAutomation.run($iFrameBody[0], {}, function () {
                    ok(!focusRaised, 'focus handler was not called');
                    startNext();
                });
            });
        });
    });

    asyncTest('T286582 - Menu item has hover state in jssite tests, but is does not hovered', function () {
        var style = [
            '<style>',
            'input {border-bottom-width: 0;}',
            'input:hover {border-bottom-width: 10px;}',
            '</style>'
        ].join('\n');

        $(style)
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo(body);

        var $input1 = createInput()
            .css('position', 'fixed')
            .css('margin-top', '50px')
            .appendTo(body);

        var $input2 = $input1
            .clone()
            .css('margin-left', '200px')
            .appendTo(body);

        ClickPlaybackAutomation.run($input1[0], {}, function () {
            strictEqual($input1.css('border-bottom-width'), '10px');
            $input1.css('margin-top', '0px');
            ClickPlaybackAutomation.run($input2[0], {}, function () {
                strictEqual($input1.css('border-bottom-width'), '0px');
                startNext();
            })
        })
    });

    /*asyncTest('T323441 - textarea has incorrect scroll position after Backspace key is pressed', function () {
        var value = 'text123';
        var textarea = $('<textarea>').attr('value', value).addClass(TEST_ELEMENT_CLASS).appendTo('body')[0];

        var actualSelections = [];
        var expectedSelections = [
            {start: 1, end: 1},
            {start: 1, end: 1},
            {start: 2, end: 2}
        ];

        textarea.addEventListener('input', function () {
            actualSelections.push({start: textarea.selectionStart, end: textarea.selectionEnd});

            if (actualSelections.length === expectedSelections.length) {
                deepEqual(actualSelections, expectedSelections);
                start();
            }
        });

        textarea.focus();

        PressPlaybackAutomation.run('right right backspace delete enter', function () {
        });
    });*/


    asyncTest('B254020 - act.type in input does not type sometimes to input on motorolla Xoom pad.', function () {
        var newText = '123',
            $input = createInput()
                .attr('placeholder', 'Type here...')
                .css('-webkit-user-modify', 'read-write-plaintext-only');

        TypePlaybackAutomation.run($input[0], newText, {}, function () {
            equal($input[0].value, newText);
            equal(TextSelection.getSelectionStart($input[0]), newText.length, 'start selection correct');
            equal(TextSelection.getSelectionEnd($input[0]), newText.length, 'end selection correct');
            startNext();
        });
    });

    // NOTE: Test must be runned on zoomed page or in Hi-DPI mode
    asyncTest('T432527 - A test crashes when page is scrolled or the cursor is moved when page scaling is used', function () {
        var testElement = document.createElement('div');
        var hoverRaised = false;

        testElement.className   = TEST_ELEMENT_CLASS;
        testElement.style       = 'position:absolute; top: 9000px; left: 0';
        testElement.textContent = 'Test';

        document.body.appendChild(testElement);

        testElement.addEventListener('mouseover', function () {
            hoverRaised = true;
        });

        HoverPlaybackAutomation.run(testElement, {}, function () {
            ok(hoverRaised);
            startNext();
        });
    });

    module('regression tests with input type="number"');

    if(!Util.isIE || Util.browserVersion !== 9) {
        asyncTest('B254340 - click on input with type="number"', function () {
            var input = createInput('number').val('123')[0],
                caretPos = 2,
                clickCount = 0;

            input.addEventListener('click', function () {
                clickCount++;
            });

            ClickPlaybackAutomation.run(input, {
                caretPos: caretPos
            }, function () {
                equal(TextSelection.getSelectionStart(input), caretPos, 'start selection correct');
                equal(TextSelection.getSelectionEnd(input), caretPos, 'end selection correct');
                equal(clickCount, 1);
                startNext();
            });
        });

        if (!(Util.isMozilla && Util.browserVersion >= 29)) {
            asyncTest('B254340 - select in input with type="number"', function () {
                var initText = '12345678987654321',
                    input = createInput('number').val(initText)[0],
                    startPos = 5,
                    endPos = 11,
                    backward = true;

                SelectPlaybackAutomation.run(input, {
                    startPos: endPos,
                    endPos:   startPos
                }, function () {
                    equal(TextSelection.getSelectionStart(input), startPos, 'start selection correct');
                    equal(TextSelection.getSelectionEnd(input), endPos, 'end selection correct');
                    equal(TextSelection.hasInverseSelection(input), backward, 'selection direction correct');
                    startNext();
                });
            });
        }

        asyncTest('T133144 - Incorrect typing into an input with type "number" in FF during test executing (without caretPos)', function () {
            var initText = '12345',
                text = '123',
                newText = initText + text,
                input = createInput('number').val(initText)[0];

            TypePlaybackAutomation.run(input, text, {}, function () {
                equal(input.value, newText);
                    equal(TextSelection.getSelectionStart(input), newText.length, 'start selection correct');
                    equal(TextSelection.getSelectionEnd(input), newText.length, 'end selection correct');

                startNext();
            });
        });

        asyncTest('T133144 - Incorrect typing into an input with type "number" in FF during test executing (with caretPos)', function () {
            var initText = '12345',
                text = '123',
                input = createInput('number').val(initText)[0],
                caretPos = 2;

            TypePlaybackAutomation.run(input, text, {
                caretPos: caretPos
            }, function () {
                equal(input.value, initText.substring(0, caretPos) + text + initText.substring(caretPos));
                equal(TextSelection.getSelectionStart(input), caretPos + text.length, 'start selection correct');
                equal(TextSelection.getSelectionEnd(input), caretPos + text.length, 'end selection correct');

                startNext();
            });
        });

        asyncTest('T133144 - Incorrect typing into an input with type "number" in FF during test executing (with replace)', function () {
            var initText = '12345',
                text = '678',
                input = createInput('number').val(initText)[0];

            TypePlaybackAutomation.run(input, text, {
                replace: true
            }, function () {
                equal(input.value, text);
                equal(TextSelection.getSelectionStart(input), text.length, 'start selection correct');
                equal(TextSelection.getSelectionEnd(input), text.length, 'end selection correct');

                startNext();
            });
        });

        asyncTest('T138385 - input type="number" leave out "maxlength" attribute (act.type)', function () {
            var input = createInput('number').attr('maxlength', 2)[0],
                inputEventCount = 0;

            input.addEventListener('input', function () {
                inputEventCount++;
            });

            TypePlaybackAutomation.run(input, '123', {}, function () {
                equal(inputEventCount, 3);
                equal(input.value, Util.isIE ? '12' : '123');
                startNext();
            });
        });

        asyncTest('T138385 - input type="number" leave out "maxlength" attribute (act.press)', function () {
            var input = createInput('number').attr('maxlength', 2)[0],
                inputEventCount = 0;

            input.addEventListener('input', function () {
                inputEventCount++;
            });

            input.focus();

            PressPlaybackAutomation.run('1 2 3', function () {
                equal(inputEventCount, 3);
                equal(input.value, Util.isIE ? '12' : '123');
                startNext();
            });
        });

        asyncTest('B254340 - type letters in input with type="number" (symbol in start)', function () {
            var input = createInput('number')[0];

            TypePlaybackAutomation.run(input, '+12', {}, function () {
                equal(input.value, '12');
                input.value = '';

                TypePlaybackAutomation.run(input, '-12', {}, function () {
                    equal(input.value, '-12');
                    input.value = '';

                    TypePlaybackAutomation.run(input, '.12', {}, function () {
                        equal(input.value, '.12');
                        input.value = '';

                        TypePlaybackAutomation.run(input, '+-12', {}, function () {
                            equal(input.value, '-12');
                            input.value = '';

                            TypePlaybackAutomation.run(input, 'a12', {}, function () {
                                equal(input.value, '12');
                                input.value = '';

                                TypePlaybackAutomation.run(input, '$12', {}, function () {
                                    equal(input.value, '12');

                                    startNext();
                                });
                            });
                        });
                    });
                });
            });
        });

        asyncTest('B254340 - type letters in input with type="number" (symbol in the middle)', function () {
            var input = createInput('number')[0];

            TypePlaybackAutomation.run(input, '1+2', {}, function () {
                equal(input.value, '12');
                input.value = '';

                TypePlaybackAutomation.run(input, '1-2', {}, function () {
                    equal(input.value, '12');
                    input.value = '';

                    TypePlaybackAutomation.run(input, '1.2', {}, function () {
                        equal(input.value, '1.2');
                        input.value = '';

                        TypePlaybackAutomation.run(input, '1+-2', {}, function () {
                            equal(input.value, '12');
                            input.value = '';

                            TypePlaybackAutomation.run(input, '1a2', {}, function () {
                                equal(input.value, '12');
                                input.value = '';

                                TypePlaybackAutomation.run(input, '1$2', {}, function () {
                                    equal(input.value, '12');

                                    document.body.removeChild(input);
                                    startNext();
                                });
                            });
                        });
                    });
                });
            });
        });

        asyncTest('B254340 - type letters in input with type="number" (symbol in the end)', function () {
            var input = createInput('number')[0];

            TypePlaybackAutomation.run(input, '12+', {}, function () {
                equal(input.value, '12');
                input.value = '';

                TypePlaybackAutomation.run(input, '12-', {}, function () {
                    equal(input.value, '12');
                    input.value = '';

                    TypePlaybackAutomation.run(input, '12.', {}, function () {
                        equal(input.value, '12');
                        input.value = '';

                        TypePlaybackAutomation.run(input, '12+-', {}, function () {
                            equal(input.value, '12');
                            input.value = '';

                            TypePlaybackAutomation.run(input, '12a', {}, function () {
                                equal(input.value, '12');
                                input.value = '';

                                TypePlaybackAutomation.run(input, '12$', {}, function () {
                                    equal(input.value, '12');

                                    document.body.removeChild(input);
                                    startNext();
                                });
                            });
                        });
                    });
                });
            });
        });

        asyncTest('B254340 - type letters in input with type="number" (one symbol)', function () {
            var input = createInput('number').val('12')[0];

            TypePlaybackAutomation.run(input, '+', {caretPos: 0}, function () {
                equal(input.value, '12');
                input.value = '12';

                TypePlaybackAutomation.run(input, '-', {caretPos: 0}, function () {
                    equal(input.value, '-12');
                    input.value = '12';

                    TypePlaybackAutomation.run(input, '.', {caretPos: 0}, function () {
                        equal(input.value, '.12');
                        input.value = '12';

                        TypePlaybackAutomation.run(input, '+-', {caretPos: 0}, function () {
                            equal(input.value, '-12');
                            input.value = '12';

                            TypePlaybackAutomation.run(input, '$', {caretPos: 0}, function () {
                                equal(input.value, '12');
                                input.value = '12';

                                TypePlaybackAutomation.run(input, '-12', {caretPos: 0}, function () {
                                    equal(input.value, '-1212');

                                    document.body.removeChild(input);
                                    startNext();
                                });
                            });
                        });
                    });
                });
            });
        });
    }
});
