var Hammerhead = HammerheadClient.get('Hammerhead'),
    $ = Hammerhead.$,
    async = Hammerhead.async,
    Util = Hammerhead.Util,
    ShadowUI = Hammerhead.ShadowUI,

    Automation = TestCafeClient.get('Automation'),
    ClickPlaybackAutomation = TestCafeClient.get('Automation.Click.Playback'),
    PressPlaybackAutomation = TestCafeClient.get('Automation.Press.Playback'),
    CursorWidget = TestCafeClient.get('UI.Cursor');

Hammerhead.init();
Automation.init();
CursorWidget.init();

$(document).ready(function () {
    //consts
    var TEST_ELEMENT_CLASS = 'testElement',
        OPTION_CLASS = 'tcOption',
        OPTION_GROUP_CLASS = 'tcOptionGroup',
        OPTION_LIST_CLASS = 'tcOptionList';

    //utils
    var createOption = function (parent, text) {
        return $('<option></option>').text(text)
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo(parent);
    };

    var createOptionGroup = function (select, label) {
        return $('<optgroup></optgroup>').attr('label', label)
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo(select)[0];
    };

    var createSelect = function () {
        var select = $('<select><select/>')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo('body')[0];

        createOption(select, 'one');
        createOption(select, 'two');
        createOption(select, 'three');
        createOption(select, 'four');
        createOption(select, 'five');
        return $(select);
    };

    var createSelectWithGroups = function () {
        var select = $('<select><select/>')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo('body')[0];

        var firstGroup = createOptionGroup(select, 'First');
        createOption(firstGroup, 'one');
        createOption(firstGroup, 'two');
        createOption(firstGroup, 'three');

        var secondGroup = createOptionGroup(select, 'Second');
        createOption(secondGroup, 'four');
        createOption(secondGroup, 'five');
        createOption(secondGroup, 'six');

        var thirdGroup = createOptionGroup(select, 'Third');
        createOption(thirdGroup, 'sevent');
        createOption(thirdGroup, 'eight');
        createOption(thirdGroup, 'nine');
        return $(select);
    };

    var createSelectWithGroupsForCheckPress = function () {
        var select = $('<select><select/>')
            .addClass(TEST_ELEMENT_CLASS)
            .appendTo('body')[0];

        var firstGroup = createOptionGroup(select, 'Group 1');
        createOption(firstGroup, 'one');

        var secondGroup = createOptionGroup(select, 'Group 2');
        createOption(secondGroup, 'two');

        var thirdGroup = createOptionGroup(select, 'Group 3');
        createOption(thirdGroup, 'thee');
        $(thirdGroup).attr('disabled', 'disabled');

        createOption(select, 'four');

        return $(select);
    };

    $('body').css('height', 1500);

    var startNext = function () {
        if (Util.isIE) {
            removeTestElements();
            window.setTimeout(start, 30);
        }
        else
            start();
    };

    var removeTestElements = function () {
        $('.' + TEST_ELEMENT_CLASS).remove();
    };

    var pressDownUpKeysActions = function ($select) {
        async.series({
            firstPressDownAction: function (callback) {
                equal($select[0].selectedIndex, 0);

                PressPlaybackAutomation.run('down', function () {
                    equal($select[0].selectedIndex, 1);
                    callback();
                });
            },
            secondPressDownAction: function (callback) {
                equal($select[0].selectedIndex, 1);

                PressPlaybackAutomation.run('down', function () {
                    equal($select[0].selectedIndex, 2);
                    callback();
                });
            },
            thirdPressDownAction: function (callback) {
                equal($select[0].selectedIndex, 2);

                PressPlaybackAutomation.run('down', function () {
                    equal($select[0].selectedIndex, 3);
                    callback();
                });
            },
            fourthPressDownAction: function (callback) {
                equal($select[0].selectedIndex, 3);

                PressPlaybackAutomation.run('down', function () {
                    equal($select[0].selectedIndex, 4);
                    callback();
                });
            },
            firstPressUpAction: function (callback) {
                equal($select[0].selectedIndex, 4);

                PressPlaybackAutomation.run('up', function () {
                    equal($select[0].selectedIndex, 3);
                    callback();
                });
            },
            secondPressUpAction: function (callback) {
                equal($select[0].selectedIndex, 3);

                PressPlaybackAutomation.run('up', function () {
                    equal($select[0].selectedIndex, 2);
                    callback();
                });
            },
            thirdPressUpAction: function (callback) {
                equal($select[0].selectedIndex, 2);

                PressPlaybackAutomation.run('up', function () {
                    equal($select[0].selectedIndex, 1);
                    callback();
                });
            },
            fourthPressUpAction: function () {
                equal($select[0].selectedIndex, 1);

                PressPlaybackAutomation.run('up', function () {
                    equal($select[0].selectedIndex, 0);
                    startNext();
                });
            }
        });
    };

    var pressRightLeftKeysActions = function ($select) {
        async.series({
            firstPressRightAction: function (callback) {
                equal($select[0].selectedIndex, 0);

                PressPlaybackAutomation.run('right', function () {
                    equal($select[0].selectedIndex, Util.isMozilla ? 1 : 0);
                    callback();
                });
            },
            secondPressRightAction: function (callback) {
                equal($select[0].selectedIndex, Util.isMozilla ? 1 : 0);

                PressPlaybackAutomation.run('right', function () {
                    equal($select[0].selectedIndex, Util.isMozilla ? 2 : 0);
                    callback();
                });
            },
            thirdPressRightAction: function (callback) {
                equal($select[0].selectedIndex, Util.isMozilla ? 2 : 0);

                PressPlaybackAutomation.run('right', function () {
                    equal($select[0].selectedIndex, Util.isMozilla ? 3 : 0);
                    callback();
                });
            },
            fourthPressRightAction: function (callback) {
                equal($select[0].selectedIndex, Util.isMozilla ? 3 : 0);

                PressPlaybackAutomation.run('right', function () {
                    equal($select[0].selectedIndex, Util.isMozilla ? 4 : 0);
                    callback();
                });
            },
            firstPressLeftAction: function (callback) {
                equal($select[0].selectedIndex, Util.isMozilla ? 4 : 0);

                PressPlaybackAutomation.run('left', function () {
                    equal($select[0].selectedIndex, Util.isMozilla ? 3 : 0);
                    callback();
                });
            },
            secondPressLeftAction: function (callback) {
                equal($select[0].selectedIndex, Util.isMozilla ? 3 : 0);

                PressPlaybackAutomation.run('left', function () {
                    equal($select[0].selectedIndex, Util.isMozilla ? 2 : 0);
                    callback();
                });
            },
            thirdPressLeftAction: function (callback) {
                equal($select[0].selectedIndex, Util.isMozilla ? 2 : 0);

                PressPlaybackAutomation.run('left', function () {
                    equal($select[0].selectedIndex, Util.isMozilla ? 1 : 0);
                    callback();
                });
            },
            fourthPressLeftAction: function () {
                equal($select[0].selectedIndex, Util.isMozilla ? 1 : 0);

                PressPlaybackAutomation.run('left', function () {
                    equal($select[0].selectedIndex, 0);
                    startNext();
                });
            }
        });
    };

    var pressDownUpKeysActionsForSelectWithOptgroups = function ($select, testCallback) {
        async.series({
            pressDownFirstTime: function (callback) {
                equal($select[0].selectedIndex, 0);

                PressPlaybackAutomation.run('down', function () {
                    equal($select[0].selectedIndex, 1);
                    callback();
                });
            },
            pressDownSecondTime: function (callback) {
                PressPlaybackAutomation.run('down', function () {
                    equal($select[0].selectedIndex, 3);
                    callback();
                });
            },
            pressUpFirstTime: function (callback) {
                PressPlaybackAutomation.run('up', function () {
                    equal($select[0].selectedIndex, 1);
                    callback();
                });
            },
            pressUpSecondTime: function () {
                PressPlaybackAutomation.run('up', function () {
                    equal($select[0].selectedIndex, 0);
                    testCallback();
                });
            }
        });
    };

    var pressRightLeftKeysActionsForSelectWithOptgroups = function ($select, testCallback, notChangeInChrome) {
        async.series({
            pressDownFirstTime: function (callback) {
                equal($select[0].selectedIndex, 0);

                PressPlaybackAutomation.run('right', function () {
                    equal($select[0].selectedIndex, Util.isIE || notChangeInChrome ? 0 : 1);
                    callback();
                });
            },
            pressDownSecondTime: function (callback) {
                PressPlaybackAutomation.run('right', function () {
                    equal($select[0].selectedIndex, Util.isIE || notChangeInChrome ? 0 : 3);
                    callback();
                });
            },
            pressUpFirstTime: function (callback) {
                PressPlaybackAutomation.run('left', function () {
                    equal($select[0].selectedIndex, Util.isIE || notChangeInChrome ? 0 : 1);
                    callback();
                });
            },
            pressUpSecondTime: function () {
                PressPlaybackAutomation.run('up', function () {
                    equal($select[0].selectedIndex, Util.isIE || notChangeInChrome? 0 : 0);
                    testCallback();
                });
            }
        });
    };

    QUnit.testDone = function () {
        if (!Util.isIE)
            removeTestElements();
    };

    //tests
    module('actions with select element');

    asyncTest('press down/up/right/left when option list closed', function () {
        var $select = createSelect();

        $select.focus();

        async.series({
            pressDownAction: function (callback) {
                equal($select[0].selectedIndex, 0);

                PressPlaybackAutomation.run('down', function () {
                    equal($select[0].selectedIndex, 1);
                    callback();
                });
            },
            pressUpAction: function (callback) {
                PressPlaybackAutomation.run('up', function () {
                    equal($select[0].selectedIndex, 0);
                    callback();
                });
            },
            pressRightAction: function (callback) {
                PressPlaybackAutomation.run('right', function () {
                    equal($select[0].selectedIndex, Util.isIE ? 0 : 1);
                    callback();
                });
            },
            pressLeftAction: function () {
                PressPlaybackAutomation.run('left', function () {
                    equal($select[0].selectedIndex, 0);
                    startNext();
                });
            }
        });
    });

    asyncTest('press down/up/right/left when option list opened', function () {
        var $select = createSelect();

        async.series({
            openSelectList: function (callback) {
                equal($select[0].selectedIndex, 0);

                ClickPlaybackAutomation.run($select[0], {}, function () {
                    //NOTE: we should wait for binding handlers to the document
                    equal($select[0].selectedIndex, 0);
                    window.setTimeout(callback, 0);
                });
            },
            pressDownAction: function (callback) {
                PressPlaybackAutomation.run('down', function () {
                    equal($select[0].selectedIndex, 1);
                    callback();
                });
            },
            pressUpAction: function (callback) {
                PressPlaybackAutomation.run('up', function () {
                    equal($select[0].selectedIndex, 0);
                    callback();
                });
            },
            pressRightAction: function (callback) {
                PressPlaybackAutomation.run('right', function () {
                    equal($select[0].selectedIndex, Util.isMozilla ? 1 : 0);
                    callback();
                });
            },
            pressLeftAction: function (callback) {
                PressPlaybackAutomation.run('left', function () {
                    equal($select[0].selectedIndex, 0);
                    callback();
                });
            },
            closeSelectList: function () {
                ClickPlaybackAutomation.run($select[0], {}, function () {
                    window.setTimeout(startNext, 0);
                });
            }
        });
    });

    asyncTest('click select and press enter', function () {
        var $select = createSelect();

        async.series({
            'Click on select': function (callback) {
                equal($select[0].selectedIndex, 0);

                ClickPlaybackAutomation.run($select[0], {}, function () {
                    window.setTimeout(callback, 0);
                });
            },

            'Press enter': function (callback) {
                equal($select[0], document.activeElement);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), true);
                equal($select[0].selectedIndex, 0);

                PressPlaybackAutomation.run('enter', callback);
            },

            'Check assertions': function () {
                equal($select[0], document.activeElement);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), false);
                equal($select[0].selectedIndex, 0);
                startNext();
            }
        });
    });

    asyncTest('click select and press tab', function () {
        var $select = createSelect();

        async.series({
            'Click on select': function (callback) {
                equal($select[0].selectedIndex, 0);

                ClickPlaybackAutomation.run($select[0], {}, function () {
                    window.setTimeout(callback, 0);
                });
            },

            'Press tab': function (callback) {
                PressPlaybackAutomation.run('tab', callback);
            },

            'Check assertions': function () {
                notEqual($select[0], document.activeElement);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), false);
                equal($select[0].selectedIndex, 0);
                startNext();
            }
        });
    });

    asyncTest('click select and press esc', function () {
        var $select = createSelect();

        async.series({
            'Click on select': function (callback) {
                equal($select[0].selectedIndex, 0);

                ClickPlaybackAutomation.run($select[0], {}, function () {
                    window.setTimeout(callback, 0);
                });
            },

            'Press esc': function (callback) {
                PressPlaybackAutomation.run('esc', callback);
            },

            'Check assertions': function () {
                equal($select[0], document.activeElement);
                equal(ShadowUI.select('.' + OPTION_LIST_CLASS).is(':visible'), false);
                equal($select[0].selectedIndex, 0);
                startNext();
            }
        });
    });

    asyncTest('click on select and on option', function () {
        var $select = createSelect(),
            option = $select.children()[2];
        ClickPlaybackAutomation.run($select[0], {}, function () {
            equal($select[0].selectedIndex, 0);

            ClickPlaybackAutomation.run(option, {}, function () {
                equal($select[0].selectedIndex, 2);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        });
    });


    module('actions with select element with attribute size more than one');

    asyncTest('press down/up in select', function () {
        var $select = createSelect();

        $select.attr('size', '2');
        //NOTE: IE11 set selectedIndex = -1 after setting attribute size != 1
        $select[0].selectedIndex = 0;
        $select.focus();

        pressDownUpKeysActions($select);
    });

    asyncTest('press right/left in select', function () {
        var $select = createSelect();

        $select.attr('size', '2');
        //NOTE: IE11 set selectedIndex = -1 after setting attribute size != 1
        $select[0].selectedIndex = 0;
        $select.focus();

        pressRightLeftKeysActions($select);
    });

    asyncTest('press down/up in select with "size" more than options count', function () {
        var $select = createSelect();

        $select.attr('size', '10');
        //NOTE: IE11 set selectedIndex = -1 after setting attribute size != 1
        $select[0].selectedIndex = 0;
        $select.focus();

        pressDownUpKeysActions($select);
    });

    asyncTest('press right/left in select with "size" more than options count', function () {
        var $select = createSelect();

        $select.attr('size', '10');
        //NOTE: IE11 set selectedIndex = -1 after setting attribute size != 1
        $select[0].selectedIndex = 0;
        $select.focus();

        pressRightLeftKeysActions($select);
    });


    module('select with option groups');

    asyncTest('click select and option', function () {
        var $select = createSelectWithGroups(),
            option = $select.find('option')[3],
            changeHandled = false;

        //T280587 - Selecting an option does not trigger the on('change', ...) event
        $($select).bind('change', function(){
            changeHandled = true;
        });

        ClickPlaybackAutomation.run($select[0], {}, function () {
            equal($select[0].selectedIndex, 0);
            equal(ShadowUI.select('.' + OPTION_CLASS).length, 9);
            equal(ShadowUI.select('.' + OPTION_GROUP_CLASS).length, 3);

            ClickPlaybackAutomation.run(option, {}, function () {
                window.setTimeout(function () {
                    equal($select[0].selectedIndex, 3);
                    ok(changeHandled, 'change event raised');
                    startNext();
                }, 0);
            });
        });
    });

    asyncTest('click select and option (optgroup with empty label)', function () {
        var $select = createSelectWithGroups(),
            $optgroup = $select.find('optgroup').eq(0).attr('label', ''),
            option = $optgroup.find('option')[1];

        ClickPlaybackAutomation.run($select[0], {}, function () {
            equal($select[0].selectedIndex, 0);
            equal(ShadowUI.select('.' + OPTION_CLASS).length, 9);
            equal(ShadowUI.select('.' + OPTION_GROUP_CLASS).length, 3);

            ClickPlaybackAutomation.run(option, {}, function () {
                equal($select[0].selectedIndex, 1);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        });
    });

    asyncTest('click select and option (in disabled group)', function () {
        var $select = createSelectWithGroups(),
            $optgroup = $select.find('optgroup').eq(1).attr('disabled', ''),
            option = $optgroup.find('option')[0];

        ClickPlaybackAutomation.run($select[0], {}, function () {
            equal($select[0].selectedIndex, 0);
            equal(ShadowUI.select('.' + OPTION_CLASS).length, 9);
            equal(ShadowUI.select('.' + OPTION_GROUP_CLASS).length, 3);

            ClickPlaybackAutomation.run(option, {}, function () {
                equal($select[0].selectedIndex, 0);
                ClickPlaybackAutomation.run($select.find('option')[1], {}, function () {
                    equal($select[0].selectedIndex, 1);
                    window.setTimeout(function () {
                        startNext();
                    }, 0);
                });
            });
        });
    });

    asyncTest('click select and optgroup', function () {
        var $select = createSelectWithGroups(),
            group = $select.find('optgroup')[2];

        ClickPlaybackAutomation.run($select[0], {}, function () {
            equal($select[0].selectedIndex, 0);
            equal(ShadowUI.select('.' + OPTION_CLASS).length, 9);
            equal(ShadowUI.select('.' + OPTION_GROUP_CLASS).length, 3);

            ClickPlaybackAutomation.run(group, {}, function () {
                equal($select[0].selectedIndex, 0);

                //NOTE: to close options list
                ClickPlaybackAutomation.run($select.find('option')[1], {}, function () {
                    equal($select[0].selectedIndex, 1);
                    window.setTimeout(function () {
                        startNext();
                    }, 0);
                });
            });
        });
    });

    asyncTest('click select and option in subgroup', function () {
        var $select = createSelectWithGroups(),
            $optgroup = $select.find('optgroup').eq(1),
            $newOptgroup = $('<optgroup label="subgroup"></optgroup>')
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo($optgroup[0]),
            $newOption = $('<option></option>').text('sub option')
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo($newOptgroup[0]);

        ClickPlaybackAutomation.run($select[0], {}, function () {
            equal($select[0].selectedIndex, 0);
            equal(ShadowUI.select('.' + OPTION_CLASS).length, 10);
            equal(ShadowUI.select('.' + OPTION_GROUP_CLASS).length, 4);

            ClickPlaybackAutomation.run($newOption[0], {}, function () {
                equal($select[0].selectedIndex, 6);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        });
    });

    asyncTest('click select and subgroup', function () {
        var $select = createSelectWithGroups(),
            $optgroup = $select.find('optgroup').eq(1),
            $newOptgroup = $('<optgroup label="subgroup"></optgroup>')
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo($optgroup[0]),
            $newOption = $('<option></option>').text('sub option')
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo($newOptgroup[0]);

        ClickPlaybackAutomation.run($select[0], {}, function () {
            equal($select[0].selectedIndex, 0);
            equal(ShadowUI.select('.' + OPTION_CLASS).length, 10);
            equal(ShadowUI.select('.' + OPTION_GROUP_CLASS).length, 4);

            ClickPlaybackAutomation.run($newOptgroup[0], {}, function () {
                equal($select[0].selectedIndex, 0);

                //NOTE: to close options list
                ClickPlaybackAutomation.run($select.find('option')[1], {}, function () {
                    equal($select[0].selectedIndex, 1);
                    window.setTimeout(function () {
                        startNext();
                    }, 0);
                });
            });
        });
    });

    asyncTest('click select and option out of group', function () {
        var $select = createSelectWithGroups(),
            $optgroup = $select.find('optgroup').eq(1),
            $newOption = $('<option></option>').text('outer option')
                .addClass(TEST_ELEMENT_CLASS)
                .insertAfter($optgroup);

        ClickPlaybackAutomation.run($select[0], {}, function () {
            equal($select[0].selectedIndex, 0);
            equal(ShadowUI.select('.' + OPTION_CLASS).length, 10);
            equal(ShadowUI.select('.' + OPTION_GROUP_CLASS).length, 3);

            ClickPlaybackAutomation.run($newOption[0], {}, function () {
                equal($select[0].selectedIndex, 6);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        });
    });

    asyncTest('press down/up when option list closed', function () {
        var $select = createSelectWithGroupsForCheckPress();

        $select[0].selectedIndex = 0;
        $select.focus();

        pressDownUpKeysActionsForSelectWithOptgroups($select, startNext);
    });

    asyncTest('press right/left when option list closed', function () {
        var $select = createSelectWithGroupsForCheckPress();

        $select[0].selectedIndex = 0;
        $select.focus();

        pressRightLeftKeysActionsForSelectWithOptgroups($select, startNext);
    });

    asyncTest('press down/up when option list opened', function () {
        var $select = createSelectWithGroupsForCheckPress();

        $select[0].selectedIndex = 0;
        $select.focus();

        async.series({
            openSelectList: function (callback) {
                equal($select[0].selectedIndex, 0);

                ClickPlaybackAutomation.run($select[0], {}, function () {
                    //NOTE: we should wait for binding handlers to the document
                    equal($select[0].selectedIndex, 0);
                    window.setTimeout(callback, 0);
                });
            },
            pressDownUpKeysActionsForSelectWithOptgroups: function (callback) {
                pressDownUpKeysActionsForSelectWithOptgroups($select, callback);
            },
            closeSelectList: function () {
                ClickPlaybackAutomation.run($select[0], {}, function () {
                    window.setTimeout(startNext, 0);
                });
            }
        });
    });

    asyncTest('press right/left when option list opened', function () {
        var $select = createSelectWithGroupsForCheckPress();

        async.series({
            openSelectList: function (callback) {
                equal($select[0].selectedIndex, 0);

                ClickPlaybackAutomation.run($select[0], {}, function () {
                    //NOTE: we should wait for binding handlers to the document
                    equal($select[0].selectedIndex, 0);
                    window.setTimeout(callback, 0);
                });
            },
            pressRightLeftKeysActionsForSelectWithOptgroups: function (callback) {
                pressRightLeftKeysActionsForSelectWithOptgroups($select, callback, Util.isWebKit);
            },
            closeSelectList: function () {
                ClickPlaybackAutomation.run($select[0], {}, function () {
                    window.setTimeout(startNext, 0);
                });
            }
        });
    });


    module('select with option groups and size more than one');

    asyncTest('click optgroup', function () {
        var $select = createSelectWithGroups().css({
                position: 'absolute',
                top: '200px',
                left: '300px'
            }),
            group = $select.find('optgroup')[2];

        $select.attr('size', '4');
        $select[0].selectedIndex = 0;

        window.setTimeout(function () {
            //NOTE: during set selected option IE and Mozilla scroll select
            $select.scrollTop(0);

            ClickPlaybackAutomation.run(group, {}, function () {
                ok($select.scrollTop() > 0);
                equal($select[0].selectedIndex, 0);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        }, 100);
    });

    asyncTest('click option', function () {
        var $select = createSelectWithGroups().css({
                position: 'absolute',
                top: '200px',
                left: '300px'
            }),
            option = $select.find('option')[1];

        $select.attr('size', '5');
        $select[0].selectedIndex = 0;

        window.setTimeout(function () {
            //NOTE: during set selected option IE and Mozilla scroll select
            $select.scrollTop(0);
            ClickPlaybackAutomation.run(option, {}, function () {
                equal($select.scrollTop(), 0);
                equal($select[0].selectedIndex, 1);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        }, 100);
    });

    asyncTest('click option with scroll down', function () {
        var $select = createSelectWithGroups().css({
                position: 'absolute',
                top: '200px',
                left: '300px'
            }),
            option = $select.find('option')[8];

        $select.attr('size', '5');
        $select[0].selectedIndex = 0;

        window.setTimeout(function () {
            //NOTE: during set selected option IE and Mozilla scroll select
            $select.scrollTop(0);
            ClickPlaybackAutomation.run(option, {}, function () {
                ok($select.scrollTop() > 0);
                equal($select[0].selectedIndex, 8);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        }, 100);
    });

    asyncTest('click option with scroll up', function () {
        var $select = createSelectWithGroups().css({
                position: 'absolute',
                top: '200px',
                left: '300px'
            }),
            option = $select.find('option')[4];

        $select.attr('size', '5');
        $select[0].selectedIndex = 8;

        window.setTimeout(function () {
            ok($select.scrollTop() > 0);
            ClickPlaybackAutomation.run(option, {}, function () {
                ok($select.scrollTop() > 0);
                equal($select[0].selectedIndex, 4);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        }, 100);
    });

    asyncTest('click option in subgroup', function () {
        var $select = createSelectWithGroups().css({
                position: 'absolute',
                top: '200px',
                left: '300px'
            }),
            $optgroup = $select.find('optgroup').eq(1),
            $newOptgroup = $('<optgroup label="subgroup"></optgroup>')
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo($optgroup[0]),
            $newOption = $('<option></option>').text('sub option')
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo($newOptgroup[0]);

        $select.attr('size', '5');
        $select[0].selectedIndex = 0;

        window.setTimeout(function () {
            //NOTE: during set selected option IE and Mozilla scroll select
            $select.scrollTop(0);
            ClickPlaybackAutomation.run($newOption[0], {}, function () {
                ok($select.scrollTop() > 0);
                equal($select[0].selectedIndex, 6);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        }, 100);
    });

    asyncTest('click subgroup', function () {
        var $select = createSelectWithGroups().css({
                position: 'absolute',
                top: '200px',
                left: '300px'
            }),
            $optgroup = $select.find('optgroup').eq(1),
            $newOptgroup = $('<optgroup label="subgroup"></optgroup>')
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo($optgroup[0]),
            $newOption = $('<option></option>').text('sub option')
                .addClass(TEST_ELEMENT_CLASS)
                .appendTo($newOptgroup[0]);

        $select.attr('size', '5');
        $select[0].selectedIndex = 0;

        window.setTimeout(function () {
            //NOTE: during set selected option IE and Mozilla scroll select
            $select.scrollTop(0);
            ClickPlaybackAutomation.run($newOptgroup[0], {}, function () {
                ok($select.scrollTop() > 0);
                equal($select[0].selectedIndex, 0);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        }, 100);
    });

    asyncTest('click option out of group', function () {
        var $select = createSelectWithGroups(),
            $optgroup = $select.find('optgroup').eq(2),
            $newOption = $('<option></option>').text('outer option')
                .addClass(TEST_ELEMENT_CLASS)
                .insertAfter($optgroup);

        $select.attr('size', '5');
        $select[0].selectedIndex = 0;

        window.setTimeout(function () {
            //NOTE: during set selected option IE and Mozilla scroll select
            $select.scrollTop(0);
            ClickPlaybackAutomation.run($newOption[0], {}, function () {
                ok($select.scrollTop() > 0);
                equal($select[0].selectedIndex, 9);
                window.setTimeout(function () {
                    startNext();
                }, 0);
            });
        }, 100);
    });

    asyncTest('press down/up', function () {
        var $select = createSelectWithGroupsForCheckPress();

        $select.attr('size', '5');
        $select[0].selectedIndex = 0;
        $select.focus();

       pressDownUpKeysActionsForSelectWithOptgroups($select, startNext);
    });

    asyncTest('press right/left', function () {
        var $select = createSelectWithGroupsForCheckPress();

        $select.attr('size', '5');
        $select[0].selectedIndex = 0;
        $select.focus();

        pressRightLeftKeysActionsForSelectWithOptgroups($select, startNext, Util.isWebKit);
    });
});


