var Hammerhead = HammerheadClient.get('Hammerhead'),
    $          = Hammerhead.$,
    Util       = Hammerhead.Util;

var NativeMethods = Hammerhead.NativeMethods;

$(document).ready(function () {
    var Settings  = HammerheadClient.get('Settings');
    var Transport = HammerheadClient.get('Transport');

    Settings.SERVICE_MSG_URL = '/service-msg/100';

    var savedAjaxOpenMethod = NativeMethods.xhrOpen;
    var savedAjaxSendMethod = NativeMethods.xhrSend;

    var requestIsAsync = false;

    function registerAfterAjaxSendHook (callback) {
        callback = callback || function () {
            };

        NativeMethods.xhrOpen = function () {
            requestIsAsync = arguments[2];
            if (typeof requestIsAsync === 'undefined')
                requestIsAsync = true;

            savedAjaxOpenMethod.apply(this, arguments);
        };
        NativeMethods.xhrSend = function () {
            savedAjaxSendMethod.apply(this, arguments);
            callback(this);
        };
    }

    function unregisterAfterAjaxSendHook () {
        NativeMethods.xhrOpen = savedAjaxOpenMethod;
        NativeMethods.xhrSend = savedAjaxSendMethod;
    }

    asyncTest('Queued async service msg', function () {
        var savedAsyncServiceMsgFunc = Transport.asyncServiceMsg;

        Transport.asyncServiceMsg = function (msg, callback) {
            window.setTimeout(function () {
                callback(msg.duration);
            }, msg.duration);
        };

        var completeMsgReqs = [];

        var msgCallback = function (duration) {
            completeMsgReqs.push(duration);

            if (completeMsgReqs.length === 5) {
                var expectedCompleteMsgReqs = [10, 500, 200, 300, 200];

                deepEqual(completeMsgReqs, expectedCompleteMsgReqs);
                Transport.asyncServiceMsg = savedAsyncServiceMsgFunc;

                start();
            }
        };

        expect(1);

        Transport.queuedAsyncServiceMsg({ cmd: 'Type1', duration: 500 }, msgCallback);
        Transport.queuedAsyncServiceMsg({ cmd: 'Type2', duration: 10 }, msgCallback);
        Transport.queuedAsyncServiceMsg({ cmd: 'Type1', duration: 200 }, msgCallback);
        Transport.queuedAsyncServiceMsg({ cmd: 'Type1', duration: 300 }, msgCallback);
        Transport.queuedAsyncServiceMsg({ cmd: 'Type1', duration: 200 }, msgCallback);
    });

    if (!Util.isWebKit) {
        asyncTest('Resend aborted async service msg', function () {
            var xhrCount      = 0,
                callbackCount = 0;

            var onAjaxSend = function (xhr) {
                xhrCount++;

                var expectedAsync = xhrCount === 1;

                strictEqual(requestIsAsync, expectedAsync);

                xhr.abort();
            };

            registerAfterAjaxSendHook(onAjaxSend);

            Transport.asyncServiceMsg({}, function () {
                callbackCount++;
            });

            expect(3);

            window.setTimeout(function () {
                equal(callbackCount, 1);

                unregisterAfterAjaxSendHook();
                start();
            }, 200);
        });
    }
    else {
        asyncTest('Resend aborted async service msg (WebKit)', function () {
            var Hammerhead = HammerheadClient.get('Hammerhead'),
                Settings   = HammerheadClient.get('Settings'),
                Transport  = HammerheadClient.get('Transport');

            Settings.JOB_UID = '%%%testUid%%%';

            var xhrCount      = 0,
                callbackCount = 0,
                value         = 'testValue';


            ok(!window.localStorage.getItem(Settings.JOB_UID));

            var onAjaxSend = function (xhr) {
                xhrCount++;
                xhr.abort();
            };

            registerAfterAjaxSendHook(onAjaxSend);

            var msg = {
                test: value
            };

            Transport.asyncServiceMsg(msg, function () {
                callbackCount++;
            });

            window.setTimeout(function () {
                equal(callbackCount, 1);
                equal(xhrCount, 1);

                var storedMsgStr = window.localStorage.getItem(Settings.JOB_UID),
                    storedMsg    = JSON.parse(storedMsgStr)[0];

                ok(storedMsgStr);
                equal(storedMsg.test, value);

                unregisterAfterAjaxSendHook();

                window.localStorage.removeItem(Settings.JOB_UID);
                start();
            }, 200);
        });

        asyncTest('Do not double messages in store (WebKit)', function () {
            var Hammerhead = HammerheadClient.get('Hammerhead'),
                Settings   = HammerheadClient.get('Settings'),
                Transport  = HammerheadClient.get('Transport');

            Settings.JOB_UID = '%%%testUid%%%';

            var callbackCount = 0,
                value         = 'testValue';


            ok(!window.localStorage.getItem(Settings.JOB_UID));

            var onAjaxSend = function (xhr) {
                xhr.abort();
            };

            registerAfterAjaxSendHook(onAjaxSend);

            var msg = {
                test: value
            };

            Transport.asyncServiceMsg(msg, function () {
                callbackCount++;
            });

            Transport.asyncServiceMsg(msg, function () {
                callbackCount++;
            });

            unregisterAfterAjaxSendHook();

            window.setTimeout(function () {
                equal(callbackCount, 2);

                var storedMsgStr = window.localStorage.getItem(Settings.JOB_UID),
                    storedMsgArr = JSON.parse(storedMsgStr);

                equal(storedMsgArr.length, 1);

                window.localStorage.removeItem(Settings.JOB_UID);
                start();
            }, 200);
        });
    }

    module('regression');

    asyncTest('T445562 - transport should use native (not overriden) xhr', function () {
        var originalSendMethod          = window.XMLHttpRequest.send;
        var originalSendPrototypeMethod = window.XMLHttpRequest.prototype.send;

        var Hammerhead = HammerheadClient.get('Hammerhead');
        var Transport  = HammerheadClient.get('Transport');

        var callbackCount = 0;
        var value         = 'testValue';
        var isError       = false;

        var msg = {
            test: value
        };

        var overiddenMethod = function ()  {
            isError = true;
        };

        window.XMLHttpRequest.send           = overiddenMethod;
        window.XMLHttpRequest.prototype.send = overiddenMethod;

        Transport.asyncServiceMsg(msg, function () {
            callbackCount++;
        });

        window.setTimeout(function () {
            equal(callbackCount, 1);
            ok(!isError);

            window.XMLHttpRequest.send           = originalSendMethod;
            window.XMLHttpRequest.prototype.send = originalSendPrototypeMethod;

            window.localStorage.removeItem(Settings.JOB_UID);

            start();
        }, 200);
    });
});
