var Express = require('express'),
    Process = require('child_process'),
    Path = require('path'),
    FS = require('fs'),
    Http = require('http'),
    mime = require('mime'),
    Hammerhead = require('../../hammerhead').Hammerhead,
    ServiceChannel = require('../../hammerhead/lib/service_channel'),
    Recorder = require('../../lib/server/recorder').Recorder,
    AssetsManager = require('../../lib/server/assets_manager');

//Const
var PROXY_PORT_1 = 1601,
    PROXY_PORT_2 = 1602,
    SERVER_PORT = 1600,

    FAKE_TEST_FILE = Path.resolve(__dirname, 'temp.test.js'),
    FAKE_TEST_FILE_CONTENT = '\'@fixture fix\';\n\'@page ./page.html\';';

var fixtures = {};

var sites = JSON.parse(FS.readFileSync(Path.resolve(__dirname, 'sites.json')).toString());

function addFixture(uid, page) {
    fixtures[uid] = {
        page: page,
        fileName: new Date().getTime(),
        path: []
    };
}

var vfsMock = {
    getSuite: function(callback) {
        callback({fixtures: fixtures});
    },
    getRealPath: function(fixturePath, fixtureFileName, callback) {
        callback(FAKE_TEST_FILE);
    },
    removeFSWatchers: function() {},
    forceBuild: function(callback) {
        callback();
    }
};

var uxlogMock = {
    addEntry: function() {}
};

var uploadedFilesCache = {};

var MAX_FILES_COUNT = 5,
    CHACHE_LIVE_TIME = 3e5, // 5 min
    MAX_FILE_SIZE = 15e5;   // ~ 1Mb

function getFileCount(files) {
    var result = 0;

    for(var file in files) {
        if(files.hasOwnProperty(file))
            result++;
    }

    return result;
}

function uploadFile(jobUid, data, fileNames) {
    if(!uploadedFilesCache[jobUid]) {
        uploadedFilesCache[jobUid] = {};
        setTimeout(function() {
            delete uploadedFilesCache[jobUid];
        }, CHACHE_LIVE_TIME);
    }

    var result = [];

    for(var i = 0; i < fileNames.length; i++) {
        if(data[i].length >= MAX_FILE_SIZE || getFileCount(uploadedFilesCache[jobUid]) > MAX_FILES_COUNT)
            result.push({err: true});
        else
            uploadedFilesCache[jobUid][fileNames[i]] = data[i];
    }

    return result;
}

function getUploadedFile(jobUid, filePaths) {
    var result = [];

    for(var i = 0; i < filePaths.length; i++) {
        var fileName = filePaths[i].replace('./uploads/', '');

        if(uploadedFilesCache[jobUid] && uploadedFilesCache[jobUid][fileName]) {
            result.push({
                data: uploadedFilesCache[jobUid][fileName],
                info: {
                    name: fileName,
                    type: mime.lookup(filePaths[i]),
                    lastModifiedDate: new Date()
                }
            });
        } else {
            result.push({
                filePath: filePaths[i],
                code: true
            });
        }
    }

    return result;
}

exports.start = function () {
    var app = Express(),
        recorder = null,
        hammerheadOptions = {
            staticResources: AssetsManager.getStaticResources()
        },
        hammerhead = new Hammerhead(PROXY_PORT_1, PROXY_PORT_2, 'localhost', hammerheadOptions),

        appServer = Http.createServer(app);

    var injectedFunction = function() {
        var $ = Hammerhead.$,
            ShadowUI = Hammerhead.ShadowUI,
            Settings = TestCafeClient.get('Settings'),
            Transport = TestCafeClient.get('Base.Transport'),
            ServiceCommands = TestCafeClient.get('Shared.ServiceCommands'),

            returnUrl = Settings.RETURN_URL;

        var WELCOME_MSG_BOX_CLASS = 'welcome-msg-box',
            CONTENT_CLASS = 'content',
            RIGHT_CLASS = 'right',
            IMAGE_CLASS = 'image',
            IMAGE_CONTAINER_CLASS = 'image-container',
            TITLE_CLASS = 'title',
            MESSAGE_CLASS = 'message',
            BACKGROUND_CLASS = 'background',
            CLOSE_CLASS = 'close';

        var storedGetRoot = ShadowUI.getRoot,
            storedQueuedAsyncServiceMsg = Transport.queuedAsyncServiceMsg,
            hidden = true;

        Transport.queuedAsyncServiceMsg = function(msg, callback) {
            if(msg.cmd === ServiceCommands.CHANGE_SHOW_STEPS)
                localStorage['showSteps'] = msg.show;
            else
                return storedQueuedAsyncServiceMsg.call(Transport, msg, callback);
        };

        ShadowUI.getRoot = function() {
            var $res = storedGetRoot.call(ShadowUI);

            if(hidden && !localStorage[Settings.JOB_UID + '_show_mssg'])
                $res[0].style.display = 'none';
            else
                $res[0].style.display = '';

            return $res;
        };

        function addResources() {
            $('<style type="text/css">' + css +'</style>').appendTo('head');
        }

        function createWelcomeMsgBox(title, message, closeText) {
            TestCafeClient.get('UI.RecorderWidgets.Toolbar').hide();

            var $background = $('<div>').appendTo(ShadowUI.getRoot()).html(''),
                $msgBox = $('<div>').appendTo($background);

            var $content = $('<div>').appendTo($msgBox),
                $imageContainer = $('<div>').css('visibility', 'hidden').appendTo($content),
                $image = $('<div>').appendTo($imageContainer),
                $right = $('<div>').css('visibility', 'hidden').appendTo($content),
                $title = $('<div>').appendTo($right).html(title),
                $message = $('<div>').appendTo($right).html(message),
                $close = $('<div>').appendTo($right),
                $closeLink = $('<a href="javascript:;">').html(closeText).appendTo($close);

            ShadowUI.addClass($background, BACKGROUND_CLASS);
            ShadowUI.addClass($msgBox, WELCOME_MSG_BOX_CLASS);
            ShadowUI.addClass($content, CONTENT_CLASS);
            ShadowUI.addClass($right, RIGHT_CLASS);
            ShadowUI.addClass($image, IMAGE_CLASS);
            ShadowUI.addClass($imageContainer, IMAGE_CONTAINER_CLASS);
            ShadowUI.addClass($title, TITLE_CLASS);
            ShadowUI.addClass($message, MESSAGE_CLASS);
            ShadowUI.addClass($close, CLOSE_CLASS);

            $background.css('height', $('body').height() + "px");
            $background.css('width', $('body').width() + "px");

            $msgBox.css('top', Math.max(0, ($(window).height() - $msgBox.outerHeight()) / 2) + "px");
            $msgBox.css('left', Math.max(0, ($(window).width() - $msgBox.outerWidth()) / 2) + "px");
            $msgBox.css('display', 'none');

            $msgBox.show('slow', function () {
                $imageContainer.css('visibility', 'visible');
                $right.css('visibility', 'visible');

                var close = function(){
                    $background.remove();
                    TestCafeClient.get('UI.RecorderWidgets.Toolbar').show();
                };

                $closeLink.bind('click', close);
            });
        }

        $(document).ready(function() {
            var id = window.setInterval(function() {
                var saveButtonContainer = ShadowUI.select(".buttons-container");

                if(saveButtonContainer.length) {
                    clearInterval(id);

                    var $saveButton = $(saveButtonContainer[1].childNodes[1]),
                        $exitButton = $(saveButtonContainer[1].childNodes[2]),
                        $newExitButton = $exitButton.clone(),
                        $newSaveButton = $saveButton.clone();

                    $saveButton.replaceWith($newSaveButton);
                    $exitButton.replaceWith($newExitButton);

                    ShadowUI.addClass($newSaveButton, 'disable');

                    $newExitButton.bind('click', function(){
                        window.location = returnUrl;
                    });

                    hidden = false;
                    if(!localStorage[Settings.JOB_UID + '_show_mssg']) {
                        localStorage[Settings.JOB_UID + '_show_mssg'] = true;
                        addResources();
                        createWelcomeMsgBox('Record a test visually using TestCafe Recorder!', 'In this demo, you can record your own test and replay the recorded steps after that. Play with the test page and then click the Playback button.',
                        'Start demo');
                    }
                    ShadowUI.getRoot();
                }
            }, 200);
        });
    };

    var cssScript = 'var css = "' + FS.readFileSync(Path.join(__dirname, './style.css')).toString().replace(/\r?\n|\r/g, '') + '";';

    var renderTaskScriptMeth = ServiceChannel.renderTaskScript,
        injectedScript = cssScript + injectedFunction.toString().replace(/^\s*function\s*\(\s*\)\s*{|}\s*$/g, '');

    ServiceChannel.renderTaskScript = function(vars) {
        var data = renderTaskScriptMeth(vars);
        data = data.replace('delete window.TestCafeClient;', '');
        data += 'if(window === window.top) {\n' + injectedScript + '\n}\n';
        data = data.replace(/SHOW_STEPS\s=\s(true|false)/, 'SHOW_STEPS = (typeof localStorage["showSteps"] === "undefined" || localStorage["showSteps"] === "true")');
        return data;
    };

    Recorder.prototype._onUploadFiles = function(jobUid, data, fileNames, callback){
        callback(uploadFile(jobUid, data, fileNames));
    };

    Recorder.prototype._onUploadedFilesRequired = function(jobUid, filePaths, callback){
        callback(getUploadedFile(jobUid, filePaths));
    };

    recorder = new Recorder(hammerhead, vfsMock, uxlogMock, true, function() {});

    hammerhead.start();

    FS.writeFileSync(FAKE_TEST_FILE, FAKE_TEST_FILE_CONTENT);
    FS.watchFile(FAKE_TEST_FILE, function () {
        FS.writeFileSync(FAKE_TEST_FILE, FAKE_TEST_FILE_CONTENT);
    });

    app
        .use(Express.bodyParser())
        .set('view engine', 'ejs')
        .set('view options', {layout: false})
        .set('views', Path.join(__dirname, './views'));

    app.get('/index', function (req, res) {
        res.render('index');
    });

    app.post('/', function (req, res) {
        var url = req.param('url');

        if (!url || !sites[url]) {
            res.status(403);
            res.render('403');
        }
        else {
            res.statusCode = 301;

            var jobUid = new Date().getTime();

            addFixture(jobUid, sites[url]);
            recorder.startRecording(jobUid, 'http://localhost:' + SERVER_PORT, function(err, recordingUrl) {
                if(!err)
                    res.setHeader('location', recordingUrl);
                res.end();
            }, '');
        }
    });


    hammerhead.start();
    appServer.listen(SERVER_PORT);

    console.log('Server listens on port ' + SERVER_PORT);
    Process.exec('start http://localhost:' + SERVER_PORT + '/index');
};